//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       CheckpointView.cpp	[ CheckpointView.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CCheckpointView class
//
//***************************************************************************

#include "stdafx.h"
#include <sys/stat.h>

#include "SourceMetrics.h"
#include "SourceMetricsDoc.h"
#include "CheckpointView.h"
#include "MetricDetailsDlg.h"
#include "ViewSourceDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCheckpointView

IMPLEMENT_DYNCREATE(CCheckpointView, CListView)

BEGIN_MESSAGE_MAP(CCheckpointView, CListView)
	//{{AFX_MSG_MAP(CCheckpointView)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_ITEM_DELETE, OnFileDelete)
	ON_UPDATE_COMMAND_UI(ID_ITEM_DELETE, OnUpdateFileDelete)
	ON_NOTIFY_REFLECT(NM_CLICK, OnFileClick)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_WM_DESTROY()
	ON_COMMAND(ID_CKPT_VIEW_METRICS_DETAILS, OnViewFileMetricsDetails)
	ON_COMMAND(ID_CKPT_VIEW_VIEWSOURCE, OnCkptViewViewsource)
	ON_UPDATE_COMMAND_UI(ID_CKPT_VIEW_VIEWSOURCE, OnUpdateCkptViewViewsource)
	ON_COMMAND(ID_CKPT_VIEW_CHART, OnCkptViewChart)
	ON_UPDATE_COMMAND_UI(ID_CKPT_VIEW_METRICS_DETAILS, OnUpdateCkptViewMetricsDetails)
	ON_COMMAND(IDR_CK_CONTEXT_HELP, OnCkContextHelp)
	ON_UPDATE_COMMAND_UI(IDR_CK_CONTEXT_HELP, OnUpdateCkContextHelp)
	ON_COMMAND(ID_CKPT_EDIT_VIEWSOURCE, OnCkptEditViewsource)
	ON_UPDATE_COMMAND_UI(ID_CKPT_EDIT_VIEWSOURCE, OnUpdateCkptEditViewsource)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

CCheckpointView::CCheckpointView() : CListView(), m_poProject(NULL)
{
	//{{AFX_DATA_INIT(CCheckpointView)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CCheckpointView::~CCheckpointView()
{
   m_poSMDoc->UnsetView(m_lViewID);	
}

BOOL CCheckpointView::PreCreateWindow(CREATESTRUCT& cs) 
{
   // set the view control style here
   BOOL fResult = CListView::PreCreateWindow(cs);
   cs.style or_eq LVS_REPORT bitor LVS_SHOWSELALWAYS;        // set list style to report mode
	return fResult;
}

/////////////////////////////////////////////////////////////////////////////
// CCheckpointView diagnostics

#ifdef _DEBUG
void CCheckpointView::AssertValid() const
{
	CListView::AssertValid();
}

void CCheckpointView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}
#endif //_DEBUG

void CCheckpointView::OnInitialUpdate() 
{
   DWORD       BackgroundColor = GetSysColor(COLOR_MENU);
   CString     sLabel;                       // scratch label holder
   CDWordArray iaColumnOrder;                // raw order numbers as read from the header control
   int         i;                            // loop index (reused)

   // one-time initialization
   m_poSMDoc = dynamic_cast<CSourceMetricsDoc *>(m_pDocument);
   m_poProject = m_poSMDoc->GetProject();
   m_iViewColumns = m_poProject->GetLanguage()->Columns() + FIXED_HEADERS;
   m_iMetricColumns = m_poProject->GetLanguage()->Columns();
   m_lViewID = (long)m_hWnd;                 // create a unique token for this view
   m_poSMDoc->SetView(m_lViewID);            // put ourselves in the view map
   for( i = 0; i < m_iViewColumns; ++i )     // initialize column order for change comparison
   {
      sLabel.Format("%d", i);
      m_saColumnOrder.Add(sLabel);
   }
   m_poListCtrl = &GetListCtrl();                  // this is the view's wrapped list control
   m_sSection = CSourceMetricsApp::OPTION_SETTINGS + m_poProject->GetLanguage()->Name();  // set option profile key
   m_iCheckpointIndex = m_poSMDoc->CheckpointIndex();
   m_poProject->GetCheckpointByIndex(m_iCheckpointIndex, m_oCkpt);
   m_sProjectCaption = m_poSMDoc->GetProjectCaption();

   // initialize the list view's background and columns
   m_poListCtrl->SetBkColor(BackgroundColor);
   m_poListCtrl->SetTextBkColor(BackgroundColor);
   sLabel.LoadString(IDS_CKPT_VIEW_FILES_LABEL);
   m_poListCtrl->InsertColumn(0, sLabel, LVCFMT_LEFT, 0, 0);
   m_poListCtrl->SetColumnWidth(0, LVSCW_AUTOSIZE);

   // now set the extra styles for an enhanced list view
   m_poListCtrl->SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, 0,
         (LPARAM)(LVS_EX_FULLROWSELECT bitor LVS_EX_HEADERDRAGDROP));
   m_saColumnOrder.SetSize(m_iViewColumns);
   m_baSortOrder.SetSize(m_iViewColumns);

   // establish the order in which columns should appear in the list
   // (and initialize sort order to descending)
   for( i = 0; i < m_iViewColumns; ++i )    // first set default values
   {
      sLabel.Format("%d", i);
      m_saColumnOrder[i] = sLabel;
      m_baSortOrder[i] = SORT_DESCENDING;
   }
   // if user has changed the order, override defaults with option
   m_poSMDoc->GetApp()->GetProfileStringList(m_sSection, CSourceMetricsApp::CKPT_VIEW_COLUMN_ORDER,
                                  m_saColumnOrder, CString(_T(",;")), CString(_T("")));
   sLabel.LoadString(IDS_CKPT_VIEW_CAPTION);
   m_sCaption.Format(sLabel, m_poProject->GetLanguage()->Name(),
                        m_poSMDoc->GetProject()->Name(), m_oCkpt.Name());
   GetParent()->SetWindowText(m_sCaption);

   // now do initial update
   OnUpdate(NULL, 0, NULL);

   return;	
}

void CCheckpointView::OnUpdate(CView* /* pSender */, LPARAM /* lHint */, CObject* /* pHint */) 
{
   CWaitCursor    oWaitCursor;               // let user know this may take awhile
   SMFile         oFile;                     // scratch for property extraction
   int            i;                         // loop index (reused)
   CHeaderCtrl *  poViewHeader = (CHeaderCtrl *)m_poListCtrl->GetDlgItem(0);
   bool           fIsFirstUpdate =
                     (poViewHeader not_eq NULL) and (poViewHeader->GetItemCount() <= FIXED_HEADERS);

   // set the column labels, first time only
   if( fIsFirstUpdate )
   {
      CDWordArray iaColumnOrder;    // raw order numbers as read from the header control

      m_poProject->GetLanguage()->InitLabels(); // refresh the project's labels
      for( i = 0; i < m_iMetricColumns; ++i )
      {
         m_poListCtrl->InsertColumn(i + FIXED_HEADERS, m_poProject->GetCaption(i),
                                     LVCFMT_RIGHT, 0, i + FIXED_HEADERS);
      }
      // now size the columns to fit the labels
      for( i = 0; i < m_iMetricColumns; ++i )
      {
         m_poListCtrl->SetColumnWidth(i + FIXED_HEADERS, LVSCW_AUTOSIZE_USEHEADER);
      }
      // now transfer the column order into an int array and send it to the control
      iaColumnOrder.SetSize(m_iViewColumns);
      for( i = 0; i < m_iViewColumns; ++i )
      {
         iaColumnOrder[i] = _ttol(m_saColumnOrder[i]);
      }
      m_poListCtrl->SendMessage(LVM_SETCOLUMNORDERARRAY, (WPARAM)m_iViewColumns,
                                 (LPARAM)iaColumnOrder.GetData());
   }
   else
   {
      // make sure we have the latest version
      SMCheckpoint   oUpdatedCkpt;

      if( m_poProject->GetUpdatedCheckpoint(m_oCkpt, oUpdatedCkpt) )
      {
         if( not (oUpdatedCkpt == m_oCkpt) )
         {
            m_oCkpt = oUpdatedCkpt;
         }
      }
   }
   // fill the list with file metrics
   m_poListCtrl->DeleteAllItems();
   for( i = 0; i < m_oCkpt.FileCount(); ++i )
   {
      if( m_oCkpt.GetNextFile(oFile, i == 0) )
      {
         CString  sFileName(oFile.PathName());

         m_poListCtrl->InsertItem(i, sFileName, 0);
         m_poListCtrl->SetItemData(i, i);             // set lParam to identify items for sorting
         for( int v = 0; v < m_iMetricColumns; ++v )
         {
            m_poListCtrl->SetItemText(i, v + FIXED_HEADERS, oFile.m_poLanguage->GetValue(v));
         }
      }
   }
   // now size the columns to fit the data or heading, whichever is wider
   if( fIsFirstUpdate )
   {
      int      iAllColumnsWidth = 0;
      int      iFrameWidth;
      CWnd *   poParentFrame = GetParentFrame();
      CRect    oParentRect;

      poParentFrame->GetWindowRect(&oParentRect);
      iFrameWidth = oParentRect.Width();           // save original size of the frame
      oParentRect.right = oParentRect.left + 10;   // make sure the columns don't fit so right column size is OK
      poParentFrame->MoveWindow(oParentRect);
      for( i = 0; i < m_iViewColumns; ++i )
      {
         int iColumnWidth = m_poListCtrl->GetColumnWidth(i);

         m_poListCtrl->SetColumnWidth(i, LVSCW_AUTOSIZE);
         if( iColumnWidth > m_poListCtrl->GetColumnWidth(i) )
         {
            // put it back
            m_poListCtrl->SetColumnWidth(i, LVSCW_AUTOSIZE_USEHEADER);
            iAllColumnsWidth += iColumnWidth;
         }
         else
         {
            iAllColumnsWidth += m_poListCtrl->GetColumnWidth(i);
         }
      }
      oParentRect.right = oParentRect.left + iFrameWidth;
      poParentFrame->MoveWindow(oParentRect);      // restore the window width
   }
   return;
}

// ***********************************************************************************
// warn user if file is older than checkpoint
void
CCheckpointView::CheckModified(CString sFileName)
{
   struct _stat   oFileStatus;
   CTime          oFileModifiedDate;

   _tstat(sFileName, &oFileStatus);
   oFileModifiedDate = oFileStatus.st_mtime;
   if( oFileModifiedDate > m_oCkpt.Date() )
   {
      CString  sMessage;
      CString  sCaption;

      sMessage.LoadString(IDS_CKPT_VIEW_CHANGED_WARNING);
      sCaption.LoadString(IDS_CKPT_VIEW_CHANGED_CAPTION);
      MessageBox(sMessage, sCaption, MB_OK bitor MB_ICONINFORMATION);
   }
}

// ***********************************************************************************
// static sort callback
int
CCheckpointView::SortCompareCallback(LPARAM leftItemData, LPARAM rightItemData, LPARAM pThis)
{
   return ((CCheckpointView *)pThis)->SortCompare(leftItemData, rightItemData);
}

// ***********************************************************************************
// real sort compare
int
CCheckpointView::SortCompare(LPARAM leftItemData, LPARAM rightItemData)
{
   SMFile   oLeftFile;
   SMFile   oRightFile;
   CString  sLeft;
   CString  sRight;
   char     bSortOrder = m_baSortOrder[m_iSortColumn];
   int      iResult = 0;

   m_oCkpt.GetFileByIndex(leftItemData, oLeftFile);
   m_oCkpt.GetFileByIndex(rightItemData, oRightFile);
   switch( m_iSortColumn )
   {
   case 0:
      sLeft = oLeftFile.PathName();
      sRight = oRightFile.PathName();
      if( (sLeft.Find('.') >= 0) and (sRight.Find('.') >= 0) )
      {
         CString  sRightExtension;
         CString  sLeftExtension;

         sLeftExtension = sLeft.Mid(sLeft.Find('.') + 1);
         sRightExtension = sRight.Mid(sRight.Find('.') + 1);
         if( sLeftExtension.Mid(0, 1) != sRightExtension.Mid(0, 1) )
         {
            sLeft = sLeftExtension;          // sort by extension if they are different
            sRight = sRightExtension;
         }
      }
      iResult = _tcsicmp(sLeft, sRight);
      break;

   default:
      sLeft = oLeftFile.m_poLanguage->GetValue(m_iSortColumn - FIXED_HEADERS, false);
      sRight = oRightFile.m_poLanguage->GetValue(m_iSortColumn - FIXED_HEADERS, false);
      iResult = (int)(100 * (_tcstod(sLeft, NULL) - _tcstod(sRight, NULL)));
   }
   if( bSortOrder != 0 )
   {
      iResult = - iResult;    // sort descending
   }
   return iResult;
}

/////////////////////////////////////////////////////////////////////////////
// CCheckpointView message handlers
//

void CCheckpointView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CMenu    oContextMenu;
   CPoint   oClientPoint(point);          // converted coordinates for hit test

   pWnd->ScreenToClient(&oClientPoint);   // convert to client coordinates for hit text
   oContextMenu.LoadMenu(IDR_CHECKPOINT_VIEW_CONTEXT);
   oContextMenu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN bitor TPM_RIGHTBUTTON,
      point.x, point.y, GetParentFrame());

   return;
}

void CCheckpointView::OnFileDelete() 
{
   CWaitCursor oWaitCursor;               // let user know this may take awhile
   int         iItemsToDelete = m_poListCtrl->GetSelectedCount();
   bool        fFileDeleted = false;      // assume the user chickens out
   CString     sCaption;
   CString     sMessage;

   sCaption.LoadString(IDS_DELETE_FILE_CAPTION);
   sMessage.LoadString(IDS_DELETE_FILE_QUERY);
   if( MessageBox(sMessage, sCaption, MB_ICONQUESTION bitor MB_YESNO) == IDYES )
   {
      for( int i = 0; i < m_poListCtrl->GetItemCount(); ++i )
      {
         // search for the selected items
         if( (m_poListCtrl->GetItemState(i, LVIS_SELECTED) bitand LVIS_SELECTED) != 0 )
         {
            fFileDeleted = m_oCkpt.DeleteFile(m_poListCtrl->GetItemText(i, 0));
            if( --iItemsToDelete == 0 )
            {
               break;                     // done 'em all
            }
         }
      }
   }
   if( fFileDeleted )
   {
      // update sums, then update the project's copy of the checkpoint
      m_oCkpt.ComputeMetrics();           // recompute sums without the deleted files
      m_poProject->UpdateCheckpoint(m_oCkpt);
	   m_poSMDoc->SetModifiedFlag();
      m_poSMDoc->UpdateAllViews(NULL);    // update us, too
   }
}

void CCheckpointView::OnUpdateFileDelete(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED) >= 0);	
}

void CCheckpointView::OnFileClick(NMHDR* /* pNMHDR */, LRESULT* pResult) 
{
   m_iSelectedIndex = m_poListCtrl->GetNextItem(-1, LVNI_SELECTED);   // get selected item
	*pResult = 0;
}

void CCheckpointView::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW *  pNMListView = (NM_LISTVIEW*)pNMHDR;

   m_iSortColumn = pNMListView->iSubItem;
   m_poListCtrl->SortItems(CCheckpointView::SortCompareCallback, (LPARAM)this);

   // toggle sort order for this column
   m_baSortOrder[m_iSortColumn] = (BYTE)(m_baSortOrder[m_iSortColumn] xor SORT_DESCENDING);

   *pResult = 0;
}

void CCheckpointView::OnDestroy() 
{
   CDWordArray    iaColumnOrder;       // raw order numbers as read from the header control
   CStringArray   saColumnOrder;       // order numbers as strings
   bool           fIsChanged = false;  // true if any column has been moved by user
   
   // get the column arrangement and save it if it's different
   iaColumnOrder.SetSize(m_iViewColumns);
   saColumnOrder.SetSize(m_iViewColumns);
   if( SendMessage(LVM_GETCOLUMNORDERARRAY, (WPARAM)m_iViewColumns, (LPARAM)iaColumnOrder.GetData()) )
   {
      CString  sColumnNumber;

      for( int i = 0; i < m_iViewColumns; ++i )
      {
         sColumnNumber.Format("%d", iaColumnOrder[i]);
         if( sColumnNumber != m_saColumnOrder[i] )
         {
            fIsChanged = true;
         }
         saColumnOrder[i] = sColumnNumber;
      }
      if( fIsChanged )
      {
         m_poSMDoc->GetApp()->WriteProfileStringList(m_sSection,
                                          CSourceMetricsApp::CKPT_VIEW_COLUMN_ORDER,
                                          saColumnOrder, CString(_T(";")));
      }
   }
	CListView::OnDestroy();
}

BOOL CCheckpointView::OnPreparePrinting(CPrintInfo* pInfo) 
{
	return CListView::DoPreparePrinting(pInfo);
}

void CCheckpointView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
   CString  sPageCaption;

   sPageCaption.Format("%s: %s", m_sProjectCaption, m_sCaption);

   // use the common view print routines
   m_poSMDoc->BeginPrinting(pDC, pInfo, m_poListCtrl, m_iViewColumns, sPageCaption);
	CListView::OnBeginPrinting(pDC, pInfo);
}

void CCheckpointView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
   m_fPrinting = m_poSMDoc->PrintPage(pDC, pInfo);
}

void CCheckpointView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
   m_poSMDoc->EndPrinting();	
	CListView::OnEndPrinting(pDC, pInfo);
}

void CCheckpointView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
   if( pDC->IsPrinting() and (not m_fPrinting) )
   {
      pInfo->m_bContinuePrinting = FALSE;
   }
	CListView::OnPrepareDC(pDC, pInfo);
}

void CCheckpointView::OnViewFileMetricsDetails() 
{
   SMFile            oFile;
   CMetricDetailsDlg oDlg(this);
   CString           sEntity;

   m_oCkpt.GetFileByName(m_poListCtrl->GetItemText(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED), 0), oFile);
   sEntity.LoadString(IDS_FILE_DETAILS_CAPTION);
   oDlg.m_sCaptionSuffix.Format(sEntity, oFile.PathName());

   // put our four items in first
   sEntity.LoadString(IDS_DETAILS_DIRECTORY_LABEL);
   oDlg.m_saEntities.Add(sEntity);
   oDlg.m_saValues.Add(m_poProject->Directory());
   sEntity.LoadString(IDS_CKPT_VIEW_FILES_LABEL);
   oDlg.m_saEntities.Add(sEntity);
   oDlg.m_saValues.Add(oFile.PathName());

   // now let the language add some more entities
   oFile.m_poLanguage->GetDetails(m_oCkpt.Version(), oDlg.m_saValues, oDlg.m_iaBlockDepths, false, true,
                                    oDlg.m_saEntities);
   oDlg.DoModal();
}

void CCheckpointView::OnCkptViewViewsource() 
{
   SMFile         oFile;
   CString        sFileName = m_poProject->Directory();
   CViewSourceDlg oDlg(this);
   CString        sLabel;
   int            iLineNumber;

   // get the file name first
   m_oCkpt.GetFileByName(m_poListCtrl->GetItemText(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED), 0), oFile);
   sFileName += oFile.PathName();

   // if file has changed since checkpoint was created, tell the user
   CheckModified(sFileName);
   oDlg.SetFileName(sFileName);
   if( oFile.m_poLanguage->GetDeepestBlockLine(sLabel, &iLineNumber) and (iLineNumber > 0) )
   {
      oDlg.EnableDeepestButton(sLabel, iLineNumber);
   }
   if( oFile.m_poLanguage->GetBiggestModuleLine(sLabel, &iLineNumber) )
   {
      oDlg.EnableBiggestModule(sLabel, iLineNumber);
   }

   oDlg.DoModal();
}

void CCheckpointView::OnUpdateCkptEditViewsource(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetSelectedCount() == 1);	
}

void CCheckpointView::OnCkptEditViewsource() 
{
   STARTUPINFO oStartupData;
   PROCESS_INFORMATION
               oResultData;                  // ignored
   CString     sFileToken;
   int         iTokenOffset;
   CString     sFileName(m_poProject->Directory());
   CString     sCommandLine;   

   for( int i = 0; i < m_poListCtrl->GetItemCount(); ++i )
   {
      // search for the selected item
      if( (m_poListCtrl->GetItemState(i, LVIS_SELECTED) bitand LVIS_SELECTED) != 0 )
      {
         sFileName += m_poListCtrl->GetItemText(i, 0);
         break;                     // there is only one if this gets called
      }
   }
   // if file has changed since checkpoint was created, tell the user
   CheckModified(sFileName);
   sFileToken.LoadString(IDS_VIEWER_FILE_TOKEN);
   sCommandLine = AfxGetApp()->GetProfileString(CSourceMetricsApp::OPTION_SETTINGS,
                                                CSourceMetricsApp::SOURCE_VIEWER, CString(_T("")));
   if( sCommandLine.GetLength() > 0 )
   {
      // replace token with the actual file pathname
      if( (iTokenOffset = sCommandLine.Find(sFileToken)) >= 0 )
      {
         CString  sCommandWithToken(sCommandLine);

         sCommandLine.Format("%s\"%s\"%s", sCommandWithToken.Left(iTokenOffset), sFileName,
                  sCommandWithToken.Right(sCommandWithToken.GetLength() - (iTokenOffset + sFileToken.GetLength())));
      }
      else
      {
         sCommandLine += _T(" ") + sFileName;
      }
   }
   else
   {
      int   iDirectoryLength;

      iDirectoryLength = GetWindowsDirectory(NULL, 0);
      GetWindowsDirectory(sCommandLine.GetBuffer(iDirectoryLength), iDirectoryLength);
      sCommandLine.ReleaseBuffer();
      sCommandLine += _T("\\");
      sCommandLine += _T("NOTEPAD.EXE");
      sCommandLine = sCommandLine + _T(" ") + sFileName;
   }
   // now see if the file has been changed since the checkpoint was created
   // TODO: create CFile and CFileStatus and check the modified date of the file about to be opened
   // if( oFileStatus.m_mtime < m_oCkpt.GetDate() )
   memset(&oStartupData, 0, sizeof(oStartupData));
   oStartupData.cb = sizeof(oStartupData);
   oStartupData.dwFlags = STARTF_USESHOWWINDOW;
   oStartupData.wShowWindow = SW_NORMAL;

   // TODO: replace all this nonsense with ShellExecuteEx() ?
   if( CreateProcess(NULL, sCommandLine.GetBuffer(sCommandLine.GetLength()), NULL, NULL,
                     FALSE, NORMAL_PRIORITY_CLASS, NULL, NULL, &oStartupData, &oResultData) == 0 )
   {
      CString  sErrorMessage;
      DWORD    dwErrorNumber = GetLastError();
      
      if( dwErrorNumber )
      {
         sErrorMessage.LoadString(IDS_CKPT_VIEW_VIEWER_ERROR);
         m_poProject->ReportFileError(sErrorMessage, dwErrorNumber);
      }
   }
   sCommandLine.ReleaseBuffer();
}

void CCheckpointView::OnUpdateCkptViewViewsource(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetSelectedCount() == 1);	
}

void CCheckpointView::OnCkptViewChart() 
{
   // most of the following code copied from CMDIFrameWnd::OnWindowNew()
   // in MFC file winmdi.cpp
   CSourceMetricsApp *
                  poApp = (CSourceMetricsApp*)AfxGetApp();
	CMDIChildWnd * pActiveChild = poApp->m_pAppMainFrame->MDIGetActive();  // ((CMainFrame*)m_pMainWnd)
	CDocument *    pDocument;     // metrics document class pointer
   CDocTemplate * pTemplate;     // new view template pointer
   CFrameWnd *    pFrame;        // new view class window pointer

	if( (pActiveChild == NULL) or ((pDocument = pActiveChild->GetActiveDocument()) == NULL) )
	{
		TRACE0("Warning: No active document for Chart Checkpoint command.\n");
		AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
	}
   else
   {
	   // otherwise we have a new frame !
	   pTemplate = poApp->m_pFreqGraphTemplate;
	   ASSERT_VALID(pTemplate);
	   pFrame = pTemplate->CreateNewFrame(pDocument, pActiveChild);
	   if (pFrame == NULL)
	   {
		   TRACE0("Warning: failed to create new frame.\n");
		   AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
	   }
      else
      {
         if( m_iCheckpointIndex >= 0 )
         {
            m_poSMDoc->CheckpointIndex(m_iCheckpointIndex);
            m_poSMDoc->SetProjectCaption(m_sCaption);          // pass our caption thru for graph view printing
            pTemplate->InitialUpdateFrame(pFrame, pDocument);
         }
      }
   }
}

void CCheckpointView::OnUpdateCkptViewMetricsDetails(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetSelectedCount() == 1);	
}

void CCheckpointView::OnCkContextHelp() 
{
	AfxGetApp()->WinHelp(IDR_CK_CONTEXT_HELP, HELP_CONTEXT);
}

void CCheckpointView::OnUpdateCkContextHelp(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CCheckpointView::OnEditUndo() 
{
   if( m_oCkpt.RestoreDeletedFile() )
   {
	   m_poSMDoc->SetModifiedFlag();
      m_poSMDoc->UpdateAllViews(NULL);       // update us, too
   }
}

void CCheckpointView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_oCkpt.CanUndoFile() );	
}
