//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       DisplayDetails.cpp	[ DisplayDetails.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CDisplayDetails class. This is a stand-
//             alone display for a single file, invoked from the command line
//
//***************************************************************************

#include "stdafx.h"
#include "SourceMetrics.h"
#include "SMCpp.h"
#include "SMVisualBasic.h"
#include "SMJava.h"
#include "SMHTML.h"
#include "SMDelphi.h"
#include "DisplayDetails.h"
#include "MetricDetailsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDisplayDetails

CDisplayDetails::CDisplayDetails(TCHAR cLanguage, CString & sFilePathname)
: m_sFilePathname(sFilePathname), m_poLanguageExemplar(NULL)
{
   switch( cLanguage )
   {
   case 'c':   // C++
      m_poLanguageExemplar = new SMCpp(Exemplar());
      break;

   case 'j':   // Java
      m_poLanguageExemplar = new SMJava(Exemplar());
      break;

   case 'd':   // Delphi (Pascal)
      m_poLanguageExemplar = new SMDelphi(Exemplar());
      break;

   case 'h':   // HTML
      m_poLanguageExemplar = new SMHTML(Exemplar());
      break;

   case 'v':   // Visual Basic
      m_poLanguageExemplar = new SMVisualBasic(Exemplar());
      break;
   }
   m_poLanguageExemplar->InitParser();
}

CDisplayDetails::~CDisplayDetails()
{
   m_poLanguageExemplar->CloseParser();
   delete m_poLanguageExemplar;
}

// ******************************************************************
// build and display the details dialog for one file
bool
CDisplayDetails::Show(int iMajorVersion, int iMinorVersion)
{
   SMFile   oFile;
   CString  sFilename;
   CString  sDirectory;
   CString  sExtension;
   CString  sDrive;
   bool     fResult = true;   // assume success

   _tsplitpath(m_sFilePathname, sDrive.GetBuffer(_MAX_DRIVE), sDirectory.GetBuffer(_MAX_DIR),
               sFilename.GetBuffer(_MAX_FNAME), sExtension.GetBuffer(_MAX_EXT));
   sDrive.ReleaseBuffer();
   sDirectory.ReleaseBuffer();
   sFilename.ReleaseBuffer();
   sExtension.ReleaseBuffer();
   sDirectory = sDrive + sDirectory;
   sFilename += sExtension;
   oFile.PathName(sFilename);
   oFile.m_poLanguage = m_poLanguageExemplar->MakeNew();
   oFile.m_poLanguage->Start(SMLanguage::mtFile);
   if( oFile.m_poLanguage->ProcessLines(m_sFilePathname) )
   {
      CMetricDetailsDlg oDlg;
      CString           sEntity;
      SMVersion         oVersion;

      oVersion.SetVersion(iMajorVersion, iMinorVersion);
      sEntity.LoadString(IDS_FILE_DETAILS_CAPTION);
      oDlg.m_sCaptionSuffix.Format(sEntity, oFile.PathName());

      // put our two items in first
      sEntity.LoadString(IDS_DETAILS_DIRECTORY_LABEL);
      oDlg.m_saEntities.Add(sEntity);
      oDlg.m_saValues.Add(sDirectory);
      sEntity.LoadString(IDS_CKPT_VIEW_FILES_LABEL);
      oDlg.m_saEntities.Add(sEntity);
      oDlg.m_saValues.Add(sFilename);

      // now let the language add some more entities
      oFile.m_poLanguage->GetDetails(oVersion, oDlg.m_saValues, oDlg.m_iaBlockDepths, false, true,
                                       oDlg.m_saEntities);
      oDlg.DoModal();
   }
   else
   {
      fResult = false;
   }
   return fResult;
}
