//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       FreqGraphView.cpp	[ FreqGraphView.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CFreqGraphView class
//
//***************************************************************************

#include "stdafx.h"

#include "SourceMetrics.h"
#include "SourceMetricsDoc.h"
#include "FreqGraphView.h"
#include "GraphOptionsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFreqGraphView

IMPLEMENT_DYNCREATE(CFreqGraphView, CGraphView)

BEGIN_MESSAGE_MAP(CFreqGraphView, CGraphView)
	//{{AFX_MSG_MAP(CFreqGraphView)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_GRAPH_OPTIONS, OnGraphOptions)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_OPTIONS, OnUpdateGraphOptions)
	ON_COMMAND(ID_GRAPH_VIEW_CLOSE, OnRequestClose)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_VIEW_CLOSE, OnUpdateRequestClose)
	ON_COMMAND(ID_FREQ_GRAPH_HELP, OnFreqGraphHelp)
	ON_UPDATE_COMMAND_UI(ID_FREQ_GRAPH_HELP, OnUpdateFreqGraphHelp)
	ON_COMMAND(ID_GRAPH_NEXT_METRIC, OnGraphNextMetric)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_NEXT_METRIC, OnUpdateGraphNextMetric)
	ON_COMMAND(ID_GRAPH_PREV_METRIC, OnGraphPrevMetric)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_PREV_METRIC, OnUpdateGraphPrevMetric)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CFreqGraphView::CFreqGraphView()
   : CGraphView(), m_poProject(NULL), m_iPlotMetricIndex(0), m_refPlotColor(CXCLR_BLUE)
{
	//{{AFX_DATA_INIT(CFreqGraphView)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CFreqGraphView::~CFreqGraphView()
{
   m_poSMDoc->UnsetView(m_lViewID);	
}

/////////////////////////////////////////////////////////////////////////////
// CFreqGraphView diagnostics

#ifdef _DEBUG
void CFreqGraphView::AssertValid() const
{
	CGraphView::AssertValid();
}

void CFreqGraphView::Dump(CDumpContext& dc) const
{
	CGraphView::Dump(dc);
}
#endif //_DEBUG

void CFreqGraphView::OnInitialUpdate() 
{
   SRGraphStyle *    poGraphStyle;              // scratch containers for graph initialization
   SRGraphDisplay *  poGraphDisplay;
   SRGraphTitle *    poGraphTitle;
	SRGraphBackground *  poGraphBackground;
   COLORREF             oBackgroundColor = GetSysColor(COLOR_WINDOW);

   // one-time initialization
   m_poSMDoc = dynamic_cast<CSourceMetricsDoc *>(m_pDocument);
   m_poProject = m_poSMDoc->GetProject();
   m_lViewID = (long)m_hWnd;                 // create a unique token for this view
   m_poSMDoc->SetView(m_lViewID);            // put ourselves in the view map
   m_sSection = CSourceMetricsApp::OPTION_SETTINGS + m_poProject->GetLanguage()->Name();  // set option profile key
   m_poProject->GetCheckpointByIndex(m_poSMDoc->CheckpointIndex(), m_oCkpt);

   // establish the metric to appear in the graph and graph color, if saved by user
   m_sPlotMetric =
         m_poSMDoc->GetApp()->GetProfileString(m_sSection, CSourceMetricsApp::FREQ_GRAPH_METRIC,
                                                m_oCkpt.m_poLanguage->GetMetricName(0));
   m_refPlotColor = 
         m_poSMDoc->GetApp()->GetProfileInt(m_sSection, CSourceMetricsApp::FREQ_GRAPH_COLOR, CXCLR_RED);
   SetMetricIndex();

   // paint a plain background to clear the client area
	poGraphBackground = new SRGraphBackground;
   poGraphStyle = poGraphBackground->GetStyle();
	poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
	poGraphStyle->SetColor(oBackgroundColor);
	m_oGraph.AddComponent(poGraphBackground);

   // initialize the graph display window
   poGraphDisplay = new SRGraphDisplay;
   poGraphDisplay->SetMeasurement(SRGraphComponent::PERCENT);
   poGraphDisplay->SetRect(0, 7, 100, 98);
   poGraphStyle = poGraphDisplay->GetStyle();
   poGraphStyle->SetGraphStyle(CX_GRAPH_EX_VBAR);
   poGraphStyle->SetAxisStyle(CX_AXIS_ISO_CLASSIC);
   poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
   poGraphStyle->SetColor(GetSysColor(COLOR_WINDOW));
   poGraphStyle->SetShowXTicks(FALSE);
   m_oGraph.AddComponent(poGraphDisplay);

   // initialize title
   poGraphTitle = new SRGraphTitle;
   poGraphTitle->SetMeasurement(SRGraphComponent::PERCENT);
   poGraphTitle->SetRect(0, 2, 100, 7);
   poGraphTitle->SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
   poGraphTitle->SetFontSize(12);                        // 12 point
   poGraphTitle->SetFontStyle(1);                        // bold
//   poGraphTitle->SetFontStyle(CX_FONT_AUTOSIZE bitor 1); // bold
   poGraphTitle->SetJustification(2);                    // center the title
   poGraphStyle = poGraphTitle->GetStyle();
   poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
   poGraphStyle->SetColor(GetSysColor(COLOR_WINDOW));
   m_oGraph.AddComponent(poGraphTitle);

   // get the window caption set
   m_sCaption.LoadString(IDS_CKPT_GRAPH_CAPTION);
   m_sCaption += _T(" '") + m_oCkpt.Name() + _T("'");
   GetParent()->SetWindowText(m_sCaption);
   CGraphView::OnInitialUpdate();
   return;	
}

void CFreqGraphView::OnUpdate(CView* /* pSender */, LPARAM /* lHint */, CObject* /* pHint */)
{
   CWaitCursor       oWaitCursor;         // let user know this may take awhile

   UpdateGraph();
   return;
}

// ************************************************************************************************
// pass data into the graph for display
void
CFreqGraphView::UpdateGraph()
{
   SRGraphStyle *    poGraphStyle;           // scratch container for bar color change
   CString           sTitle;
   SMFile            oFile;                  // scratch for access to metrics values
   int               i;                      // loop index
   int               iValueCount = m_oCkpt.FileCount();
   CString           sValue;                 // scratch value string
   long              alFrequencies[10];

   // set the graph's title
   sTitle.LoadString(IDS_FREQ_GRAPH_METRIC);
   sTitle += _T(" '") + m_sPlotMetric + _T("'");
   m_oGraph.SetGraphTitle(sTitle, 1);

   // load up the graph with labels and data items
   m_oCkpt.GetNextFile(oFile, true);      // get the first file and metric value, initialize search
   sValue = oFile.m_poLanguage->GetValue(m_iPlotMetricIndex);

   // start with all frequencies = 0
   for( i = 0; i < 10; ++i )
   {
      alFrequencies[i] = 0;
   }
   if( sValue.Find('.') >= 0 )            // values for this metric are floating point
   {
      double * padValues = new double[iValueCount];
      double   dSpan;
      double   dMaxValue = _tcstod(sValue, NULL);
      double   dValue;
      int      iLabelDigits;
      double   dOffset;       // offset for end of one cell
      double   dDeltaRange;   // upper range displayed in a histogram cell

      *padValues = dMaxValue;
      for( i = 1; i < iValueCount; ++i )
      {
         if( m_oCkpt.GetNextFile(oFile, i == 0) )
         {
            dValue = _tcstod(oFile.m_poLanguage->GetValue(m_iPlotMetricIndex, false), NULL);
            *(padValues + i) = dValue;
            dMaxValue = max(dMaxValue, dValue);
         }
      }
      // add small abount to make sure the cell index calculation never overflows
      dSpan = dMaxValue / 10.0 + 0.0009;                 // span of values for each of 10 bars
      for( i = 0; i < iValueCount; ++i )
      {
         alFrequencies[int(*(padValues + i) / dSpan)] += 1;   // accumulate frequency counts
      }
      // label the axes and pump out numbers to the graph
      sValue.Format("%d", (int)dMaxValue);
      iLabelDigits = 3 - sValue.GetLength();
      iLabelDigits = max(iLabelDigits, 0);               // must have positive value
      switch( iLabelDigits )
      {
      case 1:
         dOffset = -0.1;
         break;
      case 2:
         dOffset = -0.01;
         break;
      case 3:
         dOffset = -0.001;
         break;
      default:
         dOffset = -1.0;
         break;
      }
      dSpan -= 0.0009;                    // remove overflow buffer
      dDeltaRange = dSpan + dOffset;
      for( i = 0; i < 10; ++i )
      {
         double   dRangeLow = dSpan * i;

         if( i == 9 )
         {
            dDeltaRange = dSpan;          // last cell contains the remainder, all the way to the limit
         }
         sValue.Format("%.*f-%.*f", iLabelDigits, dRangeLow, iLabelDigits, dRangeLow + dDeltaRange);
         m_oGraph.SetValue(i, 0, double(alFrequencies[i]));
         m_oGraph.GetGroup(0)->GetIndex(i)->SetAnnotation(sValue);
      }
      delete padValues;
   }
   else                                   // metrics values are integers
   {
      long *   palValues = new long[iValueCount];
      int      iSpan;
      long     lMaxValue = _tcstol(sValue, NULL, 10);
      long     lValue;

      *palValues = lMaxValue;
      for( i = 1; i < iValueCount; ++i )
      {
         if( m_oCkpt.GetNextFile(oFile, i == 0) )
         {
            lValue = _tcstol(oFile.m_poLanguage->GetValue(m_iPlotMetricIndex, false), NULL, 10);
            *(palValues + i) = lValue;
            lMaxValue = max(lMaxValue, lValue);
         }
      }
      iSpan = (lMaxValue / 10) + 1;                 // span of values for each of 10 bars
      for( i = 0; i < iValueCount; ++i )
      {
         alFrequencies[*(palValues + i) / iSpan] += 1;  // accumulate frequency counts
      }
      // label the axes and pump out numbers to the graph
      for( i = 0; i < 10; ++i )
      {
         int   iRangeLow = iSpan * i;
         int   iRangeHigh = iRangeLow + iSpan - 1;

         m_oGraph.SetValue(i, 0, double(alFrequencies[i]));
         if( iRangeLow == iRangeHigh )
         {
            sValue.Format("%d", iRangeLow);
         }
         else
         {
            sValue.Format("%d-%d", iRangeLow, iRangeHigh);
         }
         m_oGraph.GetGroup(0)->GetIndex(i)->SetAnnotation(sValue);
      }
      delete palValues;
   }
   // set the bar color
   poGraphStyle = m_oGraph.GetGroup(0)->GetIndex(0)->GetStyle();
   poGraphStyle->SetInteriorStyle(CX_INTERIOR_COLOR);
   poGraphStyle->SetColor(m_refPlotColor);
   m_oGraph.SetGlobalStyle(0, 0, -1, -1, poGraphStyle);
   m_oGraph.SetSignificantChanges();

   return;
}

// ***********************************************************************************************
// initialize the graph object
void
CFreqGraphView::SetMetricIndex()
{
   m_iMetricsCount = m_poProject->GetLanguage()->Columns();
   for( int i = 0; i < m_iMetricsCount; ++i )
   {
      if( m_oCkpt.m_poLanguage->GetMetricName(i) == m_sPlotMetric )
      {
         m_iPlotMetricIndex = i;
         break;
      }
   }
}

// ***********************************************************************************************
// redisplay graph for different metric
void
CFreqGraphView::RedisplayGraph()
{
   m_poSMDoc->GetApp()->WriteProfileString(m_sSection, CSourceMetricsApp::HIST_GRAPH_METRIC, m_sPlotMetric);
   UpdateGraph();
   Invalidate();
}


// Display the value represented by the bar under the cursor
//
void CFreqGraphView::ShowFeedback(SRGraphFeedback *pFeedback)
{
   CString  sTipLabel;
	CString  sTip;
	CPoint   oPoint = m_MousePoint;

   sTipLabel.LoadString(IDS_FREQ_GRAPH_TIP_LABEL);
   sTip.Format("%s %g ", sTipLabel, m_oGraph.GetGroup(pFeedback->m_nGroup)->GetIndex(pFeedback->m_nIndex)->GetValue());
	ClientToScreen(&oPoint);
	m_GraphTip.ShowTip(sTip, oPoint, this);
}

/////////////////////////////////////////////////////////////////////////////
// CFreqGraphView message handlers
//

void CFreqGraphView::OnContextMenu(CWnd* /* pWnd */, CPoint point) 
{
	CMenu    oContextMenu;

   oContextMenu.LoadMenu(IDR_GRAPH_VIEW_CONTEXT);
   oContextMenu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN bitor TPM_RIGHTBUTTON,
                                                point.x, point.y, GetParentFrame());

   return;
}

void CFreqGraphView::OnGraphOptions() 
{
   CGraphOptionsDlg  oDlg(this);

   oDlg.m_saMetrics.SetSize(m_iMetricsCount);
   for( int i = 0; i < m_iMetricsCount; ++i )
   {
      oDlg.m_saMetrics[i] = m_oCkpt.m_poLanguage->GetMetricName(i);
   }
   oDlg.m_iPlotMetric = m_iPlotMetricIndex;
   oDlg.m_sPlotMetric = m_sPlotMetric;
   oDlg.m_refPanelColor = m_refPlotColor;

   if( oDlg.DoModal() == IDOK )
   {
      m_sPlotMetric = oDlg.m_sPlotMetric;
      m_refPlotColor = oDlg.m_refPanelColor;
      SetMetricIndex();
      m_poSMDoc->GetApp()->WriteProfileInt(m_sSection, CSourceMetricsApp::FREQ_GRAPH_COLOR, m_refPlotColor);
      RedisplayGraph();
   }
}

void CFreqGraphView::OnUpdateGraphOptions(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CFreqGraphView::OnGraphNextMetric() 
{
   m_iPlotMetricIndex++;
   if( m_iPlotMetricIndex >= m_iMetricsCount )
   {
      m_iPlotMetricIndex = 0;                   // wrap around to the beginning
   }
   m_sPlotMetric = m_oCkpt.m_poLanguage->GetMetricName(m_iPlotMetricIndex);
   RedisplayGraph();
}

void CFreqGraphView::OnUpdateGraphNextMetric(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CFreqGraphView::OnGraphPrevMetric() 
{
   m_iPlotMetricIndex--;
   if( m_iPlotMetricIndex < 0 )
   {
      m_iPlotMetricIndex = m_iMetricsCount - 1; // wrap around to the end
   }
   m_sPlotMetric = m_poProject->GetLanguage()->GetMetricName(m_iPlotMetricIndex);
   RedisplayGraph();
}

void CFreqGraphView::OnUpdateGraphPrevMetric(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CFreqGraphView::OnRequestClose() 
{
	GetParentFrame()->PostMessage(WM_CLOSE);	
}

void CFreqGraphView::OnUpdateRequestClose(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CFreqGraphView::OnFreqGraphHelp() 
{
	AfxGetApp()->WinHelp(ID_FREQ_GRAPH_HELP, HELP_CONTEXT);
}

void CFreqGraphView::OnUpdateFreqGraphHelp(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}
