/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.
 */

#ifndef _FILEOBJ_H_
#define _FILEOBJ_H_

#include "IDataPath.h"

// helper for creating psuedo IUNKNOWN implementation
#define DUMMY_IUNKNOWN	\
	HRESULT __stdcall QueryInterface(REFIID , void ** ppv){ return S_OK;}	\
	ULONG __stdcall AddRef(){return 0;}				\
	ULONG __stdcall Release(){return 0;}			

// This is a pale shade of the class we use in the real system. It contains
// only the minimal funcionality for reading and writing from files.
class FILE_OBJECT : public IDataSink2, public IDataSource2
{
public:
	DUMMY_IUNKNOWN

	FILE_OBJECT();
	~FILE_OBJECT();

	// creates Win32 file - The parameters are explained in win32 manual - see CreateFile() 
	// The fileobj will close the handle when calling close
	HRESULT CreateFile(
	  LPCTSTR lpFileName,          // pointer to name of the file
	  DWORD dwDesiredAccess,       // access (read-write) mode
	  DWORD dwShareMode,           // share mode
	  DWORD dwCreationDisposition,  // how to create
	  DWORD dwFlagsAndAttributes  // file attributes
	);

	HRESULT Close();

	HRESULT UseFile( HANDLE hFile );


	// -- IDataSink2 Interface 
	virtual HRESULT Write( char* buff, ULONG size, ULONG *pcbWritten = NULL );
	virtual HRESULT Flush(){ return E_NOTIMPL; }
	virtual HRESULT Write_( char* buff, ULONG size);
	
	// -- IDataSource2 Interface  
	virtual HRESULT	Read(char * buff, ULONG sizeToRead, ULONG * actualRead );
	virtual HRESULT	Read_(char * buff, ULONG sizeToRead );
	virtual	BOOL	AtEnd();
	virtual HRESULT		Size(offset_t & theSize) { return E_NOTIMPL; } 


private:
	HANDLE	m_filePtr;
	BOOL	m_isEndOfFile;
#define _MAX_FILE_NAME 256
	char m_FileName[_MAX_FILE_NAME];
	bool m_bShouldClose;
};

#endif