/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.

*/

#ifndef _SourceTestPoint_H
#define _SourceTestPoint_H

#include "IDataPath.h"
//#include "SourceStream.h"
#include "SinkTee.h" 

#if 0
/*
The source testpoint is an IDataSource filter that writes whatever was 
read from it into a dumping sink. The dumping sink is a test point supplied
by the TestPointManager.
Errors in the source testpoint must be handled gracefully ( and silently ).
*/
class SourceTestPoint // ?? : public IDataSource
{
public:
	SourceTestPoint(); // create it unnamed
	virtual ~SourceTestPoint(); 

	/*
	Connect the pInput to a SourceTee, create a TP handler and
	connect it to the SourceTee. Return the pulling leg of the SourceTee
	in pSource.
	IN ANY CASE, IT IS GUARANTEED THAT ppSource EITHER CONNECTS TO pInput
	OR TO SOME OTHER VALID, INVARIANT FILTER.

	Return:
	S_OK	- All connections made succesfuly.
	S_FALSE - the TP is not active; connection made through.
	E_INVALIDARG - bad TestPointId
	E_NULLPTR	 - pSource == NULL
	E_NOINTERFACE - handler does not support the required interface
	E_CLASSNOTREG - the handler is not registered
	other errors such as "file not found" - may indicate bad registry entries.
	*/
	HRESULT Connect(IN const GUID & TestPointId, // required TP ID
		IN	IDataSource* pInput,	// pointer to the upstream stage
		OUT IDataSource** ppSource		// pointer to the new source
		);

	// Disconnect the handler.
	// WARNING: the pSource returned from Connect() is invalid after this call.
	HRESULT Disconnect();

private:
	IUnknown * m_pHandlerUnk;
	SourceStreamTee2 * m_pTee; 

};

#endif

/*
The sink testpoint is an IDataSink filter that writes whatever was 
written to it into a dumping sink. The dumping sink is a test point supplied
by the TestPointManager.
Errors in the sink testpoint must be handled gracefully ( and silently ).
*/
class SinkTestPoint
{
public:
	SinkTestPoint(); // create it unnamed
	virtual ~SinkTestPoint(); 

	/*
	Connect the pInput to a SinkTee, create a TP handler and
	connect it to the SinkTee.
	IN ANY CASE, IT IS GUARANTEED THAT ppSource EITHER CONNECTS TO pInput
	OR TO SOME OTHER VALID, INVARIANT FILTER.

	Return:
	S_OK	- All connections made succesfuly.
	S_FALSE - the TP is not active; connection made through.
	E_INVALIDARG - bad TestPointId
	E_NULLPTR	 - pSource == NULL
	E_NOINTERFACE - handler does not support the required interface
	E_CLASSNOTREG - the handler is not registered
	other errors such as "file not found" - may indicate bad registry entries.
	*/
	bool IsConnected(){ return m_bIsConnected;}
	HRESULT Connect(IN const GUID & TestPointId, // required TP ID
		IN	IDataSink2* pOutput,	// pointer to the downstream stage
		OUT IDataSink2** ppNewSink		// pointer to the new sink
		);

	// Disconnect the handler.
	// WARNING: the pSource returned from Connect() is invalid after this call.
	HRESULT Disconnect();

private:
	IUnknown * m_pHandlerUnk;
	SinkTee2 * m_pTee; 
	bool m_bIsConnected;
};


#endif