/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.
*/

// CRegistry.cpp

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "CRegistry.H"

CRegistry::CRegistry() 
{
	this->hRegKey = HKEY_LOCAL_MACHINE;
}

CRegistry::~CRegistry() 
{
	RegCloseKey( hRegKey );
}

void CRegistry::StopParamLogger()
{
}

LONG CRegistry::SetRoot(
	LPCTSTR path,
	REGSAM samDesired )
{
	DWORD ntStat;

	ntStat = RegOpenKeyEx(
		HKEY_LOCAL_MACHINE,			 	// handle of open key
		path,     							 	// address of name of subkey to open
		(DWORD)0,									// Reserved
		(REGSAM)samDesired,					// Security access mask
		&hRegKey									// Address of handle of open key
	);
	strcpy( this->currentPath, path );
	return ntStat;
}

/*
	The ChangeDirectory adds an up ( ".." ) operation not supported by winNt.
	The current path is always kept in the class private data so that 
	 when an up operation is asked, the new path is created from the 
	 current path.
*/													 						  

LONG CRegistry::ChangeDirectory(
	LPCTSTR path ,
	REGSAM samDesired )
{
	DWORD ntStat;
	HKEY hTmp;
	PTCHAR pch;
	char tmpPath[200];

	if ( strcmp( path, ".." )	== 0 ) {
		//
		// Simulate Up
		//
		strcpy( tmpPath, this->currentPath );
		
		//
		// Find the last '\' character in the current path
		//
		for ( pch = tmpPath + strlen( tmpPath );
					pch > tmpPath && *pch != '\\' ;
					pch-- );
		//
		// Change the last '\' char to '\0' so that actually, the last dir 
		//  is removed from the string. If the current path contain no 
		//  '\0' chars than	it is an error
		//

		if ( pch == tmpPath ) // No more up !!!
			return ERROR_FILE_NOT_FOUND;
		*pch = '\0';
		
		ntStat = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,			 	// handle of open key
			tmpPath,								 	// address of name of subkey to open
			(DWORD)0,									// Reserved
			(REGSAM)samDesired,					// Security access mask
			&hTmp											// Address of handle of open key
		);
		if ( ntStat == ERROR_SUCCESS ) 	{
			strcpy( this->currentPath, tmpPath );
			RegCloseKey( hRegKey );
			hRegKey = hTmp;
		}
		else
		{}
		return ntStat;
	
	} else	{

		ntStat = RegOpenKeyEx(
			hRegKey,								 	// handle of open key
			path,     							 	// address of name of subkey to open
			(DWORD)0,									// Reserved
			(REGSAM)samDesired,					// Security access mask
			&hTmp											// Address of handle of open key
		);
		if ( ntStat == ERROR_SUCCESS ) {
			sprintf( this->currentPath, "%s\\%s", this->currentPath, path );
			RegCloseKey( hRegKey );
			hRegKey = hTmp;
		}

	}
	return ntStat;
}

class Names
{
public:
	Names() { m_nEntries = 0; }
	~Names() 
	{ 
		for (int i=0; i < m_nEntries; i++)
		{
			free(m_names[i]);
		}
		m_nEntries = 0;
	}
	
	BOOL Add( const char *name )
	{
		if (m_nEntries >= 1024)
			return FALSE;
		m_names[m_nEntries] = strdup(name);
		m_nEntries++;
		return TRUE;
	}

	const char *Get( ULONG idx )
	{
		if (idx >= static_cast<UINT>(m_nEntries))
			return NULL;
		return m_names[idx];
	}

private:
	int m_nEntries;
	char *m_names[1024];
};
/*
LONG CRegistry::EnumerateSubTree( 
	HKEY hKey, 
	LPCTSTR lpSubKey, 
	BOOL IsEnumerateValues, 
	EnumerationFunc pEnumFunc, 
	DWORD param )
{
	long retVal;
	int index;
	HKEY hSubKey;
	BOOL bErr;
	DWORD dwErr;

	retVal = RegOpenKeyEx(
		hKey, 
		lpSubKey, 
		0, // ulOptions, reserved
		KEY_ALL_ACCESS,
		&hSubKey);

	if(retVal != ERROR_SUCCESS)
		return retVal;
	
	{ // Shahar (12-Apr-99): Need to do two phases because the enum function may delete entries
		//  which will effect the enumeration
		// Phase 1: Read all subkeys names to buffer
		ULONG nSubKeys;
		Names names;
		char subkey_name_buffer[512];

		for (index=0; ; index++)
		{
			DWORD cbname = 512;
			retVal = RegEnumKeyEx(hSubKey, index, subkey_name_buffer, &cbname, 
				NULL,  // pReserved 
				NULL, NULL, // lpClass, lpcbClass
				NULL);

			if (retVal == ERROR_NO_MORE_ITEMS)
				break;

			if(	retVal != ERROR_SUCCESS)
			{
				RegCloseKey(hSubKey);
				return retVal;
			}
			if (!names.Add(subkey_name_buffer))
			{
				RegCloseKey(hSubKey);
				return -1;
			}
		}

		// Phase 2: Call functions
		for (index=0; ; index++)
		{
			const char *key_name = names.Get( index );
			if (key_name == NULL)
				break;
			bErr = (*pEnumFunc)(hKey, lpSubKey, key_name, NULL, param);
			if (!bErr)
			{ // Stop enumerating
				RegCloseKey(hSubKey);
				return ERROR_SUCCESS;
			}				
		}
	}

	if (IsEnumerateValues)
	{
		// Enumerate Values
		for (index=0; ; index++)
		{
			char value_name[512];

			DWORD cbname = 512;
			retVal = RegEnumValue(hSubKey, index, value_name, &cbname, 
				NULL,  // pReserved 
				NULL,  // address of buffer for type code
				NULL, // address of buffer for value data
				NULL); // address for size of data buffer

			if (retVal == ERROR_NO_MORE_ITEMS)
				break;

			if(	retVal != ERROR_SUCCESS)
			{
				RegCloseKey(hSubKey);
				return retVal;
			}
			bErr = (*pEnumFunc)(hKey, lpSubKey, NULL, value_name, param);
			if (!bErr)
			{ // Stop enumerating
				RegCloseKey(hSubKey);
				return ERROR_SUCCESS;
			}				
		}
	}
	RegCloseKey(hSubKey);
	return ERROR_SUCCESS;
}
*/
typedef struct tagDeleteTreeData
{
	DWORD result_code;
} _DeleteTreeData;

/*
BOOL CRegistry::_DeleteSubTree( 
	HKEY hKey, 
	LPCSTR lpBaseKeyName, 
	LPCSTR lpSubTreeName, 
	LPCSTR ValueName, 
	DWORD param )
{
	DWORD dwErr;
	_DeleteTreeData *pDeleteData = (_DeleteTreeData*)param;

	if (lpSubTreeName != NULL)
	{ // This is a subtree, first, remove all it's values, then remove the subtree
		CRegistry reg;
		char tree_name[1024];

		sprintf( tree_name, "%s\\%s", lpBaseKeyName, lpSubTreeName );

		dwErr = CRegistry::EnumerateSubTree(
			hKey,
			tree_name,
			FALSE,
			(CRegistry::EnumerationFunc)_DeleteSubTree,
			param);
		if (dwErr != ERROR_SUCCESS || 
				pDeleteData->result_code != ERROR_SUCCESS)
		{
			printf( "ERROR: Enumeration of subtree: '%s' failed, dwErr = %d", tree_name, dwErr);
			pDeleteData->result_code = dwErr;
			return FALSE;
		}
		{
			HKEY base_key;

			dwErr = RegOpenKeyEx(
				hKey,			 	// handle of open key
				lpBaseKeyName,     							 	// address of name of subkey to open
				(DWORD)0,									// Reserved
				(REGSAM)KEY_ALL_ACCESS,					// Security access mask
				&base_key									// Address of handle of open key
			);
			
			if (dwErr != ERROR_SUCCESS)
			{
				printf( "ERROR: RegCreateKeyEx() of key: '%s' failed, dwErr = %d", lpBaseKeyName, dwErr);
				pDeleteData->result_code = dwErr;
				return FALSE;
			}
			dwErr = CRegistry::DeleteKey(base_key, lpSubTreeName);
			if (dwErr != ERROR_SUCCESS)
			{
				RegCloseKey(base_key);
				printf( "ERROR: DeleteKey : '%s' failed, dwErr = %d", tree_name, dwErr);
				pDeleteData->result_code = dwErr;
				return FALSE;
			}
			RegCloseKey(base_key);

		}

	} 
	return TRUE;
}

LONG CRegistry::DeleteKey(
	HKEY hKey, 
	LPCTSTR lpSubKeyName)
{
	LONG dwErr;

	dwErr = RegDeleteKey(hKey, lpSubKeyName);
	return dwErr;
}

LONG CRegistry::RemoveSubTree( 
	HKEY hKey, 
	LPCTSTR lpSubKeyName )
{
	DWORD dwErr;
	_DeleteTreeData delete_data;

	// Remove all content of lpSubKeyName
	delete_data.result_code = ERROR_SUCCESS;
	dwErr = EnumerateSubTree( hKey, lpSubKeyName, FALSE, (CRegistry::EnumerationFunc)_DeleteSubTree, (DWORD)&delete_data);
	if (dwErr != ERROR_SUCCESS)
		return dwErr;
	if (delete_data.result_code != ERROR_SUCCESS)
		return delete_data.result_code;
	// Remove the key itself
	return CRegistry::DeleteKey(hKey, lpSubKeyName);
}

typedef struct tagCopyTreeData
{
	HKEY hDestKey;
	const char *pDestBaseName;
	DWORD result_code;
} _CopyTreeData;

BOOL CRegistry::_CopySubTree( 
	HKEY hKey, 
	LPCSTR lpBaseKeyName, 
	LPCSTR lpSubKeyName, 
	LPCSTR ValueName, 
	DWORD param )
{
	DWORD dwErr;
	_CopyTreeData *pCopy_tree_data = (_CopyTreeData*)param;

	if (lpSubKeyName != NULL)
	{ // This is a subtree, first, remove all it's values, then remove the subtree
		char dest_name[1024];
		char src_name[1024];
		HKEY h_dest;
		_CopyTreeData copy_data;

		sprintf( dest_name, "%s\\%s", pCopy_tree_data->pDestBaseName, lpSubKeyName );
		dwErr = RegCreateKeyEx(
			pCopy_tree_data->hDestKey,
			dest_name,
			0, // Reserved; must be zero. 
			NULL, // 	Pointer to a null-terminated string that specifies the class 
			REG_OPTION_NON_VOLATILE,
			KEY_ALL_ACCESS,
			NULL, // Pointer to a SECURITY_ATTRIBUTES structure 
			&h_dest, 
			NULL );	// address of disposition value buffer
		
		if (dwErr != ERROR_SUCCESS)
		{
			printf( "ERROR: RegCreateKeyEx() of key: '%s' failed, dwErr = %d", dest_name, dwErr);
			pCopy_tree_data->result_code = dwErr;
			return FALSE;
		}
		RegCloseKey(h_dest);
		copy_data.hDestKey = pCopy_tree_data->hDestKey;
		copy_data.pDestBaseName = dest_name;
		sprintf( src_name, "%s\\%s", lpBaseKeyName, lpSubKeyName );
		dwErr = EnumerateSubTree( hKey, src_name, TRUE, (CRegistry::EnumerationFunc)_CopySubTree, (DWORD)&copy_data);
		if (dwErr != ERROR_SUCCESS)
		{
			printf( "ERROR: EnumerateSubTree() of key: '%s' failed, dwErr = %d", dest_name, dwErr);
			pCopy_tree_data->result_code = dwErr;
			return FALSE;
		}
	}
	if (ValueName != NULL)
	{
		CRegistry reg_src, reg_dst;
		char value_data[2048];
		DWORD type_code, value_data_sz;

		dwErr = reg_src.SetRoot( lpBaseKeyName, KEY_EXECUTE );
		value_data_sz = 2048;
		dwErr = reg_src.QueryValue( ValueName, &type_code, (LPBYTE)value_data, &value_data_sz );

		dwErr = reg_dst.SetRoot( pCopy_tree_data->pDestBaseName, KEY_SET_VALUE );
		dwErr = reg_dst.SetValue( ValueName, type_code, (LPBYTE)value_data, value_data_sz );
	} 
	return TRUE;
}

LONG CRegistry::CopyTree(
	HKEY hKeyDest, LPCTSTR DestName, 
	HKEY hKeySrc, LPCTSTR SrcName )
{
	DWORD dwErr;
	HKEY h_dest, h_src;

	// 1. Create the dest key
	// 2. Enumerate through the source tree:
	//  2.1 foreach key, create a similar dest key, iterate through the source again
	//  2.2 foreach value, copu from source to dest
	_CopyTreeData copy_data;

	copy_data.hDestKey = hKeyDest;
	copy_data.pDestBaseName = DestName;
	copy_data.result_code = ERROR_SUCCESS;
	dwErr = EnumerateSubTree( hKeySrc, SrcName, TRUE, (CRegistry::EnumerationFunc)_CopySubTree, (DWORD)&copy_data);
	if (dwErr != ERROR_SUCCESS)
		return dwErr;
	return copy_data.result_code;
}
*/
/*
LONG CRegistry::DeleteSubKey( 
	HKEY hKey, 
	LPCTSTR lpSubKey )
{
	long retVal;
	DWORD index=0;
	char subkeyName[256];
	HKEY hSubKey;

	// defining a new key for the enumaration to come.
	retVal=RegOpenKeyEx(hKey, lpSubKey, 
		0, // ulOptions, reserved
		KEY_ALL_ACCESS,
		&hSubKey);
	if(retVal != ERROR_SUCCESS)
	{
		return retVal;
	}

	// I'm enumerating because the subkey can't be deleted before it is
	// clear of underlying subkeys, so we have to delete them first.
	retVal=RegEnumKey(hSubKey,index,subkeyName,256);
	if((retVal!=ERROR_SUCCESS)&&(retVal!=ERROR_NO_MORE_ITEMS)){
		return retVal;
	}
	
	// For each subkey found, we delete it. and enumerate the next
	// until no more underlying subkeys are present.
	while(retVal!=ERROR_NO_MORE_ITEMS){
		retVal= RegDeleteKey(hSubKey,subkeyName);
		if(retVal!=ERROR_SUCCESS){
			return retVal;
		}
		index++;
		retVal=	RegEnumKey(hSubKey,index,subkeyName,256);
		if((retVal!=ERROR_SUCCESS)&&(retVal!=ERROR_NO_MORE_ITEMS)){
			return retVal;
		}
	}

	// close the subkey defined erlier.
	retVal=RegCloseKey(hSubKey);
	if(retVal!=ERROR_SUCCESS){
		return retVal;
	}
	// Delete the subkey at last.
	return RegDeleteKey(m_key,m_lpcszSubKey);
	
}
*/
LONG CRegistry::DirKeys(
	DWORD iSubkey,
	LPTSTR lpszName,
	LPDWORD lpcchName )
{
	LONG ntStat;
	
	ntStat = RegEnumKeyEx(
		hRegKey,					// Handle of key to enumerate
		iSubkey,					// Index of subkey to enumerate			
		lpszName,					// Address of buffer for subkey name
		lpcchName,				// Address for size of subkey buffer
		NULL,							// Reserved
		NULL,							// Address of buffer for class string
		NULL,							// Address for size of class buffer
		NULL							// Address for time key last wriiten to
	);

	return ntStat; // Return ERROR_SUCCESS or ERROR_NO_MORE_ITEMS
}

LONG CRegistry::DirValues(
		DWORD   iValue,
		LPTSTR  lpszValue,  // Points to value name  
		LPDWORD lpcchValue,
		LPDWORD lpdwType,
		LPBYTE	lpbData,
		LPDWORD lpcbData )
{
	LONG ntStat;

	ntStat = RegEnumValue(
			hRegKey,				// Handle of key to query
			iValue,					// Index of value to query
			lpszValue,			// Address of buffer for value string
			lpcchValue,			// Address for size of value buffer	
			NULL,						// reserved	
			lpdwType,				// Address of buffer for type code	
			lpbData,				// Address of buffer for value data	
			lpcbData				// Address for size of data buffer
	);
	return ntStat; // Return ERROR_SUCCESS or ERROR_NO_MORE_ITEMS
}

LONG CRegistry::QueryValue(
		LPCTSTR  lpszValueName,	// Address of name of value to query
		LPDWORD lpdwType,				// Address of buffer for type code
		LPBYTE  lpbData,				// Address of buffer for value data
		LPDWORD lpcbData )			// Address for size of data buffer
{
	LONG ntStat;

	ntStat = RegQueryValueEx(
			hRegKey,				// Handle of key to query
			lpszValueName,	// address of name of value to query	
			NULL,						// reserved	
			lpdwType,				// address of buffer for value type	
			lpbData,				// address of data buffer	
			lpcbData				// address of data buffer size	
	);
	
	return ntStat;
}

LONG CRegistry::SetValue(
		IN LPCTSTR lpszValueName,	 // Address of name of value to set
		IN DWORD  fdwType,				 // Flag for value type
		IN CONST  LPBYTE  lpbData, // Address of buffer for value data
		IN DWORD  cbData )			   // Size of data buffer
{
	LONG ntStat;

	ntStat = RegSetValueEx(
			hRegKey,				// Handle of key to set value for
			lpszValueName,	// address of name of value to set
			(DWORD)0,       // Reserved	
			fdwType,				// address of buffer for value type	
			lpbData,				// address of data buffer	
			cbData	  			// address of data buffer size	
	);

	return ntStat;
}

LONG CRegistry::QueryInfoKey(
		LPDWORD lpcSubKeys,					// Address of buffer for number of subkeys	
		LPDWORD lpcValues,					// Address of buffer for number of value entries	
		LPDWORD lpcchMaxSubkey,			// Address of buffer for longest subkey name length	
		LPDWORD lpcchMaxValueName,	// Address of buffer for longest value name length	
		LPDWORD lpcbMaxValueData )	// Address of buffer for longest value data length	
{
	LONG ntStat;

	ntStat = RegQueryInfoKey(
		hRegKey,						// Handle of key to query	
		NULL,								// Address of buffer for class string	
		NULL, 							// Address of size of class string buffer	
		NULL,				 				// Reserved	
		lpcSubKeys,					// Address of buffer for number of subkeys	
		lpcchMaxSubkey,			// Address of buffer for longest subkey name length	
		NULL,								// Address of buffer for longest class string length	
		lpcValues,					// Address of buffer for number of value entries	
		lpcchMaxValueName,	// Address of buffer for longest value name length
		lpcbMaxValueData,		// Address of buffer for longest value data length	
		NULL,								// Address of buffer for security descriptor length	
		NULL 								// Address of buffer for last write time	
	);
	return ntStat;
}

DWORD CRegistry::FlushKey()
{																			  
	DWORD dwErr;

	dwErr = RegFlushKey( hRegKey );
	return dwErr;
}

/////////////////////////////////////////////////////////////////////////
// CRegistry_EX class
/////////////////////////////////////////////////////////////////////////

LONG CRegistry_EX::getValueDW( 
	LPTSTR name, 
	LPDWORD pValue )
{
	DWORD typeCode, bufSize=sizeof( DWORD );
	LONG result;

	result = CRegistry::QueryValue(
				(LPTSTR)name, // Address of name of value to query
				&typeCode,
				(LPBYTE)pValue, // Address of buffer for value data
				&bufSize );

	if ( result != ERROR_SUCCESS )
		return result;

	if ( typeCode != REG_DWORD )
		return 0x1;

	return ERROR_SUCCESS;
}

LONG CRegistry_EX::getValueREG_SZ( 
	LPTSTR name, 
	LPBYTE pStr, 
	PDWORD pStrSize )
{
	DWORD typeCode;
	LONG result;

	result = CRegistry::QueryValue(
				(LPTSTR)name, // Address of name of value to query
				&typeCode,
				(LPBYTE)pStr, // Address of buffer for value data
				pStrSize );

	if ( result != ERROR_SUCCESS )
		return result;

	if ( typeCode != REG_SZ )
		return 0x1;

	pStr[ *pStrSize ] = '\0';

	return ERROR_SUCCESS;
}

LONG CRegistry_EX::getValueREG_MULTI_SZ( 
	LPTSTR name, 
	LPBYTE pStr, 
	PDWORD pStrSize )
{
	DWORD typeCode;
	LONG result;

	result = CRegistry::QueryValue(
				(LPTSTR)name, // Address of name of value to query
				&typeCode,
				(LPBYTE)pStr, // Address of buffer for value data
				pStrSize );

	if ( result != ERROR_SUCCESS )
		return result;

	if ( typeCode != REG_MULTI_SZ )
		return 0x1;

	pStr[ *pStrSize ] = '\0';
	pStr[ *pStrSize+1 ] = '\0';

	return ERROR_SUCCESS;
}
