// Partial Copyright "Inside COM" By Dale Rogerson , MS press


//
// ComRegistry.cpp
//
#include <stdlib.h>
#include <objbase.h>
#include <assert.h>
#include "ComRegistry.h"

#ifdef unix
#include "WinReg.h"

static char gModuleName[500];
void SetModuleName(const char * moduleName)
{
	strcpy(gModuleName, moduleName);
}


void CoTaskMemFree( void * p)
{
	if( 0 != p )
		free(p);
}
///////////////////////////////////////////////////////////////
// mockup of the original Windows function
DWORD GetModuleFileName(void * hModule, 
		                char * szModule,
		                DWORD buffLen) 
{
	int len = strlen(gModuleName);
	if( len >= buffLen)
		return 0;
	strcpy(szModule,gModuleName);
	return len;
}
#endif
////////////////////////////////////////////////////////
//
// Internal helper functions prototypes
//

// Set the given key and its value.
long setKeyAndValue(const char* pszPath,
                    const char* szSubkey,
                    const char* szValue,
					const char* szValueName = NULL) ;

// Convert a CLSID into a char string.
void CLSIDtochar(const CLSID& clsid, 
                 char* szCLSID,
                 int length) ;

// Delete szKeyChild and all of its descendents.
LONG recursiveDeleteKey(HKEY hKeyParent, const char* szKeyChild) ;

////////////////////////////////////////////////////////
//
// Constants
//

// Size of a CLSID as a string
const int CLSID_STRING_SIZE = 39 ;

/////////////////////////////////////////////////////////
//
// Public function implementation
//

//
// Register the component in the registry.
//
// noam 27/04/99: added error checking from the registry
//

#define TRY_(s) if( (stat = s) != ERROR_SUCCESS) throw stat

HRESULT RegisterServer(HINSTANCE hModule,            // DLL module handle
                       const CLSID& clsid,         // Class ID
                       const char* szFriendlyName, // Friendly Name
                       const char* szVerIndProgID, // Programmatic
                       const char* szProgID)       //   IDs
{
	LONG stat;
	// Get server location.
	char szModule[512] ;
	DWORD dwResult =
		::GetModuleFileName(hModule, 
		                    szModule,
		                    sizeof(szModule)/sizeof(char)) ;
	assert(dwResult != 0) ;

	// Convert the CLSID into a char.
	char szCLSID[CLSID_STRING_SIZE] ;
	CLSIDtochar(clsid, szCLSID, sizeof(szCLSID)) ;

	// Build the key CLSID\\{...}
	char szKey[64] ;
	strcpy(szKey, "CLSID\\") ;
	strcat(szKey, szCLSID) ;
	try
	{
		// Add the CLSID to the registry.
		TRY_(setKeyAndValue(szKey, NULL, szFriendlyName)) ;

		// Add the server filename subkey under the CLSID key.
		TRY_(setKeyAndValue(szKey, "InprocServer32", szModule)) ;
		TRY_(setKeyAndValue(szKey, "InprocServer32", "Both" ,"ThreadingModel")) ; 

		// Add the ProgID subkey under the CLSID key.
		TRY_(setKeyAndValue(szKey, "ProgID", szProgID)) ;

		// Add the version-independent ProgID subkey under CLSID key.
		TRY_(setKeyAndValue(szKey, "VersionIndependentProgID",
					   szVerIndProgID)) ;

		// Add the version-independent ProgID subkey under HKEY_CLASSES_ROOT.
		TRY_(setKeyAndValue(szVerIndProgID, NULL, szFriendlyName)) ; 
		TRY_(setKeyAndValue(szVerIndProgID, "CLSID", szCLSID)) ;
		TRY_(setKeyAndValue(szVerIndProgID, "CurVer", szProgID)) ;

		// Add the versioned ProgID subkey under HKEY_CLASSES_ROOT.
		TRY_(setKeyAndValue(szProgID, NULL, szFriendlyName)) ; 
		TRY_(setKeyAndValue(szProgID, "CLSID", szCLSID)) ;
	}
	catch( LONG result)
	{
		return HRESULT_FROM_WIN32(result);
	}
	return S_OK ;
}

#undef TRY_
//
// Remove the component from the registry.
//
LONG UnregisterServer(const CLSID& clsid,         // Class ID
                      const char* szVerIndProgID, // Programmatic
                      const char* szProgID)       //   IDs
{
	// Convert the CLSID into a char.
	char szCLSID[CLSID_STRING_SIZE] ;
	CLSIDtochar(clsid, szCLSID, sizeof(szCLSID)) ;

	// Build the key CLSID\\{...}
	char szKey[64] ;
	strcpy(szKey, "CLSID\\") ;
	strcat(szKey, szCLSID) ;

	// Delete the CLSID Key - CLSID\{...}
	LONG lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szKey) ;
	assert((lResult == ERROR_SUCCESS) ||
	       (lResult == ERROR_FILE_NOT_FOUND)) ; // Subkey may not exist.

	// Delete the version-independent ProgID Key.
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szVerIndProgID) ;
	assert((lResult == ERROR_SUCCESS) ||
	       (lResult == ERROR_FILE_NOT_FOUND)) ; // Subkey may not exist.

	// Delete the ProgID key.
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szProgID) ;
	assert((lResult == ERROR_SUCCESS) ||
	       (lResult == ERROR_FILE_NOT_FOUND)) ; // Subkey may not exist.

	return S_OK ;
}

///////////////////////////////////////////////////////////
//
// Internal helper functions
//

// Convert a CLSID to a char string.
void CLSIDtochar(const CLSID& clsid,
                 char* szCLSID,
                 int length)
{
	assert(length >= CLSID_STRING_SIZE) ;
	// Get CLSID
	LPOLESTR wszCLSID = NULL ;
	HRESULT hr = StringFromCLSID(clsid, &wszCLSID) ;
	assert(SUCCEEDED(hr)) ;

	// Covert from wide characters to non-wide.
	wcstombs(szCLSID, wszCLSID, length) ;

	// Free memory.
	CoTaskMemFree(wszCLSID) ;
}

//
// Delete a key and all of its descendents.
//
LONG recursiveDeleteKey(HKEY hKeyParent,           // Parent of key to delete
                        const char* lpszKeyChild)  // Key to delete
{
	// Open the child.
	HKEY hKeyChild ;
	LONG lRes = RegOpenKeyEx(hKeyParent, lpszKeyChild, 0,
	                         KEY_ALL_ACCESS, &hKeyChild) ;
	if (lRes != ERROR_SUCCESS)
	{
		return lRes ;
	}

	// Enumerate all of the decendents of this child.
	FILETIME time ;
	char szBuffer[256] ;
	DWORD dwSize = 256 ;
	while (RegEnumKeyEx(hKeyChild, 0, szBuffer, &dwSize, NULL,
	                    NULL, NULL, &time) == S_OK)
	{
		// Delete the decendents of this child.
		lRes = recursiveDeleteKey(hKeyChild, szBuffer) ;
		if (lRes != ERROR_SUCCESS)
		{
			// Cleanup before exiting.
			RegCloseKey(hKeyChild) ;
			return lRes;
		}
		dwSize = 256 ;
	}

	// Close the child.
	RegCloseKey(hKeyChild) ;

	// Delete this child.
	return RegDeleteKey(hKeyParent, lpszKeyChild) ;
}

//
// Create a key and set its value.
//   - This helper function was borrowed and modifed from
//     Kraig Brockschmidt's book Inside OLE.
//
// noam 27/04/99: changed the ret value to be long - what is returned from Reg*
// ERROR_SUCCESS if OK.
long setKeyAndValue(const char* szKey,
                    const char* szSubkey,
                    const char* szValue,
					const char* szValueName)
{
	HKEY hKey;
	char szKeyBuf[1024] ;

	// Copy keyname into buffer.
	strcpy(szKeyBuf, szKey) ;

	// Add subkey name to buffer.
	if (szSubkey != NULL)
	{
		strcat(szKeyBuf, "\\") ;
		strcat(szKeyBuf, szSubkey ) ;
	}

	// Create and open key and subkey.
	long lResult = RegCreateKeyEx(HKEY_CLASSES_ROOT ,
	                              szKeyBuf, 
	                              0, NULL, REG_OPTION_NON_VOLATILE,
	                              KEY_ALL_ACCESS, NULL, 
	                              &hKey, NULL) ;
	if (lResult != ERROR_SUCCESS)
	{
		// noam 27/04/99 return FALSE ;
		return lResult;
	}

	// Set the Value.
	if (szValue != NULL)
	{
		lResult = RegSetValueEx(hKey, szValueName, 0, REG_SZ, 
		              (BYTE *)szValue, 
		              strlen(szValue)+1) ;
		if (lResult != ERROR_SUCCESS)
		{
			return lResult;
		}
	}

	RegCloseKey(hKey) ;
	return ERROR_SUCCESS ;
}
