/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.
*/

#include <StdAfx.h>
#include "fileObj.h"

FILE_OBJECT::~FILE_OBJECT()
{
	Close();
}

HRESULT FILE_OBJECT::Close()
{
	TRACE("Closing %s\n", m_FileName);
	if(m_bShouldClose) // allow multiple calls to Close()
	{
		CloseHandle(m_filePtr);
		m_bShouldClose = false;
		strncpy(m_FileName, "No file",_MAX_FILE_NAME);
	}
	return S_OK;
}
HRESULT FILE_OBJECT::Write_( char* buff, ULONG size)
{
	ULONG actuallyWritten, offset;
	offset = 0;
	while( size > 0 )
	{
		HRESULT	stat = Write(buff + offset,size, &actuallyWritten);
		if(FAILED(stat)) return stat;
		// we are not tolerating zero bytes output; we assume
		// the Write() above will block untill data is written or an error
		if(actuallyWritten == 0 ) 
			return E_FAIL;
		size -= actuallyWritten;
		offset += actuallyWritten;
	}
	return S_OK;
}

HRESULT FILE_OBJECT::Write( 
	char* buff, 
	ULONG size, 
	ULONG *pcbWritten )
{
	HRESULT hRes  = E_FAIL;

	BOOL bErr;
	DWORD dwErr;

	bErr = WriteFile( m_filePtr, buff, size, pcbWritten, NULL );
	if ( !bErr )
	{
		dwErr = GetLastError();													 
		
		return HRESULT_FROM_WIN32(dwErr);
	}
	return S_OK;
};

HRESULT FILE_OBJECT::Read_( char* buff, ULONG size)
{
	ULONG actuallyRead, offset;
	offset = 0;
	while( size > 0 )
	{
		HRESULT	stat = Read(buff + offset,size, &actuallyRead);
		if(FAILED(stat)) return stat;
		// we are not tolerating zero bytes input; we assume
		// the Read() above will block untill there is data or an error
		if(actuallyRead == 0 ) 
			return E_FAIL;
		size -= actuallyRead;
		offset += actuallyRead;
	}
	return S_OK;
}

HRESULT FILE_OBJECT::Read( 
	char* buff, 
	ULONG sizeToRead, 
	ULONG *pActualRead )
{
	BOOL bErr;
	DWORD dwErr;

	bErr = ReadFile( m_filePtr, buff, sizeToRead, pActualRead, NULL );
	if ( !bErr )
	{
		dwErr = GetLastError();
		return HRESULT_FROM_WIN32(dwErr);
	}

	if ( *pActualRead == 0 )
		m_isEndOfFile = TRUE;
	else
		m_isEndOfFile = FALSE;
	return S_OK;
}

BOOL FILE_OBJECT::AtEnd() 
{
	return m_isEndOfFile;
}

FILE_OBJECT::FILE_OBJECT()
{
	m_isEndOfFile = FALSE;
	m_bShouldClose = false;
	strncpy(m_FileName, "No file",_MAX_FILE_NAME);
}
HRESULT FILE_OBJECT::CreateFile(
	  LPCTSTR lpFileName,          // pointer to name of the file
	  DWORD dwDesiredAccess,       // access (read-write) mode
	  DWORD dwShareMode,           // share mode
	  DWORD dwCreationDisposition,  // how to create
	  DWORD dwFlagsAndAttributes  // file attributes
	)
{
	DWORD dwErr;
	HANDLE hFile;
	HRESULT hRes  = S_OK;
	hFile = ::CreateFile(
		lpFileName,
		dwDesiredAccess,
		dwShareMode,
		NULL,
		dwCreationDisposition,
		dwFlagsAndAttributes,
		NULL);

	if ( hFile == INVALID_HANDLE_VALUE )
	{ // Could not open new file
		dwErr = GetLastError();
		switch (dwErr)
		{
		case ERROR_NOT_ENOUGH_MEMORY:
		case ERROR_OUTOFMEMORY:
		case ERROR_HANDLE_DISK_FULL:
		
			hRes = E_FAIL;
			break;
		case ERROR_NOT_READY:
			hRes = E_FAIL;
			break;
		case ERROR_ACCESS_DENIED:
			hRes = E_FAIL;
			break;
		case ERROR_FILE_EXISTS:
			hRes = E_FAIL;
			break;
		case ERROR_FILE_NOT_FOUND :
		case ERROR_PATH_NOT_FOUND:
		default :
			hRes = E_FAIL;
			break;
		}
	}
	else
	{
		m_filePtr = hFile; 
		m_bShouldClose = true;
		strncpy(m_FileName, lpFileName,_MAX_FILE_NAME);
		TRACE("Opened %s\n", m_FileName);
	}
	return hRes;
}


