/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.
*/

#include <StdAfx.h>
#include "SinkTee.h"

SinkTee2::~SinkTee2()
{
	if(NULL != m_pSink1)
	{		m_pSink1->Release();
		m_pSink1 = NULL;
	}
	if(NULL != m_pSink2)
	{		m_pSink2->Release();
		m_pSink2 = NULL;
	}
}
BOOL SinkTee2::Init(
	IDataSink2 *pSink1, 
	IDataSink2 *pSink2 )
{
	if ( pSink1 == NULL || pSink2 == NULL )
		return FALSE;

	m_pSink1 = pSink1;
	m_pSink2 = pSink2;
	m_pSink1->AddRef();
	m_pSink2->AddRef();
	return TRUE;
}

HRESULT SinkTee2::Write( 
	char* buff, 
	ULONG size, 
	ULONG *pcbWritten )
{
	HRESULT hRes;
	ULONG actually_written;

	if ( m_pSink1 == NULL || m_pSink2 == NULL )
		return E_FAIL;

	hRes = m_pSink1->Write( buff, size, &actually_written );
	if ( FAILED(hRes) )
		return hRes;

	// Should write the same amount to Sink2
	hRes = m_pSink2->Write_( buff, actually_written );
	if ( FAILED(hRes) )
		return hRes;

	if ( pcbWritten != NULL )
		*pcbWritten = actually_written;

	return S_OK;
}

HRESULT SinkTee2::Write_( 
	char* buff, 
	ULONG size )
{
	HRESULT hRes;
		
	if ( m_pSink1 == NULL || m_pSink2 == NULL )
		return E_FAIL;

	hRes = m_pSink1->Write_( buff, size );
	if ( FAILED(hRes) )
		return hRes;

	hRes = m_pSink2->Write_( buff, size );
	if ( FAILED(hRes) )
		return hRes;
	return S_OK;
}
