/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.
*/

#include <objbase.h>
#include "SourceTestPoint.h" // ... and SinkTestPoint
#include "ITestPoint.h"

SinkTestPoint::SinkTestPoint()
{
	m_pHandlerUnk = NULL;
	m_pTee = NULL;
}

SinkTestPoint::~SinkTestPoint()
{
	Disconnect();
}


HRESULT SinkTestPoint::Connect(IN const GUID & TestPointId, // required TP ID
		IN	IDataSink2* pOutput,	// pointer to the downstream stage
		OUT IDataSink2** ppSink		// pointer to the new sink
		)
{

	m_pHandlerUnk = NULL;
	IDataSink2 * pTpSink = NULL;
	ITestPointManager * pTpm;
	HRESULT hr;

	*ppSink = pOutput; // normal connection. This will be overidden
						// if the TP is valid and active

	// load the manager
	hr = CoCreateInstance(CLSID_TestPointManager,NULL,CLSCTX_INPROC_SERVER,
			IID_TestPointManager,(void**)&pTpm); 
	if(FAILED(hr))
		return hr; // "failed getting TP manager"
	
	hr = pTpm->IsActive(TestPointId);
	if( S_OK == hr ) // must check for S_OK explicitly!
	{
		hr = pTpm->QueryInterfaceForTp(TestPointId, &m_pHandlerUnk);
		// when the TP manger is no longer needed, release it.
		// you might want to keep a cached value and release later.
		pTpm->Release();
		pTpm = NULL;

		if(SUCCEEDED(hr))
		{
			// Get a pointer to what we really need
			hr = m_pHandlerUnk->QueryInterface(IID_IDataSink2,(void**)&pTpSink);
			if(SUCCEEDED(hr))
			{
				// connect a T sink
				//you must not put on the stack :) 
				m_pTee = new SinkTee2(); 
				m_pTee->Init(pOutput,pTpSink ); // The Tee AddRef's the two pointer, so we don't have to Add-Release them here.
				// and use the T.
				*ppSink = m_pTee;

				// since we are done with pTpSink, Release() it.
				pTpSink->Release();
				return S_OK; 
			}
			else
			{
				// Failed getting the required interface -
				// you can ignore it silently since the normal connection
				// has be set before as a default,
				// or you can try other interfaces.

				return hr;
			}
		}
		else
		{ // no handler for this TP ( or anyway, it could not be launched
			return hr;
		}

	} // init the test point
	else
	{
		// make the connection anyhow - even if error.
		return hr;
	}
//	ASSERT(false); // never reached
}


HRESULT SinkTestPoint::Disconnect()
{

	// note: the order of destruction should not matter,
	// but since not all our interfaces support IUnkown, 
	// I have to realse it in this order.
	if(NULL != m_pTee)
	{
		delete m_pTee;
		m_pTee = NULL;
	}
	
	if(NULL != m_pHandlerUnk)
	{
		m_pHandlerUnk->Release();
		m_pHandlerUnk = NULL;
	}
	return S_OK;
}