//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       HistGraphView.cpp	[ HistGraphView.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CHistGraphView class
//
//***************************************************************************

#include "stdafx.h"

#include "SourceMetrics.h"
#include "SourceMetricsDoc.h"
#include "HistGraphView.h"
#include "GraphOptionsDlg.h"

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHistGraphView

IMPLEMENT_DYNCREATE(CHistGraphView, CGraphView)

BEGIN_MESSAGE_MAP(CHistGraphView, CGraphView)
	//{{AFX_MSG_MAP(CHistGraphView)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_GRAPH_OPTIONS, OnGraphOptions)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_OPTIONS, OnUpdateGraphOptions)
	ON_COMMAND(ID_GRAPH_VIEW_CLOSE, OnRequestClose)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_VIEW_CLOSE, OnUpdateRequestClose)
	ON_COMMAND(ID_FREQ_GRAPH_HELP, OnFreqGraphHelp)
	ON_UPDATE_COMMAND_UI(ID_FREQ_GRAPH_HELP, OnUpdateFreqGraphHelp)
	ON_COMMAND(ID_GRAPH_NEXT_METRIC, OnGraphNextMetric)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_NEXT_METRIC, OnUpdateGraphNextMetric)
	ON_COMMAND(ID_GRAPH_PREV_METRIC, OnGraphPrevMetric)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_PREV_METRIC, OnUpdateGraphPrevMetric)
	//}}AFX_MSG_MAP
// use to try out various graph options with OC's graph editor dialog
//   ON_COMMAND(ID_SM_GRAPH_EDIT, CGraphView::OnGraphEdit)
END_MESSAGE_MAP()

CHistGraphView::CHistGraphView()
   : CGraphView(), m_poProject(NULL), m_iPlotMetricIndex(0), m_refPlotColor(CXCLR_BLUE)
{
	//{{AFX_DATA_INIT(CHistGraphView)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CHistGraphView::~CHistGraphView()
{
   m_poSMDoc->UnsetView(m_lViewID);	
}

/////////////////////////////////////////////////////////////////////////////
// CHistGraphView diagnostics

#ifdef _DEBUG
void CHistGraphView::AssertValid() const
{
	CGraphView::AssertValid();
}

void CHistGraphView::Dump(CDumpContext& dc) const
{
	CGraphView::Dump(dc);
}
#endif //_DEBUG

void CHistGraphView::OnInitialUpdate() 
{
   SRGraphStyle *       poGraphStyle;              // scratch containers for graph initialization
   SRGraphDisplay *     poGraphDisplay;
   SRGraphTitle *       poGraphTitle;
	SRGraphBackground *  poGraphBackground;
   COLORREF             oBackgroundColor = GetSysColor(COLOR_WINDOW);

   // one-time initialization
   m_poSMDoc = dynamic_cast<CSourceMetricsDoc *>(m_pDocument);
   m_poProject = m_poSMDoc->GetProject();
   m_lViewID = (long)m_hWnd;                 // create a unique token for this view
   m_poSMDoc->SetView(m_lViewID);            // put ourselves in the view map
   m_sSection = CSourceMetricsApp::OPTION_SETTINGS + m_poProject->GetLanguage()->Name();  // set option profile key

   // establish the metric to appear in the graph and graph color, if saved by user
   m_sPlotMetric =
         m_poSMDoc->GetApp()->GetProfileString(m_sSection, CSourceMetricsApp::HIST_GRAPH_METRIC,
                                                m_poProject->GetLanguage()->GetMetricName(0));
   m_refPlotColor = 
         m_poSMDoc->GetApp()->GetProfileInt(m_sSection, CSourceMetricsApp::HIST_GRAPH_COLOR, CXCLR_RED);
   SetMetricIndex();

   // paint a plain background to clear the client area
	poGraphBackground = new SRGraphBackground;
   poGraphStyle = poGraphBackground->GetStyle();
	poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
	poGraphStyle->SetColor(oBackgroundColor);
	m_oGraph.AddComponent(poGraphBackground);

   // initialize the graph display
   poGraphDisplay = new SRGraphDisplay;
   poGraphDisplay->SetRect(0, 7, 100, 98);
   poGraphDisplay->SetMeasurement(SRGraphComponent::PERCENT);
   poGraphStyle = poGraphDisplay->GetStyle();
   poGraphStyle->SetGraphStyle(CX_GRAPH_LINE);
   poGraphStyle->SetAxisStyle(CX_AXIS_CLASSIC);
   poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
   poGraphStyle->SetColor(oBackgroundColor);
   m_oGraph.AddComponent(poGraphDisplay);

   // initialize title
   poGraphTitle = new SRGraphTitle;
   poGraphTitle->SetRect(0, 2, 100, 7);
   poGraphTitle->SetMeasurement(SRGraphComponent::PERCENT);
   poGraphTitle->SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
   poGraphTitle->SetFontSize(12);                        // 12 point
   poGraphTitle->SetFontStyle(1);                        // bold
//   poGraphTitle->SetFontStyle(CX_FONT_AUTOSIZE bitor 1); // bold
   poGraphTitle->SetJustification(2);                    // center the title
   poGraphStyle = poGraphTitle->GetStyle();
   poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
   poGraphStyle->SetColor(oBackgroundColor);
   m_oGraph.AddComponent(poGraphTitle);

   // get the window caption set
   m_sCaption.LoadString(IDS_PROJ_GRAPH_CAPTION);
   m_sCaption += _T(" '") + m_poProject->Name() + _T("'");
   GetParent()->SetWindowText(m_sCaption);
   CGraphView::OnInitialUpdate();
   return;	
}

void CHistGraphView::OnUpdate(CView* /* pSender */, LPARAM /* lHint */, CObject* /* pHint */)
{
   CWaitCursor       oWaitCursor;         // let user know this may take awhile

   UpdateGraph();
   return;
}

// ************************************************************************************************
// pass data into the graph for display
void
CHistGraphView::UpdateGraph()
{
   SRGraphStyle *    poGraphStyle;           // scratch container for bar color change
   CString           sTitle;
   SMCheckpoint      oCkpt;                  // scratch for access to metrics values
   int               i;                      // loop index
   int               iValueCount = m_poProject->CheckpointCount();
   double            dValue;                 // plotted value

   // set the graph's title
   sTitle.LoadString(IDS_HIST_GRAPH_METRIC);
   sTitle += _T(" '") + m_sPlotMetric + _T("'");
   m_oGraph.SetGraphTitle(sTitle, 1);

   // load up the graph with labels and data items
   for( i = 0; i < iValueCount; ++i )
   {
      // visit the checkpoints in chronological order
      if( m_poProject->GetCheckpointByIndex(iValueCount - i - 1, oCkpt) )
      {
         dValue = _tcstod(oCkpt.m_poLanguage->GetValue(m_iPlotMetricIndex, false), NULL);
         m_oGraph.SetValue(i, 0, dValue);
         m_oGraph.GetGroup(0)->GetIndex(i)->SetAnnotation((oCkpt.Date()).Format("%d %b %Y"));
      }
   }
   // set the bar color
   poGraphStyle = m_oGraph.GetGroup(0)->GetIndex(0)->GetStyle();
   poGraphStyle->SetInteriorStyle(CX_INTERIOR_COLOR);
   poGraphStyle->SetColor(m_refPlotColor);
   m_oGraph.SetGlobalStyle(0, 0, -1, -1, poGraphStyle);
   m_oGraph.SetSignificantChanges();

   return;
}

// ***********************************************************************************************
// initialize the graph object
void
CHistGraphView::SetMetricIndex()
{
   m_iMetricsCount = m_poProject->GetLanguage()->Columns();
   for( int i = 0; i < m_iMetricsCount; ++i )
   {
      if( m_poProject->GetLanguage()->GetMetricName(i) == m_sPlotMetric )
      {
         m_iPlotMetricIndex = i;
         break;
      }
   }
}

// ***********************************************************************************************
// redisplay graph for different metric
void
CHistGraphView::RedisplayGraph()
{
   m_poSMDoc->GetApp()->WriteProfileString(m_sSection, CSourceMetricsApp::HIST_GRAPH_METRIC, m_sPlotMetric);
   UpdateGraph();
   Invalidate();
}

// Display the value represented by the bar under the cursor
//
void CHistGraphView::ShowFeedback(SRGraphFeedback *pFeedback)
{
   CString  sTipLabel;
	CString  sTip;
	CPoint   oPoint = m_MousePoint;

   sTipLabel.LoadString(IDS_HIST_GRAPH_TIP_LABEL);
   sTip.Format("%s %g ", sTipLabel, m_oGraph.GetGroup(pFeedback->m_nGroup)->GetIndex(pFeedback->m_nIndex)->GetValue());
	ClientToScreen(&oPoint);
	m_GraphTip.ShowTip(sTip, oPoint, this);
}

/////////////////////////////////////////////////////////////////////////////
// CHistGraphView message handlers
//

void CHistGraphView::OnContextMenu(CWnd * /* pWnd */, CPoint point) 
{
	CMenu    oContextMenu;

   oContextMenu.LoadMenu(IDR_GRAPH_VIEW_CONTEXT);
   oContextMenu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN bitor TPM_RIGHTBUTTON,
                                                point.x, point.y, GetParentFrame());

   return;
}

void CHistGraphView::OnGraphOptions() 
{
   CGraphOptionsDlg  oDlg(this);

   oDlg.m_saMetrics.SetSize(m_iMetricsCount);
   for( int i = 0; i < m_iMetricsCount; ++i )
   {
      oDlg.m_saMetrics[i] = m_poProject->GetLanguage()->GetMetricName(i);
   }
   oDlg.m_iPlotMetric = m_iPlotMetricIndex;
   oDlg.m_sPlotMetric = m_sPlotMetric;
   oDlg.m_refPanelColor = m_refPlotColor;

   if( oDlg.DoModal() == IDOK )
   {
      m_sPlotMetric = oDlg.m_sPlotMetric;
      m_refPlotColor = oDlg.m_refPanelColor;
      SetMetricIndex();
      m_poSMDoc->GetApp()->WriteProfileInt(m_sSection, CSourceMetricsApp::HIST_GRAPH_COLOR, m_refPlotColor);
      RedisplayGraph();
   }
}

void CHistGraphView::OnUpdateGraphOptions(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CHistGraphView::OnGraphNextMetric() 
{
   m_iPlotMetricIndex++;
   if( m_iPlotMetricIndex >= m_iMetricsCount )
   {
      m_iPlotMetricIndex = 0;                   // wrap around to the beginning
   }
   m_sPlotMetric = m_poProject->GetLanguage()->GetMetricName(m_iPlotMetricIndex);
   RedisplayGraph();
}

void CHistGraphView::OnUpdateGraphNextMetric(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CHistGraphView::OnGraphPrevMetric() 
{
   m_iPlotMetricIndex--;
   if( m_iPlotMetricIndex < 0 )
   {
      m_iPlotMetricIndex = m_iMetricsCount - 1; // wrap around to the end
   }
   m_sPlotMetric = m_poProject->GetLanguage()->GetMetricName(m_iPlotMetricIndex);
   RedisplayGraph();
}

void CHistGraphView::OnUpdateGraphPrevMetric(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CHistGraphView::OnRequestClose() 
{
	GetParentFrame()->PostMessage(WM_CLOSE);	
}

void CHistGraphView::OnUpdateRequestClose(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CHistGraphView::OnFreqGraphHelp() 
{
	AfxGetApp()->WinHelp(ID_FREQ_GRAPH_HELP, HELP_CONTEXT);
}

void CHistGraphView::OnUpdateFreqGraphHelp(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}
