//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       MetricDetailsDlg.cpp	[ MetricDetailsDlg.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CMetricDetailsDlg class
//
//***************************************************************************

#include "stdafx.h"
#include "SourceMetrics.h"
#include "MetricDetailsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMetricDetailsDlg dialog

BEGIN_MESSAGE_MAP(CMetricDetailsDlg, CDialog)
	//{{AFX_MSG_MAP(CMetricDetailsDlg)
	ON_BN_CLICKED(ID_DETAILS_HELP, OnDetailsHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CMetricDetailsDlg::CMetricDetailsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMetricDetailsDlg::IDD, pParent), m_poParent(pParent)
{
	//{{AFX_DATA_INIT(CMetricDetailsDlg)
	//}}AFX_DATA_INIT
}

void CMetricDetailsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMetricDetailsDlg)
	DDX_Control(pDX, IDC_BLOCK_GRAPH, m_btnGraph);
	//}}AFX_DATA_MAP

   if( not pDX->m_bSaveAndValidate )
   {
      CString  sCaption;
      
      GetWindowText(sCaption);
      sCaption += m_sCaptionSuffix;
      SetWindowText(sCaption);
      m_btnGraph.m_poGraph = &m_oGraph;
      CenterWindow(m_poParent);           // put us in the middle of our parent's window
   }
}

/////////////////////////////////////////////////////////////////////////////
// CMetricDetailsDlg message handlers

BOOL CMetricDetailsDlg::OnInitDialog() 
{
   CStatic *         poGraphControl = (CStatic *)GetDlgItem(IDC_BLOCK_GRAPH);
   CListCtrl *       poListCtrl = (CListCtrl *)GetDlgItem(IDC_LIST_DETAILS);
   CHeaderCtrl *     poViewHeader = (CHeaderCtrl *)poListCtrl->GetDlgItem(0);
   CString           sValue;
   int               iListCount = m_saEntities.GetSize();   // size of both lists
   int               iBlockItems = m_iaBlockDepths.GetSize();
   SRGraphStyle *    poGraphStyle;                          // scratch containers for graph initialization
   SRGraphDisplay *  poGraphDisplay;
   int            i;                                        // loop index (reused)

	CDialog::OnInitDialog();

   // set the list header captions
   sValue.LoadString(IDS_DETAILS_ENTITY_CAPTION);
   poListCtrl->InsertColumn(0, sValue, LVCFMT_LEFT, 0, 0);
   sValue.LoadString(IDS_DETAILS_VALUE_CAPTION);
   poListCtrl->InsertColumn(1, sValue, LVCFMT_LEFT, 0, 1);

   // now size the columns to fit the labels
   poListCtrl->SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
   poListCtrl->SetColumnWidth(1, LVSCW_AUTOSIZE_USEHEADER);

   // set the entities labels into the list view items
   for( i = 0; i < iListCount; ++i )
   {
      poListCtrl->InsertItem(i, m_saEntities[i], 0);
      sValue = m_saValues[i];
      sValue.TrimLeft();
      poListCtrl->SetItemText(i, 1, sValue);
   }
   // now size the two columns to fit the data or heading, whichever is wider
   for( i = 0; i < 2; ++i )
   {
      int iColumnWidth = poListCtrl->GetColumnWidth(i);

      poListCtrl->SetColumnWidth(i, LVSCW_AUTOSIZE);
      if( iColumnWidth > poListCtrl->GetColumnWidth(i) )
      {
         // put it back
         poListCtrl->SetColumnWidth(i, LVSCW_AUTOSIZE_USEHEADER);
      }
   }
   // now create and display the block depth histogram, if any
   if( iBlockItems > 0 )
   {
      // initialize the graph display window
      poGraphDisplay = new SRGraphDisplay;
      poGraphDisplay->SetRect(-1, -1, -1, -1);
      poGraphDisplay->SetMeasurement(SRGraphComponent::PERCENT);
      poGraphStyle = poGraphDisplay->GetStyle();
      poGraphStyle->SetGraphStyle(CX_GRAPH_EX_VBAR);
      poGraphStyle->SetAxisStyle(CX_AXIS_ISO_CLASSIC);
      poGraphStyle->SetComponentFillStyle(CX_SOLID_FILL);
      poGraphStyle->SetColor(GetSysColor(COLOR_WINDOW));
      m_oGraph.AddComponent(poGraphDisplay);

      // load up the graph with labels and data items
      for( i = 0; i < iBlockItems; ++i )
      {
         int      iValue = m_iaBlockDepths[i];

         m_oGraph.SetValue(i, 0, double(iValue));
         if( i == (iBlockItems - 1) )
         {
            sValue.Format(" %d+", i);
         }
         else
         {
            sValue.Format("%d", i);
         }
         m_oGraph.GetGroup(0)->GetIndex(i)->SetAnnotation(sValue);
      }
   }
   else        // resize the dialog after hiding the graph control
   {
      CButton *   poCloseButton = (CButton *)GetDlgItem(IDOK);
      CRect       oDlgRect;
      CRect       oCloseRect;
      CRect       oGraphRect;
      int         iCloseOffset;

      GetWindowRect(&oDlgRect);
      poGraphControl->GetWindowRect(&oGraphRect);
      poCloseButton->GetWindowRect(&oCloseRect);
      iCloseOffset = oGraphRect.Width() - oCloseRect.Width();
      oCloseRect.left -= iCloseOffset;
      oCloseRect.right -= iCloseOffset;
      ScreenToClient(&oCloseRect);
      poCloseButton->MoveWindow(&oCloseRect);
      poGraphControl->ShowWindow(SW_HIDE);
      ((CStatic *)GetDlgItem(IDC_BLOCK_GRAPH_LABEL))->ShowWindow(SW_HIDE);
      ((CStatic *)GetDlgItem(IDC_BLOCK_GRAPH_LABEL2))->ShowWindow(SW_HIDE);
      oDlgRect.right -= iCloseOffset;
      MoveWindow(&oDlgRect);
   }

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMetricDetailsDlg::OnDetailsHelp() 
{
   CSourceMetricsApp * poApp = dynamic_cast<CSourceMetricsApp *>(AfxGetApp());

   poApp->WinHelp(CMetricDetailsDlg::IDD);  	
}
