//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       MetricsProgressDlg.cpp	[ MetricsProgressDlg.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CMetricsProgressDlg class (shows user the
//             progress as metrics for files in a checkpoint are computed)
//
//***************************************************************************

#include "stdafx.h"
#include <afxmt.h>

#include "SourceMetrics.h"
#include "MetricsProgressDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMetricsProgressDlg dialog

BEGIN_MESSAGE_MAP(CMetricsProgressDlg, CDialog)
	//{{AFX_MSG_MAP(CMetricsProgressDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// *******************************************************************************
// modified constructor used for modeless operation
CMetricsProgressDlg::CMetricsProgressDlg(int iFiles, CWnd * pParent /* = NULL */)
	: CDialog(), m_iFileCount(iFiles), m_fIsCancelled(false), m_iFilesProcessed(0), m_poParent(pParent)
{
}

void CMetricsProgressDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMetricsProgressDlg)
	DDX_Control(pDX, IDCANCEL, m_btnCancel);
	DDX_Control(pDX, IDC_PROGRESS, m_barProgress);
	DDX_Control(pDX, IDC_PROGRESS_PERCENT, m_staticPercent);
	DDX_Control(pDX, IDC_PROGRESS_FILE_NAME, m_staticFileName);
	//}}AFX_DATA_MAP
   if( not pDX->m_bSaveAndValidate )
   {
      m_barProgress.SetRange(0, (short)m_iFileCount);
      m_barProgress.SetStep(1);
      m_oEvent.SetEvent();          // we're ready to display status: let the user go
   }
}

// ******************************************************************************
// create the modeless dialog
void
CMetricsProgressDlg::DoModeless()
{
   Create(CMetricsProgressDlg::IDD, m_poParent);
}

// ******************************************************************************
// post the name of the file whose metrics are now being evaluated
// update the progress bar and percent label
// (if caller is in another thread, must wait for m_oEvent to be signalled)
void
CMetricsProgressDlg::SetFileName(CString & sFileName)
{
   CWaitCursor oWaitCursor;
   int         iPercentDone;
   CString     sPercent;
   
   SetFocus();
   m_staticFileName.SetWindowText(sFileName);
   iPercentDone = (m_iFilesProcessed * 100) / m_iFileCount;
   if( iPercentDone > 0 )
   {
      m_barProgress.StepIt();
   }
   sPercent.Format("%d %%", iPercentDone);
   m_staticPercent.SetWindowText(sPercent);
   ++m_iFilesProcessed;
}

// *******************************************************************************
// report the state of the abort flag, pump messages through for the dialog
bool
CMetricsProgressDlg::IsCancelled()
{
   MSG   oMessage;

   // let the cancel button message do it's thing
   while( ::PeekMessage(&oMessage, NULL, NULL, NULL, PM_NOREMOVE) )
   {
      AfxGetThread()->PumpMessage();
   }
   return m_fIsCancelled;
}

/////////////////////////////////////////////////////////////////////////////
// CMetricsProgressDlg message handlers

void CMetricsProgressDlg::OnCancel() 
{
   // don't call base class, just record user's desire in a private member
   m_fIsCancelled = true;

   // but disable this button
   m_btnCancel.EnableWindow(FALSE);
}

