//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc 

//@module DScale.cpp | Class implementation for SRGDecimalScale. This class manages the display of decimal scales axis labels

#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"
#include <math.h>
#include <afxdisp.h> // for COleDateTime...

#ifdef _DEBUG

#define new DEBUG_NEW

#endif

//@cmember NEW This function sets all the label vectors angles for the whole block 
//so that the labels all rest at right angles to the line drawn between the values held in the 
//m_ptStart and m_ptEnd members 
void SRGDecimalScale::SetRTAngles()
{
	double dAngle=GetLineAngle(m_ptStart,m_ptEnd);
	if(m_bAscending)
	{
		dAngle-=PI;
		if(dAngle<0)
			dAngle+=TWO_PI;
	}

	// Set the label angles

	SetBlockOrientations(DEGREES(dAngle));

	// Decide upon their location points and vectors

	if(m_bLabelSide)
	{	// Labels are to the right of the line
		SetBlockLocationPoints(MidLeft);
		SetBlockVectors(cos(dAngle),sin(dAngle));
	}
	else
	{	// to the left...
		SetBlockLocationPoints(MidRight);
		SetBlockVectors(-cos(dAngle),sin(dAngle));
	}

}

//@mfunc Creates the list of numbers along a decimal axis.
//@rem This routine dynamically re-evaluates the number of labels displayed by factoring the available line length
// between start and end points and the size of the font used to display the line.
//@devnote Labels are always created perpendicular to the line vector. If this does not suit your need you may use the standard SRGraphLabel or SRGraphLabelBlock 
// commands to re-position or re-orient the labels. <nl> Overlap calculations are 
// first performed on straight (0 degree) text and then the whole string of labels is rotated to the correct orientation just before draw.
// A choice can be made for making the labels to the left of the line or to the right of the line. 
void SRGDecimalScale::CreateLabels()
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return; // Must have a valid m_pDC
	}

	//The Mul list allows successive multiplication by 2, 5 and 10 to determine the best display
	// this gives decimal displays which, if possible, will show all the information asked for.
	// If all information wont fit then 1/5 of the information might. Failing that 1/2, failing that 1/10th.
	// This cyclic test may be re computed many times to arrive at the final scale which may be a small
	// fraction of the originally desired resolutuion
	
	double MulList[4]={
		1.0,
		2.0,
		5.0,
		10.0
	};

	int nMulListPos=0;

	// We start with the real resolution as desired by the user but
	// all calculations are done using copies of that value.

	double dTempRes,dRealRes;

	if(m_dResolution==0)
		m_dResolution=1;

	dRealRes=m_dResolution;

	// Semaphore to signify attainment of our numerical nirvana
	BOOL bFoundFinalRes=FALSE;

	
	//Select the correct font into the DC
	CFont *OldFont=(CFont *)m_pDC->SelectObject(GetFont());

	//Check for and correct a null format string.
	if(m_strFormatString.IsEmpty())
		m_strFormatString=" %g ";

	// These are the actual strings which we use to test the sizes with
	CString strTestA,strTestB;

	//Fill the labels according to the format string
	strTestA.Format(m_strFormatString,m_dMinValue);
	strTestB.Format(m_strFormatString,m_dMaxValue);

	// and put the longest string into A
	if(strTestA.GetLength()<strTestB.GetLength())
		strTestA=strTestB;


	// Find the physical size of the largest label
	CSize LabelSize=m_pDC->GetTextExtent(strTestA);
	int nLineLength=(int)GetLineLength(m_ptStart,m_ptEnd);
	if(nLineLength<1)
	{
		m_pDC->SelectObject(OldFont);
		return;
	}
	do
	{
		dTempRes=dRealRes;	// copy the real resolution
		dTempRes*=MulList[nMulListPos]; //multiply it by the current multiply value
		
		if(nLineLength/LabelSize.cy >= ((m_dMaxValue-m_dMinValue)/dTempRes))
			bFoundFinalRes=TRUE;
		
		if(!bFoundFinalRes)
		{
			++nMulListPos&=0x03;
			if(!nMulListPos)
				dRealRes=dTempRes;
		}
	}
	while(!bFoundFinalRes);

	dRealRes=dTempRes;

	// resolutions are found. Now make a list of labels at the correct numerical spacing

	double dRange=m_dMaxValue-m_dMinValue;

	double xWidth=(double)m_ptEnd.x-m_ptStart.x,
		   yHeight=(double)m_ptEnd.y-m_ptStart.y;

	double xScale=xWidth/dRange,
		   yScale=yHeight/dRange;

	int nLabelIndex=0;
	
	for(double dCount= (m_bAscending ? m_dMinValue : m_dMaxValue);
	           (m_bAscending ? dCount<=m_dMaxValue : dCount>=m_dMinValue);
			   dCount+= (m_bAscending ? dRealRes : -dRealRes))
		{
			CString strCountString;
			SRGraphLabel * pL;
			if(m_LabelList.IsEmpty() || m_LabelList.GetCount()<=nLabelIndex )
			{
				pL=new SRGraphLabel;
 				AddLabel(pL);
			}
			else
				pL=(SRGraphLabel*)m_LabelList.GetAt(m_LabelList.FindIndex(nLabelIndex));

			ASSERT(pL!=NULL); // if it is I dont know why!
			
			nLabelIndex++;

			strCountString.Format(m_strFormatString,dCount);
			pL->SetAnnotation(strCountString);
			pL->SetTextPosition(m_ptStart.x+(int)(xScale*(dCount-m_dMinValue)),
				                m_ptStart.y+(int)(yScale*(dCount-m_dMinValue)));
		}

	
			   
	while(nLabelIndex<m_LabelList.GetCount())
		delete ((SRGraphLabel *)m_LabelList.RemoveTail());

    // Now we need to set all the labels at right angles to the line
	
	SetRTAngles();
	
	UpdateTickList();

	m_pDC->SelectObject(OldFont);
}

//@mfunc Constructor with default initialisation
SRGDecimalScale::SRGDecimalScale()
{
	m_bAscending=FALSE;
	m_ptStart=CPoint(0,0);
	m_ptEnd=CPoint(0,0);
	m_dMinValue=0;
	m_dMaxValue=0;
	m_dResolution=0.1;
	m_strFormatString.Empty();
	m_bLabelSide=FALSE;
	m_bCheckOverlaps=FALSE;
}

//@mfunc Constructor with explicit initialisation.
//@parm The start position anchor.
//@parm the end position anchor.
//@parm the minimum value to display.
//@parm the maximum value to display.
//@parm The resolution of the steps between minimum and maximum values.
//@parm a text format string similar to the Printf function strings with one floating point type and any other text required.
//(for example " $%.2g" will show $250 $300 $350 etc.)
SRGDecimalScale::SRGDecimalScale(CPoint ptStart, CPoint ptEnd, double dMinValue, double dMaxValue, double dResolution,CString & strFormatString)
{
	m_bAscending=FALSE;
	m_bLabelSide=FALSE;
	m_strFormatString=strFormatString;
	m_dMaxValue=dMaxValue;
	m_dMinValue=dMinValue;
	m_ptStart=ptStart;
	m_ptEnd=ptEnd;
	m_dResolution=dResolution;
	m_bCheckOverlaps=FALSE;
}
		
//@mfunc Destroys the label list and deletes all labels from it.
SRGDecimalScale::~SRGDecimalScale()
{
	while(!m_LabelList.IsEmpty())
		delete ((SRGraphLabel *)m_LabelList.RemoveHead());
}

//@mfunc Constructor with explicit initialisation.
//@parm The start position anchor.
//@parm the end position anchor.
//@parm the minimum value to display.
//@parm the maximum value to display.
//@parm The resolution of the steps between minimum and maximum values.
//@parm a text format string similar to the Printf function strings with one floating point type and any other text required.
//(for example " $%.2g" will show $250 $300 $350 etc.)
SRGLogScale::SRGLogScale(CPoint ptStart, CPoint ptEnd, double dMinValue, double dMaxValue, double dResolution,CString & strFormatString)
{
	m_bAscending=FALSE;
	m_bLabelSide=FALSE;
	m_strFormatString=strFormatString;
	m_dMaxValue=dMaxValue;
	m_dMinValue=dMinValue;
	m_ptStart=ptStart;
	m_ptEnd=ptEnd;
	m_dResolution=dResolution;
	m_bCheckOverlaps=TRUE;
}


//@mfunc Creates the list of numbers along a logarithmic axis.
//@rem This routine dynamically re-evaluates the number of labels displayed by factoring the available line length
// between start and end points and the size of the font used to display the line.
void SRGLogScale::CreateLabels()
{

	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return; // Must have a valid m_pDC
	}

	if(m_dMinValue<0 || m_dMaxValue<1)
		SRThrowGraphException(SRGraphException::logscale);

	double dLogMaxValue=log10(m_dMaxValue),
		   dLogMinValue=log10(m_dMinValue);

	double dLogArray[10]={
			0.000000000,
			0.301029996,
			0.477121255,
			0.602059991,
			0.698970004,
			0.778151250,
			0.845098040,
			0.903089987,
			0.954242509,
			1.000000000};




	//Select the correct font into the DC
	CFont *OldFont=(CFont *)m_pDC->SelectObject(GetFont());

	//Check for and correct a null format string.
	if(m_strFormatString.IsEmpty())
		m_strFormatString=" 10x%d ";

	CString strTestA;

	strTestA.Format(m_strFormatString,pow((double)10.0,dLogMaxValue));

	// Find the physical size of a label. We are only interested in the font height really
	CSize LabelSize=m_pDC->GetTextExtent(strTestA);
	int nLineLength=(int)GetLineLength(m_ptStart,m_ptEnd);


	// resolutions are found. Now make a list of labels at the correct numerical spacing

	double dRange=dLogMaxValue-dLogMinValue;;

	double xWidth=(double)m_ptEnd.x-m_ptStart.x,
		   yHeight=(double)m_ptEnd.y-m_ptStart.y;

	double xScale=xWidth/dRange,
		   yScale=yHeight/dRange;

	int nLabelIndex=0,nx,ny;
	
	double dRef=0;

	for(double dCount=dLogMinValue;dCount<=dLogMaxValue;dCount++)
	{
		CString strCountString;
		strCountString.Format(" 10x%d ",(int)dCount);
		AddLabelIndex(strCountString,nLabelIndex);
		SRGraphLabel* pL=GetLabelIndex(nLabelIndex);
		pL->SetTextPosition(m_ptStart.x+(int)(xScale*dRef),
				            m_ptStart.y+(int)(yScale*dRef));
		nLabelIndex++;
		for(int i=1;i<10;i++)
		{
			AddLabelIndex("",nLabelIndex);
			pL=GetLabelIndex(nLabelIndex);
			nx=m_ptStart.x+(int)(xScale*(dRef+dLogArray[i]));
			ny=m_ptStart.y+(int)(yScale*(dRef+dLogArray[i]));
			if(dRef+dLogArray[i]>dLogMaxValue)
				break;
			else
				pL->SetTextPosition(nx,ny);
			nLabelIndex++;
		}
		dRef++;
	}	
			   
	while(nLabelIndex<m_LabelList.GetCount())
		delete ((SRGraphLabel *)m_LabelList.RemoveTail());

	SetRTAngles();

	UpdateTickList();
}


//@mfunc Creates a set of labels based on the COleDateTime format for use on chart scales. 
//@rem This function will automatically work out the best date and time combination based upon 
//the "span" of time between m_dMinValue and m_dMaxValue.
void SRGDateScale::CreateLabels()
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return; // Must have a valid m_pDC
	}

	double TimeArray[]={
		//1.157407407407e-10,		// A 1/100000 second
		//1.157407407407e-9,		// A 1/10000 second
		//1.157407407407e-8,		// A 1/1000 second
		//1.157407407407e-7,		// A 1/100 second
		//1.157407407407e-6,		// A 1/10 second
		1.157407407407e-5,		// A second
		2.314814814814e-5,		// Two seconds
		5.787037037035e-5,		// Five seconds
		1.157407407407e-4,		// Ten seconds
		3.472222222221e-4,		// Thirty seconds
		6.944444444444e-4,		// One minute
		1.388888888889e-3,		// Two minutes
		0.003472222222222,		// Five minutes
		6.944444444444e-3,		// Ten minutes
		0.02083333333333,		// Thirty minutes
		0.04166666666667,		// An hour
		0.083333333332,			// Two hours
		0.125,					// Three hours
		0.166666666664,			// Four hours
		0.25,					// Six hours
		0.333333333333,			// Eight hours
		1.0,					// The day
		7.0,					// A week,
		14.0,					// A Fortnight
		30.0,					// Special case.. A month
		365.0,					// A year
		730.0,					// Two years
		1825.0,					// Five Years
		3650.0					// Ten Years
		};


	// Time spans bigger than a day are handled without the array

	//Select the correct font into the DC
	CFont *OldFont=(CFont *)m_pDC->SelectObject(GetFont());

	// Now to find out exactly what system we are using

	TEXTMETRIC tm;
	CSize s=m_pDC->GetTextMetrics(&tm);

	int nLineLength=(int)GetLineLength(m_ptStart,m_ptEnd);
	int nMaxLabelCount=nLineLength/tm.tmHeight;
	double dSpan=m_dMaxValue-m_dMinValue,dNow=m_dMinValue;
	BOOL bUseDate=FALSE;
 	int nLastDate=(int)dNow;

	if(nLastDate)
		bUseDate=TRUE;

	double dTempRes;

	if(m_dResolution==0)
	{ // No specific resolution has been chosen
		dTempRes=max(TimeArray[0],dSpan/nMaxLabelCount);
		int nResIndex=1;
		BOOL bFoundRes=FALSE;

		while(TimeArray[nResIndex]!=3650 && !bFoundRes)
		{
			if(TimeArray[nResIndex]>=dTempRes)
			{
				dTempRes=TimeArray[nResIndex];
				bFoundRes=TRUE;
			}
			else
				nResIndex++;
		}
		if(!bFoundRes)
			dTempRes=3650.0;
	}
	else
		dTempRes=m_dResolution;

	// To make the dates and or times

	double dXStep=0.0,dYStep=0.0;		  

	CString strFormatString; // A duplicate of the chosen format or a calculated format

	if(m_strFormatString.IsEmpty())
	{
		if(dSpan<=6.944444444444e-4)
			strFormatString=_T("%S");
		else if(dSpan<=0.04166666666667)
			strFormatString=_T("%M:%S");
		else if(dSpan<=1)
			strFormatString=_T("%H:%M:%S");
		else
			strFormatString=_T("%x");
	}
	else
		strFormatString=m_strFormatString;


	// Now build the list.
	double dRange=m_dMaxValue-m_dMinValue;

	double xWidth=(double)m_ptEnd.x-m_ptStart.x,
		   yHeight=(double)m_ptEnd.y-m_ptStart.y;

	double xScale=xWidth/dRange,
		   yScale=yHeight/dRange;

	int nLabelIndex=0;

	COleDateTime dt;
	int nEndCount;
	
	if(dTempRes>29.999999)
	{
		int day,month,year;
		dt=(COleDateTime)(m_bAscending ? m_dMinValue : m_dMaxValue);
		day=dt.GetDay();
		month=dt.GetMonth();
		year=dt.GetYear();
		nEndCount=0;
		double dCount=dt.m_dt;
		while(nEndCount<2)
		{
			dt=COleDateTime(year,month,day,0,0,0);
			SRGraphLabel * pL;
			if(m_LabelList.IsEmpty() || m_LabelList.GetCount()<=nLabelIndex )
			{
				pL=new SRGraphLabel;
 				AddLabel(pL);
			}
			else
				pL=(SRGraphLabel*)m_LabelList.GetAt(m_LabelList.FindIndex(nLabelIndex));

			ASSERT(pL!=NULL);
			
			nLabelIndex++;

			pL->SetAnnotation(dt.Format(strFormatString));
			pL->SetTextPosition(m_ptStart.x+(int)(xScale*(dCount-m_dMinValue)),
								m_ptStart.y+(int)(yScale*(dCount-m_dMinValue)));
			

			if(m_bAscending)
			{
				switch((int)dTempRes)
				{
				case 30:
					month++;
					break;
				default:
				case 365:
					year++;
					break;
				case 730:
					year+=2;
					break;
				case 1825:
					year+=5;
					break;
				case 3650:
					year+=10;
					break;
				}

				if(month>12)
				{
					year++;
					month-=12;
				}

				day=1;
				dt=COleDateTime(year,month,day,0,0,0);
			}
			else
			{
				switch((int)dTempRes)
				{
				case 30:
					month--;
					break;
				default:
				case 365:
					year--;
					break;
				case 730:
					year-=2;
					break;
				case 1825:
					year-=5;
					break;
				case 3650:
					year-=10;
					break;
				}

				if(month<1)
				{
					year--;
					month+=12;
				}
				day=1;
				dt=COleDateTime(year,month,day,0,0,0);
			}
			dCount=dt.m_dt;
			if(m_bAscending)
			{
				if(dCount>m_dMaxValue)
				{
					dCount=m_dMaxValue;
					nEndCount++;
				}
			}
			else
			{
				if(dCount<m_dMinValue)
				{
					dCount=m_dMinValue;
					nEndCount++;
				}
			}
		}
	}
	else
	{
		nEndCount=0;
		double dCount= (m_bAscending ? m_dMinValue : m_dMaxValue);
		while(nEndCount<2)
			{
				SRGraphLabel * pL;
				if(m_LabelList.IsEmpty() || m_LabelList.GetCount()<=nLabelIndex )
				{
					pL=new SRGraphLabel;
 					AddLabel(pL);
				}
				else
					pL=(SRGraphLabel*)m_LabelList.GetAt(m_LabelList.FindIndex(nLabelIndex));

				ASSERT(pL!=NULL); // if it is I dont know why!
				
				nLabelIndex++;

				dt=(COleDateTime)dCount;

				pL->SetAnnotation(dt.Format(strFormatString));
				pL->SetTextPosition(m_ptStart.x+(int)(xScale*(dCount-m_dMinValue)),
									m_ptStart.y+(int)(yScale*(dCount-m_dMinValue)));
				double dTempStep=0.0;
				if(m_bAscending)
				{
					dCount+=dTempRes;
					if(dCount>m_dMaxValue)
					{
						dCount=m_dMaxValue;
						nEndCount++;
					}
				}
				else
				{
					dCount-=dTempRes;
					if(dCount<m_dMinValue)
					{
						dCount=m_dMinValue;
						nEndCount++;
					}
				}

			}
	}

	
			   
	while(nLabelIndex<m_LabelList.GetCount())
		delete ((SRGraphLabel *)m_LabelList.RemoveTail());


	SetRTAngles();

	UpdateTickList();

	m_pDC->SelectObject(OldFont);
}


SRGDateScale::SRGDateScale()
{
	m_bAscending=FALSE;
	m_ptStart=CPoint(0,0);
	m_ptEnd=CPoint(0,0);
	m_dMinValue=0;
	m_dMaxValue=0;
	m_dResolution=0;
	m_strFormatString.Empty();
	m_bLabelSide=FALSE;
	m_bCheckOverlaps=TRUE;
}

//@mfunc Sets the m_dMaxValue member using a COleDateTime value.
//@parm The date-time object
void SRGDateScale::SetMaxValue(COleDateTime dt)
{
	m_dMaxValue=(double)dt;
}

//@mfunc Sets the m_dMinValue member using a COleDateTime value.
//@parm The date-time object
void SRGDateScale::SetMinValue(COleDateTime dt)
{
	m_dMinValue=(double)dt;
}

//@mfunc Sets the m_dResolution member using a COleDateTimeSpan value.
//@parm The date-time object
void SRGDateScale::SetResolution(COleDateTimeSpan dt)
{
	m_dResolution=(double)dt;
}

SRGIndexScale::SRGIndexScale()
{
	m_bAscending=FALSE;
	m_ptStart=CPoint(0,0);
	m_ptEnd=CPoint(0,0);
	m_strFormatString.Empty();
	m_bLabelSide=FALSE;
	m_bCheckOverlaps=FALSE;
}

SRGGroupScale::SRGGroupScale()
{
	m_bAscending=FALSE;
	m_ptStart=CPoint(0,0);
	m_ptEnd=CPoint(0,0);
	m_strFormatString.Empty();
	m_bLabelSide=FALSE;
	m_bCheckOverlaps=FALSE;
}

void SRGGroupScale::CreateLabels()
{
	
	int nMinSet,nSetCount;

	nMinSet=m_nMinSet==-1 ? 0 : m_nMinSet;
	nSetCount=m_nMaxSet==-1 ? m_pParentGraph->GetGroupCount()-nMinSet : m_nMaxSet-nMinSet;

	for(int CurrentSet=0; CurrentSet<nSetCount; CurrentSet++)
	{
		SRGraphLabel * pL;
		if(m_LabelList.IsEmpty() || m_LabelList.GetCount()<=CurrentSet )
		{
			pL=new SRGraphLabel;
 			AddLabel(pL);
		}
		else
			pL=(SRGraphLabel*)m_LabelList.GetAt(m_LabelList.FindIndex(CurrentSet));

		pL->SetAnnotation(m_pParentGraph->GetGroup(CurrentSet+nMinSet)->GetAnnotation(m_pParentGraph->GetCountryCode()));
	}

	while(CurrentSet<m_LabelList.GetCount())
		delete ((SRGraphLabel *)m_LabelList.RemoveTail());

	SetRTAngles();

	UpdateTickList();

}

void SRGIndexScale::CreateLabels()
{
	int nMinIndex,nIndexCount,nGroup;

	nMinIndex=m_nMinIndex==-1 ? 0 : m_nMinIndex;
	nIndexCount=m_nMaxIndex==-1 ? m_pParentGraph->GetMaxXDivisions()-nMinIndex : m_nMaxIndex-nMinIndex;
	nGroup=m_nMinSet==-1 ? 0 : m_nMinSet;

	for(int x=0; x<nIndexCount; x++)
	{
		SRGraphLabel * pL;
		if(m_LabelList.IsEmpty() || m_LabelList.GetCount()<=x )
		{
			pL=new SRGraphLabel;
 			AddLabel(pL);
		}
		else
			pL=(SRGraphLabel*)m_LabelList.GetAt(m_LabelList.FindIndex(x));

		pL->SetAnnotation(m_pParentGraph->GetGroup(nGroup)->GetIndex(x+nMinIndex)->GetAnnotation(m_pParentGraph->GetCountryCode()));
	}

	while(x<m_LabelList.GetCount())
		delete ((SRGraphLabel *)m_LabelList.RemoveTail());

	SetRTAngles();

	UpdateTickList();

}
