//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc

//@module DScale.h | Class definition file for SRGDecimalScale class. This class is derived from
// SRGraphLabelBlock and handles the display of decimal numeric scales along the sides of axes.

///////////////////////////////////////////////////////////////////////
// Autoduck comment block for SRGDecimal Scale
//


//@mdata BOOL | SRGDecimalScale | m_bAscending | Semaphore. When TRUE numeric values count up from m_ptStart to m_ptEnd. 
//when FALSE they count down.
//@see <mf SRGDecimalScale::SetAscending> <mf SRGDecimalScale::SetDescending>

//@mdata CPoint | SRGDecimalScale | m_ptStart | Defines the point in the device context where the scale line should commence. In the case of 
//the classic axis for example this line starts at the graph origin.
//@see <md SRGDecimalScale::m_ptEnd>	<mf SRGDecimalScale::SetStartPos>

//@mdata CPoint | SRGDecimalScale | m_ptEnd | Defines the point in the device context where the scale line should end.
//@see <md SRGDecimalScale::m_ptStart> <mf SRGDecimalScale::SetEndPos>

//@mdata double | SRGDecimalScale | m_dMinValue | The minimum numeric value of the scale. The numeric scale will be shown in steps of <md SRGDecimalScale::m_dResolution> until the value in <md SRGDecimalScale::m_dMaxValue> is reached
//@see <mf SRGDecimalScale::SetMinValue> <mf SRGDecimalScale::GetMinValue>

//@mdata double | SRGDecimalScale | m_dMaxValue | The maximum numeric value of the scale. The numeric scale will be shown in steps of <md SRGDecimalScale::m_dResolution> from the value in <md SRGDecimalScale::m_dMinValue> until this value is reached.
//@see <mf SRGDecimalScale::SetMaxValue> <mf SRGDecimalScale::GetMaxValue>

//@mdata double | SRGDecimalScale | m_dResolution | The step of numeric resolution between labels. By default this value is 0.01 but it may be set to any desired value.<nl>
//@rem This value will automatically be changed to suit the length of the line and the size of the font. The formula for these changes allows a sensible scale to be created 
//which works with most graph types. For example if a decimal scale is require from 0 to 10 the line length and font size will be taken into account and steps may be displayed as..<nl>
//0 1 2 3 4 5 6 7 8 9 10 or..<nl>
//0 2 4 6 8 10 or<nl>
//0 0.5 1 1.5 2 2.5 3 3.5 etc.<nl>
//@see <mf SRGDecimalScale::SetResolution> <mf SRGDecimalScale::GetResolution>

//@mdata CString | SRGDecimalScale | m_strFormatString | This is a printf style format string which should contain only one floating point type format specifier and any other text that you want on your labels.
//@rem You can use this to embellish your labels. For example you might supply the format string of " $%.2f " for displaying dollar pricings
//@see <mf SRGDecimalScale::SetFormatString> 11GetFormatString>

//@mdata BOOL | SRGDecimalScale | m_bLabelSide | The side of the imagiinary line upon which labels are placed. The default (FALSE) is the left. To set them on the right this semaphore must be TRUE.
//@ex |
//FALSE			TRUE
//
//10 |			| 10
// 8 |			| 8
// 6 |			| 6
// 4 |			| 4
// 2 |			| 2
// 0 |			| 0

//@mfunc void | SRGDecimalScale | SetStartPos | Defines the point at which the scale line starts.
//@parm CPoint | prStart | The point position
//@rem Between this point and the point defined in m_ptEnd an imaginary line is drawn. The length of this line, its angle and the 
//size of the current font will all contribute to the final scale and the angle at which the labels are displayed.
//@see <md SRGDecimalScale::m_ptrStart> <md SRGDecimalScale::m_ptEnd> <mf SRGDecimalScale::SetEndPos>

//@mfunc  none (constuctor) | SRGDecimalScale | SRGDecimalScale | Constructor with parameters
//@parm CPoint | ptStart | The start point for the scale line
//@parm CPoint | ptEnd | The end point.
//@parm double | dMinValue | The minimum or starting value.
//@parm double | dMaxValue | The maximum or ending value.
//@parm double | dResolution | The step resolution.
//@parm LPCSTR | strFormatString | The printf style format string for the labels.
//@rem This constructor provides a one-shot setup for most of the decimal label needs.
//@see <md SRGDecimalScale::m_ptStart> <md SRGDecimalScale::m_ptEnd> <md SRGDecimalScale::m_dMaxValue> <md SRGDecimalScale::m_dMinValue> <md SRGDecimalScale:: m_dMaxValue> <md SRGDecimalScale::m_strFormatString> <md SRGDecimalScale::m_dResolution>

//@mfunc void | SRGDecimalScale | SetEndPos | Defines the point at which the scale line ends.
//@parm CPoint | ptEnd | The point position
//@see <md SRGDecimalScale::m_ptrStart> <md SRGDecimalScale::m_ptEnd> <mf SRGDecimalScale::SetStartPos>

//@mfunc void | SRGDecimalScale | SetResolution | Initializes the resolution of the steps in between labels.
//@parm double | dResolution | The resolution.
//@rem If your minimum and maximum values are 0 and 10 respectively and this value is set to 1.0 the scale will be calculated 
//Providing there is room for all the labels... 0 1 2 3 4 5 6 7 8 9 10<nl>
//If there is not enough room the next best solution is... 0 2 4 6 8 10<nl>
//If that wont fit then.. 0 5 10 will be calculated.
//Whatever the range of values the scale will allways be divided by 1, 5, 2 and 10 repeatedly until a fit is found.
//@see <mf SRGDecimalScale::GetResolution> <md SRGDecimalScale::m_dResolution>

//@mfunc void | SRGDecimalScale | SetMaxValue | This sets the maximum value which a scale may display.
//@parm double | dMaxValue | The value.
//@ex |
//	SetMaxValue(10);
//	SetMinValue(0);
//	SetResolution(0.1);
//@normal will make an ideal percentage scale.
//@see <md SRGDecimalScale::m_dMinValue> <mf SRGDecimalScale::SetMaxValue> <mf SRGDecimalScale::GetMinValue>

//@mfunc void | SRGDecimalScale | SetMinValue | This sets the maximum value which a scale may display.
//@parm double | dMinValue | The actual value.
//@ex |
//	SetMaxValue(10);
//	SetMinValue(0);
//	SetResolution(0.1);
//@normal will make an ideal percentage scale.
//@see <md SRGDecimalScale::m_dMaxValue> <mf SRGDecimalScale::SetMinValue> <mf SRGDecimalScale::GetMaxValue>


//@mfunc void | SRGDecimalScale | SetFormatString | Initialises a printf style format string for the labels.
//@parm LPCSTR | strFormatString | The format string.
//@see <md SRGDecimalScale::m_strFormatString> <mf SRGDecimalScale::GetFormatString>

//@mfunc void | SRGDecimalScale | SetLabelSide | Initializes the semaphore which decides which side of the line between m_ptStart and m_ptEnd the labels are drawn.
//@parm BOOL | b | TRUE to set labels on the right of the line, FALSE to set them to the left.
//@see <md SRGDecimalScale::m_bLabelSide> <mf SRGDecimalScale::GetLabelSide>

//@mfunc CPoint | SRGDecimalScale | GetStartPos | Interrogates the m_ptStart member.
//@rdesc a CPoint object containing the start position of the scale line.
//@see <md SRGDecimalScale::m_ptStart> <md SRGDecimalScale::m_ptEnd> <mf SRGDecimalScale::SetStartPos> <mf SRGDecimalScale::SetEndPos> <mf SRGDecimalScale::GetStartPos> <mf SRGDecimalScale::GetEndPos>

//@mfunc CPoint | SRGDecimalScale | GetEndPos | Interrogates the m_ptEnd member.
//@rdesc a CPoint object containing the end position of the scale line.
//@see <md SRGDecimalScale::m_ptStart> <md SRGDecimalScale::m_ptEnd> <mf SRGDecimalScale::SetStartPos> <mf SRGDecimalScale::SetEndPos> <mf SRGDecimalScale::GetStartPos> <mf SRGDecimalScale::GetEndPos>

//@mfunc double | SRGDecimalScale | GetResolution | Interrogates the m_dResolution member
//@rdesc The value which is used to step from the minimum to the maximum values for this object
//@see <md SRGDecimalScale::m_dResolution> <mf SRGDecimalScale::SetResolution>

//@mfunc void | SRGDecimalScale | SetAscending | Call to force labels to be drawn in ascending order.
//@see <mf SRGDecimalScale::SetDescending> <md SRGDecimalScale::m_bAscending>

//@mfunc void | SRGDecimalScale | SetDescending | Call to force labels to be drawn in descending order.
//@see <mf SRGDecimalScale::SetAscending> <md SRGDecimalScale::m_bAscending>

//@mfunc CString& | SRGDecimalScale |  GetFormatString | Interrogates the m_strFormatString member
//@rdesc Reference to a CString object containing the printf style format string.
//@see <md SRGDecimalScale::m_strFormatString> <mf SRGDecimalScale::SetFormatString>

//@mfunc BOOL | SRGDecimalScale | GetLabelSide | Interrogates the m_bLabelSide member.
//@rdesc TRUE if labels are set to display on the right side of the line. FALSE if they are on the left.
//@see <md SRGDecimalScale::m_bLabelSide> <mf SRGDecimalScale::SetLabelSide>
	
//@mfunc double | SRGDecimalScale | GetMaxValue | Interrogates the m_dMaxValue member.
//@rdesc The contents of the m_dMaxValue member.

//@mfunc double | SRGDecimalScale | GetMinValue |  Interrogates the m_dMinValue member.
//@rdesc The contents of the m_dMinValue member.
	
////////////////////////////////////////////////////////////////////////

#include <afxdisp.h>

#ifndef __SRGDECIMALSCALE__
#define __SRGDECIMALSCALE__


#ifdef _OBJCHART_DLL
	#undef AFXAPP_DATA
	#define AFXAPP_DATA AFXAPI_DATA
	#undef AFX_DATA
	#define AFX_DATA OC_DATA
#endif


//@class Derived from SRGraphLabelBlock this class manages a decimal numeric scale which can be 
//placed between two screen point positions and which will automatically calculate the best possible 
//sequence of numbers and resolution depending upon the length of the line and the size of the font.<nl>
//This class will by default draw all the labels perpendicular to the imaginary line joining the two end points.
//After the labels are created they may be manipulated using the standard SRGraphLabelBlock methods.
class SRGDecimalScale : public SRGraphLabelBlock
{
//@access Protected data members
protected:
	//@cmember TRUE when the list of numerals generated is to go from low values toward high values
	BOOL m_bAscending;
	//@cmember The start position of the line of numerals
	CPoint m_ptStart;
	//@cmember The end position of the line of numerals
	CPoint m_ptEnd;
	//@cmember The minimum numeric value in the range of generated numbers
	double m_dMinValue;
	//@cmember The maximum numeric value in the range of generated numbers
	double m_dMaxValue;
	//@cmember The preferred or initial resolution of steps in the number list.
	double m_dResolution;
	//@cmember The format string used to generate the numeral strings. (see sprintf)
	CString m_strFormatString;
	//@cmember The side of the imaginary line between m_ptStart and m_ptEnd on which labels are displayed. FALSE=left. TRUE=right.
	BOOL m_bLabelSide;

//@access public member functions 
public:
	//@cmember Constructor
	SRGDecimalScale();
	
	//@cmember Constructor with initialisation parameters
	SRGDecimalScale(CPoint ptStart, CPoint ptEnd, double dMinValue, double dMaxValue, double dResolution,CString & strFormatString);
	
	//@cmember Destructor
	virtual ~SRGDecimalScale();

	//@cmember Initialise the start position of the scale line
	virtual void SetStartPos(CPoint ptStart){m_ptStart=ptStart;}

	//@cmember Initialize the end position of the scale line
	virtual void SetEndPos(CPoint ptEnd){m_ptEnd=ptEnd;}
	
	//@cmember Initialise the label step resolution (defaults to 0.1)
	virtual void SetResolution(double dRes){m_dResolution=dRes;}
	
	//@cmember  Sets the maximum value for this scale
	virtual void SetMaxValue(double dMaxValue){m_dMaxValue=dMaxValue;}
	
	//@cmember 	Sets the minimum value for this scale
	virtual void SetMinValue(double dMinValue){m_dMinValue=dMinValue;}
	
	//@cmember Initialises a printf style format string for the labels.
	virtual void SetFormatString(LPCSTR strFormatString){m_strFormatString=strFormatString;}
	
	//@cmember Initialises the m_bLabelSide semaphore.
	virtual void SetLabelSide(BOOL bRight){m_bLabelSide=bRight;}
	
	//@cmember  Sets ascending label order
	virtual void SetAscending(){m_bAscending=TRUE;};
	
	//@cmember Sets decsending label order
	virtual void SetDescending(){m_bAscending=FALSE;}
	
	//@cmember Interrogates the m_ptStart member.
	virtual CPoint GetStartPos(){return m_ptStart;}
	
	//@cmember Interrogates the m_ptEnd member.
	virtual CPoint GetEndPos(){return m_ptEnd;}
	
	//@cmember Interrogates the m_dResolution member
	virtual double GetResolution(){return m_dResolution;}
	
	//@cmember Interrogates the m_dMaxValue member.
	virtual double GetMaxValue(){return m_dMaxValue;}
	
	//@cmember Interrogates the m_dMinValue member.
	virtual double GetMinValue(){return m_dMinValue;}
	
	//@cmember Interrogates the m_strFormatString member
	virtual CString & GetFormatString(){return m_strFormatString;}
	
	//@cmember Interrogates the m_bLabelSide member
	virtual BOOL GetLabelSide(){return m_bLabelSide;}
	
	//@cmember Creates the label list from available information.
	virtual void CreateLabels();

	//@cmember Sets all labels perpendicular to the line between the start and end points
	virtual void SetRTAngles();

};

//@mfunc void | SRGLogScale | SRGLogScale | Constructor.
//@parm CPoint | ptStart | The starting point for the line of labels
//@parm CPoint | ptEnd | The end point for the line of labels
//@parm double | dMinValue | Maximum numeric value
//@parm double | dMaxValue |  Minimum numeric value
//@parm double | dResolution | Resolution between steps  (Unused [ignored])
//@parm CString | strFormatString | The format string  (Unused [ignored])

//@class This class manages the logarithmic equivalent of the decimal scale
class SRGLogScale : public SRGDecimalScale
{
public:
	//@cmember Constructor
	SRGLogScale(){m_bCheckOverlaps=TRUE;}
	//@cmember Constructor with parameters
	SRGLogScale(CPoint ptStart, CPoint ptEnd, double dMinValue, double dMaxValue, double dResolution,CString & strFormatString);

	//@cmember The only significant change from the ancestor class.
	virtual void CreateLabels();
};

//@class Derived from and similar in operation to the SRGDecimalScale this axis labelling scale provides
//date and time support via the COLEDateTime functions. The maximum and minimum scales can be set to decimal values which 
//correspond to the new date and time standards and sensible annotations will be derived from them.
class SRGDateScale : public SRGDecimalScale
{
public:
	//@cmember Constructor
	SRGDateScale();

	//@cmember Builds the label list.
	virtual void CreateLabels();

//Implementation

	//@cmember Sets the maximum scale value as a COleDateTime object
	virtual void SetMaxValue(COleDateTime t);
	//@cmember Sets the minimum scale value as a COleDateTime object
	virtual void SetMinValue(COleDateTime t);
	//@cmember Sets the resolution value as a COleDateTimeSpan object
	virtual void SetResolution(COleDateTimeSpan s);

};

//@class Creates and displays a label-block style chart scale from the index annotations of an SRGraph object.
class SRGIndexScale : public SRGDecimalScale
{
public:
	SRGIndexScale();

	virtual void CreateLabels();
};

//@class Creates and displays a label-block style chart scale from the group annotations of an SRGraph object.
class SRGGroupScale : public SRGDecimalScale
{
public:
	SRGGroupScale();

	virtual void CreateLabels();
};


#endif //__SRGDECIMALSCALE__

#undef AFX_DATA
#define AFX_DATA
#undef AFXAPP_DATA
#define AFXAPP_DATA NEAR
