//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       GraphView.cpp	[ GraphView.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner - based on original that was purchased with Objective Chart (see above)
//
// OVERVIEW:   Implements a graph view class, CGraphView, derived from CView
//
//***************************************************************************

#include "stdafx.h"

#include <mmsystem.h>
#include <math.h>			

#include "SourceMetrics.h"
#include "SourceMetricsDoc.h"
#include "GraphView.h"

#include "TreeCtrl.h"   // need these only if using the graph edit dialog (during development)
#include "grpeddlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphView

IMPLEMENT_DYNCREATE(CGraphView, CView)

BEGIN_MESSAGE_MAP(CGraphView, CView)
	//{{AFX_MSG_MAP(CGraphView)
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	//Standard graph view commands
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphView construction/destruction

// Constructor. Initialises vital variables
CGraphView::CGraphView()
   : CView(), m_bInFeedback(FALSE), m_bShowingTip(FALSE), m_bMouseTimerIsRunning(FALSE), m_iValueDisplayLatency(250)
{
}

//mfunc Destructor 
CGraphView::~CGraphView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CGraphView drawing
// The drawing is very simply performed by calling the DrawComponentList 
// member of the graph which is stored in the document
void CGraphView::OnDraw(CDC* pDC)
{
	m_oGraph.DrawComponentList(pDC, this); // This is all you need to do to draw the graph
}

/////////////////////////////////////////////////////////////////////////////
// CGraphView printing

// Simple default print preparation
BOOL CGraphView::OnPreparePrinting(CPrintInfo* pInfo)
{
   pInfo->SetMaxPage(1);      // graphs always print on a single page

	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGraphView::OnBeginPrinting(CDC * /* pDC */, CPrintInfo * /* pInfo */)
{
   CRect oPrintRect(0, 0, 1024, 768);

	// Here we can set up the SRGraph item in the document to obey our page setup choice
   GetClientRect(&oPrintRect);
//   m_oGraph.SetMaxArea(oPrintRect, TRUE);
//   m_oGraph.SetAlignment((DWORD) ((1+1) | ((1+1)<<4)));
}

// End printing resets the SRGraph so that it goes back to using the normal screen DC
// and the default sizing and drawing modes.
void CGraphView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// Here we reset the SRGraph object to the default page behaviour for on-screen display.
	m_oGraph.SetUseMaxArea(FALSE);
	m_oGraph.SetSignificantChanges();
	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CGraphView diagnostics

#ifdef _DEBUG
void CGraphView::AssertValid() const
{
	CView::AssertValid();
}

void CGraphView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraphView message handlers

// Note that the erase background is completely superfluous. The graph itself will handle all of the painting of 
// the client area. The one caveat is that you must remember to have one item on the component list which
// draws a fill of some description over the whole screen area. If you do not then you will find that the
// background is filled with whatever rubbish happened to be in the memory at the time. 
// If you use the OnEraseBkgnd routine here you will get a nasty screen flicker even if the graph is back
// buffering the drawing process.
BOOL CGraphView::OnEraseBkgnd(CDC* /* pDC */) 
{
	return TRUE;	// The graph class manages the image in a backbuffer
	//return CView::OnEraseBkgnd(pDC);
}

// Every time the mouse moves in our window we trigger a timer to measure m_iValueDisplayLatency of elapsed time.
// If the mouse keeps moving the timer will never reach m_iValueDisplayLatency milliseconds.
// If it stops for more than m_iValueDisplayLatency milliseconds the OnTimer() routine will be invoked and we can
// perform our timed task.
void CGraphView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_bMouseTimerIsRunning)
		KillTimer(SG_MOUSE_TIMER_EVENT); // kill it.
	if(point==m_MousePoint)
		return;                          // this mouse move was probably generated by change of window Z order

	if(m_GraphTip.IsShowing())
		m_GraphTip.EraseTip();

	m_bMouseTimerIsRunning=TRUE; //Always start a new timer
	m_TimerIdent=SetTimer(SG_MOUSE_TIMER_EVENT, m_iValueDisplayLatency, NULL);
	m_MousePoint=point;	               // and save the position	

	CView::OnMouseMove(nFlags, point);
}

// If this timer has been allowed to fire the stored mouse position (i.e. that where the mouse has been left
// inactive for 1.5 seconds) will be checked for a feedback polygon. If one is found a chart tip is displayed.
// If not the timer is simply stopped.
void CGraphView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent == SG_MOUSE_TIMER_EVENT)
	{	// its def'nitly ours
		KillTimer(nIDEvent);
		SRGraphFeedback *pFeedback = m_oGraph.GetFeedback(m_MousePoint.x,m_MousePoint.y);
		if(pFeedback!=NULL)
			ShowFeedback(pFeedback);
	}
	else
		CView::OnTimer(nIDEvent);
}

// Copies the screen bitmap to the clipboard for pasting into your Wordprocessor files etc.
void CGraphView::OnEditCopy() 
{
	if(m_GraphTip.IsShowing())
		m_GraphTip.EraseTip();
	CRect r;
	CDC memDC, *pDC=GetDC();
	GetClientRect(&r);
	memDC.CreateCompatibleDC(pDC);
	CBitmap CopyBitmap;
	CopyBitmap.CreateCompatibleBitmap(pDC,r.Width(),r.Height());
	CBitmap *obm=(CBitmap *)memDC.SelectObject(&CopyBitmap);
	memDC.BitBlt(0,0,r.Width(),r.Height(),pDC,0,0,SRCCOPY);
	memDC.SelectObject(obm);
	memDC.DeleteDC();
	ReleaseDC(pDC);

	if(OpenClipboard())
	{
	   EmptyClipboard();
	   HBITMAP hbm=(HBITMAP)CopyBitmap.Detach();
	   SetClipboardData(CF_BITMAP,hbm);
	   CloseClipboard();
	}
	else
		CopyBitmap.DeleteObject();
}

// The default feedback display is more of a demonstration default. To perform some useful 
// feedback function you should override this function in your own class derivation.
void CGraphView::ShowFeedback(SRGraphFeedback *pFeedback)
{
	CString strText;
	strText.Format("Index=%d Group=%d Value=%g",
		pFeedback->m_nIndex,
		pFeedback->m_nGroup,
		m_oGraph.GetGroup(pFeedback->m_nGroup)->GetIndex(pFeedback->m_nIndex)->GetValue());

	CPoint tPoint=m_MousePoint;
	
	ClientToScreen(&tPoint);
	
	m_GraphTip.ShowTip(strText,tPoint,this);
}
