//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc
// GraphView.cpp : implementation of the SRGraphView class. Also includes
// GraphTip, CSelectQuery and PageSetup.
//
#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"

#include "SRGEdit.h"
#include <mmsystem.h>
#include <math.h>			

#include /* */ "GraphDoc.h"
#include /* */ "GraphVw.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SRGraphView

IMPLEMENT_DYNCREATE(SRGraphView, CView)

BEGIN_MESSAGE_MAP(SRGraphView, CView)
	//{{AFX_MSG_MAP(SRGraphView)
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_TIMER()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	//Standard graph view commands
	ON_COMMAND(ID_GRAPH_ZOOMOUT, OnGraphZoomout)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_ZOOMOUT, OnUpdateGraphZoomout)
	ON_COMMAND(ID_GRAPH_PAGESETUP, OnGraphPagesetup)
	ON_COMMAND(ID_GRAPH_EDIT, OnGraphEdit)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// SRGraphView construction/destruction

//@mfunc Constructor. Initialises vital variables
SRGraphView::SRGraphView()
{
	m_bSelecting=FALSE;
	m_bDragging=FALSE;
	m_bInFeedback=FALSE;
	m_bShowingTip=FALSE;
	m_bZoomed=FALSE;
	m_pZoomedDisplay=NULL;
	m_bMouseTimerIsRunning=FALSE;
	m_bUseMaxArea=FALSE;
	m_nValign=1;
	m_nHalign=1;
	m_nPageSizeCode=0;
}

//mfunc Destructor 
SRGraphView::~SRGraphView()
{
}

/////////////////////////////////////////////////////////////////////////////
// SRGraphView drawing


//@mfunc The drawing is very simply performed by calling the DrawComponentList 
//member of the graph which is stored in the document.
//@parm A pointer to the device context.
void SRGraphView::OnDraw(CDC* pDC)
{
	CGraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	GetGraph()->DrawComponentList(pDC,this);	// This is all you need to do to draw the graph
}

/////////////////////////////////////////////////////////////////////////////
// SRGraphView printing

//@mfunc Simple default print preparation
//@parm The print info structure
BOOL SRGraphView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

//@mfunc Begin the printing process
//@parm a print DC
//@parm the print info structure
//@rem This routine performs simple pagination including the setting up of a page size to one which may have been chosen from a list by the page-setup dialog. 
//It also arranges for the page to be positioned in the available area. Note that the SRGraph object itself manages pagination details, albeit in a simple form, 
//The view class simply passes along the details.
//@devnote Large pages which are aligned in the page can take longer to print because the SRGraph object may not be able to draw directly to the provided print DC. 
//When this happens a compatible DC of similar proportions is created and the drawing performed there before it is all copied to the print DC.
void SRGraphView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	pInfo; //UNUSED
	// Here we can set up the SRGraph item in the document to obey our page setup choice
	CGraphDoc *pDoc=GetDocument();
	CRect AreaRect;

	if(m_bUseMaxArea)
	{
		int nResX=pDC->GetDeviceCaps(LOGPIXELSX),
			nResY=pDC->GetDeviceCaps(LOGPIXELSY);
		switch(m_nPageSizeCode)
		{
		case 0:
			//320 X 240
			AreaRect=CRect(0,0,320,240);
			break;
		case 1:
			//640 X 480
			AreaRect=CRect(0,0,640,480);
			break;
		case 2:
			//800 X 600
			AreaRect=CRect(0,0,800,600);
			break;
		case 3:
			//1024 X 768
			AreaRect=CRect(0,0,1024,768);
			break;
		case 4:
			//A4 portrait
			AreaRect=CRect(0,0,(int)((double)nResX*8.267),(int)((double)nResY*11.692));
			break;
		case 5:
			//A4 landscape
			AreaRect=CRect(0,0,(int)((double)nResX*11.692),(int)((double)nResY*8.267));
			break;
		case 6:
			//A5 portrait
			AreaRect=CRect(0,0,(int)((double)nResX*5.874),(int)((double)nResY*8.267));
			break;
		case 7:
			//A5 landscape
			AreaRect=CRect(0,0,(int)((double)nResX*8.267),(int)((double)nResY*5.874));
			break;
		}
		GetGraph()->SetMaxArea(AreaRect,TRUE);

		GetGraph()->SetAlignment((DWORD) ((1+m_nValign) | ((1+m_nHalign)<<4)));
	}
}

//@mfunc End printing resets the SRGraph so that it goes back to using the normal screen DC and the default sizing and drawing modes.
//@parm Not used
//#parm Not used
void SRGraphView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// Here we reset the SRGraph object to the default page behaviour for on-screen display.
	CGraphDoc *pDoc=GetDocument();
	GetGraph()->SetUseMaxArea(FALSE);

}

/////////////////////////////////////////////////////////////////////////////
// SRGraphView diagnostics

#ifdef _DEBUG
void SRGraphView::AssertValid() const
{
	CView::AssertValid();
}

void SRGraphView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGraphDoc* SRGraphView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGraphDoc)));
	return (CGraphDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// SRGraphView message handlers

//@mfunc Note that the erase background is completely superfluous. The graph itself will handle all of the painting of 
//the client area. The one caveat is that you must remember to have one item on the component list which draws a fill of some description 
//over the whole screen area. If you do not then you will find that the background is filled with whatever rubbish happened to be in the memory at the time. 
//If you use the OnEraseBkgnd routine here you will get a nasty screen flicker even if the graph is back buffering the drawing process.
BOOL SRGraphView::OnEraseBkgnd(CDC* pDC) 
{
	pDC; //UNUSED
	return TRUE;	// The graph class manages the image
					// in a backbuffer
	//return CView::OnEraseBkgnd(pDC);
}


//@mfunc  Every time the mouse moves in our window we trigger a timer to measure 1.5 seconds of elapsed time.
// If the mouse keeps moving the timer will never reach 1.5 seconds. if it stops for more than 1.5 seconds
// the OnTimer() routine will be invoked and we can perform our timed task.
//@parm The flags (unused)
//@parm The mouse point.
void SRGraphView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CGraphDoc *pDoc=GetDocument();

	if(m_bMouseTimerIsRunning)
		KillTimer(MOUSE_TIMER_EVENT); // kill it.
	if(point==m_MousePoint)
		return; // this mouse move was probably generated by change of window Z order

	if(m_GraphTip.IsShowing())
		m_GraphTip.EraseTip();

	m_bMouseTimerIsRunning=TRUE; //Always start a new timer
	m_TimerIdent=SetTimer(MOUSE_TIMER_EVENT,1500,NULL); //of 1.5 seconds
	m_MousePoint=point;	// and save the position	
	if(m_bDragging)
		MouseDrag(nFlags,point);
	if(m_bSelecting)
		DrawSelection(point);

	CView::OnMouseMove(nFlags, point);
}


//@mfunc Draws an inverted selection rectangle from the first point which was stored 
//when the mouse button was first pressed to the point supplied. 
//This routine uses information returned by <c SRGraphPosition> to decide what type of selection is to be drawn. 
//It may be horizontal, vertical, a free form banded box or a circular selection depending on the type of graph being 
//displayed.
//@parm the current point which defines the opposite end of the selection.
void SRGraphView::DrawSelection(CPoint point)
{
	CGraphDoc *pDoc=GetDocument();
	SRGraphPosition *gp=GetPosition(&pDoc->m_Graph,point);
	if(gp->GetPosType()==CX_POSITION_SINGLE)
	{
		CRect dR=((SRGraphDisplay *)gp->GetComponent())->GetDisplayRect();

		switch(gp->GetSelectionType())
		{
		case CX_POSITION_SELECT_H:
			if(dR.PtInRect(m_SelectionPointA) && dR.PtInRect(m_SelectionPointB) && dR.PtInRect(point))
			{
				CDC *pDC=GetDC();
				CRect sR;
				sR.SetRect(m_SelectionPointA.x,dR.top,m_SelectionPointB.x,dR.bottom);
				sR.NormalizeRect();
				pDC->InvertRect(sR);
				if(point!=m_SelectionPointB)
				{
					m_SelectionPointB=point;
					sR.SetRect(m_SelectionPointA.x,dR.top,m_SelectionPointB.x,dR.bottom);
					sR.NormalizeRect();
					pDC->InvertRect(sR);
				}
				ReleaseDC(pDC);
			}
			break;
		case CX_POSITION_SELECT_V:
			if(dR.PtInRect(m_SelectionPointA) && dR.PtInRect(m_SelectionPointB) && dR.PtInRect(point))
			{
				CDC *pDC=GetDC();
				CRect sR;
				sR.SetRect(dR.left,m_SelectionPointA.y,dR.right,m_SelectionPointB.y);
				sR.NormalizeRect();
				pDC->InvertRect(sR);
				if(point!=m_SelectionPointB)
				{
					m_SelectionPointB=point;
					sR.SetRect(dR.left,m_SelectionPointA.y,dR.right,m_SelectionPointB.y);
					sR.NormalizeRect();
					pDC->InvertRect(sR);
				}
				ReleaseDC(pDC);
			}
			break;
		case CX_POSITION_SELECT_B:
			if(/*point!=m_SelectionPointB && */dR.PtInRect(m_SelectionPointA) && dR.PtInRect(m_SelectionPointB) && dR.PtInRect(point))
			{
				CDC *pDC=GetDC();
				CRect sR;
				sR.SetRect(m_SelectionPointA.x,m_SelectionPointA.y,m_SelectionPointB.x,m_SelectionPointB.y);
				sR.NormalizeRect();
				pDC->InvertRect(sR);
				if(point!=m_SelectionPointB)
				{
					m_SelectionPointB=point;
					sR.SetRect(m_SelectionPointA.x,m_SelectionPointA.y,m_SelectionPointB.x,m_SelectionPointB.y);
					sR.NormalizeRect();
					pDC->InvertRect(sR);
				}
				ReleaseDC(pDC);
			}
			break;
		case CX_POSITION_SELECT_C:
			if(dR.PtInRect(m_SelectionPointA) && dR.PtInRect(m_SelectionPointB) && dR.PtInRect(point))
			{
				const int nClockDivs=512;
				const double dClockStep=TWO_PI/nClockDivs;

				CPoint points[nClockDivs+2]; // max of 144 around plus  two center points
				int nPointCount;

				int nCenterX=dR.left+(dR.Width()/2),
					nCenterY=dR.top+(dR.Height()/2);

				double dRadius=0.8*(double)min(dR.Height(),dR.Width())/2;
				if(dRadius<1)
					break;
				
				double dAngleA,dAngleB;

				CRgn rgn;

				CDC *pDC=GetDC();

				for(int pass=0;pass<2;pass++)
				{

					// here we can steal a function from the component we are drawing..

					dAngleA=gp->GetComponent()->GetLineAngle(m_SelectionPointA,CPoint(nCenterX,nCenterY));
					//dAngleA-=PI/2;  // Trig functions start at 3 o'clock so adding 90 degrees will shift them to the right place
					if(dAngleA<0)
						dAngleA+=TWO_PI; // Un-wind the angles if neccesary.

					dAngleB=gp->GetComponent()->GetLineAngle(m_SelectionPointB,CPoint(nCenterX,nCenterY));
					//dAngleB-=PI/2;
					if(dAngleB<0)
						dAngleB+=TWO_PI;

					if(dAngleA>dAngleB)
					{
						double dTemp=dAngleA;
						dAngleA=dAngleB;
						dAngleB=dTemp;
					}

					if(dAngleB-dAngleA<dClockStep)
						dAngleB=dAngleA+dClockStep;

					nPointCount=1;
					points[0]=CPoint(nCenterX,nCenterY);
					
					for(dAngleA;dAngleA<dAngleB;dAngleA+=dClockStep)
					{
						points[nPointCount++]=CPoint(-(int)(dRadius*cos(dAngleA-(PI/2)))+nCenterX,(int)(dRadius*sin(dAngleA-(PI/2)))+nCenterY);
					}
					points[nPointCount++]=points[0];

					rgn.CreatePolygonRgn(&points[0],nPointCount,ALTERNATE);
					pDC->InvertRgn(&rgn);
					rgn.DeleteObject();
					if(m_SelectionPointB!=point)
						m_SelectionPointB=point;
					else
						break;
				}

				ReleaseDC(pDC);
			}
			break;
		default:
			DrawCustomSelection(point);
			break;
		}
	}

	delete(gp);
}


//@mfunc When the button is pressed there may be a number of possible actions to take. If the point is in a feedback area then data dragging will commence. 
//If the point is elsewhere in the display a selection area will be started.
//@parm the mouse flags
//@parm the point at which the button was pressed
void SRGraphView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CGraphDoc *pDoc=GetDocument();

	// If the user has dragged outside the client area the click sequence may be upset
	// so if any of our flags are set we should run the button up procedure first

	if(m_bSelecting || m_bDragging)
		OnLButtonUp(nFlags,point);

	
	SRGraphFeedback *pFB=GetGraph()->GetFeedback(point.x,point.y);
	if(pFB!=NULL)
	{
		m_bDragging=TRUE;	
		m_bInFeedback=TRUE;
		m_nDragGroup=pFB->m_nGroup;
		m_nDragIndex=pFB->m_nIndex;
	}
	else
	{
		m_SelectionPointA=m_SelectionPointB=point;
		m_bSelecting=TRUE;
		m_bInFeedback=FALSE;
	}
	CView::OnLButtonDown(nFlags, point);
}

//@mfunc If data was being dragged then the process is stopped. If a selection was being made then the relavent 
//data items are selected or zooming of a type in contect with the graph style is performed.
//@parm The mouse flags.
//@parm The position at which the mouse button was released.
void SRGraphView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CGraphDoc *pDoc=GetDocument();

	m_bDragging=FALSE;
	m_bInFeedback=FALSE;
	if(m_bSelecting)
	{

		SRGraphPosition *gp=GetPosition(&pDoc->m_Graph,point);
		// un-invert the selection
		DrawSelection(point);

		if(gp->GetPosType()==CX_POSITION_SINGLE)
		{


			// Process the selection
 			SRGraphComponent *pD=gp->GetComponent();
			int nTopLeftGroup=pD->GetMinGroup(),
				nTopLeftIndex=pD->GetMinIndex(),
				nBottomRightGroup=pD->GetMaxGroup(),
				nBottomRightIndex=pD->GetMaxIndex();
			
			if(!m_bZoomed)
			{
				m_nUnzoomMinGroup=pD->GetMinGroup();
				m_nUnzoomMaxGroup=pD->GetMaxGroup();
				m_nUnzoomMinIndex=pD->GetMinIndex();
				m_nUnzoomMaxIndex=pD->GetMaxIndex();
				m_bZoomed=TRUE;
				m_pZoomedDisplay=pD;
			}
			
			if(m_bZoomed && m_pZoomedDisplay!=pD)
			{
				OnGraphZoomout(); //Restore the previous zoomed display
			}


			gp->SetGraphPosition(m_SelectionPointA);
			gp->CalcPos(0);

			int nSelection=3;

			double dVX1,dVX2,dVY1,dVY2;

			if(gp->HasValueX() && gp->HasValueY())
			{
				dVX1=gp->GetValueX();
				dVY1=gp->GetValueY();
			}

			if(gp->HasIndex() && gp->HasGroup())
			{
				CSelectQuery dlg;
				dlg.m_Selection=0;

				if(dlg.DoModal()==IDOK)
				{
					nSelection=dlg.m_Selection+1; // Adding 1 turns it into some useful bitwize flags.
				}
				else
				{
					m_bSelecting=FALSE;
					return; // user cancelled
				}

			}

			if(gp->HasIndex() && (nSelection & 1))
				nTopLeftIndex=gp->GetIndex();
			
			if(gp->HasGroup() && (nSelection & 2))
				nTopLeftGroup=gp->GetGroup();

			gp->SetGraphPosition(m_SelectionPointB);
			gp->CalcPos(0);

			if(gp->HasValueX() && gp->HasValueY())
			{
				dVX2=gp->GetValueX();
				dVY2=gp->GetValueY();
				
				// we can set the zoom in this condition too!

				CScale zoomx,zoomy;
				zoomx+=dVX1;
				zoomx+=dVX2;
				zoomy+=dVY1;
				zoomy+=dVY2;
				((SRGraphDisplay *)pD)->SetZoomScaleX(zoomx); //We can cast to SRGraphDisplay safely because 
				((SRGraphDisplay *)pD)->SetZoomScaleY(zoomy); //SRGraphPosition would have complained by now if it wasnt one
			}

			if(gp->HasIndex() && (nSelection & 1))
				nBottomRightIndex=gp->GetIndex();

			if(gp->HasGroup() && (nSelection & 2))
				nBottomRightGroup=gp->GetGroup();
			
			pD->SetMinIndex(min(nTopLeftIndex,nBottomRightIndex));
			pD->SetMinGroup(min(nTopLeftGroup,nBottomRightGroup));
			pD->SetMaxIndex(max(nTopLeftIndex,nBottomRightIndex));
			pD->SetMaxGroup(max(nTopLeftGroup,nBottomRightGroup));

			pD->SetSignificantChange();

		}
		else if(gp->GetPosType()==CX_POSITION_LIST)		
			AfxMessageBox("Cannot select overlaid displays");
		
		delete(gp);

	}
	m_bSelecting=FALSE;
	Invalidate();
	CView::OnLButtonUp(nFlags, point);
}

//@mfunc If this timer has been allowed to fire the stored mouse position (i.e. that where the mouse has been left inactive for 1.5 seconds) 
//will be checked for a feedback polygon. If one is found a chart tip is displayed. If not the timer is simply stopped.
//@parm the timer which fired. (normally only the one ident will be found here but its checked anyway)
void SRGraphView::OnTimer(UINT nIDEvent) 
{
	CGraphDoc *pDoc=GetDocument();
	if(nIDEvent==MOUSE_TIMER_EVENT)
	{	// its def'nitly ours
		KillTimer(nIDEvent);
		SRGraphFeedback *pFeedback=GetGraph()->GetFeedback(m_MousePoint.x,m_MousePoint.y);
		if(pFeedback!=NULL)
			ShowFeedback(pFeedback);
	}
	else
		CView::OnTimer(nIDEvent);
}

//@mfunc Called from OnMouseMove while the left button is down. This routine will eeither updata a dragged item or update 
//the selection area.
//@parm the mouse flags (not used)
//@parm	The point to which the mouse has been dragged.
void SRGraphView::MouseDrag(int nFlags,CPoint point)
{
	nFlags; //UNUSED
	CGraphDoc *pDoc=GetDocument();

	if(m_bInFeedback)
	{
		SRGraphPosition *gp=GetPosition(&pDoc->m_Graph,point);
		if(gp->HasValueY() || (gp->HasValueX() && gp->HasValueY()))
		{

			SRGValueBlock vb;
			SRGraphComponent *pC=gp->GetComponent();

			switch(((SRGraphDisplay *)pC)->GetStyle()->GetGraphStyle())
			{
			case CX_GRAPH_XYSCATTERA:	// data is in alternate x,y format
 				GetGraph()->SetValue(m_nDragIndex,m_nDragGroup,gp->GetValueX());
				vb.index=m_nDragIndex+1;
				vb.group=m_nDragGroup+1;
				vb.value=gp->GetValueX();
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				vb.value=gp->GetValueY();
				if(m_nDragIndex+1==pC->GetDivCount()+pC->GetMinDiv())
				{
					vb.index=pC->GetMinDiv()+1;
					vb.group=m_nDragGroup+2;
				}
				else
				{
					vb.index=m_nDragIndex+2;
					vb.group=m_nDragGroup+1;
				}
 				GetGraph()->SetValue(vb.index-1,vb.group-1,gp->GetValueY());
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				Invalidate();
				break;
			case CX_GRAPH_XYSCATTERI:
 				GetGraph()->SetValue(m_nDragIndex,m_nDragGroup,gp->GetValueX());
				vb.index=m_nDragIndex+1;
				vb.group=m_nDragGroup+1;
				vb.value=gp->GetValueX();
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				vb.value=gp->GetValueY();
				vb.index=m_nDragIndex+2;
				vb.group=m_nDragGroup+1;
 				GetGraph()->SetValue(m_nDragIndex+1,m_nDragGroup,gp->GetValueY());
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				Invalidate();
				break;
			case CX_GRAPH_XYSCATTERG:
 				GetGraph()->SetValue(m_nDragIndex,m_nDragGroup,gp->GetValueX());
				vb.index=m_nDragIndex+1;
				vb.group=m_nDragGroup+1;
				vb.value=gp->GetValueX();
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				vb.value=gp->GetValueY();
				vb.index=m_nDragIndex+1;
				vb.group=m_nDragGroup+2;
 				GetGraph()->SetValue(m_nDragIndex,m_nDragGroup+1,gp->GetValueY());
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				Invalidate();
				break;
			default:
				GetGraph()->SetValue(m_nDragIndex,m_nDragGroup,gp->GetValueY());
				vb.index=m_nDragIndex+1;
				vb.group=m_nDragGroup+1;
				vb.value=gp->GetValueY();
				pDoc->UpdateParentDocument(OGRIDDOC_CMND_CELLVALUE,(DWORD)&vb);
				Invalidate();
				break;
			}
		}

		delete(gp);
	}
}


//@mfunc Copies the screen bitmap to the clipboard for pasting into your Wordprocessor files etc.
void SRGraphView::OnEditCopy() 
{
	if(m_GraphTip.IsShowing())
		m_GraphTip.EraseTip();
	CRect r;
	CDC memDC, *pDC=GetDC();
	GetClientRect(&r);
	memDC.CreateCompatibleDC(pDC);
	CBitmap CopyBitmap;
	CopyBitmap.CreateCompatibleBitmap(pDC,r.Width(),r.Height());
	CBitmap *obm=(CBitmap *)memDC.SelectObject(&CopyBitmap);
	memDC.BitBlt(0,0,r.Width(),r.Height(),pDC,0,0,SRCCOPY);
	memDC.SelectObject(obm);
	memDC.DeleteDC();
	ReleaseDC(pDC);


	if(OpenClipboard())
	{
	EmptyClipboard();
	HBITMAP hbm=(HBITMAP)CopyBitmap.Detach();
	SetClipboardData(CF_BITMAP,hbm);
	CloseClipboard();
	}
	else
		CopyBitmap.DeleteObject();
}

//@mfunc A graph which has been zoomed in will have had its original state stored. The zoomin in process may be performed 
//many times but only the first stored state will be restored at zoomout.
void SRGraphView::OnGraphZoomout() 
{
	if(m_bZoomed)
	{
		m_bZoomed=FALSE;
		m_pZoomedDisplay->SetMinIndex(m_nUnzoomMinIndex);
		m_pZoomedDisplay->SetMaxIndex(m_nUnzoomMaxIndex);
		m_pZoomedDisplay->SetMinGroup(m_nUnzoomMinGroup);
		m_pZoomedDisplay->SetMaxGroup(m_nUnzoomMaxGroup);
		((SRGraphDisplay *)m_pZoomedDisplay)->CleanZoomScaleX();
		((SRGraphDisplay *)m_pZoomedDisplay)->CleanZoomScaleY();
		m_pZoomedDisplay=NULL;
		Invalidate();
	}
}

//@mfunc Enables or disables the button and menu item which allows zooming out. If the graph has been zoomed the button and menu will be active 
//If the graph is in its original state then the button will be inactive
//@parm The button and/or menu command update interface
void SRGraphView::OnUpdateGraphZoomout(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bZoomed);	
}

//@mfunc invokes a simple dialog which allows pagination of the graph at print time.
void SRGraphView::OnGraphPagesetup() 
{
	PageSetup dlg;

	dlg.m_nValign=m_nValign;
	dlg.m_nHalign=m_nHalign;
	dlg.m_nPageSizeCode=m_nPageSizeCode;
	dlg.m_bUseMaxArea=m_bUseMaxArea;
	if(dlg.DoModal()==IDOK)
	{
		m_nValign=dlg.m_nValign;
		m_nHalign=dlg.m_nHalign;
		m_nPageSizeCode=dlg.m_nPageSizeCode;
		m_bUseMaxArea=dlg.m_bUseMaxArea;
	}
	
}

//@mfunc When dynamically linked this routine invokes the editor in standalone mode via the EditGraph routine. When 
//statically linked the routine constructs and uses the standard SRGGraphEditDialog in modal form. In both cases the only
//data required is a pointer to the graph to be edited. The edit dialog handles all the details.
void SRGraphView::OnGraphEdit() 
{
	CGraphDoc *pDoc=GetDocument();

#ifdef LATER

	EditGraph(&pDoc->m_Graph);	// one call does it all to the DLL
	
#endif

	SRGraphEditDialog dlg;		// or access the dialog directly when statically linked
	dlg.m_pGraph=&pDoc->m_Graph;
	dlg.DoModal();

	GetGraph()->SetSignificantChanges();
	Invalidate();
}



//@mfunc The default feedback display is more of a demonstration default. To perform some useful 
//feedback function you should override this function in your own class derivation.
//@parm A pointer to a SRGraphFeedback object
void SRGraphView::ShowFeedback(SRGraphFeedback *pFeedback)
{
	CString strText;
	strText.Format("Index=%d Group=%d Value=%g",
		pFeedback->m_nIndex,
		pFeedback->m_nGroup,
		GetDocument()->m_Graph.GetGroup(pFeedback->m_nGroup)->GetIndex(pFeedback->m_nIndex)->GetValue());

	CPoint tPoint=m_MousePoint;
	
	ClientToScreen(&tPoint);
	
	m_GraphTip.ShowTip(strText,tPoint,this);
}
				   

/////////////////////////////////////////////////////////////////////////////
// CSelectQuery dialog


CSelectQuery::CSelectQuery(CWnd* pParent /*=NULL*/)
	: CDialog(CSelectQuery::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSelectQuery)
	m_Selection = -1;
	//}}AFX_DATA_INIT
}


void CSelectQuery::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSelectQuery)
	DDX_Radio(pDX, IDC_GVW_RADIO1, m_Selection);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSelectQuery, CDialog)
	//{{AFX_MSG_MAP(CSelectQuery)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSelectQuery message handlers



/////////////////////////////////////////////////////////////////////////////
// PageSetup dialog


PageSetup::PageSetup(CWnd* pParent /*=NULL*/)
	: CDialog(PageSetup::IDD, pParent)
{
	m_nPageSizeCode=0;
	//{{AFX_DATA_INIT(PageSetup)
	m_bUseMaxArea = FALSE;
	m_nValign = 1;
	m_nHalign = 1;
	//}}AFX_DATA_INIT
}


void PageSetup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(PageSetup)
	DDX_Control(pDX, IDC_GVW_COMBO1, m_Size);
	DDX_Check(pDX, IDC_GVW_CHECK1, m_bUseMaxArea);
	DDX_Radio(pDX, IDC_GVW_RADIO1, m_nValign);
	DDX_Radio(pDX, IDC_GVW_RADIO5, m_nHalign);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(PageSetup, CDialog)
	//{{AFX_MSG_MAP(PageSetup)
	ON_BN_CLICKED(IDC_GVW_CHECK1, OnGvwCheck1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PageSetup message handlers

BOOL PageSetup::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_Size.SetCurSel(m_nPageSizeCode);

	OnGvwCheck1();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void PageSetup::OnOK() 
{
	m_nPageSizeCode=m_Size.GetCurSel();
	
	CDialog::OnOK();
}


//@mfunc NEW. Returns a pointer to the graph object which this view displays and interfaces to.
//@rdesc A pointer to the graph object.
//@rem You may override this virtual function to return your own graph object.
SRGraph * SRGraphView::GetGraph()
{
	// The standard handling is to return a pointer to the m_Graph object stored in the CGraphDoc object
	// supplied with this kit.

	CGraphDoc *pGraphDoc=(CGraphDoc *)GetDocument();
	ASSERT(pGraphDoc->IsKindOf(RUNTIME_CLASS(CGraphDoc)));
	return &pGraphDoc->m_Graph;
}

void PageSetup::OnGvwCheck1() 
{
	BOOL bState=(((CButton *)GetDlgItem(IDC_GVW_CHECK1))->GetState()&1);
	GetDlgItem(IDC_GVW_RADIO1)->EnableWindow(bState);
	GetDlgItem(IDC_GVW_RADIO2)->EnableWindow(bState);
	GetDlgItem(IDC_GVW_RADIO3)->EnableWindow(bState);
	GetDlgItem(IDC_GVW_RADIO5)->EnableWindow(bState);
	GetDlgItem(IDC_GVW_RADIO6)->EnableWindow(bState);
	GetDlgItem(IDC_GVW_RADIO7)->EnableWindow(bState);	
	GetDlgItem(IDC_GVW_COMBO1)->EnableWindow(bState);	
}

//@mfunc This virtual function provides the ability to return your own 
//position class derived from SRGraphPosition.
//@parm The graph object being interrogated
//@parm The point position of the interrogatioin coordinates
//@rdesc A pointer to a newly created SRGraphPoition object
SRGraphPosition * SRGraphView::GetPosition(SRGraph * pGraph, CPoint point)
{
	//Override this to return your derived position class
	return(new SRGraphPosition(pGraph,point,this));
}

void SRGraphView::DrawCustomSelection(CPoint point)
{
	point; // Unused
	//You have declared an unknown selection style in your
	//SRGraphPosition derived class. Override this function
	//To draw the selection area for your custom zoom style.
	ASSERT(FALSE);
}
