// SRGraphAll header file..

#ifndef __OCHART_H__
#define __OCHART_H__


/////////////////////////////////////////////////////////////////////////////
// Master version number
/////////////////////////////////////////////////////////////////////////////

#define _OC_VER 0x0101 // Objective Chart 1.01

// Changes have been made to serialization since version 0 (beta)
#define _OCHART_FILE_VERSION 1

//Graph classes
class SRGraph;
	class SRDynamicGraph;

//Component classes
class SRGraphComponent;
	class SRGraphBackground;
	class SRGraphTitle;
		class SRGraphLegend;
		class SRGraphDisplay;
		class SRGraphLabel;
			class SRGraphLabelBlock;
				class SRGDecimalScale;
					class SRGLogScale;
					class SRGDateScale;
					class SRGGroupScale;
					class SRGIndexScale;

//Data storage classes
class SRGraphData;
	class SRGraphDynamicData;

//Multi language annotation class
class SRGraphAnnotation;

//List classes
class SRGraphDataList;
class SRGraphAnnotationList;

//Exception class
class SRGraphException;

//helper classes
class SRGraphRect; 
class CScale;
class SRGraphFeedback;
class SRGCoordinate;
class SRGCompiledPolygon;
class SRGraphPosition;

/////////////////////////////////////////////
// Extension DLL link management

#ifdef _OBJCHART_DLL

	#ifdef _OCNOAUTOLIB
		#define OC_DATA _declspec(dllexport)
	#else
		#define OC_DATA _declspec(dllimport)
	#endif

#endif


/////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//				Objective Chart
// Automatic library inclusion macros that use the #pragma/lib feature
/////////////////////////////////////////////////////////////////////////////

// definitions useful to the graphics process
#define PI	3.1415926

#define TWO_PI	(PI*2)

// some useful macro's

#define DEGREES(radians) \
	((double)radians*(double)180/(double)PI)

#define RADIANS(degrees) \
	((double)degrees/(double)180*(double)PI)

// To get rid of misspelled "indices"

#define GetShowIndeces GetShowIndices
#define SetShowIndeces SetShowIndices

//Definitions for the communications systems used in SRGraph

#define SRG_NOOP					0x00000000	// No Operation
#define SRG_SET_TEXT				0x00000001	// used to update headers and annotations
#define SRG_SET_VALUE				0x00000002	// updates data values
#define SRG_SET_TITLE				0x00000003	// updates graph titles. Use with care because it updates all titles in all connected graphs
#define SRG_SET_HEADER				0x00000004	// changes header text
#define SRG_DYNAMIC_SETVALUE		0x00000005	// pokes dynamic graphs
#define SRG_SET_HIGHLIGHT			0x00000006	// Turns on highlighting as a certain group[index]
#define SRG_RESET_HIGHLIGHT			0x00000007	// Turns on highlighting as a certain group[index]
#define SRG_SET_DISPLAY_MINGROUP	0x00000008	// Sets the minimum group in a display
#define SRG_SET_DISPLAY_MAXGROUP	0x00000009	// Sets the maximum group in a display
#define SRG_SET_DISPLAY_MININDEX	0x0000000a	// Sets the minimum index in a display
#define SRG_SET_DISPLAY_MAXINDEX	0x0000000b	// Sets the maximum index in a display
#define SRG_SET_LEGEND_MINGROUP		0x0000000c	// As above but for legends
#define SRG_SET_LEGEND_MAXGROUP		0x0000000d	// Probably redundant now because legends link to displays automatically
#define SRG_SET_LEGEND_MININDEX		0x0000000e	// Note that these definitions may be removed in a later version.
#define SRG_SET_LEGEND_MAXINDEX		0x0000000f

// typedefs for helper structures

typedef struct SRG_TEXT_BLOCK {
	UINT group;
	UINT index;
	UINT textsize;
	char text;
} SRGTextBlock;

typedef struct SRG_VALUE_BLOCK {
	UINT group;
	UINT index;
	double value;
	BOOL bNullFlag;
} SRGValueBlock;


/////////////////////////////////////////////////////////////////
//	Definitions for the SRGraph alignment codes
//

#define CX_ALIGN_NONE		0x00000000
#define CX_ALIGN_VTOP		0x00000001
#define CX_ALIGN_VCENTER	0x00000002
#define CX_ALIGN_VBOTTOM	0x00000003
#define CX_ALIGN_HLEFT		0x00000010
#define CX_ALIGN_HCENTER	0x00000020
#define CX_ALIGN_HRIGHT		0x00000030

////////////////////////////////////////////////////////////
//	Definitions for the SRGraphStyle object
//

// style masks for m_dwStyle

#define CX_FRAME_STYLE_MASK		0x0000000f
#define CX_INTERIOR_STYLE_MASK	0x000000f0
#define CX_OBJECT_STYLE_MASK	0x0000ff00
#define CX_GRAPH_STYLE_MASK		0x00ff0000
#define CX_AXIS_STYLE_MASK		0xff000000


// shift values
#define CX_AXIS_SHIFT			24
#define CX_GRAPH_SHIFT			16
#define CX_LEGEND_SHIFT			0
#define CX_INTERIOR_SHIFT		4
#define CX_OBJECT_SHIFT			8
#define CX_FRAME_SHIFT			0

// frame styles
#define CX_FRAME_NONE			0x00000000
#define CX_FRAME_AUTOMATIC		0x00000001
#define CX_FRAME_LINE			0x00000002
#define CX_FRAME_PATTERN		0x00000003
#define CX_FRAME_COLOR			0x00000004
#define CX_FRAME_USER			0x0000000c
								
//interior styles
#define CX_INTERIOR_NONE		0x00000000
#define CX_INTERIOR_AUTOMATIC	0x00000010
#define CX_INTERIOR_COLOR		0x00000020
#define CX_INTERIOR_VSHADE		0x00000030
#define CX_INTERIOR_HSHADE		0x00000040
#define CX_INTERIOR_ASHADE		0x00000050
#define CX_INTERIOR_RESOURCE	0x00000060
#define CX_INTERIOR_IMAGE		0x00000070
#define CX_INTERIOR_PATTERN		0x00000080
#define CX_INTERIOR_USER		0x000000c0

//Object styles
#define CX_OBJECT_NONE			0x00000000
#define CX_OBJECT_AUTOMATIC		0x00000100
#define CX_OBJECT_POINT			0x00000200
#define CX_OBJECT_EXPOINT		0x00000300
#define CX_OBJECT_LINE			0x00000400
#define CX_OBJECT_BAR			0x00000500
#define CX_OBJECT_EXBAR			0x00000600
#define CX_OBJECT_ROD			0x00000700
#define CX_OBJECT_AREA			0x00000800
#define CX_OBJECT_SLICE			0x00000900
#define CX_OBJECT_POLYGON		0x00000a00
#define CX_OBJECT_EXLINE		0x00000b00
#define CX_OBJECT_USER			0x0000c000  //User objects must lie in the range 0000c000 to 0000ff00 All other values are reserved

//Graph styles
#define CX_GRAPH_NONE			0x00000000	//No style defined
#define CX_GRAPH_LINE			0x00010000	//Simple line plot
#define CX_GRAPH_HBAR			0x00020000	//simple horizontal bar chart
#define CX_GRAPH_VBAR			0x00030000	//simple vertical bar chart
#define CX_GRAPH_PIE			0x00040000	//pie chart
#define CX_GRAPH_HISTOGRAM		0x00050000	//Histogram or frequency spectrum
#define CX_GRAPH_AREA			0x00060000	//area under a curve
#define CX_GRAPH_STOCK			0x00070000	//Highs Lows and now (or close) dynamic
#define CX_GRAPH_OSCILLISCOPE	0x00080000	//dynamic multiple plots
#define CX_GRAPH_ISO_BAR		0x00090000	//3D "manhattan" graph
#define CX_GRAPH_ISO_AREA		0x000a0000	//3D "Rooftop" graph
#define CX_GRAPH_STACK_VBAR		0x000b0000	//2D stacked vertical bars
#define CX_GRAPH_STACK_HBAR		0x000c0000	//2d stacked horizontal bars
#define CX_GRAPH_ISO_PIE		0x000d0000	//3D isographic pie chart
#define CX_GRAPH_STRATA			0x000e0000	//Strata graph
#define CX_GRAPH_EX_VBAR		0x000f0000	//Extended VBar (Pseudo 3D)
#define CX_GRAPH_EX_HBAR		0x00100000	//Extended HBar (pseudo 3D horizontal bars)
#define CX_GRAPH_RIBBON			0x00110000	//Ribbon graphs are rooftop graphs without sides or ends
#define CX_GRAPH_WEB			0x00120000  //Web charts are polar or radar graphs
#define CX_GRAPH_XYSCATTERA		0x00130000	//XY scatter plot using alternate x and y values
#define CX_GRAPH_XYSCATTERI		0x00140000  //XY Scatter plot using the index as the X's and the index+1 as the Y's
#define CX_GRAPH_XYSCATTERG		0x00150000  //XY Scatter plot using the groups as the x's and group+1 as the y's
#define CX_GRAPH_TIMESPAN		0x00160000	//Horizontal time-span bars
#define CX_GRAPH_AREAWEB		0x00170000	//Polar shart with area display
#define CX_GRAPH_AREASTOCK		0x00180000	//Area style stock quote (hi-lo)
#define CX_GRAPH_FREESTYLE		0x00190000	//Classic graph with free style objects
#define CX_GRAPH_POLAR			0x001a0000	//NEW Polar chart with phase and magnitude.
#define CX_GRAPH_AREAPOLAR		0x001b0000	//NEW Polar chart with filled areas
#define CX_GRAPH_CANDLE			0x001c0000	//NEW Candle chart style
#define CX_GRAPH_HILO_OPENCLOSE	0x001d0000	//NEW Hi Lo Open Close chart
#define CX_GRAPH_GANTT			0x001e0000	//NEW Gantt chart
#define CX_GRAPH_USER			0x00C00000	//User styles are in the range 00c00000 to 00ff0000 all other values are reserved

//axis styles
#define CX_AXIS_NONE			0x00000000	//No style defined
#define CX_AXIS_AUTOMATIC		0x01000000  //Style chosen from graph
#define CX_AXIS_CLASSIC			0x02000000	//Automatic XY axes with automatic negative Y
#define CX_AXIS_OSCILLISCOPE	0x03000000	//+-X +-Y for 2D plots.
#define CX_AXIS_PLATFORM_SE		0x04000000	//Isographic ribbon viewed from south east
#define CX_AXIS_PLATFORM_SW		0x05000000	//Viewed from south west
#define CX_AXIS_STAGE_SE		0x06000000	// 3/4 stage views from south east
#define CX_AXIS_STAGE_SW		0x07000000	// 3/4 stage from south west
#define CX_AXIS_H_FRAME			0x08000000	// "Rugby" (American football?) goal posts
#define CX_AXIS_PIE				0x09000000	// Pie chart labelling
#define CX_AXIS_YX				0x0a000000	// Classic axis but with x extending upward
#define CX_AXIS_ISO_CLASSIC		0x0b000000	// Classic axis but with pseudo 3D effect
#define CX_AXIS_CLASSIC_RIGHT	0x0c000000	// Classic axis with Y on the right
#define CX_AXIS_ISO_CLASSIC_R	0x0d000000	// classic axis, pseudo 3D, y axis on right
#define CX_AXIS_WEB				0x0e000000	// Web chart axis
#define CX_AXIS_XYSCATTER		0x0f000000	// Scatter graph axis.
#define CX_AXIS_ISO_CLASSIC_YX	0x10000000	// Iso classic for horizontal graphs
#define CX_AXIS_POLAR			0x11000000	// Polar chart axes.
#define CX_AXIS_DATE			0x12000000	// Classic axis with date scale  
#define CX_AXIS_DATE_R			0x13000000	// Classic axis with date scale on the right
#define CX_AXIS_ISO_DATE		0x14000000	// Iso classic axis with date scale
#define CX_AXIS_ISO_DATE_R		0x15000000	// Iso classic axis with date scale on the right
#define CX_AXIS_ISO_DATE_YX		0x16000000	// Iso classic axis with date scale on a YX axis
#define CX_AXIS_DATE_YX			0x17000000	// Classic axis with date scale  
#define CX_AXIS_GANTT_VALUE		0x18000000	// Gantt chart axis, value scale
#define CX_AXIS_GANTT_DATE		0x19000000	// Gantt chart axis, date scale
#define CX_AXIS_USER			0xc0000000  // User styles are in the range c0000000 to ff000000 All other values are reserved

// Used in m_dwStyleB
#define CX_LEGEND_STYLE_MASK	0x000000ff
//Legending styles
#define CX_LEGEND_NONE			0x00000000	//No style defined
#define CX_LEGEND_AUTOMATIC		0x00000001	//Automatic
#define CX_LEGEND_BY_INDEX		0x00000002	//Uses index headers to annotate legend
#define CX_LEGEND_BY_GROUP		0x00000003	//Uses Group headers to annotate legend
#define CX_LEGEND_BY_RECTANGLE	0x00000004	//uses individual annotations
#define CX_LEGEND_HEAD_GROUP	0X00000005	//titles legend by group and uses indices to annotate
#define CX_LEGEND_HEAD_INDEX	0x00000006	//titles legend by index and uses groups to annotate
#define CX_LEGEND_USER			0x000000c0  //User legend styles. 0x000000c0 to 0x000000ff All other values are reserved.

// component and object styles styles

#define	CX_NO_BORDER			0x00000000
#define	CX_THIN_RAISED_BORDER	0x00000001
#define	CX_THIN_SUNKEN_BORDER	0x00000002
#define	CX_THIN_3D_BORDER		0x00000003
#define	CX_THICK_RAISED_BORDER	0x00000004
#define	CX_THICK_SUNKEN_BORDER	0x00000005
#define	CX_THICK_3D_BORDER		0x00000006
#define CX_LINE_BORDER			0x00000007
#define CX_USER_BORDER			0x0000000c	
								
#define CX_NO_SHADOW			0x00000000
#define	CX_THIN_SHADOW			0x00000010
#define	CX_MEDIUM_SHADOW		0x00000020
#define	CX_THICK_SHADOW			0x00000030
#define CX_USER_SHADOW			0x000000c0

#define	CX_NO_FILL				0x00000000
#define	CX_SOLID_FILL			0x00000100
#define	CX_VERTICAL_FILL		0x00000200
#define	CX_HORIZONTAL_FILL		0x00000300
#define CX_ANGLE_FILL			0x00000400
#define	CX_RESOURCE_FILL		0x00000500
#define	CX_IMAGE_FILL			0x00000600
#define CX_USER_FILL			0x00000c00

#define CX_BORDER_MASK			0x0000000f
#define CX_BORDER_SHIFT			0
#define CX_SHADOW_MASK			0x000000f0
#define CX_SHADOW_SHIFT			4
#define CX_FILL_MASK			0x00000f00
#define CX_FILL_SHIFT			8
#define CX_FOREGROUND_MASK		0x0000f000
#define CX_FOREGROUND_SHIFT		12
#define CX_STRETCH_MASK			0x80000000

// Defines used for semaphores in the m_dwStyleA member

#define CX_SIZE_MASK			0x40000000
#define CX_FILL_AXIS_MASK		0x20000000
#define CX_SHADE_OBJECT_MASK	0x10000000
#define CX_LOG_SCALE_MASK		0x08000000
#define CX_MAX_SCALE_MASK		0x04000000
#define CX_EMPTY_GRAPH_MASK		0x02000000
#define CX_HIDE_NUMERALS_MASK	0x01000000
#define CX_HIDE_INDICES_MASK	0x00800000
#define CX_HIDE_GROUPS_MASK		0x00400000
#define CX_LEGEND_LINE_MASK		0x00020000
#define CX_HIDE_XGRID_MASK		0x00010000
#define CX_HIDE_YGRID_MASK		0x00008000
#define CX_HIDE_ZGRID_MASK		0x00004000
#define CX_HIDE_ZERO_MASK		0x00002000
#define CX_HIDE_XTICKS_MASK		0x00001000
#define CX_HIDE_YTICKS_MASK		0x00000800
#define CX_HIDE_ZTICKS_MASK		0x00000400
#define CX_SORT_VALUES_MASK		0x00000200
#define CX_OVERLAY_GRID_MASK	0x00000100
#define CX_DISPLAYLOCK_MASK		0x00000080
#define CX_MINORTICKS_MASK		0x00000040
#define CX_TIER_XLABELS_MASK	0x00000020
#define CX_TIER_YLABELS_MASK	0x00000010



// defines used in the m_dwStyleC member

#define CX_TYPE_MASK			0x0000000f
#define CX_TYPE_SHIFT			0
								
#define CX_TYPE_NONE			0x00000000
#define CX_TYPE_COMPONENT		0x00000001
#define CX_TYPE_OBJECT			0x00000002
#define CX_TYPE_GRAPH			0x00000003
#define CX_TYPE_DISPLAY			0x00000004
#define CX_TYPE_LEGEND			0x00000005
#define CX_TYPE_TITLE			0x00000006
#define CX_TYPE_LABEL			0x00000007
#define CX_TYPE_LABELBLOCK		0x00000008
#define CD_TYPE_DSCALE			0x00000009
#define CX_TYPE_LOGSCALE		0x0000000a
#define CX_TYPE_DATESCALE		0x0000000b



#define CX_LABEL_MASK			0x000000f0
#define CX_LABEL_SHIFT			4

#define CX_LABEL_NONE				0x00000000
#define CX_LABEL_AUTOMATIC			0x00000010
#define CX_LABEL_VALUE				0x00000020
#define CX_LABEL_PERCENT			0x00000030
#define CX_LABEL_VALUE_PERCENT		0x00000040
#define CX_LABEL_TEXT				0x00000050
#define CX_LABEL_TEXT_VALUE			0x00000060
#define CX_LABEL_TEXT_VALUE_PERCENT	0x00000070
#define CX_LABEL_TEXT_PERCENT		0x00000080
#define CX_LABELBLOCK_HIDELABELS	0x00000090
#define CX_LABELBLOCK_MOVELABELS	0x000000a0

#define CX_HIGHLIGHT_MASK		0x00000f00
#define CX_HIGHLIGHT_SHIFT		8

#define CX_HIGHLIGHT_NONE		0x00000000
#define CX_HIGHLIGHT_AUTOMATIC	0x00000100
#define CX_HIGHLIGHT_SMALL		0x00000200
#define CX_HIGHLIGHT_MEDIUM		0x00000300
#define CX_HIGHLIGHT_LARGE		0x00000400
#define CX_HIGHLIGHT_CUSTOM		0x00000500

#define CX_KEY_STYLE_MASK		0x0000f000
#define CX_KEY_SHIFT			12

#define CX_KEY_NONE				0x00000000
#define CX_KEY_AUTOMATIC		0x00001000
#define CX_KEY_LINE				0x00002000
#define CX_KEY_BOX				0x00003000
#define CX_KEY_WIGET			0x00004000
#define CX_KEY_CUSTOM			0x0000c000


/////////////////////////////////////////////////////
//	Definitions for the SRGraphPosition class
//


#define CX_POSITION_MASK		0x0000000f

#define CX_POSITION_NONE		0x00000000	
#define CX_POSITION_SINGLE		0x00000001	// position is for a single component
#define CX_POSITION_LIST		0x00000002	// for a list of components

#define CX_POSITION_VALUEY		0x00000010	// The Y value is known
#define CX_POSITION_VALUEX		0x00000020	// The X value is known
#define CX_POSITION_GROUP		0x00000040	// The group is known
#define CX_POSITION_INDEX		0x00000080	// The index is known

#define CX_POSITION_SELECT_MASK	0x000f0000
#define CX_POSITION_SELECT_H	0x00000000	// Selection is horizontal (classic)
#define CX_POSITION_SELECT_V	0x00010000	// selection is vertical (yx)
#define CX_POSITION_SELECT_B	0x00020000	// selection is both (x,y scatter)
#define CX_POSITION_SELECT_C	0x00030000	// Selection is circular (webs)


/////////////////////////////////////////////////////
// various colors to use in filling and framing

#define CXCLR_WHITE		RGB(255,255,255)
#define CXCLR_BLACK		RGB(0,0,0)
#define CXCLR_RED		RGB(255,0,0)
#define CXCLR_LTRED		RGB(255,92,92)
#define CXCLR_CORAL		RGB(255,0,128)
#define CXCLR_MAROON	RGB(128,0,0)
#define CXCLR_OLIVE		RGB(128,128,0)
#define CXCLR_TEAL		RGB(0,128,128)
#define CXCLR_GREEN		RGB(0,128,0)
#define CXCLR_SEAGREEN	RGB(0,255,128)
#define CXCLR_LIME		RGB(0,255,0)
#define CXCLR_LTGREEN	RGB(128,255,0)
#define CXCLR_BLUE		RGB(0,0,255)
#define CXCLR_MIDNIGHT	RGB(0,0,128)
#define CXCLR_SKY		RGB(0,128,255)
#define CXCLR_PURPLE	RGB(128,0,128)
#define CXCLR_VIOLET	RGB(128,0,255)
#define CXCLR_BEIGE		RGB(255,128,0)
#define CXCLR_CYAN		RGB(0,255,255)
#define CXCLR_MAGENTA	RGB(255,0,255)
#define CXCLR_YELLOW	RGB(255,255,0)
#define CXCLR_GRAY10	RGB(230,230,230)
#define	CXCLR_GRAY20	RGB(205,205,205)
#define CXCLR_GRAY30	RGB(179,179,179)
#define CXCLR_GRAY40	RGB(152,152,152)
#define CXCLR_GRAY50	RGB(128,128,128)
#define CXCLR_GRAY60	RGB(102,102,102)
#define CXCLR_GRAY70	RGB(76,76,76)
#define CXCLR_GRAY80	RGB(51,51,51)
#define CXCLR_GRAY90	RGB(25,25,25)


///////////////////////////////////////////////////////////
//
// Standard inclusions
//

#include  "comdoc.h"
#include  "SRGRes.h"
#include  "SRGExc.h"
#include  "SRGDat.h"
#include  "SRGFbck.h"
#include  "SRGCPly.h"
#include  "SRGComp.h"
#include  "SRGraph.h"
#include  "DScale.h"
#include  "srgposn.h"
#include  "docmngr.h"
#include  "graphdoc.h"
#include  "grphtip.h"
#include  "graphvw.h"


#endif // __OCHART_H__
