//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc
//@module SRGCPly.cpp | class implementation  of the SRGCompiledPolygon class<nl>
//Copyright <cp> Stingray Software 1996.<nl>
//Written by Bob Powell (bob@stingsoft.com)

#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"

#ifdef _DEBUG

#define new DEBUG_NEW

#endif

//@mfunc Constructor. Initialises the <md SRGCompiledPolygon::m_bDirty> <md SRGCompiledPolygon::m_pPoints> and <md SRGCompiledPolygon::m_nCount> members.
SRGCompiledPolygon::SRGCompiledPolygon()
{
	m_bDirty=TRUE;
	m_pPoints=NULL;
	m_nCount=0;
}

//@mfunc Destructor. Performs cleanup operations by deleting both point lists.
SRGCompiledPolygon::~SRGCompiledPolygon()
{
	while(!m_Coords.IsEmpty())
		delete (SRGCoordinate *)m_Coords.RemoveHead();

	if(m_pPoints!=NULL)
		delete (m_pPoints);
}

//@mfunc Retrieves a pointer to the integer point list. If the point list is dirty then 
//a new integer list is compiled from the floating point master.
//@rdesc A pointer containing the contents of the <md SRGCompiledPolygon::m_pPoint> member
LPPOINT SRGCompiledPolygon::GetPoints()
{
	if(m_bDirty)
	{// The polygon must be recompiled
		
		AssignPointBlock();

		int i=0;
		
		for(POSITION p=m_Coords.GetHeadPosition();p!=NULL;i++)
		{
			SRGCoordinate *pC=(SRGCoordinate *)m_Coords.GetNext(p);
			m_pPoints[i]=CPoint((int)(pC->GetX()+0.5),(int)(pC->GetY()+0.5));
		}
	}

	m_bDirty=FALSE;
	return m_pPoints;
}

//@mfunc Adds a floating point coordinate to the list. Data is supplied as a ready made SRGCoordinate objrect
//@parm. The coordinate object to add.
void SRGCompiledPolygon::AddCoord(SRGCoordinate *c)
{	
	m_bDirty=TRUE;
	m_Coords.AddTail(c);
}

//@mfunc Adds a floating point coordinate to the list. Data is supplied using floating point values for x and y
//@parm The X coordinate.
//@parm The Y coordinate
void SRGCompiledPolygon::AddCoord(float xc,float yc)
{
	SRGCoordinate *c=new SRGCoordinate;
	c->SetX(xc);
	c->SetY(yc);
	AddCoord(c);
}

//@mfunc Adds a floatig point coordinate to the list. Data is supplied as integers
//@parm The X coordinate.
//@parm The Y coordinate
void SRGCompiledPolygon::AddCoord(int nx,int ny)
{
	AddCoord((float)nx,(float)ny);
}


//@mfunc Initialises the name of the polygon.
//@parm A pointer to the name string. May also be supplied as a reference to a CString object
void SRGCompiledPolygon::SetName(LPCSTR name)
{
	m_strName=name;
}

//@mfunc Interrogates the name of this polygon
//@rdesc A reference to a CString object containing the name.
CString & SRGCompiledPolygon::GetName()
{
	return m_strName;
}

//@mfunc Compiles a polygon list with X and Y offsets. Used primarily for placing wigets which are polygone centered about the 0,0 coodinate.
//@parm The x offset
//@parm the Y offet
void SRGCompiledPolygon::OffsetPolygon(float xo,float yo)
{

	AssignPointBlock();

	int i=0;

	for(POSITION p=m_Coords.GetHeadPosition();p!=NULL;i++)
	{
		SRGCoordinate *pC=(SRGCoordinate *)m_Coords.GetNext(p);
		m_pPoints[i]=CPoint((int)(xo+pC->GetX()+0.5),(int)(yo+pC->GetY()+0.5));
	}

	m_bDirty=FALSE;
}

//@mfunc Assigns re-assigns or reclaims memory used for the integer point list. To increase efficiency the
// assignment routine grabs blocks of 128 CPoint objects at a time. Therefore no integer  polygon will be less than 128
// points in size. It should be noted that polygons having less than 128 floating point coordinates will simply not use
// some of the integer points. This method has been used to decrease re-allocation problems and reduce memory fragmentation.
void SRGCompiledPolygon::AssignPointBlock()
{
	if(m_Coords.GetCount()>m_nCount)
	{	// memory is assigned in blocks of 128 CPoints to reduce fragmentation
		int nNewCount=128;
		while(nNewCount<m_Coords.GetCount())
			nNewCount+=128;
		CPoint *pNewPoints=new CPoint[nNewCount];
		if(m_pPoints!=NULL)
			delete (m_pPoints);
		m_pPoints=pNewPoints;
		m_nCount=nNewCount;
	}
}

//@mfunc Scales a floating point polygon to fit a specific rectangle.
//@parm The rectangle into which the polygon must fit.
//@rdesc a pointer to the compiled integer polygon.
LPPOINT SRGCompiledPolygon::GetFittedPoints(CRect &r)
{
	AssignPointBlock();

	CScale Width,Height;
		
	// First ascertain the ranges of values in this polygon
	for(POSITION p=m_Coords.GetHeadPosition();p!=NULL;)
	{
		SRGCoordinate *pC=(SRGCoordinate *)m_Coords.GetNext(p);
		Width+=pC->GetX();
		Height+=pC->GetY();
	}

	// The ranges of Width and Height are now known so the final CPoint based polygon can be created

	int i=0;
	for(p=m_Coords.GetHeadPosition();p!=NULL;i++)
	{
		SRGCoordinate *pC=(SRGCoordinate *)m_Coords.GetNext(p);
		m_pPoints[i]=CPoint(r.left+(int)((((pC->GetX()-Width.Min())/Width.Range())*r.Width())+0.5),
			                r.bottom-(int)((((pC->GetY()-Height.Min())/Height.Range())*r.Height())+0.5));
	}

	// The CPoint polygon now contains a correctly positioned and scaled polygon
	// which will fit neatly into the desired rectangle.
	m_bDirty=FALSE;
	return m_pPoints;
}
