//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc
//@module SRGCPly.h | class definition  of the SRGCompiledPolygon class<nl>
//Copyright <cp> Stingray Software 1996.<nl>
//Written by Bob Powell (bob@stingsoft.com)

#ifdef _OBJCHART_DLL
	#undef AFXAPP_DATA
	#define AFXAPP_DATA AFXAPI_DATA
	#undef AFX_DATA
	#define AFX_DATA OC_DATA
#endif


#ifndef __SRGCOMPILEDPOLYGON__
#define __SRGCOMPILEDPOLYGON__

/////////////////////////////////////////////
// Autoduck comment block for SRGCoordinate

//@mdata float | SRGCoordinate | x | The floating point X coordinate value.

//@mdata float | SRGCoordinate | y | The floating point y coordinate value.

//@mfunc 	| SRGCoordinate |SRGCoordinate | Initializing constructor. Sets x and y to 0

//@mfunc 	| SRGCoordinate |~SRGCoordinate | destructor;

//@mfunc 	void | SRGCoordinate | SetX | Sets the floating point x coordinate

//@mfunc 	void | SRGCoordinate | SetX | Sets the floating point x coordinate from a double

//@mfunc 	void | SRGCoordinate | SetX | Sets the floating point x coordinate from an integer

//@mfunc 	float | SRGCoordinate | GetX | Gets the floating point x coordinate

//@mfunc 	void | SRGCoordinate | SetY | Sets the floating point y coordinate

//@mfunc 	void | SRGCoordinate | SetY | Sets the floating point y coordinate	from a double

//@mfunc 	void | SRGCoordinate | SetY | Sets the floating point y coordinate	from an integer

//@mfunc 	float | SRGCoordinate | GetY | Gets the floating point y coordinate

//@class SRGCoordinate | Manages a single floating point coordinate. Used for re-scaleable polygons
class SRGCoordinate
{
//@access public data members
public:

	//@cmember The X coordinate
	float x;
	//@cmember the Y coordinate
	float y;

//@access public member functions
public:
	//@cmember Initializing constructor. Sets x and y to 0
	SRGCoordinate(){x=y=0.0f;}
	//@cmember destructor
	virtual ~SRGCoordinate(){};
	//@cmember Sets the floating point x coordinate
	virtual void SetX(float f){x=f;}
	//@cmember Sets the floating point x coordinate
	virtual void SetX(double f){x=(float)f;}
	//@cmember Sets the floating point x coordinate
	virtual void SetX(int f){x=(float)f;}
	//@cmember Gets the floating point x coordinate
	virtual float GetX(){return x;}
	//@cmember Sets the floating point y coordinate
	virtual void SetY(float f){y=f;}
	//@cmember Sets the floating point y coordinate
	virtual void SetY(double f){y=(float)f;}
	//@cmember Sets the floating point y coordinate
	virtual void SetY(int f){y=(float)f;}

	//@cmember Gets the floating point y coordinate
	virtual float GetY(){return y;}
};

//////////////////////////////////////////////////////
// SRGCompiledPolygon Autoduck comment block

//@mdata CString | SRGCompiledPolygon | m_strName | The name of the polygon.

//@mdata LPPOINT | SRGCompiledPolygon | m_pPoints | A pointer to the list of CPoints which is generated by compiling the 
//floating point coodinates.

//@mdata int | SRGCompiledPolygon | m_nCount | A count of the number of CPoint objects in the list pointed to by m_pPoints. Note
// that this may be different to the number of points in the floating point coordinate list

//@mdata CPtrList | SRGCompiledPolygon | m_Coords | A linked list of SRGCoordinate objects. 
//One for each point in the polygon. By storing the coordinates as floating 
//point data the scaling and positioning of graphic shapes may be carried out with great accurracy.

//@mdata BOOL | SRGCompiledPolygon | m_bDirty | As floating point coordinates are added to the list the integer CPoint list 
//will become out of date. This semaphore signifies that the floating point list must be recompiled.

//@mfunc int | SRGCompiledPolygon | GetCount | Returns the number of points or SRGCoordinates used to describe this polygon

//@class Manages a single re-scaleable polygon object
//@devnote When repeated movement, scaling or other operations are carried 
//out on polygons the coordinates, through small computational inaccuraccies, 
//may become degraded and the polygon become mis-shapen. By keeping a pristine floating point 
//"master list" of points and generating a compile CPoint list polygon from the master every time
//an operation takes place, the polygon will always be an accurate representation of the original.
class SRGCompiledPolygon : public CObject
{
//@access protected data members
protected:
	//@cmember Assigns or reassigns required memory.
	virtual void AssignPointBlock();
	//@cmember The name of the polygon or wiget
	CString m_strName;
	//@cmember A pointer to the points
	LPPOINT m_pPoints;
	//@cmember The number of points in the integer polygon
	int m_nCount;
	//@cmember a list of the floating point coordinates
	CPtrList m_Coords;
	//@cmember Semaphore showing an alteration of the 
	BOOL m_bDirty;
	

//@access public member functions
public:
	//@cmember Converts the floating point list into a CPoint list which is scaled to fit neatly into the desired rectangle.
	virtual LPPOINT GetFittedPoints(CRect &r);
	//@cmember Compiles the CPoint list to be an integer representation of the floating point list with x and y offset's added.
	virtual void OffsetPolygon(float xo,float yo);
	//@cmember Returns the name of this polygon
	virtual CString & GetName();
	//@cmember Initializes the name of this polygon
	virtual void SetName(LPCSTR name);
	//@cmember Constructor
	SRGCompiledPolygon();
	//@cmember Virtual Destructor
	virtual ~SRGCompiledPolygon();
	//@cmember Returns a pointer to the integer (compiled) point block
	virtual LPPOINT GetPoints();
	//@cmember Adds a floating point coord to the list
	virtual void AddCoord(SRGCoordinate *c);
	//@cmember Adds a coord using floats
	virtual void AddCoord(float xc,float yc);
	//@cmember adds a coord using integers
	virtual void AddCoord(int nx,int ny);
	//@cmember Returns the number of points in the floating point list.
	virtual int GetCount(){return m_Coords.GetCount();}
};




#endif

// Reset DLL declaration context

#undef AFX_DATA
#define AFX_DATA
#undef AFXAPP_DATA
#define AFXAPP_DATA NEAR
