//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc
//@module SRGraphComponent.h | class definition  of the SRGraphComponent class<nl>
//Copyright <cp> Stingray Software 1996.<nl>
//Written by Bob Powell (bob@stingsoft.com)

#ifdef _OBJCHART_DLL
	#undef AFXAPP_DATA
	#define AFXAPP_DATA AFXAPI_DATA
	#undef AFX_DATA
	#define AFX_DATA OC_DATA
#endif


#ifndef __GRAPH_COMPONENT__
#define __GRAPH_COMPONENT__

/////////////////////////////////////////////////////////
// Autoduck comment block for SRGraphRect

//@mfunc  | SRGraphRect | SRGraphRect | Constructor with assignment of data from device independant coordinates.
//@parm double | left | left side coordinate
//@parm double | topt | top edge coordinate
//@parm double | right | right side coordinate
//@parm double | bottom | bottom edge coordinate

//@mfunc  | SRGraphRect | SRGraphRect | Default Constructor

//@mfunc void | SRGraphRect | operator = | Assignment operator (overload #1)
//@parm SRGraphRect & | r | Reference to an existing SRGraphRect.

//@mfunc void | SRGraphRect | operator = | Assignment operator (overload #2)
//@parm CRect & | r | Reference to a CRect object. The left, top, right and bottom are converted to doubles and stored in the equivalent variable in this SRGraphRect.

//@mfunc double | SRGraphRect | Width | Take the width of the object
//@rdesc Returns right-left
//@see <mf SRGraphRect::Height> 
 
//@mfunc double | SRGraphRect | Height | Take the height of the object
//@rdesc returns bottom-top
//@see <mf SRGraphRect::Width>

//@mfunc void | SRGraphRect | Move | Shifts the rectangle by the x and y values provided
//@parm double | x | The value by which to move horizontally (may be negative)
//@parm double | y | The value by which to move vertically (may be negative)

//@mdata double | SRGraphRect | left | The left side coordinate

//@mdata double | SRGraphRect | top |  The top edge coordinate

//@mdata double | SRGraphRect | right |	 The right side coordinate

//@mdata double | SRGraphRect | bottom |  The bottom edge coordinate

//@mdata int  | SRGraphRect | m_nType |	The type of measurement used
//Values may be one of.<nl>
//0 = PIXELS<nl>
//1 = MILLIMETERS<nl>
//2 = INCHES<nl>
//3 = PERCENTAGE of available area<nl>

//@mdata CRect | SRGraphRect | m_LogRect | Private member. Used in the calculation of logical rectangles from device independant rectangles.

/////////////////////////////////////////////////////////

//@class device independant rectangle class
class SRGraphRect : public CObject
{
//@access public data members
public:
	//@cmember The device independant coordinates for the left of the rectangle
	double left;
	//@cmember The device independant coordinates for the top of the rectangle
	double top;
	//@cmember The device independant coordinates for the right of the rectangle
	double right;
	//@cmember The device independant coordinates for the bottom of the rectangle
	double bottom;

	enum _MeasurementTypes {
		pixel,
		millimeter,
		inch,
		percentage
	};


//@access protected data members
protected:
	//@cmember The type of measurement used. 0=pixels, 1=millimeters, 2=inches.
	int  m_nType;

//@access private data members
private:	
	//@cmember CRect object for internal conversions and calculations
	CRect m_LogRect;



//@access public member functions
public:
	//@cmember Returns m_nType data member
	int GetMeasurement();
	//@cmember Initialises m_nType data member
	void SetMeasurement(int m);
	//@cmember Initialises the rectangle from a CRect
	virtual void SetRect(const CRect& r);
	//@cmember Initialises the rectangle with device independant coordinates
	virtual void SetRect(double l,double t,double r, double b);
	//@cmember construction
	SRGraphRect(){left=top=right=bottom=(double)-1.0;m_nType=0;}
	//@cmember construction
	SRGraphRect(double l,double t,double r,double b){left=l; top=t; right=r; bottom=b;}
	//@cmember construction
	SRGraphRect(CRect& r);


	//@cmember calculate logical rectangle from device independant data;
	CRect CalcCRect(CDC *pDC,CWnd *pCWnd=NULL);


	DECLARE_SERIAL(SRGraphRect);
	//@cmember To store or retrieve information via a CArchive
	virtual void Serialize(CArchive& ar);


	//@cmember Assignment of data from an existing SRGraphRect object
	void operator =(const SRGraphRect& r){top=r.top; left=r.left; bottom=r.bottom; right=r.right;}
	//@cmember Assignment of data from a CRect object
	void operator =(const CRect& r){top=(double)r.top; 
									left=(double)r.left; 
									bottom=(double)r.bottom; 
									right=(double)r.right;}

	//@cmember Returns the width of the rectangle
	double Width(){return right-left;}
	//@cmember Returns the Height of the rectangle
	double Height(){return bottom-top;}
	//@cmember Displaces a rectangle
	void Move(double x, double y){left+=x; right+=x; top+=y; bottom+=y;}
	//@cmember Increases a rectangular area by X and Y units
	void Grow(double x, double y);
	//@cmember Calculates device independant measurements
	void CRectToSRGRect(CRect& rect,CDC *pDC);

	//@cmember Calculates device independant measurements (special overload for percentage measuring system)
	void CRectToSRGRect(CRect& rect,CDC *pDC,CWnd *pCWnd);

	//todo Decide if I need this or not.
	friend class SRGraphComponent;
};

/////////////////////////////////////////////////////////////////////
// defines used for hints in the basic component classes Uses 32 bit LPARAM hints

#define COMPONENT_FOREGROUND	0x00001000
#define LEGEND_TEXT				0x00002000
#define LEGEND_KEYS				0x00003000
#define DISPLAY_AXES			0x00004000
#define DISPLAY_LABELS			0x00005000
#define DISPLAY_DATA			0x00006000

#define COMPONENT_BACKGROUND	0x00000100

#define COMPONENT_BORDER		0x00000001

#define COMPONENT_SHADOW		0x00000010

// types

#define COMPONENT 0
#define OBJECT 1

// values above 0x0000ffff are available for your own custom hints


//@struct GESorter | This structure holds a value which is sorted by comparison with other 
//GESorter elements in an array and an index which associates the value with a group or index 
//within a group.<nl>
//GESorter elements are sorted by the qsort function using the <mf SRGraphComponent::qsSorterCompare> 
//callback. Once elements are sorted in ascending order of value the array of GESorter structures may 
//be sequentially indexed to return an index indirection to the final group or data item.<nl>
//This structure is used in the stacked bar graph and 3D bar and area graphs to ensure a front-to-back ordering 
//of data elements.
typedef struct tagGraphElementSortingStruct {
	double	value;	//@field The value to be compared and sorted.
	int index;		//@field The index of the item associated with this value.<nl>Used to dereference the item after sorting has taken place.
} GESorter;

//@struct PointSorter | This structure holds a CPoint which is sorted by comparison with other 
//GESorter elements in an array and an index which associates the value with a group or index 
//within a group.<nl>
//PointSorter elements are sorted by the qsort function using the <mf SRGraphComponent::qsPointSorterYCompareA> 
//callback. Once elements are sorted in ascending order of value the array of GESorter structures may 
//be sequentially indexed to return an index indirection to the final group or data item.<nl>
//This structure is used primarily in the drawing of 3D pie chart elements to perform rudimentary Z ordering.
typedef struct tagPointSortingStruct {
	CPoint p;	//@field The value to be compared and sorted.
	int i;		//@field The index of the item associated with this value.<nl>Used to dereference the item after sorting has taken place.
} PointSorter;


//////////////////////////////////////////////////
// Autoduck tagged comments for SRGraphComponent

//@mdata BOOL | SRGraphComponent | m_bDrawing | TRUE if the component is currently in a draw cycle. Otherwise FALSE
//@rem This semaphore is used to determine whether the current m_pCWnd and m_pDC pointers are valid. 
//Only when this flag is TRUE can the system rely on being able to gather correct device capability data 
//or rely on access to the window.

//@mdata CBrush | SRGraphComponent | m_AutoBrush | The brush which is used in auto coloring
//@see <mf SRGraphComponent::CreateAutoBrush> <md SRGraphComponent::m_AutoPen>

//@mdata CPen | SRGraphComponent | m_AutoPen | The pen which is used in auto coloring
//@see <mf SRGraphComponent::CreateAutoPen> <md SRGraphComponent::m_AutoBrush>

//@mdata BOOL | SRGraphComponent | m_bHiLo | Semaphore which is set if this component needs to take notice of HiLo (<c CScale>) data

//@mdata double | SRGraphComponent | m_dMarginX | The device independant margin size for the left and right side of a component
//@see <md SRGraphComponent::m_dMarginY> 

//@mdata int | SRGraphComponent | m_nMaxIndex| The maximum index that this component notices. -1 to autodetect and cover the whole range

//@mdata int | SRGraphComponent | m_nMinIndex|  The minimum index that this component notices. -1 to autodetect and cover the whole range

//@mdata int | SRGraphComponent | m_nMaxGroup|  The maximum group that this component notices. -1 to autodetect and cover the whole range

//@mdata int | SRGraphComponent | m_nMinGroup|  The minimum group that this component notices. -1 to autodetect and cover the whole range

//@mdata int | SRGraphComponent | m_nDivCount| The calculated count of indices spanned by this component

//@mdata int | SRGraphComponent | m_nMaxDiv| Internal maximum index which this component notices. 
// This is re-calculated at every draw cycle and depends on the value in <md SRGraphComponent::m_nMaxIndex>

//@mdata int | SRGraphComponent | m_nMinDiv| Internal minimum index which this component notices. 
// This is re-calculated at every draw cycle and depends on the value in <md SRGraphComponent::m_nMinIndex>
 
//@mdata int | SRGraphComponent | m_nSetCount| The calculated count of groups (sets) spanned by this component
 
//@mdata int | SRGraphComponent | m_nMinSet| Internal minimum group which this component notices. 
// This is re-calculated at every draw cycle and depends on the value in <md SRGraphComponent::m_nMinGroup>
 
//@mdata int | SRGraphComponent | m_nMaxSet| Internal maximum group which this component notices. 
// This is re-calculated at every draw cycle and depends on the value in <md SRGraphComponent::m_nMaxGroup>
 
//@mdata CScale | SRGraphComponent | m_ScaleY| The range of data values spanned by this component. Updated each draw cycle this variable is used in the dynamic re-calculation of graph scaling.

//@mdata double | SRGraphComponent | m_dTickSize| The size of the tick marks on graph axes. This physiscal size depends on the measurement standard used in the m_Rect variable.
//@see <mf SRGraphRect::SetMeasurement> <md SRGraphComponent::m_Rect>

//@mdata SRGraphRect | SRGraphComponent | m_Rect| The rectangle which contains this component. This is similar to the CRect object but used floating point coordinates rather than integer coordinates. 
//This is so that the rectangular region which is described by this variable may be measured in pixels, millimeters or inches.
//@see <c SRGraphRect>

//@mdata int | SRGraphComponent | m_nCurrentIndex| The number in this integer variable is used to generate the auto pens, fills and wiget shapes. Normally this variable is updated during the compoment 
//drawing cycle according to graph type. Bar and line graphs increment the variable according to the group currently being drawn. Pie charts update it once for each pie wedge.

//@mdata SRGraphStyle | SRGraphComponent | m_Style| The style variable for this component. Used to define the appearence of the item on screen.
//@see <c SRGraphStyle>

//@mdata CWnd * | SRGraphComponent | m_pCWnd| The draw time pointer to the window in which this component is to be displayed. Passed to the graph routines in the <mf SRGraph::DrawComponentList> function.

//@mdata CDC * | SRGraphComponent | m_pDC| The draw time pointer to the device context upon which this component is to be displayed. Passed to the graph routines in the <mf SRGraph::DrawComponentList> function. 

//@mdata SRGraph * | SRGraphComponent | m_pParentGraph| A pointer to the SRGraphObject which owns this component. Allows components to reference data stored by the graph.

//@mdata UINT | SRGraphComponent | m_nType| The type of component. This identifier is the resource ID of the string which describes the component. See the SRGraphResource (SRGRES.RC) files for details of IDS_SRG_TITLETYPE etc.

//@mdata BOOL | SRGraphComponent | m_bSignificantChange | May be set to tell the component that an external agent has made a significant change to the graph's data structures or to the components parameters.
// When this semaphore is TRUE the next draw cycle will recalculate all of the relavent sizes, labels parameters etc and then reset this variable to signify handling of the condition.

//@mdata CObList | SRGraphComponent | m_PolygonList | A linked list of previously compiled polygons which are available to this component. Whenever a polygon resource is requested the component will examin this list to see if it has already been compiled.
// If so then it will use the existing model. If not an attempt will be made to find, compile and add the requested polygon before use.

//@mdata CScale | SRGraphComponent | m_ScaleX | Used to hold the calculated scale for the X axis of scatter graphs and similar types. Normally calculated at the beginning of the draw cycle by the relavent preparation routine

//@mdata double | SRGraphComponent | m_dMarginY | Defines the Y margin around the component. May be set to a position independant value. The data returned by the GetMargin functions will use the type setting of the SRGraphRect m_Rect member to adjust the size.

//@mdata int | SRGraphComponent | m_nCountryCode | Defines which annotation is to be used for text items in this component. Any SRGraphAnnotationList based text will use this country code to decide which one of the many text strings available will be used.

//@mfunc BOOL | SRGraphComponent | GetHiLo | Returns the semaphore which decides whether hi-lo data gathering is to be used. If the return is TRUE then the pre-draw preparation routines should take the current values of the hi-lo records into account when working out any scales

//@mfunc void | SRGraphComponent | SetHiLo | Initialises the semaphore which decides whether hi-lo data gathering is to be used. If the parameter is TRUE then the pre-draw preparation routines should take the current values of the hi-lo records into account when working out any scales
//@parm BOOL | style | TRUE if hi-lo gathering is to be used

//@mfunc BOOL | SRGraphComponent | IsDrawing | Returns the m_bIsDrawing member. If this item is being drawn or if the item has been told to use a specific CDC and CWnd combination then this will return TRUE

//@mfunc CDC * | SRGraphComponent | GetpDC | Returns the pointer to the CDC object being used to draw the component. Note that this is only valid during a draw cycle or after the component has been suitably informed by the <mf SRGraphComponent::DrawingUsing> command

//@mfunc CWnd * | SRGraphComponent | GetpCWnd | Returns the pointer to the CWnd object being used to draw the component. Note that this is only valid during a draw cycle or after the component has been suitably informed by the <mf SRGraphComponent::DrawingUsing> command

//@mfunc int | SRGraphComponent | GetDivCount | Returns the number of indices or divisions  which this component currently has in its scope.

//@mfunc int | SRGraphComponent | GetSetCount | Returns the number of groups or Sets which this component currently has in its scope.

//@mfunc CScale & | SRGraphComponent | GetScaleX | Returns a reference to the m_ScaleX member

//@mfunc CScale & | SRGraphComponent | GetScaleY | Returns a reference to the m_ScaleY member

//@mfunc int | SRGraphComponent | GetCountryCode | Returns the contents of the m_nCountryCode member

//@mfunc int | SRGraphComponent | GetMaxDiv | Returns the maximum index which this component may currently act upon

//@mfunc int | SRGraphComponent | GetMinDiv | Returns the minimum index which this component may currently act upon

//@mfunc int | SRGraphComponent | GetMaxSet | Returns the maximum group which this component may currently act upon

//@mfunc int | SRGraphComponent | GetMinSet | Returns the minimum group which this component may currently act upon

//@mfunc int | SRGraphComponent | GetMMeasurement | Returns the type of measurement which this component is using. 
//@see <mf SRGraphRect::GetMeasurement> for code details.

//@mfunc SRGraph * | SRGraphComponent | GetParent | Returns the pointer (m_pParentGraph) which references the SRGraph which owns this component

//@mfunc void | SRGraphComponent | SetCountryCode | Sets the code used to decide which annotation set should be used in this component
//@parm int | nCode | The code to use

//@mfunc void | SRGraphComponent | SetpCWnd | Sets the CWnd object used to draw this component
//@parm CWnd * | pCWnd | The CWnd object to use

//@mfunc void | SRGraphComponent | SetpDC | Sets the CDC object used to draw this component
//@parm CDC * | pDC | The CDC object to use

//@mfunc void | SRGraphComponent | SetSignificantChange | Sets the semaphore which flags an important change in the data or parameters used by this component


//////////////////////////////////////////////////

//@class Base class for graphical components
class SRGraphComponent :  public CObject
{
//@access protected data members
protected:
	//@cmember Semaphore showing that a significant change has taken place either in this component or in the graph which owns it
	BOOL m_bSignificantChange;
	//@cmember A list of SRGCompiledPolygons for this object.
	CObList m_PolygonList;
	//@cmember The integer localising code for multi lingual annotations
	int m_nCountryCode;
	//@cmember Semaphore used to show a requirement for the use of hi-lo history data
	BOOL m_bHiLo;
	//@cmember The maximum index upon which this component should act
	int m_nMaxIndex;
	//@cmember The minimum index upon which this component should act
	int m_nMinIndex;
	//@cmember The maximum group upon which this component should act
	int m_nMaxGroup;
	//@cmember The minimum group upon which this component should act
	int m_nMinGroup;
	//@cmember The number of indices which this component may span
	int m_nDivCount;
	//@cmember The maximum index which this component may use
	int m_nMaxDiv;
	//@cmember The minimum index which this component may use
	int m_nMinDiv;
	//@cmember The number of groups which this component may span
	int m_nSetCount;
	//@cmember The minimum group
	int m_nMinSet;
	//@cmember The maximum group
	int m_nMaxSet;
	//@cmember The Y scale calculated for this components range
	CScale m_ScaleY;
	//@cmember The X scale calculated for this components range
	CScale m_ScaleX;
	//@cmember The device independant size for tick marks
	double m_dTickSize;
	//@cmember The device independant size for the Y margin
	double m_dMarginY;
	//@cmember The device independant size for the X margin
	double m_dMarginX;
	// The pen used to frame automatically framed objects
	CPen m_AutoPen;
	// The brush used to fill automatically filled objects
	CBrush m_AutoBrush;
	//@cmember device independant rect.
	SRGraphRect m_Rect;
	//@cmember Holds the index of the SRGraphData object being drawn. For use with automatic filling and framing
	int m_nCurrentIndex;
	//@cmember Drawing style flags
	SRGraphStyle m_Style;
	//@cmember Semaphore. Set when drawing is taking place
	BOOL m_bDrawing;
	//@cmember Pointer to the CWnd object which requested the drawing
	CWnd *m_pCWnd;
	//@cmember Pointer to the CDC of the window which requested the drawing
	CDC *m_pDC;
	//@cmember Pointer to the graph owning this object
	SRGraph * m_pParentGraph;
	//@cmember The component identifier. Used in reading, writing and editing of graph templates
 	UINT m_nType;


//@access public member functions
public:
	
	//@cmember Initializes the X margin size
	void SetXMargin(double d){m_dMarginX=d;}
	
	//@cmember Initializes the X margin size
	void SetYMargin(double d){m_dMarginY=d;}

	//@cmember Initializes the tick mark size
	void SetTickSize(double d){m_dTickSize=d;}

	//@cmember Draws custom borders
	virtual void DrawCustomBorder();

	//@cmember Draws custom shadows
	virtual void DrawCustomShadow();

	//@cmember Draws custom fills
	virtual void DrawCustomRgnFill(CRgn *rgn,SRGraphStyle *style,DWORD type,DWORD StyleOverride);

	//@cmember Reads the parameters for this component from the template line
	virtual int ReadTemplateLine(LPCSTR line);

	//@cmember Reads the basic rectangle and component fill, border and shadow parameters for all components
	virtual int ReadBasicParameters(LPCSTR line);

	//@cmember Sets a semaphore signifying a significant change in either the component or the graph which owns it
	virtual void SetSignificantChange(){m_bSignificantChange=TRUE;}

	//@cmember returns the minimum index which this component currently accesses
	virtual int GetMinDiv(){return m_nMinDiv;}

	//@cmember returns the maximum index which this component currently accesses
	virtual int GetMaxDiv(){return m_nMaxDiv;}

	//@cmember returns the minimum group which this component currently accesses
	virtual int GetMinSet(){return m_nMinSet;}

	//@cmember returns the maximum group which this component currently accesses
	virtual int GetMaxSet(){return m_nMaxSet;}

	//@cmember Returns the Y scale range
	virtual CScale &GetScaleY(){return m_ScaleY;}

	//@cmember Returns the X scale range
	virtual CScale &GetScaleX(){return m_ScaleX;}

	//@cmember returns the number of groups which are spanned by this component
	virtual const int GetSetCount(){return m_nSetCount;}

	//@cmember returns the number of indices which are spanned by this component
	virtual const int GetDivCount(){return m_nDivCount;}

	//@cmember Draws a wiget/line combination from one point to another in the style required.
	virtual void DrawPointObject(CPoint firstpoint,CPoint lastpoint,SRGraphStyle *ptrStyle,int nObjectMode);

	//@cmember Constructor
	SRGraphComponent();	

	//@cmember  destructor
	virtual ~SRGraphComponent();	

	//@cmember Creates a brush which can be used for filling the interiors of objects according to an index
	BOOL CreateAutoBrush(int key);	

	//@cmember Creates a pen which can be used for framing the boundaries of objects according to an index
	virtual BOOL CreateAutoPen(int key,int nPenWidth=1);	

	//@cmember	Retrieves pertinent data from the parent graph.
	virtual void GetParentContent(); 

	//@cmember Interrogates the device independant Y margin value
	virtual double GetYMargin();

	//@cmember Interrogates the device independant X margin value
	virtual double GetXMargin();

	//@cmember Interrogates the device independant axis tick value
	virtual double GetTickSize();

	//@cmember Initialises the maximum index that a component may use
	virtual void SetMaxIndex(int index);

	//@cmember Initialises the maximum index that a component may use
	virtual void SetMinIndex(int index);

	//@cmember Initialises the maximum group that a component may use
	virtual void SetMaxGroup(int group);

	//@cmember Initialises the minimum group that a component may use
	virtual void SetMinGroup(int group);

	//@cmember Interrogates the maximum index setting
	virtual int GetMaxIndex();

	//@cmember Interrogates the minimum index setting
	virtual int GetMinIndex();

	//@cmember Interrogates the maximum group setting
	virtual int GetMaxGroup();

	//@cmember Interrogates the minimum group setting
	virtual int GetMinGroup();

	//@cmember Fetches the type of the component (eg. background, title etc.)
	virtual UINT GetType();

	//@cmember Initialises the pointer to the SRGraph which owns this object
	virtual void SetParent(SRGraph *pParent);

	//@cmember Initialises the name of the bitmap resource used to tile the background
	virtual void SetResourceName(LPCSTR name);

	//@cmember Initialises the m_Angle member
	virtual void SetAngle(int angle);

	//@cmember Initialises the RGB color values of the filled background
	virtual void SetColor(COLORREF a,COLORREF b);

	//@cmember Retrieves the style of this object
	virtual SRGraphStyle *GetStyle();

	//@cmember Initialises the style of this object
	virtual void SetStyle(DWORD style);

	//@cmember Initialises the device independant rectangle from a CRect
	virtual void SetRect(const CRect& r);

	//@cmember Initialises the device independant rectangle with floating point coordinates.
	virtual void SetRect(double l, double t, double r, double b);

	//@cmember Initialises the device independant rectangle with the contents of a second.
	virtual void SetRect(const SRGraphRect& r);

	//@cmember Retrieves a pointer to the device independant rectangle which defines the boundary of this object
	virtual SRGraphRect* GetRect();

	//@cmember Initialises the measurement type of the m_Rect data member 0=Pixels, 1=Millimeters, 2=Inches;
	virtual void SetMeasurement(int m=PIXELS);

	//@cmember Called by the CWnd managing the display of graph data to draw the graph.
	virtual void Draw(CDC *pDC,CWnd *pCWnd);

	//@cmember Fills a poygon held in an array of points according to a style
	virtual void FillPoly(LPPOINT points,int nCount,SRGraphStyle *style,DWORD StyleOverride);

	//@cmember Frames a poygon held in an array of points according to a style
	virtual void FramePoly(LPPOINT points,int nCount,SRGraphStyle *style,DWORD StyleOverride);

	//@cmember Frames a pre defined region according to a style
	virtual void FrameRgn(CRgn *rgn,SRGraphStyle *style,DWORD type,DWORD StyleOverride);

	//@cmember Fills a pre defined region according to a style
	virtual void FillRgn(CRgn *rgn,SRGraphStyle *style,DWORD type,DWORD StyleOverride=0);

	//@cmember decides which areas of the display need to be invalidated after a CWnd::OnUpdate with a hint
	virtual void ProcessHint(CWnd *pCWnd);

	//@cmember callback routine for the qsort function.
	static int __cdecl qsSorterCompareA(const void *a,const void *b);

	//@cmember callback routine for the qsort function.
	static int __cdecl qsSorterCompareD(const void *a,const void *b);

	//@cmember Writes the component to a temlplate file
	virtual void WriteComponent(CStdioFile* file);

	//@cmember Writes this component name to a template file
	virtual void WriteName(CStdioFile* file);

	//@cmember Writes font details of this component to a template file
	virtual void WriteFontParameters(CStdioFile* file);

	//@cmember Writes the main details for this component to a template file
	virtual void WriteMainParameters(CStdioFile* file);

	//@cmember Writes specific details of this component to a template file
	virtual void WriteSpecificParameters(CStdioFile* file);

	//@cmember Static member callback. Supplied to qsort to order Y components for pie chart wedges into acending order
	static int _cdecl qsPointYSorterCompareA(const void *a,const  void *b);

	//@cmember Static member callback. Supplied to qsort to order Y components for pie chart wedges into decending order
	static int _cdecl qsPointYSorterCompareD(const void *a,const  void *b);

	//@cmember Returns the m_bHiLo semaphore
	virtual BOOL GetHiLo(){return m_bHiLo;}

	//@cmember Initializes the m_bHiLo semaphore.
	virtual void SetHiLo(BOOL hilo=TRUE){m_bHiLo=hilo;}

	//@cmember Fetches the measurement standard from the m_Rect member
	virtual int GetMeasurement(){return m_Rect.GetMeasurement();}

	//@cmember Initialises the m_pDC member
	virtual void SetpDC(CDC *pDC){m_pDC=pDC;}

	//@cmember Interrogates the m_pDC member
	virtual CDC * GetpDC(){return m_pDC;}

	//@cmember Initialises the m_pCWnd member
	virtual void SetpCWnd(CWnd *pCWnd){m_pCWnd=pCWnd;}

	//@cmember Interrogates the m_pCWnd member
	virtual CWnd * GetpCWnd(){return m_pCWnd;}

	//@cmember  Draws a line from ptStart to ptEnd in the desired style.
	virtual void DrawStyledLine(CPoint ptStart,CPoint ptEnd,SRGraphStyle *ptrStyle=NULL, int nKey=0);

	//@cmember Draws bars for bar graphs
	virtual void DrawBar(CPoint topleft,CPoint bottomright, int group, int index,SRGraphStyle* ptrStyle,DWORD dwStyleOverride=0 );

	//@cmember Moves a polygon stored as a list of CPoints by specified X and Y offsets.
	virtual void OffsetPolygon(CPoint * points,int nPointCount, int xOffset,int yOffset);

	//@cmember Sets the m_nCountryCode member
	virtual void SetCountryCode(int c=1);

	//@cmember Interrogates the m_nCountryCode member
	virtual int GetCountryCode(){return m_nCountryCode;}

	//@cmember Interrogates the m_pParentGraph member
	virtual SRGraph * GetParent(){return m_pParentGraph;}

	//@cmember Calculates the width and height of a given polygon from its points
	virtual CSize GetPolgonSize(CPoint * points,int nPointCount);

	//@cmember Calculates the length of the line between the two points
	double GetLineLength(CPoint ptStart,CPoint ptEnd);

	//@cmember Calculates the angle of ptEnd with respect to ptStart
	double GetLineAngle(CPoint ptStart,CPoint ptEnd);

	//@cmember Invalidates the m_bDrawing semeaphore
	virtual void NotDrawing();

	//@cmember Informs a component that drawoperations are taking place using a specific set of CWnd and CDC pointers.
	virtual void DrawingUsing(CDC *pDC,CWnd *pCWnd);

	//@cmember Interrogates the m_bDrawing member.
	virtual BOOL IsDrawing();

	//@cmember Draws a graphical token at a point on screen
	virtual void DrawWiget(CPoint point,SRGraphStyle *ptrStyle);


	DECLARE_SERIAL(SRGraphComponent);

	//@cmember To store or retrieve information via a CArchive
	virtual void Serialize(CArchive& ar);


#ifdef _DEBUG

	void Dump(CDumpContext& dc) const;

#endif

//@cmember,menum The measurement standard used in this rectangle
enum Measurement_type {
	PIXELS,			//@@emem No conversion. Data is in pixels
	MILLIMETERS,	//@@emem The floating point value is in millimeters.
	INCHES,			//@@emem The floating point value is in inches.
	PERCENT			//@@emem The value is a percentage of the available client area (Not currently implemented. Do not use)
};

public:


//@access protected member functions
protected:
	//@cmember Draws the filled background to this component (if any)
	virtual void DrawFill();
	//@cmember Draws the border to this component (if any)
	virtual void DrawBorder();
	//@cmember Override this virtual function to draw the forground content of this component (if any)
	virtual void DrawForeground(){}
	//@cmember Draws the shadow around the object.
	virtual void DrawShadow();

};

//////////////////////////////////////////////////////////////////
// standard graph components

//////////////////////////////////////////////////////////////////
// Autoduck comment block for SRGraphTitle

//@mdata int  | SRGraphTitle | m_nJustification | The character justification.<nl>
//0=Left justified<nl>
//1=Right justified<nl>
//2=Centered in component
//@see <mf SRGraphTitle::SetText> <mf SRGraphTitle::GetJustification> <mf SRGraphTitle::SetJustification>

//@mdata CString  | SRGraphTitle | m_strTitle | The text to be displayed.
//@rem Note that setting of this textoutside the draw loop  will never be seen because the component updates the text using <mf SRGraphTitle::GetParentContent> every draw cycle.

//@mdata CFont  | SRGraphTitle | m_Font | A CFont object which defines the type style used
//@see <mf SRGraphTitle::GetFont> <mf SRGraphTitle::SetFont>

//@mdata COLORREF  | SRGraphTitle | m_RGBTextColor | Sets the RGB color used to display text.
//@see <mf SRGraphTitle::SetTextColor> <mf SRGraphTitle::GetTextColor>

//@mdata BOOL  | SRGraphTitle | MakeFont | Internal semaphore. When TRUE the component updates the font information and creates a new font. Use this variabe to ensure correct update of fonts after changes to font style.

//@mdata int  | SRGraphTitle | m_nSize | Use positive numbers to make a true point sized font or use negative numbers to create a font which is a specific number of screen pixels high.
//
//@ex for example. | <nl>
//	SetFontSize(10);	
//	SetMakeFont();	// Sets a font of 10 points
//
//	SetFontSize(-10);
//	SetMakeFont();	//Sets a font of 10 pixels.
//@see <mf SRGraphTitle::SetFontSize> <mf SRGraphTitle::GetFontSize> <md SRGraphTitle::MakeFont> <mf SRGraphTitle::SetMakeFont>

//@mdata int  | SRGraphTitle | m_nStyle | Bitwise flage which define the font settings for Bold, Italic and other styles.<nl>
//bit 0 = BOLD<nl>
//bit 1 = ITALIC<nl>
//bit 2 = UNDERLINE<nl>
//bit 3 = STIKEOUT<nl>
//@see <mf SRGraphTitle::SetFontStyle> <mf SRGraphTitle::GetFontStyle>

//@mdata CString  | SRGraphTitle | m_strFaceName | The name of the current typeface eie. "Times New Roman"
//@see <mf SRGraphTitle::SetFaceName> <mf SRGraphTitle::GetFaceName> <mf SRGraphTitle::SetFont>

//@mfunc  void | SRGraphTitle |  SetFontSize | Initialise the font size
//@parm int | Size | The new font size.
//@rem Positive numbers make point size fonts. The absolute values of negative numbers make pixel sized fonts.
//@see <mf SRGraphTitle::GetFontSize> <md SRGraphTitle::m_nFontSize>

//@mfunc  void | SRGraphTitle |  SetFontStyle | Sets the bitwise flags which define the font style
//@parm int | style | Logical or combination of the required flags
//@see For bit combinations.. <md SRGraphTitle::m_nStyle> <mf SRGraphTitle::GetFontStyle>

//@mfunc  void | SRGraphTitle |  SetJustification | Sets the justification style.
//@parm int | Justification | The type of justification required
//@see For justification types.. <md SRGraphTitle::m_nJustification> <mf SRGraphTitle::GetJustification>

//@mfunc  int | SRGraphTitle |  GetFontSize | Fetches the current font size.
//@rdesc The contents of the m_nSize member.
//@see <mf SRGraphTitle::SetSize> <md SRGraphTitle::m_nSize>

//@mfunc  int | SRGraphTitle |  GetFontStyle | Fetches the current font style
//@rdesc The contents of the m_nStyle member
//@see <mf SRGraphTitle::SetFontStyle> <md SRGraphTitle::m_nStyle>

//@mfunc  int | SRGraphTitle |  GetJustification | Fetches the current font justification.
//@rdesc The contents of the m_nJustification member.
//@see <mf SRGraphTitle::SetJustification> <md SRGraphTitle::m_nJustification>

//@mfunc  COLORREF | SRGraphTitle |  GetTextColor | Fetches the color currently used for text
//@rdesc The contents of the m_RGBTextColor member.
//@see <mf SRGraphTitle::SetTextColor> <md SRGraphTitle::m_RGBTextColor>

//@mfunc  CString& | SRGraphTitle |  GetText | Fetches the current text for this title.
//@rdesc The contents of the m_strTitle member.
//@see <mf SRGraphTitle::SetText> <md SRGraphTitle::m_strTitle>

//@mfunc  CFont* | SRGraphTitle |  GetFont | Fetches a pointer to the m_Font member
//@rdesc A pointer to the m_Font member.
//@see <md SRGraphTitle::m_Font> <mf SRGraphTitle::SetFont>

//@mfunc void | SRGraphTitle | SetMakeFont | Sets or resets the MakeFont variable. You should set the MakeFont variable after changing font styles. The font is updated at draw time and is only changed if the MakeFont semaphore is set.
//@parm BOOL | make | TRUE (default) to set the semaphore. FALSE to reset
//@rem This method is used because the UpdateFont routine needs to get information directly from the CDC used to draw the font. Therefore the request to change font style is defferred until the m_pDC pointer is known to be valid.
//@see <md SRGraphTitle::MakeFont> <mf SRGraphTitle::UpdateFont> <mf SRGraphTitle::SetFontStyle> <mf SRGraphTitle::SetFontSize>

//@mfunc void | SRGraphTitle | SetTextColor | Initialises the text color used in this object.
//@parm COLORREF | c | The desired color.


//////////////////////////////////////////////////////////////////

//@class Title manager derived from <c SRGraphComponent>
class SRGraphTitle : public SRGraphComponent
{
//@access public data members
public:
	virtual int ReadTemplateLine(LPCSTR line);

//@access protected data members
protected:
	//@cmember The type of text justification to use
	int m_nJustification;
	//@cmember The title text.
	CString m_strTitle;
	//@cmember The font in which to display text.
	CFont m_Font;
	//@cmember The color of the text
	COLORREF m_RGBTextColor;
	//@cmember Semaphore which signifies that a font has been set or that an existing font needs updating.
	BOOL MakeFont;
	//@cmember Size of font. Negative numbers set sizes in pixels. Positive numbers set sizes in points
	int m_nSize;
	//@cmember Style of font.
	int m_nStyle;
	//@cmember Font typeface
	CString m_strFaceName;

//@access Protected member functions
protected:

	//@cmember To draw the text.
	virtual void DrawForeground();

	//@cmember Keeps font data current. 
	virtual BOOL UpdateFont();

	//@cmember Retrieves pertinent data from the parent
	virtual void GetParentContent();

//@access public member functions
public:
	//@cmember Sets or resets the MakeFont semaphore
	virtual void SetMakeFont(BOOL make=TRUE);
	//@cmember Returns logical size of text
	virtual CSize GetTextDimension(LPCSTR text);
	//@cmember Constructor
	SRGraphTitle();
	//@cmember Writes the name "title" to the template file
	virtual void WriteName(CStdioFile* file);
	//@cmember Outputs the current font details to the template file
	virtual void WriteFontParameters(CStdioFile* file);
	//@cmember Sets the point or pixel size of the current font
	virtual void SetFontSize(int Size){m_nSize=Size;}
	//@cmember Sets the style of the font. ie Bold, Underline etc.
	virtual void SetFontStyle(int style){m_nStyle=style;}
	//@cmember Sets the font justification to left (0), right (1) or centered (2)
	virtual void SetJustification(int Justification){m_nJustification=Justification;}
	//@cmember Fetches the font size data
	virtual int GetFontSize(){return m_nSize;}
	//@cmember Fetches the font style data
	virtual int GetFontStyle(){return m_nStyle;}
	//@cmember Fetches the font justification
	virtual int GetJustification(){return m_nJustification;}
	//@cmember Fetches the RGB color used to display text
	virtual COLORREF GetTextColor(){return m_RGBTextColor;}
	//@cmember Returns the title text itself
	virtual CString& GetText(){return m_strTitle;}
	//@cmember Fetches a pointer the the m_Font (CFont) object from this component
	virtual CFont* GetFont(){return &m_Font;}
	//@cmember Sets the typeface name (ie "Times New Roman")
	virtual void SetFaceName(CString &FaceName);
	//@cmember Fetches the type face name currently in use
	virtual CString& GetFaceName();

#ifdef _DEBUG

	void Dump(CDumpContext& dc) const;

#endif


	DECLARE_SERIAL(SRGraphTitle);
	//@cmember To store or retrieve information via a CArchive
	virtual void Serialize(CArchive &ar);


	//@cmember Destructor. Ensures that no GDI objects remain.
	virtual ~SRGraphTitle();


	//@cmember Sets font size, style and typeface.
	virtual BOOL SetFont(int nSize,int nStyle,CString FaceName);

	//@cmember Modifies the contents of the text.
	virtual void SetText(LPCSTR text,int justification=0);

	//@cmember Sets the color of text displayed to an RGB value.
	virtual void SetTextColor(COLORREF color){m_RGBTextColor=color;}

};

//@class Based on SRGraphComponent this class handles painting of the background.
class SRGraphBackground : public SRGraphComponent
{
//@access public member functions
public:
	//@cmember Constructor
	SRGraphBackground();
	
	virtual int ReadTemplateLine(LPCSTR line);
	virtual void WriteName(CStdioFile* file);


	DECLARE_SERIAL(SRGraphBackground);

//@access protected member functions
protected:
	//@cmember Overridden from base class. Does nothing because backgrounds dont have foregrounds.
	virtual void DrawForeground(){}
};


////////////////////////////////////////////////////////
// Autoduck comment block for SRGraphDisplay
//

//@mdata double | SRGraphDisplay | m_dMinRange | The minimum range which this display will show on its axes
//@rem If this range is exeeded by the data then there is no overflow procedure. it is the responsibility of
//the application to keep data within the correct range when these limits are used
//@see <mf SRGraphDisplay::SetMinRange>  <mf SRGraphDisplay::GetMinRange>

//@mdata double | SRGraphDisplay | m_dMaxRangeY |  The maximum range which this display will show on its axes
//@rem If this range is exeeded by the data then there is no overflow procedure. it is the responsibility of
//the application to keep data within the correct range when these limits are used
//@see <mf SRGraphDisplay::SetMaxRange>  <mf SRGraphDisplay::GetMaxRange>

//@mdata double | SRGraphDisplay | m_dMaxScaleY | Used internally to calculate the scaling factor for data. This variable
// is usually set by taking the maximum range of data an multiplying by 1.1. This ensures a maximum data scale which is
// always a little bit larger than the data in the graph. Axes, labels and scales are all re-calculated using this number to 
// keep data from overflowing the display area

//@mdata CRect | SRGraphDisplay | m_LabelRectX | Rectangle used during the drawing of labels.

//@mdata CRect | SRGraphDisplay | m_LabelRectY | Rectangle used during the drawing of labels.

//@mdata CRect | SRGraphDisplay | m_DisplayRect | Rectanle which contains the usable data display area. 
//@rem This starts out each draw cycle with the whole component area. As items are drawn this rectangle is reduced in size 
// so that subsequent items know how much area is left to work with. For example The component reduces the size to account 
// for the borders. The labels then reduce the size to account for thier widths and heights leaving the axis and data to be drawn 
// in the area that is left.

//@mdata CRect | SRGraphDisplay | m_DrawnRect | A record of the last final m_DisplayRect values
//@see <md SRGraphDisplay::m_DisplayRect>

//@mdata BOOL | SRGraphDisplay | m_bDisplayDrawn | Set to TRUE when the display has been drawn at least once.

//@mdata CPtrList | SRGraphDisplay | m_XTickList | The X, Y and Z tick lists are optionally built by the label system to allow accurate positioning of the tick marks and grid lines
//@rem The tick list contains a simple CPoint list. Each point is derived directly from the position of a label used on the graph's axes. These positions are normally only used to position the 
// ticks or grid lines vertically or horizontally.

//@mdata SRGraphLabelBlock * | SRGraphDisplay | m_ptrXLabels | The X, Y and Z label pointers are placeholders for any one of the SRGraphLabelBlock (or derived) objects. The axis drawing system will normally assign a semi permanent label block 
//(<c SRGraphLabelBlock>) or decimal scale (<c SRGDecimalScale>) to one of these pointers for the duration of its existence. Keeping the same labelblock and modifying it requires far less time than constantly creating and destroying large complex objects on the stack.



////////////////////////////////////////////////////////

//@class Handles graph data display. Inherits font abilities from SRGraphTitle
class SRGraphDisplay : public SRGraphTitle
{
//@access public data members
public:

//@access protected data members.
protected:
	//@cmember The Y scale to be used when data is zoomed up. Used only on XY Scatter graphs for the moment
	CScale m_ZoomScaleY;
	//@cmember The X scale to be used when data is zoomed up. Used only on XY Scatter graphs for the moment
	CScale m_ZoomScaleX;
	//@cmember the minimum Y numeric value allowed for this graph
	double m_dMinRangeY;
	//@cmember the maximum Y numeric value allowed for this graph
	double m_dMaxRangeY;
	//@cmember The maximum Y scale of the values in this graph display
	double m_dMaxScaleY;
	//@cmember the minimum X numeric value allowed for this graph
	double m_dMinRangeX;
	//@cmember the maximum X numeric value allowed for this graph
	double m_dMaxRangeX;
	//@cmember The maximum X scale of the values in this graph display
	double m_dMaxScaleX;
	//@cmember A pointer to a block of labels used to annotate the X axis.
	SRGraphLabelBlock * m_ptrXLabels;
	//@cmember A pointer to a block of labels used to annotate the Y axis.
 	SRGraphLabelBlock * m_ptrYLabels;
	//@cmember A pointer to a block of labels used to annotate the Z axis.
	SRGraphLabelBlock * m_ptrZLabels;
	//@cmember A list of points which have been provided by the X label block. Used to position tick marks and grid lines
	CPtrList m_YTickList;
	//@cmember A list of points which have been provided by the Y label block. Used to position tick marks and grid lines
	CPtrList m_XTickList;
	//@cmember A list of points which have been provided by the Z label block. Used to position tick marks and grid lines
	CPtrList m_ZTickList;
	//@cmember The rectangle enclosing the X labels
	CRect m_LabelRectX;
	//@cmember The rectangle enclosing the Y labels
	CRect m_LabelRectY;
	//@cmember Initially the whole component display area. This rectangle gets 
	//re-sized according to room taken up by labels etc.
	CRect m_DisplayRect;
	//@cmember Used to hold the rectangle previously drawn
	CRect m_DrawnRect;
	//@cmember Semaphore TRUE if the display has been drawn
	BOOL m_bDisplayDrawn;

	
//@access public member functions
public:
	//@cmember Constructor.
	SRGraphDisplay();


	DECLARE_SERIAL(SRGraphDisplay);

	//@cmember Stores or retrieve this object.
	virtual void Serialize(CArchive& ar);
	//@cmember Pre display preparation for strata graphs
	virtual void PrepareStrata();
	//@cmember Returns the m_dMaxScaleY member
	virtual double GetMaxScaleY(){return m_dMaxScaleY;}
	//@cmember Returns the m_dMaxScaleX member
	virtual double GetMaxScaleX(){return m_dMaxScaleX;}
	//@cmember If a change is made to the graph type after its first draw cycle then the display must be refreshed.
	virtual void Refresh();
	//@cmember Reads a single line from the graph template 
	virtual int ReadTemplateLine(LPCSTR line);
	//@cmember Sets the X data zooming scale. Currently only used for scattergraphs
	virtual void SetZoomScaleX(CScale scale){m_ZoomScaleX=scale;}
	//@cmember Sets the Y data zooming scale. Currently only used for scattergraphs
	virtual void SetZoomScaleY(CScale scale){m_ZoomScaleY=scale;}
	//@cmember Gets the X data zooming scale. Currently only used for scattergraphs
	virtual CScale GetZoomScaleX(){return m_ZoomScaleX;}
	//@cmember Gets the Y data zooming scale. Currently only used for scattergraphs
	virtual CScale GetZoomScaleY(){return m_ZoomScaleY;}
	//@cmember Zeroes the X data zooming scale. Currently only used for scattergraphs
	virtual void CleanZoomScaleX(){m_ZoomScaleX.Zero();}
	//@cmember Zeroes the Y data zooming scale. Currently only used for scattergraphs
	virtual void CleanZoomScaleY(){m_ZoomScaleY.Zero();}
	//@cmember Writes the name of this component to file
	virtual void WriteName(CStdioFile* file);
	//@cmember Writes component parameters for Display components to file
	virtual void WriteSpecificParameters(CStdioFile* file);
	//@cmember Returns the display rectangle
	virtual CRect GetDisplayRect(){return m_DisplayRect;}
	//@cmember Returns the minimum numeric value allowed for this graph's Y axis
 	virtual double GetMinRangeY();
	//@cmember Retrieves the maximum numeric value allowed for this graph
	virtual double GetMaxRangeY();
	//@cmember Sets the minimum numeric value allowed for this graph
	virtual void SetMinRangeY(double dValue);
	//@cmember Sets the maximum numeric value allowed for this graph
	virtual void SetMaxRangeY(double dValue);
 	virtual double GetMinRangeX();
	//@cmember Retrieves the maximum numeric value allowed for this graph
	virtual double GetMaxRangeX();
	//@cmember Sets the minimum numeric value allowed for this graph
	virtual void SetMinRangeX(double dValue);
	//@cmember Sets the maximum numeric value allowed for this graph
	virtual void SetMaxRangeX(double dValue);
	//@cmember Allows initialisation and locking of the display rectangle
	virtual void SetDisplayRect(CRect r,BOOL lock=FALSE);
	//@cmember Virtual destructor for this class
	virtual ~SRGraphDisplay();

//@access Protected member functions
protected:
	//@cmember Draws Gantt labels
	virtual void DrawGanttLabels();
	//@cmember Draws Gantt data
	virtual void DrawGanttData();
	//@cmember Draws polar chart labels
	virtual void DrawPolarLabels();
	//@cmember Draws polar chart axes
	virtual void DrawPolarAxis();
	//@cmember Draws candle chart labels
	virtual void DrawCandleData();
	//@cmember prepares the display for drawing polar data
	virtual void PreparePolar();
	//@cmember Draws polar chart data
	virtual void DrawPolarData();
	//@cmember Draws area chart data
	virtual void DrawAreaData();
	//@cmember Draws the pseudo 3D axis
	virtual void DrawIsoClassicAxis();
	//@cmember Draws the data components of FreeStyle graphs
	virtual void DrawFreeStyleData();
	//@cmember Virtual function ato allow extention of the preparation process
	virtual void PrepareCustom();
	//@cmember prepares displays for the drawing of manhattan type graphs
	virtual void PrepareIsoBar();
	//@cmember Prepares the display component for drawing
	virtual void PrepareDisplay();
	//@cmember Calculates scale ranges for most simple graph types.
	virtual void PrepareYScale();
	//@cmember Calculates X and Y scale information for scatter graphs
	virtual void PrepareScatter();
	//@cmember Draws Web char data
	virtual void DrawWebData();
	//@cmember Draws Web chart labels.
	virtual void DrawWebLabels();
	//@cmember Draws Web chart axes.
	virtual void DrawWebAxis();
	//@cmember Draws X,Y scatter data
	virtual void DrawScatterData();
	//@cmember Draws X,Y scatter Labels
	virtual void DrawScatterLabels();
	//@cmember Draws X,Y scatter axes 
	virtual void DrawScatterAxis();
	//@cmember Called to draw stock quote hi-lo-close graphs
	virtual void DrawStockData();
	//@cmember Called to draw labels for the YX graphs
	virtual void DrawYXLabels();
	//@cmember Generates a list of numbers which are used to annotate the Y axis of graphs
	virtual void CreateNumberList(double multiplier,CStringList *list);
	//@cmember Returns the step required to generate numbers for a graph
	virtual double GetIdealScale();
	//@cmember Draws strata graphs
	virtual void DrawStrataData();
	//@cmember Draws the 3D pie chart
	virtual void DrawIsoPieData();
	//@cmember Draws the YX axis for use with horizontal bar charts
	virtual void DrawYXAxis();
	//@cmember Draws horizontal bar data
	virtual void DrawHBarData();
	//@cmember Draws vertical stacked bars
	virtual void DrawStackVBarData();
	//@cmember Draws horizontal stacked bars
	virtual void DrawStackHBarData();
	//@cmember called by label drawing, axis plotting and display routines to size the stage and platform graphs
	virtual int StageCalcSize(CRect *BackPlane);
	//@cmember draws labels and annotations on the stage or platform graphs
	virtual void DrawStageLabels();
	//@cmember Override to draw custom axes
	virtual void DrawCustomAxis();
	//@cmember Override to draw custom labels
	virtual void DrawCustomLabels();
	//@cmember Override to draw custom data
	virtual void DrawCustomData();
	//@cmember Draws "Rooftop" objects.
	virtual void DrawIsoAreaData();
	//@cmember Draws "Manhattan" bars.
	virtual void DrawIsoBarData();
	//@cmember Draws the "stage" type axis
	virtual void DrawStageAxis();
	//@cmember Draws pie chart labels.
	virtual void DrawPieLabels();
	//@cmember Called to draw pie chart data
	virtual void DrawPieData();
	//@cmember Called to draw standard X,Y labels
	virtual void DrawClassicLabels();
	//@cmember Called to draw standard X,Y axes
	virtual void DrawClassicAxis();
	//@cmember Called to draw a line plot of the data
	virtual void DrawLineData();
	//@cmember Called from DrawForeground to display the data representation.
	virtual void DrawData();
	//@cmember Called from DrawForeground to display the axes
	virtual void DrawAxis();
	//@cmember Called from DrawForeground to draw the labels
	virtual void DrawLabels();
	//@cmember Called to draw vertical bar chart data
	virtual void DrawVBarData();
	//@cmember Called to draw pie chart data
	virtual void DrawForeground();
	//@cmember Draws a warning sign for null graphs
	virtual void DrawEmptyGraph();
	//@cmember Returns the X tick list
	virtual CPtrList * GetXTickList(){return &m_XTickList;}
	//@cmember Returns the X tick list
 	virtual CPtrList * GetYTickList(){return &m_YTickList;}
	//@cmember Returns the X tick list
	virtual CPtrList * GetZTickList(){return &m_ZTickList;}
	//@cmember Empties a tick list and deletes its contents
	virtual void KillTickList(CPtrList * pTickList);

};

//@mdata int | SRGraphLabel | m_nLocationPoint | Defines the point at which a label will be anchored. This is not the same as the text position. A label may be fixed to its screen position 
// by any one of nine points. See the overview for this class for details.

//@mdata SRGraphAnnotationList | SRGraphLabel | m_Labels | The multi language annotation list which holds text for this label.
//@rem As with other SRGraphComponent based objects the SRGrapkLabel may display text which differs according to the user defined locale data

//@class Displays labels on graphs and charts in various orientations and without overlapping
// other labels.
class SRGraphLabel : public SRGraphTitle
{
//@access protected data members
protected:
	//@cmember Device independant Y coordinate
	double m_dPosY;
	//@cmember Device independant Y coordinate
	double m_dPosX;
	//@cmember Semaphore signifies that the vector is 0, 90, 180 or 270 degrees.
	BOOL m_bVectorStandard;
	//@cmember Semaphore signifies that the label is oriented to 0, 90, 180 or 270 degrees.
	BOOL m_bLabelStandard;
	//@cmember Determines the movability of this label May move when TRUE (note unimplemented)
	BOOL m_bMoveable;
	//@cmember The direction (-1 to 1) which the label can move in the X axis
	double m_dVectorX;
	//@cmember The direction (-1 to 1) which the label can move in the Y axis
	double m_dVectorY;
	//@cmember The location point code
	int m_nLocationPoint;
	//@cmember A pointer to the parent label block Only valid if the m_bBlockMemeber is set TRUE
	SRGraphLabelBlock * m_pLabelBlock;
	//@cmember designates this label as part of a label block when TRUE
	BOOL m_bBlockMember;
	//@cmember designates use of the font description local to this label when TRUE
	BOOL m_bUseLocalFont;
	//@cmember The screen coordinate of the location point
	CPoint m_TextPosition;
	//@cmember An annotation list containing the coded labels.
	SRGraphAnnotationList m_Labels;

//@access public data members
public:
	virtual int ReadTemplateLine(LPCSTR line);
	// note LocationPoints are arranged so that you may access this list
	// by dividing the number of degrees by 45 to give a relavent outer
	// edge point.

	//@cmember,menum The nine points at which a label can be located and about which it may be rotated
	enum LocationPoints {
		MidLeft,		//@@emem =0
		TopLeft,		//@@emem =1
		TopCenter,		//@@emem =2
		TopRight,		//@@emem =3
		MidRight,		//@@emem =4
		BottomRight,	//@@emem =5
		BottomCenter,	//@@emem =6
		BottomLeft,		//@@emem =7
		MidCenter		//@@emem =8
	};

	//@cmember,menum Standard vectors in which a label may move. These are converted to thier sine and cosine
	//equivalents before being stored
	enum LabelVectors {
		VectorUp,	 //@@emem Label moves up (x=0, y=-1)
		VectorLeft,	 //@@emem Label moves left (x=-1, y=0)
		VectorRight, //@@emem Label moves right (x=1, y=0)
		VectorDown,	 //@@emem Label moves down (x=0, y=1)
		VectorCustom //@@emem Label moves in some other durection
	};

	//@cmember,menum The standard label orientations (measurements in degrees)
	enum LabelStandards {
		Normal,		//@@emem 0	
		Upward,		//@@emem 90
		Inverted,	//@@emem 180
		Downward,	//@@emem 270
		Angled		//@@emem Some value other than the four aboove.
	};


//@access Public member functions
public:
//attributes
	//@cmember Constructor
	SRGraphLabel();
	//@cmember destructor.
	virtual ~SRGraphLabel(){};

	DECLARE_SERIAL(SRGraphLabel);

//operations
	//@cmember Sets text X position
	virtual void SetTextPosX(double dPosX);
	//@cmember Sets text Y position
	virtual void SetTextPosY(double dPosY);
	//@cmember Gets text Y position
	virtual double GetTextPosY();
	//@cmember Gets text Y position
	virtual double GetTextPosX();
	//@cmember Sets text device independant position
	virtual void SetTextPosition(double dXPos,double dYPos);
	//@cmember Retrieves the annotation list for external access
	SRGraphAnnotationList *GetAnnotationList(){return &m_Labels;}
	//@cmember Calculates the pixel position from the device independant position.
	void CalcPos();
	//@cmember To return the text associated with this label.
	virtual CString & GetAnnotation(int code=1){return m_Labels.GetAnnotation(code);}
	//@cmember Returns a pointer to this objects parent block
	virtual SRGraphLabelBlock * GetLabelBlock(){return m_pLabelBlock;}
	//@cmember Returns a pointer to the graph which owns this object
	virtual SRGraph * GetParent();
	//@cmember Returns the width and height of the label in pixels
	virtual CSize GetLabelSize();
	//@cmember Returns the text color
	virtual COLORREF GetTextColor();
	//@cmember Returns the m_bLabelStandard semaphore
	virtual BOOL GetLabelStandard(){return m_bLabelStandard;}
	//@cmember Saves or restores this object via the archive.
	virtual void Serialize(CArchive &ar);
	//@cmember Initializes the pointer to the parent block.
	virtual void SetLabelBlock(SRGraphLabelBlock *block){m_pLabelBlock=block;}
	//@cmember Returns the m_bDrawing semaphore. TRUE if drawing is possible
	virtual BOOL IsDrawing();
	//@cmember Returns the pointer to the CWnd being used to draw the object.
	virtual CWnd * GetpCWnd();

	//@cmember Returns the pointer to the CDC being used to draw the object.
	virtual CDC * GetpDC();
	//@cmember Initializes the vector in which the label will move to avoid other labels.
	virtual void SetVector(double dAngle);
	//@cmember Initialises the angle at which the label is displayed.
	virtual void SetOrientation(double o){GetStyle()->SetZAngle(o);}
	//@cmember Returns the angle at which the label is displayed
	virtual double GetOrientation(){return GetStyle()->GetZAngle();}
	//@cmember Initializes the vector in which the label will move to avoid other labels.
	virtual void SetVector(double x,double y){m_dVectorX=x; m_dVectorY=y; m_bVectorStandard=FALSE;}
	//@cmember Returns the X anti collision vector
	virtual double GetVectorX(){return m_dVectorX;}
	//@cmember Returns the Y anti collision vector
	virtual double GetVectorY(){return m_dVectorY;}
	//@cmember Initialises the point at which the text will anchored
	virtual void SetLocationPoint(int p){m_nLocationPoint=p;}
	//@cmember Returns the point to which the text is anchored
	virtual int GetLocationPoint(){return m_nLocationPoint;}
	//@cmember returns a CRgn object which covers the area that the text occupies
	virtual CRgn* GetBoundingRegion();
 	//@cmember returns Fills an array of points with the area outline that the text occupies
	virtual void GetBoundingPolygon(CPoint *points,int nPointCount=5);
	//@cmember Returns the point at which text must be output to the screen to place it correctly in the desired position.
	virtual CPoint GetTextOrigin();
	//@cmember Returns the coordinates of the labels location point
	virtual CPoint GetTextPosition(){CalcPos();return m_TextPosition;}
	//@cmember Sets the position of the labels location point
	virtual void SetTextPosition(CPoint p);
	//@cmember Sets the position of the labels location point
	virtual void SetTextPosition(int x,int y);
	//@cmember Orients a label according to a standard code.
	virtual void SetLabelStandard(int nLabelStandard);
	//@cmember Sets the label vector to a standard direction using a pre-defined code.
	virtual void SetVector(int nVectorCode);
	//@cmember Returns the font used to display this label.
	virtual CFont * GetFont();
	//@cmember Returns a pointer to the style object
	virtual SRGraphStyle * GetStyle();
	//@cmember Sets the text of the label
	virtual void SetAnnotation(int code,LPCSTR strText);
	//@cmember Sets the text of the label
	virtual void SetAnnotation(LPCSTR strText);
	//@cmember Initialises a semaphore which signifies that this label is a member of a block of labels.
	virtual void SetBlockMember(BOOL b){m_bBlockMember=b;}
	//@cmember Returns the m_bBlockmember semaphore.
	virtual BOOL GetBlockMember(){return m_bBlockMember;}
	//@cmember Initialises the semaphore which signifies that this label manages its own font.
	virtual void SetUseLocalFont(BOOL b){m_bUseLocalFont=b;}
	//@cmember Checks to see if this label manages its own font.
	virtual BOOL GetUseLocalFont(){return m_bUseLocalFont;}
	//@cmember Refreshes font information before drawing.
	virtual BOOL UpdateFont();
	//@cmember Overridden to prevent the labels from performing the base class behaviour.
	virtual void GetParentContent(){}; // overridden to do nothing
	//@cmember User extention function to allow you to define your own fill routine for this label
	virtual void DrawCustomLabelFill();
	//@cmember User extention function to allow you to define your own border drawing routine for this label
	virtual void DrawCustomLabelBorder();
	//@cmember User extention function to allow you to define your own shadow drawing routine for this label
	virtual void DrawCustomLabelShadow();
	//@cmember The main drawing function for this label
	virtual void Draw(CDC *pDC,CWnd *pWnd);
	//@cmember Writes label details to the template
	virtual void WriteSpecificParameters(CStdioFile * file);
	//@cmember writes the object name to thw template
	virtual void WriteName(CStdioFile * file);

protected:
	//@cmember Draws the text of the label
	virtual void DrawForeground();		// Overridden to draw foreground text
	//@cmember fills the background of this label
	virtual void DrawFill();			// overrides from the basic component
	//@cmember Draws the border around the label
	virtual void DrawBorder();
	//@cmember Draws the shadow under the label
	virtual void DrawShadow();
};

//@class Manages blocks of SRGraphLabel objects
//@rem Label blocks may perform anti-collision calculations on labels which they control. 
//If the facility is required then any label found to be overlapping another may optionally 
//be removed or moved in a specific direction.<nl>
//Label blocks are also capable of maintaining a list of points, effectively the points at 
//which each label was displayed, for the positioning of tick marks or grid lines.
class SRGraphLabelBlock : public SRGraphLabel
{
//@access protected data members
protected:
	//@cmember When reset label placement and sizing takes place but no text is output. When set the text is generated.
	BOOL m_bPass1;
	//@cmember Set by the label block when a tick logging request is made but no tick list is provided.
	BOOL m_bUseMyOwnTickList;
	//@cmember When TRUE the label block checks all labels for overlaps and performs anti collision vectoring.
	BOOL m_bCheckOverlaps;
	//@cmember After pass 0 or pass 1 processing this contains the rectangle of screen covered by the label block
	CRect m_BlockRect;
	//@cmember Contains a CSiz which is equivalent to the accumulated un-rotated size of the label block.
	CSize m_TotalBlockSize;
	//@cmember The list of SRGraphLabel objects that this block holds.
	CObList m_LabelList;
	//@cmember Semaphore which enables the logging of label locations in a tick-list
	BOOL m_bLogTickPositions;
	//@cmember A pointer t the tick position list which this label maintains.
	CPtrList* m_pTickPositionList;

//@access public member functions
public:
	virtual void CreateLabels();
	//@cmember Returns the number of labels in this block.
	virtual int GetLabelCount();
	//@cmember Deletes the "tail" label on the label list.
	virtual void RemoveLastLabel();
	//@cmember Removes and deletes all labels in the list
	virtual void KillLabels();
	//@cmember Initializes the pass semaphore to allow pre-draw sizing of the labels or actual output.
	virtual void SetPass(BOOL b){m_bPass1=b;}
	//@cmember Interrogates the m_bPass member
	virtual BOOL GetPass(){return m_bPass1;}
	//@cmember Adds a label or updates a label in an indexed position.
	virtual void AddLabelIndex(LPCSTR LabelText,int nIndex,int nCode=1);
	//@cmember Transfers all label locations to a list for use when drawing grid-lines and ticks.
	virtual void UpdateTickList();
	//@cmember Retrieves a label from the list referenced by index.
	virtual SRGraphLabel * GetLabelIndex(int nIndex);
	//@cmember Adds a label to the tail of the label list.
	virtual void AddLabel(SRGraphLabel *label,BOOL bUseLocalFont=FALSE);
	//@cmember Globally sets the location points for all labels in the block.
	virtual void SetBlockLocationPoints(int nLocationPoint);
	//@cmember Globally sets the orientation angles for all labels in the block.
	virtual void SetBlockOrientations(double dAngle);
	//@cmember "Pegs" all labels in a block along a line between the two points supplied.
	virtual void SetBlockPositions(CPoint StartPoint, CPoint EndPoint);
	//@cmember Globally sets the anti collision vectors for all labels in the block.
	virtual void SetBlockVectors(double dVectorX,double dVectorY);
	//@cmember Adds a label to the list (overload).
	virtual void AddLabel(LPCSTR strLabelText);
	//@cmember Returns the rectangle which is calculated to fully cover the drawn block.
	virtual CRect GetBlockRect(){return m_BlockRect;}
	//@cmember Initializes the pointer to the blocks TickList.
	virtual void SetptrTickPositionList(CPtrList *ptr){m_pTickPositionList=ptr;}
	//@cmember Enables or disables the tick position logging feature.
	virtual void SetLogTickPositions(BOOL b){m_bLogTickPositions=b;}
	//@cmember,mfunc Retrieves the pointer to the tick position list associated with this label block.
	virtual CPtrList *GetptrTickPositionList(){return m_pTickPositionList;}
	//@cmember Enables or disables the blocks anti collision feature.
	virtual void SetCheckOverlaps(BOOL b){m_bCheckOverlaps=b;}
	//@cmember Returns the anti collision test semaphore
	virtual BOOL GetCheckOverlaps(){return m_bCheckOverlaps;}
	//@cmember Constructor	
	SRGraphLabelBlock();
	//@cmember Destructor
	virtual ~SRGraphLabelBlock();

	DECLARE_SERIAL(SRGraphLabelBlock);
	//@cmember Allows storage or retrieval via the archive.
	virtual void Serialize(CArchive &ar);

enum BlockTitle {
	None,
	Above,
	Right,
	Below,
	Left
};

//@cmember,menum May be used instead of a positive integer index to return the first or last label on the list.
enum LabelPositions {
	LabelFirst=-1,	//@@emem Get the first on the list -1
	LabelLast=-2	//@@emem Get the last -2
};


//@access protected member functions
protected:
	//@cmember Overridden from the base class to prevent the drawing of a component panel.
	void DrawBorder(){};
	//@cmember Overridden from the base class to prevent the drawing of a component panel.
	void DrawFill(){};
	//@cmember Overridden from the base class to prevent the drawing of a component panel.
	void DrawShadow(){};
	//@cmember Sizes or draws the text with anti collision if enabled.
	void DrawForeground();

//@access Protected data members
protected:
	int m_nBlockTitle;
};


//@mdata SRGraphDisplay * | SRGraphLegend | m_pDisplay | When a legend finds itself immediately after a display in the component list it will lock itself to and display legend information tailiored to that display type. 
//The locked legend will examine the scope of the graph display in order to find out what range of daya needs keys etc.

//@mdata CRect | SRGraphLegend | m_LegendRect | Used to keep track of the automatically re-sized legend's extents.

//@mdata SRGraphRect | SRGraphLegend | m_RectStore | At draw time a legend may re-size itself or move itself. In order to keep the setup true to the requested size and position the SRGraphRect for this object is stored here during draw cycles. 
//The information will be replaced after the draw is complete so that the graph remains the same as the requested template or setup data




//@class Derived from SRGraphTitle. Handles painting of the legend panel and inherits the SRGraphTitle class font management.
class SRGraphLegend : public SRGraphTitle
{
//@access Protected data members
protected:
	//@cmember stores the height of the key items between passes.
	int m_nKeyHeight;
	//@cmember Keeps a copy of the device independant rectangle so that the original settings can be restored after each draw cycle.
	SRGraphRect m_RectStore;
	//cmember A label block list of text items used to annotate the legend keys.
	SRGraphLabelBlock m_LegendLabels;
	//@cmember A pointer to the SRGraphDisplay to which this legend has locked itself. Do Not alter.
	SRGraphDisplay * m_pDisplay;
	//@cmember Basic rectangle of the legend panel
	CRect m_LegendRect;
	//@cmember Calculated rectangle of the text size in the panel. 
	CRect m_TextRect;
	//@cmember Semaphore to prevent or allow the auto re-size to move the horizontal origin of the rectangle
	BOOL m_bKeepTop;
	//@cmember Semaphore to prevent or allow the auto re-size to move the vertical origin of the rectangle
	BOOL m_bKeepLeft;
	//@cmember Semaphore to prevent or allow the auto re-size to change the height of the component
	BOOL m_bAutoSizeX;
	//@cmember Semaphore signifying a demand to re-calculate the component size. see <mf SRGraphLegend::DrawForeground>
	BOOL m_bSizeForeground;
	//@cmember TRUE if the legend is to automatically extend its bottom boundary to contain the list.
	BOOL m_bAutoSizeY;
	//@cmember Width of key data
	//@devnote a key is the pattern or color filled box which accompanies the text in the legend panel. 
	//heights for keys are calculated according to the height of text used. Widths are decided by you or 
	//the user via the template or explicit initialisation of the m_dKeySize member.
	double m_dKeySize;
	//@cmember A count of the number of keys currently in this legend.
	int m_nKeyCount;
	

//@access Protected member functions
protected:
	//@Cmember Draws text or re calculates the size of the component.
	virtual void DrawForeground();
	//@cmember Draws legend keys onto the component
	virtual void DrawKey(CPoint topleft,CPoint bottomright, int index, int group, int CurrentKey);
	//@cmember Calculates size of graphic keys
	virtual int CalcKeySize();
	//@cmember Customisation system for legend keys
	virtual void DrawCustomKey(CPoint topleft,CPoint bottomright,int group,int index,SRGraphStyle *ptrStyle);


//@access public member functions
public:
	//@cmember Writes special legend template parameters
	virtual void WriteSpecificParameters(CStdioFile *f);
	//@cmember Reads a legend template line
	virtual int ReadTemplateLine(LPCSTR line);
	//@cmember Destructor.
	virtual ~SRGraphLegend();
	//@cmember Writes the objects template name to the template file.
	virtual void WriteName(CStdioFile *file);
	//@cmember Initialises the size of graphic keys
	void SetKeySize(double keysize);
	//@cmember initialises the auto-sizing flags.
	virtual void SetAutoSize(int sizebits);
	//@cmember Overrides <mf SRGraphComponent::Draw> to enable auto re-sizing
	virtual void Draw(CDC *pDC,CWnd *pCWnd);
	//@cmember Retrieves pertinent data from the parent <c SRGraph>
	virtual void GetParentContent();
	//@cmember Constructor
	SRGraphLegend();

	DECLARE_SERIAL(SRGraphLegend);
	//@cmember To store or retrieve information via a CArchive
	virtual void Serialize(CArchive& ar); // not used at this time
};



#endif //__GRAPH_COMPONENT__

// Reset DLL declaration context
#undef AFX_DATA
#define AFX_DATA
#undef AFXAPP_DATA
#define AFXAPP_DATA NEAR
