//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////
// SRGraphData implementation file
//
// copyright Stingray Software 1996 
//
// Written by Bob Powell (bob@stingsoft.com)
//

#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif


#include "stdafx.h"


#include <math.h>
#include <memory.h>

#ifdef _DEBUG

#define new DEBUG_NEW

#endif


IMPLEMENT_SERIAL(SRGraphAnnotation,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphAnnotationList,CObList,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphData,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphDataList,CObList,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphDynamicData,SRGraphData,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphStyle,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(CScale,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);



/////////////////////////////////////////////////////////////
// SRGraphAnnotation member functions

//@doc
//@module SRGraphData.cpp | SRGraphData Implementation file

/////////////////////////////////////////////////////////////
// SRGraphAnnotation member function implementations

//@mfunc Constructor with initialization of data members
//@parm A country code identifying the language of the annotation.
//@parm The annotation text
//@see <mf SRGraphAnnotation::SetText> <mf SRGraphAnnotation::GetText> <md SRGraphAnnotation::m_nCode> <md SRGraphAnnotation::m_strAnnotation>
SRGraphAnnotation::SRGraphAnnotation(int code, LPCSTR text)
{
	m_nCode=code;
	m_strAnnotation=text;
}


//@mfunc Serialize member function
//@parm Reference to the CArchive used to store this data
void SRGraphAnnotation::Serialize(CArchive &ar)
{
	CObject::Serialize(ar);

	if(ar.IsStoring())
	{
		// write data to the archive
		ar << m_nCode << m_strAnnotation;
	}
	else
	{
		// read the data from the archive
		ar >> m_nCode >> m_strAnnotation;
	}
}



#ifdef _DEBUG
//@mfunc Dumps the contents of this object
//@parm Reference to a CDumpContext
void SRGraphAnnotation::Dump(CDumpContext &dc) const
{
	CObject::Dump(dc); // dump this object's base
	dc << "Annotation data: country=" << m_nCode << " text=" << m_strAnnotation << "/r/n"; // and the members for the derived class
}

#endif

//@mfunc To query the m_nCode protected data member
//@rdesc Integer containing the international dialling code of the annotation
//@see <mf SRGraphAnnotation::SetCode> <md SRGraphAnnotation::m_nCode>
int SRGraphAnnotation::GetCode()
	{
		return m_nCode;
	}

//@mfunc To query the m_strAnnotation protected data member
//@rdesc Refefence to a CString containing the text of the annotation
//@see <mf SRGraphAnnotation::SetText> <md SRGraphAnnotation::m_strAnnotation>
CString& SRGraphAnnotation::GetText()
{
	return m_strAnnotation;
}  

//@mfunc To set the m_nCode protected data member
//@parm The country code (ie 01=USA, 44=UK)
//@see <mf SRGraphAnnotation::GetCode> <md SRGraphAnnotation::m_nCode>
void SRGraphAnnotation::SetCode(int code)
{
	m_nCode=code;
} 

//@mfunc To set the m_strAnnotation protected data member
//@parm CString object containing the text to set
//@see <mf SRGraphAnnotation::GetText> <md SRGraphAnnotation::m_strAnnotation>
void SRGraphAnnotation::SetText(CString s)
{
	m_strAnnotation=s;
} 


/////////////////////////////////////////////////////
// SRGraphAnnotationList member functions

//@mfunc Clean up and destruction of the list.
//@devnote It is important to remember that CObList and derivatives
// will not delete the data pointed to. Instead it just rudely disconnects their references and leaves
// them in memory. This will cause memory leaks if not attended to so in the destructor of all list and array
// based objects we have a clean-up routine which repeatedly deletes the top element from the list until the list is empty
SRGraphAnnotationList::~SRGraphAnnotationList()
{
	while(!IsEmpty())
		delete ((SRGraphAnnotation *)RemoveHead());
}

//@mfunc Adds or replaces an annotation string with the corresponding country code
//@parm The country code
//@parm The annotation text
//@see <c SRGraphAnnotation>
void SRGraphAnnotationList::SetAnnotation(int code,LPCSTR text)
{
	if(!IsEmpty())
	{ // search for the corresponding code
		POSITION p;
		for(p=GetHeadPosition();p!=NULL;)
		{
			SRGraphAnnotation* ga=(SRGraphAnnotation *)GetNext(p);
			if(code==ga->GetCode())
			{
				ga->SetText(text);
				return; // job done
			}
		}
	}
	// corresponding code was not found or list is empty
	AddTail(new SRGraphAnnotation(code,text));
}

//@mfunc Empties the annotation list of data without destroying the list itself
void SRGraphAnnotationList::KillList(void)
{
	while(!IsEmpty())
	{
		delete (RemoveHead());
	}
}



//@mfunc Handle the storage and retrieval of the object
// CObList serializes all CObject derived classes for us. so the function
// simply calls the base class function
//@parm Reference to the CArchive
void SRGraphAnnotationList::Serialize(CArchive &ar) 
{
	CObList::Serialize(ar); 
}


#ifdef _DEBUG
//@mfunc Dumps the contents of this object to the debugger
//@parm Reference to a CDumpContext 
void SRGraphAnnotationList::Dump(CDumpContext &dc) const
{
	CObList::Dump(dc); // CObList dumps this object's and all of the CObject derived classes held in the pointers
}

#endif

///////////////////////////////////////////////////////////
// SRGraphData member functions
//



//@mfunc Handles storage and retrieval of graph data
void SRGraphData::Serialize(CArchive& ar) //@parm The CArchive object to store to or retrieve from
{
	CObject::Serialize(ar);

	m_strAnnotationList.Serialize(ar);
	m_Style.Serialize(ar);
	m_HiLoValue.Serialize(ar);

	if(ar.IsStoring())
		ar << m_dValue << m_bNULL;
	else
		ar >> m_dValue >> m_bNULL;
}


//@mfunc Constructor with initialisation.
//@parm Value of this graph element
//@parm An annotation string
//@parm The data style flags
SRGraphData::SRGraphData(double Value,CString text,int Style)		
{
	m_dValue=Value;
	m_Style.SetStyle(Style);
	SetAnnotation(text);
}

//@mfunc Overloaded function version 1 sets the annotation in the most comprehensive way 
//by specifying all of the parameters.
void SRGraphData::SetAnnotation(
								int code,		//@parm The country code of this annotation
								LPCSTR text)	//@parm The annotation text
{
	m_strAnnotationList.SetAnnotation(code,text);// bump it to the SRGraphAnnotationList class
}

//@mfunc Overloaded function to set the annotation in the most basic fashion.
//omit the code and clear variables to set the default annotation string.<nl>
//Set clear to TRUE to wipe the existing list and substitute this annotation<nl>
//SetAnnotation("Capital investment") for example would be the simplest method of annotating a graph data element.
//@parm The annotation
//@parm if TRUE then all entries in the existing list are wiped. default=FALSE
//@parm The country code. default=01 (USA)
void SRGraphData::SetAnnotation(LPCSTR text,BOOL clear,int code)		
{
	if(clear)
		m_strAnnotationList.KillList();
	m_strAnnotationList.SetAnnotation(code,text);
}

#ifdef _DEBUG

//@mfunc To dump the object for debug purposes
//@parm Reference to the CDumpContext to dump to.
void SRGraphData::Dump(CDumpContext &dc) const
{
	CObject::Dump(dc);
	m_strAnnotationList.Dump(dc);
	dc << "m_dValue=" << m_dValue;
	dc << "m_HiLoValue =";
	m_HiLoValue.Dump(dc);
	m_Style.Dump(dc);
}

#endif

//@mfunc Construction
//@rem Sets various defaults
SRGraphData::SRGraphData()
{
	m_dValue=0.0;
	m_Style.SetType(CX_TYPE_OBJECT);
	SetNull(TRUE);
}


//////////////////////////////////////////////////////
// SRGraphDataList member functions


//@mfunc Constructor
//@rem Sets various defaults
SRGraphDataList::SRGraphDataList()
{
	m_bDynamic=FALSE;
	m_Style.SetType(CX_TYPE_OBJECT); //make sure that graphics interprets this as an object not a component panel.
}

//@mfunc Handles storage and retrieval of graph data
//@parm The CArchive object to store to or retrieve from
void SRGraphDataList::Serialize(CArchive &ar) 
{
	CObList::Serialize(ar);

	m_strAnnotationList.Serialize(ar);
	m_Style.Serialize(ar);

	if(ar.IsStoring())
		ar << m_bDynamic;
	else
		ar >> m_bDynamic;
}


//@mfunc Constructor with initialisation.
//@parm An annotation string
//@parm The annotation locale code
SRGraphDataList::SRGraphDataList(LPCSTR text,int code)		
{
	SetAnnotation(code,text);
}

//@mfunc Destructor with clean up. <nl>
//@devnote It is important to remember that CObList and derivatives
// will not delete the data pointed to. Instead it just rudely disconnects their references and leaves
// them in memory. This will cause memory leaks if not attended to so in the destructor of all list and array
// based objects we have a clean-up routine which repeatedly deletes the top element from the list until the list is empty
SRGraphDataList::~SRGraphDataList(void)
{
	while(!IsEmpty())
		delete RemoveHead();
}

//@mfunc Overloaded function version 1 sets the annotation in the most comprehensive way 
//by specifying all of the parameters.
void SRGraphDataList::SetAnnotation(
								int code,		//@parm The country code of this annotation
								LPCSTR text)	//@parm The annotation text
{
	m_strAnnotationList.SetAnnotation(code,text);// bump it to the SRGraphAnnotationList class
}

//@mfunc Overloaded function to set the annotation in the most basic fashion.
//omit the code and clear variables to set the default annotation string.<nl>
//Set clear to TRUE to wipe the existing list and substitute this annotation<nl>
//SetAnnotation("Capital investment") for example would be the simplest method of annotating a graph data element.
void SRGraphDataList::SetAnnotation(
								LPCSTR text,	//@parm The annotation
								BOOL clear,		//@parm if TRUE then all entries in the existing list are wiped. default=FALSE
								int code)		//@parm The country code. default=01 (USA)
{
	if(clear)
		m_strAnnotationList.KillList();
	m_strAnnotationList.SetAnnotation(code,text);
}

#ifdef _DEBUG

//@mfunc Dumps data to the CDumpContext
//@parm Reference to a dump context
void SRGraphDataList::Dump(CDumpContext &dc) const
{
	CObList::Dump(dc);
	m_strAnnotationList.Dump(dc);
}

#endif

//@mfunc Returns a pointer to the indexed data. Grows the list if neccesary.
//@parm The zero based index of the desired data
//@parm TRUE to automatically make the list grow to fit the requested index.
SRGraphData *SRGraphDataList::GetIndex(UINT index,BOOL GrowList)
{
	if(IsDynamic())
	{
		// the index returned is a pointer to the SRGraphDynamic data with the 
		//output pointer set to the correct position..
		if(IsEmpty())
		{
			ASSERT(FALSE);
			SRThrowGraphException(SRGraphException::dynamicNotready);//Dynamic lists dont grow..
			return NULL;
		}
		SRGraphDynamicData *ddp=(SRGraphDynamicData *)GetHead();
		if(ddp->GetBufferSize()<index)
		{
			ASSERT(FALSE);
			SRThrowGraphException(SRGraphException::index);
			return NULL;
		}
		ddp->SetOutputPointer(index);
		return ddp;
	}
	else
	{	// the index is the index of the n'th SRGraphData object.
		if((UINT)GetCount()<(index+1))
		{
			if(GrowList)
			{
				while((UINT)GetCount()<(index+1))
					AddTail(new SRGraphData);
			}
			else
			{
				ASSERT(FALSE);
				SRThrowGraphException(SRGraphException::index);
				return NULL;
			}
		}
		POSITION p=FindIndex(index);
		ASSERT(p!=NULL);
		return (SRGraphData *)GetAt(p);
	}
}

//@mfunc Stores numeric data
//@parm  The number to store
//@rem Sets the m_dValue member and also updates the m_HiLoValue member.
void SRGraphData::SetValue(double number)
{
	m_dValue=number;
	m_HiLoValue+=number;
	SetNull(FALSE);
}

//@mfunc Retrieves stored data
//@parm TRUE if the m_dValue data should be converted to the base 10 logarithm, FALSE if no conversion is to be made (default)
//@rdesc The contents of the m_dValue data member with the appropriate conversion applied.
//@rem The conversion to a logarithm allows scale compression for high bandwidth data sets. Without logarithmic conversion 
//a graph with the values 9 and 9,000,000 would be scaled so that the value 9 was too small to register on the plot. With logarithmic 
//conversion however the data would be scaled so that all elements would be clearly visible against an appropriately scaled axis.
//@see <mf SRGraph::SetLog> <mf SRGraph::GetLog>
double SRGraphData::GetValue(BOOL bLogData)
{
	if(bLogData)
		return log10(fabs(m_dValue));
	else
		return m_dValue;
}

//@mfunc void | CScale | total | Private function Adds this value to the scale. If it is more than m_dMax or less than m_dMin it updates the relavent value.
//@parm double | n | The data to be recorded
void CScale::total(double n)
{
	if(m_bClean)
	{
		m_dMax=m_dMin=n;
		m_bClean=FALSE;
	}
	else
	{
		m_dMax=max(m_dMax,n);
		m_dMin=min(m_dMin,n);
	}
}

//@mfunc Retrieves an annotation from the list
//@parm The country code of the annotation
//@rdesc Refence to a CString object containing the annotation or ""
//@see <c SRGraphAnnotation>
CString& SRGraphAnnotationList::GetAnnotation(int code)
{
	if(IsEmpty())
	{
		m_strEmptyString.Empty(); // just in case its been altered
		return m_strEmptyString;
	}
	POSITION p;
	for(p=GetHeadPosition();p!=NULL;)
	{
		SRGraphAnnotation *pAnnotation=(SRGraphAnnotation *)GetHead();
		if(pAnnotation->GetCode()==code)
			return pAnnotation->GetText();
	}
	m_strEmptyString.Empty();
	return m_strEmptyString;
}

//@mfunc Retrieves an annotation from the list
//@parm The country code of the annotation
//@rdesc Refence to a CString object containing the annotation or ""
CString& SRGraphDataList::GetAnnotation(int code)
{
	return m_strAnnotationList.GetAnnotation(code); 
}

//@mfunc Retrieves an annotation from the list
//@parm The country code of the annotation
//@rdesc Refence to a CString object containing the annotation or ""
CString& SRGraphData::GetAnnotation(int code)
{
	return m_strAnnotationList.GetAnnotation(code); 
}



//@mfunc Constructor. Sets default styles.
SRGraphStyle::SRGraphStyle()
{
	m_dwStyle=CX_FRAME_LINE | CX_INTERIOR_AUTOMATIC | CX_OBJECT_AUTOMATIC;
	m_dwComponentStyle=CX_SOLID_FILL;
	m_dwStyleA=0;
	m_dwStyleB=0;
	m_dwStyleC=CX_KEY_AUTOMATIC;
	m_dMaxWidth=m_dMaxHeight=m_dMaxDepth=10000000; // just a huge number!
	m_RGBColorA=CXCLR_WHITE;
	m_RGBColorB=CXCLR_BLACK;
	m_RGBFrameColor=CXCLR_BLACK;
	m_RGBGridColor=CXCLR_TEAL;
	m_dAngle=0;
	m_nFrameWidth=1;
	SetLabelStyle(CX_LABEL_AUTOMATIC);
	m_dXAngle=30;
	m_dYAngle=m_dZAngle=0;
}

//@mfunc Interrogates the graph style.
//@rdesc Bit masked value representing the graph style data
//@see <mf SRGraphStyle::SetGraphStyle> <md SRGraphStyle::m_dwStyle>
DWORD SRGraphStyle::GetGraphStyle()
{
	return m_dwStyle & CX_GRAPH_STYLE_MASK;
}

//@mfunc Interrogates the axis style. See <f Graph style constants>
//@rdesc Bit masked value representing the axis style
//@see <mf SRGraphStyle::SetAxisStyle> <md SRGraphStyle::m_dwComponentStyle>
DWORD SRGraphStyle::GetAxisStyle()
{
	return m_dwStyle & CX_AXIS_STYLE_MASK;
}

//@mfunc safely sets the graph style bits. Used in <c SRGraphDisplay> objects this command programs the type of 
//chart or graph used. By having more than one display component each programmed
// to a different graph style you may show graphs with multiple overlaying displays or multiple axes.
//@parm the new style value for the graph type
//@see <mf SRGraphStyle::GetGraphStyle> <md SRGraphStyle::m_dwStyle>
void SRGraphStyle::SetGraphStyle(DWORD style)
{
	m_dwStyle &= ~CX_GRAPH_STYLE_MASK;
	style &= CX_GRAPH_STYLE_MASK;
	m_dwStyle |= style;
}

//@mfunc safely sets the axis style bits.
//@parm the new style value for the axis type
//@see <mf SRGraphStyle::GetAxisStyle> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetAxisStyle(DWORD style)
{
	m_dwStyle &= ~CX_AXIS_STYLE_MASK;
	style &= CX_AXIS_STYLE_MASK;
	m_dwStyle |= style;
}

//@mfunc Interrogates the legend style bits. See <f Graph style constants>
//@rdesc The masked legend style
//@see <mf SRGraphStyle::SetLegendStyle> <md SRGraphStyle::m_dwStyleB>
DWORD SRGraphStyle::GetLegendStyle()
{
	return m_dwStyleB & CX_LEGEND_STYLE_MASK;
}

//@mfunc safely sets the legend style bits. This command only applies to legend components and allows the programming of various formats of data key display.
//@parm the new style value for the legend type
//@see <mf SRGraphStyle::GetLegendStyle> <md SRGraphStyle::m_dwStyleB>
void SRGraphStyle::SetLegendStyle(DWORD style)
{
	m_dwStyleB &= ~CX_LEGEND_STYLE_MASK;
	style &= CX_LEGEND_STYLE_MASK;
	m_dwStyleB |= style;
}





//@mfunc Removes the memory allocated to this object on deletion.
SRGraphDynamicData::~SRGraphDynamicData()
{
	if(m_bReady)
		delete (m_pBuffer);
}

//@mfunc Allocates or reallocates a buffer for the data.
//@parm Size of allocation (in SRGDataBuffers)
//@devnote Re-allocation of memory assigned by the new operator is not supported under MFC. 
//Therefore when reallocation is required a process of allocation of a new memory block followed 
//by copying of the old memory block contents followed by the deletion of the original data block 
//takes place. While it is possible to allocate memory without using the new operator this becomes 
//less portable and falls outside of the boundaries of MFC programming.
//@see <mf SRGraphDynamicData::GetBufferSize>
void SRGraphDynamicData::SetBufferSize(UINT size)
{
	if(m_bReady)
	{// a "realloc" is required but its a no-no for MFC so we have to do it ourselves..
		m_bReady=FALSE;
		UINT OldBufferSize=m_nBufferSize;
		SRGDataBuffer *newbuf=new SRGDataBuffer[size];
		CScale *newhilo=NULL;

		if(m_bHiLo)
			newhilo=new CScale[size];
		
		if(newbuf==NULL)
		{	// this error is not a disaster
			AfxMessageBox("SRGraphDynamicData reallocation error",MB_ICONEXCLAMATION);
			m_bReady=TRUE;
			return;
		}

		m_nBufferSize=size;
		for(UINT c=0;c<m_nBufferSize && c<OldBufferSize;c++)
		{
			newbuf[c]=m_pBuffer[c];
			if(m_bHiLo)
			  newhilo[c]=m_HiLoBuffer[c];
		}
		delete (m_pBuffer);
		delete (m_HiLoBuffer);

		m_pBuffer=newbuf;
		m_HiLoBuffer=newhilo;

		m_bReady=TRUE;
		m_nBufferSize=size;
	}
	else
	{
		m_pBuffer=new SRGDataBuffer[size];
		if(m_pBuffer==NULL)
		{
			AfxMessageBox("SRGraphDynamicData allocation error",MB_ICONSTOP);
			return;
		}
		m_nBufferSize=size;
		if(m_bHiLo)
		{ // if this data needs hi-lo records assign storage
			m_HiLoBuffer=new CScale[size];
			if(m_HiLoBuffer==NULL)
			{
				AfxMessageBox("SRGraphDynamicData HiLo buffer allocation error",MB_ICONSTOP);
				return;
			}

		}
		m_bReady=TRUE;
	}
}

//@mfunc Places character values into the buffer
//@parm The character value
//@rem The character is placed into the buffer and the input pointer updated (incremented by 1) 
//If the pointer overflows the bounds of the buffer there are three courses of action<nl>
//1..In the case that the buffer is cyclic the pointer is reset to zero<nl>
//2..If the buffer is not cyclic and the grow size is greater than 1 a reallocation of the buffer will take blace with more data<nl>
//3..An exception will be thrown
void SRGraphDynamicData::SetValue(char value)
{
	if(m_nDataType!=-1 && m_nDataType!=SRGD_CHAR)
		SRThrowGraphException(SRGraphException::type);
	if(!m_bReady)
		SRThrowGraphException(SRGraphException::dynamicNotready);
	if(m_bHiLo)
	{
		m_HiLoBuffer[m_nInIndex]+=value;
	}
	m_nDataType=SRGD_CHAR;
	m_pBuffer[m_nInIndex++].c=value;
	if(m_nInIndex==m_nBufferSize)
	{
		if(m_bCyclic)
		{
			m_nInIndex=0;
		}
		else
		{
			SetBufferSize(m_nBufferSize+m_nGrowSize);
		}
	}
}

//@mfunc Places integer values into the buffer
//@parm The integer value
//@rem The integer is placed into the buffer and the input pointer updated (incremented by 1) 
//If the pointer overflows the bounds of the buffer there are three courses of action<nl>
//1..In the case that the buffer is cyclic the pointer is reset to zero<nl>
//2..If the buffer is not cyclic and the grow size is greater than 1 a reallocation of the buffer will take blace with more data<nl>
//3..An exception will be thrown
void SRGraphDynamicData::SetValue(int value)
{
	if(m_nDataType!=-1 && m_nDataType!=SRGD_INT)
		SRThrowGraphException(SRGraphException::type);
	m_nDataType=SRGD_INT;
	if(!m_bReady)
		SRThrowGraphException(SRGraphException::dynamicNotready);
	if(m_bHiLo)
	{
		m_HiLoBuffer[m_nInIndex]+=value;
	}
	m_pBuffer[m_nInIndex++].n=value;
	if(m_nInIndex==m_nBufferSize)
	{
		if(m_bCyclic)
		{
			m_nInIndex=0;
		}
		else
		{
			SetBufferSize(m_nBufferSize+m_nGrowSize);
		}
	}
}

//@mfunc Places double precision floating point values into the buffer
//@parm The double precision floating point value
//@rem The value is placed into the buffer and the input pointer updated (incremented by 1) 
//If the pointer overflows the bounds of the buffer there are three courses of action<nl>
//1..In the case that the buffer is cyclic the pointer is reset to zero<nl>
//2..If the buffer is not cyclic and the grow size is greater than 1 a reallocation of the buffer will take blace with more data<nl>
//3..An exception will be thrown
void SRGraphDynamicData::SetValue(double value)
{
	if(!m_bReady)
		SRThrowGraphException(SRGraphException::dynamicNotready);
	if(m_bHiLo)
	{
		m_HiLoBuffer[m_nInIndex]+=value;
	}
	switch(m_nDataType)
	{
	case SRGD_NONE:
	case SRGD_DOUBLE:
		m_nDataType=SRGD_DOUBLE;
		m_pBuffer[m_nInIndex++].d=value;
		break;
	case SRGD_INT:
		if(value>INT_MAX || value<INT_MIN)
			SRThrowGraphException(SRGraphException::dynamicType);
		else
			m_pBuffer[m_nInIndex++].n=(int)value;
		break;
	case SRGD_CHAR:
		if(value>UCHAR_MAX)
			SRThrowGraphException(SRGraphException::dynamicType);
		else
			m_pBuffer[m_nInIndex++].c=(char)value;
		break;
	default:
			SRThrowGraphException(SRGraphException::dynamicType);
	break;
	}
	if(m_nInIndex==m_nBufferSize)
	{
		if(m_bCyclic)
		{
			m_nInIndex=0;
		}
		else
		{
			SetBufferSize(m_nBufferSize+m_nGrowSize);
		}
	}
}


//@mfunc Constructor with initialisation of default values
SRGraphDynamicData::SRGraphDynamicData()
{	// Set up all the default attributes
	m_bReady=FALSE;
	m_nInIndex=m_nOutIndex=0;
	m_bCyclic=FALSE;
	m_nBufferSize=0;
	m_nGrowSize=0;
	m_nDataType=-1;
	m_bHiLo=FALSE;
	m_nDataType=SRGD_NONE;
}

//@mfunc Sets the size by which non cyclic buffers grow.
//@parm The new GrowSize.
//@see <mf SRGraphDynamicData::GetGrowSize> <md SRGraphDynamicData::m_nGrowSize>
void SRGraphDynamicData::SetGrowSize(UINT size)
{
	m_nGrowSize=size;
}

//@mfunc Interrogates the GrowSize variable
//@rdesc the value in the GrowSize variable. Default is 64.
//@see <mf SRGraphDynamicData::SetGrowSize>  <md SRGraphDynamicData::m_nGrowSize>
UINT SRGraphDynamicData::GetGrowSize()
{
	return m_nGrowSize;
}

//@mfunc Interrogates the BufferSize variable
//@rdesc The number of <t SRGDataBuffer> stored in this object.
//@see <mf SRGraphDynamicData::SetBufferSize>	<md SRGraphDynamicData::m_nBuffersize>
UINT SRGraphDynamicData::GetBufferSize()
{
	return m_nBufferSize;
}

//@mfunc Returns the output index to the start of the buffer
//@see <md SRGraphDynamicData::m_nOutIndex> <mf SRGraphDynamicData::SetOutputPointer>
void SRGraphDynamicData::ResetOutputPointer()
{
	m_nOutIndex=0;
}

//@mfunc Fetches a value from the buffer at the m_nOutputIndex position
//@parm TRUE if the m_dValue data should be converted to a logarithm, FALSE if no conversion is to be made (default)
//@rdesc the value at that position cast to a double
//@see <mf SRGraphDynamicData::GetValue>
double SRGraphDynamicData::GetValue(BOOL bLogData)
{
	double value;
	switch(m_nDataType)	
	{
	case SRGD_DOUBLE:
		value=m_pBuffer[m_nOutIndex++].d;
		break;
	case SRGD_INT:
		value=(double)m_pBuffer[m_nOutIndex++].n;
		break;
	case SRGD_CHAR:
		value=(double)m_pBuffer[m_nOutIndex++].c;
		break;
	default:
		return 0;
	}
	if(m_nOutIndex>m_nBufferSize)
		m_nOutIndex=0;
	if(bLogData)
		return log10(fabs(value));
	else
		return value;
}


//@mfunc  Handles the storage and retrieval of this object
//@parm Reference to a CArchive object used in the serialization process
void SRGraphDynamicData::Serialize(CArchive &ar)
{
	SRGraphData::Serialize(ar);
	if(ar.IsStoring())
	{
		ar << m_bCyclic << m_bHiLo << m_bReady << m_nBufferSize <<m_nDataType << m_nGrowSize;
		if(m_bReady)
		{
			for(UINT x=0;x<m_nBufferSize;x++)
			{
				ar << m_pBuffer[x].d;
				if(m_bHiLo)
					m_HiLoBuffer[x].Serialize(ar);
			}
		}
	}
	else
	{
		ar >> m_bCyclic >> m_bHiLo >> m_bReady >> m_nBufferSize >>m_nDataType >> m_nGrowSize;
		if(m_bReady)
		{
			if(m_bHiLo)
				m_HiLoBuffer=new CScale[m_nBufferSize];
			
			m_pBuffer=new SRGDataBuffer[m_nBufferSize];

			for(UINT x=0;x<m_nBufferSize;x++)
			{
				ar >> m_pBuffer[x].d;
				if(m_bHiLo)
					m_HiLoBuffer[x].Serialize(ar);
			}
		}
	}
}


//@mfunc Initialises the m_nOutIndex member. Subsequent data request will come from that position
//@parm The index of the data within the buffer
//@rdesc	TRUE if the function succeeded
//@rem	May throw an SRGraphException if the buffer bounds are overflowed
//@see <md SRGraphDynamicData::m_nOutIndex>
BOOL SRGraphDynamicData::SetOutputPointer(UINT index)
{
	if(index>m_nBufferSize)
	{
		SRThrowGraphException(SRGraphException::index);
		m_nOutIndex=0;
		return FALSE;
	}
	else
		m_nOutIndex=index;
	return TRUE;
}

//@mfunc Enables or disables the hi-lo data gathering
//@parm	TRUE to enable FALSE to disable
//@rem Hi low data gathering automatically updates the CScale object associated 
//with this data item thereby allowing you to keep a record of the maximum and 
//minimum values that were used
//@see <mf SRGraphDynamicData::GetGrowSize>
void SRGraphDynamicData::SetHiLoFunction(BOOL b)
{
	if(b==FALSE)
	{
		if(m_bHiLo)
			delete (m_HiLoBuffer);
	}
	else
	{
		if(!m_bHiLo)
			m_HiLoBuffer=new CScale[m_nBufferSize];
	}

	m_bHiLo=b;
}

//@mfunc Interrogates the m_bHiLo member
//@rdesc TRUE if Hi-Lo gathering is enabled, otherwise FALSE.
//@see <mf SRGraphDynamicData::SetHiLoFunction> <md SRGraphDynamicData::m_bHiLo>
BOOL SRGraphDynamicData::GetHiLoFunction()
{
	return m_bHiLo;
}

//@mfunc Returns the input pointer to the beginning of the data buffer
//@see <mf SRGraphDynamicData::SetInputPointer> 
void SRGraphDynamicData::ResetInputPointer()
{
	m_nInIndex=0;
}


//@mfunc Initialises the m_nInIndex member. Subsequent data updates will go to that position
//@parm The index of the data within the buffer
//@rdesc	TRUE if the function succeeded
//@rem	May throw an SRGraphException if the buffer bounds are overflowed
//@see <md SRGraphDynamicData::m_nInIndex>
BOOL SRGraphDynamicData::SetInputPointer(UINT index)
{
	if(index>m_nBufferSize)
	{
		SRThrowGraphException(SRGraphException::index);
		m_nInIndex=0;
		return FALSE;
	}
	else
		m_nInIndex=index;
	return TRUE;
}

//@mfunc Fetches the maximum and minimum values used in this data object
//@rdesc a CScale object containing the values
CScale SRGraphData::GetHiLoData()
{
	return m_HiLoValue;
}

//@mfunc Fetches the maximum and minimum values used in this data object
//@rdesc a CScale object containing the values from the position accessed by the m_nOutIndex member
//@see <md SRGraphDynamicData::m_HiLoBuffer>
CScale SRGraphDynamicData::GetHiLoData()
{
	if(m_bHiLo && m_bReady)
		return m_HiLoBuffer[m_nOutIndex++];
	else
		return CScale();
}

//@mfunc Sets the semaphore which decides whether buffers cycle or grow
//@parm TRUE to have input and output pointers recycle on overflow, FALSE to expand the buffer
//@see <md SRGraphDynamicData::m_bCyclic>
void SRGraphDynamicData::SetCyclic(BOOL b)
{
	m_bCyclic=b;
}

//@mfunc Sets the flag which allows this list to store dynamic data
//@parm TRUE to allow dynamic data storage
//@rem So that display routines cannot see a difference between dynamic and static data 
//the GetIndex() routine has two methods. If this flag is set then rather than return a pointer 
//to the nth SRGraphData object it returns a pointer to the first SRGraphDynamicData object after 
//first setting the m_nOutputIndex member to the required index.
void SRGraphDataList::SetDynamic(BOOL b)
{
	m_bDynamic=b;
}

//@mfunc Initialises the colors used in filling objects
//@parm RGB value for color A
//@parm RGB value for color B (default = BLACK)
//@see <mf SRGraphStyle::GetColor> <md SRGraphStyle::m_RGBColorA> <md SRGraphStyle::m_RGBColorB>
void SRGraphStyle::SetColor(COLORREF a,COLORREF b)
{
	m_RGBColorA=a; m_RGBColorB=b;
}

//@mfunc Retrieves component style data
//@rdesc The component styles (un adulterated)
//@rem This routine simply returns the contents of the m_dwComponentStyle member variable. If a 
//specific style (ie. Shadow style or border style) is required use the specialised routines.
//@see <mf SRGraphStyle::GetComponentShadowStyle> <mf SRGraphStyle::GetComponentBorderStyle> <mf SRGraphStyle::GetComponentFillStyle>
//@devnote The use of this routine is not recommended. Use the Get and Set routines provided.
DWORD SRGraphStyle::GetComponentStyle()
{
	return m_dwComponentStyle;
}

//@mfunc Initialises the component style with an absolute value
//@parm the combined component style
//@rem NOTE that this flag setting routine is not safe inasmuch as it affects all flags in the m_dwStyle variable 
//for safe initialisation of only the required variables use the specific routines
//@see <mf SRGraphStyle::SetComponentShadowStyle>  <mf SRGraphStyle::SetComponentBorderStyle>  <mf SRGraphStyle::SetComponentFillStyle>
void SRGraphStyle::SetComponentStyle(DWORD style)
{
	m_dwComponentStyle=style;
}

//@mfunc Retrieves component shadow flag data
//@rdesc The bitmasked flags which deal with the shadow styles for component panels
//@see <mf SRGraphStyle::SetComponentShadowStyle> <md SRGraphStyle::m_dwComponentStyle> <mf SRGraphStyle::GetComponentBorderStyle> <mf SRGraphStyle::GetComponentFillStyle>
DWORD SRGraphStyle::GetComponentShadowStyle()
{
	return m_dwComponentStyle & CX_SHADOW_MASK;
}

//@mfunc Retrieves component fillflag data
//@rdesc The bitmasked flags which deal with the styles associated with the filling of component panels
//@see <mf SRGraphStyle::SetComponentFillStyle> <md SRGraphStyle::m_dwComponentStyle>
DWORD SRGraphStyle::GetComponentFillStyle()
{
	return m_dwComponentStyle & CX_FILL_MASK;
}

//@mfunc Retrieves component border information
//@rdesc The bitmasked flags which deal with the styles defining the borders around component panels
//@see <mf SRGraphStyle::SetComponentBorderStyle> <md SRGraphStyle::m_dwComponentStyle>
DWORD SRGraphStyle::GetComponentBorderStyle()
{
	return m_dwComponentStyle & CX_BORDER_MASK;
}


//@mfunc Safe initialisation of the type flags. Types are used to differentiate between components which are used to draw panels etc. and objects which represent single data items.
//@parm The new type
//@see <mf SRGraphStyle::GetType> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetType(DWORD type)
{
	m_dwStyleC &= ~CX_TYPE_MASK;
	type &= CX_TYPE_MASK;
	m_dwStyleC |= type;
}


//@mfunc Retrieves type flag data
//@rdesc The bitmasked flags which deal with the type styles
//@see <mf SRGraphStyle::SetType> <md SRGraphStyle::m_dwStyleA>
DWORD SRGraphStyle::GetType()
{
	return m_dwStyleC & CX_TYPE_MASK;
}

//@mfunc Retrieves interior flag data
//@rdesc The bitmasked flags which deal with the styles used to fill interiors of objects
//@see <mf SRGraphStyle::SetInteriosStyle> <md SRGraphStyle::m_dwStyle>
DWORD SRGraphStyle::GetInteriorStyle()
{
	return m_dwStyle & CX_INTERIOR_STYLE_MASK;
}

//@mfunc retrieves the resource or image name used to fill graphic objects
//@rdesc Reference to the CString object containing the name
//@see <mf SRGraphStyle::SetResourceName> <md SRGraphStyle::m_strImage>
CString & SRGraphStyle::GetResourceName()
{
	return m_strImage;
}

//@mfunc Initialises the resource or image name used to fill graphic objects
//@parm The name
//@see <mf SRGraphStyle::GetResourceName> <md SRGraphStyle::m_strImage>
void SRGraphStyle::SetResourceName(LPCSTR name)
{
	m_strImage=name;
}

//@mfunc Sets the object style. This can be used in the style member of an SRGraphData item to define the type of data object seen on the graph. 
//For example use OBJECT_POINT to display simple points on line graphs or OBJECT_EXPOINT to show a wiget on the point. By default object style 
//is set to OBJECT_AUTOMATIC so that the graph routines can choose an object in context with the graph style. 
//@parm The new style.
//@see <mf SRGraphStyle::GetObjectStyle> <md SRGraphStyle::m_dwStyle>
void SRGraphStyle::SetObjectStyle(DWORD style)
{
	m_dwStyle &= ~CX_OBJECT_STYLE_MASK;
	style &= CX_OBJECT_STYLE_MASK;
	m_dwStyle |= style;
}

//@mfunc Retrieves object flag data
//@rdesc The bitmasked flags which deal with the object styles
//@see <mf SRGraphStyle::SetObjectStyle> <md SRGraphStyle::m_dwStyle>
DWORD SRGraphStyle::GetObjectStyle()
{
	return m_dwStyle & CX_OBJECT_STYLE_MASK;
}

//@mfunc Retrieves frame flag data
//@rdesc The bitmasked flags which deal with the frame styles
//@see <mf SRGraphStyle::SetFrameStyle> <md SRGraphStyle::m_dwStyle>
DWORD SRGraphStyle::GetFrameStyle()
{
	return m_dwStyle & CX_FRAME_STYLE_MASK;

}

//@mfunc Retrieves label flag data
//@rdesc The bitmasked flags which deal with the label styles
//@see <mf SRGraphStyle::SetLabelStyle> <md SRGraphStyle::m_dwStyleA>
DWORD SRGraphStyle::GetLabelStyle()
{
	return m_dwStyleC & CX_LABEL_MASK;
}

//@mfunc Safe initialisation of the label flags. Label styles aply particularly to pie chart displays. You can choose to display data about a pie wedge as any combination of 
//data value, percentage and annotation.
//@parm The flags to update
//@see <mf SRGraphStyle::GetLabelStyle> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetLabelStyle(DWORD style)
{
	m_dwStyleC &= ~CX_LABEL_MASK;
	style &= CX_LABEL_MASK;
	m_dwStyleC |= style;
}

//@mfunc Retrieves highlight flag data
//@rdesc The bitmasked flags which deal with the highlight styles
//@see <mf SRGraphStyle::SetHighlight> <md SRGraphStyle::m_dwStyleA>
DWORD SRGraphStyle::GetHighlight()
{
	return m_dwStyleC & CX_HIGHLIGHT_MASK;
}

//@mfunc Safe initialisation of the highlight style
//@parm The style which is used to highlight pie wedges by exploding them from the pie.
//@see <mf SRGraphStyle::GetHighlight> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetHighlight(DWORD style)
{
	m_dwStyleC &= ~CX_HIGHLIGHT_MASK;
	style &= CX_HIGHLIGHT_MASK;
	m_dwStyleC |= style;
}

//@mfunc Stores a name string to be associated with this object. 
//This name can be used to identify resources or files needed in the display of this data item. 
//For example it is used to identify the polygonal shape used to outline custom shaped bars.
//@parm Pointer to a string containing the text to use.
//@see <mf SRGraphStyle::GetObjectName> <md SRGraphStyle::m_strObjectName>
void SRGraphStyle::SetObjectName(LPCSTR name)
{
	m_strObjectName=name;
}

//@mfunc Returns the name associated with this graphic object. See <mf SRGraphStyle::SetObjectName> for details.
//@rdesc A reference to the m_strObjectName member
//@see <mf SRGraphStyle::SetObjectName> <md SRGraphStyle::m_strObjectName>
CString& SRGraphStyle::GetObjectName()
{
	return m_strObjectName;
}

//@mfunc Sets or resets the stretch mode semaphore held in m_dwComponentStyle. If this semaphore is true then 
//images used to fill the component area will be stretched rather than tiled to cover the required space.
//@parm Boolean TRUE or FALSE
//@see <mf SRGraphStyle::GetStretchMode> <md SRGraphStyle::m_dwComponentStyle>
void SRGraphStyle::SetStretchMode(BOOL m)
{
	m_dwComponentStyle &= ~CX_STRETCH_MASK;
	m_dwComponentStyle |=	(m ? CX_STRETCH_MASK : 0);
}

//@mfunc Returns the value of the stretch mode semaphore held in m_dwComponentStyle. 
//When TRUE images used to cover the filled backgrounds are stretched to fill the space rather than tiled.
//@rdesc Boolean TRUE or FALSE
//@see <mf SRGraphStyle::SetStretchMode> 
BOOL SRGraphStyle::GetStretchMode()
{
	return m_dwComponentStyle & CX_STRETCH_MASK;
}

//@mfunc returns the total range of values covered by this scale
//@rdesc m_dMax-m_dMin
//@see <md CScale::m_dMax> <md CScale::m_dMin>
double CScale::Range()
{
	return m_dMax-m_dMin;
}

//@mfunc Sets the style of the data items interior fill.
//@parm The fill style. May be one of..<nl>
//CX_INTERIOR_NONE <nl>
//CX_INTERIOR_AUTOMATIC <nl>
//CX_INTERIOR_COLOR <nl>
//CX_INTERIOR_VSHADE <nl>
//CX_INTERIOR_HSHADE <nl>
//CX_INTERIOR_ASHADE <nl>
//CX_INTERIOR_RESOURCE	<nl>
//CX_INTERIOR_IMAGE <nl>
//CX_INTERIOR_PATTERN	<nl>
//@see <mf SRGraphStyle::GetInteriorStyle> <md SRGraphStyle::m_dwStyle>
void SRGraphStyle::SetInteriorStyle(DWORD style)
{
	m_dwStyle &= ~CX_INTERIOR_STYLE_MASK;
	style &= CX_INTERIOR_STYLE_MASK;
	m_dwStyle |= style;
}

//@mfunc Sets the frame style for the data item.
//@parm the style to set. May be one of..<nl>
//FRAME_NONE <nl>
//FRAME_AUTOMATIC <nl>
//FRAME_LINE <nl>
//FRAME_PATTERN	<nl>
//FRAME_COLOR <nl>
//@see <mf SRGraphStyle::GetFrameStyle> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetFrameStyle(DWORD style)
{
	m_dwStyle &= ~CX_FRAME_STYLE_MASK;
	style &= CX_FRAME_STYLE_MASK;
	m_dwStyle |= style;
}

//@mfunc Sets the style used to fill the interior of component panels.
//@parm The style. May be one of..<nl>
//INTERIOR_NONE	<nl>
//INTERIOR_AUTOMATIC <nl>
//INTERIOR_COLOR<nl>	
//INTERIOR_VSHADE<nl>	
//INTERIOR_HSHADE<nl>	
//INTERIOR_ASHADE<nl>	
//INTERIOR_RESOURCE	<nl>
//INTERIOR_IMAGE<nl>	
//INTERIOR_PATTERN <nl>
//@see <mf SRGraphStyle::GetComponentFillStyle> <md SRGraphStyle::m_dwComponentStyle>
void SRGraphStyle::SetComponentFillStyle(DWORD style)
{
	m_dwComponentStyle &= ~CX_FILL_MASK;
	style &= CX_FILL_MASK;
	m_dwComponentStyle |= style;
}

//@mfunc Sets the style used to draw the borders of component panels.
//@parm The style. May be one of..<nl>
//CX_NO_BORDER				<nl>
//CX_THIN_RAISED_BORDER		<nl>
//CX_THIN_SUNKEN_BORDER		<nl>
//CX_THIN_3D_BORDER			<nl>
//CX_THICK_RAISED_BORDER		<nl>
//CX_THICK_SUNKEN_BORDER		<nl>
//CX_THICK_3D_BORDER			<nl>
//CX_LINE_BORDER				<nl>
//@see <mf SRGraphStyle::GetComponentBorderStyle> <md SRGraphStyle::m_dwComponentStyle>
void SRGraphStyle::SetComponentBorderStyle(DWORD style)
{
	m_dwComponentStyle &= ~CX_BORDER_MASK;
	style &= CX_BORDER_MASK;
	m_dwComponentStyle |= style;
}

//@mfunc Sets the style used to draw the shadows of component panels.
//@parm The style. May be one of..<nl>
//CX_NO_SHADOW				<nl>
//CX_THIN_SHADOW			<nl>
//CX_MEDIUM_SHADOW			<nl>
//CX_THICK_SHADOW			<nl>
//@see <mf SRGraphStyle::GetComponentShadowStyle> <md SRGraphStyle::m_dwComponentStyle>
void SRGraphStyle::SetComponentShadowStyle(DWORD style)
{
	m_dwComponentStyle &= ~CX_SHADOW_MASK;
	style &= CX_SHADOW_MASK;
	m_dwComponentStyle |= style;
}

//@mfunc Gets the maximum depth of data items. Used to limit the stretching of bars etc.
//@rdesc The contents of the m_dMaxDepth member
//@see <mf SRGraphStyle::SetMaxObjectDepth> <md SRGraphStyle::m_dMaxDepth>
double SRGraphStyle::GetMaxObjectDepth()
{
	return m_dMaxDepth;
}

//@mfunc Gets the maximum width of data items. Used to limit the stretching of bars etc.
//@rdesc The contents of the m_dMaxWidth member
//@see <mf SRGraphStyle::SetMaxObjectWidth> <md SRGraphStyle::m_dMaxWidth>
double SRGraphStyle::GetMaxObjectWidth()
{
	return m_dMaxWidth;
}

//@mfunc Gets the maximum height of data items. Used to limit the stretching of bars etc.
//@rdesc The contents of the m_dMaxHeight member
//@see <mf SRGraphStyle::SetMaxObjectHeight> <md SRGraphStyle::m_dMaxHeight>
double SRGraphStyle::GetMaxObjectHeight()
{
	return m_dMaxHeight;
}

//@mfunc Sets the "use max size" semaphore in m_dwStyleA. If this is true then the maximum size
// that bars and other data items can stretch to will be limited by the m_dMaxHeight, m_dMaxDepth and m_dMaxWidth members
//@parm Boolean TRUE or FALSE
//@see <mf SRGraphStyle::GetUseMaxSize> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetUseMaxSize(BOOL flag)
{
	m_dwStyleA &= ~CX_SIZE_MASK;
	m_dwStyleA |= flag ? CX_SIZE_MASK : 0;
}

//@mfunc Interrogates the "use max size" semaphore which prevents bars and other objects from growing larger than a pre-set maximum.
//@rdesc TRUE if the size is to be limited. Otherwise FALSE
//@see <mf SRGraphStyle::SetUseMaxSize> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetUseMaxSize()
{
	return m_dwStyleA & CX_SIZE_MASK ? TRUE : FALSE;
}

//@mfunc Interrogates the m_RGBColorA or m_RGBColorB members.
//@parm FALSE to retrieve the m_RGBColorA data. TRUE to retreive the m_RGBColorB information.
//@rdesc a COLORREF taken from the requested data member.
//@see <mf SRGraphStyle::SetColor> <md SRGraphStyle::m_RGBColorA> <md SRGraphStyle::m_RGBColorB>
COLORREF SRGraphStyle::GetColor(BOOL GetColorB)
{
	return GetColorB ? m_RGBColorB : m_RGBColorA;
}

//@mfunc Interrogates the m_RGBFrameColor member. This color is used to draw the frames of bars etc.
//@rdesc a COLORREF containing the requested color
//@see <mf SRGraphStyle::SetFrameColor> <md SRGraphStyle::m_RGBFrameColor>
COLORREF SRGraphStyle::GetFrameColor()
{
	return m_RGBFrameColor;
}

//@mfunc Returns the axis fill style. For details see <mf SRGraphStyle::SetAxisFillStyle>
//TRUE if axes are to be filled with the m_RGBColorA value.
//@see <mf SRGraphStyle::SetAxisFillStyle> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetAxisFillStyle()
{
	return (m_dwStyleA & CX_FILL_AXIS_MASK) ? TRUE : FALSE;
}

//@mfunc Sets the semaphore in m_dwStyleA which decides whether areas covered by axes are filled
//or simply framed. If the style is TRUE then axes will be painted with the color stored in the m_RGBColorA member. 
//if False then the will only be framed with the color set in the m_RGBFrameColor member.
//@parm TRUE to fill the area covered by the axis. Otherwise FALSE.
//@see <mf SRGraphStyle::GetAxisFillStyle> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetAxisFillStyle(BOOL style)
{
	m_dwStyleA &= ~CX_FILL_AXIS_MASK;
	m_dwStyleA |= style ? CX_FILL_AXIS_MASK : 0;
}

//@mfunc Sets the object stretching limit for bars etc.
//@parm the maximum attainable size.
//@see <mf SRGraphStyle::GetMaxObjectWidth> <mf SRGraphStyle::SetMaxObjectDepth> <mf SRGraphStyle::SetMaxObjectHeight> <md SRGraphStyle::m_dMaxWidth>
void SRGraphStyle::SetMaxObjectWidth(double size)
{
	m_dMaxWidth=size;
}

//@mfunc Sets the object stretching limit for bars etc. For example the Manhattan graph uses this value to prevent the graph being 
//stretched to the whole width of the window if such stretching would make the bars wider than this setting.
//@parm the maximum attainable size.
//@see <mf SRGraphStyle::GetMaxObjectDepth> <mf SRGraphStyle::SetMaxObjectWidth> <mf SRGraphStyle::SetMaxObjectHeight> <md SRGraphStyle::m_dMaxDepth>
void SRGraphStyle::SetMaxObjectDepth(double size)
{
	m_dMaxDepth=size;
}

//@mfunc Sets the object stretching limit for bars etc.
//@parm the maximum attainable size.
//@see <mf SRGraphStyle::GetMaxObjectHeight> <mf SRGraphStyle::SetMaxObjectWidth> <mf SRGraphStyle::SetMaxObjectDepth> <md SRGraphStyle::m_dMaxHeight>
void SRGraphStyle::SetMaxObjectHeight(double size)
{
	m_dMaxHeight=size;

}

//@mfunc Sets the color used to draw frames around graphic data items.
//@parm a COLORREF containing the RGB color
//@see <mf SRGraphStyle::GetFrameColor> <md SRGraphStyle::m_RGBFrameColor>
void SRGraphStyle::SetFrameColor(COLORREF color)
{
	m_RGBFrameColor=color;
}

//@mfunc Interrogates the shade style semaphore in m_dwStyleA. If true then objects such as 3D isographic bars
//used in the "Manhattan" graphs etc will be shaded to provide a more pleasing 3D effect. One side panel will be
//shaded and the top panels will have reflective glare added.<nl>
//{bmc shaded.bmp}
//@rdesc TRUE if objects require shading otherwise FALSE.
//@see <mf SRGraphStyle::SetObjectShadeStyle> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetObjectShadeStyle()
{
	return m_dwStyleA & CX_SHADE_OBJECT_MASK ? TRUE : FALSE;
}

//@mfunc Sets the shade style semaphore in m_dwStyleA. See <mf SRGraphStyle::GetObjectShadeStyle> for details.
//@parm TRUE to turn shading on. FALSE to turn it off.
//@see <mf SRGraphStyle::GetObjectShadeStyle> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetObjectShadeStyle(BOOL style)
{
	m_dwStyleA &= ~CX_SHADE_OBJECT_MASK;
	m_dwStyleA |= style ? CX_SHADE_OBJECT_MASK : 0;
}

//@mfunc Resets the data and the corresponding scale object to zero
void SRGraphData::Zero()
{
	 m_dValue=0;
	 m_HiLoValue.Zero();
}

//@mfunc Used for scale compression in graphs which hold wide bandwidth data
//@rdesc the base 10 logartithm of the value stored.
double SRGraphData::GetLogValue()
{
	return log10(m_dValue);
}

//@mfunc Sets the logarithmic scaling semaphore	defined in m_dwStyleA
//@parm TRUE to turn on log scaling. (default)<nl>FALSE to turn off log scaling.
//@see <mf SRGraphStyle::GetLog> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetLog(BOOL logFlag)
{
	m_dwStyleA &= ~CX_LOG_SCALE_MASK;
	m_dwStyleA |= logFlag ? CX_LOG_SCALE_MASK : 0;
}

//@mfunc Interrogates the log scaling semaphore defined in m_dwStyleA
//@rdesc TRUE if flag is set, FALSE if flag is reset.
//@see <mf SRGraphStyle::SetLog> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetLog()
{
	return m_dwStyleA & CX_LOG_SCALE_MASK ? TRUE : FALSE;
}


//@mfunc Serializes this object.
//@parm A reference to a CArchive object used in the serialization process.
void SRGraphStyle::Serialize(CArchive &ar)
{
	CObject::Serialize(ar);

	if(ar.IsStoring())
	{
		ar << m_dwStyle << m_dwStyleA << m_dwStyleB << m_dwComponentStyle;
		ar << (DWORD)m_RGBColorA << (DWORD)m_RGBColorB << (DWORD)m_RGBFrameColor;
		ar << m_dAngle << m_dMaxDepth << m_dMaxHeight << m_dMaxWidth;
		ar << m_nFrameWidth;
		ar << m_strImage << m_strObjectName;
		ar << m_dwStyleC;
		ar << (DWORD)m_RGBGridColor;
		ar << m_dXAngle << m_dYAngle << m_dZAngle;
	}
	else
	{
		ar >> m_dwStyle >> m_dwStyleA >> m_dwStyleB >> m_dwComponentStyle;
		ar >> (DWORD)m_RGBColorA >> (DWORD)m_RGBColorB >> (DWORD)m_RGBFrameColor;
		ar >> m_dAngle >> m_dMaxDepth >> m_dMaxHeight >> m_dMaxWidth;
		ar >> m_nFrameWidth;
		ar >> m_strImage >> m_strObjectName;
		ar >> m_dwStyleC;
		ar >> m_RGBGridColor;
		ar >> m_dXAngle >> m_dYAngle >> m_dZAngle;
	}
}



//@mfunc void | CScale | Serialize | Saves or retrieves this object via serialization
//@parm CArchive & | ar | Reference to a CArchive object used in loading or storing
void CScale::Serialize(CArchive &ar)
{
	
	CObject::Serialize(ar);

	if(ar.IsStoring())
	{
		ar << m_dMax << m_dMin << m_bClean;
	}
	else
	{
		ar >> m_dMax >> m_dMin >> m_bClean;
	}

}


//@mfunc | CScale | CScale | Constructs a CScale object and assigns values by copying
//@parm CScale & | s | Reference to a CSale object who's values are assigned to this new object

CScale::CScale(const CScale& s)
{
	m_dMax=s.m_dMax;
	m_dMin=s.m_dMin;
	m_bClean=s.m_bClean;
}


void SRGraphStyle::operator =(const SRGraphStyle &s)
{
	m_dMaxHeight=s.m_dMaxHeight;
	m_dMaxDepth=s.m_dMaxDepth;
	m_dMaxWidth=s.m_dMaxWidth;
	m_dAngle=s.m_dAngle;
	m_dXAngle=s.m_dXAngle;
	m_dYAngle=s.m_dYAngle;
	m_dZAngle=s.m_dZAngle;
	m_dwStyle=s.m_dwStyle;
	m_dwStyleA=s.m_dwStyleA;
	m_dwStyleB=s.m_dwStyleB;
	m_dwStyleC=s.m_dwStyleC;
	m_dwComponentStyle=s.m_dwComponentStyle;
	m_RGBGridColor=s.m_RGBGridColor;
	m_RGBFrameColor=s.m_RGBFrameColor;
	m_RGBColorA=s.m_RGBColorA;
	m_RGBColorB=s.m_RGBColorB;
	m_nFrameWidth=s.m_nFrameWidth;
	m_strFramePattern=s.m_strFramePattern;
	m_strObjectName=s.m_strObjectName;
	m_strImage=s.m_strImage;
}



#ifdef _DEBUG

//@mfunc Dumps this object for debug purposes
//@parm The dump context
void SRGraphStyle::Dump(CDumpContext &dc) const
{
	CObject::Dump(dc);

#ifdef __SRGRAPH_DEBUG_VERBOSE__

	dc <<  "m_dMaxHeight=" << m_dMaxHeight;
	dc <<  "m_dMaxDepth=" << m_dMaxDepth;
	dc <<  "m_dMaxWidth=" << m_dMaxWidth;
	dc <<  "m_strObjectName=" << m_strObjectName;
	dc <<  "m_dwStyle=" << m_dwStyle;
	dc <<  "m_dwStyleA=" << m_dwStyleA;
	dc <<  "m_dwStyleB=" << m_dwStyleB;
	dc <<  "m_dwStyleC=" << m_dwStyleC;
	dc <<  "m_dwComponentStyle=" << m_dwComponentStyle;
	dc <<  "Frame style=" << (GetFrameStyle()>>CX_FRAME_SHIFT);
	dc <<  "Object style=" << (GetObjectStyle()>>CX_OBJECT_SHIFT);
	dc <<  "Axis=" << (GetAxisStyle()>>CX_AXIS_SHIFT);
	dc <<  "Graph=" << (GetGraphStyle()>>CX_GRAPH_SHIFT);
	dc <<  "Legend" << (GetLegendStyle()>>CX_LEGEND_SHIFT);
	dc <<  "Interior=" << (GetInteriorStyle()>>CX_INTERIOR_SHIFT);
	dc <<  "Border=" << (GetComponentBorderStyle()>>CX_BORDER_SHIFT);
	dc <<  "Shadow=" << (GetComponentShadowStyle()>>CX_SHADOW_SHIFT);
	dc <<  "Fill=" << (GetComponentFillStyle()>>CX_FILL_SHIFT);
	dc <<  "Type=" << (GetType()>>CX_TYPE_SHIFT);
	dc <<  "Label=" << (GetLabelStyle()>>CX_LABEL_SHIFT);
	dc <<  "Highlight=" << (GetHighlight()>>CX_HIGHLIGHT_SHIFT);
	dc <<  "Use Max Size=" << (GetUseMaxSize() ? "TRUE" : "FALSE");
	dc <<  "Fill axis=" << (GetAxisFillStyle() ? "TRUE" : "FALSE");
	dc <<  "Shade object=" << (GetObjectShadeStyle() ? "TRUE" : "FALSE");
	dc <<  "Log scale=" << (GetLog() ? "TRUE" : "FALSE");
	dc <<  "m_RGBFrameColor=RGB(" << GetRValue(m_RGBFrameColor) << "," << GetGValue(m_RGBFrameColor) << "," << GetBValue(m_RGBFrameColor) << ")";
	dc <<  "m_strFramePattern=" <<	m_strFramePattern;
	dc <<  "m_nFrameWidth=" << m_nFrameWidth;
	dc <<  "m_RGBColorA=RGB(" << GetRValue(m_RGBColorA) << "," << GetGValue(m_RGBColorA) << "," << GetBValue(m_RGBColorA) << ")";
	dc <<  "m_RGBColorB=RGB(" << GetRValue(m_RGBColorB) << "," << GetGValue(m_RGBColorB) << "," << GetBValue(m_RGBColorB) << ")";
	dc <<  "m_dAngle=" << m_dAngle;
	dc <<  "m_strImage="
	m_strImage.Dump(dc);

#else

	dc <<  "m_dMaxHeight=" << m_dMaxHeight;
	dc <<  "m_dMaxDepth=" << m_dMaxDepth;
	dc <<  "m_dMaxWidth=" << m_dMaxWidth;
	dc <<  "m_strObjectName=" << m_strObjectName;
	dc <<  "m_dwStyle=" << m_dwStyle;
	dc <<  "m_dwStyleA=" << m_dwStyleA;
	dc <<  "m_dwStyleB=" << m_dwStyleB;
	dc <<  "m_dwStyleB=" << m_dwStyleC;
	dc <<  "m_dwComponentStyle=" << m_dwComponentStyle;
	dc <<  "m_RGBFrameColor=" << m_RGBFrameColor;
	dc <<  "m_strFramePattern=" << m_strFramePattern;;
	dc <<  "m_nFrameWidth=" << m_nFrameWidth;
	dc <<  "m_RGBColorA=" << m_RGBColorA;
	dc <<  "m_RGBColorB=" << m_RGBColorB;
	dc <<  "m_dAngle=" << m_dAngle;
	dc <<  "m_strImage=" << m_strImage;


#endif
	  
}

#endif //_DEBUG


//@mfunc Initialises the semaphore for use of maximum scale values
//@parm TRUE to use maximum scales
//@rem When set the display object which owns this style object will use
// the maximum and minimum scales set in the m_dMaxRangeY and m_dMinRangeY 
//members.
//@see <mf SRGraphDisplay::GetMaxRange> <mf SRGraphDisplay::GetMinRange>
void SRGraphStyle::SetUseMaxScale(BOOL style)
{
	m_dwStyleA &= ~CX_MAX_SCALE_MASK;
	m_dwStyleA |= style ? CX_MAX_SCALE_MASK : 0;
}

//@mfunc Gets the semaphore which forces SRGraphDisplay objects to lock thier scales to a specific range.
//@rdesc TRUE if scales are to be locked. Otherwise FALSE.
//@see <mf SRGraphStyle::SetUseMaxScale> <md SRGraphDisplay::m_dMaxRangeY>  <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetUseMaxScale()
{
	return m_dwStyleA & CX_MAX_SCALE_MASK ? TRUE : FALSE;
}

//@mfunc Sets the semaphore which enables the display of the "empty graph" sign.
//@parm TRUE to enable otherwise FALSE
//@see <mf SRGraphStyle::GetUseEmptyGraph> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetUseEmptyGraph(BOOL style)
{
	m_dwStyleA &= ~CX_EMPTY_GRAPH_MASK;
	m_dwStyleA |= style ? CX_EMPTY_GRAPH_MASK : 0;
}

//@mfunc Gets the semaphore which enables the display of the "empty graph" sign.
//@rdesc TRUE if enabled otherwise FALSE
//@see <mf SRGraphStyle::SetUseEmptyGraph> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetUseEmptyGraph()
{
	return m_dwStyleA & CX_EMPTY_GRAPH_MASK ? TRUE : FALSE;
}

//@mfunc Sets the semaphore which enables the display of groups on an axis
//@parm TRUE to enable display. FALSE to disable.
//@see <mf SRGraphStyle::SetShowGroups> <mf SRGraphStyle::SetShowIndices> <mf SRGraphStyle::GetShowNumerals>
void SRGraphStyle::SetShowNumerals(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_NUMERALS_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_NUMERALS_MASK;
}

//@mfunc Sets the semaphore which enables the display of groups on an axis
//@parm TRUE to enable display. FALSE to disable.
//@see <mf SRGraphStyle::SetShowGroups> <mf SRGraphStyle::GetShowIndices> <mf SRGraphStyle::SetShowNumerals>
void SRGraphStyle::SetShowIndices(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_INDICES_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_INDICES_MASK;
}

void SRGraphStyle::SetShowXGrid(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_XGRID_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_XGRID_MASK;
}

void SRGraphStyle::SetShowYGrid(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_YGRID_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_YGRID_MASK;
}

void SRGraphStyle::SetShowZGrid(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_ZGRID_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_ZGRID_MASK;
}

void SRGraphStyle::SetShowXTicks(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_XTICKS_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_XTICKS_MASK;
}

void SRGraphStyle::SetShowYTicks(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_YTICKS_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_YTICKS_MASK;
}

void SRGraphStyle::SetShowZTicks(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_ZTICKS_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_ZTICKS_MASK;
}

void SRGraphStyle::SetSort(BOOL style)
{
	m_dwStyleA &= ~CX_SORT_VALUES_MASK;
	m_dwStyleA |= style ? 0 : CX_SORT_VALUES_MASK;
}

void SRGraphStyle::SetGridOrder(BOOL style)
{
	m_dwStyleA &= ~CX_OVERLAY_GRID_MASK;
	m_dwStyleA |= style ? 0 : CX_OVERLAY_GRID_MASK;
}

void SRGraphStyle::SetAlwaysShowZero(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_ZERO_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_ZERO_MASK;
}

//@mfunc Locks or unlocks the position of the m_DisplayRect member
//@parm TRUE to lock
//@rem Use this function to override the normal automatic sizing and positioning 
//behaviour of the SRGraphDisplay object which owns this style. 
//Using this method you may exactly define the size and position of tha data display 
//portion of the graph and its axes. Note however that labels etc will be drawn outside 
//of this area in the normal fashion. This implies that you are responsible for ensuring 
//that sufficient room is allowed for the external text etc. otherwise cropping of the 
//graphics may occur.
void SRGraphStyle::DRLock(BOOL style)
{
	m_dwStyleA &= ~CX_DISPLAYLOCK_MASK;
	m_dwStyleA |= style ? CX_DISPLAYLOCK_MASK : 0;
}

//@mfunc Sets the semaphore which enables the display of groups on an axis
//@parm TRUE to enable display. FALSE to disable.
//@see <mf SRGraphStyle::GetShowGroups> <mf SRGraphStyle::SetShowIndices> <mf SRGraphStyle::SetShowNumerals>
void SRGraphStyle::SetShowGroups(BOOL style)
{
	m_dwStyleA &= ~CX_HIDE_GROUPS_MASK;
	m_dwStyleA |= style ? 0 : CX_HIDE_GROUPS_MASK;
}

//@mfunc Returns the semaphore which enables or disables the display of the numeric scale (for example the Y axis of a classic XY graph) on graph axes
//@rdesc TRUE to display the scale Otherwise FALSE;
//@see <mf SRGraphStyle::GetShowGroups> <mf SRGraphStyle::GetShowIndices> <mf SRGraphStyle::SetShowNumerals> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetShowNumerals()
{
	return m_dwStyleA & CX_HIDE_NUMERALS_MASK ? FALSE : TRUE;
}

//@mfunc Returns the semaphore which enables or disables the display of the groups on graph axes
//@rdesc TRUE to display the groups. Otherwise FALSE;
//@see <mf SRGraphStyle::SetShowGroups> <mf SRGraphStyle::GetShowIndices> <mf SRGraphStyle::GetShowNumerals> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetShowGroups()
{
	return m_dwStyleA & CX_HIDE_GROUPS_MASK ? FALSE : TRUE;
}

//@mfunc Returns the semaphore which enables or disables the display of the indices on graph axes
//@rdesc TRUE to display the indices. Otherwise FALSE;
//@see <mf SRGraphStyle::GetShowGroups> <mf SRGraphStyle::SetShowIndices> <mf SRGraphStyle::GetShowNumerals> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetShowIndices()
{
	return m_dwStyleA & CX_HIDE_INDICES_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetShowXGrid()
{
	return m_dwStyleA & CX_HIDE_XGRID_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetShowYGrid()
{
	return m_dwStyleA & CX_HIDE_YGRID_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetShowZGrid()
{
	return m_dwStyleA & CX_HIDE_ZGRID_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetAlwaysShowZero()
{
	return m_dwStyleA & CX_HIDE_ZERO_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetShowXTicks()
{
	return m_dwStyleA & CX_HIDE_XTICKS_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetShowYTicks()
{
	return m_dwStyleA & CX_HIDE_YTICKS_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetShowZTicks()
{
	return m_dwStyleA & CX_HIDE_ZTICKS_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetSort()
{
	return m_dwStyleA & CX_SORT_VALUES_MASK ? FALSE : TRUE;
}

BOOL SRGraphStyle::GetGridOrder()
{
	return m_dwStyleA & CX_OVERLAY_GRID_MASK ? FALSE : TRUE;
}

//@mfunc Sets the semaphore which switches legends to either line key display or filled box keys.
//@parm TRUE to use a line type legend key. FALSE to use a filled box key.
//@see <mf SRGraphStyle::GetLegendLine> <md SRGraphStyle::m_dwStyleA>
void SRGraphStyle::SetLegendLine(BOOL style)
{
	m_dwStyleA &= ~CX_LEGEND_LINE_MASK;
	m_dwStyleA |= style ? 0 : CX_LEGEND_LINE_MASK;
}

//@mfunc Returns the semaphore which signifies whether legends are line-types or fill types
//@rdesc TRUE for line type. FALSE otherwise
//@see <mf SRGraphStyle::SetLegendLine> <md SRGraphStyle::m_dwStyleA>
BOOL SRGraphStyle::GetLegendLine()
{
	return m_dwStyleA & CX_LEGEND_LINE_MASK ? FALSE : TRUE;
}

//@mfunc Retrieves a pointer to the m_Style member.
//@rdesc A pointer to the SRGraphStyle item used to control the appearence
// of this object.
//@devnote You are strongly advised to use the SomeSRGraphObject.GetStyle()->StyleFunction() method for accessing style data 
//Especially if you use the DLL versions of the libraries. If you follow this advice later releases of the DLL's should not neccessitate 
//the re-compilation of your applications.
SRGraphStyle * SRGraphData::GetStyle()
{
	return &m_Style;
}

//@mfunc Retrieves a pointer to the m_Style member.
//@rdesc A pointer to the SRGraphStyle item used to control the appearence
// of this object.
//@devnote You are strongly advised to use the SomeSRGraphObject.GetStyle()->StyleFunction() method for accessing style data 
//Especially if you use the DLL versions of the libraries. If you follow this advice later releases of the DLL's should not neccessitate 
//the re-compilation of your applications.
SRGraphStyle * SRGraphDataList::GetStyle()
{
	return &m_Style;
}

//@mfunc Gets the pen width used for framing objects
//@rdesc the contents of the m_nFrameWidth member
//@see <mf SRGraphStyle::SetFrameWidth> <md SRGraphStyle::m_nFrameWidth>
int SRGraphStyle::GetFrameWidth()
{
	return m_nFrameWidth;
}

//@mfunc Gets the angle used for filling angled vignettes
//@rdesc The contents of the m_dAngle member variable;
//@see <mf SRGraphStyle::SetAngle> <md SRGraphStyle::m_dAngle>
double SRGraphStyle::GetAngle()
{
	return m_dAngle;
}

//@mfunc This function has two contexts. When used in the context of a data item and attached to an SRGraphData or derived object this function Sets the angle used for filling angled vignettes used on the interiors of those objects.
// When used in the context of an SRGraphDisplay or derived object it may define the angle of tilt on a pie chart etc.
//@parm The angle to set in degrees
//@see <mf SRGraphStyle::GetAngle> <md SRGraphStyle::m_dAngle>
void SRGraphStyle::SetAngle(double angle)
{
	m_dAngle=angle;
}

//@mfunc Alternative method of setting the color members.
//@parm The color to set.
//@parm FALSE (default) Sets m_RGBColorA. TRUE to set m_RGBColorB
//@see <mf SRGraphStyle::GetColor> <md SRGraphStyle::m_RGBColorA> <md SRGraphStyle::m_RGBColorB>
void SRGraphStyle::SetColor(COLORREF color,BOOL bSetB)
{
	if(bSetB)
		m_RGBColorB=color;
	else
		m_RGBColorA=color;
}

//@mfunc Returns a pointer to the m_AnnotationList member
SRGraphAnnotationList * SRGraphDataList::GetAnnotationList()
{
	return &m_strAnnotationList;
}


//@mfunc Destructor
SRGraphData::~SRGraphData()
{
}

SRGraphStyle::SRGraphStyle(const SRGraphStyle &s)
{
	*this=s;
}


void SRGraphAnnotationList::WriteList(CStdioFile *file)
{
	CString s;
	for(POSITION p=GetHeadPosition();p!=NULL;)
	{
		SRGraphAnnotation *pA=(SRGraphAnnotation *)GetNext(p);
		s.Format("annotation %d %s\n",pA->GetCode(),pA->GetText());
		file->WriteString(s);
	}
}


void SRGraphDataList::Roll(double dValue,BOOL bDirection)
{
	if(IsDynamic())
	{
		SRGraphDynamicData *pDD=(SRGraphDynamicData *)GetIndex(0);
		pDD->Roll(dValue,bDirection);
	}
	else
	{
		SRGraphData *pD;

		if(bDirection)
		{
			pD=(SRGraphData *)RemoveTail();
			pD->SetValue(dValue);
			AddHead(pD);
		}
		else
		{
			pD=(SRGraphData *)RemoveHead();
			pD->SetValue(dValue);
			AddTail(pD);
		}
	}

}

void SRGraphDynamicData::Roll(double dValue,BOOL bDirection)
{
	if(!m_bReady)
		return;

	SRGDataBuffer *pDB=(SRGDataBuffer *)m_pBuffer;

	if(bDirection)
	{
		memmove(&pDB[0],&pDB[1],(m_nBufferSize-1)*sizeof(SRGDataBuffer));
		switch(m_nDataType)
		{
		case SRGD_DOUBLE:
			pDB[0].d=dValue;
			break;
		case SRGD_INT:
			pDB[0].n=(int)dValue;
			break;
		case SRGD_CHAR:
			pDB[0].c=(char)dValue;
			break;
		}
	}
	else
	{
		memmove(&pDB[1],&pDB[0],(m_nBufferSize-1)*sizeof(SRGDataBuffer));
		switch(m_nDataType)
		{
		case SRGD_DOUBLE:
			pDB[m_nBufferSize-1].d=dValue;
			break;
		case SRGD_INT:
			pDB[m_nBufferSize-1].n=(int)dValue;
			break;
		case SRGD_CHAR:
			pDB[m_nBufferSize-1].c=(char)dValue;
			break;
		}
	}

	if(m_bHiLo) // Rolling the hi_lo buffers will take more time.
	{
		int x;
		if(!bDirection)
		{
			for(x=1;x<(int)m_nBufferSize;x++)
				m_HiLoBuffer[x-1]=m_HiLoBuffer[x];
			m_HiLoBuffer[0].Zero();
		}
		else
		{
			for(x=m_nBufferSize-1;x>0;x--)
				m_HiLoBuffer[x-1]=m_HiLoBuffer[x];
			m_HiLoBuffer[m_nBufferSize-1].Zero();
		}
	}


}

//@mfunc NEW. Sets or resets the flag that allows x-labels to be either hidden when 
//overlapping or moved into a staggered or "tiered" formation
//@parm TRUE to set the x-labels to tiered format
void SRGraphStyle::SetTierXLabels(BOOL style)
{
	m_dwStyleA &= ~CX_TIER_XLABELS_MASK;
	m_dwStyleA |= style ? CX_TIER_XLABELS_MASK : 0;
}

//@mfunc NEW. Sets or resets the flag that allows y-labels to be either hidden when 
//overlapping or moved into a staggered or "tiered" formation
//@parm TRUE to set the y-labels to tiered format
void SRGraphStyle::SetTierYLabels(BOOL style)
{
	m_dwStyleA &= ~CX_TIER_YLABELS_MASK;
	m_dwStyleA |= style ? CX_TIER_YLABELS_MASK : 0;
}

//@mfunc NEW. Retrieves the flag that defines whether x-labels should be hidden or moved 
//into a "tiered" format in case of overlap.
//@rdesc TRUE if the labels should be moved, otherwise FALSE.
BOOL SRGraphStyle::GetTierXLabels()
{
	return m_dwStyleA & CX_TIER_XLABELS_MASK;// ? FALSE : TRUE;
}

//@mfunc NEW. Retrieves the flag that defines whether y-labels should be hidden or moved 
//into a "tiered" format in case of overlap.
//@rdesc TRUE if the labels should be moved, otherwise FALSE.
BOOL SRGraphStyle::GetTierYLabels()
{
	return m_dwStyleA & CX_TIER_YLABELS_MASK;// ? FALSE : TRUE;
}

//@mfunc NEW. Initializes the legend key style. Note that you have the option of setting a custom style
//@parm The legend key style to set
//@rem Current styles are:<nl>
//CX_KEY_NONE<nl>
//CX_KEY_AUTOMATIC<nl>
//CX_KEY_LINE<nl>
//CX_KEY_BOX<nl>
//CX_KEY_WIGET<nl>
//CX_KEY_CUSTOM<nl>
//@see <c SRGraphLegend> <mf SRGraphStyle::GetLegendKeyStyle>
void SRGraphStyle::SetLegendKeyStyle(DWORD dwStyle)
{
	m_dwStyleC &= ~CX_KEY_STYLE_MASK;
	m_dwStyleC |= dwStyle & CX_KEY_STYLE_MASK;
}

//@mfunc NEW. Initializes the legend key style. Note that 
//@rdesc The legend key style.
//@see <c SRGraphLegend> <mf SRGraphStyle::SetLegendKeyStyle>
DWORD SRGraphStyle::GetLegendKeyStyle()
{
	 return m_dwStyleC & CX_KEY_STYLE_MASK;
}



//@mfunc NEW. Places a SRGraphData or derived class object in the data list at the desires position
//@parm Pointer to the object to be added to the list.
//@parm The positive index position or a negative code signifying that the obeject 
//should be placed last (-1) or first (-2).
POSITION SRGraphDataList::SetIndex(SRGraphData *pData,int nIndex)
{
	switch(nIndex)
	{
	case -1:
		return AddTail(pData);
	case -2:
		return AddHead(pData);
	default:
		if(GetCount()>nIndex)
		{ // The object should replace an existing one
			POSITION p=FindIndex(nIndex);
			SRGraphData *tpD=(SRGraphData *)GetAt(p);
			SetAt(p,pData);
			delete(tpD);
			return p;
		}
		else
		{	// we need to fill a set of dummies and add to the end
			if(nIndex>0)
				GetIndex(nIndex-1,TRUE);
			return AddTail(pData);
		}
	}
}
