//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

// @doc
// @module SRGraphData.h | 
// SRGraphData header file
//
// copyright <cp>Stingray Software 1996 
//
//Dll declaration context
#ifdef _OBJCHART_DLL
	#undef AFXAPP_DATA
	#define AFXAPP_DATA AFXAPI_DATA
	#undef AFX_DATA
	#define AFX_DATA OC_DATA
#endif


//////////////////////////////////////////////////////////////////////////
// Various helper functions and classes
#ifndef __SCALE__
#define __SCALE__

/////////////////////////////////////////////////////////////
// AutoDuck supplementary help section for CScale

//@mfunc  | CScale | CScale | Constructs and zeroes a CScale object
//@see <mf CScale::Zero>

//@mfunc double | CScale | Range | Returns the range of values enclosed by this CScale object
//@rdesc A double calulated from (m_dMax-m_dMin)

//@mdata double | CScale | m_dMax | The maximum cumulative value 

//@mdata double | CScale | m_dMin | The minimum cumulative value 

//@mdata BOOL | CScale | m_bClean | Private data member TRUE if this CScale object has not accumulated any data

//@mfunc void | CScale | Zero | Resets the dMax and dMin values to zero. Sets the m_bClean semaphore

//@mfunc double | CScale | Max | Interrogates and returns the m_dMax variable

//@mfunc double | CScale | Min | Interrogates and returns the m_dMin variable

//@mfunc double | CScale | Min | Returns the m_dMin (Minimum historic value) member
//@rdesc The lowest number ever passed to this object since construction or zeroing.
//@see <mf CScale::Zero> <md CScale::m_dMin> <mf CScale::Max>
											
//@mfunc double | CScale | Max | Returns the m_dMax (Maximum historic value) member
//@rdesc The Highest number ever passed to this object since construction or zeroing.
//@see <mf CScale::Zero> <md CScale::m_dMax> <mf CScale::Min> 

//@mfunc void | CScale | Zero | Resets all values to zero and sets the m_bClean flag.

//@mfunc BOOL | CScale | IsClean | Ascertains whether the object has been used since it was created or zeroed.
//@rem A clean CScale holds no minimum or maximum information.




/////////////////////////////////////////////////////////////


//@class collates and returns minimum and maximum values from a set
class CScale : public CObject
{
//@access public data members
public:
	//@cmember Returns TRUE if this object has never been used or has been zeroed.
	virtual BOOL IsClean(){return m_bClean;}
	//@cmember Maximum accumulated value since creation or Zeroing.
	double m_dMax;
	//@cmember Minimum accumulated value since creation or Zeroing.
	double m_dMin;

//@access private data members 
private:
	//@cmember Signifies that this object is empty and has no accumulated min-max values
	BOOL m_bClean;

//@access private member functions
private:
	//@cmember Adds the value to the accumulated min-max values
	void total(double n);
	
////////////////////////////
// Construction destruction

//@access public member functions
public:
	//@cmember Constructor. Zeroes the object
	CScale(){Zero();}
	//@cmember Assignment Constructor
	CScale(const CScale& s);

	///////////////////////////
// Operations

	//@cmember Use the += operator to add the number to the min max data 
	//Overloaded to provide support for addition (+=) of.. <nl>
	// const double <nl>
	// const float <nl>
	// const int <nl>
	// const unsigned int <nl>
	// const char <nl>
	// const unsigned char <nl>
	void operator +=(const double n){total(n);}
	void operator +=(const float n){total((double)n);}
	void operator +=(const int n){total((double)n);}
	void operator +=(const unsigned int n){total((double)n);}
	void operator +=(const char n){total((double)n);}
	void operator +=(const unsigned char n){total((double)n);}
	//@cmember Assignment operator
	void operator =(const CScale& s){m_dMax=s.m_dMax; m_dMin=s.m_dMin; m_bClean=s.m_bClean;}
	//@cmember Zeroes the accumulated Hi-Lo data
	void Zero(void){m_bClean=TRUE; m_dMax=m_dMin=0;}
	//@cmember Returns the Max value
	double Max(void){return m_dMax;}
	//@cmember Returns the Min value
	double Min(void){return m_dMin;}
	//@cmember Returns range of values
	double Range();
	


	DECLARE_SERIAL(CScale);

	//@cmember Serialization.
	virtual void Serialize(CArchive &ar);


};


#endif //__SCALE__


#ifndef __SRGRAPHSTYLE__
#define __SRGRAPHSTYLE__


/////////////////////////////////////////////////////////////////////////////
// Autoduck help section for SRGraphStyle

//@mdata double | SRGraphStyle | m_dMaxHeight | The maximum height in pixels that an object may be.
//@see <mf SRGraphStyle::GetMaxObjectHeight> <mf SRGraphStyle::SetMaxObjectHeight> <md SRGraphStyle::m_dMaxDepth> <md SRGraphStyle::m_dMaxWidth>

//@mdata double | SRGraphStyle | m_dMaxDepth | The maximum depth in pixels that an object may be. Used to limit sizes of bars in Manhattan graphs etc
//@see <mf SRGraphStyle::GetMaxObjectDepth> <mf SRGraphStyle::SetMaxObjectDepth> <md SRGraphStyle::m_dMaxWidth> <md SRGraphStyle::m_dMaxHeight>

//@mdata double | SRGraphStyle | m_dMaxWidth | The maximum width in pixels that an object may be. Used to limit the width of bars etc. 
//@see <mf SRGraphStyle::GetMaxObjectWidth> <mf SRGraphStyle::SetMaxObjectWidth> <md SRGraphStyle::m_dMaxDepth> <md SRGraphStyle::m_dMaxHeight>

//@mdata CString | SRGraphStyle | m_strObjectName | Used to identify a resource or file associated with this object.
//Particularly used for the resource name used to define polygon bars.
//@see <mf SRGraphStyle::SetObjectName> <mf SRGraphStyle::GetObjectName>

//@mdata DWORD | SRGraphStyle | m_dwStyle | Various style semaphores and bit combinations.
//You are strongly advised to use the correct initialisation and interrogation routines for these styles.
//@head3 Frame styles. These define the type of frame seen around bars etc.| Use <mf SRGraphStyle::GetFrameStyle> and <mf SRGraphStyle::SetFrameStyle><nl><nl>
//CX_FRAME_NONE<nl>
//CX_FRAME_AUTOMATIC<nl>	A frame is chosen by the drawing system. Style depends on context<nl>
//CX_FRAME_LINE<nl>	The frame is a simple line<nl>
//CX_FRAME_PATTERN<nl>	A pattern will be used to paint a frame<nl>
//CX_FRAME_COLOR<nl>	The m_RGBFrameColor member has an RGB color to be used when drawing the frame<nl><nl>
//@head3 Interior styles which define how an object is filled|Use <mf SRGraphStyle::GetInteriorStyle> and <mf SRGraphStyle::SetInteriorStyle><nl><nl>
//CX_INTERIOR_NONE<nl><nl>
//CX_INTERIOR_AUTOMATIC<nl>	The interior is chosen using an automatic process <nl>
//CX_INTERIOR_COLOR<nl>	The fill is made using a solid color as defined in m_colorA<nl>
//CX_INTERIOR_VSHADE<nl>	A shaded vertical vignette which begins with the value in m_colorA and ends with the value in m_coloB<nl>
//CX_INTERIOR_HSHADE<nl>	A shaded horizontal vignette starting with m_colorA and ending with m_colorB<nl>
//CX_INTERIOR_ASHADE<nl>	A shaded vignette which is angled using m_dAngle and spanning the colors from m_colorA to m_colorB Note that colors within the polygon may not be 
// guaranteed to lie in this range because the span of colors is actually calculated within a bounding box which may or may not allow the color range end points to be seen<nl>
//CX_INTERIOR_RESOURCE<nl>	Uses the name held in m_strImageName to load a resource bitmap which may be stretched or tiled over the surface of the object<nl>
//CX_INTERIOR_IMAGE<nl>	Not currently used (reserved)<nl>
//CX_INTERIOR_PATTERN<nl>	Not currently used (reserved)<nl><nl>
//@head3 Object styles define the type of object used for a particular data item.| Use <mf SRGraphStyle::SetObjectStyle> and <mf SRGraphStyle::GetObjectStyle><nl><nl>
//CX_OBJECT_NONE<nl>
//CX_OBJECT_AUTOMATIC<nl>	An object is chosen in context with the data type or graph style<nl>
//CX_OBJECT_POINT<nl>	The object is a simple point (not plotted)<nl>
//CX_OBJECT_EXPOINT<nl>	The object is a point which is covered by a wiget<nl>
//CX_OBJECT_LINE<nl>	The object is a line<nl>
//CX_OBJECT_BAR<nl>	A bar is used <nl>
//CX_OBJECT_EXBAR<nl>	A bar or polygon is used<nl>
//CX_OBJECT_ROD<nl>	Not used (Reserved)<nl>
//CX_OBJECT_AREA<nl>	Not used (Reserved)<nl>
//CX_OBJECT_SLICE<nl>	Not used (Reserved)<nl>
//CX_OBJECT_POLYGON<nl>	A polygonal shape whose definition is described in m_strObjectName is drawn <nl><nl>
//CX_OBJECT_USER<nl>	User defined object styles lie in the range xxxxc0xx to xxxxffxx. All other values are reserved
//@head3 Graph styles which define the type of graph display.| Use <mf SRGraphStyle::SetGraphStyle> and <mf SRGraphStyle::GetGraphStyle><nl><nl>
//CX_GRAPH_NONE<nl>
//CX_GRAPH_LINE<nl>	Simple line-plot.<nl>
//CX_GRAPH_HBAR<nl>	Horizontal bar chart.<nl>
//CX_GRAPH_VBAR<nl>	Vertical bar chart.<nl>
//CX_GRAPH_PIE<nl>	Standard pie chart<nl>
//CX_GRAPH_HISTOGRAM<nl>	Not used (reserved)<nl>
//CX_GRAPH_AREA<nl>	Not used (reserved)<nl>
//CX_GRAPH_STOCK<nl>	Stock control chart with highest and lowest value history<nl>
//CX_GRAPH_OSCILLISCOPE<nl>	Not used (reserved)<nl>
//CX_GRAPH_ISO_BAR<nl>	3D bar chart or "Manhattan" chart<nl>
//CX_GRAPH_ISO_AREA<nl>	3D area chart or "Rooftop" graph<nl>
//CX_GRAPH_STACK_VBAR<nl>	Stacked vertical bar chart.<nl>
//CX_GRAPH_STACK_HBAR<nl>	Stacked horizontal bar chart<nl>
//CX_GRAPH_ISO_PIE<nl>	3D pie chart<nl>
//CX_GRAPH_STRATA<nl>	Strata graph. The topmost point of the data shows overall value  and the layers within show the contribution each set makes to the total<nl>
//CX_GRAPH_EX_VBAR<nl>	Not used (reserved)<nl>
//CX_GRAPH_RIBBON<nl>	3D ribbon graphs for use on stage and platform axes.<nl>
//CX_GRAPH_USER<nl>	User graph styles are in the range xxc0xxxx to xxffxxxx all other values are reserved.	
//@head3 Axis styles. Each graph data display needs an axis upon which data is projected.| Use <mf SRGraphStyle::SetAxisStyle> and <mf SRGraphStyle::GetAxisStyle><nl><nl>
//CX_AXIS_NONE<nl>
//CX_AXIS_AUTOMATIC<nl>	An axis is chosen in context with the graph type<nl>
//CX_AXIS_CLASSIC<nl>	The classic x,y plot axis<nl>
//CX_AXIS_OSCILLISCOPE<nl>	Not used (reserved)<nl>
//CX_AXIS_PLATFORM_SE<nl>	A 3/4 view platform seen from the "south-east"<nl>
//CX_AXIS_PLATFORM_SW<nl>	A 3/4 view platform seen from the "south-west"<nl>
//CX_AXIS_STAGE_SE<nl>	A 3/4 view stage seen from the "south-east"	<nl>
//CX_AXIS_STAGE_SW<nl>	A 3/4 view stage seen from the "south-west"<nl>
//CX_AXIS_H_FRAME<nl>	Not used (reserved)<nl>
//CX_AXIS_PIE<nl>	Circular display axis for standard or 3D pie charts<nl>
//CX_AXIS_YX<nl>	Similar to the classic X,Y axis but with axes reversed so that the "y" axis extends across the window<nl>
//CX_AXIS_ISO_CLASSIC<nl>	Not used (reserved)<nl>
//CX_AXIS_USER<nl>	User defined axis styles lie in the range c0xxxxxx to ffxxxxxx. All other values are reserved

//@mdata DWORD | SRGraphStyle | m_dwStyleA | Contains variuos single bit styles .<nl><nl>
//You are strongly advised to use the correct initialisation and interrogation routines for these styles.
//@head3 Settings are all single bit semaphores |<nl>
//CX_SIZE_MASK			=	0x40000000<nl>	Determines if the maximum object size is to be used <mf SRGraphStyle::SetUseMaxSize> <mf SRGraphStyle::GetUseMaxSize><nl>
//CX_FILL_AXIS_MASK		=	0x20000000<nl>	Enables or disables the filling of the areas covered by axes <mf SRGraphStyle::SetAxisFillStyle> <mf SRGraphStyle::GetAxisFillStyle><nl>
//CX_SHADE_OBJECT_MASK	=	0x10000000<nl>	Sets or resets the shading semaphore for certain objects or graph types <mf SRGraphStyle::SetObjectShadeStyle> <mf SRGraphStyle::GetObjectShadeStyle><nl>
//CX_LOG_SCALE_MASK		=	0x08000000<nl>	Enables the logarithmic scale conversions <mf SRGraphStyle::SetLog> <mf SRGraphStyle::GetLog><nl>
//CX_MAX_SCALE_MASK		=	0x04000000<nl>	Display specific. If enabled the graph will not auto-scale. <mf SRGraphStyle::SetUseMaxScale> <mf SRGraphStyle::GetUseMaxScale><nl>
//CX_EMPTY_GRAPH_MASK	=	0x02000000<nl>	When enabled and the graph holds either no or null data a symbol is shown signifying an empty graph. <mf SRGraphStyle::SetUseEmptyGraph> <mf SRGraphStyle::GetUseEmptyGraph><nl>
//CX_HIDE_NUMERALS_MASK	=	0x01000000<nl>	To Show or hide the numeric scale on a graph axis. <mf SRGraphStyle::SetShowNumerals> <mf SRGraphStyle::GetShowNumerals><nl>
//CX_HIDE_INDICES_MASK	=	0x00800000<nl>	To Show or hide the indices on a graph axis. <mf SRGraphStyle::SetShowIndices> <mf SRGraphStyle::GetShowIndices><nl>
//CX_HIDE_GROUPS_MASK	=	0x00400000<nl>	To Show or hide the groups on a graph axis. <mf SRGraphStyle::SetShowGroups> <mf SRGraphStyle::GetShowGroups><nl>
//CX_LEGEND_LINE_MASK	=	0x00020000<nl>	Switches between line keys and filled box keys on legends. <mf SRGraphStyle::SetLegendLine> <mf SRGraphStyle::SetLegendLine><nl>
//CX_HIDE_XGRID_MASK	=	0x00010000<nl>	When TRUE no X gridlines are shown <nl>
//CX_HIDE_YGRID_MASK	=	0x00008000<nl>	When TRUE no Y gridlines are shown <nl>
//CX_HIDE_ZGRID_MASK	=	0x00004000<nl>	When TRUE no Z gridlines are shown <nl>
//CX_HIDE_ZERO_MASK		=	0x00002000<nl>	If set the zero will only be visible if it is within the minimum and maximum range<nl>
//CX_HIDE_XTICKS_MASK	=	0x00001000<nl>	when TRUE no x tick marks are shown<nl>
//CX_HIDE_YTICKS_MASK	=	0x00000800<nl>	when TRUE no y tick marks are shown<nl> 
//CX_HIDE_ZTICKS_MASK	=	0x00000400<nl>	when TRUE no z tick marks are shown<nl> 
//CX_SORT_VALUES_MASK	=	0x00000200<nl>	When Used to sort the average group values in manhattan and rooftop graphs <mf SRGraphStyle::SetSort><nl>
//CX_OVERLAY_GRID_MAS	=	0x00000100<nl>	May be used to make the grid overlay the drawn data<nl>
//CX_DISPLAYLOCK_MASK	=	0x00000080<nl>	When TRUE the m_DisplayRect rectangle of the owning SRGraphDisplay will never automatcally resize<nl> 

//@mdata DWORD | SRGraphStyle | m_dwStyleB | Style data.<nl><nl>
//@head3 Legend styles. | Use <mf SRGraphStyle::SetLegendStyle> and <mf SRGraphStyle::GeteLegendStyle>
//CX_LEGEND_NONE<nl>
//CX_LEGEND_AUTOMATIC<nl>	Legend is chosen according to graph context<nl>
//CX_LEGEND_BY_INDEX<nl>	Uses index headers to annotate legend<nl>
//CX_LEGEND_BY_GROUP<nl>	Uses Group headers to annotate legend<nl>
//CX_LEGEND_BY_RECTANGLE<nl>	Uses individual annotations<nl>
//CX_LEGEND_HEAD_GROUP<nl>	Titles the legend by group and uses indices to annotate<nl>
//CX_LEGEND_HEAD_INDEX<nl>	Titles the legend by index and uses groups to annotate<nl>
//CX_LEGEND_USER<nl>	User legend styles. xxxxxxc0 to xxxxxxff All other values are reserved.<nl>
//@head3 Note | Stingray Software reserves all other bits in the m_dwStyleB variable.<nl>

//@mdata DWORD | SRGraphStyle | m_dwComponentStyle | Styles for the basic component panel.<nl>
//Use only the correct get and set routines to change these parameters.<nl><nl>
//@head3 Component border styles | Use <mf SRGraphStyle::SetComponentBorderStyle> and <mf SRGraphStyle::GetComponentBorderStyle><nl><nl>
//CX_NO_BORDER<nl>
//CX_THIN_RAISED_BORDER<nl>
//CX_THIN_SUNKEN_BORDER<nl>
//CX_THIN_3D_BORDER<nl>
//CX_THICK_RAISED_BORDER<nl>
//CX_THICK_SUNKEN_BORDER<nl>
//CX_THICK_3D_BORDER<nl>
//CX_LINE_BORDER<nl><nl>
//@head3 Shadow styles.| Use <mf SRGraphStyle::SetComponentShadowStyle> and <mf SRGraphStyle::GetComponentShadowStyle><nl><nl>
//CX_NO_SHADOW<nl>
//CX_THIN_SHADOW<nl>
//CX_MEDIUM_SHADOW<nl>
//CX_THICK_SHADOW<nl><nl>
//@head3 Background fill styles. | Use <mf SRGraphStyle::GetComponentFillStyle> and <mf SRGraphStyle::SetComponentFillStyle><nl><nl>
//CX_NO_FILL<nl>
//CX_SOLID_FILL<nl>
//CX_VERTICAL_FILL<nl>
//CX_HORIZONTAL_FILL<nl>
//CX_ANGLE_FILL<nl>
//CX_RESOURCE_FILL<nl>
//CX_IMAGE_FILL (not currently implemented)<nl> 

//@mdata DWORD | SRGraphStyle | m_dwStyleC | Contains type, label and higklight settings
//@head3 Type styles. Used to identify the type of object which owns the SRGraphStyle variable | Use <mf SRGraphStyle::GetType> and <mf SRGraphStyle::SetType><nl><nl>
//CX_TYPE_NONE<nl>		
//CX_TYPE_COMPONENT<nl>	
//CX_TYPE_OBJECT<nl>			
//CX_TYPE_GRAPH<nl>			
//CX_TYPE_DISPLAY<nl>		
//CX_TYPE_LEGEND<nl>		
//CX_TYPE_TITLE<nl>
//CX_TYPE_LABEL<nl>
//CX_TYPE_LABELBLOCK<nl>
//CD_TYPE_DSCALE<nl>
//CX_TYPE_LOGSCALE<nl>
//CX_TYPE_DATESCALE<nl><nl>
//@head3 Label styles. Particularly used on pie charts to show information normally shown on the axes of other graphs.| use <mf SRGraphStyle::SetLabelStyle> and <mf SRGraphStyle::GetLabelStyle><nl><nl>
//CX_LABEL_NONE<nl>				
//CX_LABEL_AUTOMATIC<nl>	Label style is chosen in context with the graph type<nl>			
//CX_LABEL_VALUE<nl>	The label is the numeric value<nl>			
//CX_LABEL_PERCENT<nl> The label is the percentage of the current value in relation to the sum of all values in the graph<nl>		
//CX_LABEL_VALUE_PERCENT<nl>	Combination of LABEL_VALUE and LABEL_PERCENT<nl>
//CX_LABEL_TEXT<nl>	The label is the annotation from the data item<nl>		
//CX_LABEL_TEXT_VALUE<nl>	Combination of LABEL_VALUE and LABEL_TEXT<nl>	
//CX_LABEL_TEXT_VALUE_PERCENT<nl>	Combination of LABEL_TEXT LABEL_VALUE and LABEL_PERCENT<nl>
//CX_LABEL_TEXT_PERCENT<nl>	Combination of LABEL_TEXT and LABEL_PERCENT<nl><nl>	
//CX_LABELBLOCK_HIDELABELS<nl>	Used on label blocks to hide labels which would otherwise overprint another (Quickest method for lots of labels)<nl>
//CX_LABELBLOCK_MOVELABELS<nl>	Used on label blocks to stack labels in a multi tier fashion. (Slower but neat and with all the labels shown)
//@head3 Highlight styles for pie wedge slices. A highlight setting will "explode" the wedge from the pie.|Use <mf SRGraphStyle::SetHighlight> and  <mf SRGraphStyle::GetHighlight><nl><nl>
//CX_HIGHLIGHT_NONE<nl><nl>		
//CX_HIGHLIGHT_AUTOMATIC<nl>	A highlight is chosen in context with the graph style<nl>	
//CX_HIGHLIGHT_SMALL<nl>	The wedge is exploded 5% of the radius of the pie<nl>		
//CX_HIGHLIGHT_MEDIUM<nl>	The wedge is exploded 10% of the radius of the pie<nl>		
//CX_HIGHLIGHT_LARGE<nl>	The wedge is exploded 20% of the radius of the pie<nl>
//CX_HIGHLIGHT_CUSTOM<nl>	See SRGraphDisplays DrawIsoPieData routine for handling methods<nl><nl>

//@mdata COLORREF | SRGraphStyle | m_RGBFrameColor | The RGB color value associated with frames around objects, graph axes etc.
//@see <mf SRGraphStyle::SetFrameColor> <mf SRGraphStyle::GetFrameColor>

//@mdata CString | SRGraphStyle | m_strFramePattern | The resource name of the pattern used to paint frames.
//@see <mf SRGraphStyle::SetFramePattern> and <mf SRGraphStyle::GetFamePattern>

//@mdata int | SRGraphStyle | m_nFrameWidth | The pen width for drawing frames.
//@see <mf SRGraphStyle::SetFrameWidth> and <mf SRGraphStyle::GetFrameWidth>

//@mdata COLORREF | SRGraphStyle | m_RGBColorA | The RGB color value used to color objects and backgrounds. Also used as the first color value in vignette drawing.
//@see <mf SRGraphStyle::SetColor> and <mf SRGraphStyle::GetColor>

//@mdata COLORREF | SRGraphStyle | m_RGBColorB | The second color used in vignette drawing.
//@see <mf SRGraphStyle::SetColor> and <mf SRGraphStyle::GetColor>

//@mdata double | SRGraphStyle | m_dAngle | The angle (in degrees) used for drawing angled vignettes.
//@see <mf SRGraphStyle::SetAngle> <mf SRGraphStyle::GetAngle>

//@mdata CString &| SRGraphStyle | m_strImage | The name of the resource used to paint or tile image fills.
//@see <mf SRGraphStyle::SetResourceName> <mf SRGraphStyle::GetResourceName>

//@mfunc CString &| SRGraphStyle | GetFramePattern | Fetches the resource name for the bitmap used to generate pattern brushes for the frames of objects etc.
//@see <mf SRGraphStyle::SetFramePattern> <md SRGraphStyle::m_strFramePattern>

//@mfunc void | SRGraphStyle | SetFramePattern | Initialises the resource name for the bitmap used to generate pattern brushes for the frames of objects etc.
//@see <mf SRGraphStyle::GetFramePattern> <md SRGraphStyle::m_strFramePattern>

//@mfunc int| SRGraphStyle | SetFrameWidth | Sets the size of the pen used for the frames of objects etc.
//@see <mf SRGraphStyle::GetFrameWidth> <md SRGraphStyle::m_nFrameWidth>

//@mfunc DWORD | SRGraphStyle | GetStyle | Retrieves a raw style.
//@rdesc DWORD with all current styles from the m_dwStyle member.
//@see <mf SRGraphStyle::SetStyle>
//@devnote If possible you should not use this or the setstyle members. Use instead the provided Get and Set member functions. These routines ensure that no bit mask confusion can arise from direct access to the m_dwStyle?? members

//@mfunc void | SRGraphStyle | SetStyle | Sets a raw style.
//@parm DWORD | style | A bit combination with any or all style settings.
//@see <mf SRGraphStyle::GetStyle>
//@devnote If possible you should not use this or the Getstyle members. Use instead the provided Get and Set member functions. These routines ensure that no bit mask confusion can arise from direct access to the m_dwStylex members

//@mfunc void | SRGraphStyle | SetGridColor | Initialises the RGB color used to draw grig-lines The default is CXCLR_TEAL
//@parm COLORREF | c | The new color

//@mfunc void | SRGraphStyle | SetXAngle | Initialises the X rotation for the object viewing angle. Currently used only for pie charts. A rotation angle will tilt the pie to a specific angle
//@parm double | dAngle | The requred angle (0-90) in Degrees.

//@mfunc void | SRGraphStyle | SetYAngle | Initialises the X rotation for the object viewing angle. Currently unused.
//@parm double | dAngle | The requred angle (0-90) in Degrees.

//@mfunc void | SRGraphStyle | SetZAngle | Initialises the X rotation for the object viewing angle. Currently unused.
//@parm double | dAngle | The requred angle (0-90) in Degrees.


/////////////////////////////////////////////////////////////////////////////

//@class Manages style data for graphic objects in the graph class kit
class SRGraphStyle : public CObject
{

//@access public data members
public:
	SRGraphStyle(const SRGraphStyle &s);

//@access Protected data members
protected:
	
	//@cmember Various style flags.
	DWORD m_dwStyleC;
	
	//@cmember The RGB color reference for the grid lines
	COLORREF m_RGBGridColor;
	
	//@cmember various style flags see bitmask definitions
	DWORD m_dwStyleB;

	//@cmember Holds the maximum object height
	double m_dMaxHeight;

	//@cmember Holds the maximum object depth
	double m_dMaxDepth;

	//@cmember Holds the maximum object Width
	double m_dMaxWidth;

	//@cmember Holds name used to retrieve resource or file data for this graph display item (ie. The polygon resource for custom bars)
	CString m_strObjectName;
  
	//@cmember various style flags see bitmask definitions
	DWORD m_dwStyle;

	//@cmember various style flags see bitmask definitions
	DWORD m_dwStyleA;

	//@cmember the style for this component
	DWORD m_dwComponentStyle;

	//@cmember colour variable for frame RGB value
	COLORREF m_RGBFrameColor;

	//@cmember name of the pattern used to fill the frame
	CString m_strFramePattern;

	//@cmember Width of the frame in pixels
	int m_nFrameWidth;

	//@cmember colour variable for interior RGB values
	COLORREF m_RGBColorA;

	//@cmember colour variable for interior RGB values
	COLORREF m_RGBColorB;

	//@cmember angle of shade
	double m_dAngle;

	//@cmember Name of an image or resource
	CString m_strImage;

	//@cmember The number of degrees the display should be rotated about the X axis (currently used only in Pie charts)
	double m_dXAngle;
	
	//@cmember The number of degrees the display should be rotated about the Y axis (currently unused)
	double m_dYAngle;
	
	//@cmember The number of degrees the display should be rotated about the Z axis (currently unused)
	double m_dZAngle;



//@access public member functions
public:	
	DECLARE_SERIAL(SRGraphStyle);
	
	//@cmember Initializes the key style
	virtual void SetLegendKeyStyle(DWORD dwStyle);

	//@cmember Fetches the legend key style.
	virtual DWORD GetLegendKeyStyle();
	
	//@cmember Initializes the flag that controls the "tiered" display of label blocks attached to the x-axis
	virtual void SetTierXLabels(BOOL style);

	//@cmember Initializes the flag that controls the "tiered" display of label blocks attached to the y-axis
	virtual void SetTierYLabels(BOOL style);

	//@cmember Returns the flag that controls the "tiered" display of label blocks attached to the x-axis
	virtual BOOL GetTierXLabels();

	//@cmember Returns the flag that controls the "tiered" display of label blocks attached to the y-axis
	virtual BOOL GetTierYLabels();
	
	//@cmember Initializes the color used to display grid lines.
	virtual void SetGridColor(COLORREF c){m_RGBGridColor=c;}
	
	//@cmember,mfunc Fetches the grid line color.
	virtual COLORREF GetGridColor(){return m_RGBGridColor;}
	
	//@cmember Sets the angle which the display should be rotated about the X axis (Currently used only on Pie charts)
	virtual void SetXAngle(double d){m_dXAngle=d;}
	
	//@cmember Sets the angle which the display should be rotated about the Y axis (Currently unused)
	virtual void SetYAngle(double d){m_dYAngle=d;}
	
	//@cmember Sets the angle which the display should be rotated about the Z axis (Currently unused)
	virtual void SetZAngle(double d){m_dZAngle=d;}
	
	//@cmember,mfunc Returns the angle which the display should be rotated about the X axis (Currently used only on Pie charts)
	virtual double GetXAngle(){return m_dXAngle;}
	
	//@cmember,mfunc Returns the angle which the display should be rotated about the Y axis (Implemented but unused)
 	virtual double GetYAngle(){return m_dYAngle;}
	
	//@cmember,mfunc Returns the angle which the display should be rotated about the Z axis (Implemented but unused)
 	virtual double GetZAngle(){return m_dZAngle;}
	
	//@cmember Assignment (copy) operator for Style objects
	virtual void operator =(const SRGraphStyle &s);
	
	//@cmember Constructor
	SRGraphStyle();


	//@cmember Sets axis styles
	virtual void SetAxisStyle(DWORD style);

	//@cmember Interrogates the axis style
	virtual DWORD GetAxisStyle();

	//@cmember Interrogates the m_dwStyle member
	virtual DWORD GetGraphStyle();
	 
	//@cmember Interrogates legend style
	virtual DWORD GetLegendStyle();

	//@cmember Initialises the legend style
	virtual void SetLegendStyle(DWORD style);

	//@cmember Sets the m_dwStyle member with an absolute code
	void SetStyle(DWORD style){m_dwStyle=style;}

	//@cmember Retrieves the m_dwStyle code without masking any bits
	DWORD GetStyle(){return m_dwStyle;}

	//@cmember Sets the highlight flags
	virtual void SetHighlight(DWORD style);

	//@cmember returns the highlight flags
	virtual DWORD GetHighlight();

	//@cmember Sets the label flags
	virtual void SetLabelStyle(DWORD style);

	//@cmember Returns the label flags
	virtual DWORD GetLabelStyle();

	//@cmember Returns the frame flags
	virtual DWORD GetFrameStyle();

	//@cmember Returns the object flags
	virtual DWORD GetObjectStyle();

	//@cmember Sets the object flags
	virtual void SetObjectStyle(DWORD style);

	//@cmember Sets the name of the resource or image used in filling 
	virtual void SetResourceName(LPCSTR name);

	//@cmember Retrieves the name of the resource or image used in filling 
	virtual CString & GetResourceName();

	//@cmember Returns the interior flags
	virtual DWORD GetInteriorStyle();

	//@cmember Returns the type flags
	virtual DWORD GetType();

	//@cmember Sets the type flags
	virtual void SetType(DWORD type);

	//@cmember Returns the shadow flags
	virtual DWORD GetComponentShadowStyle();

	//@cmember Returns the shadow flags
	virtual DWORD GetComponentFillStyle();

	//@cmember Returns the shadow flags
	virtual DWORD GetComponentBorderStyle();

	//@cmember Alternative color setting routine
	virtual void SetColor(COLORREF color,BOOL bSetB=FALSE);

	//@cmember Sets the style flags
	virtual void SetComponentStyle(DWORD style);

	//@cmember Returns the style flags
	virtual DWORD GetComponentStyle();

	//@cmember Sets the component flags
	virtual void SetColor(COLORREF a,COLORREF b);

	//@cmember Initialises graph styles
	virtual void SetGraphStyle(DWORD style);

	//@cmember Initialises the shading style for data objects
 	virtual void SetObjectShadeStyle(BOOL style);

	//@cmember Interrogates the shading style for data objects	
	virtual BOOL GetObjectShadeStyle();

	//@cmember Sets the pen color used to outline objects
	virtual void SetFrameColor(COLORREF color);

	//@cmember Sets the maximum height of a data object
	virtual void SetMaxObjectHeight(double size);

	//@cmember Sets the maximum depth of a data object
	virtual void SetMaxObjectDepth(double size);

	//@cmember Sets the maximum width of a data object
	virtual void SetMaxObjectWidth(double size);

	//@cmember Sets the axis filling function.
	virtual void SetAxisFillStyle(BOOL style);

	//@cmember Interrogates the axis filling semaphore
	virtual BOOL GetAxisFillStyle();

	//@cmember Interrogates the m_FrameColor member
	virtual COLORREF GetFrameColor();

	//@cmember Interrogates the m_colourA or m_colorB members. 
	virtual COLORREF GetColor(BOOL GetColorB=FALSE);

	//@cmember Ascertains whether the maximum data object size is to be used
	virtual BOOL GetUseMaxSize();

	//@cmember Sets or resets the maximum data object size semaphore
	virtual void SetUseMaxSize(BOOL flag);

	//@cmember Interrogates the m_dMaxHeight member
	virtual double GetMaxObjectHeight();

	//@cmember Interrogates the m_dMaxWidth member
	virtual double GetMaxObjectWidth();

	//@cmember Interrogates the m_dMaxDepth member
	virtual double GetMaxObjectDepth();

	//@cmember Sets the shadow style for component display panels
	virtual void SetComponentShadowStyle(DWORD style);

	//@cmember Sets the border style for component display panels
	virtual void SetComponentBorderStyle(DWORD style);

	//@cmember Sets the interior fill style for component display panels
	virtual void SetComponentFillStyle(DWORD style);

	//@cmember Sets the style used for framing graphic data objects
	virtual void SetFrameStyle(DWORD style);

	//@cmember Sets the style used to fill the interior areas of graphic data objects
	virtual void SetInteriorStyle(DWORD style);

	//@cmember Interrogates the image stretching semaphore
	virtual BOOL GetStretchMode();

	//@cmember Sets or resets the image stretching semaphore
	virtual void SetStretchMode(BOOL m);

	//@cmember Returns the name of the graphic data object
	virtual CString& GetObjectName();

	//@cmember Sets the name of the graphic data object
	virtual void SetObjectName(LPCSTR name);

	//@cmember Assigns the angle for angled shaded fills
	virtual void SetAngle(double angle);

	//@cmember Retrieves the angle for angled shaded fills
	virtual double GetAngle();

	//@cmember Retrieves the pen-width of lines used to draw frames
	virtual int GetFrameWidth();

	//@cmember Determines if the legend is a line or fill type
	virtual BOOL GetLegendLine();

	//@cmember Initialises legend style to line or fill type
	void SetLegendLine(BOOL style);

	//@cmember Determines whether indices should be drawn on axes
	virtual BOOL GetShowIndices();

	//@cmember Determines whether groups should be drawn on axes
	virtual BOOL GetShowGroups();

	//@cmember Determines whether values should be drawn on axes
	virtual BOOL GetShowNumerals();

	//@cmember Defines whether groups should be drawn on axes
	virtual void SetShowGroups(BOOL style);

	//@cmember Defines whether indices should be drawn on axes
	virtual void SetShowIndices(BOOL style);

	//@cmember Defines whether values should be drawn on axes
	virtual void SetShowNumerals(BOOL style);

	//@cmember Determines whether the "empty" graph symbol should be shown
	virtual BOOL GetUseEmptyGraph();

	//@cmember Defines whether the "empty" graph symbol should be shown
	virtual void SetUseEmptyGraph(BOOL style);

	//@cmember Determines whether the maximum scale values should be used
	virtual BOOL GetUseMaxScale();

	//@cmember Defines whether the maximum scale values should be used
	virtual void SetUseMaxScale(BOOL style);

	//@cmember Initialises the frame pattern resource name
	virtual void SetFramePattern(CString& framepattern){m_strFramePattern=framepattern;}

	//@cmember Retrieves the frame pattern resource name
	virtual CString& GetFramePattern(){return m_strFramePattern;}

	//@cmember,mfunc Initialises the pen width for frames around objects
	//@parm The width in logical units
	virtual void SetFrameWidth(int FrameWidth){m_nFrameWidth=FrameWidth;}
	//@cmember,mfunc Gets the Log scaling flag 
	virtual BOOL GetLog();
    //@cmember,mfunc Sets or resets the logarithmic scaling function
	virtual void SetLog(BOOL logFlag=TRUE);

	//@cmember,mfunc Interrogates the semaphore which enable or disables the display of X grid lines on the axis
	virtual BOOL GetShowXGrid();
	//@cmember,mfunc Interrogates the semaphore which enable or disables the display of Y grid lines on the axis
	virtual BOOL GetShowYGrid();
	//@cmember,mfunc Interrogates the semaphore which enable or disables the display of Z grid lines on the axis
	virtual BOOL GetShowZGrid();
	//@cmember,mfunc Initializes the semaphore which enable or disables the display of X grid lines on the axis
	virtual void SetShowXGrid(BOOL style);
	//@cmember,mfunc Initializes the semaphore which enable or disables the display of Y grid lines on the axis
	virtual void SetShowYGrid(BOOL style);
	//@cmember,mfunc Initializes the semaphore which enable or disables the display of Z grid lines on the axis
	virtual void SetShowZGrid(BOOL style);
	//@cmember,mfunc Interrogates the semaphore which enable or disables the display of X tick marks
	virtual BOOL GetShowXTicks();
	//@cmember,mfunc Interrogates the semaphore which enable or disables the display of Y tick marks
	virtual BOOL GetShowYTicks();
	//@cmember,mfunc Interrogates the semaphore which enable or disables the display of Z tick marks
	virtual BOOL GetShowZTicks();
	//@cmember,mfunc Initializes the semaphore which enable or disables the display of X tick marks
	virtual void SetShowXTicks(BOOL style);
	//@cmember,mfunc Initializes the semaphore which enable or disables the display of Y tick marks
	virtual void SetShowYTicks(BOOL style);
	//@cmember,mfunc Initializes the semaphore which enable or disables the display of Z tick marks
	virtual void SetShowZTicks(BOOL style);
	//@cmember,mfunc Initializes the AutoSort flag. if TRUE then auto sorting takes place on Manhattan and Rooftop charts.
	virtual void SetSort(BOOL style);
	//@cmember,mfunc Interrogates the AutoSort flag. in <md SRGraphStyle::m_dwStyleC>
	virtual BOOL GetSort();
	//@cmember,mfunc Sets the order in which grid and data overlay one another
	virtual void SetGridOrder(BOOL style);
	//@cmember,mfunc Gets the order in which grid and data overlay one another
	virtual BOOL GetGridOrder();
	//@cmember Enables or disables the locking of the position of m_DisplayRect.
	virtual void DRLock(BOOL style);
	//@cmember,mfunc Checks to see if the position of m_DisplayRect is locked.
	virtual BOOL DRIsLocked(){return m_dwStyleA & CX_DISPLAYLOCK_MASK;}
	//@cmember,mfunc Interrogates the "Always Show Zero" flag
	virtual BOOL GetAlwaysShowZero();
	//@cmember,mfunc Initializes the "Always Show Zero" flag. If true the graph will allways extend to or cover the zero mark.
	virtual void SetAlwaysShowZero(BOOL style);


	//@cmember Serialization
	virtual void Serialize(CArchive &ar);


#ifdef _DEBUG

	//@cmember Debug dumping
	virtual void Dump(CDumpContext &dc) const;

#endif


};

#endif //__SRGRAPHSTYLE__

#ifndef __SRGRAPHANNOTATION__
#define __SRGRAPHANNOTATION__

//@mdata int | SRGraphAnnotation | m_nCode | The country code of the annotation

//@mdata CString | SRGraphAnnotation | m_strAnnotation | The actual text of the annotation

//@mfunc  | SRGraphAnnotation | SRGraphAnnotation | Constructor
//@rem Sets the m_nCode variable to the default (01 U.S.A.)
//@see <md SRGraphAnnotation::m_nCode>


//@class SRGraphAnnotation |
//This class is the basic storage medium for multi language annotations for graphs.<nl>
//The m_nCode class data member holds an integer which corresponds to the country code of the annotation.<nl>
//The m_strAnnotation data member is a CString object which holds the text in the specific language of the country<nl>
//for example:						 <nl>
//m_nCode=01(USA) m_strAnnotation="Annual peanut production world-wide"<nl>
//m_nCode=33(France) m_strAnnotation="Production annuel des cachuettes (mondial)"<nl>
class SRGraphAnnotation : public CObject
{
// @access data members
protected:
	
	// @cmember Country code corresponding to the international telephone dialling code
	int m_nCode; 

	// @cmember The text assosiated with this annotation
	CString m_strAnnotation;

// @access  member functions
public:
	//@cmember default constructor. This also initialises the country code to 01 (USA).
	SRGraphAnnotation(){m_nCode=01;}

	//@cmember Constructor with code and text initializers
	SRGraphAnnotation(int code, LPCSTR text);

	
	DECLARE_SERIAL(SRGraphAnnotation)
	
	// @cmember handle storage and retrieval of the data
	virtual void Serialize(CArchive& ar);

	
	//@cmember to query the country code data member
	int GetCode();
	//@cmember to set the country code data member
	void SetCode(int code); 
	
	//@cmember Retrieves the text for this annotation
	CString& GetText();  
	
	//@cmember Stores text in this annotation
	void SetText(CString s); 

#ifdef _DEBUG
	
	//@cmember DEBUG ONLY dumps the contents of this object
	void Dump(CDumpContext& dc) const;
#endif

};

#endif //__SRGraphAnnotation__

#ifndef __SRGRAPHANNOTATIONLIST__
#define __SRGRAPHANNOTATIONLIST__

//@mfunc void | SRGraphAnnotationList | SetAnnotation | Sets the default annotation using the text supplied
//@parm LPCSTR | text | Pointer to the annotation text.

//@class handles lists of <c SRGraphAnnotation> objects <nl>
//
//Annotation lists allow you to have a single graph with multiple annotations in each element. The graph when displayed will take notice of the locale
// of the computer used to display the graph and will use the corresponding entry. The default locale is 01 (USA).
//

class SRGraphAnnotationList : public CObList
{
//@access protected data member
protected:
	CString m_strEmptyString; // Allways empty.

//@access data members
public:
	virtual void WriteList(CStdioFile *file);

//@access member functions
public:
	
	//@cmember Retrieves an annotation for a specific code
	virtual CString& GetAnnotation(int code=1);
	//@cmember Default constructor
	SRGraphAnnotationList(){}

	//@cmember Destructor... must handle cleanup of stored <c SRGraphAnnotation> objects
	virtual ~SRGraphAnnotationList();


	DECLARE_SERIAL(SRGraphAnnotationList)

	//@cmember Serialisation 
	virtual void Serialize(CArchive &ar);


#ifdef _DEBUG
	//@cmember Dump this object to a CDumpContext
	void Dump(CDumpContext &dc) const;
#endif


	//@cmember Adds an annotation or replaces one with c corresponding country code
	void SetAnnotation(int code, LPCSTR text);

	//@cmember Sets annotation in default locale
	void SetAnnotation(LPCSTR text){SetAnnotation(1,text);}

	//@cmember Deletes all data in a list
	void KillList(void);
};

#endif //__SRGraphAnnotationList__

#ifndef __SRGRAPHDATA__
#define __SRGRAPHDATA__


//@mdata BOOL | SRGraphData | m_bNULL | Semaphore which signifies that the data in this object is to be considered null or invalid.
//@rem When this member is TRUE the data value should be ignored and treated as  being unreliable

//@mdata CScale | SRGraphData | m_HiLoValue | This CScale object will keep a record of all the highest and lowest values ever sent to this data item.

//@mdata double | SRGraphData | m_dValue | The value stored in this data item

//@mdata SRGraphAnnotationList | SRGraphData | m_strAnnotationList | Each data item has a multi language text item associated with it. You may use this item to annotate individual items of data.

//@mdata SRGraphStyle | SRGraphData | m_Style | Each data item has a style associated with it. This may be used to define the look of the graphic object used to display the data.

//@mfunc BOOL | SRGraphData | GetNull | Returns the <md SRGraphData::m_bNULL> member



//@class Manages one item of data within a graph.<nl><nl>
//This class can be considered as the controller for one node in a chart, one bar on a graph, one slice of a pie chart etc..<nl><nl>
//Each node has a value. (held in the m_Value) member and an annotation list (held in the m_strAnnotationList member)<nl><nl>
//The <c SRGraphAnnotationList> enables a single node to hold many text annotations which are recognised according to locale specific data. 
//These annotations may then be used to reproduce graphs for different languages without needing to alter the data itself.
//@devnote the data items now contain a null flag (m_bNull) which may be set to exclude a data item from the display.
class SRGraphData : public CObject
{
//@access data members
public:
	virtual ~SRGraphData();
	virtual SRGraphStyle * GetStyle();
	//@cmember Accumulated high-low values
	CScale m_HiLoValue;

	//@cmember the floating point value of the graph data item
	double m_dValue;		

	//@cmember manages the multi lingual annotation lists
	SRGraphAnnotationList m_strAnnotationList; 

	//@cmember Style data for this data object
	SRGraphStyle m_Style; 

//@access member functions
public:
	//@cmember Sets the m_bNull member
	virtual void SetNull(BOOL b){m_bNULL=b;}
	//@cmember Interrogates the m_bNull member
	virtual BOOL GetNull(){return m_bNULL;}
	
	//@cmember Returns the true logarithm value for m_value
	virtual double GetLogValue();
	
	//@cmember Returns all data to zero and cleans the hi-lo data
	virtual void Zero();
	
	//@cmember Returns the hi-lo data value
	virtual CScale GetHiLoData();

	//@cmember Returns FALSE because this is not a dynamic data storage object
	virtual BOOL IsDynamic(){return FALSE;}

	//@cmember Retrieves the annotation for this object
	virtual CString& GetAnnotation(int code=1);

	//@cmember Retrieves the value for this object
	virtual double GetValue(BOOL bLogData=FALSE);

	//@cmember Initialises the value for this object
	virtual void SetValue(double number);

	//@cmember Constructor with default initialisation of value.
	SRGraphData();

	//@cmember Constructor with initialisation of data
	SRGraphData(double value, CString text, int style=0);

	DECLARE_SERIAL(SRGraphData)

	//@cmember handles storage and retrieval
	virtual void Serialize(CArchive& ar);

	
	//@cmember Overload 1. Sets text in an annotation.
	virtual void SetAnnotation(int code, LPCSTR text);
	
	//@cmember Overload 2. Sets text in an annotation.
	virtual void SetAnnotation(LPCSTR text, BOOL clear=FALSE, int code=1);

	//@cmember Returns a pointer to the annotation list associated with this object.
	virtual SRGraphAnnotationList *GetAnnotationList(){return &m_strAnnotationList;}

	//@cmember Returns a pointer to the CScale value history item associated with this object.
	virtual CScale *GetHiLoValue(){return &m_HiLoValue;}

#ifdef _DEBUG

	//@cmember Dumps this object to a CDumpContext
	void Dump(CDumpContext &dc) const;

#endif

protected:
	//@cmember Semaphore. When TRUE this denotes that the data held in this item is NULL and should not be used
	BOOL m_bNULL;
};



#endif //__SRGraphData__


#ifndef __SRGRAPHDATALIST__
#define __SRGRAPHDATALIST__     

//@class Holds a linked list of <c SRGraphData> objects.
//
//This class is the top level data store for the <c SRGraph> class. the m_strAnnotation member of this class 
//effectively names the graph for example..
//
// SRGraphDataList <nl>
//		m_strAnnotation (01),("January 1996") <nl>
//		SRGraphData <nl>
//			m_Value 50,000 <nl>
//			m_Annotoation (01),("Capital expenditure") <nl>
//		SRGraphData <nl>
//			m_Value 17,000 <nl>
//			m_Annotoation (01),("Wages") <nl>
//		SRGraphData <nl>
//			m_Value 77,000 <nl>
//			m_Annotoation (01),("gross income") <nl>
//		SRGraphData <nl>
//			m_Value 10,000 <nl>
//			m_Annotoation (01),("profit") <nl>
//
//

class SRGraphDataList : public CObList
{
	//@access data members
public:
	//@cmember Holds style information which may globally affect the styles of the whole list
	SRGraphStyle m_Style;
	//@cmember Holds the annotations for this list
	SRGraphAnnotationList m_strAnnotationList;

	//@access member functions
public:

	//@cmembe Places a ready created SRGraphData or derived object into the list
	virtual POSITION SetIndex(SRGraphData *pData,int nIndex=-1);

	//@cmember Rolls a data set forwards or backwards
	virtual void Roll(double dValue,BOOL bDirection=FALSE);

	//@cmember Returns a pointer to the nnotation list
	virtual SRGraphAnnotationList * GetAnnotationList();

	//@cmember Returns a pointer to this objects style member
	virtual SRGraphStyle * GetStyle();

	//@cmember Sets or Resets the dynamic chart flag
	virtual void SetDynamic(BOOL b=TRUE);

	//@cmember default constructor
	SRGraphDataList();

	//@cmember Constructor with annotation initialization
	SRGraphDataList(LPCSTR text,int code=01);

	//@cmember Destructor with clean up of stored objects
	virtual ~SRGraphDataList();

	//@cmember Retreives annotations for this list. Used to fetch group headers
	virtual CString& GetAnnotation(int code=1);

	
	DECLARE_SERIAL(SRGraphDataList)

	//@cmember handles storage and retrieval of the data
	virtual void Serialize(CArchive &ar);


	//@cmember Overload 1. Sets text in an annotation.
	void SetAnnotation(int code, LPCSTR text);
	
	//@cmember Overload 2. Sets text in an annotation.
	void SetAnnotation(LPCSTR text, BOOL clear=FALSE, int code=1);

	//@cmember Returns therequested SRGraphData possibly making the list grow if it's too short.
	SRGraphData *GetIndex(UINT index,BOOL GrowList=TRUE);

	//@cmember Interrogates the m_bDynamic member
	virtual BOOL IsDynamic(){return m_bDynamic;}

#ifdef _DEBUG

	//@cmember Dumps the contents of this object to a CDumpContext
	void Dump(CDumpContext &) const;

#endif

//@access Protected data members
protected:
	//@cmember Set TRUE when this list holds a single dynamic data object instead of a list of individual non dynamic objects
	BOOL m_bDynamic;
};

#endif //__SRGraphDataList__

#ifndef __SRGRAPHDYNAMICDATA__
#define __SRGRAPHDYNAMICDATA__

//@type SRGDataBuffer | This union contains the various values which may be contained in dynamic data objects.
typedef union _TAG_SRGDATABUFFER {
	char c;		//@field A character
	int n;		//@field An integer
	double d;	//@field A double precision floating point value
} SRGDataBuffer;

//@enum The type of data being dealt with by a dynamic data object
enum SRGD_TYPES {
	SRGD_NONE,	//@emem No data has been entered
	SRGD_DOUBLE,//@emem The first data used was a double
	SRGD_INT,	//@emem The first data used was an int
	SRGD_CHAR	//@emem The first data used was a char
};
/////////////////////////////////////////
// Autoduck comment block for SRGraphDynamicData

//@mdata CScale* | SRGraphDynamicData | m_HiLoBuffer | Points to a buffer containing CScale objects. One for every data item.
// This bufffer is only initialised if the m_bHiLo semaphore is set.

//@mdata BOOL | SRGraphDynamicData | m_bHiLo | When TRUE this semaphore signifies that hi-lo history gathering is required. A buffer will be allocated for the CScale objects automatically.
//@see <md SRGraphDynamicData::m_HiLoBuffer> <mf SRGraphDynamicData::SetHiLoFunction>

//@mdata int | SRGraphDynamicData | m_nDataType | Holds the data type currently being used. So that raw information can be quickly stored without
//conversion to a double the object keeps a record of the first data type used. If data of a different type is used then an exception will be thrown.
//@see <c SRGraphException> 

//@mdata UINT | SRGraphDynamicData | m_nGrowSize | The size (in SRGDataBuffers) by which the buffer must grow in case of an overflow.
//@see <mf SRGraphDynamicData::SetGrowSize> <mf SRGraphDynamicData::SetValue>

//@mdata BOOL | SRGraphDynamicData | m_bCyclic | When TRUE any potential overflow of the input or output indices will result in the relavent index being reset to zero. This allows repeated filling of the fixed length buffer.
//@see 	<mf SRGraphDynamicData::SetCyclic>

//@mdata BOOL | SRGraphDynamicData | m_bReady |	Set to TRUE by the object after a successful initialisation. Also tells the object to delete the buffers on destruction. Reset it at your peril.

//@mdata UINT | SRGraphDynamicData | m_nBufferSize | The size (in SRGDataBuffers) of the buffer.

//@mdata UINT | SRGraphDynamicData | m_nOutIndex | The index within the buffer from which data and hi-lo information is taken. Once an access has been made this index is 
//automatically incremented to the next position in the buffer. Overflow behaviour depends on the settings of <md SRGraphDynamicData::m_nGrowSize> and <md SRGraphDynamicData::m_bCyclic>

//@mdata UINT | SRGraphDynamicData | m_nInIndex | The index into which data is placed within the data buffer.  Once an access has been made this index is 
//automatically incremented to the next position in the buffer. Overflow behaviour depends on the settings of <md SRGraphDynamicData::m_nGrowSize> and <md SRGraphDynamicData::m_bCyclic>

//@mdata SRGDataBuffer* | SRGraphDynamicData | m_pBuffer | A pointer to a block of SRGraphDatabuffer structures which contain the data.


////////////////////////////////////////////////////////////////

//@class Manages the storage and manipulation of dynamic information in various formats.
class SRGraphDynamicData : public SRGraphData
{
//@access Public member functions
public:
	virtual void Roll(double dValue=0.0,BOOL bDirection=FALSE);
	//@cmember Enables cycling of the buffer indices.
	virtual void SetCyclic(BOOL b=TRUE);
	//@cmember Interrogates the CScale object attached to a particular value
	virtual CScale GetHiLoData();
	//@cmember Positions the input pointer within the buffer
	virtual BOOL SetInputPointer(UINT index);
	//@cmember Sets the input pointer to zero
	virtual void ResetInputPointer();
	//@cmember Determines whether the hi-lo gathering is enabled
	virtual BOOL GetHiLoFunction();
	//@cmember Enables or disables the hi-lo gathering feature
	virtual void SetHiLoFunction(BOOL b=TRUE);
	//@cmember Positions the output pointer within the buffer
	virtual BOOL SetOutputPointer(UINT index);
	//@cmember Returns a value from the buffer. The log value is available on request
	virtual double GetValue(BOOL bLogData=FALSE);
	//@cmember Resets the output pointer to zero
	virtual void ResetOutputPointer();
	//@cmember Interrogates the BufferSize member
	virtual UINT GetBufferSize();
	//@cmember Interrogates the GrowSize member
	virtual UINT GetGrowSize();
	//@cmember Initialises the GrowSize member
	virtual void SetGrowSize(UINT size);
	//@cmember Constructor
	SRGraphDynamicData();
	//@cmember Destructor
	virtual ~SRGraphDynamicData();
	//@cmember returns TRUE because this is a dynamic data storage object
	virtual BOOL IsDynamic(){return TRUE;}
	//@cmember Allocates or re-allocates a data buffer
	virtual void SetBufferSize(UINT nSize);
	//@cmember Places a char value into the buffer
	virtual void SetValue(char value);
	//@cmember Places an integer value into the buffer
	virtual void SetValue(int value);
	//@cmember Places a double precision floating point value into the buffer
	virtual void SetValue(double value);


	DECLARE_SERIAL(SRGraphDynamicData);

	//@cmember Manages storage and retrieval of this object
	virtual void Serialize(CArchive &ar);



//@access protected data members
protected:
	//@cmember Pointer to a block of CScale objects
	CScale *m_HiLoBuffer;
	//@cmember The type of data (int,char or double) that this object stores.
	BOOL m_bHiLo;
	int m_nDataType;
	//@cmember The size by which the buffer grows in case of overflow
	UINT m_nGrowSize;
	//@cmember TRUE if the buffer remains a fixed size and pointers re=cycle to the beginning
	BOOL m_bCyclic;
	//@cmember TRUE if this object has been properly initialised with a data buffer.
	BOOL m_bReady;
	//@cmember The number of SRGDataBuffers in storage.
	UINT m_nBufferSize;
	//@cmember Index of the output pointer
	UINT m_nOutIndex;
	//@cmember index of the input pointer.
	UINT m_nInIndex;
	//@cmember pointer to the stored data.
	SRGDataBuffer* m_pBuffer;	
};

#endif //__SRGRAPHDYNAMICDATA__

// Reset DLL declaration context


#undef AFX_DATA
#define AFX_DATA
#undef AFXAPP_DATA
#define AFXAPP_DATA NEAR
