//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

// @doc
// @module SRGraphException.cpp | 
// SRGraphException implementation file
//
// copyright <cp>Stingray Software 1996 
//
// Written by Bob Powell (bob@stingsoft.com)

#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"


IMPLEMENT_DYNCREATE(SRGraphException,CException)

//@func THROWS an exception of type SRGraphException
//@parm the integer number specifying the cause of the exception
//@devnote Although other methods are available it is reccommended that you use this global
//function to throw SRGraph exceptions. Furthermore you are advised to use the TRY and CATCH
//macros to initiate and trap your exceptions because these macro's also handle the memory 
//management of objects based on CException. Not using the macro form and relying on the C++
//try and catch keywords will make your software responsible for such internal management functions.
void SRThrowGraphException(int cause)
{
	THROW(new SRGraphException(cause));
}



//@mfunc To decode error reporting strings according to the cause of this exception
//@parm a pointer to a string which will recieve the report text.
//@parm the maximum length of the report text. If the actual string is too long then it will be truncated.
//@parm pointer to a variable which will recieve the help context generated from the error report.
BOOL SRGraphException::GetErrorMessage(LPSTR lpszError, UINT nMaxError, PUINT pnHelpContext)
{
	CString message;
	if(lpszError==NULL)
		return FALSE;
	if(m_nCause!=0)
	{
		UINT ErrorID=GetErrorID();
		if(message.LoadString(ErrorID))
		{
			if(pnHelpContext!=NULL)
				*pnHelpContext=ErrorID;
			int size=nMaxError>(UINT)message.GetLength() ? message.GetLength() : nMaxError;
			strncpy(lpszError,(const char *)message,size);
			lpszError[size]='\0';
			return TRUE;
		}
	}
	*lpszError='\0';
	if(pnHelpContext!=NULL)
		*pnHelpContext=0;
	return FALSE;
}


//@mfunc Constructor. Initialses the m_nCause member with the explicit information passed.
//@parm The cause of this exception
SRGraphException::SRGraphException(int cause)
{
	m_nCause=cause;
}

//@mfunc Translates the numeric m_nCause number into a string resource ID which is then used to
// display a message about the exception
UINT SRGraphException::GetErrorID()
{
	switch(m_nCause)
	{
	case none:
		return IDS_SRGEX_NONE;
	case generic:
		return IDS_SRGEX_GENERIC;
	case group:
		return IDS_SRGEX_GROUP;
	case index:
		return IDS_SRGEX_INDEX;
	case dynamic:
		return IDS_SRGEX_DYNAMIC;
	case dynamicType:
		return IDS_SRGEX_DYNAMICTYPE;
	case component:
		return IDS_SRGEX_COMPONENT;
	case fillstyle:
		return IDS_SRGEX_FILLSTYLE;
	case font:
		return IDS_SRGEX_FONT;
	case legend:
		return IDS_SRGEX_LEGEND;
	case data:
		return IDS_SRGEX_DATA;
	case command:
		return IDS_SRGEX_COMMAND;
	case border:
		return IDS_SRGEX_BORDER;
	case dynamicNotready:
		return IDS_SRGEX_DYNAMICNOTREADY;
	case type:
		return IDS_SRGEX_TYPE;
	case syntax:
		return IDS_SRGEX_SYNTAX;
	case label:
		return IDS_SRGEX_LABEL;
	case drawing:
		return IDS_SRGEX_DRAWING;
	case wiget:
		return IDS_SRGEX_WIGET;
	case polygon:
		return IDS_SRGEX_POLYGON;
	case graphtype:
		return IDS_SRGEX_GRAPHTYPE;
	case logscale:
		return IDS_SRGEX_LOGSCALE;
	case odd:
		return IDS_SRGEX_ODD;  
	case null:
		return IDS_SRGEX_NULL;
	default:
		return UserExceptionID(); 
	}
}

//@mfunc Displays a message box containing text for the current exception.
void SRGraphException::ShowErrorMessage()
{
  char message[256];
  GetErrorMessage(message,256,NULL);
  AfxMessageBox(message,MB_ICONEXCLAMATION);
}

//@mfunc If an unknown exception id is used then this function will be called. Override this function to return the string ID of the required text.
//@rdesc the resource string id of the text associated with this exception.
UINT SRGraphException::UserExceptionID()
{
	ASSERT(FALSE); // override this routine to handle your own exception text reporting
	return IDS_SRGEX_DEFAULT;
}
