//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////
//@doc
/////////////////////////////////////////////////////////////


//@module SRGPosn.h |
//Copyright <cp>Stingray Software 1996-1997. Written by Bob Powell ( bob@stingsoft.com )


#ifdef _OBJCHART_DLL
	#undef AFXAPP_DATA
	#define AFXAPP_DATA AFXAPI_DATA
	#undef AFX_DATA
	#define AFX_DATA OC_DATA
#endif

#ifndef __SRGRAPHPOSITION__
#define __SRGRAPHPOSISION__

//////////////////////////////////////////////////////
//  Autoduck comment block for SRGraphPosition


//@mdata int | SRGraphPosition | m_nGroup | The zero based index of the group in which the point was found
//@rem This is only valid if <mf SRGaphPosition::HasGroup> returns TRUE	 Retrieve this value using <mf SRGraphPosition::GetGroup>

//@mdata int | SRGraphPosition | m_nIndex |	The zero based index of the group in which the point was found
//@rem This is only valid if <mf SRGaphPosition::HasIndex> returns TRUE. Retrieve this value using <mf SRGraphPosition::GetIndex>


//@mdata double | SRGraphPosition | m_dValueY |
//@rem This is only valid if <mf SRGaphPosition::HasValueY> returns TRUE. Retrieve this value using <mf SRGraphPosition::GetValueY>


//@mdata double | SRGraphPosition | m_dValueX |
//@rem This is only valid if <mf SRGaphPosition::HasValueX> returns TRUE. Retrieve this value using <mf SRGraphPosition::GetValueX>


//@mdata SRGraph *| SRGraphPosition | m_pParent | Used internally to allow access to the graph being interrogated.


//@mdata DWORD | SRGraphPosition | m_dwFlags | A set of bitwize flags which communicate data to and from this class<nl>
//CX_POSITION_MASK			0x0000000f	Bitmask for separating the position code<nl>
//<nl>
//CX_POSITION_NONE			0x00000000	<nl>
//CX_POSITION_SINGLE		0x00000001	 position is for a single component. The m_pSubject will point to an SRGraphComponent<nl>
//CX_POSITION_LIST			0x00000002	 for a list of components the m_pSubject will point to a CPtrList containing multiple SRGraphComponent pointers<nl>
//<nl>
//CX_POSITION_VALUEY		0x00000010	 The Y value is known<nl>
//CX_POSITION_VALUEX		0x00000020	 The X value is known<nl>
//CX_POSITION_GROUP			0x00000040	 The group is known	 
//CX_POSITION_INDEX			0x00000080	 The index is known<nl>
//<nl>
//CX_POSITION_SELECT_MASK	0x000f0000						   <nl>
//CX_POSITION_SELECT_H		0x00000000	 Selection is horizontal (classic)<nl>
//CX_POSITION_SELECT_V		0x00010000	 selection is vertical (yx)<nl>
//CX_POSITION_SELECT_B		0x00020000	 selection is both (x,y scatter)<nl>
//CX_POSITION_SELECT_C		0x00030000	 Selection is circular (webs)<nl>


//@mdata void *| SRGraphPosition | m_pSubject | Points to the subject of the interrogation
//@rem This may either be a pointer to an SRGraphComponent object or a pointer to a CPtrList containing in turn pointers to multiple SRGraphComponent objects. 
//To ascertain the position type (ie single or multiple SRGraphDisplay hits) call GetPosType first. If the position is single (CX_POSITION_SINGLE)
//then use GetComponent() to retrieve the pointer. If its multiple you must use GetList() to retrieve the CPtrList pointer. Effectively these routines 
//simply return the m_pSubject pointer cast to the correct type. However some error checking and verification are done first so its best to use the routines provided.


//@mdata POSITION | SRGraphPosition | m_pos | Currently reserved and unused


//@mdata CWnd *| SRGraphPosition | m_pCWnd | The calculations are made relative to the CWnd which is displaying the 
//graph. This is the stored CWnd pointer

//@mdata CPoint | SRGraphPosition | m_Point | The mouse position or the x,y coordinates to test for hits, positions and values.



//////////////////////////////////////////////////////

//@class Manages the passing of position information from the graph to the user. Used primarily in data dragging 
//this class defines where in the graph data structure the mouse pointer is.
//@rem To use graph positions you simply call the constructor with a pointer to the graph being interrogated and a mouse pointer position. 
//The SRGraphPosition system then scans the graph structures taking into account the type of graph display and fills its member variables with pertinent information. 
//The calling function or class then interrogates the position to find out what type of information is available. For example calling SRGraphPosition::HasValueX() will 
//return TRUE if the value currently stored in the m_dValueX data member was obtained on the last scan. If this routine returns FALSE the data in that member will be 
//All data members are protected and should be accessed using the correct methods.
//un-defined and un reliable.
class SRGraphPosition : public CObject
{
//@access Protected data members
protected:
	  //@cmember The calculated group in which the point was found
	  int m_nGroup;
	  //@cmember The calculated group in which the point was found
	  int m_nIndex;
	  //@cmember The Y value relative to the current graph axis at which the point was placed
	  double m_dValueY;
	  //@cmember The X value relative to the current graph axis at which the point was placed
	  double m_dValueX;
	  //@cmember A pointer to the graph being interrogated
	  SRGraph *m_pParent;
	  //@cmember Bitwize flags
	  DWORD m_dwFlags;
	  //@cmember A pointer to either the graph component (SRGrapDisplay) or a list of components in which the point was found
	  void *m_pSubject;
	  //@cmember The type of position for selection purposes
	  POSITION m_pos;
	  //@cmember A pointer to the CWnd being used to draw the graph
	  CWnd *m_pCWnd;
	  //@cmember The point at which calculations are to be made
	  CPoint m_Point;

//@access public member functions
public:
	//@cmember Sets the position of the hit-test point
	virtual void SetGraphPosition(CPoint p);
	//@cmember Returns TRUE if a Y scale value has been calculated
	virtual BOOL HasValueY();
	//@cmember Returns TRUE if an X scale value has been calculated
	virtual BOOL HasValueX();
	//@cmember Returns TRUE if an index has been calculated
	virtual BOOL HasIndex();
	//@cmember Returns TRUE if an group has been calculated
	virtual BOOL HasGroup();
	//@cmember Allows the addition of a user defined calculation routine for your own graph types
	virtual void CalcUserDisplayPosition(SRGraphDisplay *pD);
	//@cmember Calculates the data from the given display. Data returned will depend on the type of graph being used	
	virtual void CalcDisplayPosition(SRGraphDisplay *pD);
	//@cmember Calculates the positions and values with reference to the indexed display in the list
	virtual void CalcPos(int nDisplay=0);
	//@cmember Returns the position type.
	virtual DWORD GetPosType();
	//@cmember Returns the type of selection made (ie horizontal, vertical etc.)
	virtual DWORD GetSelectionType(){return m_dwFlags & CX_POSITION_SELECT_MASK;}
	//@cmember Sets the type of position to use as a basis for the calculations
	virtual void SetPosType(DWORD postype);
	//@cmember Constructor
	SRGraphPosition();
	//@cmember Constructor 
	SRGraphPosition(SRGraph *pGraph,CPoint point,CWnd *pWnd);
	//@cmember Destructor
	virtual ~SRGraphPosition();
	//@cmember,mfunc Method to set the group
	virtual void SetGroup(int g){m_nGroup=g;}
	//@cmember,mfunc Returns the group. Do not rely on this data unless <mf SRGraphPosition::HasGroup> returns TRUE.
	virtual int GetGroup(){return m_nGroup;}
	//@cmember,mfunc Method to Set the index
	virtual void SetIndex(int i){m_nIndex=i;}
	//@cmember,mfunc Returns the index  Don't rely on this data unless <mf SRGraphPosition::HasIndex> returns TRUE
	virtual int GetIndex(){return m_nIndex;}
	//@cmember,mfunc Method to set the X value
	virtual void SetValueX(double d){m_dValueX=d;}
	//@cmember,mfunc Method to set the Y value
	virtual void SetValueY(double d){m_dValueY=d;}
	//@cmember,mfunc Retrieves the calculated X value. Don't rely on this data unless <mf SRGraphPosition::HasValueX> returns TRUE
	virtual double GetValueX(){return m_dValueX;}
	//@cmember,mfunc Retreives the Y value  Don't rely on this data unless <mf SRGraphPosition::HasValueY> returns TRUE
	virtual double GetValueY(){return m_dValueY;}
	//@cmember Gets a pointer to the list of items in which the point is found
	virtual CPtrList *GetList();
	//@cmember Gets a pointer to the unique display component in which the point has been found.
	virtual SRGraphComponent *GetComponent();
};


#endif //__SRGRAPHPOSITION__

// Reset DLL declaration context

#undef AFX_DATA
#define AFX_DATA
#undef AFXAPP_DATA
#define AFXAPP_DATA NEAR


