//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////
//@doc
//@module SRGraph.cpp | Implementation file for the <f SRGraph> class.<nl>
//Copyright <cp>Stingray Software 1996. Written by Bob Powell (bob@stingsoft.com)

#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"



#ifdef _DEBUG

#define new DEBUG_NEW

#endif

//@mfunc Constructor.
//@rem Sets various default values.
SRGraph::SRGraph()
{
	
	m_bSignificantChange=TRUE;

	m_bIsEmpty=TRUE;
	m_nDefaultGroup=0;
#if __OG_SEE_DRAW__
	m_bBackgroundDraw=FALSE;
#else
	m_bBackgroundDraw=TRUE;
#endif
	m_bHintPending=FALSE;
	m_pHint=NULL;
	m_Style.SetType(CX_TYPE_GRAPH);
	m_bFeedbackLogging=TRUE;
	m_nCountryCode=1; // default to USA
	m_bUseMaxArea=FALSE;
	m_dwAlignInDC=0;
}


IMPLEMENT_SERIAL(SRGraph,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION)

//@mfunc Manages storage and retrieval of graph data
//@parm Reference to a CArchive object used in the save or load process.
void SRGraph::Serialize(CArchive& ar)
{

	CObject::Serialize(ar);

	m_ComponentList.Serialize(ar);
	m_Style.Serialize(ar);
	m_strTitle.Serialize(ar);

	if(ar.IsStoring())
	{
		ar << m_bBackgroundDraw << m_bIsEmpty << m_nDefaultGroup;
		ar << m_data.GetCount();

		for(POSITION p=m_data.GetHeadPosition();p!=NULL;)
		{
			SRGraphDataList* pDL=(SRGraphDataList*)m_data.GetNext(p);
			pDL->Serialize(ar);
		}
		ar << m_nCountryCode;
	}
	else
	{
		ar >> m_bBackgroundDraw >> m_bIsEmpty >> m_nDefaultGroup;
		// load in the data lists (if any)
		int count;

		ar >> count;
		
		while(!m_data.IsEmpty())
			delete ((SRGraphDataList *)m_data.RemoveHead());
		
		for(int x=0;x<count;x++)
		{
			SRGraphDataList *pDL=new SRGraphDataList;
			pDL->Serialize(ar);;
			m_data.AddTail(pDL);
		}
		
		// Reset the dynamic pointers
		for(POSITION p=m_ComponentList.GetHeadPosition();p!=NULL;)
			((SRGraphComponent *)m_ComponentList.GetNext(p))->SetParent(this);

		ar >> m_nCountryCode;

		m_bSignificantChange=TRUE;
	}
}



//@enum types of SRGraph string parsing
enum substring_types {
	ALL,		//@emem All types read
	NUMERIC,	//@emem expecting numeric data
	HEADER,		//@emem expecting header data
	SUBHEADER,	//@emem expecting sub header data
	DATA,		//@emem	accept only numeric data
	EMPTY,		//@emem this is just an empty cell
	SUBEND,		//@emem end of sub line
	END			//@emem end of data
};

//@mfunc Removes and deletes all data in the graph without destroying the
//SRGraph object.
//@see <md SRGraph::m_data> <md SRGraph::m_ComponentList> <md SRGraph::m_FeedbackList>
void SRGraph::KillGraph(void)
{
	m_strTitle.KillList();
	while(!m_data.IsEmpty())
		delete ((SRGraphDataList*)m_data.RemoveHead());

	while(!m_ComponentList.IsEmpty())
		delete ((SRGraphComponent *)m_ComponentList.RemoveHead());
	
	while(!m_FeedbackList.IsEmpty())
		delete ((SRGraphFeedback *)m_FeedbackList.RemoveHead());

	m_bIsEmpty=TRUE;
}

//@mfunc Set data value in the SRGraphData object
//@parm The zero based index of the <c SRGraphData> item
//@parm the zero based index of the group holding the list
//@parm The value to place in the dereferenced object
void SRGraph::SetValue(int x,int group,double number)
{
	//TRACE("Setting numeric data %g in group %d index %d\n",number,group,x);
	SRGraphDataList *gdl=GetGroup(group);
	ASSERT(gdl!=NULL); // why did this happen if the list was supposed to grow?
	gdl->GetIndex(x)->SetValue(number); // GetIndex grows the list too if needs be
	m_bIsEmpty=FALSE;
}

//@mfunc Set text annotation in the SRGraphData object referenced by group[x] where group
//is the relative position of SRDataList objects in the main m_data storage.
//@parm The zero based index of the <c SRGraphData> item
//@parm the zero based index of the group holding the list
//@parm The text to place in the annotation of the dereferenced object
void SRGraph::SetText(int x,int group,LPCSTR text)
{
	TRACE("Setting text data \"%s\" in group %d index %d\n",text,group,x);
	SRGraphDataList *gdl=GetGroup(group);
	ASSERT(gdl!=NULL); // why did this happen if the list was supposed to grow?
	gdl->GetIndex(x)->SetAnnotation(text); // GetIndex grows the list too if needs be
	SetSignificantChanges();		
}

//@mfunc Set text annotation in the SRGraphDataList object referenced by [group] where group
//is the relative position of SRDataList objects in the main m_data storage. This header may then
//be applied to the whole list of SRGraphData objects within that group.
//@parm The zero based index of the group which will recieve the header
//@parm The text of the new header.
void SRGraph::SetHeader(int group,LPCSTR text)
{
	TRACE("Setting header \"%s\" in group %d \n",text,group);
	SRGraphDataList *gdl=GetGroup(group);
	ASSERT(gdl!=NULL); // if this happened its a mystery because the list should grow. Maybe its a memory error.
	gdl->SetAnnotation(text);
	SetSignificantChanges();
}

//@mfunc Returns a pointer to the SRGraphDataList which holds the group.
//@parm The zero based index of the group which was requested
//@parm When TRUE and the group does not exist then new groups will be added 
//so that the function always returns a valid pointer. If FALSE and the group does
//not exist then an <c SRGraphException> is thrown
//@rdesc A pointer to the <c SRGraphDataList> requested.
SRGraphDataList *SRGraph::GetGroup(int group,BOOL GrowList)
{
	if(group<0)
		SRThrowGraphException(SRGraphException::group);
	if(m_data.GetCount()<(group+1))
	{	// expand the size of the list if needed
		if(GrowList)
		{
			while(m_data.GetCount()<(group+1))
				m_data.AddTail(new SRGraphDataList);
		}
		else
			SRThrowGraphException(SRGraphException::group); // a mistake has been made
	}

	POSITION p=m_data.FindIndex(group);
	ASSERT(p!=NULL); // this should never happen because the group is valid or the list grew if needed
	return (SRGraphDataList *)m_data.GetAt(p);
}

//@mfunc Reads a string from a tab delimited text input and returns data and analysis of the type of data via the
//pointers supplied.
//@parm Pointer to the text being scanned.
//@parm Reference to a CString object which will store the actual text found (if any).
//@parm pointer to the offset index into the data referenced by parameter 1. NOTE Do not alter this index 
//except to zero it at the beginning of a parsing run.
//@parm Pointer to a double used to store the number resulting from scanning the text.
//@rdesc the return codes are <nl> ALL.. a text string has been read <nl> EMPTY a tab character was found without preceeding text<nl>
//SUBEND A carriage return or carriage return/linefeed pair was read. <nl> END The end of text ('\0') mark was found. <nl>
//NUMERIC a number was successfully converted from the text string.
int SRGraph::SubString(LPCSTR text,CString &section,DWORD *index,double *number)
{
	section.Empty(); // clear the string result
	float temp;
	int i=*index;
	switch(text[i])
	{
	case '\t':
		i++;
		*index=i;
		return EMPTY;
	case '\r':
		i++;
		if(text[i]=='\n')
			i++;
		*index=i;
		return SUBEND;
	case '\0':
		return END;
	default:
		do
			section+=text[i++]; // fetch all other characters from the string
		while(	(text[i]!='\t') &&
				(text[i]!='\r') &&
				(text[i]!='\0') &&
				(text[i]!='\n') );
		if(text[i]=='\t')
			i++; // step over the delimiter
		*index=i;
		if(sscanf((const char *)section,"%g",&temp)==1)
		{
			*number=(double)temp;
			return NUMERIC;
		}
		else
			return ALL;
		break;
	}

	ASSERT(FALSE); //Why am I here???
	return END;
}




//@mfunc To interpret a string of tab delimited text into an SRGraphData list.
//@parm Pointer to the text to parse. For a fuller description of this function 
void SRGraph::ParseText(LPCSTR text)
{
	int operation=ALL;	//operator type flag
	CString section;	//string containing the current text substring.
	DWORD index;		//index into the string of text.
	double number;		//numeric data
	int x,group,heading;//array counters
	BOOL ReadData=FALSE;//header pass semaphore
	BOOL AutoNumber=FALSE;

	//basic operation of this parser.

	//if the first data read is numeric then only read numeric data thereafter
	//otherwise scan in the first line as a set of data sub headings and store them in the first data group
	//Thereafter read in one sub heading followed by "n" numeric values. Store the heading in the main list
	//for that group and the data at the correct indices.	
	
	
	if(!m_bIsEmpty)
		KillGraph();

	// starting with an empty graph structure
	index=0; //ensure an index to the start of the string 

	//TRACE("SRGraph Beginning text scan\n");

	int op=SubString(text,section,&index,&number);

	switch(op)
	{
	case NUMERIC:
		//TRACE("Found numeric data.. Accepting only numeric from now on\n");
		operation=DATA;			// accept only numeric data from now on
		AutoNumber=TRUE;
		x=group=heading=0;			// begin array initialisation
		SetValue(x,group,number);	// store this number in the array
		x++;					// increase x array index
		ReadData=TRUE;			// Dont gather a header in the case of data only matrices.
		break;
	case SUBEND:
	case END:
		// error. This is bad
		ASSERT(FALSE); // first data is an end
		break;
	case EMPTY:
		//TRACE("Found empty data.. Assuming header scan\n");
		operation=HEADER;		//scan a header
		x=group=heading=0;
		break;
	default:
		// data is 
		//TRACE("Setting title to %s.. Starting header scan\n",section);
		operation=HEADER;
		x=group=heading=0;
		m_strTitle.SetAnnotation(section);
		break;
	}

	do
	{
		op=SubString(text,section,&index,&number);
		if(op!=END)
		{ // not the end of text yet
			if(op==SUBEND)
			{
				//TRACE("End of subline\n");
				x=0;	// reset x index
				if(ReadData)
					heading++;
				if((operation!=HEADER) && ReadData)
					group++; //only increase groups after all headers have been read in
				if(	(operation==HEADER) ||
					(operation==NUMERIC))
					operation=SUBHEADER;
				ReadData=TRUE;
			}
			else
			{
				switch(operation) // what is the expectation?
				{
				case HEADER:
					switch(op)
					{
					case EMPTY:
						//TRACE("Skipping blank header to %d,%d\n",x,group);
						x++;
						break;
					case ALL:
					case NUMERIC:
						//TRACE("Storing data subheader ( %s )to %d,%d\n",section,x,group);
						SetText(x,group,section);
						if(op==ALL)
							GetGroup(group)->GetIndex(x)->SetNull(TRUE);
						x++;
						break;
					}
					break;
				case DATA:
					if(op==NUMERIC)
					{
						//TRACE("Storing data to %d,%d\n",x,group);
						SetValue(x,group,number);
						x++;
					}
					else
					{
						//TRACE("Storing blank data to %d,%d\n",x,group);
						GetGroup(group)->GetIndex(x)->SetNull(TRUE);
						x++;
					}
					break;
				case SUBHEADER:
					switch(op)
					{
					case EMPTY:
						//TRACE("Skipping blank sub heading %s for %d.. Switching to NUMERIC\n",section,heading);
						break;
					case ALL:
					case NUMERIC:
						//TRACE("Storing sub heading %s for %d.. Switching to NUMERIC\n",section,heading);
						SetHeader(heading,section);
						break;
					}
					operation=NUMERIC;
					break;
				case NUMERIC:
					switch(op)
					{
					case ALL:
						//TRACE("Skipping non numeric value for %d,%d\n",number,x,group);
						GetGroup(group)->GetIndex(x)->SetNull(TRUE);
						x++;
						break;
					case EMPTY:
						//TRACE("Skipping blank  value for %d,%d\n",number,x,group);
						SetValue(x,group,0);
						GetGroup(group)->GetIndex(x)->SetNull(TRUE);
						//TRACE("And setting data object to null\n");
						x++;
						break;
					case NUMERIC:
						//TRACE("Storing value %g to %d,%d\n",number,x,group);
						SetValue(x,group,number);
						x++;
						break;
					}
					break;
				}
			}
		}
	}
	while(op!=END);
	//TRACE("End of input\n");
	m_bIsEmpty=FALSE;
	if(AutoNumber)
	{
		CString headerText;
		for(int n=0;n<GetGroupCount();n++)
		{
			headerText.Format("%d",n);
			SetHeader(n,headerText);
		}
		for(n=0;n<GetMaxXDivisions();n++)
		{
			int num=n;
			headerText.Empty();
			index=0;
			do
			{
				headerText+=(char)((num % 26) + 'A');
				num/=26;
			}
			while(num);
			headerText.MakeReverse();
			SetText(n,0,headerText);
		}
	}
}


//@mfunc Cleans up and shuts down the SRGraph data structures.
//@rem Deletes all data from m_data and all the components from m_ComponentList
SRGraph::~SRGraph()
{
	while(!m_data.IsEmpty())
		delete ((SRGraphDataList *)m_data.RemoveHead());
	while(!m_ComponentList.IsEmpty())
		delete ((SRGraphComponent *)m_ComponentList.RemoveHead());
	while(!m_FeedbackList.IsEmpty())
		delete ((SRGraphFeedback *)m_FeedbackList.RemoveHead());
}

//@mfunc Walks all data lists and totals max and min values of all data. Use this to 
//@parm TRUE if the Hi-Lo history is to be included (defaults to FALSE)
//determine the maximum Y size of a graph.
//@rdesc A CScale object containing the minimum and maximum values found in the graph
CScale SRGraph::GetAutoScaleRange(BOOL HiLo)
{
	POSITION p,q;
	CScale s;

	if(!m_data.IsEmpty())
	{
		for(p=m_data.GetHeadPosition();p!=NULL;)
		{
			SRGraphDataList *gdl=(SRGraphDataList *)m_data.GetNext(p);
			for(q=gdl->GetHeadPosition();q!=NULL;)
			{
				if(HiLo)
				{
					s+=((SRGraphData *)gdl->GetAt(q))->GetHiLoValue()->Max();
					s+=((SRGraphData *)gdl->GetNext(q))->GetHiLoValue()->Min();
				}
				else
					s+=((SRGraphData *)gdl->GetNext(q))->GetValue();
			}
		}
	}

	return s;
}

//@mfunc Returns the largest count of SRGraphData objects from all of the SRGraphDataLists
//held in the main m_data list of lists.
//@rdesc The count of the longest list
//@rem Use this, for example, to determine the number
//of divisions across the  X axis of a graph.
int SRGraph::GetMaxXDivisions()
{
	int DivisionCount=0;
	POSITION p;
	for(p=m_data.GetHeadPosition();p!=NULL;)
	{
		SRGraphDataList *gdl=(SRGraphDataList *)m_data.GetNext(p);
		DivisionCount=max(DivisionCount,gdl->GetCount());
	}
	return DivisionCount;
}

//@mfunc Returns the total number of SRGraphDataList objects in storage. 
//Use this to determine the number of categories in a graph.
int SRGraph::GetGroupCount()
{
	return m_data.GetCount(); //This count is the "y" width of the array
}

//@mfunc To fetch a string of label text from the graph.
//@parm country code of title for use with international annotation system (default=1)
//@rdesc Reference to a CString containing the text or ""
//@see <md SRGraph::m_strTitle>
CString& SRGraph::GetGraphTitle(int code)
{
	return m_strTitle.GetAnnotation(code);
}

//@mfunc fetches label text from a given <c SRGraphData> object referenced by group, index and country code.
//@parm The zero based index of the data group.
//@parm The zero based index of the data within the group
//@parm country code of text for use with international annotation system
//@rdesc Reference to a CString object containing the text or ""
CString& SRGraph::GetText(int group,int index,int code)
{
	if(GetGroupCount()!=0)
	{
		SRGraphDataList *gdl=GetGroup(group,FALSE);
		if(gdl!=NULL)
		{
			SRGraphData *gd=gdl->GetIndex(index,FALSE);
			if(gd!=NULL)
				return gd->GetAnnotation(code);
		}
	}
	m_strEmptyString.Empty();
	return m_strEmptyString;
}

//@mfunc To fetch the header string from a group
//@parm the group from which to take the header
//@parm optional country code
//@rdesc A reference to a string containing the header text
CString& SRGraph::GetHeader(int group,int code)
{
	SRGraphDataList *gdl=GetGroup(group,FALSE);
	ASSERT(gdl!=NULL);
	return gdl->GetAnnotation(code);
}

char CommandSet[20][30]=
{ 
	"surface",
	"background",
	"title",
	"legend",
	"data",
	"rect",
	"font",
	"style",
	"label",
	"labelblock",
	"annotation",
	"endblock",
	"",
	""
	 };

#define MaxCommand 12


//@mfunc To read in template information
//@parm Pointer to a template resource name or template file name
//@rem Templates provide a quick and simple method of defining the look of your graph.
// This routine reads and translates templates which may be stored as resources or as files.
void SRGraph::ReadTemplate(LPCSTR TemplateName)
{
	HINSTANCE hInst=AfxFindResourceHandle(TemplateName,"SRGRAPH_TEMPLATE");
	HRSRC hRes=FindResource(hInst,TemplateName,"SRGRAPH_TEMPLATE");
	HGLOBAL hMem;
	LPSTR Text;
	BOOL LoadedResource=FALSE;
	char line[256];


	TRY
	{
		if(hRes==NULL)
		{
			//assume TemplateName was a filename and load it into memory
			CFile tf(TemplateName,CFile::modeRead);
			DWORD memSize=tf.GetLength();
			hMem=GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,memSize);
			ASSERT(hMem!=NULL);
			Text=(LPSTR)GlobalLock(hMem);
			if(tf.ReadHuge(Text,memSize)!=memSize)
				AfxThrowFileException(CFileException::generic);
			tf.Close();
			GlobalUnlock(hMem);
			LoadedResource=FALSE;
		}
		else
		{
			hMem=LoadResource(hInst,hRes);
			LoadedResource=TRUE;
		}
		
		//hMem holds a handle to the global memory containing the resource.

		if(LoadedResource)
			Text=(LPSTR)LockResource(hMem);
		else
			Text=(LPSTR)GlobalLock(hMem);

		if(Text==NULL)
		{
			AfxMessageBox("Memory error locking graph template");
			return;
		}

		DWORD index=0, length;
		
		if(LoadedResource)
			length=SizeofResource(NULL,hRes);
		else
			length=GlobalSize(hMem);

		BOOL done=FALSE,bInBlock=FALSE;
		SRGraphComponent *component,*pBlockComponent=NULL;
		UINT nLastComponentType=0;

		do
		{
			// read in a line

			int c=0;
			do
				line[c++]=Text[index++];	// read all chars up to and including the carriage return
			while((c<255) && (line[c-1]!='\r') && (index<length));
			line[c-1]='\0';	// ensure termination of the line by replacing the CR with 0
			ASSERT(c!=255); //-->Template line must be less than 255 characters long
			if(index>=length)
				done=TRUE;
			if(Text[index]=='\n')
				index++;	// skip the line feed
			if(CString(line,2)!="//" && CString(line)!="")
			{	//only read non comment lines
				int n,i=0; //characters scanned and index accumulator
				char command[30];
				//This is not a comment line. Proceed with the parse

				if(sscanf(line,"%s%n",command,&n)!=1)
					SRThrowGraphException(SRGraphException::command);
				i+=n;

				int nCommand=0;
				BOOL foundCommand=FALSE;
				CString sCommand=command;
				do
				{
					if(sCommand==CommandSet[nCommand])
					{
						switch(nCommand)
						{
						case 0: // surface
							foundCommand=TRUE;
							break;
						case 1: // background
							component=new SRGraphBackground;
							nLastComponentType=IDS_SRG_BACKGROUNDTYPE;
							i+=component->ReadBasicParameters(&line[i]);
							component->ReadTemplateLine(&line[i]);
							AddComponent(component);
							foundCommand=TRUE;
							break;
						case 2: //titles
							component=new SRGraphTitle;
							nLastComponentType=IDS_SRG_TITLETYPE;
							i+=component->ReadBasicParameters(&line[i]);
							component->ReadTemplateLine(&line[i]);
							AddComponent(component);
							foundCommand=TRUE;
							break;
						case 3://Legend
							component=new SRGraphLegend;
							nLastComponentType=IDS_SRG_LEGENDTYPE;
							i+=component->ReadBasicParameters(&line[i]);
							component->ReadTemplateLine(&line[i]);
							AddComponent(component);
							foundCommand=TRUE;
							break;
						case 4: //Data
							component=new SRGraphDisplay;
							nLastComponentType=IDS_SRG_DISPLAYTYPE;
							i+=component->ReadBasicParameters(&line[i]);
							component->ReadTemplateLine(&line[i]);
							AddComponent(component);
							foundCommand=TRUE;
							break;
						case 5: // rect
						case 6: // font
						case 7:	// style
							break;
						case 8: // labels
							component=new SRGraphLabel;
							nLastComponentType=IDS_SRG_LABELTYPE;
							i+=component->ReadBasicParameters(&line[i]);
							component->ReadTemplateLine(&line[i]);
							if(bInBlock && pBlockComponent!=NULL)
							{
								if(pBlockComponent->IsKindOf(RUNTIME_CLASS(SRGraphLabelBlock)))
								{
									((SRGraphLabelBlock *)pBlockComponent)->AddLabel((SRGraphLabel *)component);
									component=NULL;
								}
							}
							else
								AddComponent(component);
							foundCommand=TRUE;
							break;
						case 9: // label block
							component=new SRGraphLabelBlock;
							nLastComponentType=IDS_SRG_LABELTYPE;
							pBlockComponent=component;
							bInBlock=TRUE;
							i+=component->ReadBasicParameters(&line[i]);
							component->ReadTemplateLine(&line[i]);
							AddComponent(component);
							foundCommand=TRUE;
							break;
						case 10: // annotation
							if(nLastComponentType==IDS_SRG_LABELTYPE)
							{
								int nCode,n;
								sscanf(&line[i],"%d %n",&nCode,&n);
								i+=n;
								((SRGraphLabel *)component)->SetAnnotation(nCode,(LPCSTR)&line[i]);
								foundCommand=TRUE;
							}
							break;
						case 11: // endblock
							bInBlock=FALSE;
							pBlockComponent=NULL;
							foundCommand=TRUE;
							component=NULL;
						}//switch(nCommand)
					}//if(sCommand=CommandSet[nCommand])
				}
				while((CString(CommandSet[nCommand++])!="") && !foundCommand);
				if(!foundCommand)
					SRThrowGraphException(SRGraphException::command);
			}//if(CString(line,2)!="//")
		}
		while(!done);

		if(LoadedResource)
			FreeResource(hMem);
		else
		{
			GlobalUnlock(hMem);
			GlobalFree(hMem);
		}
	}
	CATCH(CFileException,e)
	{
		AfxMessageBox("SRGraph template not found",MB_ICONSTOP);
		return;
	}
	AND_CATCH(SRGraphException,e)
	{
		switch(e->m_nCause)
		{
		case SRGraphException::command: // for the moment the only one that is handled... all the others report the error
			UserParser(line);
			break;
		default:
			e->ShowErrorMessage();
			TemplateSyntax(line);
			break;
		}
	}
	END_CATCH
}

//@mfunc reports on syntax errors in the template resource
//@parm pointer to the offending line
void SRGraph::TemplateSyntax(LPCSTR line)
{
	AfxMessageBox(CString("Syntax error in..>")+line,MB_ICONEXCLAMATION);
}


//@mfunc Commands not recognised by the standard parser send the whole line to the user parser for decoding.
// Override this function to add your own template command lines
//@parm pointer to the line containing unrecognised command;
BOOL SRGraph::UserParser(LPSTR line)
{
	line; //UNUSED
#ifdef _DEBUG
	ASSERT(FALSE); // You must override this to handle your own template file commands
#endif
	return FALSE;
}

//@mfunc Called by the CWnd which displays this graph. This routine walks the 
//linked list of components, drawing them one by one onto the CDC provided. 
//@parm Pointer to the device context used to paint the window
//@parm Pointer to the CWnd owning the device context.
//@devnote Depending on the state of the <md SRGraph::m_bBackgroundDraw> semaphore the CDC passed may in fact 
//be a memory DC used as a back buffer. Drawing onto the DC in memory hides the re-draw process from the user 
//and gives a smooth flicker-free operation.
void SRGraph::DrawComponentList(CDC *pDC,CWnd *pCWnd)
{
	if(m_ComponentList.IsEmpty())
		return;

	if(pDC->IsPrinting())
		SetSignificantChanges(); // if we print, we print everything.

	POSITION p;
	
	CDC *DrawDC;
	CBitmap shadow,*OldBitmap;
	
	CRect DCRect,r;
	CSize OldExtent;
	CPoint OldOrg;
	
	pCWnd->GetClientRect(&r);

	if(pDC->IsPrinting())
	{
		r.right=pDC->GetDeviceCaps(HORZRES);
		r.bottom=pDC->GetDeviceCaps(VERTRES);
	}

	DCRect=r;


	if(!m_bBackgroundDraw  || (pDC->IsPrinting() && !GetUseMaxArea()))
		DrawDC=pDC;
	else
	{
		DrawDC=new CDC;
		ASSERT(DrawDC!=NULL);
		DrawDC->CreateCompatibleDC(pDC);
		if(GetUseMaxArea())
		{
			if(GetMaxArea().Width()<r.Width() && GetMaxArea().Height()<r.Height())
				r=GetMaxArea();
			else
			{
				r.right=min(r.right,GetMaxArea().right);
				r.bottom=min(r.bottom,GetMaxArea().bottom);
			}
			
		}

		shadow.CreateCompatibleBitmap(pDC,r.Width(),r.Height());
		OldBitmap=(CBitmap *)DrawDC->SelectObject(&shadow);
		if(GetUseMaxArea())
		{	// the pDC needs clearing
			CPen *oldpen=(CPen *)pDC->SelectStockObject(NULL_PEN);
			CBrush *oldbrush=(CBrush *)pDC->SelectStockObject(WHITE_BRUSH);
			pDC->Rectangle(DCRect);
			pDC->SelectObject(oldbrush);
			pDC->SelectObject(oldpen);

		}
	}


	while(!m_FeedbackList.IsEmpty())
		delete ((SRGraphFeedback *)m_FeedbackList.RemoveHead());
	
	for(p=m_ComponentList.GetHeadPosition();p!=NULL;)
	{
		SRGraphComponent *gc=(SRGraphComponent *)m_ComponentList.GetNext(p);
		TRY{
			gc->Draw(DrawDC,pCWnd);
		}
		CATCH(SRGraphException,e)
		{
			e->ShowErrorMessage();
			p=NULL; //Force exit of the draw loop;
		}
		END_CATCH;
	}

	if(m_bHintPending)
		m_bHintPending=FALSE;

	if(!m_bBackgroundDraw  || (pDC->IsPrinting() && !GetUseMaxArea()))
	{
		m_bSignificantChange=FALSE;
		if(pDC->IsPrinting())
			SetSignificantChanges(); // everything gets updated when we swop DC's
		return;
	}

	if(GetUseMaxArea())
	{
		int nDestX=0,nDestY=0;

		switch(m_dwAlignInDC & 0x0000000f)
		{
		case CX_ALIGN_NONE:
			break;
		case CX_ALIGN_VTOP:
			nDestY=0;
			break;
		default:
		case CX_ALIGN_VCENTER:
			nDestY=(DCRect.Height()/2)-(r.Height()/2);
			break;
		case CX_ALIGN_VBOTTOM:
			nDestY=DCRect.bottom-r.Height();
			break;
		}

		switch(m_dwAlignInDC & 0x000000f0)
		{
		case CX_ALIGN_NONE:
			break;
		case CX_ALIGN_HLEFT:
			nDestX=0;
			break;
		default:
		case CX_ALIGN_HCENTER:
			nDestX=(DCRect.Width()/2)-(r.Width()/2);
			break;
		case CX_ALIGN_HRIGHT:
			nDestX=DCRect.Width()-r.Width();
			break;
		}
	
		pDC->BitBlt(nDestX,nDestY,r.Width(),r.Height(),DrawDC,0,0,SRCCOPY);
	}
	else
		pDC->BitBlt(0,0,r.Width(),r.Height(),DrawDC,0,0,SRCCOPY);
	
	DrawDC->SelectObject(OldBitmap);
	shadow.DeleteObject();
	DrawDC->DeleteDC();
	delete (DrawDC);
	m_bSignificantChange=FALSE;
}

//@mfunc Adds a component to the list of components to be displayed at each paint cycle.
//@parm pointer to the component to be added
//@rdesc A POSITION for this component in the list
//@see <md SRGraph::m_ComponentList> <c SRGraphComponent>
POSITION SRGraph::AddComponent(SRGraphComponent *gc)
{
	gc->SetParent(this);
	return m_ComponentList.AddTail(gc);
}

//@mfunc Walks all values in a group to find the total of all data items.
//@parm The group to total. Defaults to -1 in which case the m_nDefaultGroup member is used to denote the group;
//@rdesc The sum of all data values for this group.
double SRGraph::GetDataTotal(int group)
{
	POSITION q;
	double t=0;
	if(group==-1)
		group=m_nDefaultGroup;

	if(!m_data.IsEmpty())
	{
		SRGraphDataList *gdl=GetGroup(group);
		for(q=gdl->GetHeadPosition();q!=NULL;)
			t+=((SRGraphData *)gdl->GetNext(q))->GetValue();
	}

	return t;

}

//@mfunc Called to determine if this is a dynamic list
//@rdesc FALSE SRGraph is not dynamic
BOOL SRGraph::IsDynamic()
{
	return FALSE;
}


	IMPLEMENT_SERIAL(SRDynamicGraph,SRGraph,VERSIONABLE_SCHEMA)


//@mfunc Stores and retrieves data through serialization of the object.
//@parm A reference to the CArchive object used in the save or load process.
void SRDynamicGraph::Serialize(CArchive &ar)
{
		SRGraph::Serialize(ar);
}



//@mfunc Returns the maximum and minimum of all values contained in this graph
//@parm TRUE if the stored hi-lo history value is to be included.
//@rdesc A CScale object containing the minimum and maximum values.
//@rem This override of the SRGraph function totals values stored in dynamic graph data
//buffers.
//@see <c SRGDynamicData> <c CScale>
CScale SRDynamicGraph::GetAutoScaleRange(BOOL HiLo)
{
	POSITION p;
	CScale s;

	if(!m_data.IsEmpty())
	{
		for(p=m_data.GetHeadPosition();p!=NULL;)
		{
			SRGraphDataList *gdl=(SRGraphDataList *)m_data.GetNext(p);
			if(!gdl->IsDynamic())
				SRThrowGraphException(SRGraphException::generic);
			SRGraphDynamicData *ddp=(SRGraphDynamicData *)gdl->GetHead();
			if(ddp==NULL)
				SRThrowGraphException(SRGraphException::generic);
			ddp->ResetOutputPointer();
			for(UINT x=0;x<ddp->GetBufferSize();x++)
			{
				if(HiLo)
				{
					CScale tempScale=ddp->GetHiLoData();
					s+=tempScale.Max();
					s+=tempScale.Min();
				}
				else
					s+=ddp->GetValue();
			}
		}
	}

	return s;
}

//@mfunc Finds the total of all data items stored in this group.
//@parm The zero based group index.
//@rdesc The additive total of all numeric data items in the requested group 
//@rem Used to find the 100% value when individual indices need to show a percentage of the group total
double SRDynamicGraph::GetDataTotal(int group)
{
	double t=0;
	if(group==-1)
		group=m_nDefaultGroup;

	if(!m_data.IsEmpty())
	{
		SRGraphDataList *gdl=GetGroup(group);
		SRGraphDynamicData *ddp=(SRGraphDynamicData *)gdl->GetHead();
		ASSERT(ddp->IsKindOf(RUNTIME_CLASS(SRGraphDynamicData)));
		ddp->ResetOutputPointer();
		for(UINT q=0;q<ddp->GetBufferSize();q++)
			t+=ddp->GetValue();
	}

	return t;

}

//@mfunc Finds the maximum index used in this graph.
//@rdesc The actual count of indices.
//@rem Note that indices are zero based so the value returned by this routine must not be used
//to index a data value. index-1 is the last entry.
//@see <mf SRGraph::GetMaxXDivisions>
int SRDynamicGraph::GetMaxXDivisions()
{
	UINT DivisionCount=0;
	POSITION p;
	for(p=m_data.GetHeadPosition();p!=NULL;)
	{
		SRGraphDataList *gdl=(SRGraphDataList *)m_data.GetNext(p);
		SRGraphDynamicData* ddp=(SRGraphDynamicData*)gdl->GetHead();
		DivisionCount=max(DivisionCount,ddp->GetBufferSize());
	}
	return DivisionCount;
}

//@mfunc Interrogates the text label for the requested group
//@parm The zero based group index.
//@parm The zero based data index (ignored in this override)
//@parm	An international country code specifying the language selection
//@rdesc Reference to a CString object containing the text.
//@rem Overridden for compatibility with <mf SRGraph::GetText> this routine effectively 
//returns the header for the group. No labels or annotations are stored within the data 
//buffer of a dynamic graph.
//@see <c SRGraphAnnotation> <c SRGraphAnnotationList> <c SRGraphData>
CString& SRDynamicGraph::GetText(int group, int index, int code)
{

	index;//UNUSED
	if(GetGroupCount()!=0)
	{
		SRGraphDataList *gdl=GetGroup(group,FALSE);
		if(gdl!=NULL)
		{
			if(!gdl->IsEmpty())
			{
				SRGraphDynamicData *dd=(SRGraphDynamicData *)gdl->GetHead();
				if(dd!=NULL)
					return dd->GetAnnotation(code);
			}
		}
	}
	m_strEmptyString.Empty();
	return m_strEmptyString;
}


//@mfunc Overridden to trap calls to ParseText.
//@parm Pointer to the text data.
//@rem This routine ASSERTS in the debug version and then throws an SRGraphException. 
//Support for text parsing has been disabled in the dynamic graph system.
//@see <mf SRGraph::ParseText>
void SRDynamicGraph::ParseText(LPCSTR text)
{
	text;//UNUSED
	ASSERT(FALSE);
	// Dynamic graphs dont parse text.
	//You can do it all yourself by building the data bit by bit
	//or derive from this class and override this function
	SRThrowGraphException(SRGraphException::none);
}

//@mfunc Sets a number within the dynamic data buffer.
//@parm The zero based index of the data within the buffer.
//@parm The zero based index of the group.
//@parm the floating point double data value.
//@rem This function is overridden from the SRGraph version and also overloaded to 
//provide integer, double and char data storage.
//@see <mf SRGraph::SetValue>
void SRDynamicGraph::SetValue(int x,int group,double number)
{	// the X index is left in but ignored
	x; //UNUSED
	//TRACE("Setting numeric data %g in dynamic list %d\n",number,group);
	SRGraphDataList *gdl=GetGroup(group);
	ASSERT(gdl!=NULL); // why did this happen if the list was supposed to grow?
	SRGraphDynamicData *ddp;
	if(gdl->IsEmpty())
	{
		SRThrowGraphException(SRGraphException::dynamicNotready);
		return;
	}
	ddp=(SRGraphDynamicData *)gdl->GetHead();
	ASSERT(ddp!=NULL);
	ddp->SetValue(number);
}

//@mfunc Sets a number within the dynamic data buffer.
//@parm The zero based index of the data within the buffer.
//@parm The zero based index of the group.
//@parm the integer data value.
//@rem This function is overridden from the SRGraph version and also overloaded to 
//provide integer, double and char data storage.
//@see <mf SRGraph::SetValue>
void SRDynamicGraph::SetValue(int x,int group,int number)
{	// the X index is left in but ignored
	x;  //UNUSED
	//TRACE("Setting numeric data %d in dynamic list index %d\n",number,group);
	SRGraphDataList *gdl=GetGroup(group);
	if(gdl->IsEmpty())
	{
		SRThrowGraphException(SRGraphException::dynamicNotready); 
		return;
	}
	SRGraphDynamicData *ddp=(SRGraphDynamicData *)gdl->GetHead();
	ASSERT(ddp!=NULL);
	ddp->SetValue(number);
}

//@mfunc Sets a number within the dynamic data buffer.
//@parm The zero based index of the data within the buffer.
//@parm The zero based index of the group.
//@parm the char data value.
//@rem This function is overridden from the SRGraph version and also overloaded to 
//provide integer, double and char data storage.
//@see <mf SRGraph::SetValue>
void SRDynamicGraph::SetValue(int x,int group,char number)
{	// the X index is left in but ignored
	x; //UNUSED
	//TRACE("Setting numeric data %c in dynamic list %d\n",number,group);
	SRGraphDataList *gdl=GetGroup(group);
	ASSERT(gdl!=NULL); // why did this happen if the list was supposed to grow?
	if(gdl->IsEmpty())
	{
		SRThrowGraphException(SRGraphException::dynamicNotready);
		return;
	}
	SRGraphDynamicData *ddp=(SRGraphDynamicData *)gdl->GetHead();
	ASSERT(ddp!=NULL);
	ddp->SetValue(number);
}

//@mfunc Sets an annotation at the required group index.
//@parm An index to a data item (ignored in this override)
//@parm The zero based index of the group in which the annotation should be placed.
//@parm Pointer to the text data.
//@rem Overridden for compatibility with <mf SRGraph::SetText> this routine effectively 
//sets the header for the group. No labels or annotations are stored within the data 
//buffer of a dynamic graph.
//@see <c SRGraphAnnotation> <c SRGraphAnnotationList> <c SRGraphData>
void SRDynamicGraph::SetText(int x,int group,LPCSTR text)
{
	x; //UNUSED
	//TRACE("Setting text data \"%s\" in dynamic data group %d \n",text,group);
	SRGraphDataList *gdl=GetGroup(group);
	ASSERT(gdl!=NULL); // why did this happen if the list was supposed to grow?
	SRGraphDynamicData *ddp=(SRGraphDynamicData *)gdl->GetHead();
	ASSERT(ddp!=NULL);
	ddp->SetAnnotation(text);
}


//@mfunc Redraw hints generated by the document, view or owner window are sent here to 
//update the redrawing of components
//@parm Pointer to the owner window which generated or processes hints.
//@parm the hint code
//@parm Pointer to a hint object. (not implemented at this time) normally NULL.
void SRGraph::SetComponentHint(CWnd * pCWnd,LPARAM lHint,CObject * pHint)
{
	m_bHintPending=TRUE;
	m_lHint=lHint;
	m_pHint=pHint;
	for(POSITION p=m_ComponentList.GetHeadPosition();p!=NULL;)
	{
		SRGraphComponent *component=(SRGraphComponent *)m_ComponentList.GetNext(p);
		component->ProcessHint(pCWnd);
	}
}

//@mfunc Sets an annotation in the m_strTitle member
//@parm Pointer to the text to be set.
//@parm Country code of the text for use in multi language annotation.
void SRGraph::SetGraphTitle(LPCSTR title, int code)
{
	m_strTitle.SetAnnotation(code,title);
}

//@mfunc Simply removes and deletes all component objects in the m_ComponentList member.
//@rem Used to clear a list before loading a template. This operation does not effect the stored data.
//@see <md SRGraph::m_ComponentList>
void SRGraph::KillComponentList()
{
	while(!m_ComponentList.IsEmpty())
	  delete ((SRGraphComponent *)m_ComponentList.RemoveHead());
}

//@mfunc Takes the average of all data items within a specific group
//@parm Zero based index of the required group.
//@rdesc The sum of all data items devided by the value returned by GetMaxXDivisions.
//@rem Used to increase readability of isographic bar and area charts. The data sets 
//with the larger averages are displayed near the back of the chart and smaller averages 
//are moved to the front. This helps to prevent a particular data set from hiding another.
double SRGraph::GetGroupAverage(int group)
{
	if(group>GetGroupCount())
		return 0;
	
	SRGraphDataList *gdl=GetGroup(group);
	
	if(gdl->IsEmpty())
		return 0;
	
	double total=0;

	for(POSITION p=gdl->GetHeadPosition();p!=NULL;)
	{
		SRGraphData *gdp=(SRGraphData *)gdl->GetNext(p);
		total+=gdp->GetValue();
	}

	return total / gdl->GetCount();
}
						  

//@mfunc Interrogates the data values for a particular index across all groups<nl>
//Can THROW an SRGraphException if the index value is too large
//@parm The index to interrogate.
//@parm the lowest group in the range
//@parm the highest group in the range
//@rdesc The total of all values from the requested index taken from all groups.
double SRGraph::GetIndexTotal(int index,int nMinGroup,int nMaxGroup)
{
	if(index>=GetMaxXDivisions())
		SRThrowGraphException(SRGraphException::index);
	double total=0;
	if(nMaxGroup==0 && nMinGroup==0)
		nMaxGroup=GetGroupCount()-1;
	for(int g=nMinGroup;g<=nMaxGroup;g++)
		total+=GetGroup(g,FALSE)->GetIndex(index,FALSE)->GetValue();
	return total;
}

#ifdef _DEBUG

//@mfunc Object dumping. Debug only
//@parm Reference to a CDumpContext.
void SRGraph::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "Title annotations.." ;
	m_strTitle.Dump(dc);
	dc << "Data list..";
	m_data.Dump(dc);
	dc << "Component list..";
	m_ComponentList.Dump(dc);
	dc << "Style..";
	m_Style.Dump(dc);
	dc << "m_bIsEmpty=" << (m_bIsEmpty ? "TRUE " : "FALSE ");
	dc << "Background draw=" << (m_bBackgroundDraw  ? "TRUE " : "FALSE ");
	dc << "Hint pending=" << (m_bHintPending  ? "TRUE " : "FALSE ");
}

#endif //_DEBUG


//@mfunc Calculates the total value of all items in the graph.
//@rdesc The sum of all values in the graph
//@rem Used to form a baseline for percentage calculations
double SRGraph::GetGraphTotal()
{
	double total=0;

	for(POSITION g=m_data.GetHeadPosition();g!=NULL;)
	{
		SRGraphDataList *gdl=(SRGraphDataList *)m_data.GetNext(g);
		for(POSITION i=gdl->GetHeadPosition();i!=NULL;)
		{
			SRGraphData *pD=(SRGraphData *)gdl->GetNext(i);
			if(!pD->GetNull())
				total+=pD->GetValue();
		}
	}
	return total;
}

//@mfunc Calculates the average value of a particular range of indices within a group. Used to order groups in some 3D graphs
//@parm The lowest index
//@parm The highest index
//@parm the group which sould be totalled across all given indices
//@rdesc The average value of the specific  group from all indices in the range
double SRGraph::GetGroupAverage(int MinIndex,int MaxIndex,int Group)
{
	double total=0;

	for(int i=MinIndex;i<=MaxIndex;i++)
	{
		SRGraphData *pD=GetGroup(Group)->GetIndex(i);
		if(!pD->GetNull())
			total+=pD->GetValue();
	}
	return total/(1+(MaxIndex-MinIndex));
}

//@mfunc Adds a polygonal region to the feedback list.
//@parm A pointer to the CPoint list containing the polygon.
//@parm The number of elements in the list.
//@parm the group
//@parm and index which is to be associated with this feedback hot-spot
//@see <c SRGraphFeedback> <md SRGraph::m_FeedbackList>
void SRGraph::LogPoly(LPPOINT points,int pointCount,int group,int index)
{
	if(m_bFeedbackLogging)
		m_FeedbackList.AddTail(new SRGraphFeedback(points,pointCount,group,index,this));
}

//@mfunc Adds a point to the feedback list
//@parm the xy coordinate of the point.
//@parm the group
//@parm and index to be associated with this point
//@rem So that the point is easier to find with the mouse cursor a 5 pixel margin making a 10 by 10 area on screen is added to the feedback list.
//@see <c SRGraphFeedback> <md SRGraph::m_FeedbackList>
void SRGraph::LogPoint(CPoint point,int group,int index)
{
	if(m_bFeedbackLogging)
	{
		CPoint p[5];
		p[0]=CPoint(point.x-5,point.y-5);
		p[1]=CPoint(p[0].x+10,p[0].y);
		p[2]=CPoint(p[1].x,p[0].y+10);
		p[3]=CPoint(p[0].x,p[2].y);
		p[4]=p[0];

		m_FeedbackList.AddTail(new SRGraphFeedback((LPPOINT)&p,5,group,index,this));
	}
}

//@mfunc NEW 1.01 Adds a rectangle to the feedback list.
//@parm A CRect object describing the feedback area to add.
//@parm The group to associate with this feedback
//@parm The index to associate with this feedback.
//@see <mf SRGraph::LogPoly> <mf SRGraph::LogPoint> <c SRGraphFeedback>
void SRGraph::LogRectangle(CRect r,int group,int index)
{
	if(m_bFeedbackLogging)
	{
		r.NormalizeRect();

		CPoint p[5];
		p[0]=CPoint(r.left,r.top);
		p[1]=CPoint(r.right,r.top);
		p[2]=CPoint(r.right,r.bottom);
		p[3]=CPoint(r.left,r.bottom);
		p[4]=p[0];

		m_FeedbackList.AddTail(new SRGraphFeedback((LPPOINT)&p,5,group,index,this));
	}
}


//@mfunc Searches the feedback list for a feedback area
//@parm the X
//@parm and Y coordinates of the window position to search
//@rdesc If found, a pointer to the SRGraphFeedback object associated with the screen position. Otherwise NULL.
//@see <c SRGraphFeedback> <md SRGraph::m_FeedbackList>
SRGraphFeedback *SRGraph::GetFeedback(int x,int y)
{
	SRGraphFeedback *data=NULL;

	if(m_FeedbackList.IsEmpty())
		return data;

	for(POSITION p=m_FeedbackList.GetHeadPosition();p!=NULL;)
	{
		SRGraphFeedback *pFB=(SRGraphFeedback *)m_FeedbackList.GetNext(p);
		if(pFB->ptInFeedback(x,y))
			data=pFB;
	}
	//  NOTE that by looking at all the feedbacks and keeping a 
	//  record of the last on in the list to have a hit we are
	//	allowing the painters algorithm style drawing to sort out the z
	//  order of the objects for us. In other words the last one on the list 
	//  is the nearest one to us on screen
	return data;
}


//@mfunc Finds data totals for a block of data which may or may not cover the whole data array
//@parm The first group
//@parm The first index
//@parm the last group
//@parm the last index
//@rdesc The floating point total of the nominated block
double SRGraph::GetDataTotal(int mingroup, int  minindex, int maxgroup, int maxindex)
{
	double total=0;
	for(int g=mingroup;g<=maxgroup;g++)
	{
		for(int x=minindex;x<=maxindex;x++)
		{
			SRGraphData *pD=GetGroup(g)->GetIndex(x);
			if(!pD->GetNull())
				total+=pD->GetValue();
		}
	}
	return total;
}


//@mfunc Retrieves a component pointer from the m_ComponentList member. 
//If the Type parameter is set to a valid component type then the Nth component 
//of that type will be returned.
//@parm The number of the component to fetch
//@parm The component type to search for
//@rdesc A pointer to the required component. otherwise NULL.
//@see <md SRGraph::m_ComponentList>
SRGraphComponent *SRGraph::GetComponent(int nComponent,UINT Type)
{
	int CurrentComponent=0;
	for(POSITION p=m_ComponentList.GetHeadPosition();p!=NULL;)
	{
		SRGraphComponent *comp=(SRGraphComponent *)m_ComponentList.GetNext(p);
		if(Type==-1)
		{
			if(CurrentComponent==nComponent)
				return comp;
			CurrentComponent++;
		}
		else
		{
			if(comp->GetType()==Type)
			{
				if(CurrentComponent==nComponent)
					return comp;
				CurrentComponent++;
			}
		}
	}
	return NULL;
}

//@mfunc Allows access to component data ranges via this graph.
//@parm The count of the component
//@parm The index to be set as the minimum operational index
//@parm The type of component
//@rem This function delegates the actual setting of the components data span to the component specified by type and number
// Primarily used for automatic updates via the ComDoc system
//@see <mf SRGraphComponent::SetMinIndex> <mf SRGraph::GetComponent>
void SRGraph::SetComponentMinIndex(int nComponent,int index,UINT Type)
{
	SRGraphComponent *pC=GetComponent(nComponent,Type);
	if(pC!=NULL)
		pC->SetMinIndex(index);
}

//@mfunc Allows access to component data ranges via this graph.
//@parm The count of the component
//@parm The index to be set as the maximum operational index
//@parm The type of component
//@rem This function delegates the actual setting of the components data span to the component specified by type and number
// Primarily used for automatic updates via the ComDoc system
//@see <mf SRGraphComponent::SetMaxIndex> <mf SRGraph::GetComponent>
void SRGraph::SetComponentMaxIndex(int nComponent,int index,UINT Type)
{
	SRGraphComponent *pC=GetComponent(nComponent,Type);
	if(pC!=NULL)
		pC->SetMaxIndex(index);
}

//@mfunc Allows access to component data ranges via this graph.
//@parm The count of the component
//@parm The group to be set as the minimum operational group
//@parm The type of component
//@rem This function delegates the actual setting of the components data span to the component specified by type and number
// Primarily used for automatic updates via the ComDoc system
//@see <mf SRGraphComponent::SetMinGroup> <mf SRGraph::GetComponent>
void SRGraph::SetComponentMinGroup(int nComponent,int group,UINT Type)
{
	SRGraphComponent *pC=GetComponent(nComponent,Type);
	if(pC!=NULL)
		pC->SetMinGroup(group);
}

//@mfunc Allows access to component data ranges via this graph.
//@parm The count of the component
//@parm The group to be set as the maximum operational group
//@parm The type of component
//@rem This function delegates the actual setting of the components data span to the component specified by type and number
// Primarily used for automatic updates via the ComDoc system
//@see <mf SRGraphComponent::SetMaxGroup>  <mf SRGraph::GetComponent>
void SRGraph::SetComponentMaxGroup(int nComponent,int group,UINT Type)
{
	SRGraphComponent *pC=GetComponent(nComponent,Type);
	if(pC!=NULL)
		pC->SetMaxGroup(group);
}


//@mfunc Returns the POSITION of the first component as specified in the parameter Type
//@parm -1 to fetch the first position regardless of type. Otherwise use a valid SRGraphComponent type.
// IDS_SRG_DATATYPE returns the first data component <nl>
// IDS_SRG_LEGENDTYPE returns the first legend component <nl>
// IDS_SRG_BACKGROUNDTYPE returns the first background component <nl>
// IDS_SRG_TITLETYPE returns the first title component <nl>
//@rdesc A valid POSITION or NULL if unsuccessful
//@rem may be used to iterate the component list
//@see <mf SRGraph::GetComponentList> <md SRGraph::m_ComponentList> <mf SRGraph::GetNextComponent>
POSITION SRGraph::GetFirstComponentPosition(UINT Type)
{
	if(Type==-1)
		return m_ComponentList.GetHeadPosition();
	else
	{
		for(POSITION p=m_ComponentList.GetHeadPosition();p!=NULL;)
		{
			SRGraphComponent *pC=(SRGraphComponent *)m_ComponentList.GetAt(p);
			if(pC!=NULL)
			{
				if(pC->GetType()==Type)
					return p;
				else
					m_ComponentList.GetNext(p);
			}
		}
	}
	return NULL;
}

//@mfunc Searches the m_ComponentList member for the next qualifying SRGraphComponent based object
//@parm A POSITION within the list
//@parm -1 for the next component otherwise use a valid component type
// IDS_SRG_DATATYPE returns the next data component <nl>
// IDS_SRG_LEGENDTYPE returns the next legend component <nl>
// IDS_SRG_BACKGROUNDTYPE returns the next background component <nl>
// IDS_SRG_TITLETYPE returns the next title component <nl>
//@rdesc a pointer to a valid SRGraphComponent based object or NULL if unsuccessful
//@rem May be used to iterate the m_ComponentList when re-positioning or re-formatting components
//@see <mf SRGraph::GetFirstComponentPosition> <md SRGraph::m_ComponentList>
SRGraphComponent * SRGraph::GetNextComponent(POSITION& rPos,UINT Type)
{
	if(Type==-1)
		return (SRGraphComponent *)m_ComponentList.GetNext(rPos);
	for(rPos;rPos!=NULL;)
	{
		SRGraphComponent *pC=(SRGraphComponent *)m_ComponentList.GetNext(rPos);
		if(pC!=NULL)
		{
			if(pC->GetType()==Type)
				return pC;
		}
	}
	return NULL;
}

//@mfunc Get data value in the SRGraphData object
//@parm The zero based index of the <c SRGraphData> item
//@parm the zero based index of the group holding the list
//@rdesc The value held in the SRGraphData object referred to by group and index
//@see <c SRGraphData>
double SRGraph::GetValue(int index,int group)
{
	return GetGroup(group)->GetIndex(index)->GetValue();
}

//@mfunc Retrieves a pointer to the m_Style member
//@rdesc The required pointer. It is safe to store this pointer while the graph is in scope
//@rem You should not use SRGraph.m_Style or SRGraph->m_Style directly unless your libraries are statically linked
//@see <c SRGraphStyle> <md SRGraph::m_Style>
SRGraphStyle *SRGraph::GetStyle()
{
	return &m_Style;
}

//@mfunc Gets a pointer to this graphs m_ComponentList data member
//@rdesc A pointer to a CObList object
//@rem Reccomended method for finding the graphs component list. I t is safe to store this pointer while the graph is in the current scope
//@see <md SRGraph::m_ComponentList>
CObList * SRGraph::GetComponentList()
{
   return &m_ComponentList;
}

//@mfunc Gets a pointer to this graphs m_FeedbackList data member
//@rdesc A pointer to a CObList object
//@rem Reccomended method for finding the graphs feedback list. I t is safe to store this pointer while the graph is in the current scope
//@see <md SRGraph::m_FeedbackList>
CObList * SRGraph::GetFeedbackList()
{
   return &m_FeedbackList;
}

//@mfunc Gets a pointer to this graphs m_data data member
//@rdesc A pointer to a CPtrList object
//@rem Reccomended method for finding the graphs data list. I t is safe to store this pointer while the graph is in the current scope
//@see <md SRGraph::m_data>
CPtrList * SRGraph::GetDataList()
{
	return &m_data;
}

//@mfunc Retrieves the first position from the m_data member
//@rdesc a valid POSITION or NULL if unsuccessful
//@rem May be used to iterate the m_data list
//@see <mf SRGraph::GetDataList> <md SRGraph::m_data> <mf SRGraph::GetNextData>
POSITION SRGraph::GetFirstDataPosition()
{
	return m_data.GetHeadPosition();
}

//@mfunc Fetches SRGraphDataList objects from the m_data list
//@parm a valid POSITION for this list
//@rdesc a void pointer to an SRGraphDataList based object
//@rem Used to iterate the m_data list You should not use a direct reference to SRGraph.m_data or SRGraph->m_data
//@see <mf SRGraph::GetFirstDataPosition> <md SRGraph::m_data>
void * SRGraph::GetNextData(POSITION& p)
{
	return m_data.GetNext(p);
}

//@mfunc Returns a pointer to the m_strTitle annotation list
//@rdesc a pointer to the SRGraphAnnotationList held in m_strTitle
//@rem You should not use SRGraph->m_strTitle or SRGraph.m.Title unless the Objective Chart libraries are statically linked
// It is safe to store this pointer while the graph remains in scope
//@see <c SRGraphAnnotationList> <md SRGraph::m_strTitle>
SRGraphAnnotationList * SRGraph::GetTitle()
{
	return &m_strTitle;
}


//@mfunc Sets the style of a specified range of data items.
//@parm The first group
//@parm the last group
//@parm the first index
//@parm the last index
//@parm The pointer to the pattern style
//@parm TRUE if the styles in the data lists are to be set too. (default FALSE)
//@rem Any block of data items of group headers may be set to a particular style using this function. You must create and set a style and pass a pointer to this routine along with 
//the range of the desired block of data items which are to be updated. The range may contain -1 at any position whereupon a sensible range will be substituted. This substitutuion 
//will be 0 in the case of -1 in nStartGroup or nStartIndex or the last group or index for -1 in any of the end ranges.<nl>
//If the bSetHeaders flag is TRUE then the style for the selected range of groups will also be updated.
void SRGraph::SetGlobalStyle(int nStartGroup,int nEndGroup,int nStartIndex,int nEndIndex,SRGraphStyle *ptrStyle,BOOL bSetHeaders)
{
	if(nStartGroup==-1)
		nStartGroup=0;
	if(nEndGroup==-1)
		nEndGroup=GetGroupCount()-1;
	if(nStartIndex==-1)
		nStartIndex=0;
	if(nEndIndex==-1)
		nEndIndex=GetMaxXDivisions()-1;

	for(int CurrentSet=nStartGroup;CurrentSet<=nEndGroup;CurrentSet++)
	{
		BOOL GroupSet=FALSE ;
		for(int x=nStartIndex;x<=nEndIndex;x++)
		{
			// The system will try to set all those data items which exist. 
			// If they dont exist the list will not be "grown" to accommodate
			// but no errors will be flagged

			TRY
			{
				SRGraphDataList *pDL=GetGroup(CurrentSet,FALSE);
				if(bSetHeaders && !GroupSet)
				{
					*pDL->GetStyle()=*ptrStyle;
					GroupSet=TRUE;
				}
				SRGraphData *pD=pDL->GetIndex(x,FALSE);
				*pD->GetStyle()=*ptrStyle;
			}
			CATCH(SRGraphException,e)
			{
				if(e->m_nCause==SRGraphException::group || e->m_nCause==SRGraphException::index)
					TRACE("Ignoring non fatal error at location %d,%d\n",x,CurrentSet);
				else
					e->ShowErrorMessage();
			}
			END_CATCH
		}
	}
	SetSignificantChanges();
}

//@mfunc Used to flag a noticable change in the data structure.
//@rem Component's may or may not take notice of thier significant change flags at the next re-draw cycle. 
//@see <mf SRGraphComponent::SetSignificantChange> <md SRGraphComponent::m_bSignificantChange>
void SRGraph::SetSignificantChanges()
{
	if(m_bSignificantChange)
		return; // no need to overburden the component list with extra searches
	
	m_bSignificantChange=TRUE;

	for(POSITION p=m_ComponentList.GetHeadPosition();p;)
	{
		SRGraphComponent *pC=(SRGraphComponent *)m_ComponentList.GetNext(p);
		pC->SetSignificantChange();
	}
}

//@mfunc NEW Returns the m_bFeedbackLogging flag
//@rdesc TRUE if feedback logging is currently enabled, otherwise FALSE.
//@see <md SRGraph::m_bFeedbackLogging>
BOOL SRGraph::GetFeedbackLogging()
{
	return m_bFeedbackLogging;
}


//@mfunc Rolls all indices in a group by 1 place.
//@parm The group to roll.
//@parm The value to place in the new index.
//@parm if FALSE the group rolls toward the low indices. If TRUE the group rolls toard the high indices.
//@rem Used to scroll data for "performance indicator" style charts. As data is scrolled an 
//empty place will be created at one or other end of the group list. The value provided will 
//be placed in the empty index.
void SRGraph::RollGroup(int nGroup, double dValue, BOOL bDirection)
{

	GetGroup(nGroup)->Roll(dValue,bDirection);
}

