#ifndef GRAPHVIEW_H
#define GRAPHVIEW_H
//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       GraphView.h	[ GraphView.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner - based on original that was purchased with Objective Chart (see above)
//
// OVERVIEW:   Defines a graph view class, CGraphView, derived from CView
//
//***************************************************************************

#include "SRGraphResource.h"
#include "ochart.h"	   // Include the objective chart header

class CGraphView : public CView
{
protected:
	SRGraph  m_oGraph;            // this is the heart of the view (Stingray has it in CDocument)

   enum { SG_MOUSE_TIMER_EVENT = 3860 };

	UINT     m_TimerIdent;        // The mouse timer identifier.
	SRGraphTip
            m_GraphTip;          // The single graph tip object.
	DWORD    m_dwSelectionType;   // The current selection type. i.e. horizontal, vertical, box or circular.
	BOOL     m_bShowingTip;       // TRUE if a graph tip is currently on display.
                                 //    Only one exists so a call for a new tip will recycle the old one.
	BOOL     m_bInFeedback;       // TRUE if the mouse point was clicked inside a feedback ared (data dragging will commence)
	CPoint   m_MousePoint;        // The stored mouse position. Used to ascertain if the mouse has rested still in a feedback area for the requisite 1.5 seconds.
	BOOL     m_bMouseTimerIsRunning; // TRUE if the mouse is considered to be "still" and the timeout is waiting to fire.

protected:
	virtual void ShowFeedback(SRGraphFeedback *pFeedback);	// Displays a feedback chart tip.

	// Constructor
	CGraphView();

	DECLARE_DYNCREATE(CGraphView)

// Attributes

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGraphView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
   void     SetListCtrl(CListCtrl * ctl)              { m_poListCtrl = ctl; }

	// Destructor
	virtual  ~CGraphView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
   CListCtrl *
         m_poListCtrl;           // reference to the parent view's embedded list view control
   int   m_iValueDisplayLatency; // user must hold steady this many milliseconds to see the value displayed

// Generated message map functions
protected:
	//{{AFX_MSG(CGraphView)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnEditCopy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};
#endif // GRAPHVIEW_H

/////////////////////////////////////////////////////////////////////////////
