//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

// @doc
// @module GrphTip.cpp | 
// SRGraphTip implementation file
//
// copyright <cp>Stingray Software 1996 
//
#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"
#include /* */"grphtip.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SRGraphTip

SRGraphTip::SRGraphTip()
{
	m_bShowing=FALSE;
}

SRGraphTip::~SRGraphTip()
{
}


BEGIN_MESSAGE_MAP(SRGraphTip, CWnd)
	//{{AFX_MSG_MAP(SRGraphTip)
	ON_WM_PAINT()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// SRGraphTip message handlers

//@mfunc Paints the text of the Graph Tip window.
void SRGraphTip::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CFont *OldFont=(CFont *)dc.SelectStockObject(SYSTEM_FONT);
	COLORREF OldColor=dc.SetTextColor(RGB(0,0,0));	// black text
	int OldMode=dc.SetBkMode(TRANSPARENT);
	// Text is output ensuring that the characters dont interfere with the border.
	dc.TextOut(2*GetSystemMetrics(SM_CXBORDER),2*GetSystemMetrics(SM_CYBORDER),m_TextString);//m_Point.x,m_Point.y,m_TextString);
	dc.SelectObject(OldFont);
	dc.SetTextColor(OldColor);
	dc.SetBkMode(OldMode);
}

//@mfunc Removes the Graph Tip window.
void SRGraphTip::EraseTip()
{
	if(m_bShowing)
		DestroyWindow();
	m_bShowing=FALSE;
}

//@mfunc Shows a Graph Tip window
//@parm The text to show.
//@parm The point at which to show the text. (see remarks)
//@parm A pointer to the parent window.
//@rem A Graph Tip may only show a single line of text. Any lines with carriage returns or line feeds will be truncated. 
//The point at which the chart tip is displayed is always relative to the screen. Not to the owner window. Therefore the 
//point which is sent to this function must be modified by calling ClientToScreen in the owner window first. The reason for this is that the 
//Graph tip may overlap the frame of the owner window and indeed may even attempt to overlap the side or top of the screen
//itself. Size and position data are therefore taken from the main window.
//You may if you wish call this function without erasing the existing display. In this case the window will simply track 
//to the new position and display the supplied text.
void SRGraphTip::ShowTip(LPCSTR strText,CPoint point,CWnd *pParentWnd)
{
	if(m_bShowing)
	{
	   ShowWindow(SW_HIDE);
	   UpdateWindow();
	}
	else
	{
		// perhaps we need to register??
		if(m_ClassName.IsEmpty())
		{
			CBrush bkBrush(RGB(255,255,192)); // a cream sort of color
			m_ClassName = ::AfxRegisterWndClass(CS_BYTEALIGNCLIENT |
												CS_SAVEBITS |
												CS_HREDRAW | 
												CS_VREDRAW, 
												0 ,
												(HBRUSH)bkBrush.Detach());
			ASSERT(m_ClassName!="");
		}

		CRect r;
		r.SetRect(0,0,0,0);
		CreateEx(WS_EX_STATICEDGE, m_ClassName, "", WS_POPUP,
                 r.left,r.top,r.right,r.bottom,
                 pParentWnd->m_hWnd, (HMENU)NULL);
	}

	CWnd *pScreenWindow=GetDesktopWindow();
	CDC *pScreenDC=pScreenWindow->GetDC();
	
	m_TextString=strText;	// copy the const string
	m_Point=point;			//and save the location
	
	// dont allow line feeds etc.
	for(int c=0;c<m_TextString.GetLength();c++)
	{
		if(m_TextString[c]=='\n' || m_TextString[c]=='\r')
		{
			m_TextString=m_TextString.Left(c); // truncate if neccesary
			break;
		}
	}

	// Get the size of the text using the main window DC
	CFont *OldFont=(CFont *)pScreenDC->SelectStockObject(SYSTEM_FONT);
	CSize size=pScreenDC->GetTextExtent(m_TextString);
	pScreenDC->SelectObject(&OldFont);

	//Increase the size by using the non sizeable border width *4
	size.cx+=4*GetSystemMetrics(SM_CXBORDER);
	size.cy+=4*GetSystemMetrics(SM_CYBORDER);
	
	//Find the screen size
	CRect sr,tr;
	pScreenWindow->GetClientRect(&sr);
	
	//Place the window taking care not to go off the edges
	tr.left=point.x+size.cx < sr.right ? point.x : point.x-(sr.Width()-(sr.Width()-size.cx));
	tr.top=(int)(point.y-(1.5*size.cy)>0 ? point.y-(1.5*size.cy) : point.y+(1.5*size.cy));
	tr.right=tr.left+size.cx;
	tr.bottom=tr.top+size.cy;


	m_Point=point; // save the position for later

	
	// move and update the window

	MoveWindow(&tr,TRUE);
	ShowWindow(SW_SHOWNOACTIVATE);
	UpdateWindow();
	m_bShowing=TRUE;
	pScreenWindow->ReleaseDC(pScreenDC);

	SetTimer(2000,5000,NULL);
	
}


//@mfunc If a mouse move event occurs within this window it will automatically erase itself.
//@parm The flags (ignored)
//@parm The mouse point
void SRGraphTip::OnMouseMove(UINT nFlags, CPoint point) 
{
	point; //UNUSED
	nFlags; //UNUSED

	//if the mouse moves in our window its time to go..
	EraseTip();
}

void SRGraphTip::OnTimer(UINT nIDEvent) 
{
	nIDEvent; //UNUSED
	// If we have elapsed!!
	KillTimer(2000);
	EraseTip();
}
