//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       OptionsGeneralPage.cpp	[ OptionsGeneralPage.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the COptionsGeneralPage class
//
//***************************************************************************

#include "stdafx.h"
#include "SourceMetrics.h"
#include "OptionsGeneralPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COptionsGeneralPage property page

IMPLEMENT_DYNCREATE(COptionsGeneralPage, CPropertyPage)

COptionsGeneralPage::COptionsGeneralPage()
   : CPropertyPage(COptionsGeneralPage::IDD), m_sSection(CSourceMetricsApp::OPTION_SETTINGS),
      m_poApp(dynamic_cast<CSourceMetricsApp *>(AfxGetApp()))
{
   COLORMAP oColorMap;
   CString  sCommandLine;                 // this includes the full path to the Windows directory
   CString  sFileToken;
   int      iDirectoryLength;

   oColorMap.from = RGB(255, 255, 255);   // background is white
   oColorMap.to = GetSysColor(COLOR_BTNFACE);
   m_bitmapBrowse.LoadMappedBitmap(IDB_BROWSE_BUTTON, 0, &oColorMap, 1);

   // create a default file viewer command line
   sFileToken.LoadString(IDS_VIEWER_FILE_TOKEN);
   iDirectoryLength = GetWindowsDirectory(NULL, 0);
   GetWindowsDirectory(sCommandLine.GetBuffer(iDirectoryLength), iDirectoryLength);
   sCommandLine.ReleaseBuffer();
   sCommandLine += _T("\\");
   sCommandLine += _T("NOTEPAD.EXE");
   sCommandLine += _T(" ") + sFileToken;

	//{{AFX_DATA_INIT(COptionsGeneralPage)
	m_sSourceViewer = m_poApp->GetProfileString(m_sSection, CSourceMetricsApp::SOURCE_VIEWER, sCommandLine);
	m_fReportFileParseErrors = FALSE;
	//}}AFX_DATA_INIT
   m_fReportFileParseErrors =
         m_poApp->GetProfileInt(CSourceMetricsApp::OPTION_SETTINGS, CSourceMetricsApp::REPORT_ERRORS, -999);
   if( m_fReportFileParseErrors == -999 )
   {
      // default is TRUE
	   m_fReportFileParseErrors = TRUE;
   }
}

COptionsGeneralPage::~COptionsGeneralPage()
{
}

void COptionsGeneralPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionsGeneralPage)
	DDX_Control(pDX, IDC_EDIT_OPTION_FILE_VIEWER, m_editSourceViewer);
	DDX_Control(pDX, IDC_BUTTON_SOURCE_VIEWER_BROWSE, m_btnViewerBrowse);
	DDX_Text(pDX, IDC_EDIT_OPTION_FILE_VIEWER, m_sSourceViewer);
	DDX_Check(pDX, IDC_CHECK_REPORT_PARSE_ERRORS, m_fReportFileParseErrors);
	//}}AFX_DATA_MAP

   if( pDX->m_bSaveAndValidate )
   {
      int   iErrorReportValue = 0;

      if( m_fReportFileParseErrors )
      {
         iErrorReportValue = 1;
      }
      m_poApp->WriteProfileInt(CSourceMetricsApp::OPTION_SETTINGS, CSourceMetricsApp::REPORT_ERRORS, iErrorReportValue);
      m_poApp->WriteProfileString(m_sSection, CSourceMetricsApp::SOURCE_VIEWER, m_sSourceViewer);
   }
   else
   {
      m_btnViewerBrowse.SetBitmap((HBITMAP)m_bitmapBrowse.GetSafeHandle());
   }
}

BOOL
COptionsGeneralPage::OnApply()
{
   return TRUE;      // override in one page to make Apply button work
}

BEGIN_MESSAGE_MAP(COptionsGeneralPage, CPropertyPage)
	//{{AFX_MSG_MAP(COptionsGeneralPage)
	ON_BN_CLICKED(IDC_BUTTON_SOURCE_VIEWER_BROWSE, OnButtonSourceViewerBrowse)
	ON_EN_CHANGE(IDC_EDIT_OPTION_FILE_VIEWER, OnChangeEditOptionFileViewer)
	ON_BN_CLICKED(IDC_CHECK_REPORT_PARSE_ERRORS, OnCheckReportParseErrors)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COptionsGeneralPage message handlers

void COptionsGeneralPage::OnButtonSourceViewerBrowse() 
{
   CString     sFileFilter;
   TCHAR *     pcSeparator;         // pointer used to replace | with NULL in filter string
	CFileDialog oDlg(TRUE, NULL, "*.exe", OFN_PATHMUSTEXIST bitor OFN_FILEMUSTEXIST bitor OFN_HIDEREADONLY, 
                     NULL, this);

   sFileFilter.LoadString(IDS_FIND_EDITOR_FILTER);
   pcSeparator = sFileFilter.GetBuffer(0);
   while( (pcSeparator = _tcschr(pcSeparator + 1, '|')) != NULL )
   {
      *pcSeparator = '\0';                // replace all '|' characters with NULL
   }
   sFileFilter.ReleaseBuffer();
   oDlg.m_ofn.lpstrFilter = sFileFilter;
   if( oDlg.DoModal() == IDOK )
   {
      m_sSourceViewer = oDlg.GetPathName() + _T(" %filename%");
      m_editSourceViewer.SetWindowText(m_sSourceViewer);
      SetModified(TRUE);
   }

}

void COptionsGeneralPage::OnChangeEditOptionFileViewer() 
{
   SetModified(TRUE);
}

void COptionsGeneralPage::OnCheckReportParseErrors() 
{
   SetModified(TRUE);
}

BOOL COptionsGeneralPage::OnSetActive() 
{
	GetParentFrame()->m_dwPromptContext = COptionsGeneralPage::IDD;
	return CPropertyPage::OnSetActive();
}
