//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       OptionsLanguagePage.cpp	[ OptionsLanguagePage.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the COptionsLanguagePage class
//
//***************************************************************************

#include "stdafx.h"

#include "SourceMetrics.h"
#include "OptionsLanguagePage.h"
#include "SMLanguage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(COptionsLanguagePage, CPropertyPage)

BEGIN_MESSAGE_MAP(COptionsLanguagePage, CPropertyPage)
	//{{AFX_MSG_MAP(COptionsLanguagePage)
	ON_NOTIFY(LVN_ENDLABELEDIT, IDC_LANGUAGE_COLUMN_LABELS, OnEndlabeleditColumnLabels)
	ON_NOTIFY(NM_CLICK, IDC_LANGUAGE_COLUMN_LABELS, OnClickLanguageColumnLabels)
	ON_EN_CHANGE(IDC_LANGUAGE_EXTENSIONS, OnChangeExtensions)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

COptionsLanguagePage::COptionsLanguagePage()
    : CPropertyPage(COptionsLanguagePage::IDD), m_sSection(CSourceMetricsApp::OPTION_SETTINGS),
      m_poApp(dynamic_cast<CSourceMetricsApp *>(AfxGetApp())),
      m_poExemplar(NULL), m_iDefaultLabelID0(0)
{
	//{{AFX_DATA_INIT(COptionsLanguagePage)
	m_sExtensions = _T("");
	//}}AFX_DATA_INIT
}

COptionsLanguagePage::~COptionsLanguagePage()
{
}

void COptionsLanguagePage::Init(SMLanguage * exemplar, int id) 
{
   m_poExemplar = exemplar;
   m_iDefaultLabelID0 = id;
	m_sSection += m_poExemplar->Name();
   m_poExemplar->InitLabels();
   m_iColumns = m_poExemplar->Columns();

   // set the page's tab label
   m_psp.dwFlags or_eq PSP_USETITLE;
   m_psp.pszTitle = m_poExemplar->Name();
	m_sExtensions =
      m_poApp->GetProfileString(m_sSection, CSourceMetricsApp::LANGUAGE_EXTENSIONS, m_poExemplar->DefaultExtensions());
	
	return;
}

void COptionsLanguagePage::DoDataExchange(CDataExchange* pDX)
{
   CStringArray   oColumnLabels;

	CPropertyPage::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(COptionsLanguagePage)
	DDX_Control(pDX, IDC_LANGUAGE_COLUMN_LABELS, m_viewlistColumnLabels);
	DDX_Control(pDX, IDC_LANGUAGE_EXTENSIONS, m_editExtensions);
	DDX_Text(pDX, IDC_LANGUAGE_EXTENSIONS, m_sExtensions);
	//}}AFX_DATA_MAP

   oColumnLabels.SetSize(m_iColumns);
   if( pDX->m_bSaveAndValidate )
   {
      m_poApp->WriteProfileString(m_sSection, CSourceMetricsApp::LANGUAGE_EXTENSIONS, m_sExtensions);
      for( int i = 0; i < m_iColumns; ++i )
      {
         oColumnLabels[i] = m_viewlistColumnLabels.GetItemText(i, 0);
      }
      m_poApp->WriteProfileStringList(m_sSection, CSourceMetricsApp::METRICS_LABELS_KEY,
                                  oColumnLabels, CString(_T(";")));
   }
   else
   {
      CRect       oClientRect;   // make column fill the control's client area
      int         iLabelWidth;   // column widths
      int         iNameWidth;
      CString     sLabel;        // scratch for column label
      CString     sItem;
      LV_COLUMN   oColumnData;   // dummy for column existence test

      if( not m_viewlistColumnLabels.GetColumn(0, &oColumnData) )
      {
         // give list view advanced styles
         m_viewlistColumnLabels.SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, 0,
                                             (LPARAM)(LVS_EX_FULLROWSELECT bitor LVS_EX_HEADERDRAGDROP));
         m_viewlistColumnLabels.GetClientRect(oClientRect);
         if( m_iColumns > 11 )
         {
            iLabelWidth = (oClientRect.right * 38) / 100;
            iNameWidth = (oClientRect.right * 58) / 100;
         }
         else
         {
            iLabelWidth = (oClientRect.right * 40) / 100;
            iNameWidth = (oClientRect.right * 60) / 100;
         }
         sLabel.LoadString(IDS_OPTIONS_COLUMN_LABEL);
         m_viewlistColumnLabels.InsertColumn(0, sLabel, LVCFMT_LEFT, iLabelWidth, 0);
         sLabel.LoadString(IDS_OPTIONS_METRIC_LABEL);
         m_viewlistColumnLabels.InsertColumn(1, sLabel, LVCFMT_LEFT, iNameWidth, 1);
      }
      // build default label string list
      for( int i = 0; i < m_iColumns; ++i )
      {
         sItem.LoadString(m_iDefaultLabelID0 + i);       // default label
         oColumnLabels[i] = sItem;                       // add to string list
      }
      // now go around again with the user's settings for the labels and show columns
      m_poApp->GetProfileStringList(m_sSection, CSourceMetricsApp::METRICS_LABELS_KEY,
                                  oColumnLabels, CString(_T(",;")), CString(_T("")));

      // display metric and label names
      m_viewlistColumnLabels.DeleteAllItems();
      for( int j = 0; j < m_iColumns; ++j )
      {
         // first insert an item with the editable label name as the text
         sItem = oColumnLabels[j];
         m_viewlistColumnLabels.InsertItem(j, sItem);

         // then add the metric name as a subitem
         sItem = m_poExemplar->GetMetricName(j);
         m_viewlistColumnLabels.SetItemText(j, 1, sItem);
      }
   }
}

/////////////////////////////////////////////////////////////////////////////
// COptionsLanguagePage message handlers

void COptionsLanguagePage::OnChangeExtensions() 
{
   SetModified(TRUE);
}

void COptionsLanguagePage::OnEndlabeleditColumnLabels(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
   CString  sNewLabel(pDispInfo->item.pszText);

   if( sNewLabel.GetLength() > 0 )     // ignore if user tapped a key when old label was highlighted
   {
      m_viewlistColumnLabels.SetItemText(pDispInfo->item.iItem, 0, sNewLabel);
      SetModified(TRUE);
   }
	*pResult = 0;
}

void COptionsLanguagePage::OnClickLanguageColumnLabels(NMHDR* /* pNMHDR */, LRESULT* pResult) 
{
   CPoint   oCursorPosition;   
	int      iColumn0Width = m_viewlistColumnLabels.GetColumnWidth(0);

   if( GetCursorPos(&oCursorPosition) != 0 )    // start editing only if click was in column 0
   {
      m_viewlistColumnLabels.ScreenToClient(&oCursorPosition);
      if( oCursorPosition.x < iColumn0Width )
      {
         m_viewlistColumnLabels.EditLabel(m_viewlistColumnLabels.GetNextItem(-1, LVNI_SELECTED));
      }
   }
	*pResult = 0;
}

BOOL COptionsLanguagePage::OnSetActive() 
{
	GetParentFrame()->m_dwPromptContext = COptionsLanguagePage::IDD;
	return CPropertyPage::OnSetActive();
}
