//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       OptionsPrintPage.cpp	[ OptionsPrintPage.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the COptionsPrintPage class
//
//***************************************************************************

#include "stdafx.h"
#include "SourceMetrics.h"
#include "OptionsPrintPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COptionsPrintPage dialog

IMPLEMENT_DYNCREATE(COptionsPrintPage, CPropertyPage)

BEGIN_MESSAGE_MAP(COptionsPrintPage, CPropertyPage)
	//{{AFX_MSG_MAP(COptionsPrintPage)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_BOTTOM_MARGIN, OnDeltaposSpinBottomMargin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_INTERCOL_SPACE, OnDeltaposSpinIntercolSpace)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_LEFT_MARGIN, OnDeltaposSpinLeftMargin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_RIGHT_MARGIN, OnDeltaposSpinRightMargin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_TOP_MARGIN, OnDeltaposSpinTopMargin)
	ON_BN_CLICKED(IDC_SELECT_FONT, OnSelectFont)
	ON_EN_CHANGE(IDC_PRINT_BOTTOM_MARGIN, OnChangePrintBottomMargin)
	ON_EN_CHANGE(IDC_PRINT_INTERCOL_SPACE, OnChangePrintIntercolSpace)
	ON_EN_CHANGE(IDC_PRINT_LEFT_MARGIN, OnChangePrintLeftMargin)
	ON_EN_CHANGE(IDC_PRINT_RIGHT_MARGIN, OnChangePrintRightMargin)
	ON_EN_CHANGE(IDC_PRINT_TOP_MARGIN, OnChangePrintTopMargin)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

COptionsPrintPage::COptionsPrintPage()	: CPropertyPage(COptionsPrintPage::IDD),
   m_poApp(dynamic_cast<CSourceMetricsApp *>(AfxGetApp())), m_sSection(CSourceMetricsApp::OPTION_SETTINGS)

{
   m_oMargins.SetSize(5);              // 5 margins: Top, Bottom, Left, Right, Intercolumn
   m_oFontData.SetSize(3);             // 3 font items: Face Name, Size (points), Style

   for( int i = 0; i < 4; ++i )
   {
      m_oMargins[i] = _T("0.5");       // set defaults for 4 edge margins
   }
   // set intercolumn margin default
   m_oMargins[CSourceMetricsApp::INTERCOLUMN_MARGIN] = _T("0.1");
   m_poApp->GetProfileStringList(m_sSection, CSourceMetricsApp::PRINT_MARGINS_KEY,
                                  m_oMargins, CString(_T(",;")), CString(_T("")));
   m_oFontData[CSourceMetricsApp::FONT_NAME] = _T("Arial");
   m_oFontData[CSourceMetricsApp::FONT_SIZE] = _T("10");
   m_oFontData[CSourceMetricsApp::FONT_STYLE] = _T("Regular");
   m_poApp->GetProfileStringList(m_sSection, CSourceMetricsApp::PRINT_FONT_KEY,
                                  m_oFontData, CString(_T(",;")), CString(_T("")));
	//{{AFX_DATA_INIT(COptionsPrintPage)
	m_sTopMargin = m_oMargins[CSourceMetricsApp::TOP_MARGIN];
	m_sBottomMargin = m_oMargins[CSourceMetricsApp::BOTTOM_MARGIN];
	m_sLeftMargin = m_oMargins[CSourceMetricsApp::LEFT_MARGIN];
	m_sRightMargin = m_oMargins[CSourceMetricsApp::RIGHT_MARGIN];
	m_sIntercolumnMargin = m_oMargins[CSourceMetricsApp::INTERCOLUMN_MARGIN];
	m_sFontName = m_oFontData[CSourceMetricsApp::FONT_NAME];
	m_sFontSize = m_oFontData[CSourceMetricsApp::FONT_SIZE];
	m_sFontStyle = m_oFontData[CSourceMetricsApp::FONT_STYLE];
	//}}AFX_DATA_INIT
}

COptionsPrintPage::~COptionsPrintPage()
{
}

void COptionsPrintPage::DoDataExchange(CDataExchange* pDX)
{

   CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionsPrintPage)
	DDX_Control(pDX, IDC_PRINT_TOP_MARGIN, m_editTopMargin);
	DDX_Control(pDX, IDC_PRINT_BOTTOM_MARGIN, m_editBottomMargin);
	DDX_Control(pDX, IDC_PRINT_LEFT_MARGIN, m_editLeftMargin);
	DDX_Control(pDX, IDC_PRINT_RIGHT_MARGIN, m_editRightMargin);
	DDX_Control(pDX, IDC_PRINT_INTERCOL_SPACE, m_editIntercolumnMargin);
	DDX_Control(pDX, IDC_SPIN_TOP_MARGIN, m_spinTopMargin);
	DDX_Control(pDX, IDC_SPIN_RIGHT_MARGIN, m_spinRightMargin);
	DDX_Control(pDX, IDC_SPIN_LEFT_MARGIN, m_spinLeftMargin);
	DDX_Control(pDX, IDC_SPIN_INTERCOL_SPACE, m_spinIntercolumnMargin);
	DDX_Control(pDX, IDC_SPIN_BOTTOM_MARGIN, m_spinBottomMargin);
	DDX_Text(pDX, IDC_PRINT_TOP_MARGIN, m_sTopMargin);
	DDX_Text(pDX, IDC_PRINT_BOTTOM_MARGIN, m_sBottomMargin);
	DDX_Text(pDX, IDC_PRINT_LEFT_MARGIN, m_sLeftMargin);
	DDX_Text(pDX, IDC_PRINT_RIGHT_MARGIN, m_sRightMargin);
	DDX_Text(pDX, IDC_PRINT_INTERCOL_SPACE, m_sIntercolumnMargin);
	DDX_Text(pDX, IDC_EDIT_FONT_NAME, m_sFontName);
	DDX_Text(pDX, IDC_EDIT_FONT_SIZE, m_sFontSize);
	DDX_Text(pDX, IDC_EDIT_FONT_STYLE, m_sFontStyle);
	//}}AFX_DATA_MAP

   if( pDX->m_bSaveAndValidate )
   {
	   m_oMargins[CSourceMetricsApp::TOP_MARGIN] = m_sTopMargin;
	   m_oMargins[CSourceMetricsApp::BOTTOM_MARGIN] = m_sBottomMargin;
	   m_oMargins[CSourceMetricsApp::LEFT_MARGIN] = m_sLeftMargin;
	   m_oMargins[CSourceMetricsApp::RIGHT_MARGIN] = m_sRightMargin;
	   m_oMargins[CSourceMetricsApp::INTERCOLUMN_MARGIN] = m_sIntercolumnMargin;
      m_poApp->WriteProfileStringList(m_sSection, CSourceMetricsApp::PRINT_MARGINS_KEY,
                                  m_oMargins, CString(_T(";")));
      m_oFontData[CSourceMetricsApp::FONT_NAME] = m_sFontName;
      m_oFontData[CSourceMetricsApp::FONT_SIZE] = m_sFontSize;
      m_oFontData[CSourceMetricsApp::FONT_STYLE] = m_sFontStyle;
      m_poApp->WriteProfileStringList(m_sSection, CSourceMetricsApp::PRINT_FONT_KEY,
                                  m_oFontData, CString(_T(";")));
   }
}

// **********************************************************************
//
void
COptionsPrintPage::SetMargin(NM_UPDOWN * poNMUpDown, CEdit & oEditControl)
{
   CString  sMargin;

   sMargin.Format("%4.1f\"", float(poNMUpDown->iPos + poNMUpDown->iDelta) / 10.0);
   oEditControl.SetWindowText(sMargin);
}

/////////////////////////////////////////////////////////////////////////////
// COptionsPrintPage message handlers

BOOL COptionsPrintPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	m_spinTopMargin.SetRange(0, 50);
	m_spinBottomMargin.SetRange(0, 50);
	m_spinLeftMargin.SetRange(0, 50);
	m_spinRightMargin.SetRange(0, 50);
	m_spinIntercolumnMargin.SetRange(0, 50);
	m_spinTopMargin.SetPos((int)(atof(m_oMargins[0]) * 10.0));
	m_spinBottomMargin.SetPos((int)(atof(m_oMargins[1]) * 10.0));
	m_spinLeftMargin.SetPos((int)(atof(m_oMargins[2]) * 10.0));
	m_spinRightMargin.SetPos((int)(atof(m_oMargins[3]) * 10.0));
	m_spinIntercolumnMargin.SetPos((int)(atof(m_oMargins[4]) * 10.0));
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL COptionsPrintPage::OnSetActive() 
{
	GetParentFrame()->m_dwPromptContext = COptionsPrintPage::IDD;
	return CPropertyPage::OnSetActive();
}

void COptionsPrintPage::OnDeltaposSpinBottomMargin(NMHDR* pNMHDR, LRESULT* pResult) 
{
   SetMargin((NM_UPDOWN *)pNMHDR, m_editBottomMargin);
	*pResult = 0;
}

void COptionsPrintPage::OnDeltaposSpinIntercolSpace(NMHDR* pNMHDR, LRESULT* pResult) 
{
   SetMargin((NM_UPDOWN *)pNMHDR, m_editIntercolumnMargin);
	*pResult = 0;
}

void COptionsPrintPage::OnDeltaposSpinLeftMargin(NMHDR* pNMHDR, LRESULT* pResult) 
{
   SetMargin((NM_UPDOWN *)pNMHDR, m_editLeftMargin);
	*pResult = 0;
}

void COptionsPrintPage::OnDeltaposSpinRightMargin(NMHDR* pNMHDR, LRESULT* pResult) 
{
   SetMargin((NM_UPDOWN *)pNMHDR, m_editRightMargin);
	*pResult = 0;
}

void COptionsPrintPage::OnDeltaposSpinTopMargin(NMHDR* pNMHDR, LRESULT* pResult) 
{
   SetMargin((NM_UPDOWN *)pNMHDR, m_editTopMargin);
	*pResult = 0;
}

void COptionsPrintPage::OnSelectFont() 
{
   LOGFONT  oLogFont;
   CDC      oCDC;
   LONG     lYPixelsPerInch;
   TCHAR    caStyleBuffer[64];         // don't know the maximum style but it's less than 64 characters

   memset(&oLogFont, 0, sizeof(oLogFont));
   if( m_sFontName.GetLength() < LF_FACESIZE )
   {
      _tcscpy(oLogFont.lfFaceName, m_sFontName);
   }
   oCDC.CreateDC(_T("DISPLAY"), NULL, NULL, NULL);
   lYPixelsPerInch = oCDC.GetDeviceCaps(LOGPIXELSY);
   oLogFont.lfHeight = -MulDiv((int)(atof(m_sFontSize) * 10.0), lYPixelsPerInch, 720);

	CFontDialog oFontDlg(&oLogFont, CF_SCREENFONTS bitor CF_USESTYLE bitor CF_FORCEFONTEXIST, NULL, this);

   oFontDlg.m_cf.iPointSize = (int)(atof(m_sFontSize) * 10.0);
   _tcscpy(caStyleBuffer, m_sFontStyle);
   oFontDlg.m_cf.lpszStyle = caStyleBuffer;
   if( oFontDlg.DoModal() == IDOK )
   {
      m_sFontName = oFontDlg.GetFaceName();
      m_sFontStyle = oFontDlg.GetStyleName();
      m_sFontSize.Format("%4.1f", float(oFontDlg.GetSize()) / 10.0);
      SetModified(TRUE);
	   GetDlgItem(IDC_EDIT_FONT_NAME)->SetWindowText(m_sFontName);
	   GetDlgItem(IDC_EDIT_FONT_SIZE)->SetWindowText(m_sFontSize);
	   GetDlgItem(IDC_EDIT_FONT_STYLE)->SetWindowText(m_sFontStyle);
   }
}

void COptionsPrintPage::OnChangePrintBottomMargin() 
{
   SetModified(TRUE);
}

void COptionsPrintPage::OnChangePrintIntercolSpace() 
{
   SetModified(TRUE);
}

void COptionsPrintPage::OnChangePrintLeftMargin() 
{
   SetModified(TRUE);
}

void COptionsPrintPage::OnChangePrintRightMargin() 
{
   SetModified(TRUE);
}

void COptionsPrintPage::OnChangePrintTopMargin() 
{
   SetModified(TRUE);
}
