//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       ProjectView.cpp	[ ProjectView.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CProjectView class
//
//***************************************************************************

#include "stdafx.h"

#include "SourceMetrics.h"
#include "SourceMetricsDoc.h"
#include "ProjectView.h"
#include "CreateCheckpointDlg.h"
#include "MetricDetailsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProjectView

IMPLEMENT_DYNCREATE(CProjectView, CListView)

BEGIN_MESSAGE_MAP(CProjectView, CListView)
	//{{AFX_MSG_MAP(CProjectView)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_PROJECT_CHARTMETRICS, OnCheckpointCreatechart)
	ON_COMMAND(ID_ITEM_DELETE, OnCheckpointDelete)
	ON_UPDATE_COMMAND_UI(ID_ITEM_DELETE, OnUpdateCheckpointDelete)
	ON_COMMAND(ID_VIEW_CHECKPOINT, OnViewCheckpoint)
	ON_COMMAND(ID_CHECKPOINT_NEW, OnCheckpointNew)
	ON_NOTIFY_REFLECT(NM_CLICK, OnListViewClick)
	ON_NOTIFY_REFLECT(LVN_ENDLABELEDIT, OnEndLabelEdit)
	ON_COMMAND(ID_CHECKPOINT_RENAME, OnCheckpointRename)
	ON_UPDATE_COMMAND_UI(ID_CHECKPOINT_RENAME, OnUpdateCheckpointRename)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_COMMAND(ID_VIEW_PROJECT_METRICS_DETAILS, OnViewCkptMetricsDetails)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CHECKPOINT, OnUpdateViewCheckpoint)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PROJECT_METRICS_DETAILS, OnUpdateViewCkptMetricsDetails)
	ON_COMMAND(ID_PROJ_VIEW_HELP, OnProjViewHelp)
	ON_UPDATE_COMMAND_UI(ID_PROJ_VIEW_HELP, OnUpdateProjViewHelp)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_NOTIFY_REFLECT(LVN_BEGINLABELEDIT, OnBeginlabeledit)
	ON_COMMAND(ID_EXPORT_CKPT_METRICS, OnExportCkptMetrics)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_CKPT_METRICS, OnUpdateExportCkptMetrics)
	ON_COMMAND(ID_EXPORT_METRICS_ALL, OnExportMetricsAll)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_METRICS_ALL, OnUpdateExportMetricsAll)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

CProjectView::CProjectView()
   : CListView(), m_poListCtrl(NULL), m_poProject(NULL), m_iViewColumns(0), m_fLabelEdit(false)
{
}

CProjectView::~CProjectView()
{
}

BOOL CProjectView::PreCreateWindow(CREATESTRUCT& cs) 
{
   // set the view control style here
   BOOL fResult = CListView::PreCreateWindow(cs);

   // set list style to report mode, allow label edits, only one selection at a time
   cs.style or_eq LVS_REPORT bitor LVS_EDITLABELS bitor LVS_SINGLESEL bitor LVS_SHOWSELALWAYS;

	return fResult;
}

void CProjectView::OnInitialUpdate() 
{
   DWORD       BackgroundColor = GetSysColor(COLOR_MENU);
   CString     sLabel;           // scratch label holder
   int         i;                // loop index (reused)

   // one-time initialization
   m_poSMDoc = dynamic_cast<CSourceMetricsDoc *>(m_pDocument);
   m_poProject = m_poSMDoc->GetProject();
   m_iMetricColumns = m_poProject->GetLanguage()->Columns();
   m_iViewColumns = m_poProject->GetLanguage()->Columns() + FIXED_HEADERS;
   m_sSection = CSourceMetricsApp::OPTION_SETTINGS + m_poProject->GetLanguage()->Name();  // set option profile key
   m_poListCtrl = &GetListCtrl();               // this is the view's wrapped list control
   for( i = 0; i < m_iViewColumns; ++i )        // initialize column order for change comparison
   {
      sLabel.Format("%d", i);
      m_saColumnOrder.Add(sLabel);
   }

   // initialize the list view's background and columns
   m_poListCtrl->SetTextBkColor(BackgroundColor);
   m_poListCtrl->SetBkColor(BackgroundColor);
   sLabel.LoadString(IDS_PROJECT_VIEW_CKPT_LABEL);
   m_poListCtrl->InsertColumn(0, sLabel, LVCFMT_LEFT, 0, 0);
   m_poListCtrl->SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
   sLabel.LoadString(IDS_PROJECT_VIEW_DATE_LABEL);
   m_poListCtrl->InsertColumn(1, sLabel, LVCFMT_LEFT, 0, 1);
   m_poListCtrl->SetColumnWidth(1, LVSCW_AUTOSIZE_USEHEADER);
   sLabel.LoadString(IDS_PROJECT_VIEW_FILES_LABEL);
   m_poListCtrl->InsertColumn(2, sLabel, LVCFMT_RIGHT, 0, 2);
   m_poListCtrl->SetColumnWidth(2, LVSCW_AUTOSIZE_USEHEADER);
   m_poSMDoc->GetDocTemplate()->GetDocString(sLabel, CDocTemplate::docName);

   // now set the extra styles for an enhanced list view and restore the previous column order
   m_poListCtrl->SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, 0,
         (LPARAM)(LVS_EX_FULLROWSELECT bitor LVS_EX_HEADERDRAGDROP));
   m_saColumnOrder.SetSize(m_iViewColumns);
   m_baSortOrder.SetSize(m_iViewColumns);

   // establish the order in which columns should appear in the list
   // (and initialize sort order to ascending)
   for( i = 0; i < m_iViewColumns; ++i )    // first set default values
   {
      sLabel.Format("%d", i);
      m_saColumnOrder[i] = sLabel;
      m_baSortOrder[i] = SORT_DESCENDING;
   }
   // if user has changed the order, override defaults with option
   m_poSMDoc->GetApp()->GetProfileStringList(m_sSection, CSourceMetricsApp::PROJECT_VIEW_COLUMN_ORDER,
                                  m_saColumnOrder, CString(_T(",;")), CString(_T("")));
   sLabel.LoadString(IDS_PRINT_PROJ_VIEW_TITLE);
   m_sCaption.Format(sLabel, m_poProject->GetLanguage()->Name(), m_poProject->Name());
   GetParent()->SetWindowText(m_sCaption);      // set our window's title bar caption

   // now do initial update
   OnUpdate(NULL, 0, NULL);

   return;	
}

void CProjectView::OnUpdate(CView* /* pSender */, LPARAM /* lHint */, CObject* /* pHint */)
{
   CWaitCursor    oWaitCursor;               // let user know this may take awhile
   int            i;                         // loop index (reused)
   CHeaderCtrl *  poViewHeader = (CHeaderCtrl *)m_poListCtrl->GetDlgItem(0);
   bool           fIsFirstUpdate =
                     (poViewHeader not_eq NULL) and (poViewHeader->GetItemCount() <= FIXED_HEADERS);

   // set the column labels, first time only
   if( fIsFirstUpdate )
   {
      CDWordArray iaColumnOrder;    // raw order numbers as read from the header control

      m_poProject->GetLanguage()->InitLabels();   // refresh the project's labels
      for( i = 0; i < m_iMetricColumns; ++i )
      {
         m_poListCtrl->InsertColumn(i + FIXED_HEADERS, m_poProject->GetCaption(i),
                                     LVCFMT_RIGHT, 0, i + FIXED_HEADERS);
      }
      // now size the columns to fit the labels
      for( i = 0; i < m_iMetricColumns; ++i )
      {
         m_poListCtrl->SetColumnWidth(i + FIXED_HEADERS, LVSCW_AUTOSIZE_USEHEADER);
      }
      // now transfer the column order into an int array and send it to the control
      iaColumnOrder.SetSize(m_iViewColumns);
      for( i = 0; i < m_iViewColumns; ++i )
      {
         iaColumnOrder[i] = _ttol(m_saColumnOrder[i]);
      }
      m_poListCtrl->SendMessage(LVM_SETCOLUMNORDERARRAY, (WPARAM)m_iViewColumns,
                                 (LPARAM)iaColumnOrder.GetData());
   }
   // fill the list with checkpoints, if any
   m_poListCtrl->DeleteAllItems();
   for( i = 0; i < m_poProject->CheckpointCount(); ++i )
   {
      SMCheckpoint   oCheckpoint;

      if( m_poProject->GetNextCheckpoint(i == 0, oCheckpoint) )
      {
         CString sItem = (oCheckpoint.Date()).Format("%d %b %Y");

         m_poListCtrl->InsertItem(i, oCheckpoint.Name(), 0);
         m_poListCtrl->SetItemData(i, i);             // set lParam to identify items for sorting
         m_poListCtrl->SetItemText(i, 1, sItem);
         sItem.Format("%d", oCheckpoint.FileCount());
         m_poListCtrl->SetItemText(i, 2, sItem);
         for( int v = 0; v < m_iMetricColumns; ++v )
         {
            m_poListCtrl->SetItemText(i, v + FIXED_HEADERS, oCheckpoint.GetValue(v));
         }
      }
   }
   // now size the columns to fit the data or heading, whichever is wider
   if( fIsFirstUpdate )
   {
      int      iAllColumnsWidth = 0;
      int      iFrameWidth;
      CWnd *   poParentFrame = GetParentFrame();
      CRect    oParentRect;

      poParentFrame->GetWindowRect(&oParentRect);
      iFrameWidth = oParentRect.Width();           // save original size of the frame
      oParentRect.right = oParentRect.left + 10;   // make sure the columns don't fit so right column size is OK
      poParentFrame->MoveWindow(oParentRect);
      for( i = 0; i < m_iViewColumns; ++i )
      {
         int iColumnWidth = m_poListCtrl->GetColumnWidth(i);

         m_poListCtrl->SetColumnWidth(i, LVSCW_AUTOSIZE);
         if( iColumnWidth > m_poListCtrl->GetColumnWidth(i) )
         {
            // put it back
            m_poListCtrl->SetColumnWidth(i, LVSCW_AUTOSIZE_USEHEADER);
            iAllColumnsWidth += iColumnWidth;
         }
         else
         {
            iAllColumnsWidth += m_poListCtrl->GetColumnWidth(i);
         }
      }
      oParentRect.right = oParentRect.left + iFrameWidth;
      poParentFrame->MoveWindow(oParentRect);      // restore the window width
   }
   return;
}

/////////////////////////////////////////////////////////////////////////////
// CProjectView diagnostics

#ifdef _DEBUG
void CProjectView::AssertValid() const
{
	CListView::AssertValid();
}

void CProjectView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}
#endif //_DEBUG

// ***********************************************************************************
// static sort callback
int
CProjectView::SortCompareCallback(LPARAM leftItemData, LPARAM rightItemData, LPARAM pThis)
{
   return ((CProjectView *)pThis)->SortCompare(leftItemData, rightItemData);
}

// ***********************************************************************************
// real sort compare
int
CProjectView::SortCompare(LPARAM leftItemData, LPARAM rightItemData)
{
   SMCheckpoint   oLeftCkpt;
   SMCheckpoint   oRightCkpt;
   CString  sLeft;
   CString  sRight;
   char     bSortOrder = m_baSortOrder[m_iSortColumn];
   int      iResult = 0;

   m_poProject->GetCheckpointByIndex(leftItemData, oLeftCkpt);
   m_poProject->GetCheckpointByIndex(rightItemData, oRightCkpt);
   switch( m_iSortColumn )
   {
   case 0:
      sLeft = oLeftCkpt.Name();
      sRight = oRightCkpt.Name();
      iResult = _tcsicmp(sLeft, sRight);
      break;

   case 1:
      {
         CTimeSpan oDelta = oLeftCkpt.Date() - oRightCkpt.Date();

         iResult = oDelta.GetTotalSeconds();
      }
      break;

   case 2:
      iResult = oLeftCkpt.FileCount() - oRightCkpt.FileCount();
      break;

   default:
      sLeft = oLeftCkpt.m_poLanguage->GetValue(m_iSortColumn - FIXED_HEADERS, false);
      sRight = oRightCkpt.m_poLanguage->GetValue(m_iSortColumn - FIXED_HEADERS, false);
      iResult = (int)(100 * (_tcstod(sLeft, NULL) - _tcstod(sRight, NULL)));
   }
   if( bSortOrder != 0 )
   {
      iResult = - iResult;    // sort descending
   }
   return iResult;
}

/////////////////////////////////////////////////////////////////////////////
// CProjectView message handlers

BOOL CProjectView::OnPreparePrinting(CPrintInfo* pInfo) 
{
	return CListView::DoPreparePrinting(pInfo);
}

void CProjectView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
   if( pDC->IsPrinting() and (not m_fPrinting) )
   {
      pInfo->m_bContinuePrinting = FALSE;
   }
	CListView::OnPrepareDC(pDC, pInfo);
}

void CProjectView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
   // use the common view print routines
   m_poSMDoc->BeginPrinting(pDC, pInfo, m_poListCtrl, m_iViewColumns, m_sCaption);
	CListView::OnBeginPrinting(pDC, pInfo);
}

void CProjectView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
   m_fPrinting = m_poSMDoc->PrintPage(pDC, pInfo);
}

void CProjectView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
   m_poSMDoc->EndPrinting();	
	CListView::OnEndPrinting(pDC, pInfo);
}

void CProjectView::OnListViewClick(NMHDR * /* pNMHDR */ , LRESULT * pResult) 
{
   m_iSelectedIndex = m_poListCtrl->GetNextItem(-1, LVNI_SELECTED);   // get selected item
	*pResult = 0;
}

void CProjectView::OnDblclk(NMHDR * /* pNMHDR */ , LRESULT * pResult) 
{
   m_iSelectedIndex = m_poListCtrl->GetNextItem(-1, LVNI_SELECTED);   // get selected item

   if( m_iSelectedIndex >= 0 )
   {
	   OnViewCheckpoint();
   }
	*pResult = 0;
}

void CProjectView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CMenu    oContextMenu;
   CPoint   oClientPoint(point);          // converted coordinates for hit test

   pWnd->ScreenToClient(&oClientPoint);   // convert to client coordinates for hit text
   m_iSelectedIndex = m_poListCtrl->HitTest(oClientPoint);
   oContextMenu.LoadMenu(IDR_PROJECT_VIEW_CONTEXT);
   oContextMenu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN bitor TPM_RIGHTBUTTON,
      point.x, point.y, GetParentFrame());

   return;
}

void CProjectView::OnCheckpointCreatechart() 
{
   // most of the following code copied from CMDIFrameWnd::OnWindowNew()
   // in MFC file winmdi.cpp
   CSourceMetricsApp *
                  poApp = (CSourceMetricsApp*)AfxGetApp();
	CMDIChildWnd * pActiveChild = poApp->m_pAppMainFrame->MDIGetActive();  // ((CMainFrame*)m_pMainWnd)
	CDocument *    pDocument;     // metrics document class pointer
   CDocTemplate * pTemplate;     // new view template pointer
   CFrameWnd *    pFrame;        // new view class window pointer

	if( (pActiveChild == NULL) or ((pDocument = pActiveChild->GetActiveDocument()) == NULL) )
	{
		TRACE0("Warning: No active document for Chart Checkpoint command.\n");
		AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
	}
   else
   {
	   // otherwise we have a new frame !
	   pTemplate = poApp->m_pHistGraphTemplate;
	   ASSERT_VALID(pTemplate);
	   pFrame = pTemplate->CreateNewFrame(pDocument, pActiveChild);
	   if (pFrame == NULL)
	   {
		   TRACE0("Warning: failed to create new frame.\n");
		   AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
	   }
      else
      {
         m_poSMDoc->SetProjectCaption(m_sCaption);          // pass our caption thru for graph view printing
         pTemplate->InitialUpdateFrame(pFrame, pDocument);
      }
   }
}

void CProjectView::OnUpdateCheckpointDelete(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((not m_fLabelEdit) and (m_poListCtrl->GetNextItem(-1, LVNI_SELECTED) >= 0));	
}

void CProjectView::OnCheckpointDelete() 
{
   int      iSelectedIndex;
   CString  sCaption;
   CString  sMessage;

   sCaption.LoadString(IDS_DELETE_CKPT_CAPTION);
   sMessage.LoadString(IDS_DELETE_CKPT_QUERY);
   if( MessageBox(sMessage, sCaption, MB_ICONQUESTION bitor MB_YESNO) == IDYES )
   {
      iSelectedIndex = m_poListCtrl->GetNextItem(-1, LVNI_SELECTED);   // get selected item
      if( iSelectedIndex >= 0 )
      {
         SMCheckpoint   oCkpt;

         int   iCkptIndex = m_poListCtrl->GetItemData(iSelectedIndex);

         if( m_poSMDoc->IsView(iCkptIndex) )
         {
            CString  sDeleteQueryCaption;
            CString  sDeleteQuery;

            sDeleteQueryCaption.LoadString(IDS_DEL_CKPT_QUERY_CAPTION);
            sDeleteQuery.LoadString(IDS_DEL_CKPT_QUERY);
            if( MessageBox(sDeleteQuery, sDeleteQueryCaption, MB_YESNO bitor MB_ICONQUESTION) == IDYES )
            {
               m_poSMDoc->CloseViews(iCkptIndex);
            }
            else
            {
               return;
            }
         }
         m_poProject->GetCheckpointByIndex(iCkptIndex, oCkpt);
         m_poProject->DeleteCheckpoint(iCkptIndex);
	      m_poSMDoc->SetModifiedFlag();
         m_poSMDoc->UpdateAllViews(NULL);       // update us, too
      }
   }
   return;
}

void CProjectView::OnViewCheckpoint() 
{
   // most of the following code copied from CMDIFrameWnd::OnWindowNew()
   // in MFC file winmdi.cpp
   CSourceMetricsApp *
                  poApp = (CSourceMetricsApp*)AfxGetApp();
	CMDIChildWnd * pActiveChild = poApp->m_pAppMainFrame->MDIGetActive();  // ((CMainFrame*)m_pMainWnd)
	CDocument *    pDocument;     // metrics document class pointer
   CDocTemplate * pTemplate;     // new view template pointer
   CFrameWnd *    pFrame;        // new view class window pointer

	if( (pActiveChild == NULL) or ((pDocument = pActiveChild->GetActiveDocument()) == NULL) )
	{
		TRACE0("Warning: No active document for View Checkpoint command.\n");
		AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
	}
   else
   {
	   // otherwise we have a new frame !
	   pTemplate = poApp->m_pCkpointTemplate;
	   ASSERT_VALID(pTemplate);
	   pFrame = pTemplate->CreateNewFrame(pDocument, pActiveChild);
	   if (pFrame == NULL)
	   {
		   TRACE0("Warning: failed to create new frame.\n");
		   AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
	   }
      else
      {
         if( m_iSelectedIndex >= 0 )
         {
            m_poSMDoc->CheckpointIndex(m_poListCtrl->GetItemData(m_iSelectedIndex));
            m_poSMDoc->SetProjectCaption(m_sCaption);             // pass our caption thru for ckpt view printing
            pTemplate->InitialUpdateFrame(pFrame, pDocument);
            poApp->m_pAppMainFrame->MDICascade();                 // TODO: find out why MDI windows don't cascade
         }
      }
   }
   return;	
}

void CProjectView::OnUpdateViewCheckpoint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED) >= 0);	
}

void CProjectView::OnCheckpointNew() 
{
   CWaitCursor oWaitCursor;
   CStringList oCkptFiles;       // containers for new checkpoint file lists
   CStringList oOtherFiles;

   if( m_poProject->GetNewCheckpointFiles(oCkptFiles, oOtherFiles) )
   {
      CCreateCheckpointDlg oDlg(this, oCkptFiles, oOtherFiles);
      SMLanguage *         poLanguage = m_poProject->GetLanguage();

      oDlg.m_sFilter = AfxGetApp()->GetProfileString(CSourceMetricsApp::OPTION_SETTINGS + poLanguage->Name(),
                                       CSourceMetricsApp::LANGUAGE_EXTENSIONS, poLanguage->DefaultExtensions());
      if( oDlg.DoModal() == IDOK )
      {
         if( m_poProject->CreateCheckpoint(oDlg.m_sName, oCkptFiles, this) )
         {
            m_poSMDoc->UpdateAllViews(NULL);
            m_poSMDoc->SetModifiedFlag(TRUE);
         }
      }
   }
   return;	
}

void CProjectView::OnBeginlabeledit(NMHDR* /* pNMHDR */, LRESULT* pResult) 
{
   m_fLabelEdit = true;	
	*pResult = 0;
}

void CProjectView::OnEndLabelEdit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO *  pDispInfo = (LV_DISPINFO*)pNMHDR;
   CString  sNewName(pDispInfo->item.pszText);

   if( (sNewName.GetLength() > 0)               
   and m_poProject->RenameCheckpoint(pDispInfo->item.iItem, sNewName) )
   {
      m_poListCtrl->SetItemText(pDispInfo->item.iItem, 0, sNewName);
      m_poSMDoc->SetModifiedFlag(TRUE);
   }
	*pResult = 0;
   m_fLabelEdit = false;	
}

void CProjectView::OnCheckpointRename() 
{
	m_poListCtrl->EditLabel(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED));	
}

void CProjectView::OnUpdateCheckpointRename(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED) >= 0);	
}

void CProjectView::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW *  pNMListView = (NM_LISTVIEW*)pNMHDR;

   m_iSortColumn = pNMListView->iSubItem;
   m_poListCtrl->SortItems(CProjectView::SortCompareCallback, (LPARAM)this);

   // toggle sort order for this column
   m_baSortOrder[m_iSortColumn] = (BYTE)(m_baSortOrder[m_iSortColumn] xor SORT_DESCENDING);

	*pResult = 0;
}

void CProjectView::OnViewCkptMetricsDetails() 
{
   SMCheckpoint      oCkpt;
   CMetricDetailsDlg oDlg(this);
   CString           sEntity;
   CString           sValue;

   m_poProject->GetCheckpointByIndex(m_poListCtrl->GetItemData(m_iSelectedIndex), oCkpt);
   sEntity.LoadString(IDS_CKPT_DETAILS_CAPTION);
   oDlg.m_sCaptionSuffix.Format(sEntity, oCkpt.Name());

   // put our four items in first
   sEntity.LoadString(IDS_DETAILS_DIRECTORY_LABEL);
   oDlg.m_saEntities.Add(sEntity);
   oDlg.m_saValues.Add(m_poProject->Directory());
   sEntity.LoadString(IDS_PROJECT_VIEW_CKPT_LABEL);
   oDlg.m_saEntities.Add(sEntity);
   oDlg.m_saValues.Add(oCkpt.Name());
   sEntity.LoadString(IDS_PROJECT_VIEW_DATE_LABEL);
   oDlg.m_saEntities.Add(sEntity);
   sValue = (oCkpt.Date()).Format("%d %B %Y, %X %p");       // date and time per current locale
   oDlg.m_saValues.Add(sValue);
   sEntity.LoadString(IDS_PROJECT_VIEW_FILES_LABEL);
   oDlg.m_saEntities.Add(sEntity);
   sValue.Format("%d", oCkpt.FileCount());
   oDlg.m_saValues.Add(sValue);

   // now let the language add some more entities
   oCkpt.m_poLanguage->GetDetails(m_poProject->Version(), oDlg.m_saValues, oDlg.m_iaBlockDepths, false, true,
                                    oDlg.m_saEntities);
   oDlg.DoModal();
}

void CProjectView::OnUpdateViewCkptMetricsDetails(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED) >= 0);	
}

void CProjectView::OnProjViewHelp() 
{
	AfxGetApp()->WinHelp(ID_PROJ_VIEW_HELP, HELP_CONTEXT);
}

void CProjectView::OnUpdateProjViewHelp(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(TRUE);
}

void CProjectView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	if( (not bActivate) and (pActivateView == pDeactiveView) )
   {
      CDWordArray    iaColumnOrder;       // raw order numbers as read from the header control
      CStringArray   saColumnOrder;       // order numbers as strings
      bool           fIsChanged = false;  // true if any column has been moved by user
   
      if( m_iViewColumns > 0 )
      {
         // get the column arrangement and save it if it's different
         iaColumnOrder.SetSize(m_iViewColumns);
         saColumnOrder.SetSize(m_iViewColumns);
         if( SendMessage(LVM_GETCOLUMNORDERARRAY, (WPARAM)m_iViewColumns, (LPARAM)iaColumnOrder.GetData()) )
         {
            CString  sColumnNumber;

            for( int i = 0; i < m_iViewColumns; ++i )
            {
               sColumnNumber.Format("%d", iaColumnOrder[i]);
               if( sColumnNumber != m_saColumnOrder[i] )
               {
                  fIsChanged = true;
               }
               saColumnOrder[i] = sColumnNumber;
            }
            if( fIsChanged )
            {
               m_poSMDoc->GetApp()->WriteProfileStringList(m_sSection, CSourceMetricsApp::PROJECT_VIEW_COLUMN_ORDER,
                                                saColumnOrder, CString(_T(";")));
            }
         }
      }
   }	
	CListView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

void CProjectView::OnEditUndo() 
{
   if( m_poProject->RestoreDeletedCheckpoint() )
   {
	   m_poSMDoc->SetModifiedFlag();
      m_poSMDoc->UpdateAllViews(NULL);       // update us, too
   }
}

void CProjectView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_poProject->CanUndoCheckpoint() );	
}

void CProjectView::OnExportCkptMetrics() 
{
	m_poSMDoc->ExportMetrics(this, m_poListCtrl->GetNextItem(-1, LVNI_SELECTED));
}

void CProjectView::OnUpdateExportCkptMetrics(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_poListCtrl->GetNextItem(-1, LVNI_SELECTED) >= 0);	
}

void CProjectView::OnExportMetricsAll() 
{
	m_poSMDoc->ExportMetrics(this);
}

void CProjectView::OnUpdateExportMetricsAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);	
}
