//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMCheckpoint.cpp	[ SMCheckpoint.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implements data container class SMCheckpoint
//
//***************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(SMCheckpoint, CObject, 1)

// **************************************************************************
// global function for the CList of files
//
void AFXAPI
SerializeElements(CArchive &ar, SMFile *pElements, int nCount)
{
   for( int i = 0; i < nCount; ++i, ++pElements )
   {
      pElements->Serialize(ar);
   }
}

// ***************************************************************************
// constructor
SMCheckpoint::SMCheckpoint()
   : m_poLanguage(NULL), m_oDate(CTime::GetCurrentTime()), m_fDoDepthMetric(false)
{
}

// ***************************************************************************
// copy constructor
SMCheckpoint::SMCheckpoint(const SMCheckpoint & ckpt)
{
   CopyParameters(ckpt);

   return;
}

// ***************************************************************************
// assignment operator
SMCheckpoint &
SMCheckpoint::operator=(const SMCheckpoint& ckpt)
{
   if( &ckpt != this )
   {
      delete m_poLanguage;             // make sure exiting object is freed
      CopyParameters(ckpt);
   }
   return *this;
}

// ***************************************************************************
// copy parameters for copy constructor, assignment operator
void
SMCheckpoint::CopyParameters(const SMCheckpoint& ckpt)
{
   m_sName = ckpt.m_sName;
   m_oDate = ckpt.m_oDate;
   m_fDoDepthMetric = ckpt.m_fDoDepthMetric;
   m_poLanguage = ckpt.m_poLanguage->MakeNew();
   m_oVersion = ckpt.m_oVersion;
   m_oFiles.RemoveAll();
   m_oFiles.AddTail((FileList *)&ckpt.m_oFiles);
}

// **************************************************************************
//
CString
SMCheckpoint::GetValue(int index)
{
   if( (index >= 0) and (index < m_poLanguage->Columns()) )
   {
      return m_poLanguage->GetValue(index);
   }
   else
   {
      return CString("");
   }
}

// ***************************************************************************
//
bool
SMCheckpoint::ComputeMetrics()
{
   POSITION oFilePosition = m_oFiles.GetHeadPosition();
   SMFile   oCurrentFile;     // scratch file reference
   bool     fResult = true;   // assume we have it

   m_poLanguage->Start(SMLanguage::mtCheckpoint);
   while( oFilePosition != NULL )
   {
      oCurrentFile = m_oFiles.GetNext(oFilePosition);
      m_poLanguage->ProcessFile(*oCurrentFile.m_poLanguage);
   }
   return fResult;
}

// ***************************************************************************
//
bool
SMCheckpoint::AddFile(SMFile & file)
{
   bool  fResult = true;                  // assume success

   if( m_oFiles.AddTail(file) == NULL )   // copies data into the list
   {
      fResult = false;
   }
   return fResult;
}

// ***************************************************************************
//
bool
SMCheckpoint::DeleteFile(const CString & filename)
{
   POSITION oPosition;        // loop position argument
   SMFile   oCurrentFile;     // scratch file reference
   bool     fResult = false;  // assume failure

   for( int i = 0; i < m_oFiles.GetCount(); ++i )
   {
      oPosition = m_oFiles.FindIndex(i);
      oCurrentFile = m_oFiles.GetAt(oPosition);
      if( oCurrentFile.PathName() == filename )
      {
         AddUndoFile(m_oFiles.GetAt(oPosition));
         m_oFiles.RemoveAt(oPosition);
         fResult = true;            // found the checkpoint
         break;
      }
   }
   return fResult;
}

// **************************************************************************
// Restore a deleted file to its proper alphabetical order
//
bool
SMCheckpoint::RestoreDeletedFile()
{
   SMFile   oFile;            // scratch for date order search
   POSITION oPosition;        // search result
   POSITION oInsertPosition;  // position where deleted file should be inserted
   bool     fResult = false;  // assume failure

   if( not m_oDeletedFiles.IsEmpty() )
   {
      SMFile   oDeletedFile = m_oDeletedFiles.RemoveTail();

      if( m_oFiles.IsEmpty() )
      {
         fResult = not m_oFiles.AddHead(oDeletedFile) == NULL;
      }
      else
      {
         oPosition = m_oFiles.GetHeadPosition();
         while( oPosition != NULL )
         {
            oInsertPosition = oPosition;        // save compare position in case it's where we need to insert
            oFile = m_oFiles.GetNext(oPosition);
            if( oFile.PathName() > oDeletedFile.PathName() )
            {
               fResult = not m_oFiles.InsertBefore(oInsertPosition, oDeletedFile) == NULL;
               break;
            }
         }
         if( fResult == false )
         {
             fResult = not m_oFiles.AddTail(oDeletedFile) == NULL;
         }
      }
   }
   return fResult;
}

// ***************************************************************************
//
bool
SMCheckpoint::GetFileByIndex(const int index, SMFile & oResult) const
{
   bool     fResult = true;   // assume we have it

   if( (index < 0) or (index >= m_oFiles.GetCount()) )
   {
      fResult = false;
   }
   else
   {
      oResult = m_oFiles.GetAt(m_oFiles.FindIndex(index));
   }
   return fResult;
}

// ***************************************************************************
//
bool
SMCheckpoint::GetFileByName(const CString & filename, SMFile & oResult) const
{
   POSITION oPosition;        // loop position argument
   SMFile   oCurrentFile;     // scratch
   bool     fResult = false;  // assume we don't find it

   for( oPosition = m_oFiles.GetHeadPosition(); oPosition != NULL; )
   {
      oCurrentFile = m_oFiles.GetNext(oPosition);
      if( oCurrentFile.PathName() == filename )
      {
         oResult = oCurrentFile; // pass this one to caller
         fResult = true;
         break;
      }
   }
   return fResult;
}

// ***************************************************************************
// use to iterate through the entire file list
//
bool
SMCheckpoint::GetNextFile(SMFile & oFile, bool first) const
{
   bool fResult = true;                   // assume success

   if( first )                            // first time, set up the iteration reference
   {
      m_oPosition = m_oFiles.GetHeadPosition();
   }
   if( m_oPosition == NULL )
   {
      fResult = false;
   }
   else
   {
      oFile = m_oFiles.GetNext(m_oPosition);
   }
   return fResult;
}

// **************************************************************************
// get number of characters in longest file name (used in printing list)
int
SMCheckpoint::MaxFileNameLength(CDC * pDC)
{
   SMFile   oFile;
   POSITION oPosition = m_oFiles.GetHeadPosition();
   CSize    oNameSize;
   int      iResult = 0;     // start with a safe minimum

   while( oPosition != NULL )
   {
      oFile = m_oFiles.GetNext(oPosition);   // get next ckpt in list
      oNameSize = pDC->GetTextExtent(oFile.PathName());
      iResult = max(iResult, oNameSize.cx);
   }
   return iResult;
}

// ***************************************************************************
//
void
SMCheckpoint::Serialize(CArchive & ar)
{
   m_oVersion.Serialize(ar);                 // always read or write the version first

   // first do the MFC classes and C++ types
	if( ar.IsStoring() )
	{
		ar << m_sName << m_oDate << m_fDoDepthMetric << m_poLanguage;
	}
	else
	{
		ar >> m_sName >> m_oDate >> m_fDoDepthMetric >> m_poLanguage;
	}
   // then the metrics data objects
   m_poLanguage->Serialize(ar);
   m_oFiles.Serialize(ar);
}
