#ifndef  SMCheckpoint_H
#define	SMCheckpoint_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMCheckpoint.h	[ SMCheckpoint.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of data container class SMCheckpoint
//
//***************************************************************************

#include <afxtempl.h>
#include "SMVersion.h"

class SMLanguage;


// declare file list as a CList template container
void AFXAPI
   SerializeElements(CArchive &ar, SMFile *pElements, int nCount);

typedef CList<SMFile, SMFile&> FileList;

class SMCheckpoint : public CObject
{
   public:
      SMCheckpoint();                           // plain so it'll go in an object container
      SMCheckpoint(const SMCheckpoint & cpt);   // copy constructor
      SMCheckpoint &                            // assignment operator
               operator=(const SMCheckpoint& ckpt);
      virtual ~SMCheckpoint() { delete m_poLanguage; }
      int      operator==(const SMCheckpoint & ckpt) const { return m_oDate == ckpt.m_oDate; }

      // virtual overrides
	   virtual void Serialize(CArchive& ar);

      // property access methods
      CString  Name() const                        { return m_sName; }
      void     Name(const CString & name)          { m_sName = name; }
      CTime    Date() const                        { return m_oDate; }
      void     Date(const CTime & date)            { m_oDate = date; }
      BOOL     DoDepthMetric() const               { return m_fDoDepthMetric; }
      void     DoDepthMetric(const BOOL dodepth)   { m_fDoDepthMetric = dodepth; }
      SMVersion &
               Version()                           { return m_oVersion; }
      void     SetVersion(int iMajor, int iMinor)  { m_oVersion.SetVersion(iMajor, iMinor); }

      int      MetricCount() const                 { return m_poLanguage->Columns(); }
      bool     ComputeMetrics();
      CString  GetValue(int index);
      int      FileCount()                         { return m_oFiles.GetCount(); }
      bool     AddFile(SMFile & file);
      bool     DeleteFile(const CString & filename);
      bool     RestoreDeletedFile();
      bool     GetFileByIndex(const int index, SMFile & oResult) const;
      bool     GetFileByName(const CString & filename, SMFile & oResult) const;
      bool     GetNextFile(SMFile & oFile, bool first) const; // use to iterate through the list
      int      MaxFileNameLength(CDC * pDC);
      void     AddUndoFile(SMFile & file)          { m_oDeletedFiles.AddTail(file); }
      bool     CanUndoFile()                       { return not m_oDeletedFiles.IsEmpty(); }

      // this is the heart of the metrics data: set at constructor time, accessed often
      SMLanguage *   m_poLanguage;        // checkpoint-wide language-specific metric data container

   private:
      CString        m_sName;             // user's name foe this checkpoint (default: creation date)
      CTime          m_oDate;             // checkpoint creation date (or file version date)
      SMVersion      m_oVersion;          // version of Source Monitor that created this checkpoint
      FileList       m_oFiles;
      enum FLAG_DEFS { DO_DEPTH_METRIC = 1, INCLUDE_SUBFOLDERS = 2 };
      BOOL           m_fDoDepthMetric;

      // transients
      mutable POSITION  m_oPosition;      // iteration reference
      FileList          m_oDeletedFiles;  // deleted files that mey be restored via undo

      // for copy constructor and assignment operator
      void           CopyParameters(const SMCheckpoint& ckpt);

      DECLARE_SERIAL(SMCheckpoint)
};
////////////////////////////////////////////////////////////////////////////////
#endif						// SMCheckpoint_H
