#ifndef  SMCpp_H
#define	SMCpp_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMCpp.h	[ SMCpp.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of C++ metrics language class SMCpp
//
//***************************************************************************

class SSLexTable;
class SSYaccTable;
class CppLexClass;

class SMCpp : public SMLanguage
{
   public:
      // define indexes into the metrics count accumulator array
      enum COUNT_TYPE { LINE_COUNT = 0, STATEMENT_COUNT = 1, COMMENT_LINES = 2, CLASS_COUNT = 3,
                        METHOD_COUNT = 4, METHOD_STATEMENTS = 5, BLOCK_DEPTH_0 = 6,
                        BLOCK_DEPTH_COUNTS = 10, BRANCH_STATEMENTS = 16, DEEPEST_BLOCK_LINE = 17,
                        BIGGEST_METHOD_LINE = 18, TOTAL_COUNTS = 19 };

      // used only as a class factory (in SMProject and COptionsLanguagePage)
      explicit SMCpp(Exemplar /* not used */) : SMLanguage() {;}
      virtual  SMLanguage * MakeNew();                            // create a shallow copy
      virtual  ~SMCpp();

      // virtual overrides
	   virtual void Serialize(CArchive& ar);

      // property access methods
      virtual CString & Name() const                     { return CPP_NAME; }
      virtual CString & DefaultExtensions() const        { return CPP_EXTENSIONS; }

      // signature for common methods of sub-classes
      virtual bool   Start(MetricsType type);

      // accumulate metrics for one file
      virtual void   InitParser();
      virtual void   CloseParser();
      virtual bool   ProcessLines(const CString & sFileName);
      virtual void   ParseError();
      virtual void   BumpCount(int index)                { ++m_iCounts[index]; }
      virtual void   SetMethodData(const CString & sClassName, const CString & sMethodName, int iStatementCount);
      virtual void   SetBiggestMethodLine()              { m_iBeginMethodLine = m_iCounts[LINE_COUNT] + 1; }
      virtual void   SetDeepestBlockLine()               { m_iCounts[DEEPEST_BLOCK_LINE] = m_iCounts[LINE_COUNT] + 2; }
      virtual void   RemoveSpace(CString & sLexeme);

      // accumulate across group of files
      virtual bool   ProcessFile(const SMLanguage & oFileMetric);
 
      // results access methods (indexes identify columns displayed to user: base 0)
      virtual short     Columns() const                  { return TOTAL_COLUMNS; }
      virtual CString   GetMetricName(int index) const;
      virtual void      InitLabels();
      virtual CString   GetValue(int index, bool format = true) const;
      virtual int       GetIntValue(int index) const     { return m_iCounts[index]; }
      virtual bool      GetDeepestBlockLine(CString & sLabel, int * piResult) const;
      virtual bool      GetBiggestModuleLine(CString & sLabel, int * piResult) const;
      virtual void      GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                                    bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const;
      virtual int       BlockCounts()                                { return BLOCK_DEPTH_COUNTS; }

      static CString    CPP_NAME;         // this is how this language is identified to one and all
      static CString    CPP_EXTENSIONS;   // use until user changes options

   protected:
      SMCpp();                         // no public default constructor: use MakeNew method

   private:
      // define results columns NOTE: MUST KEEP STRING RESOURCES IN SYNCH WITH THESE MAGIC NUMBERS!!!
      enum COLUMN_TYPE { LINES, STATEMENTS, PERCENT_BRANCHES, PERCENT_COMMENTS, CLASSES, METHODS_PER_CLASS,
                         AVERAGE_STATEMENTS_PER_METHOD, BIGGEST_METHOD, MAXIMUM_BLOCK_DEPTH,
                         AVERAGE_BLOCK_DEPTH, TOTAL_COLUMNS };

      // persistent metrics data
      CDWordArray          m_iCounts;        // raw counts
      CMapStringToString   m_oClassList;     // classes with biggest method
                                             //   value format: <line count>@<method name>

      // transient data members
      int                  m_iBeginMethodLine;
      int                  m_iBiggestMethodStatements;

      // static data members
      static int           m_iParserUsers;   // reference count for parsing objects
      static CppLexClass * m_poLexer;
      static SSLexTable *  m_poLexTable;
      static SSYaccTable * m_poYaccTable;

      DECLARE_SERIAL(SMCpp)
};
////////////////////////////////////////////////////////////////////////////////

#endif						// SMCpp_H
