#ifndef  SMDelphi_H
#define	SMDelphi_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMDelphi.h	[ SMDelphi.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of Delphi metrics language class SMDelphi
//
//***************************************************************************

class SSLexTable;
class SSYaccTable;
class DelphiLexClass;

class SMDelphi : public SMLanguage
{
   public:
      // define indexes into the metrics count accumulator array (leave one spare at the end)
      enum COUNT_TYPE { LINE_COUNT = 0, STATEMENT_COUNT = 1, COMMENT_LINES = 2, GLOBAL_SUBROUTINE_COUNT = 3,
                        GLOBAL_VARIABLE_COUNT = 4, LOCAL_SUBROUTINE_COUNT = 5, CLASS_COUNT = 6,
                        CLASS_METHOD_COUNT = 7, CLASS_VARIABLE_COUNT = 8, BRANCH_COUNT = 9, DEEPEST_BLOCK_LINE = 10,
                        BIGGEST_SUBROUTINE_LINE = 11, BIGGEST_SUBROUTINE_STATEMENTS = 12, SUBROUTINE_STATEMENTS = 13,
                        BIGGEST_METHOD_LINE = 14, BIGGEST_METHOD_STATEMENTS = 15, METHOD_STATEMENTS = 16,
                        // leave count number 17 as a spare
                        BLOCK_DEPTH_0 = 18, BLOCK_DEPTH_COUNTS = 10, TOTAL_COUNTS = 28 };

      // used only as a class factory (in SMProject and COptionsVBPage)
      explicit SMDelphi(Exemplar /* not used */) : SMLanguage() {;}
      virtual  SMLanguage * MakeNew();                         // create a shallow copy
      virtual  ~SMDelphi();

      // virtual overrides
	   virtual void Serialize(CArchive& ar);

      // property access methods
      virtual CString & Name() const                        { return DELPHI_NAME; }
      virtual CString & DefaultExtensions() const           { return DELPHI_EXTENSIONS; }

      // signature for common methods of sub-classes
      virtual bool   Start(MetricsType type);

      // accumulate metrics for one file
      virtual void   InitParser();
      virtual void   CloseParser();
      virtual bool   ProcessLines(const CString & sFileName);
      virtual void   ParseError();
      virtual void   BumpCount(int index)                   { ++m_iCounts[index]; }
      virtual void   SetSubroutineData(const CString & sSubroutineName, int iStatementCount);
      virtual void   SetMethodData(const CString & sClassMethodNames, int iStatementCount);
      virtual void   SetBiggestMethodLine()                 { m_iBeginMethodLine = m_iCounts[LINE_COUNT] + 1; }
      virtual void   SetBiggestSubroutineLine()             { m_iBeginSubroutineLine = m_iCounts[LINE_COUNT] + 1; }
      virtual void   SetDeepestBlockLine()                  { m_iCounts[DEEPEST_BLOCK_LINE]
                                                                        = m_iCounts[LINE_COUNT] + 2; }

      // accumulate across group of files
      virtual bool   ProcessFile(const SMLanguage & oFileMetric);
 
      // results access methods (indexes identify columns displayed to user: base 0)
      virtual short     Columns() const                     { return TOTAL_COLUMNS; }
      virtual CString   GetMetricName(int index) const;
      virtual void      InitLabels();
      virtual CString   GetValue(int index, bool format = true) const;
      virtual int       GetIntValue(int index) const        { return m_iCounts[index]; }
      virtual bool      GetDeepestBlockLine(CString & sLabel, int * piResult) const;
      virtual bool      GetBiggestModuleLine(CString & sLabel, int * piResult) const;
      virtual void      GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                                    bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const;
      virtual int       BlockCounts()                                { return BLOCK_DEPTH_COUNTS; }

      static CString    DELPHI_NAME;         // this is how this language is identified to one and all
      static CString    DELPHI_EXTENSIONS;   // use until user changes options

   protected:
      SMDelphi();                            // no public default constructor: use MakeNew method

   private:
      // define results columns NOTE: MUST KEEP STRING RESOURCES IN SYNCH WITH THESE MAGIC NUMBERS!!!
      enum COLUMN_TYPE { LINES, STATEMENTS, PERCENT_COMMENTS, GLOBAL_SUBROUTINES, GLOBAL_VARIABLES,
                         LOCAL_SUBROUTINES, CLASSES, AVERAGE_METHODS_PER_CLASS, VARIABLES_PER_CLASS,
                         AVERAGE_STATEMENTS_PER_METHOD, BIGGEST_METHOD, 
                         AVERAGE_STATEMENTS_PER_SUBROUTINE, BIGGEST_SUBROUTINE, 
                         PERCENT_BRANCHES, MAXIMUM_BLOCK_DEPTH, AVERAGE_BLOCK_DEPTH, TOTAL_COLUMNS };

      // persistent metrics data
      CDWordArray m_iCounts;                 // raw counts
      CString     m_sSubroutineName;         // name of biggest subroutine
      CMapStringToString
                  m_oClassList;              // classes with biggest method
                                             //   value format: <line count>@<method name>

      // transient data members
      int         m_iBeginSubroutineLine;    // set for every subroutine implementation
      int         m_iBeginMethodLine;        // set for every method implementation

      // static data members
      static int           m_iParserUsers;   // reference count for parsing objects
      static DelphiLexClass *
                           m_poLexer;
      static SSLexTable *  m_poLexTable;
      static SSYaccTable * m_poYaccTable;

      DECLARE_SERIAL(SMDelphi)
};
////////////////////////////////////////////////////////////////////////////////

#endif						// SMDelphi_H
