//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMHTML.cpp	[ SMHTML.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implements HTML metrics language class SMHTML
//
//***************************************************************************

#include "stdafx.h"

#include "SMLanguage.h"
#include "SMHTML.h"
#include "SourceMetrics.h"
#include "resource.h"
#include <VPHTMLClasses.hpp>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(SMHTML, SMLanguage, 1)

CString        SMHTML::HTML_NAME = _T("HTML");
CString        SMHTML::HTML_EXTENSIONS = _T("*.htm;*.html;*.asp");

int            SMHTML::m_iParserUsers = 0;
HTMLLexClass * SMHTML::m_poLexer = NULL;  // parsing objects are created and deleted by App object
SSLexTable *   SMHTML::m_poLexTable = NULL;
SSYaccTable *  SMHTML::m_poYaccTable = NULL;

// ***************************************************************************
// constructor
SMHTML::SMHTML() : SMLanguage()
{
   m_iCounts.SetSize(TOTAL_COUNTS);
}

// ***************************************************************************
// destructor
SMHTML::~SMHTML()
{
}

// Class factory
SMLanguage *
SMHTML::MakeNew()           // create a shallow copy
{
   SMHTML *    poNewSMHTML;

   poNewSMHTML = new SMHTML();
   for( int i = 0; i < m_iCounts.GetSize(); ++i)
   {
      poNewSMHTML->m_iCounts[i] = m_iCounts[i];
   }
   return poNewSMHTML;
}

// ***************************************************************************
// initialize global parser objects
void
SMHTML::InitParser()
{
   if( m_iParserUsers == 0 )
   {
      m_poLexTable = new SSLexTable((unsigned int)AfxGetResourceHandle(),
                                          MAKEINTRESOURCE(IDR_HTML_LEXTABLE), _T("LEXTABLE"));
      m_poYaccTable = new SSYaccTable((unsigned int)AfxGetResourceHandle(),
                                          MAKEINTRESOURCE(IDR_HTML_YACCTABLE), _T("YACCTABLE"));
      m_poLexer = new HTMLLexClass(*SMHTML::m_poLexTable);
   }
   ++m_iParserUsers;
}

// ***************************************************************************
// shut down parser
void
SMHTML::CloseParser()
{
   if( m_iParserUsers > 0 )
   {
      --m_iParserUsers;
   }
   if( m_iParserUsers == 0 )
   {
      delete m_poLexTable;
      m_poLexTable = NULL;
      delete m_poYaccTable;
      m_poYaccTable = NULL;
      delete m_poLexer;
      m_poLexer = NULL;
   }
}

// ***************************************************************************
// compute and accumulate metrics for one line or one file

bool
SMHTML::Start(MetricsType /* type */)
{
   bool  fResult = true;

   if( m_poLexer == NULL )
   {
      fResult = false;
   }
   else
   {
      for( int i = 0; i < TOTAL_COUNTS; ++i )
      {
         m_iCounts[i] = 0;       // initialize the count accumulators
      }
   }
   return fResult;
}

bool
SMHTML::ProcessLines(const CString & sFileName)
{
   bool  fResult = true;         // assume success

   try
   {
      SSLexFileConsumer oSource(sFileName);

      m_poLexer->reset();
      m_poLexer->setConsumer(oSource);
      HTMLYaccClass(*m_poLexer, *m_poYaccTable, this).parse();
      SMLanguage::ReportErrors(sFileName);
   }
   catch(SSException oExcept)
   {
      m_sError.Format("%s [%s]", oExcept.text(), sFileName);
      fResult = false;
   }
   catch(...)
   {
      fResult = false;
   }
   return fResult;
}

// ***************************************************************************
// called by the parser's reduce() method when parser can't handle source code structure
void
SMHTML::ParseError()
{
   if( AfxGetApp()->GetProfileInt(CSourceMetricsApp::OPTION_SETTINGS, CSourceMetricsApp::REPORT_ERRORS, 0) != 0 )
   {
      CString  sLineNumber;

      sLineNumber.Format("%d", m_iCounts[LINE_COUNT] + 1);
      m_asErrorLines.Add(sLineNumber);
   }
}

bool
SMHTML::ProcessFile(const SMLanguage & oFileMetric)
{
   const SMHTML & oFileData = dynamic_cast<const SMHTML &>(oFileMetric);
   bool           fResult = true;      // assume success

   for( int i = 0; i < TOTAL_COUNTS; ++i )
   {
      m_iCounts[i] += oFileData.m_iCounts[i];   // accumulate count totals
   }
   return fResult;
}

// ***************************************************************************
//
void
SMHTML::InitLabels()
{
   m_saLabels.SetSize(TOTAL_COLUMNS);

   // set some default labels and column alignment
   for( int i = 0; i < TOTAL_COLUMNS; ++i )
   {
      CString  sLabel;

      sLabel.LoadString(IDS_HTML_DEFAULT_LABEL0 + i);
      m_saLabels[i] = sLabel;
   }
   // now override with user's labels, if any
   dynamic_cast<CSourceMetricsApp *>(AfxGetApp())->
      GetProfileStringList(CSourceMetricsApp::OPTION_SETTINGS + HTML_NAME, CSourceMetricsApp::METRICS_LABELS_KEY,
                           m_saLabels, CString(_T(";,")), CString(_T("")));
}

// ***************************************************************************
//
CString
SMHTML::GetMetricName(int index) const
{
   CString  sMetricName;

   if( (index >= 0) and (index < TOTAL_COLUMNS) )
   {
      sMetricName.LoadString(IDS_HTML_METRIC0 + index);
   }
   return sMetricName;
}

// ***************************************************************************
// results access methods
CString
SMHTML::GetValue(int index, bool format /* = true */) const       // access a metric's value
{
   CString  sValue;
   float    flValue = 0.0;
   int      iValue;

   sValue.LoadString(IDS_UNDEFINED_VALUE);
   if( (index >= 0) and (m_iCounts.GetSize() > index) )
   {
      switch( index )
      {
      case PERCENT_COMPLEX_TAGS:
         iValue = m_iCounts[SIMPLE_TAG_COUNT] + m_iCounts[COMPLEX_TAG_COUNT];
         if( iValue > 0 )
         {
            flValue = float(m_iCounts[COMPLEX_TAG_COUNT] * 100) / float(iValue);
         }
         sValue.Format("%.1f", flValue);
         break;

      default:
         if( (index >= 0) and (index < m_iCounts.GetSize()) )
         {
            iValue = m_iCounts[index];
            if( format )
            {
               sValue = FormatCount(iValue);
            }
            else
            {
               sValue.Format("%d", iValue);
            }
         }
         break;
      }
   }
   return sValue;
}

// ***************************************************************************
//
void
SMHTML::GetDetails(SMVersion & /* oVersion */, CStringArray & oValues, CDWordArray & /* oBlockCounts */,
                     bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const
{
   for( int i = 0; i < TOTAL_COLUMNS; ++i )
   {
      if( fIncludeEntities )
      {
         oEntities.Add(GetMetricName(i));
      }
      oValues.Add(GetValue(i));
   }
}

// ***************************************************************************
//
void
SMHTML::Serialize(CArchive & ar)
{
   SMLanguage::Serialize(ar);
   m_iCounts.Serialize(ar);

   return;
}
