#ifndef  SMHTML_H
#define	SMHTML_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMHTML.h	[ SMHTML.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition HTML metrics language class SMHTML
//
//***************************************************************************

class SSLexTable;
class SSYaccTable;
class HTMLLexClass;

class SMHTML : public SMLanguage
{
   public:
      // define indexes into the metrics count accumulator array
      enum COUNT_TYPE { LINE_COUNT= 0, COMMENT_COUNT = 1, SIMPLE_TAG_COUNT = 2, COMPLEX_TAG_COUNT = 3,
                        HYPERLINK_COUNT = 4, DOWNLOAD_COUNT = 5, SCRIPT_COUNT = 6, APPLET_COUNT = 7,
                        TOTAL_COUNTS = 8 };

      SMHTML(Exemplar /* not used */) : SMLanguage() {;} // used only in SMProject as factory for this class
      virtual SMLanguage * MakeNew();                    // create a shallow copy
      ~SMHTML();

      // virtual overrides
	   virtual void Serialize(CArchive& ar);

      // property access methods
      virtual CString & Name() const                              { return HTML_NAME; }
      virtual CString & DefaultExtensions() const                 { return HTML_EXTENSIONS; }

      // signature for common methods of sub-classes
      virtual bool   Start(MetricsType type);

      // compute and accumulate metrics for one line
      // accumulate metrics for one file
      virtual void   InitParser();
      virtual void   CloseParser();
      virtual bool   ProcessLines(const CString & sFileName);
      virtual void   ParseError();
      virtual void   BumpCount(int index)                   { ++m_iCounts[index]; }

      // accumulate across group of files
      virtual bool   ProcessFile(const SMLanguage & oFileMetric);
 
      // results access methods (indexes identify columns displayed to user: base 0)
      virtual short     Columns() const                     { return TOTAL_COLUMNS; }
      virtual CString   GetMetricName(int index) const;
      virtual void      InitLabels();
      virtual CString   GetValue(int index, bool format = true) const;
      virtual int       GetIntValue(int index) const        { return m_iCounts[index]; }
      virtual void      GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                                    bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const;

      static CString    HTML_NAME;        // this is how this language is identified to one and all
      static CString    HTML_EXTENSIONS;  // use until user changes options

   protected:
      SMHTML();                           // no public default constructor: use MakeNew method

   private:
      // define results columns NOTE: MUST KEEP STRING RESOURCES IN SYNCH WITH THESE MAGIC NUMBERS!!!
      enum COLUMN_TYPE { LINES, COMMENT_LINES, TAGS, PERCENT_COMPLEX_TAGS, HYPERLINKS, DOWNLOADS,
                           SCRIPTS, APPLETS, TOTAL_COLUMNS };

      CDWordArray    m_iCounts;                 // raw counts

      // static data members
      static int              m_iParserUsers;   // reference count for parsing objects
      static HTMLLexClass *   m_poLexer;
      static SSLexTable *     m_poLexTable;
      static SSYaccTable *    m_poYaccTable;

      DECLARE_SERIAL(SMHTML)
};
////////////////////////////////////////////////////////////////////////////////

#endif						// SMHTML_H
