//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMJava.cpp	[ SMJava.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implements Java metrics language class SMJava
//
//***************************************************************************

#include "stdafx.h"

#include "SMLanguage.h"
#include "SMJava.h"
#include "SourceMetrics.h"
#include "resource.h"
#include "VPJavaClasses.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(SMJava, SMLanguage, 1)


// static members
CString        SMJava::JAVA_NAME = _T("Java");
CString        SMJava::JAVA_EXTENSIONS = _T("*.java");

int            SMJava::m_iParserUsers = 0;
JavaLexClass * SMJava::m_poLexer = NULL;     // parsing objects are created and deleted by App object
SSLexTable *   SMJava::m_poLexTable = NULL;
SSYaccTable *  SMJava::m_poYaccTable = NULL;

// ***************************************************************************
// normal constructor
SMJava::SMJava() : SMLanguage()
{
   m_iCounts.SetSize(TOTAL_COUNTS);
}

// Class factory
SMLanguage *
SMJava::MakeNew()           // create a shallow copy
{
   SMJava * poNewSMJava;
   POSITION oPosition;
   CString  sClassName;
   CString  sClassValue;

   poNewSMJava = new SMJava();
   for( int i = 0; i < m_iCounts.GetSize(); ++i)
   {
      poNewSMJava->m_iCounts[i] = m_iCounts[i];
   }
   oPosition = m_oClassList.GetStartPosition(); 
   while( oPosition != NULL )
   {
      m_oClassList.GetNextAssoc(oPosition, sClassName, sClassValue);
      poNewSMJava->m_oClassList[sClassName] = sClassValue;
   }
   return poNewSMJava;
}

// ***************************************************************************
// destructor
SMJava::~SMJava()
{
}

// ***************************************************************************
// initialize global parser objects
void
SMJava::InitParser()
{
   if( m_iParserUsers == 0 )
   {
      m_poLexTable = new SSLexTable((unsigned int)AfxGetResourceHandle(),
                                          MAKEINTRESOURCE(IDR_JAVA_LEXTABLE), _T("LEXTABLE"));
      m_poYaccTable = new SSYaccTable((unsigned int)AfxGetResourceHandle(),
                                          MAKEINTRESOURCE(IDR_JAVA_YACCTABLE), _T("YACCTABLE"));
      m_poLexer = new JavaLexClass(*SMJava::m_poLexTable);
   }
   ++m_iParserUsers;
}

// ***************************************************************************
// shut down parser
void
SMJava::CloseParser()
{
   if( m_iParserUsers > 0 )
   {
      --m_iParserUsers;
   }
   if( m_iParserUsers == 0 )
   {
      delete m_poLexTable;
      m_poLexTable = NULL;
      delete m_poYaccTable;
      m_poYaccTable = NULL;
      delete m_poLexer;
      m_poLexer = NULL;
   }
}

// ***************************************************************************
// compute and accumulate metrics for one line or one file

bool
SMJava::Start(MetricsType /* type */)
{
   bool  fResult = true;

   if( m_poLexer == NULL )
   {
      fResult = false;
   }
   else
   {
      for( int i = 0; i < TOTAL_COUNTS; ++i )
      {
         m_iCounts[i] = 0;       // initialize the count accumulators
      }
      m_oClassList.RemoveAll();
      m_iBiggestMethodStatements = 0;
      m_iBeginMethodLine = 0;
   }

   return fResult;
}

bool
SMJava::ProcessLines(const CString & sFileName)
{
   bool  fResult = true;         // assume success

   try
   {
      SSLexFileConsumer oSource(sFileName);
      m_poLexer->reset();
      m_poLexer->setConsumer(oSource);
      JavaYaccClass(*m_poLexer, *m_poYaccTable, this).parse();
      SMLanguage::ReportErrors(sFileName);
   }
   catch(SSException oExcept)
   {
      m_sError.Format("%s [%s]", oExcept.text(), sFileName);
      fResult = false;
   }
   catch(...)
   {
      fResult = false;
   }
   return fResult;
}

// ***************************************************************************
// called by the parser's reduce() method when parser can't handle source code structure
void
SMJava::ParseError()
{
   if( AfxGetApp()->GetProfileInt(CSourceMetricsApp::OPTION_SETTINGS, CSourceMetricsApp::REPORT_ERRORS, 0) != 0 )
   {
      CString  sLineNumber;

      sLineNumber.Format("%d", m_iCounts[LINE_COUNT] + 1);
      m_asErrorLines.Add(sLineNumber);
   }
}

// ***************************************************************************
// called by the parser's reduce() method to bump a metrics count
void
SMJava::SetMethodData(const CString & sClassName, const CString & sMethodName, int iStatementCount)
{
   CString  sClassValue;               // class list value for biggest method test
                                       //    value format: <line count>@<method name>
   bool     fUpdateClassList = true;   // assume list needs to be changed
   
   if( m_oClassList.Lookup(sClassName, sClassValue) )   // see if the class is in the list
   {
      // check to see if biggest method is stored in the map's value
      if( _ttoi(sClassValue) > iStatementCount )
      {
         fUpdateClassList = false;     // class is there with a bigger method: do nothing
      }
   }
   if( fUpdateClassList )
   {
      sClassValue.Format("%d@%s", iStatementCount, sMethodName);
      m_oClassList[sClassName] = sClassValue;
      if( iStatementCount >= m_iBiggestMethodStatements )
      {
         m_iCounts[BIGGEST_METHOD_LINE] = m_iBeginMethodLine;
         m_iBiggestMethodStatements = iStatementCount;
      }
   }
}

bool
SMJava::ProcessFile(const SMLanguage & oFileMetric)
{
   const SMJava &
            oFileData = dynamic_cast<const SMJava &>(oFileMetric);
   CString  sClassValue;         // class list value for biggest method test
                                 //    value format: <line count>@<method name>
   POSITION oSourcePosition;     // iterator for file's class list
   CString  sClassName;          // file's current class's name
   CString  sSourceValue;        // file's current class's value
   CString  sDestinationValue;   // value for file's class in local class list
   bool     fUpdateClassList;    // true if class list needs to be changed
   bool     fResult = true;      // assume success

   for( int i = 0; i < TOTAL_COUNTS; ++i )
   {
      m_iCounts[i] += oFileData.m_iCounts[i];   // accumulate count totals
   }
   // not valid for checkpoints
   m_iCounts[DEEPEST_BLOCK_LINE] = UINT_MAX;
   m_iCounts[BIGGEST_METHOD_LINE] = UINT_MAX;

   // now build a list of unique classes with biggest method values
   oSourcePosition = oFileData.m_oClassList.GetStartPosition();
   while( oSourcePosition != NULL )
   {
      fUpdateClassList = true;         // assume change is needed
      oFileData.m_oClassList.GetNextAssoc(oSourcePosition, sClassName, sSourceValue);
      if( m_oClassList.Lookup(sClassName, sDestinationValue) )
      {
         if( _ttoi(sSourceValue) < _ttoi(sDestinationValue) )
         {
            fUpdateClassList = false;
         }
      }
      if( fUpdateClassList )
      {
         m_oClassList[sClassName] = sSourceValue;
      }
   }
   return fResult;
}

// ***************************************************************************
// results access methods

// ***************************************************************************
//
void
SMJava::InitLabels()
{
   m_saLabels.SetSize(TOTAL_COLUMNS);

   // set some default labels and column alignment
   for( int i = 0; i < TOTAL_COLUMNS; ++i )
   {
      CString  sLabel;

      sLabel.LoadString(IDS_JAVA_DEFAULT_LABEL0 + i);
      m_saLabels[i] = sLabel;
   }
   // now override with user's labels, if any
   dynamic_cast<CSourceMetricsApp *>(AfxGetApp())->
      GetProfileStringList(CSourceMetricsApp::OPTION_SETTINGS + JAVA_NAME, CSourceMetricsApp::METRICS_LABELS_KEY,
                           m_saLabels, CString(_T(";,")), CString(_T("")));
}

// ***************************************************************************
//
CString
SMJava::GetMetricName(int index) const
{
   CString  sMetricName;

   if( (index >= 0) and (index < TOTAL_COLUMNS) )
   {
      sMetricName.LoadString(IDS_JAVA_METRIC0 + index);
   }
   return sMetricName;
}

CString
SMJava::GetValue(const int index, bool format /* = true */) const       // access a metric's value
{
   CString  sValue;
   int      iValue = -1;            // start with invalid value
   float    flValue = 0.0;
   int      i;                      // loop index

   sValue.LoadString(IDS_UNDEFINED_VALUE);
   if( (index >= 0) and (m_iCounts.GetSize() > index) )
   {
      switch( index )
      {
      case LINES:
         iValue = m_iCounts[LINE_COUNT];
         break;

      case STATEMENTS:
         iValue = m_iCounts[STATEMENT_COUNT];
         break;

      case PERCENT_BRANCHES:
         if( m_iCounts[STATEMENT_COUNT] > 0 )
         {
            flValue = float(m_iCounts[BRANCH_STATEMENTS] * 100) / float(m_iCounts[STATEMENT_COUNT]);
         }
         sValue.Format("%.1f", flValue);
         break;

      case CALLS:
         iValue = m_iCounts[CALL_COUNT] - m_iCounts[METHOD_COUNT];
         iValue = max(iValue, 0);
         break;

      case PERCENT_COMMENTS:
         if( m_iCounts[LINE_COUNT] > 0 )
         {
            flValue = float(m_iCounts[COMMENT_LINES] * 100) / float(m_iCounts[LINE_COUNT]);
         }
         sValue.Format("%.1f", flValue);
         break;

      case CLASSES:
         iValue = m_iCounts[CLASS_COUNT];
         break;

      case METHODS_PER_CLASS:
         if( m_iCounts[CLASS_COUNT] > 0 )
         {
            flValue = float(m_iCounts[METHOD_COUNT]) / float(m_iCounts[CLASS_COUNT]);
         }
         sValue.Format("%.2f", flValue);
         break;

      case AVERAGE_STATEMENTS_PER_METHOD:
         if( m_iCounts[METHOD_COUNT] > 0 )
         {
            flValue = float(m_iCounts[METHOD_STATEMENTS]) / float(m_iCounts[METHOD_COUNT]);
         }
         sValue.Format("%.2f", flValue);
         break;

      case BIGGEST_METHOD:
         {
            CString  sClassName;
            CString  sClassValue;
            int      iBiggestLineCount = 0;
            POSITION oListPosition = m_oClassList.GetStartPosition();

            while( oListPosition != NULL )
            {
               m_oClassList.GetNextAssoc(oListPosition, sClassName, sClassValue);
               if( _ttoi(sClassValue) > iBiggestLineCount )
               {
                  iBiggestLineCount = _ttoi(sClassValue);
               }
            }
            if( iBiggestLineCount > 0 )
            {
               sValue.Format("%d", iBiggestLineCount);
            }
            else
            {
               sValue.Format("%d", 0);
            }
         }
         break;

      case MAXIMUM_BLOCK_DEPTH:
         {
            sValue.Format("%d +", BLOCK_DEPTH_COUNTS - 1);    // assume the worst

            for( i = 0; i < BLOCK_DEPTH_COUNTS; ++i )
            {
               if( m_iCounts[i + BLOCK_DEPTH_0] == 0 )
               {
                  // counts are base 0, depths are base 0
                  sValue.Format("%ld", max(i - 1, 0));
                  break;
               }
            }
         }
         break;

      case AVERAGE_BLOCK_DEPTH:
         {
            long  lSum = 0;
            long  lCounts = 0;

            for( i = 0; i < BLOCK_DEPTH_COUNTS; ++i )
            {
               long  lCount = m_iCounts[i + BLOCK_DEPTH_0];

               if( lCount > 0 )
               {
                  lSum += lCount * (i + 1);
                  lCounts += lCount;
               }
               else
               {
                  break;
               }
            }
            if( lCounts > 0 )                // if no blocks, avg = 0.0
            {
               flValue = (float(lSum) / float(lCounts)) - (float)1.0;
            }
            sValue.Format("%.2f", flValue);
         }
         break;
      }
   }
   if( iValue >= 0 )
   {
      if( format )
      {
         sValue = FormatCount(iValue);
      }
      else
      {
         sValue.Format("%d", iValue);
      }
   }
   return sValue;
}

// ***************************************************************************
//
bool
SMJava::GetDeepestBlockLine(CString & sLabel, int * piResult) const
{
   CString  sDeepestLabel;

   sDeepestLabel.LoadString(IDS_DEEPEST_BLOCK_LABEL);
   sLabel.Format("%s %s)", sDeepestLabel, GetValue(MAXIMUM_BLOCK_DEPTH));
   *piResult = m_iCounts[DEEPEST_BLOCK_LINE];
   return true;
}

// ***************************************************************************
//
bool
SMJava::GetBiggestModuleLine(CString & sLabel, int * piResult) const
{
   bool  fResult = false;

   if( (m_iCounts[METHOD_COUNT] > 0) and (m_iCounts[BIGGEST_METHOD_LINE] > 0) )
   {
      sLabel.LoadString(IDS_BIGGEST_METHOD_LABEL);
      *piResult = m_iCounts[BIGGEST_METHOD_LINE];
      fResult = true;
   }
   return fResult;
}

// ***************************************************************************
//
void
SMJava::GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                     bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const
{
   CString  sEntity;
   CString  sValue;
   CString  sEmpty;        // used as empty string only
   POSITION oListPosition;
   CString  sClassName;
   CString  sClassValue;
   CString  sBiggestClass;
   CString  sBiggestMethod;
   CString  sInterface;
   int      iBiggestLineCount;
   int      i;                   // loop index (reused)

   for( i = 0; i < TOTAL_COLUMNS; ++i )
   {
      switch( i )
      {
      case MAXIMUM_BLOCK_DEPTH:
         if( oVersion.IsOKVersion(1, 0)                     // added line number of deepest block in V1.1
            and (m_iCounts[DEEPEST_BLOCK_LINE] != UINT_MAX) )
         {
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_DEEPEST_LINE_NUMBER);
               oEntities.Add(sEntity);
            }
            sValue.Format("%d", m_iCounts[DEEPEST_BLOCK_LINE]);
            oValues.Add(sValue);
         }
         else if( fIncludeExportValues )
         {
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_DEEPEST_LINE_NUMBER);
               oEntities.Add(sEntity);
            }
            oValues.Add(sEmpty);
         }
         break;

      case BIGGEST_METHOD:
         {
            iBiggestLineCount = 0;
            oListPosition = m_oClassList.GetStartPosition();
            while( oListPosition != NULL )
            {
               m_oClassList.GetNextAssoc(oListPosition, sClassName, sClassValue);
               if( _ttoi(sClassValue) > iBiggestLineCount )
               {
                  sBiggestClass = sClassName;
                  sBiggestMethod = sClassValue.Mid(sClassValue.Find('@') + 1);
                  iBiggestLineCount = _ttoi(sClassValue);
               }
            }
            if( iBiggestLineCount > 0 )
            {
               sValue.Format("%s.%s", sBiggestClass, sBiggestMethod);
            }
            else
            {
               sValue.LoadString(IDS_CPP_NOCLASS_BIGMETHOD);   // set no class message
            }
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_JAVA_BIGGEST_METHOD_NAME);
               oEntities.Add(sEntity);
            }
            oValues.Add(sValue);
            if( (m_iCounts[BIGGEST_METHOD_LINE] != UINT_MAX) and (iBiggestLineCount > 0) )
            {
               if( fIncludeEntities )
               {
                  sEntity.LoadString(IDS_BIGGEST_MODULE_LINE_NUMBER);
                  oEntities.Add(sEntity);
               }
               sValue.Format("%d", m_iCounts[BIGGEST_METHOD_LINE]);
               oValues.Add(sValue);
            }
         }
         break;
      }
      if( fIncludeEntities )
      {
         oEntities.Add(GetMetricName(i));
      }
      oValues.Add(GetValue(i));
   }
   if( not fIncludeExportValues )
   {
      // now display the classes under inserted column labels
      sInterface.LoadString(IDS_JAVA_INTERFACE_BIGGEST);
      sInterface = _T("0@") + sInterface;
      sValue = _T("--------------------------------");
      oEntities.Add(sValue);        // insert separators
      oValues.Add(sValue);
      sEntity = GetMetricName(CLASSES);
      sEntity += _T(":");
      oEntities.Add(sEntity);       // label the items to follow
      sEntity.LoadString(IDS_JAVA_BIGGEST_METHOD_NAME);
      sValue.Format("%s (%s)", sEntity, GetMetricName(STATEMENTS));
      oValues.Add(sValue);
      oEntities.Add(sEmpty);        // add blank line
      oValues.Add(sEmpty);
      oListPosition = m_oClassList.GetStartPosition();
      while( oListPosition != NULL )
      {
         m_oClassList.GetNextAssoc(oListPosition, sClassName, sClassValue);
         if( sClassValue == sInterface )
         {
            sValue.LoadString(IDS_JAVA_HAS_INTERFACE);
         }
         else
         {
            sBiggestMethod = sClassValue.Mid(sClassValue.Find('@') + 1);
            iBiggestLineCount = _ttoi(sClassValue);
            sValue.Format("%s (%d)", sBiggestMethod, iBiggestLineCount);
         }
         oEntities.Add(sClassName);
         oValues.Add(sValue);
      }
   }
   // now fill the block depth array
   oBlockCounts.SetSize(BLOCK_DEPTH_COUNTS);
   for( i = 0; i < BLOCK_DEPTH_COUNTS; ++i )
   {
      oBlockCounts[i] = m_iCounts[BLOCK_DEPTH_0 + i];
   }
}

// ***************************************************************************
//
void
SMJava::Serialize(CArchive & ar)
{
   SMLanguage::Serialize(ar);
   m_iCounts.Serialize(ar);
   m_oClassList.Serialize(ar);

   return;
}
