#ifndef  SMJava_H
#define	SMJava_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMJava.h	[ SMJava.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of Java metrics language class SMJava
//
//***************************************************************************

class SSLexTable;
class SSYaccTable;
class JavaLexClass;

class SMJava : public SMLanguage
{
   public:
      // define indexes into the metrics count accumulator array (leave one spare in the middle--index = 10)
      enum COUNT_TYPE { LINE_COUNT = 0, STATEMENT_COUNT = 1, BRANCH_STATEMENTS = 2, COMMENT_LINES = 3,
                        CLASS_COUNT = 4, METHOD_COUNT = 5, METHOD_STATEMENTS = 6, CALL_COUNT = 7,
                        DEEPEST_BLOCK_LINE = 8, BIGGEST_METHOD_LINE = 9, BLOCK_DEPTH_0 = 11, BLOCK_DEPTH_COUNTS = 10, 
                        TOTAL_COUNTS = 21 };

      // used only as a class factory (in SMProject and COptionsLanguagePage)
      explicit SMJava(Exemplar /* not used */) : SMLanguage() {;} // used only in SMProject as factory for this class
      virtual  SMLanguage * MakeNew();                            // create a shallow copy
      virtual  ~SMJava();

      // virtual overrides
	   virtual void Serialize(CArchive& ar);

      // property access methods
      virtual CString & Name() const                     { return JAVA_NAME; }
      virtual CString & DefaultExtensions() const        { return JAVA_EXTENSIONS; }

      // signature for common methods of sub-classes
      virtual bool   Start(MetricsType type);

      // compute and accumulate metrics for one line
      virtual void   InitParser();
      virtual void   CloseParser();
      virtual bool   ProcessLines(const CString & sFileName);
      virtual void   ParseError();
      virtual void   BumpCount(int index)                { ++m_iCounts[index]; }
      virtual void   SetMethodData(const CString & sClassName, const CString & sMethodName, int iStatementCount);
      virtual void   SetBiggestMethodLine()              { m_iCallOrMethodLine = m_iCounts[LINE_COUNT] + 1; }
      virtual void   ConfirmBiggestMethodLine()          { m_iBeginMethodLine = m_iCallOrMethodLine; }
      virtual void   SetDeepestBlockLine()               { m_iCounts[DEEPEST_BLOCK_LINE] = m_iCounts[LINE_COUNT] + 2; }

      // accumulate across group of files
      virtual bool   ProcessFile(const SMLanguage & oFileMetric);
 
      // results access methods (indexes identify columns displayed to user: base 0)
      virtual short     Columns() const                  { return TOTAL_COLUMNS; }
      virtual CString   GetMetricName(int index) const;
      virtual void      InitLabels();
      virtual CString   GetValue(int index, bool format = true) const;
      virtual int       GetIntValue(int index) const     { return m_iCounts[index]; }
      virtual bool      GetDeepestBlockLine(CString & sLabel, int * piResult) const;
      virtual bool      GetBiggestModuleLine(CString & sLabel, int * piResult) const;
      virtual void      GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                                    bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const;
      virtual int       BlockCounts()                                { return BLOCK_DEPTH_COUNTS; }

      static CString    JAVA_NAME;           // this is how this language is identified to one and all
      static CString    JAVA_EXTENSIONS;     // use until user changes options

   protected:
      SMJava();                        // no public default constructor: use MakeNew method

   private:
      // define results columns NOTE: MUST KEEP STRING RESOURCES IN SYNCH WITH THESE MAGIC NUMBERS!!!
      enum COLUMN_TYPE { LINES, STATEMENTS, PERCENT_BRANCHES, CALLS, PERCENT_COMMENTS, CLASSES, METHODS_PER_CLASS,
                         AVERAGE_STATEMENTS_PER_METHOD, BIGGEST_METHOD, MAXIMUM_BLOCK_DEPTH,
                         AVERAGE_BLOCK_DEPTH, TOTAL_COLUMNS };

      // persistent metrics data
      CDWordArray          m_iCounts;        // raw counts
      CMapStringToString   m_oClassList;     // classes with biggest method

      // transient data members
      int                  m_iCallOrMethodLine;
      int                  m_iBeginMethodLine;
      int                  m_iBiggestMethodStatements;

      // static data members
      static int              m_iParserUsers;// reference count for parsing objects
      static JavaLexClass *   m_poLexer;
      static SSLexTable *     m_poLexTable;
      static SSYaccTable *    m_poYaccTable;

      DECLARE_SERIAL(SMJava)
};
////////////////////////////////////////////////////////////////////////////////

#endif						// SMJava_H
