//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMLanguage.cpp	[ SMLanguage.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implements data container class SMLanguage
//
//***************************************************************************

#include "stdafx.h"

#include <Locale.h>
#include "SourceMetrics.h"
#include "SMLanguage.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(SMLanguage, CObject, 1)

// static members
CStringArray   SMLanguage::m_saLabels;
CString        SMLanguage::DEFAULT_STRING = CString(_T("ERROR"));

// ***************************************************************************
// called by a language after a file has been parsed to report parsing errors
void
SMLanguage::ReportErrors(const CString & sFileName) const
{
   int   iErrors = m_asErrorLines.GetSize();
   if( iErrors > 0 )
   {
      CString  sCaption;
      CString  sMessageTemplate;
      int      iFileTokenOffset;
      int      iLineTokenOffset;
      CString  sMessage;

      sCaption.LoadString(IDS_PARSE_ERROR_CAPTION);
      sMessageTemplate.LoadString(IDS_PARSE_ERROR_TEMPLATE);
      iFileTokenOffset = sMessageTemplate.Find("%f%");
      iLineTokenOffset = sMessageTemplate.Find("%n%");

      // replace %f% with the file name, %n% with the line number(s) (in m_aiErrorLines)
      sMessage = sMessageTemplate.Left(iFileTokenOffset) + sFileName;
      sMessage += sMessageTemplate.Mid(iFileTokenOffset + 3, iLineTokenOffset - iFileTokenOffset - 3);
      for( int i = 0; i < iErrors; ++i )
      {
         sMessage += m_asErrorLines[i];

         if( i < (iErrors - 1) )
         {
            sMessage += _T(", ");;
         }
      }
      sMessage += sMessageTemplate.Mid(iLineTokenOffset + 3);
      if( AfxGetMainWnd()->MessageBox(sMessage, sCaption, MB_ICONWARNING bitor MB_YESNO) == IDNO )
      {
         AfxGetApp()->WriteProfileInt(CSourceMetricsApp::OPTION_SETTINGS, CSourceMetricsApp::REPORT_ERRORS, 0);
      }
   }
}

// ***************************************************************************
// read a string from the label list
CString
SMLanguage::GetLabel(const int index) const       // access a metric's label
{
   CString  sLabel;

   if( (index >= 0) and (index < m_saLabels.GetSize()) )
   {
      sLabel = m_saLabels[index];
   }
   return sLabel;
}

// ***************************************************************************
// Create a string from a count with thousands separators per current locale
CString
SMLanguage::FormatCount(long count)
{
   CString  sCount;                          // raw count as a string
   CString  sResult;                         // thousands separated string
   int      iLength;                         // unprocessed characters in un-separated string
   int      iSubLength;                      // characters to next separator
   int      iStartOffset;                    // index of next unprocessed charater to extract
   CString  sSeparator(localeconv()->thousands_sep);

   if( sSeparator.GetLength() == 0 )
   {
      sSeparator = _T(",");
   }
   sCount.Format("%ld", count);              // first create the unseparated numeric string
   iLength = sCount.GetLength();
   if( (iSubLength = iLength % 3) == 0 )
   {
      iSubLength = 3;
   }
   iStartOffset = 0;
   do
   {
      sResult += sCount.Mid(iStartOffset, iSubLength);
      iLength -= iSubLength;
      if( iLength > 0 )                      // see if we have another subsection to do
      {
         sResult += sSeparator;              // yes, need a spearator
         iStartOffset += iSubLength;
         iSubLength = 3;
      }
   }
   while( iLength > 0 );
   return sResult;
}

// ***************************************************************************
//
void
SMLanguage::Serialize(CArchive & ar)
{
   ComplexMetrics.Serialize(ar);

   return;
}

