#ifndef  SMLanguage_H
#define	SMLanguage_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMLanguage.h	[ SMLanguage.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of metrics language abstract base class SMLanguage
//             This class uses the abstract base exemplar idiom (see Coplien,
//             Advanced C++, p. 293ff.). There are no public constructors
//             except those that require an Exemplar class object as an
//             argument. These are used to create one (uninitialized) class
//             embedded in each SMProject object. All other language objects
//             in the project are created by the "MakeNew" method of an
//             existing language object. In this way, all will be of the same
//             specialization of SMLanguage. And introduction of new language
//             class specializations does not require any changes except to
//             the code that creates new projects.
//
//***************************************************************************

class SMVersion;

class Exemplar
{
public:
   Exemplar() {;}       // does nothing except identify a class type
};

#pragma warning(disable:4100)    // we know many parameters are not used

class SMLanguage : public CObject
{
   public:
      virtual SMLanguage * MakeNew() { return NULL; }        // create a shallow copy
      virtual ~SMLanguage()  {;}

      // virtual overrides
	   virtual void Serialize(CArchive& ar);
                     
      //
      // signitures for common methods of sub-classes
      //

      // expose the name of a language
      virtual CString & Name() const                                 { return DEFAULT_STRING; }
      virtual CString & DefaultExtensions() const                    { return DEFAULT_STRING; }

      // reset all counters and get ready to accumulate into metric data object passed in
      enum MetricsType { mtFile, mtCheckpoint };
      virtual bool      Start(MetricsType type)                      { return false; }

      // compute and accumulate metrics for one file, report metrics details
      virtual bool      ProcessLines(const CString & sFileName)      { return false; }
      virtual void      ParseError()                                 { ; }
      virtual void      InitParser()                                 { ; }
      virtual void      CloseParser()                                { ; }

      // accumulate across group of files
      virtual bool      ProcessFile(const SMLanguage & oFileMetric)  { return false; }

      // results access methods (indexes identify columns displayed to user: base 0)
      virtual short     Columns() const { return 0; }   // number of metrics value columns to display
      virtual CString   GetMetricName(int index) const               { return DEFAULT_STRING; }
      virtual void      InitLabels()                                 { return; }
      virtual CString   GetLabel(int index) const;
      virtual void      GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                                    bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const
                                                                     {;}
      virtual CString   GetValue(int index, bool format = true) const { return DEFAULT_STRING; }
      virtual bool      GetDeepestBlockLine(CString & sLabel, int * piResult) const
                                                                     { return false; }
      virtual bool      GetBiggestModuleLine(CString & sLabel, int * piResult) const
                                                                     { return false; }
      virtual int       BlockCounts()                                { return 0; }

      // data handled entirely by this base class
      int         ComplexMetricCount() const                         { return ComplexMetrics.GetSize(); }
      void        SetComplexMetricCount(const int count)             { ComplexMetrics.SetSize(count); }
      CString     ComplexMetric(int index) const                     { return ComplexMetrics.GetAt(index); }
      void        ComplexMetric(int index, CString metric)           { ComplexMetrics.SetAt(index, metric); }
      CString     GetLastError() const                               { return m_sError; }

      // helper methods
      static CString FormatCount(long count);
      void           ReportErrors(const CString & sFileName) const;

   protected:
      SMLanguage() {;}                       // no public default constructor: use MakeNew method
                                             // (must match the language key used in the registry)
      static CString    DEFAULT_STRING;

      // transient data members
      static CStringArray
                     m_saLabels;                // labels read from the registry (defined by user)
      CStringArray   m_asErrorLines;            // line numbers of lines with parse errors
      CString        m_sError;                  // text of parsing error, if any

   private:
      // persisted properties
      CStringArray   ComplexMetrics;         // container for complex metrics strings

      // make private and don't implement to prevent copies except via public MakeNew method
      SMLanguage(const SMLanguage & lang);            // copy constructor
      SMLanguage & operator=(const SMLanguage& lang); // assignment operator

      DECLARE_SERIAL(SMLanguage)
};

#pragma warning(default:4100)

////////////////////////////////////////////////////////////////////////////////

#endif						// SMLanguage_H
