//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMProject.cpp	[ SMProject.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implements data container class SMProject
//
//***************************************************************************

#include "stdafx.h"
#include <afxmt.h>

#include "SourceMetrics.h"
#include "ProgressDlgThread.h"
#include "MetricsProgressDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(SMProject, CObject, 1)

// **************************************************************************
// global function for the CList of checkpoints
//
void AFXAPI
SerializeElements(CArchive &ar, SMCheckpoint *pElements, int nCount)
{
   for( int i = 0; i < nCount; ++i, ++pElements )
   {
      pElements->Serialize(ar);
   }
}

// **************************************************************************
//
SMProject::SMProject() : m_poLanguageExemplar(NULL)
{
}

// **************************************************************************
//
SMProject::~SMProject()
{
   if( m_poLanguageExemplar != NULL )
   {
      m_poLanguageExemplar->CloseParser();
      delete m_poLanguageExemplar;
   }
   return;
}

// **************************************************************************
//
void
SMProject::SetLanguage(SMLanguage * exemplar)
{
   m_poLanguageExemplar = exemplar;

   // initialize the lex and yacc objects for the project's language
   m_poLanguageExemplar->InitParser();
}

// **************************************************************************
//
CString
SMProject::GetValue(int index) const
{
   if( (index >= 0) and (index < m_poLanguageExemplar->Columns()) )
   {
      return m_poLanguageExemplar->GetValue(index);
   }
   else
   {
      return CString("");
   }
}

// **************************************************************************
//
bool
SMProject::GetNewCheckpointFiles(CStringList & oCkptFiles, CStringList & oOtherFiles)
{
   CStringArray   saDirectories;       // array of directories to search
   CStringArray   saExtensionList;     // list of file extensions to search for
   CSourceMetricsApp *
                  poApp = dynamic_cast<CSourceMetricsApp *>(AfxGetApp());
   bool           fResult = true;      // assume success

   if( not poApp->GetProfileStringList(CSourceMetricsApp::OPTION_SETTINGS + m_poLanguageExemplar->Name(),
                                    CSourceMetricsApp::LANGUAGE_EXTENSIONS, saExtensionList, CString(_T(";,")),
                                    m_poLanguageExemplar->DefaultExtensions()) )
   {
      return false;;
   }
   // make a list of all directories to search for files
   saDirectories.Add(m_sDirectory);  // always begin with the metrics directory itself
   if( m_fIncludeSubdirectories )
   {
      BuildDirectoryList(m_sDirectory, saDirectories);
   }
   // search in each directory for files to include in the new project
   for( int i = 0; i < saDirectories.GetSize(); i++ )
   {
      CFileFind   oFindFile;        // file search object
      BOOL        fFileFound;       // true if next file found
      CString     sSearchDirectory; // directory being searched
      CString     sSubdirectory;    // path from project directory to the sub-directory being searched
      CString     sFileName;

      sSearchDirectory = saDirectories.GetAt(i);
      sSubdirectory = sSearchDirectory.Right(sSearchDirectory.GetLength() - m_sDirectory.GetLength());
      for( int e = 0; e < saExtensionList.GetSize(); ++e )
      {
         // look at files with current extension
         CString  sSearchFor = sSearchDirectory + saExtensionList[e];

         fFileFound = oFindFile.FindFile(sSearchFor);
         while( fFileFound != 0 )
         {
            fFileFound = oFindFile.FindNextFile();
            sFileName = oFindFile.GetFileName();
            if( not oFindFile.IsDirectory() )
            {
               oCkptFiles.AddTail(sSubdirectory + oFindFile.GetFileName());
            }
         }
         oFindFile.Close();
      }
      // now put all unselected files in the Other string list
      fFileFound = oFindFile.FindFile(sSearchDirectory + _T("*.*"));
      while( fFileFound )
      {
         CString  sFileName;

         fFileFound = oFindFile.FindNextFile();
         if( not oFindFile.IsDirectory() )
         {
            sFileName = sSubdirectory + oFindFile.GetFileName();
            if( (oCkptFiles.Find(sFileName) == NULL)
            and ((sFileName.Right(4)).CompareNoCase(_T(".smp")) != 0) ) // exclude our own project files
            {
               oOtherFiles.AddTail(sSubdirectory + oFindFile.GetFileName());
            }
         }
      }
      oFindFile.Close();
   }
   return fResult;
}

// **************************************************************************
//
bool
SMProject::CreateCheckpoint(const CString & sCkptName, const CStringList & oCkptFiles,
                            CWnd * poParentWnd /*= NULL*/)
{
   SMCheckpoint   oNewCkpt;
   CProgressDlgThread *
                  poProgressThread = (CProgressDlgThread *)AfxBeginThread(RUNTIME_CLASS(CProgressDlgThread),
                                                                           THREAD_PRIORITY_HIGHEST, 0, CREATE_SUSPENDED);
   CMetricsProgressDlg *
                  poProgressDlg;
   POSITION       oFilePosition = oCkptFiles.GetHeadPosition();
   CString        sErrorMessage;
   bool           fUserCancelled = false;
   bool           fResult = true;            // assume success

   // get the progress dialog up and running in its thread
   poProgressThread->InitDlg(oCkptFiles.GetCount(), poParentWnd);
   if( poProgressThread->ResumeThread() == 1 )
   {
      poProgressDlg = poProgressThread->GetDlg();
      if( CSingleLock(poProgressDlg->GetEvent()).Lock(5000) == 0 )   // wait up to 5 seconds
      {
         sErrorMessage.LoadString(IDS_NEWCKPT_PROGRESS_DLG_ERROR);
         fResult = false;
      }
      else
      {
         oNewCkpt.Name(sCkptName);
         oNewCkpt.m_poLanguage = m_poLanguageExemplar->MakeNew();
         oNewCkpt.SetVersion(CSourceMetricsApp::m_iMajorVersion, CSourceMetricsApp::m_iMinorVersion);
         oNewCkpt.m_poLanguage->Start(SMLanguage::mtCheckpoint);
      }
   }
   else
   {
      sErrorMessage.LoadString(IDS_NEWCKPT_PROGRESS_DLG_ERROR);
      fResult = false;
   }
   // then do the actual parsing of all files in the checkpoint
   while( fResult and (oFilePosition != NULL) and not fUserCancelled )
   {
      CString     sFileName = oCkptFiles.GetNext(oFilePosition);
      SMFile      oSMFile;                   // project's SMFile object

      poProgressDlg->SetFileName(sFileName);
      oSMFile.PathName(sFileName);
      oSMFile.m_poLanguage = m_poLanguageExemplar->MakeNew();
      oSMFile.m_poLanguage->Start(SMLanguage::mtFile);
      if( not oSMFile.m_poLanguage->ProcessLines(m_sDirectory + sFileName) )
      {
         // report error with the parsing of the file and then keep going
         sErrorMessage.LoadString(IDS_NEWCKPT_PARSE_ERROR_PREFIX);
         sErrorMessage += oSMFile.m_poLanguage->GetLastError();
         ReportCheckpointError(sErrorMessage);
      }
      oNewCkpt.m_poLanguage->ProcessFile(*oSMFile.m_poLanguage);
      fUserCancelled = poProgressDlg->IsCancelled();
      if( fResult and not oNewCkpt.AddFile(oSMFile) )
      {
         sErrorMessage.LoadString(IDS_NEWCKPT_ADDFILE_ERROR);
         fResult = false;
      }
   }
   // done parsing files
   if( fUserCancelled )
   {
      fResult = false;
   }
   else if( fResult )
   {
      CString  sFileName;

      poProgressDlg->SetFileName(sFileName);          // send one last (dummy) to paint the full bar
      if( m_oCheckpoints.AddHead(oNewCkpt) == NULL )  // copy data into the checkpoint list
      {
         sErrorMessage.LoadString(IDS_NEWCKPT_ADDCKPT_ERROR);
         fResult = false;
      }
   }
   else
   {
      ReportCheckpointError(sErrorMessage);
   }
   poProgressDlg->PostMessage(WM_QUIT);               // must destroy the window to stop the thread

   return fResult;
}

// **************************************************************************
// Report new checkpoint error
//
void
SMProject::ReportCheckpointError(CString & sErrorMessage)
{
   CString  sCaption;

   sCaption.LoadString(IDS_NEWCKPT_ERROR_CAPTION);
   AfxGetMainWnd()->MessageBox(sErrorMessage, sCaption, MB_ICONSTOP bitor MB_OK);
}

// **************************************************************************
// Replace a checkpoint with updated data
//
bool
SMProject::UpdateCheckpoint(SMCheckpoint & oCkpt)
{
   POSITION       oFindPosition = FindCheckpoint(oCkpt);
   bool           fResult = false;     // assume failure

   if( oFindPosition != NULL )
   {
      m_oCheckpoints.SetAt(oFindPosition, oCkpt);  // update existing copy in place
      fResult = true;
   }
   return fResult;
}

// **************************************************************************
// Delete the currently  selected checkpoint
//
bool
SMProject::DeleteCheckpoint(int index)
{
   POSITION oPosition;        // search result
   bool     fResult = false;  // assume failure

   oPosition = m_oCheckpoints.FindIndex(index);
   if( oPosition != NULL )
   {
      AddUndoCheckpoint(m_oDeletedCheckpoints.GetAt(oPosition));
      m_oCheckpoints.RemoveAt(oPosition);
      fResult = true;            // found the checkpoint
   }
   return fResult;
}

// **************************************************************************
// Restore a deleted checkpoint to its proper reverse chronological order
//
bool
SMProject::RestoreDeletedCheckpoint()
{
   SMCheckpoint   oCkpt;            // scratch for date order search
   POSITION       oPosition;        // search result
   POSITION       oInsertPosition;  // position where deleted checkpoint should be inserted
   bool           fResult = false;  // assume failure

   if( not m_oDeletedCheckpoints.IsEmpty() )
   {
      SMCheckpoint   oDeletedCkpt = m_oDeletedCheckpoints.RemoveTail();

      if( m_oCheckpoints.IsEmpty() )
      {
         fResult = not m_oCheckpoints.AddHead(oDeletedCkpt) == NULL;
      }
      else
      {
         oPosition = m_oCheckpoints.GetHeadPosition();
         while( oPosition != NULL )
         {
            oInsertPosition = oPosition;        // save compare position in case it's where we need to insert
            oCkpt = m_oCheckpoints.GetNext(oPosition);
            if( oCkpt.Date() < oDeletedCkpt.Date() )
            {
               fResult = not m_oCheckpoints.InsertBefore(oInsertPosition, oDeletedCkpt) == NULL;
               break;
            }
         }
         if( fResult == false )
         {
             fResult = not m_oCheckpoints.AddTail(oDeletedCkpt) == NULL;
         }
      }
   }
   return fResult;
}
      
// **************************************************************************
// use to iterate through the entire file list
//
bool
SMProject::GetNextCheckpoint(bool first, SMCheckpoint & oCkpt) const
{
   static POSITION oPosition;       // iteration reference
   bool fResult = true;             // assume success

   if( first )                            // first time, set up the iteration reference
   {
      oPosition = m_oCheckpoints.GetHeadPosition();
   }
   if( oPosition == NULL )
   {
      fResult = false;
   }
   else
   {
      oCkpt = m_oCheckpoints.GetNext(oPosition);
   }
   return fResult;
}

// **************************************************************************
// use to get checkpoint based on its index in the project list
//
bool
SMProject::GetCheckpointByIndex(int index, SMCheckpoint & oCkpt) const
{
   bool  fResult = true;            // assume success

   if( (index >= 0) and (index < m_oCheckpoints.GetCount()) )
   {
      oCkpt = m_oCheckpoints.GetAt(m_oCheckpoints.FindIndex(index));
   }
   else
   {
      fResult = false;
   }
   return fResult;
}

// **************************************************************************
// get current copy of a checkpoint based on its presence in the project list
//
bool
SMProject::GetUpdatedCheckpoint(const SMCheckpoint & oCkpt, SMCheckpoint & oUpdatedCkpt) const
{
   POSITION oFindPosition = FindCheckpoint(oCkpt);
   bool     fResult = false;

   if( oFindPosition != NULL )
   {
      oUpdatedCkpt = m_oCheckpoints.GetAt(oFindPosition);  // return a copy of the existing checkpoint
      fResult = true;
   }
   return fResult;
}

// **************************************************************************
// Assign a new name to a checkpoint
bool
SMProject::RenameCheckpoint(int index, CString sNewName)
{
   POSITION       oPosition = m_oCheckpoints.FindIndex(index);
   SMCheckpoint   oCkpt;
   bool           fResult = true;      // assume success

   if( oPosition == NULL )
   {
      fResult = false;
   }
   else
   {
      oCkpt = m_oCheckpoints.GetAt(oPosition);
      oCkpt.Name(sNewName);
      m_oCheckpoints.InsertAfter(oPosition, oCkpt);
      m_oCheckpoints.RemoveAt(oPosition);
   }
   return fResult;
}

// **************************************************************************
// get number of characters in longest checkpoint name (used in printing list)
int
SMProject::MaxCheckpointNameLength(CDC * pDC)
{
   SMCheckpoint   oCkpt;
   POSITION       oPosition = m_oCheckpoints.GetHeadPosition();
   CSize          oNameSize;
   int            iResult = 0;     // start with a safe minimum

   while( oPosition != NULL )
   {
      oCkpt = m_oCheckpoints.GetNext(oPosition);   // get next ckpt in list
      oNameSize = pDC->GetTextExtent(oCkpt.Name());
      iResult = max(iResult, oNameSize.cx);
   }
   return iResult;
}

// **************************************************************************
//
// Build list of sub-directories (used recursively--must be reentrant!)
//
void
SMProject::BuildDirectoryList(const CString & sSearchDirectory, CStringArray & saDirectories)
{
   HANDLE            hFind;               // open find process handle
   WIN32_FIND_DATA   oFindData;           // file search return data container
   CString           sAllFilesFilter(_T("*.*"));

   hFind = FindFirstFile(sSearchDirectory + sAllFilesFilter, &oFindData);
   if( hFind == INVALID_HANDLE_VALUE )
   {
      CString  sError;

      sError.LoadString(IDS_CKPT_DIRECTORY_OPEN_ERROR);
      ReportFileError(sError);
   }
   else
   {
      do       // search for all sub-directories
      {
         if( (oFindData.dwFileAttributes bitand FILE_ATTRIBUTE_DIRECTORY)
         and (oFindData.cFileName[0] != '.') )
         {
            CString  sFoundDirectory(sSearchDirectory + oFindData.cFileName + _T("\\"));

            // add to list as a full path with final backslash, then descend and search again
            saDirectories.Add(sFoundDirectory);
            BuildDirectoryList(sFoundDirectory, saDirectories);
         }
      } while( FindNextFile(hFind, &oFindData)  );
      if( GetLastError() != ERROR_NO_MORE_FILES )
      {
         CString  sError;

         sError.LoadString(IDS_CKPT_SUBDIRECTORY_ERROR);
         ReportFileError(sError);
      }
   }
}

// **************************************************************************
//
// Append file error reported by GetLastError() to errmsg, display to user in a message box
//
void
SMProject::ReportFileError(const CString & errmsg, DWORD dwErrNo /* = 0x20000000 */)
{
   CString  sMessage(errmsg);    // append the error text to the description
   CString  sCaption;
   char *   pcErrorText;
   DWORD    dwErrorNumber = dwErrNo;

   if( dwErrorNumber == 0x20000000 )
   {
      dwErrorNumber = GetLastError();
   }
   FormatMessage( 
         FORMAT_MESSAGE_ALLOCATE_BUFFER bitor FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         dwErrorNumber,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &pcErrorText,
         0, NULL);

   // Display the string.
   sMessage += CString(":\n") + CString(pcErrorText);
   sCaption.LoadString(IDS_PROJ_FILE_ERROR_CAPTION);
   MessageBox( NULL, sMessage, sCaption, MB_OK bitor MB_ICONINFORMATION );

   // Free the buffer.
   LocalFree(pcErrorText);
   return;
}


// **************************************************************************
//
POSITION
SMProject::FindCheckpoint(const SMCheckpoint & oCkpt) const
{
   POSITION       oPosition = m_oCheckpoints.GetHeadPosition();
   POSITION       oSearchPosition;     // scratch req'd because GetNext advances the position
   SMCheckpoint   oSearchCkpt;

   while( oPosition != NULL )
   {
      oSearchPosition = oPosition;
      oSearchCkpt = m_oCheckpoints.GetNext(oPosition);
      if( oSearchCkpt == oCkpt )
      {
         break;
      }
   }
   return oSearchPosition;
}

// **************************************************************************
//
void
SMProject::Serialize(CArchive & ar)
{
   m_oVersion.Serialize(ar);                 // always read or write the version first

   // do the MFC classes and C++ types
	if( ar.IsStoring() )
	{
		ar << m_sName << m_fIncludeSubdirectories << m_poLanguageExemplar;
	}
	else
	{
      int   iOffset;    // offset to last backslash in pathname

		ar >> m_sName >> m_fIncludeSubdirectories >> m_poLanguageExemplar;

      // now set the directory to the current file's home (with trailing backslash)
      m_sDirectory = (ar.GetFile())->GetFilePath();
      iOffset = m_sDirectory.ReverseFind('\\');
      if( iOffset > 0 )
      {
         m_sDirectory = m_sDirectory.Left(iOffset + 1);  // save all but the executable filename
      }
	}
   // then the metrics data objects
   m_poLanguageExemplar->Serialize(ar);
	if( not ar.IsStoring() )
	{
      // initialize the lex and yacc objects for the project's language
      m_poLanguageExemplar->InitParser();
   }
   m_oCheckpoints.Serialize(ar);
}
