#ifndef  SMProject_H
#define	SMProject_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMProject.h	[ SMProject.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of data container class SMProject
//
//***************************************************************************

#include <afxtempl.h>
#include <list>
#include "SMVersion.h"

class SMLanguage;

// declare checkpoint list as a CList template container
void AFXAPI
   SerializeElements(CArchive &ar, SMCheckpoint *pElements, int nCount);

// store the objects in the container so they can be serialized
typedef CList<SMCheckpoint, SMCheckpoint&> CheckpointList;

class SMProject : public CObject
{
   public:
      SMProject();
      virtual ~SMProject();

      // virtual overrides
      virtual void Serialize(CArchive& ar);

       // property access methods
      CString        Name() const                        { return m_sName; }
      void           Name(CString name)                  { m_sName = name; }
      CString        Directory() const                   { return m_sDirectory; }
      void           Directory(CString dir)              { m_sDirectory = dir; }
      BOOL           IncludeSubdirectories() const       { return m_fIncludeSubdirectories; }
      void           IncludeSubdirectories(BOOL incl)    { m_fIncludeSubdirectories = incl; }
      void           SetLanguage(SMLanguage * exemplar);
      SMLanguage *   GetLanguage()                       { return m_poLanguageExemplar; }
      SMVersion &    Version()                           { return m_oVersion; }
      void           SetVersion(int iMajor, int iMinor)  { m_oVersion.SetVersion(iMajor, iMinor); }
      CString        GetCaption(int index) const         { return m_poLanguageExemplar->GetLabel(index); }
      CString        GetValue(int index) const;

      // checkpoint list manipulation
      bool     GetNewCheckpointFiles(CStringList & ckptfiles, CStringList & otherfiles);
      bool     CreateCheckpoint(const CString & sCkptName, const CStringList & oFiles, CWnd * pWnd = NULL);
      bool     UpdateCheckpoint(SMCheckpoint & oCkpt);
      bool     DeleteCheckpoint(int index);
      bool     RestoreDeletedCheckpoint();
      int      CheckpointCount() const             { return m_oCheckpoints.GetCount(); }
      bool     GetNextCheckpoint(bool first, SMCheckpoint & oCkpt) const; // use to iterate through the list
      bool     GetCheckpointByIndex(int index, SMCheckpoint & oCkpt) const;
      bool     GetUpdatedCheckpoint(const SMCheckpoint & oCkpt, SMCheckpoint & oUpdatedCkpt) const;
      bool     RenameCheckpoint(int index, CString sNewName);
      int      MaxCheckpointNameLength(CDC * pDC);
      void     AddUndoCheckpoint(SMCheckpoint & ckpt)    { m_oDeletedCheckpoints.AddTail(ckpt); }
      bool     CanUndoCheckpoint()                       { return not m_oDeletedCheckpoints.IsEmpty(); }


      // helper methods
      void     BuildDirectoryList(const CString & sSearchDirectory, CStringArray & oaDirectories);
      void     ReportCheckpointError(CString & sErrorMessage);
      static void
               ReportFileError(const CString & errmsg, DWORD dwErrNo = 0x20000000);

   private:
      CString        m_sName;
      BOOL           m_fIncludeSubdirectories;
      SMLanguage *   m_poLanguageExemplar;      // exemplar is used to build all project language objects
      SMVersion      m_oVersion;                // version of Source Monitor that created this project
      CheckpointList m_oCheckpoints;            // list of checkponts in the project

      // transient (non-persisted) members
      CString        m_sDirectory;              // directory where .smp file is located
      CheckpointList m_oDeletedCheckpoints;     // list of deleted checkponts

      // helper methods
      POSITION       FindCheckpoint(const SMCheckpoint & oCkpt) const;

      DECLARE_SERIAL(SMProject)
};
////////////////////////////////////////////////////////////////////////////////
#endif						// SMProject_H
