//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMSplashPanel.cpp	[ SMSplashPanel.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of class SMSplashPanel
//
//***************************************************************************

#include "stdafx.h"
#include "SourceMetrics.h"
#include "DibButton.h"
#include "SMSplashPanel.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SMSplashPanel dialog


SMSplashPanel::SMSplashPanel(int runcount, CSourceMetricsApp * pApp, CWnd * pParent)
	: m_iRunCount(runcount), m_poApp(pApp), m_poParent(pParent), CDialog(SMSplashPanel::IDD, pParent)
{
   CString  sWarning;

	//{{AFX_DATA_INIT(SMSplashPanel)
	m_sRunCountMessage = _T("");
	m_sSharewareMessage = _T("");
	m_sSharewarePrices = _T("");
	//}}AFX_DATA_INIT

   // let the punishment fit the crime
   switch( m_iRunCount / 30 )
   {
   case 0:
      sWarning = _T("Thank you for giving it a try.");
      break;
   case 1:
      sWarning = _T("It is time for you to register.");
      break;
   default:
      sWarning = _T("This is not freeware. Please register!");
      break;
   }
   m_sRunCountMessage.Format("You have now run %s %d times. %s", AfxGetApp()->m_pszAppName, m_iRunCount, sWarning);
   m_sSharewareMessage.LoadString(IDS_SHAREWARE_MSG1);
   m_sSharewarePrices.LoadString(IDS_SHAREWARE_MSG2);
   m_DibLogo.LoadDib(IDB_CAMPWOOD_LOGO, true);
}

BOOL SMSplashPanel::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
   if( not m_DibLogo.SubclassDlgItem(IDC_LOGO_PICTURE, this) )
   {
      EndDialog(IDCANCEL);
      return FALSE;
   }
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void SMSplashPanel::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SMSplashPanel)
	DDX_Control(pDX, IDC_EDIT_SHAREWARE_MESSAGE2, m_editSharewareMsg2);
	DDX_Control(pDX, IDC_EDIT_SHAREWARE_MESSAGE, m_editSharewareMsg);
	DDX_Text(pDX, IDC_EDIT_RUN_COUNT, m_sRunCountMessage);
	DDX_Text(pDX, IDC_EDIT_SHAREWARE_MESSAGE, m_sSharewareMessage);
	DDX_Text(pDX, IDC_EDIT_SHAREWARE_MESSAGE2, m_sSharewarePrices);
	//}}AFX_DATA_MAP

   if( not pDX->m_bSaveAndValidate )
   {
      CenterWindow(m_poParent);                          // put us in the middle of our parent's window
   }
}


BEGIN_MESSAGE_MAP(SMSplashPanel, CDialog)
	//{{AFX_MSG_MAP(SMSplashPanel)
	ON_BN_CLICKED(ID_REGISTER_BTN, OnRegisterBtn)
	ON_BN_CLICKED(ID_SMHELP, OnSmhelp)
	ON_BN_CLICKED(ID_SMEXIT, OnSMExit)
	ON_WM_CTLCOLOR()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// SMSplashPanel message handlers


void SMSplashPanel::OnRegisterBtn() 
{
   m_poApp->Register();
   if( m_poApp->IsRegistered() )
   {
      EndDialog(IDCANCEL);             // close the splash without changing the registration ID
   }
}

void SMSplashPanel::OnSmhelp() 
{
   m_poApp->WinHelp(ID_HOW_TO_BUY_HELP);  	
}

void SMSplashPanel::OnSMExit() 
{
   EndDialog(ID_SMEXIT);
}

HBRUSH SMSplashPanel::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hBrush = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
   CBrush oBrush;

   if( pWnd->m_hWnd == m_editSharewareMsg.m_hWnd )
   {
      pDC->SetTextColor(RGB(0, 0, 0));
   }
   if( pWnd->m_hWnd == m_editSharewareMsg2.m_hWnd )
   {
      pDC->SetTextColor(RGB(0, 204, 0));
      pDC->SetBkColor(RGB(255, 255, 255));
      oBrush.CreateSolidBrush(RGB(255, 255, 255));
      hBrush = (HBRUSH)oBrush;
   }
	return hBrush;
}

void SMSplashPanel::OnPaint() 
{
	CPaintDC oDC(this);     // device context for painting
	CPen     oPen;
   CPen *   pOldPen;
   CBrush * pOldBrush;
   CRect    oDlgRect;

   oPen.CreatePen(PS_SOLID, 2, RGB(0, 0, 0));
   pOldPen = oDC.SelectObject(&oPen);
   pOldBrush = (CBrush *)oDC.SelectStockObject(NULL_BRUSH);
   GetClientRect(&oDlgRect);
   oDlgRect.DeflateRect(10, 10, 10, 10);
	oDC.Rectangle(&oDlgRect);
   oDC.SelectObject(pOldBrush);
	oPen.DeleteObject();
	
	// Do not call CDialog::OnPaint() for painting messages
}
