//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMVisualBasic.cpp	[ SMVisualBasic.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implements Visual Basic metrics language class SMVisualBasic
//
//***************************************************************************

#include "stdafx.h"

#include "SMLanguage.h"
#include "SMVisualBasic.h"
#include "SourceMetrics.h"
#include "resource.h"
#include <VPVBClasses.hpp>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(SMVisualBasic, SMLanguage, 1)

// static members
CString        SMVisualBasic::VB_NAME = _T("Visual Basic");
CString        SMVisualBasic::VB_EXTENSIONS = _T("*.frm;*.cls;*.bas");

int            SMVisualBasic::m_iParserUsers = 0;
VBLexClass *   SMVisualBasic::m_poLexer = NULL;  // parsing objects are created and deleted by App object
SSLexTable *   SMVisualBasic::m_poLexTable = NULL;
SSYaccTable *  SMVisualBasic::m_poYaccTable = NULL;

// ***************************************************************************
// normal constructor
SMVisualBasic::SMVisualBasic() : SMLanguage()
{
   m_iCounts.SetSize(TOTAL_COUNTS);
}

// ***************************************************************************
// Class factory
SMLanguage *
SMVisualBasic::MakeNew()           // create a shallow copy
{
   SMVisualBasic *   poNewSMVisualBasic;

   poNewSMVisualBasic = new SMVisualBasic();
   for( int i = 0; i < m_iCounts.GetSize(); ++i)
   {
      poNewSMVisualBasic->m_iCounts[i] = m_iCounts[i];
   }
   poNewSMVisualBasic->m_sSubroutineName = m_sSubroutineName;
   poNewSMVisualBasic->m_iSubroutineStatements = m_iSubroutineStatements;
   return poNewSMVisualBasic;
}

// ***************************************************************************
// destructor
SMVisualBasic::~SMVisualBasic()
{
}

// ***************************************************************************
// initialize global parser objects
void
SMVisualBasic::InitParser()
{
   if( m_iParserUsers == 0 )
   {
      m_poLexTable = new SSLexTable((unsigned int)AfxGetResourceHandle(),
                                          MAKEINTRESOURCE(IDR_VB_LEXTABLE), _T("LEXTABLE"));
      m_poYaccTable = new SSYaccTable((unsigned int)AfxGetResourceHandle(),
                                          MAKEINTRESOURCE(IDR_VB_YACCTABLE), _T("YACCTABLE"));
      m_poLexer = new VBLexClass(*SMVisualBasic::m_poLexTable);
   }
   ++m_iParserUsers;
}

// ***************************************************************************
// shut down parser
void
SMVisualBasic::CloseParser()
{
   if( m_iParserUsers > 0 )
   {
      --m_iParserUsers;
   }
   if( m_iParserUsers == 0 )
   {
      delete m_poLexTable;
      m_poLexTable = NULL;
      delete m_poYaccTable;
      m_poYaccTable = NULL;
      delete m_poLexer;
      m_poLexer = NULL;
   }
}

// ***************************************************************************
// compute and accumulate metrics for one line or one file
bool
SMVisualBasic::Start(MetricsType /* type */)
{
   bool  fResult = true;

   if( m_poLexer == NULL )
   {
      fResult = false;
   }
   else
   {
      for( int i = 0; i < TOTAL_COUNTS; ++i )
      {
         m_iCounts[i] = 0;       // initialize the count accumulators
      }
      m_iSubroutineStatements = 0;
   }
   return fResult;
}

// ***************************************************************************
// compute and accumulate metrics for one source file
bool
SMVisualBasic::ProcessLines(const CString & sFileName)
{
   bool  fResult = true;         // assume success

   try
   {
      SSLexFileConsumer oSource(sFileName);
      m_poLexer->reset();
      m_poLexer->setConsumer(oSource);
      VBYaccClass(*m_poLexer, *m_poYaccTable, this).parse();
      SMLanguage::ReportErrors(sFileName);
   }
   catch(SSException oExcept)
   {
      m_sError.Format("%s [%s]", oExcept.text(), sFileName);
      fResult = false;
   }
   catch(...)
   {
      fResult = false;
   }
   return fResult;
}

// ***************************************************************************
// called by the parser's reduce() method when parser can't handle source code structure
void
SMVisualBasic::ParseError()
{
   if( AfxGetApp()->GetProfileInt(CSourceMetricsApp::OPTION_SETTINGS, CSourceMetricsApp::REPORT_ERRORS, 0) != 0 )
   {
      CString  sLineNumber;

      sLineNumber.Format("%d", m_iCounts[LINE_COUNT] + 1);
      m_asErrorLines.Add(sLineNumber);
   }
}

// ***************************************************************************
// called by the parser's reduce() method to bump a metrics count
void
SMVisualBasic::SetMethodData(const CString & sSubroutineName, int iStatementCount)
{
   if( m_iSubroutineStatements < iStatementCount )    // save only the biggest one
   {
      m_sSubroutineName = sSubroutineName;
      m_iSubroutineStatements = iStatementCount;
      m_iCounts[BIGGEST_SUBROUTINE_LINE] = m_iBeginSubroutineLine;
   }
}

bool
SMVisualBasic::ProcessFile(const SMLanguage & oFileMetric)
{
   const SMVisualBasic &
         oFileData = dynamic_cast<const SMVisualBasic &>(oFileMetric);
   bool  fResult = true;      // assume success

   for( int i = 0; i < TOTAL_COUNTS; ++i )
   {
      m_iCounts[i] += oFileData.m_iCounts[i];   // accumulate count totals
   }
   // not valid for checkpoints
   m_iCounts[DEEPEST_BLOCK_LINE] = UINT_MAX;
   m_iCounts[BIGGEST_SUBROUTINE_LINE] = UINT_MAX;

   // now select the biggest subroutine
   if( oFileData.m_iSubroutineStatements > m_iSubroutineStatements )
   {
      m_sSubroutineName = oFileData.m_sSubroutineName;
      m_iSubroutineStatements = oFileData.m_iSubroutineStatements;
   }
   return fResult;
}

// ***************************************************************************
// results access methods


// ***************************************************************************
//
void
SMVisualBasic::InitLabels()
{
   m_saLabels.SetSize(TOTAL_COLUMNS);

   // set some default labels and column alignment
   for( int i = 0; i < TOTAL_COLUMNS; ++i )
   {
      CString  sLabel;

      sLabel.LoadString(IDS_VB_DEFAULT_LABEL0 + i);
      m_saLabels[i] = sLabel;
   }
   // now override with user's labels, if any
   dynamic_cast<CSourceMetricsApp *>(AfxGetApp())->
      GetProfileStringList(CSourceMetricsApp::OPTION_SETTINGS + VB_NAME, CSourceMetricsApp::METRICS_LABELS_KEY,
                           m_saLabels, CString(_T(";,")), CString(_T("")));
}

// ***************************************************************************
//
CString
SMVisualBasic::GetMetricName(int index) const
{
   CString  sMetricName;

   if( (index >= 0) and (index < TOTAL_COLUMNS) )
   {
      sMetricName.LoadString(IDS_VB_METRIC0 + index);
   }
   return sMetricName;
}

// ***************************************************************************
//
CString
SMVisualBasic::GetValue(int index, bool format /* = true */) const       // access a metric's value
{
   CString  sValue;
   int      iValue = -1;            // start with invalid value
   float    flValue = 0.0;
   int      i;                      // loop index

   sValue.LoadString(IDS_UNDEFINED_VALUE);
   if( (index >= 0) and (m_iCounts.GetSize() > index) )
   {
      switch( index )
      {
      case LINES:
         iValue = m_iCounts[LINE_COUNT];
         break;

      case STATEMENTS:
         iValue = m_iCounts[STATEMENT_COUNT];
         break;

      case PERCENT_BRANCHES:
         if( m_iCounts[STATEMENT_COUNT] > 0 )
         {
            flValue = float(m_iCounts[BRANCH_COUNT] * 100) / float(m_iCounts[STATEMENT_COUNT]);
         }
         sValue.Format("%.1f", flValue);
         break;

      case PERCENT_COMMENTS:
         if( m_iCounts[LINE_COUNT] > 0 )
         {
            flValue = float(m_iCounts[COMMENT_LINES] * 100) / float(m_iCounts[LINE_COUNT]);
         }
         sValue.Format("%.1f", flValue);
         break;

      case SUBROUTINES:
         iValue = m_iCounts[SUBROUTINE_COUNT];
         break;

      case BIGGEST_SUBROUTINE:
         {
            if( m_iSubroutineStatements > 0 )
            {
               sValue.Format("%d", m_iSubroutineStatements);
            }
            else
            {
               sValue.Format("%d", 0);
            }
         }
         break;

      case MAXIMUM_BLOCK_DEPTH:
         {
            sValue.Format("%d +", BLOCK_DEPTH_COUNTS - 1);    // assume the worst

            for( i = 0; i < BLOCK_DEPTH_COUNTS; ++i )
            {
               if( m_iCounts[i + BLOCK_DEPTH_0] == 0 )
               {
                  // counts are base 0, depths are base 0
                  sValue.Format("%ld", max(i - 1, 0));
                  break;
               }
            }
         }
         break;

      case AVERAGE_BLOCK_DEPTH:
         {
            long  lSum = 0;
            long  lCounts = 0;

            for( i = 0; i < BLOCK_DEPTH_COUNTS; ++i )
            {
               long  lCount = m_iCounts[i + BLOCK_DEPTH_0];

               if( lCount > 0 )
               {
                  lSum += lCount * (i + 1);
                  lCounts += lCount;
               }
               else
               {
                  break;
               }
            }
            if( lCounts > 0 )                // if no blocks, avg = 0.0
            {
               flValue = (float(lSum) / float(lCounts)) - (float)1.0;
            }
            sValue.Format("%.2f", flValue);
         }
         break;
      }
   }
   if( iValue >= 0 )
   {
      if( format )
      {
         sValue = FormatCount(iValue);
      }
      else
      {
         sValue.Format("%d", iValue);
      }
   }
   return sValue;
}

// ***************************************************************************
//
bool
SMVisualBasic::GetDeepestBlockLine(CString & sLabel, int * piResult) const
{
   CString  sDeepestLabel;

   sDeepestLabel.LoadString(IDS_DEEPEST_BLOCK_LABEL);
   sLabel.Format("%s %s)", sDeepestLabel, GetValue(MAXIMUM_BLOCK_DEPTH));
   *piResult = m_iCounts[DEEPEST_BLOCK_LINE];
   return true;
}

// ***************************************************************************
//
bool
SMVisualBasic::GetBiggestModuleLine(CString & sLabel, int * piResult) const
{
   bool  fResult = false;

   sLabel.LoadString(IDS_BIGGEST_SUBROUTINE_LABEL);
   if( m_iSubroutineStatements > 0 )
   {
      *piResult = m_iCounts[BIGGEST_SUBROUTINE_LINE];
      fResult = true;
   }
   return fResult;
}

void
SMVisualBasic::GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                           bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const
{
   CString  sEntity;
   CString  sValue;
   CString  sEmpty;        // used as empty string only

   for( int i = 0; i < TOTAL_COLUMNS; ++i )
   {
      switch( i )
      {
      case MAXIMUM_BLOCK_DEPTH:
         if( oVersion.IsOKVersion(1, 0)                     // added line number of deepest block in V1.1
            and (m_iCounts[DEEPEST_BLOCK_LINE] != UINT_MAX) )
         {
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_DEEPEST_LINE_NUMBER);
               oEntities.Add(sEntity);
            }
            sValue.Format("%d", m_iCounts[DEEPEST_BLOCK_LINE]);
            oValues.Add(sValue);
         }
         else if( fIncludeExportValues )
         {
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_DEEPEST_LINE_NUMBER);
               oEntities.Add(sEntity);
            }
            oValues.Add(sEmpty);
         }
         break;

      case BIGGEST_SUBROUTINE:
         if( fIncludeEntities )
         {
            sEntity.LoadString(IDS_VB_BIGGEST_SUBROUTINE_NAME);
            oEntities.Add(sEntity);
         }
         if( m_iSubroutineStatements > 0 )
         {
            sValue = m_sSubroutineName;
         }
         else
         {
            sValue.LoadString(IDS_VB_NO_SUBROUTINE_NAME);   // set no subroutine message
         }
         oValues.Add(sValue);
         if( (m_iCounts[BIGGEST_SUBROUTINE_LINE] != UINT_MAX) and (m_iSubroutineStatements > 0) )
         {
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_BIGGEST_SUBROUTINE_LINE_NUMBER);
               oEntities.Add(sEntity);
            }
            sValue.Format("%d", m_iCounts[BIGGEST_SUBROUTINE_LINE]);
            oValues.Add(sValue);
         }
         else if( fIncludeExportValues )
         {
            if( fIncludeEntities )
            {
               sEntity.LoadString(IDS_BIGGEST_SUBROUTINE_LINE_NUMBER);
               oEntities.Add(sEntity);
            }
               oValues.Add(sEmpty);
         }
         break;
      }
      if( fIncludeEntities )
      {
         oEntities.Add(GetMetricName(i));
      }
      oValues.Add(GetValue(i));
   }
   // now fill the block depth array
   oBlockCounts.SetSize(BLOCK_DEPTH_COUNTS);
   for( int b = 0; b < BLOCK_DEPTH_COUNTS; ++b )
   {
      oBlockCounts[b] = m_iCounts[BLOCK_DEPTH_0 + b];
   }
}

// ***************************************************************************
//
void
SMVisualBasic::Serialize(CArchive & ar)
{
   // first do the MFC classes and C++ types
	if( ar.IsStoring() )
	{
		ar << m_sSubroutineName << m_iSubroutineStatements;
	}
	else
	{
		ar >> m_sSubroutineName >> m_iSubroutineStatements;
	}
   // then the metrics data objects
   SMLanguage::Serialize(ar);
   m_iCounts.Serialize(ar);

   return;
}
