#ifndef  SMVISUALBASIC_H
#define	SMVISUALBASIC_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SMVisualBasic.h	[ SMVisualBasic.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Definition of Visual Basic metrics language class SMVisualBasic
//
//***************************************************************************

class SSLexTable;
class SSYaccTable;
class VBLexClass;

class SMVisualBasic : public SMLanguage
{
   public:
      // define indexes into the metrics count accumulator array (leave one spare at the end)
      enum COUNT_TYPE { LINE_COUNT = 0, STATEMENT_COUNT = 1, COMMENT_LINES = 2, SUBROUTINE_COUNT = 3,
                        BRANCH_COUNT = 4, BLOCK_DEPTH_0 = 5, BLOCK_DEPTH_COUNTS = 10, DEEPEST_BLOCK_LINE = 15,
                        BIGGEST_SUBROUTINE_LINE = 16, TOTAL_COUNTS = 17 };

      // used only as a class factory (in SMProject and COptionsVBPage)
      explicit SMVisualBasic(Exemplar /* not used */) : SMLanguage() {;}
      virtual  SMLanguage * MakeNew();                         // create a shallow copy
      virtual  ~SMVisualBasic();

      // virtual overrides
	   virtual void Serialize(CArchive& ar);

      // property access methods
      virtual CString & Name() const                        { return VB_NAME; }
      virtual CString & DefaultExtensions() const           { return VB_EXTENSIONS; }

      // signature for common methods of sub-classes
      virtual bool   Start(MetricsType type);

      // accumulate metrics for one file
      virtual void   InitParser();
      virtual void   CloseParser();
      virtual bool   ProcessLines(const CString & sFileName);
      virtual void   ParseError();
      virtual void   BumpCount(int index)                   { ++m_iCounts[index]; }
      virtual void   SetMethodData(const CString & sClassName, int iStatementCount);
      virtual void   SetBiggestMethodLine()                 { m_iBeginSubroutineLine = m_iCounts[LINE_COUNT] + 1; }
      virtual void   SetDeepestBlockLine()                  { m_iCounts[DEEPEST_BLOCK_LINE]
                                                                        = m_iCounts[LINE_COUNT] + 1; }

      // accumulate across group of files
      virtual bool   ProcessFile(const SMLanguage & oFileMetric);
 
      // results access methods (indexes identify columns displayed to user: base 0)
      virtual short     Columns() const                     { return TOTAL_COLUMNS; }
      virtual CString   GetMetricName(int index) const;
      virtual void      InitLabels();
      virtual CString   GetValue(int index, bool format = true) const;
      virtual int       GetIntValue(int index) const        { return m_iCounts[index]; }
      virtual bool      GetDeepestBlockLine(CString & sLabel, int * piResult) const;
      virtual bool      GetBiggestModuleLine(CString & sLabel, int * piResult) const;
      virtual void      GetDetails(SMVersion & oVersion, CStringArray & oValues, CDWordArray & oBlockCounts,
                                    bool fIncludeExportValues, bool fIncludeEntities, CStringArray & oEntities) const;
      virtual int       BlockCounts()                                { return BLOCK_DEPTH_COUNTS; }

      static CString    VB_NAME;         // this is how this language is identified to one and all
      static CString    VB_EXTENSIONS;   // use until user changes options

   protected:
      SMVisualBasic();                       // no public default constructor: use MakeNew method

   private:
      // define results columns NOTE: MUST KEEP STRING RESOURCES IN SYNCH WITH THESE MAGIC NUMBERS!!!
      enum COLUMN_TYPE { LINES, STATEMENTS, PERCENT_COMMENTS, PERCENT_BRANCHES, SUBROUTINES, BIGGEST_SUBROUTINE,
                         MAXIMUM_BLOCK_DEPTH, AVERAGE_BLOCK_DEPTH, TOTAL_COLUMNS };

      // persistent metrics data
      CDWordArray m_iCounts;                 // raw counts
      CString     m_sSubroutineName;         // name of biggest subroutine
      int         m_iSubroutineStatements;   // statements in biggest subroutine

      // transient data members
      int         m_iBeginSubroutineLine;    // set for every subroutine
      int         m_iBiggestSubroutine;      // statements in biggest subroutine

      // static data members
      static int           m_iParserUsers;   // reference count for parsing objects
      static VBLexClass *  m_poLexer;
      static SSLexTable *  m_poLexTable;
      static SSYaccTable * m_poYaccTable;

      DECLARE_SERIAL(SMVisualBasic)
};
////////////////////////////////////////////////////////////////////////////////

#endif						// SMVISUALBASIC_H
