#ifndef  SourceMetricsDoc_H
#define	SourceMetricsDoc_H
//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       SourceMetricsDoc.h	[ SourceMetricsDoc.cpp ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Interface of the CSourceMetricsDoc class
//
//***************************************************************************

// map of window views open for each checkpoint
// key = (long)m_hWnd for each open view
// value = index of checkpoint displayed in the view
typedef CMap<long, long, long, long> ViewMap;

class CSourceMetricsDoc : public CDocument
{
//   friend class CProjectView;          // for access to metrics data

protected: // create from serialization only
   CSourceMetricsDoc();
	DECLARE_DYNCREATE(CSourceMetricsDoc)

// Attributes
public:
   SMProject * GetProject()               { return &m_oProject; }
   CSourceMetricsApp *
         GetApp() const                   { return dynamic_cast<CSourceMetricsApp *>(AfxGetApp()); }

// Operations
public:
   void  CheckpointIndex(int index)       { m_iCurrentCheckpoint = index; }
   int   CheckpointIndex() const          { return m_iCurrentCheckpoint; }
   void  SetProjectCaption(CString capt)  { m_sProjectCaption = capt; }
   CString &
         GetProjectCaption()              { return m_sProjectCaption; }
   void  SetView(long key);
   bool  IsView(int index) const;
   void  CloseViews(int index);
   void  UnsetView(long key);

   // Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSourceMetricsDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void DeleteContents();
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual     ~CSourceMetricsDoc();

   void  BeginPrinting(CDC *poDC, CPrintInfo* pInfo,
                       CListCtrl * poListCtrl, int iColumns, CString & sTitle);
   bool  PrintPage(CDC *poDC, CPrintInfo* pInfo);
   void  EndPrinting();
   void  ExportMetrics(CWnd * pParentWnd, int iCheckpointIndex = -2);

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
   // helper methods
   void  SetPrintOptions(CDC * pDC);
   void  AdjustOffset(DWORD & dwMargin, int iOffset);
   int   PageHeight() const                     { return m_iPageHeight; }
   int   PageWidth() const                      { return m_iPageWidth; }
   void  PrintPageHeader(CDC *poDC);
   void  PrintPageFooter(CDC *poDC);
   int   GetPrintStartCol();
   int   GetPrintLimitCol(int iFirstColumn);

   // members used only during printing
   enum CKPT_VIEW_HEADER_FOOTER { HEADER_LINES = 3, FOOTER_LINES = 2 };
   CListCtrl *    m_poListCtrl;        // reference to printing view's embedded list view control
   int            m_iPrintColumns;     // total number of print columns
   int            m_iListItems;
   int            m_iPageHeight;
   int            m_iPageWidth;
   int            m_iFooterHeight;
   CDWordArray    m_iMargins;
   CString        m_sFontName;
   int            m_iFontTwips;        // points * 20
   int            m_iFontWeight;
   BYTE           m_bFontItalic;       // 0 for false, 1 for true
   CFont *        m_poPrintFont;
   int            m_iCurrentPage;
   int            m_iLastPage;
   int            m_iPagesPerCkpt;
   int            m_iLineHeight;
   int            m_iLinesPerPage;
   CStringArray   m_saPageHeaders;
   CDWordArray    m_iaColumnOrder;     // raw order numbers as read from the header control
   CDWordArray    m_iaColumnWidths;
   CDWordArray    m_iaPageColumns;     // number of columns on each page

// document data - these objects contain the metrics for a project
private:
   SMProject   m_oProject;             // project data container
   int         m_iCurrentCheckpoint;   // transient index of last checkpoint selected by user (passed to new views)
   CString     m_sProjectCaption;      // caption of last project with selected checkpoint
   ViewMap     m_oViewMap;             // map of all open views with their checkpoint
                                       // (used to manage checkpoint Delete)
   CString     m_sProjectName;         // used to pass data to OnNew if command line has non-existent project file pathname
   CString     m_sProjectDirectory;

// Generated message map functions
protected:
	//{{AFX_MSG(CSourceMetricsDoc)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(CSourceMetricsDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_DISPATCH
	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
#endif						// SourceMetricsDoc_H
