/* Copyright (C) 1999, Vsoft Ltd. All Rights Reserved.
 *
 *   Usage Restrictions
 *
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes only.
 *
 *   Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Vsoft Ltd. disclaims any and
 * all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any 
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Comments & Questions:
 * mailto:noam@vsoft.com  . Please put "TestPoint" in the subject line.
 */

// DataSinkTpHandler.cpp 

#include "stdAfx.h"
#include "DataSinkTpHandler.h"
#include "IDataPath.h"
#include "CRegistry.h"



DataSinkTpHandler::DataSinkTpHandler(IUnknown* pUnknownOuter) 
  : CUnknown(pUnknownOuter)
{
	m_pSink = NULL;
}
DataSinkTpHandler::~DataSinkTpHandler()
{
	TRACE("~DataSinkTpHandler()");
	m_DumpFile.Close();
	m_pSink = NULL; // do NOT Release() it since we did not AddRef() it.
}

// Tell the handler who is the TP that uses the handler.
// Also, initialize the data needed for this test point
HRESULT DataSinkTpHandler::Set(const IN GUID& idOfTestPoint,
				const IN char* parameterLocation )
{
	// get parameters from the registry for this TP.
	// in this implementation the TP GUID is part of the path,
	// but other implementations are possible
	CRegistry_EX reg;
//	HRESULT hr;
	long r;
	DWORD stringSize = _MAX_PATH;
	char fileName[_MAX_PATH]; // dump file name

	//strcpy(m_parameterLocation, parameterLocation); // cache it for later use

	reg.StopParamLogger();
	r = reg.SetRoot(parameterLocation);
	if(ERROR_SUCCESS != r)
		return HRESULT_FROM_WIN32(r);// bad path to parameters.
 

	// get the file name for this TP
	// TODO: add length error checking
	r = reg.getValueREG_SZ("File",reinterpret_cast<unsigned char*>(fileName), 
		&stringSize);
	if(ERROR_SUCCESS != r)
		return HRESULT_FROM_WIN32(r);

	// If this handler uses IDataSinkWithReset, we have to read more info from the registry.
	// We try to read this data anyway, even if it will not be used.
	ULONG UseNewFileAfterReset = 0; // default to false
	r = reg.getValueDW("UseNewFileAfterReset",&UseNewFileAfterReset);
	if(ERROR_SUCCESS != r)
	{ // simply ignore it
	}

	m_pSink = & m_DumpFile; // left as an exrecise for the reader

	// The full implementation creates a file name with the PID, TID and
	// other good stuff.
	return m_DumpFile.CreateFile(fileName,GENERIC_WRITE,0,
		CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL);

} 

// -------------------------------- COM support --------------------------------


/////////////////////////////////////////////////////////// 
// 
// Creation function used by CFactory 
// 
HRESULT DataSinkTpHandler::CreateInstance(IUnknown* pUnknownOuter, 
                           CUnknown** ppNewComponent) 
{ 
 *ppNewComponent = new DataSinkTpHandler(pUnknownOuter) ; 
 return S_OK ; 
} 


///////////////////
HRESULT  __stdcall DataSinkTpHandler::NondelegatingQueryInterface(const IID& iid, void** ppv) 
{ 
	if (iid == IID_ITestPoint) 
	{ 
		return FinishQI(static_cast<ITestPoint*>(this), ppv) ; 
	} 
	else if( iid == IID_IDataSink2 )
	{
		return FinishQI(static_cast<IDataSink2*>(this), ppv) ; 
	}
	else 
	{ 
		return CUnknown::NondelegatingQueryInterface(iid, ppv) ; 
	} 
} 
