//** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL ***** CONFIDENTIAL **
//   Campwood Software, Charlotte, Vermont 05445 USA
//   Copyright  1999, All Rights Reserved.
//***************************************************************************
//
// FILE:       ViewSourceDlg.cpp	[ ViewSourceDlg.h ]
//
// SUBSYSTEM:  Source Monitor Applicaton
//
// AUTHOR:     JF Wanner
//
// OVERVIEW:   Implementation of the CViewSourceDlg class
//
//***************************************************************************

#include "stdafx.h"
#include "SourceMetrics.h"
#include "ViewSourceDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CViewSourceDlg dialog

BEGIN_MESSAGE_MAP(CViewSourceDlg, CDialog)
	//{{AFX_MSG_MAP(CViewSourceDlg)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_BIGGEST_METHOD_BUTTON, OnBiggestMethodButton)
	ON_BN_CLICKED(IDC_DEEPEST_BLOCK_BUTTON, OnDeepestBlockButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CViewSourceDlg::CViewSourceDlg(CWnd* pParent /*=NULL*/) : CDialog(CViewSourceDlg::IDD, pParent),
      m_fEnableDeepestButton(false), m_fEnableBiggestButton(false), m_fInitialized(false)
{
	//{{AFX_DATA_INIT(CViewSourceDlg)
	//}}AFX_DATA_INIT
}

void CViewSourceDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(CViewSourceDlg)
	DDX_Control(pDX, IDOK, m_btnClose);
	DDX_Control(pDX, IDC_DEEPEST_BLOCK_BUTTON, m_btnDeepestBlock);
	DDX_Control(pDX, IDC_BIGGEST_METHOD_BUTTON, m_btnBiggestMethod);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_RICHEDIT_SOURCE, m_richEditSource);

   // hide buttons if language doesn't support them
   if( not pDX->m_bSaveAndValidate )
   {
      EDITSTREAM  oEditStream;
      CString     sCaption;
      CString     sHeader;                               // use dialog caption as header
      int         iFontPoints = 11;                      // use 11 points for now (TODO: make this a user option)
      int         iTabOffset = ((iFontPoints * 3) / 5) * 20 * 3;  // and tab over 3 characters worth
      PARAFORMAT  oParaFormat;
      CHARFORMAT  oCharFormat;

      m_richEditSource.HideSelection(TRUE, FALSE);
      oEditStream.dwCookie = (unsigned long)m_sSourceCode.GetBuffer(0); // set up the source text to be consumed
      oEditStream.pfnCallback = EditControlStreamInCallback;
      m_richEditSource.StreamIn(SF_TEXT, oEditStream);   // fill edit window with source text
      m_sSourceCode.ReleaseBuffer(0);                    // leave the string empty--we're done with it
      m_richEditSource.SetSel(0, -1);

      // select a font
      oCharFormat.cbSize = sizeof(oCharFormat);
      _tcscpy(oCharFormat.szFaceName, _T("Courier New"));
      oCharFormat.yHeight = iFontPoints * 20;
      oCharFormat.dwMask = CFM_FACE bitor CFM_SIZE;
      m_richEditSource.SetSelectionCharFormat(oCharFormat);

      // set up the tabs
      m_richEditSource.SetSel(0, -1);
      oParaFormat.cbSize = sizeof(oParaFormat);
      oParaFormat.cTabCount = MAX_TAB_STOPS;
      for( int i = 0; i < MAX_TAB_STOPS; ++i )
      {
         // place a tab stop every ? characters (twips?) (TODO: make a user option)
         oParaFormat.rgxTabs[i] = (i + 1) * iTabOffset;
      }
      oParaFormat.dwMask = PFM_TABSTOPS;
      m_richEditSource.SetParaFormat(oParaFormat);
      m_richEditSource.SetSel(0, 0);
      m_richEditSource.LineScroll(-m_richEditSource.GetFirstVisibleLine(), 0);
      m_richEditSource.HideSelection(FALSE, FALSE);
      m_fInitialized = true;                             // enable control movement when user re-sizes dialog
      if( m_fEnableDeepestButton )
      {
         m_btnDeepestBlock.SetWindowText(m_sDeepestLabel);
      }
      else
      {
         m_btnDeepestBlock.ShowWindow(SW_HIDE);
      }
      if( m_fEnableBiggestButton )
      {
         m_btnBiggestMethod.SetWindowText(m_sBiggestLabel);
      }
      else
      {
         m_btnBiggestMethod.ShowWindow(SW_HIDE);
      }
      // now display the file name in the caption
      GetWindowText(sHeader);
      sCaption.Format("%s '%s'", sHeader, m_sFileName);
      SetWindowText(sCaption);
   }
}

// use the pathname supplied to open the file for read only and pass the text into the edit control
bool CViewSourceDlg::SetFileName(CString sFileName)
{
   CFile    oSourceFile;
   DWORD    dwFileLength;
   bool     fResult = true;      // assume success

   try
   {
      oSourceFile.Open(sFileName, CFile::modeRead bitor CFile::shareDenyNone);
      dwFileLength = oSourceFile.GetLength();
      oSourceFile.Read(m_sSourceCode.GetBuffer(dwFileLength + 1), dwFileLength);
      m_sSourceCode.ReleaseBuffer();
      oSourceFile.Close();
      m_sFileName = sFileName;
   }
   catch(CFileException exception)
   {
      CString  sCaption;
      CString  sErrorMessage;

      sCaption.LoadString(IDS_PROJ_FILE_ERROR_CAPTION);
      sErrorMessage.LoadString(IDS_NEWCKPT_FILEOPEN_ERROR);
      MessageBox(sErrorMessage, sCaption, MB_OK bitor MB_ICONERROR);
   }
   return fResult;
}

// **************************************************************************************
DWORD CALLBACK
CViewSourceDlg::EditControlStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb)
{
   LPTSTR   pcSourceCode = (LPTSTR)dwCookie;
   long     lSourceLength = strlen(pcSourceCode);
   long     lStreamCount = min(lSourceLength, cb);

   if( lStreamCount > 0 )
   {
      _tcsncpy((char *)pbBuff, pcSourceCode, lStreamCount);
      if( lSourceLength >= lStreamCount )     // are we done?
      {
         // no: must move remaining text over
         _tcscpy(pcSourceCode, pcSourceCode + lStreamCount);   // wipe out streamed text
      }
   }
   *pcb = lStreamCount;

   return 0;
}

// specify a label and line number for the deepest block nesting in the file
void CViewSourceDlg::EnableDeepestButton(CString sLabel, int iLineNumber)
{
   m_sDeepestLabel = sLabel;
   m_iDeepestBlockLineNumber = iLineNumber;
   m_fEnableDeepestButton = true;
}

// specify a label and the name of the biggest module.
// Search for and highlight this name when button is clicked.
void CViewSourceDlg::EnableBiggestModule(CString sLabel, int iLineNumber)
{
   m_sBiggestLabel = sLabel;
   m_iBiggestModuleLineNumber = iLineNumber;
   m_fEnableBiggestButton = true;
}

/////////////////////////////////////////////////////////////////////////////
// Helper methods
void
CViewSourceDlg::HighlightLine(int iLineNumber)
{
   int   iLineIndex = m_richEditSource.LineIndex(iLineNumber);
   int   iScrollLines;

   if( iLineIndex >= 0 )
   {
      iScrollLines = iLineNumber - m_richEditSource.GetFirstVisibleLine() - 3;
      m_richEditSource.LineScroll(iScrollLines, 0);

      // highlight the selection with the carrot at the beginning of the line
      m_richEditSource.SetSel(m_richEditSource.LineIndex(iLineNumber + 1), iLineIndex);
   }
   ::SetFocus(m_richEditSource.m_hWnd);
}

/////////////////////////////////////////////////////////////////////////////
// CViewSourceDlg message handlers

void CViewSourceDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

   // bomb out until DoDataExchange has initialized the buttons
   if( not m_fInitialized ) return;

	switch( nType )
   {
   // if not one of these, don't need to move or size the controls
   case SIZE_MAXIMIZED:
   case SIZE_RESTORED:
      {
         CRect oCloseBtnRect;
         CRect oMarginRect;      // button with correct top and right margins
         CRect oEditRect;        // edit control location
         
         // put close button in the upper right corner (same distance from border as Deepest button)
         m_btnClose.GetWindowRect(&oCloseBtnRect);
         ScreenToClient(oCloseBtnRect);
         m_btnDeepestBlock.GetWindowRect(&oMarginRect);
         ScreenToClient(oMarginRect);
         m_btnClose.MoveWindow(cx - oCloseBtnRect.Width() - oMarginRect.left,
                                 oMarginRect.top, oCloseBtnRect.Width(), oCloseBtnRect.Height());

         // fill the bottom with the edit control (hold top location, use right margin at the bottom
         m_richEditSource.GetWindowRect(&oEditRect);
         ScreenToClient(oEditRect);
         m_richEditSource.MoveWindow(oEditRect.left, oEditRect.top,
                                 cx - (oMarginRect.left * 2), cy - oEditRect.top - oEditRect.left);
      }
   }
}

void CViewSourceDlg::OnBiggestMethodButton() 
{
   if( m_iBiggestModuleLineNumber >= 1 )
   {
      HighlightLine(m_iBiggestModuleLineNumber - 1);  // Edit control lines are base 0
   }
}

void CViewSourceDlg::OnDeepestBlockButton() 
{
   if( m_iDeepestBlockLineNumber >= 1 )
   {
      HighlightLine(m_iDeepestBlockLineNumber - 1);   // Edit control lines are base 0
   }
}
