/*---------------------------------------------------------------------------

					Copyright (c) 1994-1999 SandStone Technology Inc.
								  All rights reserved

		THIS PROGRAM CONSISTS OF TRADE SECRETS THAT ARE  THE PROPERTY
		OF SANDSTONE SOFTWARE INC..  THE CONTENTS  MAY NOT BE USED OR
		DISCLOSED WITHOUT THE EXPRESS WRITTEN PERMISSION OF THE OWNER.

----------------------------------------------------------------------------*/
#include "stdafx.h"

#include<sslexev.hpp>
#if !defined( SS16BIT)
#  define SSLexBufferMax   0xffffffff
#else
#  define SSLexBufferMax   0x0000ff00
#endif
#if defined( __BORLANDC__)
#     pragma warn -ccc
#     pragma warn -rch
#endif

	SSLexConsumerEvent::SSLexConsumerEvent( SSUnsigned32 qEvents,
		SSLexConsumerType qType) :	oState( lexOk), oEvents( qEvents),
		SSLexConsumer( qType)
	{
	}

	SSLexFileConsumerEvent::SSLexFileConsumerEvent( const char* qpszFile,
		SSUnsigned32 qEvents, SSLexConsumerType qType, SSUnsigned32 qulBuff,
		SSUnsigned32 qulInc) : SSLexConsumerEvent( qEvents, qType), opFile( 0)
	{
		oulBuff = qulBuff;
		oulInc = qulInc;

		char* zpszMode;
		switch ( type())
		{
			case binary:
				zpszMode = SSLexFileModeBinary;
				break;
			case unicode:
				zpszMode = SSLexFileModeUnicode;
				break;
			case character:
				zpszMode = SSLexFileModeCharacter;
				break;
		}

		opFile = fopen( qpszFile, zpszMode);
		if ( !opFile)
		{
			throwException( SSExceptionLexFileOpen, SSLexMsgFileOpen,
				qpszFile, errno);
			return;
		}

		fseek( opFile, 0L, SEEK_END);
		SSUnsigned32 zulBuff = ftell( opFile);
		oulFile = zulBuff;
		fseek( opFile, 0L, SEEK_SET);
		if ( oulBuff == SSLexConsumerAll)
			oulBuff = zulBuff;
		else if ( oulBuff > zulBuff)
			oulBuff = zulBuff;

		if ( !oulBuff)
		{
			throwException( SSExceptionLexFileLength, SSLexMsgFileLength, qpszFile);
			return;
		}

		if ( oulBuff > SSLexBufferMax)
		{
			if ( !oulInc)
				oulInc = oulBuff - SSLexBufferMax;
			oulBuff = SSLexBufferMax;
		}

		if ( oulInc > SSLexBufferMax)
			oulInc = SSLexBufferMax;

		opchBuff = new char[ oulBuff];
		oulLength = fread( opchBuff, 1, oulBuff, opFile);
		oulFile -= oulLength;
		if ( !oulFile || ( oulLength < oulBuff)) close();
	}

	SSUnsigned32 SSLexFileConsumerEvent::nextBuffer( void)
	{
		if ( !opFile)
			return 0;

		SSUnsigned32 zulRead, zulFill;
		if ( shiftBuffer( zulRead, zulFill) && expandBuffer( zulRead, zulFill))
		{
			char zchTest;
			if ( fread( &zchTest, 1, 1, opFile))
				throwException( SSExceptionLexLexemeLength,
					SSLexMsgLexemeLength, oulLength);
			else
				close();
			return 0;
		}

		SSUnsigned32 zulLength = fread( opchBuff + zulRead, 1, zulFill, opFile);
		oulFile -= zulLength;
		if ( !oulFile || !zulLength || ( zulLength < zulFill)) close();
		return zulLength;
	}

	SSLexFileConsumerEvent::~SSLexFileConsumerEvent( void)
	{
		close();
	}

	SSLexStringConsumerEvent::SSLexStringConsumerEvent( const char* qpString,
		SSUnsigned32 qulString, SSUnsigned32 qEvents, SSLexConsumerType qType,
		SSUnsigned32 qulBuff, SSUnsigned32 qulInc) :
		SSLexConsumerEvent( qEvents, qType)
	{
		oulInc = qulInc;
		if ( qpString)
		{
			if ( qulBuff)
			{
				oulBuff = qulBuff;
				oulLength = qulString ? qulString : strlen( qpString);
			}
			else
				oulBuff = oulLength = qulString ? qulString : strlen( qpString);
		}
		else
		{
			oulBuff = qulBuff ? qulBuff : SSLexConsumerEventBuff;
			oulLength = 0;
		}

		if ( oulBuff > SSLexBufferMax)
			oulBuff = SSLexBufferMax;
		if ( oulLength > oulBuff)
			oulLength = oulBuff;
		opchBuff = new char[ oulBuff];
		if ( qpString && oulLength)
			memcpy( opchBuff, qpString, oulLength);
	}

	SSUnsigned32 SSLexStringConsumerEvent::nextBuffer( void)
	{
		return 0;
	}

	SSLexConsumerEvent::SSLexConsumerEventType SSLexConsumerEvent::operator++( int qDummy)
	{
		qDummy;//Unreferenced
		switch ( oState)
		{
			case lexOk:
				if ( oulIndex >= oulLength)
				{
					if ( oEvents & lexMore)
						oState = lexMore;
					else
					{
						SSUnsigned32 zulLength = nextBuffer();
						oEndOfData = !zulLength;
						if ( oEndOfData)
							oState = lexDone;
						else
						{
							oulLength += zulLength;
							processUnit();
						}
					}
				}
				else
					processUnit();
				break;

			case lexMore:
				if ( oulIndex >= oulLength)
					oState = lexDone;
				else
				{
					processUnit();
					oState = lexOk;
				}
				break;

			default:
				break;
		}

		return oState;
	}

	void SSLexConsumerEvent::processUnit( void)
	{
		oulCurrent = oulBof;
		if ( oulCurrent)
		{
			oulBof = 0;
			return;
		}

		switch ( oType)
		{
			case binary:
				singleByte();
				break;
			case character:
				if ( dbcs().isLeadByte( opchBuff[ oulIndex]))
					doubleByte();
				else
					singleByte();
				break;
			case unicode:
				unicodeByte();
				break;
		}

		oulConsumed++;
		if ( oulCurrent == '\n')
			oulScanLine++, oulScanOffset = 1;
		else
			oulScanOffset++;
	}

	SSLexEvent::SSLexEventType SSLexEvent::operator++( int qDummy)
	{
		qDummy;//Unreferenced
		return iterate();
	}

	SSLexEvent::SSLexEventType SSLexEvent::iterate( void)
	{
		if ( !opTable || !opConsumer)
			throwException( SSExceptionLexMissingParm, SSLexMsgMissingParm);

		consumer()->adjustMark( oFinal);
		consumer()->adjustMark( oContext, SSTrue);

		do
		{
			switch ( oReturn)
			{
				case lexMore:
					processLexeme();
					break;

				case lexGotMore:
					if ( iterateLexeme() != lexMore)
						processLexeme();
					break;

				case lexDone:
					break;

				default:
					resetLexeme();
					if ( iterateLexeme() != lexMore)
						processLexeme();
					break;
			}
		}
		while ( !( oReturn & oEvents));

		return oReturn;
	}

	void SSLexEvent::resetLexeme( void)
	{
		oulState = 0;
		oConsume = SSFalse;
		oContextFinal.reset();
		oPrevFinal = iterator().lookupFinal( oulState);
	}

	SSLexEvent::SSLexEventType SSLexEvent::iterateLexeme( void)
	{
		while (( oReturn = (( SSLexEventType) ( *consumer())++)) == lexOk)
		{
			oConsume = SSTrue;
			consumer()->adjustMark( oFinal);
			consumer()->adjustMark( oContext, SSTrue);
			if ( !iterator().lookupNext( oulState, consumer()->current()))
				break;
			if ( iterator().lookupFinal( oulState).isFinal())
			{
				oFinal = consumer()->mark();
				oPrevFinal = iterator().lookupFinal( oulState);
			}
			if ( iterator().lookupFinal( oulState).isContextStart())
				oContext = consumer()->mark();
		}
		return oReturn;
	}

	SSLexEvent::SSLexEventType SSLexEvent::processLexeme( void)
	{
		if ( !oConsume)
			oReturn = lexDone;
		else
		{
			if ( oPrevFinal.isContextEnd())
				consumer()->flushEndOfLine( oFinal);
			if ( oPrevFinal.isIgnore())
			{
//				consumer()->flushLexeme( oFinal);
				if ( oPrevFinal.isPop() && oPrevFinal.isPush())
					gotoExpressionList( *table()->expressionList( oPrevFinal.push()));
				else if ( oPrevFinal.isPop())
					popExpressionList();
				else if ( oPrevFinal.isPush())
					pushExpressionList( *table()->expressionList( oPrevFinal.push()));
				if ( oPrevFinal.isStartOfLine())
					if ( consumer()->line() != 1)
						if ( consumer()->offset() != 1)
							consumer()->flushStartOfLine( oFinal);
				opLexeme = ignore( *consumer(), oPrevFinal.token(), oFinal);
				if ( !opLexeme)
					oReturn = lexIgnore;
				else
				{
					oReturn = lexOk;
		            oulLexemesValid++;
			        if ( oPrevFinal.isParseToken())
				    	opLexeme->setParseToken( oPrevFinal.parseToken());
					if ( oPrevFinal.isKeyword())
            			findKeyword( opLexeme, oPrevFinal.keyword(), consumer()->type());
				}
			}
			else if ( !oPrevFinal.isFinal())
			{
				if ( oEvents & lexError)
				{
					opLexeme = consumer()->lexemeToCurrent();
					oReturn = lexError;
				}
				else
				{
					opLexeme = error( *consumer());
					if ( opLexeme)
						oReturn = lexOk;
					else
					{
						consumer()->flushLexeme();
						oReturn = lexIgnore;
					}
				}
			}
			else
			{
				if ( oPrevFinal.isPop() && oPrevFinal.isPush())
					gotoExpressionList( *table()->expressionList( oPrevFinal.push()));
				else if ( oPrevFinal.isPop())
					popExpressionList();
				else if ( oPrevFinal.isPush())
					pushExpressionList( *table()->expressionList( oPrevFinal.push()));
				if ( oPrevFinal.isStartOfLine())
					if ( consumer()->line() != 1)
						if ( consumer()->offset() != 1)
							consumer()->flushStartOfLine( oFinal);
				opLexeme = complete( *consumer(), oPrevFinal.token(), oFinal);
				if ( !opLexeme)
					oReturn = lexIgnore;
				else
				{
					oReturn = lexOk;
		            oulLexemesValid++;
			        if ( oPrevFinal.isParseToken())
				    	opLexeme->setParseToken( oPrevFinal.parseToken());
					if ( oPrevFinal.isKeyword())
            			findKeyword( opLexeme, oPrevFinal.keyword(), consumer()->type());
				}
			}
		}

		return oReturn;
	}

	void SSLexEvent::setConsumer( const char* qpszFile)
	{
		opConsumer = new SSLexFileConsumerEvent( qpszFile);
		if ( setException( opConsumer->getException())) return;
		oOwnConsumer = SSTrue;
	}

