/*---------------------------------------------------------------------------

					Copyright (c) 1994-1997 SandStone Technology Inc.
								  All rights reserved

		THIS PROGRAM CONSISTS OF TRADE SECRETS THAT ARE  THE PROPERTY
		OF SANDSTONE SOFTWARE INC..  THE CONTENTS  MAY NOT BE USED OR
		DISCLOSED WITHOUT THE EXPRESS WRITTEN PERMISSION OF THE OWNER.

----------------------------------------------------------------------------*/
#if !defined( SSLEXEVHPP)
#  define SSLEXEVHPP
#  include<sslex.hpp>
#  include<sslextb.hpp>

#  define SSLexConsumerEventInc  4096
#  define SSLexConsumerEventBuff 4096

	class SSClass SSLexConsumerEvent : public SSLexConsumer
		{
		public:

			enum SSLexConsumerEventType
				{
				lexOk = 0x0001,
				lexMore = 0x0002,
				lexDone = 0x0004,
				lexLast = 0x0008
				};

			SSConstr               SSLexConsumerEvent( SSUnsigned32 = lexMore,
											  SSLexConsumerType = character);

			SSLexConsumerEventType operator++( int);
			void                   processUnit( void);
			SSInline void			  setEvents( SSUnsigned32);

		protected:
			SSLexConsumerEventType oState;
			SSUnsigned32			  oEvents;
		};

	SSInline void SSLexConsumerEvent::setEvents( SSUnsigned32 qEvents)
		{
		oEvents = qEvents;
		}

	class SSClass SSLexFileConsumerEvent : public SSLexConsumerEvent
		{
		public:

			SSConstr       SSLexFileConsumerEvent( const char*,
									SSUnsigned32 = lexMore,
									SSLexConsumerType = character,
									SSUnsigned32 = SSLexConsumerAll,
									SSUnsigned32 = SSLexConsumerEventInc);

			SSInline FILE* file( void);
			SSInline void  close( void);
			SSUnsigned32   nextBuffer( void);

			SSDestr			SSLexFileConsumerEvent( void);

		protected:
			FILE*				opFile;
			SSUnsigned32   oulFile;
		};

	SSInline FILE* SSLexFileConsumerEvent::file( void)
		{
		return opFile;
		}

	SSInline void SSLexFileConsumerEvent::close( void)
		{
		if ( opFile) fclose( opFile);
		opFile = 0;
		}

	class SSClass SSLexStringConsumerEvent : public SSLexConsumerEvent
		{
		public:
			SSConstr              SSLexStringConsumerEvent( const char*,
											 SSUnsigned32 = 0,
											 SSUnsigned32 = lexMore,
											 SSLexConsumerType = character,
											 SSUnsigned32 = SSLexConsumerEventBuff,
											 SSUnsigned32 = SSLexConsumerEventInc);

			SSUnsigned32          nextBuffer( void);
		};

	class SSClass SSLexEvent : public SSLex
		{
		public:

			enum SSLexEventType
				{
				lexOk = SSLexConsumerEvent::lexOk,
				lexMore = SSLexConsumerEvent::lexMore,
				lexDone = SSLexConsumerEvent::lexDone,
				lexError = SSLexConsumerEvent::lexLast << 1,
				lexGotMore = SSLexConsumerEvent::lexLast << 2,
				lexIgnore = SSLexConsumerEvent::lexLast << 3,
				lexLast = SSLexConsumerEvent::lexLast << 4
				};

			SSInline SSConstr   SSLexEvent( const char*, const char*,
										  SSUnsigned32 = lexMore);
			SSInline SSConstr   SSLexEvent( SSLexConsumerEvent&, SSLexTable&,
										  SSUnsigned32 = lexMore);

			SSInline SSLexLexeme*        lexeme( void);
			SSLexEventType               iterate( void);
			SSInline SSLexConsumerEvent* consumer( void);
			SSLexEventType               operator++( int);
			void                         resetLexeme( void);
			SSLexEventType               processLexeme( void);
			SSLexEventType      			  iterateLexeme( void);
			SSInline void       			  setEvents( SSUnsigned32);
			void 				              setConsumer( const char*);
			SSInline void                addBuffer( char*, SSUnsigned32);

		protected:
			SSLexMark           oFinal;
			SSUnsigned32        oEvents;
			SSLexEventType      oReturn;
			SSLexMark           oContext;
			SSBooleanValue      oConsume;
			SSLexLexeme*        opLexeme;
			SSLexFinalState     oPrevFinal;
			SSLexFinalState     oContextFinal;
		};

	SSInline SSLexEvent::SSLexEvent( SSLexConsumerEvent& qConsumer,
		SSLexTable& qTable, SSUnsigned32 qEvents) : SSLex( qConsumer, qTable),
		oReturn( lexOk), oEvents( qEvents)
		{
		setEvents( qEvents);
		}

	SSInline SSLexEvent::SSLexEvent( const char* qpszFile,
		const char* qpszTable, SSUnsigned32 qEvents) : SSLex( 0),
		oReturn( lexOk), oEvents( qEvents)
		{
		setTable( qpszTable);
		setConsumer( qpszFile);
		setEvents( qEvents);
		}

	SSInline void SSLexEvent::setEvents( SSUnsigned32 qEvents)
		{
		oEvents = qEvents;
		oEvents &= ~lexIgnore;
		oEvents |= lexDone | lexOk;
		consumer()->setEvents( qEvents);
		}

	SSInline SSLexConsumerEvent* SSLexEvent::consumer( void)
		{
		return ( SSLexConsumerEvent*) opConsumer;
		}

	SSInline SSLexLexeme* SSLexEvent::lexeme( void)
		{
		return opLexeme;
		}

	SSInline void SSLexEvent::addBuffer( char* qpData, SSUnsigned32 qulData)
		{
		if ( qpData && qulData)
			{
			consumer()->addBuffer( qpData, qulData);
			if ( oReturn == lexMore) oReturn = lexGotMore;
			}
		}

#endif
