/*---------------------------------------------------------------------------

               Copyright (c) 1994 - 1999 SandStone Technology Inc.
                          All rights reserved

      THIS PROGRAM CONSISTS OF TRADE SECRETS THAT ARE  THE PROPERTY
      OF SANDSTONE SOFTWARE INC..  THE CONTENTS  MAY NOT BE USED OR
      DISCLOSED WITHOUT THE EXPRESS WRITTEN PERMISSION OF THE OWNER.

----------------------------------------------------------------------------*/
#include "stdafx.h"

#include<ssyacc.hpp>
#include<sslifo.hpp>
#include<sslextb.hpp>
#include<sssetarr.hpp>
#include<ssyacctb.hpp>
#if defined( _Windows) || defined( __WIN32__) || defined( WIN32) || defined( _WINDOWS)
#  include<windows.h>
#endif
#if defined( __BORLANDC__)
#     pragma warn -aus
#endif

#  define SSYaccStackSize        64
#  define SSYaccStackInc         64
#  define SSYaccSetArraySize     64
#  define SSYaccSetArrayInc      64

#  if ( _MSC_VER == 800) || defined( SSNOTEMPLATE)
#     include<sssetulo.hpp>
#     include<sslifoyc.hpp>
#  else
		typedef SSLifoArray< SSYaccStackElementRef> SSYaccStack;
		typedef SSSetArray< SSUnsigned32>           SSSetArrayOfUnsigned32;
#  endif

	typedef SSLexIterator*							  SSLexIteratorPtr;
	typedef SSYaccPairTable SSHuge *            SSYaccPairTableHuge;

#if defined( SSAUX)
#  include<ssauxdef.cpp>
#  include<ssauxfun.cpp>
#else
	void SSAux( void) {}
#endif

	SSYacc::SSYacc( const char* qpszTable) : opLex( 0), oulState( 0),
		oEndOfInput( SSFalse), oError( SSFalse), oulError( 0), opTable( 0),
		oAbort( SSFalse), oOwner( SSFalse), opLexemeCache( 0)
		{
		if ( qpszTable)
			setTable( qpszTable);
		setEof();
		setStack();
		setCache();
		}

	SSYacc::SSYacc( SSLex& qLex, const char* qpszTable) : opLex( &qLex),
		oEndOfInput( SSFalse), oulState( 0), oError( SSFalse), oulError( 0),
		opTable( 0), oAbort( SSFalse), oOwner( SSFalse), opLexemeCache( 0)
		{
		setTable( qpszTable);
		setEof();
		setStack();
		setCache();
		}

	SSYacc::SSYacc( SSLex& qLex, SSYaccTable& qTable) : oulState( 0),
		oEndOfInput( SSFalse), oError( SSFalse), oulError( 0), oAbort( SSFalse),
		oOwner( SSFalse), opLex( 0), opTable( 0), opLexemeCache( 0)
		{
		setEof();
		setStack();
		setCache();
		setLex( qLex);
		setTable( qTable);
		}

	void SSYacc::setEof( void)
		{
		SSUnsigned32 zulLen = strlen( SSYaccEofLexeme);
		orEndLexeme = new( zulLen) SSLexLexeme( zulLen, SSYaccEofToken,
			SSYaccEofLexeme, SSYaccEofToken, SSYaccEofToken);
		orEndLexeme->refDec();
		}

	void SSYacc::setTable( const char* qpszTable)
		{
		opTable = new SSYaccTable( qpszTable);
		if ( setException( opTable->getException())) return;
		opTab = opTable->opTable;
		oOwner = SSTrue;
		}

	void SSYacc::setStack( void)
		{
		opStack = new SSYaccStack( SSYaccStackSize, SSYaccStackInc);
		oElement = stackElement();
		push();
		}

	void SSYacc::setCache( void)
		{
		opLexemeCache = new SSYaccLexemeCache;
		}

	SSYaccTable::SSYaccTable( void* qpTable, SSUnsigned32 qulLen,
		const char* qpszTable)
		{
		oulLen = qulLen;
		opTable = mallocSSSpecial( qulLen);
		copySSResource(( SSCharHuge) opTable, ( SSCharHuge) qpTable, qulLen);
		format( qulLen, qpszTable);
		SSAux();
		}

#	if defined( _Windows) || defined( __WIN32__) || defined( WIN32) || defined( _WINDOWS)
	SSYaccTable::SSYaccTable( unsigned int qhInst, const char* qpszName,
		const char* qpszType) : opTable( 0), oapIterator( 0)
		{
		HGLOBAL zhRes;
		SSCharHuge zpRes = 0;
		HINSTANCE zhInst = ( HINSTANCE) qhInst;
		HRSRC zRsc = FindResourceA( zhInst, qpszName, qpszType);
		if ( zRsc)
		{
			zhRes = LoadResource( zhInst, zRsc);
			if ( zhRes)
				zpRes = ( SSCharHuge) LockResource( zhRes);
		}

		if ( !zpRes)
			{
			throwException( SSExceptionYaccResource, SSYaccMsgResource, qpszName);
			return;
			}

		unsigned long zulLen = SizeofResource( zhInst, zRsc);
		opTable = mallocSSSpecial( zulLen);
		oulLen = zulLen;
		copySSResource(( SSCharHuge) opTable, zpRes, zulLen);
		format( zulLen, qpszName);
		FreeResource( zhRes);
      SSAux();
		}
#  endif

	void SSYaccTable::format( SSUnsigned32 qulLen, const char* qpszTable)
		{
		qulLen;//Unreferenced
#if defined( SSBIGENDIAN)
#     include<ssltbyc0.cpp>
#endif
		SSUnsigned32 zulShift = 0;
#if defined( SSCONVERTTABLE)
		zulShift = convert();
#endif
		SSYaccPairTableHuge zpTable = ( SSYaccPairTableHuge) opTable;
		if ( zpTable->type() != pairX)
			if ( zpTable->type() != newPair)
				if ( zpTable->type() != SSYaccTableTypeNewestPair0)
					{
					freeSSSpecial( zpTable, oulLen);
					throwException( SSExceptionYaccRead, SSYaccMsgRead, qpszTable);
					return;
					}

		if ( zpTable->size() >= SSYaccPairTableSize)
			{
			freeSSSpecial( zpTable, oulLen);
			throwException( SSExceptionYaccTableSize, SSYaccMsgTableSize);
			}

		unsigned long i = 0;
		unsigned long zulRow = zpTable->states();
		SSUnsigned32* zaulRow = zpTable->offsetArray( zulShift);
		SSYaccPairTableRow* zpRow = zpTable->firstRow();
		SSYaccPairTableRow** zapRow = zpTable->rowArray();
		SSCharHuge zpLar = ( SSCharHuge) zpRow;
		for ( i = 0; zulRow--; i++)
			if ( zpTable->type() == newPair || zpTable->type() == SSYaccTableTypeNewestPair0)
				{
				SSYaccPairTableRow* zpNew =( SSYaccPairTableRow*) (( SSCharHuge) zpTable + zaulRow[ i] + zulShift);
				zapRow[ i] = zpNew;
				zpLar = ( SSCharHuge) zpNew + zpNew->size();
//				zapRow[ i] = ( SSYaccPairTableRow*)
//					(( SSCharHuge) zpTable + zaulRow[ i]);
//				zpLar += zapRow[ i]->size();
				}
			else
				{
				zapRow[ i] = zpRow;
				zpRow = ( SSYaccPairTableRow*) (( char*) zpRow + zpRow->size() + zulShift);
				}

		larTables( zpLar, qpszTable);
		}

#  if defined( _Windows) || defined( __WIN32__) || defined( WIN32) || defined( _WINDOWS)
	SSYaccTable::SSYaccTable( const char* qpszTable, SSBooleanValue qVal) :
		opTable( 0), oapIterator( 0)
		{
		qVal;//Unreferenced
#  if defined( __WIN32__) || defined( WIN32)
		HFILE zFile = _lopen( qpszTable, OF_READ);
#  else
		HFILE zFile = _lopen( qpszTable, READ);
#  endif
		if ( zFile == HFILE_ERROR)
			{
			throwException( SSExceptionYaccOpen, SSYaccMsgOpen, qpszTable);
			return;
			}

		SSUnsigned32 zulLen = _llseek( zFile, 0, 2);
		_llseek( zFile, 0, 0);

		oulLen = zulLen;
		opTable = mallocSSSpecial( zulLen);
		if ( _hread( zFile, ( SSCharHuge) opTable, zulLen) == HFILE_ERROR)
			{
			freeSSSpecial( opTable, zulLen);
			throwException( SSExceptionYaccRead, SSYaccMsgRead, qpszTable);
			return;
			}
		_lclose( zFile);

		SSYaccPairTableHuge zpTable = ( SSYaccPairTableHuge) opTable;
		if ( zpTable->type() != pairX)
			if ( zpTable->type() != newPair)
				if ( zpTable->type() != SSYaccTableTypeNewestPair0)
					{
					freeSSSpecial( zpTable, oulLen);
					throwException( SSExceptionYaccRead, SSYaccMsgRead, qpszTable);
					return;
					}

		if ( zpTable->size() >= SSYaccPairTableSize)
			{
			freeSSSpecial( zpTable, oulLen);
			throwException( SSExceptionYaccTableSize, SSYaccMsgTableSize);
			}

		unsigned long i = 0;
		unsigned long zulRow = zpTable->states();
		SSUnsigned32* zaulRow = zpTable->offsetArray();
		SSYaccPairTableRow* zpRow = zpTable->firstRow();
		SSYaccPairTableRow** zapRow = zpTable->rowArray();
		SSCharHuge zpLar = ( SSCharHuge) zpRow;
		for ( i = 0; zulRow--; i++)
			if ( zpTable->type() == newPair || zpTable->type() == SSYaccTableTypeNewestPair0)
				{
				SSYaccPairTableRow* zpNew =( SSYaccPairTableRow*) (( SSCharHuge) zpTable + zaulRow[ i]);
				zapRow[ i] = zpNew;
				zpLar = ( SSCharHuge) zpNew + zpNew->size();
//				zapRow[ i] = ( SSYaccPairTableRow*)
//					(( SSCharHuge) zpTable + zaulRow[ i]);
//				zpLar += zapRow[ i]->size();
				}
			else
				{
				zapRow[ i] = zpRow;
				zpRow = ( SSYaccPairTableRow*) (( char*) zpRow + zpRow->size());
				}

		larTables( zpLar, qpszTable);
		SSAux();
		}
#endif

	SSYaccTable::SSYaccTable( const char* qpszTable) : opTable( 0),
		oapIterator( 0)
		{
		FILE* zpFile;
		zpFile = fopen( qpszTable, "rb");
		if ( !zpFile)
			{
			throwException( SSExceptionYaccOpen, SSYaccMsgOpen, qpszTable);
			return;
			}

		fseek( zpFile, 0, SEEK_END);
		unsigned long zulLen = ftell( zpFile);
		fseek( zpFile, 0, SEEK_SET);

		oulLen = zulLen;
		opTable = mallocSSSpecial( zulLen);
		if ( !readSSFile( opTable, zulLen, zpFile))
			{
			freeSSSpecial( opTable, zulLen);
			throwException( SSExceptionYaccRead, SSYaccMsgRead, qpszTable);
			return;
			}

		fclose( zpFile);

#if defined( SSBIGENDIAN)
#  include<ssltbyc1.cpp>
#endif

		SSUnsigned32 zulShift = 0;
#if defined( SSCONVERTTABLE)
		zulShift = convert();
#endif
		SSYaccPairTableHuge zpTable = ( SSYaccPairTableHuge) opTable;

		if ( zpTable->type() != pairX)
			if ( zpTable->type() != newPair)
				if ( zpTable->type() != SSYaccTableTypeNewestPair0)
					{
					freeSSSpecial( zpTable, oulLen);
					throwException( SSExceptionYaccRead, SSYaccMsgRead, qpszTable);
					return;
					}

		if ( zpTable->size() >= SSYaccPairTableSize)
			{
			freeSSSpecial( zpTable, oulLen);
			throwException( SSExceptionYaccTableSize, SSYaccMsgTableSize);
			}

		unsigned long i = 0;
		unsigned long zulRow = zpTable->states();
		SSUnsigned32* zaulRow = zpTable->offsetArray( zulShift);
		SSYaccPairTableRow* zpRow = zpTable->firstRow();
		SSYaccPairTableRow** zapRow = zpTable->rowArray();
		SSCharHuge zpLar = ( SSCharHuge) zpRow;
		for ( i = 0; zulRow--; i++)
			if ( zpTable->type() == newPair || zpTable->type() == SSYaccTableTypeNewestPair0)
				{
				SSYaccPairTableRow* zpNew =( SSYaccPairTableRow*) (( SSCharHuge) zpTable + zaulRow[ i] + zulShift);
				zapRow[ i] = zpNew;
//				zpLar += zapRow[ i]->size();
				zpLar = ( SSCharHuge) zpNew + zpNew->size();
				}
			else
				{
				zapRow[ i] = zpRow;
				zpRow = ( SSYaccPairTableRow*) (( char*) zpRow + zpRow->size() + zulShift);
				}

		larTables( zpLar, qpszTable);
		SSAux();
		}

	SSUnsigned32 SSYaccTable::convert( void)
	{
		SSUnsigned32 zulNew = sizeof( SSYaccPairTable) + sizeof( SSYaccPairTableRow*) * (( SSYaccPairTable*) opTable)->states();
		SSUnsigned32 zulSize = oulLen + zulNew;
		SSYaccPairTable* zpTable = ( SSYaccPairTable*) mallocSS( zulSize);
		memcpy( zpTable, opTable, sizeof( SSYaccPairTable));
		memcpy(( char*) zpTable + zulNew, opTable, oulLen);
		zpTable->convert(( SSYaccPairTable*) opTable, zulNew);
		freeSS( opTable);
		opTable = zpTable;
		return zulNew;
	}

	void SSYaccPairTable::convert( SSYaccPairTable* qpOld, SSUnsigned32 qShift)
	{
		oulType = qpOld->oulType;
		oulProd = qpOld->oulProd;
		oulStates = qpOld->oulStates;
		oulRowOffset = qpOld->oulRowOffset + qShift;
		oulProdOffset = qpOld->oulProdOffset + qShift;
	}

	void SSYaccTable::larTables( SSCharHuge& zpLar, const char* qpszName)
		{
		SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTable;
		if ( zpTable->lar())
			{
			SSLexIterator** zapIterator = new SSLexIteratorPtr[ zpTable->lar()];
			oapIterator = zapIterator;
			for ( unsigned int i = 0; i < zpTable->lar(); i++)
				zapIterator[ i] = new SSLexIterator( zpLar, qpszName, SSFalse);
			}
		}

	void* SSYaccTable::larIterator( SSUnsigned32 qulIter)
		{
		return (( SSLexIterator**) oapIterator)[ qulIter];
		}

	SSYaccTable::~SSYaccTable( void)
		{
		SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTable;
		if ( zpTable->lar())
			{
			SSLexIterator** zapIterator = ( SSLexIterator**) oapIterator;
			for ( unsigned int i = 0; i < zpTable->lar(); i++)
				delete zapIterator[ i];
			delete [] zapIterator;
			}
		if ( opTable) freeSSSpecial( opTable, oulLen);

		}

	SSYaccStackElement* SSYacc::stackElement( void)
		{
		SSYaccStackElement* zpElement = new SSYaccStackElement;
		zpElement->refDec( SSFalse);
		return zpElement;
		}

	SSBooleanValue SSYacc::parse( void)
		{
		if ( !opTable)
			{
			throwException( SSExceptionYaccMissingTable, SSYaccMsgMissingTable);
			return SSTrue;
			}

		if ( doGetLexeme()) return SSTrue;
		for( ;;)
			{
			if ( oAbort) return SSTrue;
			switch ( oAction)
				{
				case SSYaccAction::shift:
					if ( doShift()) return SSTrue;
					break;
				case SSYaccAction::reduce:
					if ( doReduce()) return SSTrue;
					break;
				case SSYaccAction::error:
					if ( doError()) return SSTrue;
					break;
				case SSYaccAction::accept:
					oTreeRoot = oElement;
					return accept();
				case SSYaccAction::conflict:
					if ( doConflict()) return SSTrue;
					break;
				default:
					throwException( SSExceptionYaccParse, SSYaccMsgParse);
					return SSTrue;
				}
			}
		}

	SSBooleanValue SSYacc::doShift( void)
		{
		oElement = shift( lookahead());
		SSYaccStackElement* zpEle = oElement;
		if ( !oElement) return SSTrue;
		oElement->oLexeme = lookahead();
		oElement->oulState = state();
		push();
		if ( doGetLexeme()) return SSTrue;
		oulError++;
		return SSFalse;
		}

	SSBooleanValue SSYacc::doReduce( void)
		{
		oElement = reduce( production(), productionSize());
		if ( !oElement) return SSTrue;
		pop( productionSize());
		if ( goTo( leftside())) return SSTrue;
		return SSFalse;
		}

	SSBooleanValue SSYacc::doError( void)
		{
		oError = SSTrue;
		if ( error( state(), lookahead()))
			return SSTrue;
		oulError = 0;
		return SSFalse;
		}

	SSBooleanValue SSYacc::doLarError( void)
		{
		oError = SSTrue;
		if ( larError( state(), lookahead(), *oLarLookahead))
			return SSTrue;
		oulError = 0;
		return SSFalse;
		}

	SSLexLexeme* SSYacc::getLexemeCache( void)
		{
		SSLexLexemeRef zLexeme;
		if ( oulCache != SSYaccLexemeCacheMax)
			zLexeme = lexemeCache().get( oulCache++);
		if ( !zLexeme)
			{
			oulCache = SSYaccLexemeCacheMax;
			zLexeme = nextLexeme();
			if ( !zLexeme)
				{
				oEndOfInput = SSTrue;
				zLexeme = orEndLexeme;
				}
			lexemeCache().add( *zLexeme);
			}
		return zLexeme;
		}

	SSBooleanValue SSYacc::doConflict( void)
		{
		oulCache = 0;
		SSUnsigned32 zulLar = 0;
		SSLexIterator* zpIterator = ( SSLexIterator*) opIterator;
		zpIterator->lookupNext( zulLar, oLookahead->token());
		while ( oLarLookahead = getLexemeCache())
			{
			if ( !zpIterator->lookupNext( zulLar, oLarLookahead->token()))
				break;
			SSLexFinalState& zFinal = zpIterator->lookupFinal( zulLar);
			if ( zFinal.isFinal())
				{
				if ( zFinal.isReduce())
					{
					oulProduction = zFinal.token();
					SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTab;
					SSYaccPairTableProd* zpProd = zpTable->lookupProd( oulProduction);
					oulLeftside = zpProd->leftside();
					oulProductionSize = zpProd->size();
					return doReduce();
					}
				else
					{
					oulState = zFinal.token();
					return doShift();
					}
				}
			}
		return doLarError();
		}

	SSBooleanValue SSYacc::doGetLexeme( SSBooleanValue qLook)
		{
		if ( oLookahead = lexemeCache().remove())
			{
			if ( larLookahead( *oLookahead))
				return SSTrue;
         if ( qLook)
				lookupAction( state(), lookahead().token());
			return SSFalse;
			}
		return getLexeme( qLook);
		}

	SSBooleanValue SSYacc::larLookahead( SSLexLexeme& qLexeme)
		{
		qLexeme;//Unreferenced
		return SSFalse;
		}

	SSBooleanValue SSYacc::getLexeme( SSBooleanValue qLook)
		{
		if ( oEndOfInput)
			{
			throwException( SSExceptionYaccEof, SSYaccMsgEof);
			return SSTrue;
			}
		oLookahead = nextLexeme();
		if ( !oLookahead)
			{
			oEndOfInput = SSTrue;
			oLookahead = orEndLexeme;
			}

      if ( qLook)
			lookupAction( state(), lookahead().token());
		return SSFalse;
		}

	SSLexLexeme* SSYacc::nextLexeme( void)
		{
		if ( !opLex)
			{
			throwException( SSExceptionYaccMissingLex, SSYaccMsgMissingLex);
			return 0;
			}
		return ( *opLex)++;
		}

   SSYaccStackElement* SSYacc::shift( SSLexLexeme& qLexeme)
      {
	  qLexeme;//Unreferenced
      return stackElement();
      }

   SSBooleanValue SSYacc::accept( void)
      {
      return wasError();
      }

	SSBooleanValue SSYacc::error( SSUnsigned32 qulState,
      SSLexLexeme& qLookahead)
      {
	  qulState;//Unreferenced
	  qLookahead;//Unreferenced
      return errorToken();
      }

	SSBooleanValue SSYacc::larError( SSUnsigned32 qulState,
		SSLexLexeme& qLookahead, SSLexLexeme& qLarLookahead)
		{
		qLarLookahead;//Unreferenced
		return error( qulState, qLookahead);
		}

	SSBooleanValue SSYacc::goTo( SSUnsigned32 qulGoto)
		{
		qulGoto;//Unreferenced
      if ( lookupGoto( state(), leftside())) return SSTrue;
      oElement->oulState = state();
      push();
      lookupAction( state(), lookahead().token());
      return SSFalse;
      }

   void SSYacc::selectToken( SSUnsigned32* qaToken, SSUnsigned32 qulToken,
      SSBooleanValue qGet)
      {
      if ( qaToken == 0) return;

      if ( qGet)
         {
         if ( oEndOfInput) 
            {
				throwException( SSExceptionYaccSelect, SSYaccMsgSelect);
				return;
				}
			doGetLexeme();
			}

		for ( ;;)
			{
			unsigned int i;
			SSUnsigned32* zaToken;
			for ( i = 0, zaToken = qaToken; i < qulToken; i++, zaToken++)
				if ( zaToken[ i] == lookahead().token()) return;
			if ( oEndOfInput)
				{
				throwException( SSExceptionYaccSelect, SSYaccMsgSelect);
				return;
				}
			doGetLexeme();
			}
		}

	SSBooleanValue SSYacc::errorPanic( SSUnsigned32* qaToken,
		SSUnsigned32 qulToken)
      {
      SSYaccStack& zStack = ( *(( SSYaccStack*) opStack));
		SSYaccStack zCopy( zStack);
      selectToken( qaToken, qulToken);
      for ( ;;)
         {
         while ( zStack.elements() > 1)
            {
            pop();
            lookupAction( state(), lookahead().token());
            if ( oAction != SSYaccAction::error) 
               return SSFalse;
            }
         zStack = zCopy;
         selectToken( qaToken, qulToken, SSTrue);
         if ( oAction != SSYaccAction::error)
            break;
         }

      return SSFalse;
      }

   SSBooleanValue SSYacc::errorToken( void)
      {
		SSSetArrayOfUnsigned32 zSet( SSYaccSetArraySize, SSYaccSetArrayInc);
      SSYaccStack& zStack = ( *(( SSYaccStack*) opStack));
      SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTab;
      for ( unsigned int i = 0; i < zStack.elements(); i++)
         {
         SSUnsigned32 zulState = ( *zStack[ i])->state();
         SSYaccPairTableRow* zpRow = zpTable->lookupRow( zulState);
         if ( zpRow->hasSync() || zpRow->hasSyncAll())
            {
            SSYaccPairTableEntry* zpEntry = ( SSYaccPairTableEntry*) ( zpRow + 1);
            for ( unsigned int j = 0; j < zpRow->action(); j++, zpEntry++)
               if ( zpRow->hasSyncAll() || zpEntry->hasSync())
                  {
                  SSUnsigned32 zulToken = zpEntry->token();
                  zSet.add( zulToken);
                  }
            }
         if ( zpRow->hasError())
            {
            SSYaccPairTableEntry* zpEntry = zpRow->lookupError();
            SSYaccPairTableRow* zpError = zpTable->lookupRow( zpEntry->entry());
				zpEntry = ( SSYaccPairTableEntry*) ( zpError + 1);
            for ( unsigned int j = 0; j < zpError->action(); j++, zpEntry++)
               {
               SSUnsigned32 zulToken = zpEntry->token();
               zSet.add( zulToken);
               }
            }
         }

      if ( !zSet.size())
         {
			throwException( SSExceptionYaccTokenState, SSYaccMsgTokenState);
			return SSTrue;
			}

      for ( ;;)
         {
         if ( zSet.locate( lookahead().token())) break;
//         oLookahead = nextLexeme();
//         if ( !oLookahead) 
		if ( doGetLexeme( SSFalse))
            {
				throwException( SSExceptionYaccErrorTokenEof, SSYaccMsgErrorTokenEof);
				return SSTrue;
				}
			}

      for ( ;;)
         {
         SSYaccPairTableRow* zpRow = zpTable->lookupRow( oulState);
         if ( zpRow->hasError())
            {
            lookupAction( zpRow->lookupError()->entry(), lookahead().token());
            if ( oAction != SSYaccAction::error)
               {
               SSUnsigned32 zulLen = strlen( SSYaccErrorLexeme);
               SSLexLexeme* zpLexeme = new( zulLen) SSLexLexeme( zulLen,
                  SSYaccErrorToken, SSYaccErrorLexeme, 0, 0);
               zpLexeme->refDec( SSFalse);
               oElement = stackElement();
               oElement->oLexeme = zpLexeme;
               oElement->oulState = zpRow->lookupError()->entry();
               push();
               break;
               }
				}
         if ( zpRow->hasSyncAll())
            {
            lookupAction( oulState, lookahead().token());
            if ( oAction != SSYaccAction::error)
               break;
            }
         else if ( zpRow->hasSync())
            {
            SSYaccPairTableEntry* zpEntry = zpRow->lookupAction( lookahead().token());
            if ( zpEntry)
               {
               lookupAction( oulState, lookahead().token());
               break;
               }
            }

         pop();
         }

      return SSFalse;
      }

	SSUnsigned32* SSYacc::validLookaheads( SSUnsigned32 qulState,
		SSUnsigned32& qulCount)
		{
		SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTab;
		if ( qulState > zpTable->states())
			throwException( SSExceptionYaccLook, SSYaccMsgLook, qulState);

		SSYaccPairTableRow* zpRow = zpTable->lookupRow( qulState);
		SSYaccPairTableEntry* zpEntry = ( SSYaccPairTableEntry*) ( zpRow + 1);
		SSUnsigned32* zpArray = new SSUnsigned32[ zpRow->action()];

		for ( unsigned int i = 0; i < zpRow->action(); i++, zpEntry++)
			zpArray[ i] = zpEntry->token();

		qulCount = zpRow->action();
		return zpArray;
		}

	void SSYacc::lookupAction( SSUnsigned32 qulState, SSUnsigned32 qulToken)
      {
      SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTab;
		SSYaccPairTableRow* zpRow = zpTable->lookupRow( qulState);
      SSYaccPairTableEntry* zpEntry = zpRow->lookupAction( qulToken);
      if ( !zpEntry) 
         oAction = SSYaccAction::error;
      else
         {
         switch ( oAction = zpEntry->action())
            {
            case SSYaccAction::shift:
               oulState = zpEntry->entry();
               break;

            case SSYaccAction::reduce:
               {
					SSYaccPairTableProd* zpProd = zpTable->lookupProd( zpEntry->entry());
					oulProduction = zpEntry->entry();
					oulLeftside = zpProd->leftside();
					oulProductionSize = zpProd->size();
					break;
					}

				case SSYaccAction::conflict:
					opIterator = opTable->larIterator( zpEntry->entry());
					break;

				default:
					break;
				}
			}
		}

   SSBooleanValue SSYacc::lookupGoto( SSUnsigned32 qulState, 
      SSUnsigned32 qulToken)
      {
      SSYaccPairTable* zpTable = ( SSYaccPairTable*) opTab;
      SSYaccPairTableRow* zpRow = zpTable->lookupRow( qulState);
      SSYaccPairTableEntry* zpEntry = zpRow->lookupGoto( qulToken);
      if ( !zpEntry)
         {
			throwException( SSExceptionYaccGoto, SSYaccMsgGoto, qulToken);
			return SSTrue;
			}
      else
         oulState = zpEntry->entry();

      return SSFalse;
      }

   SSBooleanValue SSYacc::push( void)
      {
      SSYaccStack* zpStack = ( SSYaccStack*) opStack;
      SSBooleanValue zPush = zpStack->push( oElement);
      if ( setException( zpStack->getException())) return SSTrue;
      return SSFalse;
      }

   SSBooleanValue SSYacc::push( SSYaccStackElement& qElement)
      {
      SSYaccStackElementRef zRef = qElement;
      SSYaccStack* zpStack = ( SSYaccStack*) opStack;
      SSBooleanValue zPush = zpStack->push( zRef);
      if ( setException( zpStack->getException())) return SSTrue;
      return SSFalse;
      }

   SSBooleanValue SSYacc::pop( SSUnsigned32 qulPop)
      {
      SSYaccStack* zpStack = ( SSYaccStack*) opStack;
      for ( unsigned int i = 0; i < qulPop; i++)
         {
         if ( zpStack->top())
            *( zpStack->top()) = 0;
         zpStack->pop();
         if ( setException( zpStack->getException())) return SSTrue;
         }
      SSYaccStackElement* zpElement = *( zpStack->top());
      if ( setException( zpStack->getException())) return SSTrue;
      oulState = zpElement->state();
      return SSFalse;
      }

   SSYaccStackElement* SSYacc::elementFromStack( SSUnsigned32 qulDepth)
      {
      SSYaccStack* zpStack = ( SSYaccStack*) opStack;
      SSYaccStackElement* zpElement = *( zpStack->operator[]( qulDepth));
      if ( setException( zpStack->getException())) return 0;
      return zpElement;
      }

   SSYaccStackElement* SSYacc::elementFromProduction( SSSigned32 qlIndex)
      {
      SSYaccStack* zpStack = ( SSYaccStack*) opStack;
      SSSigned32 zlIndex = productionSize() - qlIndex - 1;
      if ( zlIndex < 0 || zlIndex >= ( SSSigned32) zpStack->elements())
         {
			throwException( SSExceptionYaccProdSize, SSYaccMsgProdSize, qlIndex);
			return 0;
			}
		SSYaccStackElement* zpElement = *( zpStack->operator[]( zlIndex));
		if ( setException( zpStack->getException())) return 0;
		return zpElement;
		}

   void SSYacc::reset( void)
      {
      oulState = 0;
      oulError = 0;
      oLookahead = 0;
      oError = SSFalse;
      oAbort = SSFalse;
      oEndOfInput = SSFalse;
      delete ( SSYaccStack*) opStack;
      setStack();
      }

   SSYacc::~SSYacc( void)
      {
		if ( oOwner) delete opTable;
		delete ( SSYaccStack*) opStack;
		if ( opLexemeCache) delete opLexemeCache;
		}

   void SSYaccStackElement::refFree( void)
      {
      delete this;
		}

	SSYaccLexemeCache::SSYaccLexemeCache( void) : oulSize( 32), oulEnd( 0),
		oulStart( 0), oulElements( 0)
		{
		oaLexemeRef = new SSLexLexemeRef[ 32];
		}

	void SSYaccLexemeCache::add( SSLexLexeme& qLexeme)
		{
		if ( oulEnd + 1 >= oulSize)
			{
			if ( oulStart)
				{
				for ( unsigned int i = 0; i < oulElements; i++)
					oaLexemeRef[ i] = oaLexemeRef[ i + oulStart];
				oulEnd -= oulStart;
				oulStart = 0;
				}
			else
				{
				SSLexLexemeRef* zaLexemeRef = new SSLexLexemeRef[ oulSize + 32];
				for ( unsigned int i = 0; i < oulSize; i++)
					zaLexemeRef[ i] = oaLexemeRef[ i];
				oulSize += 32;
				delete [] oaLexemeRef;
				oaLexemeRef = zaLexemeRef;
				}
			}

		oaLexemeRef[ oulEnd++] = qLexeme;
		oulElements++;
		}

	SSLexLexeme* SSYaccLexemeCache::remove( void)
		{
		if ( !oulElements) return 0;
		oulElements--;
		return oaLexemeRef[ oulStart++];
		}

	SSLexLexeme* SSYaccLexemeCache::get( SSUnsigned32 qulIndex)
		{
		if (( qulIndex == SSYaccLexemeCacheMax) || ( qulIndex + 1 > oulElements))
			return 0;
		return oaLexemeRef[ oulStart + qulIndex];
		}

	void SSYaccLexemeCache::flush( void)
		{
		for ( unsigned int i = 0; i < oulSize; i++)
			oaLexemeRef[ i] = 0;
		oulElements = oulStart = oulEnd = 0;
		}

	SSYaccLexemeCache::~SSYaccLexemeCache( void)
		{
		delete [] oaLexemeRef;
		}

	SSYaccStackElement* SSYacc::addSubTree( void)
	{
		SSYaccStackElement* z_pEle = stackElement();
		z_pEle->createSubTree( oulProductionSize);
		for ( unsigned int i = 0; i < oulProductionSize; i++)
			z_pEle->addSubTree( i, elementFromProduction( i));
		return z_pEle;
	}

/*---------------------------------------------------------------------------

					Copyright (c) 1994-1999 SandStone Technology Inc.
								  All rights reserved

		THIS PROGRAM CONSISTS OF TRADE SECRETS THAT ARE  THE PROPERTY
		OF SANDSTONE SOFTWARE INC..  THE CONTENTS  MAY NOT BE USED OR
		DISCLOSED WITHOUT THE EXPRESS WRITTEN PERMISSION OF THE OWNER.

----------------------------------------------------------------------------*/
