/*---------------------------------------------------------------------------

               Copyright (c) 1994-1997 SandStone Technology Inc.
                          All rights reserved

----------------------------------------------------------------------------*/
#if !defined( SSYACC)
#  define SSYACC
#  include<ssref.hpp>
#  include<sslex.hpp>

#  define SSYaccEofLexeme        "eof"
#  define SSYaccErrorLexeme      "%error"
#  define SSYaccErrorToken       0xfffffffe
#  define SSYaccEofToken         0xffffffff
#  define SSYaccLexemeCacheMax   0xffffffff

#  define SSYaccMsgOpen          "SSYacc0101e: Open file failed, %s"
#  define SSYaccMsgParse         "SSYacc0102e: Parse failed, invalid table"
#  define SSYaccMsgEof           "SSYacc0103e: Attempted read past eof"
#  define SSYaccMsgSelect        "SSYacc0104e: Error panic failed, no valid token"
#  define SSYaccMsgTokenState    "SSYacc0105e: Error token failed, no valid token"
#  define SSYaccMsgErrorTokenEof "SSYacc0106e: Error token failed, eof"
#  define SSYaccMsgGoto          "SSYacc0107e: Invalid goto on %ld"
#  define SSYaccMsgProdSize      "SSYacc0108e: Invalid index on elementFromProduction, %ld"
#  define SSYaccMsgMissingTable  "SSYacc0109e: setTable missing, required"
#  define SSYaccMsgMissingLex    "SSYacc0110e: setLex missing, required"
#  define SSYaccMsgRead          "SSYacc0111e: Error reading table, %s"
#  define SSYaccMsgTableSize     "SSYacc0112e: Table too big"
#  define SSYaccMsgPermError     "SSYacc0113e: Permanent error"
#  define SSYaccMsgLook          "SSYacc0114e: Invalid state specified, %ld"
#  define SSYaccMsgResource      "SSYacc0115e: Resource not found, %s"
#  define SSYaccMsgSubTree       "SSYacc0116e: No Subtree, or subtree index too large"

	class SSClass SSYacc;
	class SSClass SSYaccTable;
	class SSClass SSYaccStackElement;

	class SSClass SSYaccTable : public SSExceptionPresent
    {
    friend class SSYacc;

    public:
		SSConstr SSYaccTable( const char*);
		SSConstr SSYaccTable( const char*, SSBooleanValue);
		SSConstr SSYaccTable( void*, SSUnsigned32, const char*);
		SSConstr SSYaccTable( unsigned int, const char*, const char*);

		SSUnsigned32 convert( void);
		void* larIterator( SSUnsigned32);
		void format( SSUnsigned32, const char*);
		void larTables( SSCharHuge&, const char*);

        virtual SSDestr SSYaccTable( void);

	protected:
		void* opTable;
		void* oapIterator;
		SSUnsigned32 oulLen;
    };

#  if ( _MSC_VER == 800) || defined( SSNOTEMPLATE)
#     include<ssrefstk.hpp>
#  else
	     typedef SSRef< SSYaccStackElement> SSYaccStackElementRef;
#  endif

	class SSClass SSYaccStackElement : public SSRefCount, public SSExceptionPresent
	{
    friend class SSYacc;

    public:
		SSInline SSConstr SSYaccStackElement( void);

		void refFree( void);
		SSInline SSUnsigned32 state( void);
		SSInline SSLexLexeme* lexeme( void);
		SSInline void setState( SSUnsigned32);
		SSInline void setLexeme( SSLexLexeme&);
		SSInline void createSubTree( SSUnsigned32);
		SSInline SSYaccStackElement* subTree( SSUnsigned32);
		SSInline void addSubTree( SSUnsigned32, SSYaccStackElement*);
		SSUnsigned32 numSubTree( void) { return oulSubTree;}

		virtual SSDestr SSYaccStackElement( void);
	
	protected:
		SSUnsigned32 oulState;
		SSLexLexemeRef oLexeme;
		SSUnsigned32 oulSubTree;
		SSYaccStackElementRef* opSubTree;
	};

	class SSClass SSYaccLexemeCache
	{
	public:
		SSConstr SSYaccLexemeCache( void);

		void flush( void);
		void add( SSLexLexeme&);
		SSLexLexeme* remove( void);
		SSLexLexeme* get( SSUnsigned32);
		SSInline SSUnsigned32 elements( void);

		virtual SSDestr SSYaccLexemeCache( void);

	protected:
		SSUnsigned32 oulEnd;
		SSUnsigned32 oulSize;
		SSUnsigned32 oulStart;
		SSUnsigned32 oulElements;
		SSLexLexemeRef* oaLexemeRef;
	};

	class SSClass SSYacc : public SSExceptionPresent
	{
	public:
		SSConstr SSYacc( const char* = 0);
		SSConstr SSYacc( SSLex&, const char*);
		SSConstr SSYacc( SSLex&, SSYaccTable&);

		virtual SSBooleanValue accept( void);
		virtual SSLexLexeme* nextLexeme( void);
		virtual SSYaccStackElement* stackElement( void);
		virtual SSYaccStackElement* shift( SSLexLexeme&);
		virtual SSBooleanValue larLookahead( SSLexLexeme&);
		virtual SSBooleanValue error( SSUnsigned32, SSLexLexeme&);
		virtual SSYaccStackElement* reduce( SSUnsigned32, SSUnsigned32) = 0;
		virtual SSBooleanValue larError( SSUnsigned32, SSLexLexeme&, SSLexLexeme&);


		void reset( void);
		void setEof( void);
		void setCache( void);
		SSInline SSLex* lex( void);
		void setTable( const char*);
		SSBooleanValue parse( void);
		SSInline void setAbort( void);
		SSInline void setLex( SSLex&);
		SSBooleanValue errorToken( void);
		SSInline SSUnsigned32 state( void);
		SSLexLexeme* getLexemeCache( void);
		SSYaccStackElement* addSubTree( void);
		SSInline void setTable( SSYaccTable&);
		SSBooleanValue pop( SSUnsigned32 = 1);
		SSInline SSBooleanValue syncErr( void);
		SSInline SSLexLexeme& lookahead( void);
		SSInline SSBooleanValue wasError( void);
		SSInline SSUnsigned32 production( void);
		SSBooleanValue push( SSYaccStackElement&);
		SSInline SSBooleanValue wasAborted( void);
		SSInline SSUnsigned32 productionSize( void);
		SSInline SSYaccStackElement* treeRoot( void);
		SSInline SSUnsigned32 shiftedSinceError( void);
		SSInline SSYaccLexemeCache& lexemeCache( void);
		SSYaccStackElement* elementFromProduction( SSSigned32);
		SSYaccStackElement* elementFromStack( SSUnsigned32 = 0);
		SSUnsigned32* validLookaheads( SSUnsigned32, SSUnsigned32&);
        SSBooleanValue errorPanic( SSUnsigned32* = 0, SSUnsigned32 = 0);

		virtual SSDestr SSYacc( void);

      protected:
		SSLex* opLex;
		void* opIterator;
		SSYaccTable* opTable;
		SSBooleanValue oOwner;
		SSBooleanValue oError;
		SSBooleanValue oAbort;
		SSUnsigned32 oulState;
		SSUnsigned32 oulError;
		SSLexLexemeRef oLookahead;
		SSBooleanValue oEndOfInput;
		SSUnsigned32 oulProduction;
		SSLexLexemeRef oLarLookahead;
		SSUnsigned32 oulProductionSize;
		SSYaccLexemeCache* opLexemeCache;

      protected:
		void* opTab;
		void* opStack;
		SSUnsigned32 oAction;
		SSUnsigned32 oulCache;
		SSUnsigned32 oulLeftside;
		SSLexLexemeRef orEndLexeme;
		SSYaccStackElementRef oElement;
		SSYaccStackElementRef oTreeRoot;

		SSBooleanValue doError( void);
		SSBooleanValue doShift( void);
		SSBooleanValue doReduce( void);
		SSBooleanValue doConflict( void);
		SSBooleanValue doLarError( void);
		SSBooleanValue doGetLexeme( SSBooleanValue = SSTrue);

		void setStack( void);
		SSBooleanValue push( void);
		SSUnsigned32 leftside( void);
		SSBooleanValue goTo( SSUnsigned32);
		void lookupAction( SSUnsigned32, SSUnsigned32);
		SSBooleanValue getLexeme( SSBooleanValue = SSTrue);
		SSBooleanValue lookupGoto( SSUnsigned32, SSUnsigned32);
		void selectToken( SSUnsigned32*, SSUnsigned32, SSBooleanValue = SSFalse);
      };

#  include<ssyacc.inl>

#endif
