#include "stdafx.h"

#include "SMCpp.h"
#include "VPCppClasses.hpp"
#include "VPCppFunctions.cpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

SSYaccStackElement *
CppYaccClass::reduce(SSUnsigned32 ulProd, SSUnsigned32 /* ulSize */)
{
   static CString sOverloadOpr;

   switch( ulProd )
   {
   // NOTE: many productions are ignored because they help parse but aren't relevant to any metrics
   case CppYaccError:            // metricCount -> %error \n
      m_poSMCpp->ParseError();   // report error to user, then count current line anyway
   case CppYaccSourceLine:       // metricCount -> \n 
      m_poSMCpp->BumpCount(SMCpp::LINE_COUNT);
      if( m_fInsideComment )
      {
         m_poSMCpp->BumpCount(SMCpp::COMMENT_LINES);
      }
      break;

   case CppYaccFORStatement:        // metricCount -> FORStmt
      m_iForSemicolons = 2;         // number of ";" counts to ignore (inside the "for" statement)
   case CppYaccOperatorStatement:   // metricCount -> Operator statement
      m_poSMCpp->BumpCount(SMCpp::BRANCH_STATEMENTS);
      {
         CString sLexeme = ((elementFromProduction(0))->lexeme())->asConstChar();

         m_poSMCpp->RemoveSpace(sLexeme);    // count embedded line feeds
      }
   case CppYaccPreprocessStatement: // metricCount -> Preprocess statement
      m_poSMCpp->BumpCount(SMCpp::STATEMENT_COUNT);
      m_poSMCpp->BumpCount(SMCpp::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      if( m_fInsideMethod )
      {
         m_poSMCpp->BumpCount(SMCpp::METHOD_STATEMENTS);
         ++m_iMethodStatements;
      }
      break;

   case CppYaccExpressionStatement: // metricCount -> ;
      m_poSMCpp->BumpCount(SMCpp::STATEMENT_COUNT);
      m_poSMCpp->BumpCount(SMCpp::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      if( m_fInsideMethod )
      {
         if( m_iBlockDepth == 0 )
         {
            m_fInsideMethod = false;   // this is static member initialization, not a method
         }
         else
         {
            m_poSMCpp->BumpCount(SMCpp::METHOD_STATEMENTS);
            ++m_iMethodStatements;
         }
      }
      break;

   case CppYaccBeginBlock:       // metricCount -> { 
      if( (m_fInsideMethod == true) and (m_iBlockDepth == 0) )
      {
         m_iMethodStatements = 0;
         m_poSMCpp->BumpCount(SMCpp::BLOCK_DEPTH_0);     // count the method declaration as a level 0 statement
         m_poSMCpp->BumpCount(SMCpp::METHOD_COUNT);
      }
      ++m_iBlockDepth;
      if( m_iBlockDepth > m_iMaxBlockDepth )
      {
         m_iMaxBlockDepth = m_iBlockDepth;
         m_poSMCpp->SetDeepestBlockLine();
      }
      break;

   case CppYaccEndBlock:         // metricCount -> } 
      if( m_iBlockDepth > 0 )
      {
         --m_iBlockDepth;
      }
      if( m_fInsideMethod and (m_iBlockDepth == 0) )
      {
         m_fInsideMethod = false;
         m_poSMCpp->SetMethodData(m_sClassName, m_sMethodName, m_iMethodStatements);
      }
      break;

   case CppYaccBeginComment:     // metricCount -> BeginMLComment 
      m_fInsideComment = true;
      break;

   case CppYaccEndComment:       // metricCount -> EndMLComment 
      m_poSMCpp->BumpCount(SMCpp::COMMENT_LINES);
      m_fInsideComment = false;
      break;

   case CppYaccCppComment:       // metricCount -> BeginSLComment 
      m_poSMCpp->BumpCount(SMCpp::COMMENT_LINES);
      break;

   case CppYaccClassMethod:      // metricCount -> Name :: FunctionName
   case CppYaccClassOperator:    // metricCount -> Name :: operatorName 
   case CppYaccClassDestructor:  // metricCount -> Name :: ~ FunctionName 
      if( (m_iBlockDepth == 0) and not m_fInsideMethod )
      {
         m_fInsideMethod = true;
         m_poSMCpp->SetBiggestMethodLine();
         m_sClassName = ((elementFromProduction(0))->lexeme())->asConstChar();
         m_sClassName = m_sClassName.Left(m_sClassName.GetLength() - 2);         // chop trailing '::'
         m_poSMCpp->RemoveSpace(m_sClassName);     // count embedded line feeds

         // now extract the method name
         m_sMethodName = GetMethodName(1);
         if( ulProd == CppYaccClassDestructor )
         {
            m_sMethodName += GetMethodName(2);
         }
         else if( ulProd == CppYaccClassOperator )
         {
            switch( sOverloadOpr[0] )
            {
            case '(':
            case '[':
               m_poSMCpp->RemoveSpace(sOverloadOpr);  // clean out white space, count line feeds
               sOverloadOpr = sOverloadOpr.Left(2);   // chop the trailing '('
               break;
            case 'n':   // new operator
            case 'd':   // delete operator
               m_sMethodName += _T(" ");              // add space after "operator"
               sOverloadOpr += _T(")");               // append closing ')'
               break;
            }
            m_sMethodName += sOverloadOpr;
         }
      }
      break;

   case CppYaccOverloadType0:  // overloadOpr -> one of 28 different overloadable operators 
   case CppYaccOverloadType1:
   case CppYaccOverloadType2:
   case CppYaccOverloadType3:
   case CppYaccOverloadType4:
   case CppYaccOverloadType5:
   case CppYaccOverloadType6:
   case CppYaccOverloadType7:
   case CppYaccOverloadType8:
   case CppYaccOverloadType9:
   case CppYaccOverloadType10:
   case CppYaccOverloadType11:
   case CppYaccOverloadType12:
   case CppYaccOverloadType13:
   case CppYaccOverloadType14:
   case CppYaccOverloadType15:
   case CppYaccOverloadType16:
   case CppYaccOverloadType17:
   case CppYaccOverloadType18:
   case CppYaccOverloadType19:
   case CppYaccOverloadType20:
   case CppYaccOverloadType21:
   case CppYaccOverloadType22:
   case CppYaccOverloadType23:
   case CppYaccOverloadType24:
   case CppYaccOverloadType25:
   case CppYaccOverloadType26:
   case CppYaccOverloadType27:   // overloadOpr -> '()('
   case CppYaccOverloadType28:   // overloadOpr -> '[]('
      // save operator string for possible later use
      sOverloadOpr = ((elementFromProduction(0))->lexeme())->asConstChar();
      break;
   }
   return stackElement();
}


// extract and clear up the method name
//
CString
CppYaccClass::GetMethodName(int iLexemeIndex)
{
   CString  sMethodName = ((elementFromProduction(iLexemeIndex))->lexeme())->asConstChar();

   m_poSMCpp->RemoveSpace(sMethodName);      // count embedded line feeds
   if( sMethodName[sMethodName.GetLength() - 1] == '(' )
   {
      // chop trailing paren, if any
      sMethodName = sMethodName.Left(sMethodName.GetLength() - 1);
   }
   return sMethodName;
}