#include "stdafx.h"

#include "SMDelphi.h"
#include "VPDelphiClasses.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Strategies:
// 1. Count subroutines, classes and class methods and variables in the interface section.
// 2. Count statements in each method or subroutine in the implementation section.

SSYaccStackElement *
DelphiYaccClass::reduce( SSUnsigned32 ulProd, SSUnsigned32 /* ulSize */)
{
   switch ( ulProd)
   {
   // NOTE: some productions are ignored because they help parse but aren't relevant to any metrics
   case DelphiYaccError:                        // metricCount -> %error \n
      m_poSMDelphi->ParseError();               // report error to user, then count current line anyway
   case DelphiYaccSourceLine:                   // metricCount -> \n 
      m_poSMDelphi->BumpCount(SMDelphi::LINE_COUNT);
      if( m_fInsideComment )
      {
         m_poSMDelphi->BumpCount(SMDelphi::COMMENT_LINES);
      }
      break;

   case DelphiYaccBeginComment:                 // metricCount -> BeginMLComment 
      m_fInsideComment = true;
      break;

   case DelphiYaccEndComment:                   // metricCount -> EndMLComment 
      m_poSMDelphi->BumpCount(SMDelphi::COMMENT_LINES);
      m_fInsideComment = false;
      break;

   case DelphiYaccComment:                      // metricCount -> Comment 
      m_poSMDelphi->BumpCount(SMDelphi::COMMENT_LINES);
      break;

   case DelphiYaccBeginVar:                     // metricCount -> Var 
      m_fInsideVarClause = true;
      break;

   case DelphiYaccStatement:                    // metricCount -> ; 
      m_fInsideProperty = false;
      if( not m_fInsideVarClause )
      {
         if( m_fInsideSubroutine )
         {
            ++m_iSubroutineStatements;
         }
         else if( m_fInsideMethod )
         {
            ++m_iMethodStatements;
         }
      }
   case DelphiYaccSimpleClassDef:               // metricCount -> ; Class ;
      m_poSMDelphi->BumpCount(SMDelphi::STATEMENT_COUNT);
      m_poSMDelphi->BumpCount(SMDelphi::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      break;

   case DelphiYaccPropertyMethod:               // metricCount -> Property 
      m_poSMDelphi->BumpCount(SMDelphi::CLASS_METHOD_COUNT);
      m_fInsideProperty = true;
      break;

   case DelphiYaccCaseOrVariableStatement:      // metricCount -> : 
      if( m_fInsideInterface and (not m_fInsideProperty) )
      {
         if( m_fInsideClass )
         {
            m_poSMDelphi->BumpCount(SMDelphi::CLASS_VARIABLE_COUNT);
         }
         else
         {
            m_poSMDelphi->BumpCount(SMDelphi::GLOBAL_VARIABLE_COUNT);
         }
      }
      else if( m_fInsideCase )
      {
         m_poSMDelphi->BumpCount(SMDelphi::BRANCH_COUNT);
      }
      break;

   case DelphiYaccBeginInterface:               // metricCount -> Interface \n 

      // must count the embedded \n
      m_poSMDelphi->BumpCount(SMDelphi::LINE_COUNT);
      m_fInsideInterface = true;
      break;

   case DelphiYaccBeginImplementation:          // metricCount -> Implementation \n 

      // must count the embedded \n
      m_poSMDelphi->BumpCount(SMDelphi::LINE_COUNT);
      m_fInsideInterface = false;
      m_fInsideVarClause = false;               // implementation ends a var section, if any
      break;

   case DelphiYaccSubroutine:                   // metricCount -> Subroutine 
      if( m_fInsideInterface )
      {
         // definition of either a class or a subroutine
         m_fInsideProperty = true;              // ignore semicolons in call arguments
         if( m_fInsideClass )
         {
            m_poSMDelphi->BumpCount(SMDelphi::CLASS_METHOD_COUNT);
         }
         else
         {
            m_poSMDelphi->BumpCount(SMDelphi::GLOBAL_SUBROUTINE_COUNT);
         }
      }
      else if( m_fInsideVarClause )
      {
         // this is a procedure or function embedded inside a class method or a subroutine
         m_poSMDelphi->BumpCount(SMDelphi::LOCAL_SUBROUTINE_COUNT);
      }
      else
      {
         // must be the implementation of a global subroutine
         m_fInsideSubroutine = true;
         m_iSubroutineStatements = 0;
         m_sSubroutineName = ((elementFromProduction(0))->lexeme())->asConstChar();
         m_poSMDelphi->SetBiggestSubroutineLine();
      }
      break;

   case DelphiYaccMethodImplementation:         // metricCount -> MethodImpl 

      // now we are implementing a method and must count the statements inside
      m_fInsideMethod = true;
      m_iMethodStatements = 0;
      m_sClassMethodNames = ((elementFromProduction(0))->lexeme())->asConstChar();
      m_poSMDelphi->SetBiggestMethodLine();
      break;

   case DelphiYaccBeginBlock:                   // metricCount -> Begin 
      if( not m_fInsideSubroutine )             // skip begin/end inside local subroutines
      {
         m_fInsideVarClause = false;            // begin ends a var section, if any
      }
   case DelphiYaccCaseOf:                       // metricCount -> Case 
   case DelphiYaccBeginClassDef:                // metricCount -> Class \n
   case DelphiYaccBeginRecordDef:               // metricCount -> Record 
   case DelphiYaccBeginAsmBlock:                // metricCount -> Asm
      m_poSMDelphi->BumpCount(SMDelphi::STATEMENT_COUNT);
      m_poSMDelphi->BumpCount(SMDelphi::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      ++m_iBlockDepth;
      if( m_iBlockDepth > m_iMaxBlockDepth )
      {
         m_iMaxBlockDepth = m_iBlockDepth;
         m_poSMDelphi->SetDeepestBlockLine();
      }
      switch( ulProd )
      {
      case DelphiYaccCaseOf:
         m_fInsideCase = true;
         m_poSMDelphi->BumpCount(SMDelphi::BRANCH_COUNT);
         if( m_iCaseBlockLevel == 0 )
         {
            m_iCaseBlockLevel = m_iBlockDepth;
         }
         break;
      case DelphiYaccBeginClassDef:
         m_fInsideClass = true;                          // start counting class methods and variables
         m_poSMDelphi->BumpCount(SMDelphi::CLASS_COUNT);
         m_poSMDelphi->BumpCount(SMDelphi::LINE_COUNT);  // this one comes with a new line: must count it
         break;
      case DelphiYaccBeginRecordDef:
         m_fInsideClassOrRecord = true;
         break;
      }
      break;

   case DelphiYaccEndBlock:                     // metricCount -> End 
      if( m_fInsideCase and (m_iCaseBlockLevel == m_iBlockDepth) )
      {
         m_iCaseBlockLevel = 0;
         m_fInsideCase = false;
      }
      if( m_iBlockDepth > 0 )
      {
         --m_iBlockDepth;
      }
      m_fInsideClass = false;
      m_fInsideClassOrRecord = false;
      if( m_iBlockDepth == 0 )
      {
         if( m_fInsideSubroutine )
         {
            // Name string must be parsed further to remove the prefix (usually procedure or function)
            // Do not count subroutine itself as a statement (procedure|function line doen't count)
            m_poSMDelphi->SetSubroutineData(m_sSubroutineName, m_iSubroutineStatements - 1);
            m_fInsideSubroutine = false;
         }
         else if( m_fInsideMethod )
         {
            // Name string must be parsed further to remove the prefix (usually procedure or function) and
            // extract the class.method names
            // Do not count subroutine itself as a statement (procedure|function line doen't count)
            m_poSMDelphi->SetMethodData(m_sClassMethodNames, m_iMethodStatements - 1);
            m_fInsideMethod = false;
         }
      }
      break;

   case DelphiYaccElseStatement:                // metricCount -> Else 

      // else always is preceded by a statement w/o terminating ';' so we need to count it here
      m_poSMDelphi->BumpCount(SMDelphi::STATEMENT_COUNT);
      if( not m_fInsideVarClause )
      {
         if( m_fInsideSubroutine )
         {
            ++m_iSubroutineStatements;
         }
         else if( m_fInsideMethod )
         {
            ++m_iMethodStatements;
         }
      }
   // these all have a trailing ';' so we only bump the branch count, not the statement count
   case DelphiYaccBranch:                       // metricCount -> Branch 
      m_poSMDelphi->BumpCount(SMDelphi::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      m_poSMDelphi->BumpCount(SMDelphi::BRANCH_COUNT);
      break;

   }
   return stackElement();
}





