#include "stdafx.h"

#include "VPJavaClasses.hpp"
#include "SMJava.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

SSYaccStackElement *
JavaYaccClass::reduce(SSUnsigned32 ulProd, SSUnsigned32 /* ulSize */)
{
   static CString sOverloadOpr;

   switch( ulProd )
   {
   // NOTE: many productions are ignored because they help parse but aren't relevant to any metrics
   case JavaYaccError:                       // metricCount -> %error \n
      m_poSMJava->ParseError();              // report error to user, then count current line anyway
   case JavaYaccSourceLine:                  // metricCount -> \n 
   case JavaYaccModifierSourceLine:          // metricCount -> Modifier \n
   case JavaYaccModifierListLine:            // metricCount -> modifierList Modifier \n
      m_poSMJava->BumpCount(SMJava::LINE_COUNT);
      if( m_fInsideComment )
      {
         m_poSMJava->BumpCount(SMJava::COMMENT_LINES);
      }
      break;

   case JavaYaccOperatorStatement:           // metricCount -> ControlStmt 
      m_poSMJava->BumpCount(SMJava::BRANCH_STATEMENTS);
      m_poSMJava->BumpCount(SMJava::STATEMENT_COUNT);
      m_poSMJava->BumpCount(SMJava::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      if( m_fInsideMethod )
      {
         m_poSMJava->BumpCount(SMJava::METHOD_STATEMENTS);
         ++m_iMethodStatements;
      }
      break;

   case JavaYaccExpressionStatement:         // metricCount -> SemiColon 
   case JavaYaccMemberDefinitionExpression:  // metricCount -> modifierList SemiColon 
      m_poSMJava->BumpCount(SMJava::STATEMENT_COUNT);
      m_poSMJava->BumpCount(SMJava::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      if( m_fInsideMethod )
      {
         m_poSMJava->BumpCount(SMJava::METHOD_STATEMENTS);
         ++m_iMethodStatements;
      }
      break;

   case JavaYaccBeginMethodBlock:            // metricCount -> MethodDeclaration 
      if( not m_fInsideMethod )
      {
         m_fInsideMethod = true;
         m_iMethodStatements = 0;
         m_sClassMethodNames.SetAtGrow(m_iBlockDepth, "M:" + m_sCallOrMethodName + _T("...)"));
         m_poSMJava->ConfirmBiggestMethodLine();
         m_poSMJava->BumpCount(SMJava::METHOD_COUNT);
         m_poSMJava->BumpCount(SMJava::STATEMENT_COUNT);
      }
      m_poSMJava->BumpCount(SMJava::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
   case JavaYaccBeginBlock:                  // metricCount -> { 
   case JavaYaccModifierBlock:               // metricCount -> Modifier { 
   case JavaYaccModifierListBlock:           // metricCount -> modifierList Modifier { 
   case JavaYaccClassBlock:                  // metricCount -> BeginClassBlock 
      ++m_iBlockDepth;
      if( m_iBlockDepth > m_iMaxBlockDepth )
      {
         m_iMaxBlockDepth = m_iBlockDepth;
         m_poSMJava->SetDeepestBlockLine();
      }
      break;

   case JavaYaccEndBlock:                    // metricCount -> } 
      if( m_iBlockDepth > 0 )
      {
         --m_iBlockDepth;
      }
      if( (m_sClassMethodNames.GetSize() > m_iBlockDepth)
      and (m_sClassMethodNames[m_iBlockDepth].GetLength() > 0) )
      {
         CString  sClassName;
         CString  sMethodName;

         switch( m_sClassMethodNames[m_iBlockDepth][0] )
         {
         case 'C':
            sClassName = m_sClassMethodNames[m_iBlockDepth].Mid(2);
            sMethodName.LoadString(IDS_JAVA_NO_METHODS);
            m_poSMJava->SetMethodData(sClassName, sMethodName, 0);      // ignored if any "real" methods posted
            m_sClassMethodNames[m_iBlockDepth] = "";
            break;

         case 'M':
            if( m_fInsideMethod and (m_iBlockDepth > 0) )
            {
               CString  sNameCheck;

               if( m_sClassMethodNames[m_iBlockDepth - 1].GetLength() == 0 )
               {
                  // assume this is an inside class inheriting from another class
                  sClassName = m_sClassMethodNames[m_iBlockDepth].Mid(2);
                  m_sClassMethodNames[m_iBlockDepth - 1] = "C:" + sClassName;
               }
               else
               {
                  sNameCheck = m_sClassMethodNames[m_iBlockDepth];
                  if( (sNameCheck.GetLength() > 2) and (sNameCheck.Left(2) = "M:") )
                  {
                     sClassName = m_sClassMethodNames[m_iBlockDepth - 1].Mid(2);
                     sMethodName = sNameCheck.Mid(2);
                     m_poSMJava->SetMethodData(sClassName, sMethodName, m_iMethodStatements);
                  }
               }
               m_fInsideMethod = false;
               m_sClassMethodNames[m_iBlockDepth] = "";
            }
            break;

         case 'I':
            sClassName = m_sClassMethodNames[m_iBlockDepth].Mid(2);
            sMethodName.LoadString(IDS_JAVA_INTERFACE_BIGGEST);
            m_iMethodStatements = 0;
            m_poSMJava->SetMethodData(sClassName, sMethodName, m_iMethodStatements);
            m_fInsideInterface = false;
            m_sClassMethodNames[m_iBlockDepth] = "";
            break;
         }
      }
      break;

   case JavaYaccBeginComment:                // metricCount -> BeginMLComment 
   case JavaYaccBeginModifierComment:        // metricCount -> Modifier BeginMLComment
   case JavaYaccBeginModifierListComment:    // metricCount -> modifierList Modifier BeginMLComment
      m_fInsideComment = true;
      break;

   case JavaYaccEndComment:                  // metricCount -> EndMLComment 
      m_poSMJava->BumpCount(SMJava::COMMENT_LINES);
      m_fInsideComment = false;
      break;

   case JavaYaccCppComment:                  // metricCount -> BeginSLComment 
   case JavaYaccCppModifierComment:          // metricCount -> Modifier BeginSLComment
   case JavaYaccCppModifierListComment:      // metricCount -> modifierList Modifier BeginSLComment
      m_poSMJava->BumpCount(SMJava::COMMENT_LINES);
      break;

   case JavaYaccClassUnmodified:             // metricCount -> ClassName 
   case JavaYaccClassWithModifier:           // metricCount -> modifierList ClassName 
      {
         int      iIndex = 1;          // initialize for modifier (used for name retrieval and keyword extraction)
         CString  sClassName;
         CString  sTypePrefix;

         if( ulProd == JavaYaccClassUnmodified )
         {
            iIndex = 0;
         }
         sClassName = ((elementFromProduction(iIndex))->lexeme())->asConstChar();

         // chop keyword "class" or "interface" and following white space from class name string
         if( sClassName[0] == 'c' )
         {
            iIndex = 6;
            sTypePrefix = "C:";
         }
         else if( sClassName[0] == 'i' )
         {
            iIndex = 10;
            sTypePrefix = "I:";
            m_fInsideInterface = true;
         }
         sClassName = sClassName.Mid(iIndex);
         sClassName.TrimLeft();
         m_sClassMethodNames.SetAtGrow(m_iBlockDepth, sTypePrefix + sClassName);
         m_poSMJava->BumpCount(SMJava::CLASS_COUNT);
         m_poSMJava->BumpCount(SMJava::STATEMENT_COUNT);
         m_poSMJava->BumpCount(SMJava::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
      }
      break;

   case JavaYaccClassMethod0:                // metricCount -> MethodName 
   case JavaYaccClassMethod1:                // metricCount -> modifierList MethodName 
      {
         if( m_fInsideInterface )
         {
            m_poSMJava->BumpCount(SMJava::METHOD_COUNT);       // interfaces only have abstract method declarations
         }
         else
         {
            int   iIndex = 1;          // initialize for modifier (used for name retrieval and keyword extraction)
            int   iNewLineCount;

            if( ulProd == JavaYaccClassMethod0 )
            {
               iIndex = 0;
            }
            m_sCallOrMethodName = ((elementFromProduction(iIndex))->lexeme())->asConstChar();
            iNewLineCount = m_sCallOrMethodName.Replace("\n", NULL);
            if( iNewLineCount > 0 )
            {
               for( int i = 0; i < iNewLineCount; i++ )
               {
                  m_poSMJava->BumpCount(SMJava::LINE_COUNT);   // have embedded '\n' between method name and '('
               }
            }
            m_sCallOrMethodName.Replace(" ", NULL);
            m_sCallOrMethodName.Replace("\t", NULL);
            m_poSMJava->SetBiggestMethodLine();
            m_poSMJava->BumpCount(SMJava::CALL_COUNT);
         }
      }
      break;
   }
   return stackElement();
}



















