#include "stdafx.h"

#include "VPVBClasses.hpp"
#include "SMVisualBasic.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

SSYaccStackElement *
VBYaccClass::reduce( SSUnsigned32 ulProd, SSUnsigned32 /* ulSize */)
{
   switch ( ulProd)
   {
   // NOTE: many productions are ignored because they help parse but aren't relevant to any metrics
   case VBYaccError:                      // metricCount -> %error \n
      m_poSMVisualBasic->ParseError();    // report error to user, then count current line anyway
   case VBYaccSourceLine:                 // metricCount -> \n 
      m_poSMVisualBasic->BumpCount(SMVisualBasic::LINE_COUNT);
      if( m_fInsideStatement )
      {
         m_fInsideStatement = false;
         m_fStartingIf = false;
         m_poSMVisualBasic->BumpCount(SMVisualBasic::STATEMENT_COUNT);
         m_poSMVisualBasic->BumpCount(SMVisualBasic::BLOCK_DEPTH_0 + min(m_iBlockDepth, m_iMaxDepth));
         if( m_fStartingBranchBlock )     // counted If, Loop, etc at current level, now bump for inside statements
         {
            ++m_iBlockDepth;
            if( m_iBlockDepth > m_iMaxBlockDepth )
            {
               m_iMaxBlockDepth = m_iBlockDepth;
               m_poSMVisualBasic->SetDeepestBlockLine();
            }
            m_fStartingBranchBlock = false;
         }
         if( m_fInsideSubroutine )
         {
            ++m_iSubroutineStatements;
         }
         if( m_fStartingSubroutine )
         {
            m_fInsideSubroutine = true;
            m_fStartingSubroutine = false;
         }
      }
      break;

   case VBYaccComment:                    // metricCount -> Comment 
      m_poSMVisualBasic->BumpCount(SMVisualBasic::COMMENT_LINES);
      break;

   case VBYaccName:                       // metricCount -> Name
      m_fInsideStatement = true;
      if( m_fBeginSubroutine )            // first name after start of outer subroutine = sub's name
      {
         m_fBeginSubroutine = false;
         m_fStartingSubroutine = true;    // start counting lines on next statement
         m_sSubroutineName = ((elementFromProduction(0))->lexeme())->asConstChar();
         m_poSMVisualBasic->SetBiggestMethodLine();
      }
      if( m_fStartingIf )
      {
         m_fStartingBranchBlock = false;  // single line "If" statement
      }
      break;

   case VBYaccSameLineStatement:          // metricCount -> : 
      if( m_fInsideStatement )
      {
         m_poSMVisualBasic->BumpCount(SMVisualBasic::STATEMENT_COUNT);
         if( m_fInsideSubroutine )
         {
            ++m_iSubroutineStatements;
         }
         m_fInsideStatement = false;      // make sure right side really has a statement
      }
      break;

   case VBYaccBeginSub:                   // metricCount -> Sub
   case VBYaccBeginFunction:              // metricCount -> Function 
   case VBYaccBeginProperty:              // metricCount -> Property 
      m_fInsideStatement = true;
      if( (m_iBlockDepth == 0) and not m_fInsideSubroutine )
      {
         m_iSubroutineStatements = 0;
         m_fBeginSubroutine = true;       // the next name is our subroutine's
      }
      m_poSMVisualBasic->BumpCount(SMVisualBasic::SUBROUTINE_COUNT);
      m_fStartingBranchBlock = true;      // bump level after counting this statement
      break;

   case VBYaccEndSub:                     // metricCount -> End Sub 
   case VBYaccEndFunction:                // metricCount -> End Function 
   case VBYaccEndProperty:                // metricCount -> End Property 
      m_fInsideStatement = true;
      if( m_iBlockDepth > 0 )
      {
         --m_iBlockDepth;
      }
      if( m_fInsideSubroutine and (m_iBlockDepth == 0) )
      {
         m_fInsideSubroutine = false;
         m_poSMVisualBasic->SetMethodData(m_sSubroutineName, m_iSubroutineStatements);
      }
      break;

   // controls that begin blocks
   case VBYaccBeginBlock:                 // metricCount -> Begin 
   case VBYaccBeginType:                  // metricCount -> Type 
      m_fInsideStatement = true;
      m_fStartingBranchBlock = true;
      break;

   // branch controls that also start blocks
   case VBYaccSelectCase:                 // metricCount -> Select Case 
      m_fStartingCase = true;
   case VBYaccDoLoop:                     // metricCount -> Do 
   case VBYaccDoWhile:                    // metricCount -> Do While 
   case VBYaccDoUntil:                    // metricCount -> Do Until 
   case VBYaccFor:                        // metricCount -> For 
      if( not m_fInsideStatement )        // only when this is the first word on a line is it a branch control
      {
         m_poSMVisualBasic->BumpCount(SMVisualBasic::BRANCH_COUNT);
         m_fStartingBranchBlock = true;
      }
      m_fInsideStatement = true;
      break;

   // controls that end blocks
   case VBYaccEndSelect:                  // metricCount -> End Select 
      if( not m_fStartingCase )           // if we've processed at least one Case statement
      {
         if( m_iBlockDepth > 0 )          // must go back two levels, not one
         {
            --m_iBlockDepth;
         }
      }
   case VBYaccEndBlock:                   // metricCount -> End 
   case VBYaccEndType:                    // metricCount -> End Type 
   case VBYaccEndIf:                      // metricCount -> End If 
   case VBYaccLoop:                       // metricCount -> Loop 
   case VBYaccLoopWhile:                  // metricCount -> Loop While
   case VBYaccNext:                       // metricCount -> Next 
      if( not m_fInsideStatement )        // only when this is the first word on a line is it a branch control
      {
         if( m_iBlockDepth > 0 )
         {
            --m_iBlockDepth;
         }
      }
      m_fInsideStatement = true;
      break;

   case VBYaccBeginIfThenBlock:           // metricCount -> Then 
      m_fStartingBranchBlock = true;
      m_fStartingIf = true;
      break;

   // these control keywords both end and begin a block
   case VBYaccElseIfStatement:            // metricCount -> ElseIf 
      m_poSMVisualBasic->BumpCount(SMVisualBasic::BRANCH_COUNT);
   case VBYaccElseStatement:              // metricCount -> Else 
   case VBYaccCaseElse:                   // metricCount -> Case Else 
      m_fInsideStatement = true;
      if( m_iBlockDepth > 0 )
      {
         --m_iBlockDepth;
      }
      m_fStartingBranchBlock = true;      // count at current level, then bump to next level
      break;

   case VBYaccCase:                       // metricCount -> Case 
      m_fInsideStatement = true;
      if( m_fStartingCase )               // don't bump level for first Case in Select Case statement
      {
         m_fStartingCase = false;
      }
      else if( m_iBlockDepth > 0 )
      {
         --m_iBlockDepth;
      }
      m_fStartingBranchBlock = true;      // count at current level, then bump to next level
      break;

   // these control keywords are just counted ("If" is handled by "Then" keyword)
   case VBYaccExitStatement:              // metricCount -> ExitStatement 
      m_fInsideStatement = true;
      if( m_fStartingIf )
      {
         m_fStartingBranchBlock = false;  // single line "If" statement
      }
      break;

   case VBYaccBeginIf:                    // metricCount -> If 
      m_fInsideStatement = true;
      m_poSMVisualBasic->BumpCount(SMVisualBasic::BRANCH_COUNT);
      break;

   case VBYaccBranch:						// metricCount -> GoTo, GoSub or Resume
      m_poSMVisualBasic->BumpCount(SMVisualBasic::BRANCH_COUNT);
      break;
   }
   return stackElement();
}










