//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
/////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// The text comments in this file are formatted to generate
// a help file on build using Microsofts free Autoduck tool
// The help file is available with this source and it is
// reccomended that you use this file in conjunction with the
// mfc help files
/////////////////////////////////////////////////////////////

//@doc
//@module SRGraphComponent.cpp | class implementation  of the SRGraphComponent class<nl>
//Copyright <cp> Stingray Software 1996.<nl>
//Written by Bob Powell (bob@stingsoft.com)


#ifdef _OBJCHART_DLL
#undef AFXAPI_DATA
#define AFXAPI_DATA __based(__segname("_DATA"))
#endif

#include "stdafx.h"
#include <math.h>
#include <float.h>
#include <stdlib.h>
#include <search.h>


static UINT LineStyleArray[4]=
{	PS_SOLID,
	PS_DASH,
	PS_DOT,
	PS_DASHDOT
};

static UINT HatchStyleArray[8]=
{	HS_BDIAGONAL,	//Downward hatch (left to right) at 45 degrees
	HS_CROSS,		//Horizontal and vertical crosshatch
	HS_DIAGCROSS,	//Crosshatch at 45 degrees
	HS_FDIAGONAL,	//Upward hatch (left to right) at 45 degrees
	HS_HORIZONTAL,	//Horizontal hatch
	HS_VERTICAL,	//Vertical hatch
	HS_BDIAGONAL,	//to fill up the array to a nice binary number
	HS_CROSS
};

static COLORREF DataColorArray[16]=
{	CXCLR_RED,
	CXCLR_GREEN,
	CXCLR_BLUE,
	CXCLR_YELLOW,
	CXCLR_CYAN,
	CXCLR_MAGENTA,
	CXCLR_MAROON,
	CXCLR_OLIVE,
	CXCLR_MIDNIGHT,
	CXCLR_PURPLE,
	CXCLR_BEIGE,
	CXCLR_SEAGREEN,
	CXCLR_VIOLET,
	CXCLR_SKY,
	CXCLR_CORAL,
	CXCLR_LTGREEN
};


IMPLEMENT_SERIAL(SRGraphComponent,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphRect,CObject,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphTitle,SRGraphComponent,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphBackground,SRGraphComponent,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphLegend,SRGraphTitle,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphDisplay,SRGraphTitle,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphLabel,SRGraphTitle,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION);
IMPLEMENT_SERIAL(SRGraphLabelBlock,SRGraphLabel,VERSIONABLE_SCHEMA | _OCHART_FILE_VERSION)



//@mfunc Used to calculate a SRGraphRect from a screen CRect given the measurement value stored in m_nType. 
//This is primarily used to recalculate sizes of components that have an auto size feature <c SRGraphLegend> 
//falls into this category.
//@parm The CRect to be used in the conversion
//it simply converts coordinates based on the measurement used in the SRGraphRect.
//@devnote Be aware that this routine will give results based upon the m_nType member for the SRGraphRect object used.<nl> 
//<nl>
//@ex This example Illustrates this. |
//
//	/* CDC->GetDeviceCaps reports 120 pixels per inch for X and Y */
//
//	SRGraphRect A,B,ResultA,ResultB;
//	CRect r(100,100,200,200);
//
//	A.SetMeasurement(0);
//	B.SetMeasurement(1);
//	
//	ResultA=A.CRectToSRGRect(r);
//	ResultB=B.CRectToSRGRect(r);
//
//	results....
//	ResultA.. left=100 top=100 right=200 bottom=200
//	ResultB.. left=21.16666666667 top=21.16666666667
//            right=42.33333333333 bottom=42.33333333333

void SRGraphRect::CRectToSRGRect(CRect& rect,CDC *pDC)
{
	int xRes=pDC->GetDeviceCaps(LOGPIXELSX);
	int yRes=pDC->GetDeviceCaps(LOGPIXELSY);
	double l,t,r,b;

	switch(m_nType)
	{
	default:
	case 0:
		l=(double)rect.left; // no conversion needed
		r=(double)rect.right;
		t=(double)rect.top;
		b=(double)rect.bottom;
		break;
	case 1:
		l=(double)rect.left/(((double)xRes)/25.4);
		t=(double)rect.top/(((double)yRes)/25.4);
		r=(double)rect.right/(((double)xRes)/25.4);
		b=(double)rect.bottom/(((double)yRes)/25.4);
		break;
	case 2:
		l=(double)rect.left/xRes;
		t=(double)rect.top/yRes;
		r=(double)rect.right/xRes;
		b=(double)rect.bottom/yRes;
		break;
	}
	left=l;
	top=t;
	right=r;
	bottom=b;
}

//@mfunc Overloaded for to provide CRect to SRGraphRect support
// for the percentage measurement system.
//@parm The rectangle from which to derive the new measurements.
//@parm The CDC which is being used.
//@parm The CWnd pointer from which client size is taken.
//@see <SRGraphRect::CRectToSRGRect>
void SRGraphRect::CRectToSRGRect(CRect& rect,CDC *pDC,CWnd *pCWnd)
{
	int xRes=pDC->GetDeviceCaps(LOGPIXELSX);
	int yRes=pDC->GetDeviceCaps(LOGPIXELSY);
	double l,t,r,b;

	CRect sr;

	pCWnd->GetClientRect(&sr);

	switch(m_nType)
	{
	default:
	case 0:
	case 1:
	case 2:
		CRectToSRGRect(rect,pDC);
		break;
	case 3:
		l=((double)rect.left/sr.Width())*100.0;
		t=((double)rect.top/sr.Height())*100.0;
		r=((double)rect.right/sr.Width())*100.0;
		b=((double)rect.bottom/sr.Height())*100.0;
		break;
	}
}

//@mfunc Initialises the m_nType member.
//@parm The measurement standard to use<nl>
//0=PIXELS<nl>
//1=MILLIMETERS<nl>
//2=INCHES<nl>
//@see <mf SRGraphRect::GetMeasurement> <md SRGraphRect::m_nType>
void SRGraphRect::SetMeasurement(int m)
{
	m_nType=m;
}

//@mfunc Interrogates the measurement type for this SRGraphRect object.
//@rdesc The contents of the m_nType data member
//@see <mf SRGraphRect::SetMeasurement> <md SRGraphRect::m_nType>
int SRGraphRect::GetMeasurement()
{
	return m_nType;
}


//@mfunc Similar in function to CRect::InflateRect but uses floats on a device independant rectangle.
//@parm Amount to grow in width.
//@parm Amount to grow in height.
void SRGraphRect::Grow(double x, double y)
{
	left-=x/2;
	right+=x/2;
	top-=y/2;
	bottom+=y/2;
}

//@mfunc Constructs and initialises an SRGraphRect.
//@parm reference to a CRect used to initialise this rectangle
SRGraphRect::SRGraphRect(CRect& r)
{
	left=(double)r.left;
	right=(double)r.right;
	top=(double)r.top;
	bottom=(double)r.bottom;
}

//@mfunc Calculates a rectangle in pixels given a device independant rectangle and access to the device capabilities.
//@parm pointer to the DC in use
//@parm pointer to the CWnd in use
//@rdesc A CRect calculated according to pixels, inches or millimeters taking into account the declared pixels per inch of the device context.
//@devnote New Behaviour.. Any occurrence of -1 in the to,left,bottom or right device independant coordinates will result in the corresponding
// calculated rectangle being set to the maximum limit for that "side" of the rectangle.<nl>
//For example calling SetRect(-1,200,-1,-1) will make a component cover all but the top 200 pixels of a window or printer page.
CRect SRGraphRect::CalcCRect(CDC *pDC,CWnd *pCWnd)
{
	int xRes=pDC->GetDeviceCaps(LOGPIXELSX);
	int yRes=pDC->GetDeviceCaps(LOGPIXELSY);
	CRect r;

	if(pDC->IsPrinting())
		r=CRect(0,0,pDC->GetDeviceCaps(HORZRES),pDC->GetDeviceCaps(VERTRES));
	else
	{
		CBitmap *pBM=pDC->GetCurrentBitmap();
		ASSERT(pBM!=NULL);
		BITMAP bmDetails;
		pBM->GetBitmap(&bmDetails);

		if(bmDetails.bmPlanes==1 && bmDetails.bmBitsPixel==1 && bmDetails.bmWidth==1)
			pCWnd->GetClientRect(&r); // its a default bitmap. so the direct window size is needed
		else
			r=CRect(0,0,bmDetails.bmWidth,bmDetails.bmHeight);
	}


	switch(m_nType)
	{
	default:
	case 0:
		//pixels
		m_LogRect.SetRect(	(int)left,
							(int)top,
							(int)right,
							(int)bottom);
		break;
	case 1:
		//mm
		m_LogRect.SetRect(	(int)(left*(xRes/25.4f)),
							(int)(top*(yRes/25.4f)),
							(int)(right*(xRes/25.4f)),
							(int)(bottom*(yRes/25.4f)));
		break;
	case 2:
		//inches
		m_LogRect.SetRect(	(int)(left*xRes),
							(int)(top*yRes),
							(int)(right*xRes),
							(int)(bottom*yRes));
		break;
	case 3:
		//Percentage
		m_LogRect.SetRect(	(int)(left*(r.Width()/100.0)),
							(int)(top*(r.Height()/100.0)),
							(int)(right*(r.Width()/100.0)),
							(int)(bottom*(r.Height()/100.0)));
	}

	if(top==-1)
		m_LogRect.top=r.top;

	if(left==-1)
		m_LogRect.left=r.left;

	if(right==-1)
		m_LogRect.right=r.right;

	if(bottom==-1)
		m_LogRect.bottom=r.bottom;
	
	return m_LogRect;
}

//@mfunc Initialises an SRGraphRect
//@parm device independant coordinate for the left of the rectangle
//@parm device independant coordinate for the top of the rectangle
//@parm device independant coordinate for the right of the rectangle
//@parm device independant coordinate for the bottom of the rectangle
void SRGraphRect::SetRect(double l,double t,double r, double b)
{
	left=l; top=t; right=r; bottom=b;
}

//@mfunc Initialises an SRGraphRect
//@parm Reference to a CRect used to size and position the rectangle
void SRGraphRect::SetRect(const CRect& r)
{
	left=(double)r.left;
	right=(double)r.right;
	top=(double)r.top;
	bottom=(double)r.bottom;
}


//@mfunc Saves or loads an SRGraphRect
//@parm Reference to a CArchive object used to store or retrieve the data
void SRGraphRect::Serialize(CArchive& ar)
{
	CObject::Serialize(ar);

	if(ar.IsStoring())
	{
		ar << left << top << right << bottom << m_nType;
	}
	else
	{
		ar >> left >> top >> right >> bottom >> m_nType;
	}
}

///////////////////////////////////////////////////////////////////////////////
// SRGraphComponent member functions

//@mfunc Constructor
//@rem m_nMaxSet various defaults<nl>
	//m_pParentGraph=NULL<nl>
	//m_Style Type (in m_dwStyleA)=TYPE_COMPONENT<nl>
	//m_nType=IDS_SRG_COMPONENTTYPE<nl>
SRGraphComponent::SRGraphComponent()
{
	m_pParentGraph=NULL;
	m_Style.SetType(CX_TYPE_COMPONENT);
	m_nType=IDS_SRG_COMPONENTTYPE;
	m_dMarginX=m_dMarginY=5;
	m_dTickSize=5;
	m_nMaxGroup=m_nMinGroup=m_nMaxIndex=m_nMinIndex=-1;
	m_bHiLo=0;
	m_pDC=NULL;
	m_pCWnd=NULL;
	m_bDrawing=FALSE;
	m_nCountryCode=1;
	m_bSignificantChange=TRUE; // Well, Creation's pretty significant.
}

//@mfunc Destructor
SRGraphComponent::~SRGraphComponent()
{
	while(!m_PolygonList.IsEmpty())
		delete (m_PolygonList.RemoveHead());
}

//@mfunc Initialises the m_Style member. The m_Style member contains codes for panel fill style, border style and shadow style.
//@parm The 32 bit component style data.<nl>
//Settings are;<nl>
//	CX_NO_BORDER			0x00000000<nl>
//	CX_THIN_RAISED_BORDER		0x00000001<nl>
//	CX_THIN_SUNKEN_BORDER		0x00000002<nl>
//	CX_THIN_3D_BORDER		0x00000003<nl>
//	CX_THICK_RAISED_BORDER	0x00000004<nl>
//	CX_THICK_SUNKEN_BORDER	0x00000005<nl>
//	CX_THICK_3D_BORDER		0x00000006<nl>
//	CX_LINE_FRAME			0x00000007<nl>
//<nl>
//	CX_THIN_SHADOW			0x00000010<nl>
//	CX_MEDIUM_SHADOW		0x00000020<nl>
//	CX_THICK_SHADOW			0x00000030<nl>
//<nl>
//	CX_NO_FILL				0x00000000<nl>
//	CX_SOLID_FILL			0x00000100<nl>
//	CX_VERTICAL_FILL			0x00000200<nl>
//	CX_HORIZONTAL_FILL		0x00000300<nl>
//	CX_ANGLE_FILL			0x00000400<nl>
//	CX_RESOURCE_FILL			0x00000500<nl>
//	CX_IMAGE_FILL			0x00000600<nl>
//<nl>
//NOTE These settings may be combined using a bitwise OR function but it is highly recommended that you use the 
//<mf SRGraphStyle::SetComponentBorderStyle> <mf SRGraphStyle::SetComponentShadowStyle> and <mf SRGraphStyle::SetComponentFillStyle> member functions
void SRGraphComponent::SetStyle(DWORD style)
{
	m_Style.SetComponentStyle(style);
}

//@mfunc Retrieves a pointer to the m_Style data member.
//@rdesc Pointer to m_Style
SRGraphStyle *SRGraphComponent::GetStyle()
{
	return &m_Style;
}

//@mfunc Called from the <mf SRGraphComponent::Draw> function to draw a shadow for this component.
void SRGraphComponent::DrawShadow()
{
	CBrush b,*OldBrush;
	CPen *OldPen;
	CRect r,s;
	CBitmap bm;

	if((m_Style.GetComponentShadowStyle())==CX_NO_SHADOW)
		return;

	r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
	int thickness;
	switch(m_Style.GetComponentShadowStyle())
	{
	case CX_THIN_SHADOW:
		thickness=3;
		break;
	case CX_MEDIUM_SHADOW:
		thickness=8;
		break;
	case CX_THICK_SHADOW:
		thickness=16;
		break;
	default:
		DrawCustomShadow();
		return;
	}

	bm.LoadBitmap(SRG_IDB_SHADOW);
	b.CreatePatternBrush(&bm);

	OldBrush=(CBrush *)GetpDC()->SelectObject(&b);
	OldPen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);
	// Rectangle has a nasty habit.. It wont draw the last line of pixels on the right or bottom
	// of an area. So Add 1 to the width and height to overcome this.
	int OldMode=GetpDC()->SetROP2(R2_MASKPEN);
	GetpDC()->Rectangle(r.right,r.top+thickness,1+r.right+thickness,1+r.bottom+thickness);
	GetpDC()->Rectangle(r.left+thickness,r.bottom,1+r.right,1+r.bottom+thickness);
	GetpDC()->SetROP2(OldMode);
	GetpDC()->SelectObject(OldPen);
	GetpDC()->SelectObject(OldBrush);
	b.DeleteObject();
	bm.DeleteObject();
	
}

void SRGraphComponent::DrawCustomShadow()
{
	ASSERT(FALSE); // Override this function to draw your custom shadows
}

//@mfunc Set one or both RGB color values.
//@parm first color
//@parm Second color (default RGB(0,0,0))
void SRGraphComponent::SetColor(COLORREF a,COLORREF b)
{
	m_Style.SetColor(a,b);
}

//@mfunc Initialises the angle for background shading
//@parm Angle in degrees.
void SRGraphComponent::SetAngle(int angle)
{
	m_Style.SetAngle((double)angle);
}

//@mfunc Initialises the resource name for bitmap resources used in background filling. 
//Remember to use the "IDB_NAME_IN_QUOTES" form because the numeric ID will not be available after compile time and this component may be initialised by a template command.
//@parm Zero terminated string containing the name of the resource.
void SRGraphComponent::SetResourceName(LPCSTR name)
{
	m_Style.SetResourceName(name);
}

//@mfunc Initialise the pointer to the SRGraph object that owns this component. Used by the 
//<mf SRGraphComponent::GetParentContent> member.
void SRGraphComponent::SetParent(SRGraph *pParent)
{
	m_pParentGraph=pParent;
	SetCountryCode(-1);
}


//@mfunc Override this function to retrieve pertinent data from the parent 
//Base class ASSERTS because Bare components have no content handling ability.
//@devnote The SRGraph object holds all the data so components have a pointer to the graph which owns them. 
//This allows the GetParentContent routine to fetch data pertinent to the type of component. SRGraphTitle fetches 
//title text, SRGraphLegend retrieves SRGraphDataList headers etc. etc.
void SRGraphComponent::GetParentContent()
{
	ASSERT(FALSE);
}

//@mfunc Called by the <mf SRGraph::DrawComponentList> member 
//which is called by the CWnd that manages drawing of this graph. 
//@parm pointer to the CDC used in drawing the CWnd.
//@parm pointer to the CWnd object
//@devnote Components draw themselves onto the client area of a window in response to 
//calls by the OnPaint or OnDraw functions of the window. The window pointer and CDC 
//pointer are passed at the time of the paint request and the Draw function then takes 
//over drawing fill, border, shadow and foreground in turn.
//This method of drawing allows you to build SRGraph functionality into any 
//window including dialog box controls and child windows as well as the classic CDoc CView 
//combinations.<nl>
void SRGraphComponent::Draw(CDC *pDC,CWnd *pWnd)
{
	m_bDrawing=TRUE;
	m_pDC=pDC;
	m_pCWnd=pWnd;
	DrawFill();
	DrawShadow();
	DrawBorder();
	DrawForeground();
	m_bDrawing=FALSE;
	m_bSignificantChange=FALSE;
}

//@mfunc Called by the <mf SRGraphComponent::Draw> function to fill the area covered by this component with 
//a color shade or bitmap pattern.
void SRGraphComponent::DrawFill()
{
	CRect r;

	r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
	
	CPoint points[5];
	points[0]=CPoint(r.left,r.top);
	points[1]=CPoint(r.right,r.top);
	points[2]=CPoint(r.right,r.bottom);
	points[3]=CPoint(r.left,r.bottom);
	points[4]=points[0];

	FillPoly((LPPOINT)&points,5,&m_Style,0);
}

	

//@mfunc called by the <mf SRGraphComponent::Draw> function to provide a border
//around the component.
void SRGraphComponent::DrawBorder()
{
	CBrush b,*OldBrush;
	CPen p,*OldPen;
	CRect r;
	BOOL raised;
	int thickness,c;

	switch(m_Style.GetComponentBorderStyle())
	{
	case CX_NO_BORDER:
		return;
	default:
		DrawCustomBorder();
		break;
	case CX_LINE_BORDER:
		r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
		p.CreatePen(PS_SOLID,1,m_Style.GetFrameColor());
		OldPen=(CPen *)GetpDC()->SelectObject(&p);
		OldBrush=(CBrush *)GetpDC()->SelectStockObject(NULL_BRUSH);
		GetpDC()->Rectangle(r);
		GetpDC()->SelectObject(OldPen);
		GetpDC()->SelectObject(OldBrush);
		break;
	case CX_THIN_RAISED_BORDER:
		raised=TRUE;
thin_border:
		r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
		OldBrush=(CBrush *)GetpDC()->SelectStockObject(NULL_BRUSH);
		OldPen=(CPen *)GetpDC()->SelectStockObject(raised ? WHITE_PEN : BLACK_PEN);
		GetpDC()->MoveTo(r.left,r.bottom);
		GetpDC()->LineTo(r.left,r.top);
		GetpDC()->LineTo(r.right,r.top);
		GetpDC()->SelectStockObject(raised ? BLACK_PEN : WHITE_PEN);
		GetpDC()->MoveTo(r.right,r.top);
		GetpDC()->LineTo(r.right,r.bottom);
		GetpDC()->LineTo(r.left,r.bottom);
		p.CreatePen(PS_SOLID,1,RGB(128,128,128));
		GetpDC()->SelectObject(OldPen);
		GetpDC()->SelectObject(OldBrush);
		p.DeleteObject(); 
		return;
	case CX_THIN_SUNKEN_BORDER:
		raised=FALSE;
		goto thin_border;
	case CX_THICK_RAISED_BORDER:
		raised=TRUE;
thick_border:
		r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
		OldBrush=(CBrush *)GetpDC()->SelectStockObject(NULL_BRUSH);
		OldPen=(CPen *)GetpDC()->SelectStockObject(raised ? WHITE_PEN : BLACK_PEN);
		for(c=0;c<3;c++)
		{
			GetpDC()->SelectStockObject(raised ? WHITE_PEN : BLACK_PEN);
			GetpDC()->MoveTo(r.left+c,r.bottom-c);
			GetpDC()->LineTo(r.left+c,r.top+c);
			GetpDC()->LineTo(r.right-c,r.top+c);
			GetpDC()->SelectStockObject(raised ? BLACK_PEN : WHITE_PEN);
			GetpDC()->MoveTo(r.right-c,r.top+c);
			GetpDC()->LineTo(r.right-c,r.bottom-c);
			GetpDC()->LineTo(r.left+c,r.bottom-c);
		}
		p.CreatePen(PS_SOLID,1,RGB(128,128,128));
		GetpDC()->SelectObject(OldPen);
		GetpDC()->SelectObject(OldBrush);
		p.DeleteObject();
		return;
	case CX_THICK_SUNKEN_BORDER:
		raised=FALSE;
		goto thick_border;
	case CX_THIN_3D_BORDER:
		raised=TRUE;
		thickness=1;
		goto _3D_border;
	case CX_THICK_3D_BORDER:
		raised=TRUE;
		thickness=3;
_3D_border:
		r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
		OldBrush=(CBrush *)GetpDC()->SelectStockObject(NULL_BRUSH);
		OldPen=(CPen *)GetpDC()->SelectStockObject(raised ? WHITE_PEN : BLACK_PEN);
		p.CreatePen(PS_SOLID,1,RGB(128,128,128));
		CPen lgp(PS_SOLID,1,RGB(192,192,192));
		for(c=0;c<thickness;c++)
		{
			GetpDC()->SelectStockObject(raised ? WHITE_PEN : BLACK_PEN);
			GetpDC()->MoveTo(r.left+c,r.bottom-c);
			GetpDC()->LineTo(r.left+c,r.top+c);
			GetpDC()->LineTo(r.right-c,r.top+c);
			GetpDC()->SelectStockObject(raised ? BLACK_PEN : WHITE_PEN);
			GetpDC()->MoveTo(r.right-c,r.top+c);
			GetpDC()->LineTo(r.right-c,r.bottom-c);
			GetpDC()->LineTo(r.left+c,r.bottom-c);
			GetpDC()->SelectObject(&p);
			GetpDC()->SelectObject(&lgp);
			GetpDC()->MoveTo(r.left+(c+thickness),r.bottom-(c+thickness));
			GetpDC()->LineTo(r.left+(c+thickness),r.top+(c+thickness));
			GetpDC()->LineTo(r.right-(c+thickness),r.top+(c+thickness));
			GetpDC()->SelectObject(&lgp);
			GetpDC()->SelectObject(&p);
			GetpDC()->MoveTo(r.right-(c+thickness),r.top+(c+thickness));
			GetpDC()->LineTo(r.right-(c+thickness),r.bottom-(c+thickness));
			GetpDC()->LineTo(r.left+(c+thickness),r.bottom-(c+thickness));
		}
		GetpDC()->SelectObject(OldPen);
		GetpDC()->SelectObject(OldBrush);
		p.DeleteObject();
		lgp.DeleteObject();
		return;
	}
}

//@mfunc A Chart extension function to draw component borders.
void SRGraphComponent::DrawCustomBorder()
{
	ASSERT(FALSE); // You must override this function to draw your custom border
}

//@mfunc Initialises the rectangle describing the area covered by this component. if all values are -1 then the rectangle is expanded to 
//cover the client rectangle of the CWnd that draws this component.
//@parm reference to a CRect containing the rectangle dimensions.
void SRGraphComponent::SetRect(const CRect& r)
{
	m_Rect.SetRect(r);
}

//@mfunc Initialises the rectangle describing the area covered by this component. if all values are -1 then the rectangle is expanded to 
//cover the client rectangle of the CWnd that draws this component.
//@parm Left coordinate
//@parm Top coordinate
//@parm Right coordinate
//@parm Bottom coordinate
void SRGraphComponent::SetRect(double l, double t, double r, double b)
{
	m_Rect.SetRect(l,t,r,b);
}
 //@mfunc Initialises the rectangle describing the area covered by this component. if all values are -1 then the rectangle is expanded to 
//cover the client rectangle of the CWnd that draws this component.
//@parm reference to a <c SRGraphRect> object containing the rectangle dimensions
void SRGraphComponent::SetRect(const SRGraphRect& r)
{
	m_Rect=r;
}

//@mfunc Returns A pointer to the m_Rect data member 
SRGraphRect* SRGraphComponent::GetRect()
{
	return &m_Rect;
}

//@mfunc Initialises the measurement type for the rectangle describing this component.
//@parm the measurement type<nl>
//0=PIXELS<nl>
//1=MILLIMETERS<nl>
//2=INCHES
//3=PERCENTAGE of client area
void SRGraphComponent::SetMeasurement(int m)
{
	m_Rect.m_nType=m;
}


//@mfunc Takes actions according to hints supplied to the CWnd object into which this component is drawn
//@parm Pointer to the owner CWnd object.
void SRGraphComponent::ProcessHint(CWnd *pCWnd)
{
	m_pCWnd=pCWnd;
	m_pDC=m_pCWnd->GetDC(); // we need this not for drawing but for the system metrics
	CRect r,s;

	switch(GetParent()->m_lHint)
	{
	case COMPONENT_BORDER:
		// The border size is the same as the background
		// so drop through
	case COMPONENT_BACKGROUND:
		// invalidate the rect covered by the background
		r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
		m_pCWnd->InvalidateRect(r);
		break;
	case COMPONENT_FOREGROUND:
		// do nothing probably handled by the derived class which has
		// bumped this call down to the base class 
		break;
	case COMPONENT_SHADOW:
		// calculate the shadow rects and invalidate
		if((m_Style.GetComponentShadowStyle())==CX_NO_SHADOW)
			break;
		r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
		int thickness;
		switch(m_Style.GetComponentShadowStyle())
		{
		case CX_THIN_SHADOW:
			thickness=3;
			break;
		case CX_MEDIUM_SHADOW:
			thickness=8;
			break;
		case CX_THICK_SHADOW:
			thickness=16;
			break;
		}
		m_pCWnd->InvalidateRect(CRect(r.right,r.top+thickness,1+r.right+thickness,1+r.bottom+thickness));
		m_pCWnd->InvalidateRect(CRect(r.left+thickness,r.bottom,1+r.right,1+r.bottom+thickness));
		break;
	}

	m_pCWnd->ReleaseDC(m_pDC); // dont forget to release..
}

//@mfunc Fills a pre defined region with colour, hatch patterns or images as defined in the style
//@parm Pointer to the region to be filled
//@parm pointer to the style object used for the process
//@parm the type of operation to perform
//@parm A value with style flags which may override the style object
void SRGraphComponent::FillRgn(CRgn *rgn,SRGraphStyle *style,DWORD type,DWORD StyleOverride)
{
	type; //UNUSED

	CRect bound;
	CBrush b,*OldBrush;
	CPen *OldPen,p;
	CBitmap *OldBitmap,mBitmap;
	CDC mdc;
	static BITMAPINFO bmi;
	
	HRGN hRgn=::CreateRectRgn(0,0,0,0);
	int nRestoreRgn=::GetClipRgn(m_pDC->m_hDC,hRgn);

	int RgnType=rgn->GetRgnBox(&bound);
	if(RgnType==NULLREGION)
		return; //Nothing to do.

	GetpDC()->SelectClipRgn(rgn); //Select the clipping region into the DC
	DWORD StyleToUse;
	
	if(!StyleOverride)
	{
		switch(style->GetType())
		{
		case CX_TYPE_COMPONENT:
			StyleToUse=style->GetComponentFillStyle();
			break;
		case CX_TYPE_OBJECT:
			StyleToUse=style->GetInteriorStyle();
			break;
		}
	}
	else
		StyleToUse=StyleOverride;


	switch(StyleToUse)
	{
	case	CX_NO_FILL: // also does for all the other "_NONE" definitions
		break;
	case	CX_SOLID_FILL:
	case	CX_INTERIOR_COLOR:
		b.CreateSolidBrush(style->GetColor());
		OldPen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);
		OldBrush=(CBrush *)GetpDC()->SelectObject(&b);
		bound.right++;
		bound.bottom++;	// Rectangle is very badly behaved!
		GetpDC()->Rectangle(&bound);
		GetpDC()->SelectObject(OldBrush);
		GetpDC()->SelectObject(OldPen);
		b.DeleteObject();
		break;
	case	CX_VERTICAL_FILL:
	case	CX_INTERIOR_VSHADE:
		{
			double rStep,gStep,bStep,rVal,gVal,bVal;
			COLORREF colorA=style->GetColor(),colorB=style->GetColor(TRUE);
			rVal=(double)GetRValue(colorA);
			gVal=(double)GetGValue(colorA);
			bVal=(double)GetBValue(colorA);
			rStep=((double)GetRValue(colorB)-rVal)/bound.Height();
			gStep=((double)GetGValue(colorB)-gVal)/bound.Height();
			bStep=((double)GetBValue(colorB)-bVal)/bound.Height();
			int y;
			for(y=bound.top;y<bound.bottom;y++)
			{		 
				p.CreatePen(PS_SOLID,1,RGB((BYTE)rVal,(BYTE)gVal,(BYTE)bVal));
				OldPen=(CPen *)m_pDC->SelectObject(&p);
				m_pDC->MoveTo(bound.left,y);
				m_pDC->LineTo(bound.right,y);
				m_pDC->SelectObject(OldPen);
				p.DeleteObject();
				rVal+=rStep;
				gVal+=gStep;
				bVal+=bStep;
			}
		}
		break;
	case	CX_HORIZONTAL_FILL:
	case	CX_INTERIOR_HSHADE:
		{
			double rStep,gStep,bStep,rVal,gVal,bVal;
			COLORREF colorA=style->GetColor(),colorB=style->GetColor(TRUE);
			rVal=(double)GetRValue(colorA);
			gVal=(double)GetGValue(colorA);
			bVal=(double)GetBValue(colorA);
			rStep=((double)GetRValue(colorB)-rVal)/bound.Width();
			gStep=((double)GetGValue(colorB)-gVal)/bound.Width();
			bStep=((double)GetBValue(colorB)-bVal)/bound.Width();
			for(int x=bound.left;x<=bound.right;x++)
			{
				p.CreatePen(PS_SOLID,1,RGB((BYTE)rVal,(BYTE)gVal,(BYTE)bVal));
				OldPen=(CPen *)m_pDC->SelectObject(&p);
				m_pDC->MoveTo(x,bound.top);
				m_pDC->LineTo(x,bound.bottom);
				m_pDC->SelectObject(OldPen);
				p.DeleteObject();
				rVal+=rStep;
				gVal+=gStep;
				bVal+=bStep;
			}
		}
		break;
	case	CX_ANGLE_FILL:
	case	CX_INTERIOR_ASHADE:
		{
			double rXStep,gXStep,bXStep,rYStep,gYStep,bYStep,rVal,gVal,bVal,rt,gt,bt,rEnd,bEnd,gEnd;
			COLORREF colorA=style->GetColor(),colorB=style->GetColor(TRUE);
			double Angle=RADIANS(style->GetAngle());
			rt=rVal=(double)GetRValue(colorA);
			gt=gVal=(double)GetGValue(colorA);
			bt=bVal=(double)GetBValue(colorA);
			rEnd=(double)GetRValue(colorB);
			gEnd=(double)GetGValue(colorB);
			bEnd=(double)GetBValue(colorB);
			rXStep=((double)GetRValue(colorB)-rVal)/(bound.Width())*cos(Angle);
			gXStep=((double)GetGValue(colorB)-gVal)/(bound.Width())*cos(Angle);
			bXStep=((double)GetBValue(colorB)-bVal)/(bound.Width())*cos(Angle);
			rYStep=((double)GetRValue(colorB)-rVal)/(bound.Height())*sin(Angle);
			gYStep=((double)GetGValue(colorB)-gVal)/(bound.Height())*sin(Angle);
			bYStep=((double)GetBValue(colorB)-bVal)/(bound.Height())*sin(Angle);
			for(int y=0;y<bound.Height();y++)
			{
				for(int x=0;x<bound.Width();x++)
				{
					GetpDC()->SetPixelV(bound.left+x,bound.top+y,	RGB((int)rVal,(int)gVal,(int)bVal));
					if(	((rXStep<0) && (rVal+rXStep>=rEnd)) || ((rXStep>0) && (rVal+rXStep<=rEnd)))
						rVal+=rXStep;
					if(	((gXStep<0) && (gVal+gXStep>=gEnd)) || ((gXStep>0) && (gVal+gXStep<=gEnd)))
						gVal+=gXStep;
					if(	((bXStep<0) && (bVal+bXStep>=bEnd)) || ((bXStep>0) && (bVal+bXStep<=bEnd)))
						bVal+=bXStep;
				}
				rt+=rYStep;
				gt+=gYStep;
				bt+=bYStep;
				rVal=rt;
				gVal=gt;
				bVal=bt;
			}
		}
		break;
	case	CX_RESOURCE_FILL:
	case	CX_INTERIOR_RESOURCE:
		{
			CBitmap Bitmap;
			if(!Bitmap.LoadBitmap(style->GetResourceName()))
				break;
			BITMAP bm;
			Bitmap.GetBitmap(&bm);							// find the size of the bitmap
			CDC memDC;											// make a source DC
			memDC.CreateCompatibleDC(m_pDC);					// and select the bitmap into it
			OldBitmap=(CBitmap *)memDC.SelectObject(&Bitmap);	
			if(style->GetStretchMode())
			{
				GetpDC()->StretchBlt(	bound.left,bound.top,bound.Width(),bound.Height(),
									&memDC,
									0,0,bm.bmWidth,bm.bmHeight,
									SRCCOPY);
			}
			else
			{
				int x,y;
				for(y=0;y<bound.Height();y+=bm.bmHeight)
				{
					for(x=0;x<bound.Width();x+=bm.bmWidth)
					{
						GetpDC()->BitBlt(	bound.left+x,bound.top+y,
										x+bm.bmWidth<bound.Width() ? bm.bmWidth : bound.Width()-x,
										y+bm.bmHeight<bound.Height() ? bm.bmHeight : bound.Height()-y,
										&memDC,
										0,0,
										SRCCOPY);
					}
				}
			}
			memDC.SelectObject(OldBitmap);
			Bitmap.DeleteObject();
		}
		break;
	case	CX_IMAGE_FILL:
		break;
	case CX_INTERIOR_AUTOMATIC:
		CreateAutoBrush(m_nCurrentIndex);
		OldPen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);
		OldBrush=(CBrush *)GetpDC()->SelectObject(&m_AutoBrush);
		bound.right++;
		bound.bottom++;	// Rectangle is very badly behaved!
		GetpDC()->Rectangle(&bound);
		GetpDC()->SelectObject(OldBrush);
		GetpDC()->SelectObject(OldPen);
		m_AutoBrush.DeleteObject();
		break;
	default:
		DrawCustomRgnFill(rgn,style,type,StyleOverride);
		break;
	}

	if(nRestoreRgn==1)
		::SelectClipRgn(GetpDC()->m_hDC,hRgn);
	else
		GetpDC()->SelectClipRgn(NULL);
}

void SRGraphComponent::DrawCustomRgnFill(CRgn *rgn,SRGraphStyle *style,DWORD type,DWORD StyleOverride)
{
	rgn;
	style;
	type;
	StyleOverride; // unused
	ASSERT(FALSE); // Override this function to draw your custom fills
}

//@mfunc Initialises the m_AutoPen member with a line style according to the key value
//See the <f How automatic colors and fills are chosen> topic for details
//@parm	the key value
//@parm the thickness of the pen. Defaults to 1
//@rdesc the success or failure reported by the CPen::CreatePen(..) function
BOOL SRGraphComponent::CreateAutoPen(int key,int nPenWidth)
{
	if(m_AutoPen.GetSafeHandle()!=NULL)
		m_AutoPen.DeleteObject();
	return m_AutoPen.CreatePen(LineStyleArray[(key>>3) & 0x07],nPenWidth,DataColorArray[key&0x0f]);
}

//@mfunc Initialises the m_AutoBrush member with a brush style according to the key value. 
//See the <f How automatic colors and fills are chosen> topic for details
//@parm	the key value
//@rdesc the success or failure reported by the CBrush::CreateSolidBrush(..) or CBrush::CreateHatchBrush(..) function
BOOL SRGraphComponent::CreateAutoBrush(int key)
{
	if(m_AutoBrush.GetSafeHandle()!=NULL)
		m_AutoBrush.DeleteObject();
	if(key<16)
		return m_AutoBrush.CreateSolidBrush(DataColorArray[key & 0x0f]);
	else
		return m_AutoBrush.CreateHatchBrush(key>>4,DataColorArray[key & 0x0f]);
}


//@mfunc Draws a frame around a pre defined region with colour, hatch patterns or images as defined in the style
//@parm Pointer to the region to be framed
//@parm pointer to the style object used for the process
//@parm the type of operation to perform
//@parm A value with style flags which may override the style object
void SRGraphComponent::FrameRgn(CRgn *rgn,SRGraphStyle *style,DWORD type,DWORD StyleOverride)
{
	type; //UNUSED
	CRect bound;
	CBrush b;
	CBitmap mBitmap;
	CDC mdc;
	static BITMAPINFO bmi;
	HRGN hRgn=::CreateRectRgn(0,0,0,0);
	int nRestoreRgn=::GetClipRgn(m_pDC->m_hDC,hRgn);
	int RgnType=rgn->GetRgnBox(&bound);
	if(RgnType==NULLREGION)
		return; //Nothing to do.

	GetpDC()->SelectClipRgn(rgn); //Select the clipping region into the DC
	
	DWORD StyleToUse;
	
	if(!StyleOverride)
	{
		switch(style->GetType())
		{
		case CX_TYPE_COMPONENT:
			StyleToUse=style->GetComponentBorderStyle();
			break;
		case CX_TYPE_OBJECT:
			StyleToUse=style->GetFrameStyle();
			break;
		}
	}
	else
		StyleToUse=StyleOverride;

	CBrush brush;
	CBitmap bm;

	switch(StyleToUse)
	{
	case CX_FRAME_LINE:	// always fills a 1 pixel black line
		brush.CreateSolidBrush(RGB(0,0,0));
		GetpDC()->FrameRgn(rgn,&brush,1,1);
		brush.DeleteObject();
		break;
	case CX_FRAME_PATTERN: // draws a line with a pattern brush using a specified width
		bm.LoadBitmap((const char *)style->GetFramePattern());
		brush.CreatePatternBrush(&bm);
		GetpDC()->FrameRgn(rgn,&brush,style->GetFrameWidth(),style->GetFrameWidth());
		brush.DeleteObject();
		break;
	case CX_FRAME_COLOR:	// draws a solid coloured line using a specified width
		brush.CreateSolidBrush(style->GetFrameColor());
		GetpDC()->FrameRgn(rgn,&brush,style->GetFrameWidth(),style->GetFrameWidth());
		brush.DeleteObject();
		break;
	}
	
	if(nRestoreRgn==1)
		::SelectClipRgn(GetpDC()->m_hDC,hRgn);
	else
		GetpDC()->SelectClipRgn(NULL);

}

//@mfunc Draws a frame around a polygon with colour, hatch patterns or images as defined in the style
//@parm Pointer to an array of points holding the polygon to be framed
//@parm The number of points in the polygon
//@parm pointer to the style object used for the process
//@parm A value with style flags which may override the style object
void SRGraphComponent::FramePoly(LPPOINT points,int nCount,SRGraphStyle *style,DWORD StyleOverride)
{	
	
	CBrush b;
	CPen pen,*OldPen;
	CBitmap mBitmap;
	CDC mdc;
	static BITMAPINFO bmi;

	DWORD StyleToUse;
	
	if(!StyleOverride)
	{
		switch(style->GetType())
		{
		case CX_TYPE_COMPONENT:
			StyleToUse=style->GetComponentBorderStyle();
			break;
		case CX_TYPE_OBJECT:
			StyleToUse=style->GetFrameStyle();
			break;
		}
	}
	else
		StyleToUse=StyleOverride;

	switch(StyleToUse)
	{
	case CX_FRAME_LINE:	// always draws a 1 pixel black line
		pen.CreatePen(PS_SOLID,style->GetFrameWidth(),RGB(0,0,0));
		OldPen=(CPen *)GetpDC()->SelectObject(&pen);
		GetpDC()->Polyline(points,nCount);
		GetpDC()->SelectObject(OldPen);
		pen.DeleteObject();
		break;
	case CX_FRAME_AUTOMATIC:
		CreateAutoPen(m_nCurrentIndex,style->GetFrameWidth());
		OldPen=(CPen *)GetpDC()->SelectObject(&m_AutoPen);
		GetpDC()->Polyline(points,nCount);
		GetpDC()->SelectObject(OldPen);
		m_AutoPen.DeleteObject();
		break;
	case CX_FRAME_COLOR:	// draws a solid coloured line using a specified width
		pen.CreatePen(PS_SOLID,style->GetFrameWidth(),style->GetFrameColor());
		OldPen=(CPen *)GetpDC()->SelectObject(&pen);
		GetpDC()->Polyline(points,nCount);
		GetpDC()->SelectObject(OldPen);
		pen.DeleteObject();
		break;
	default:
		{
			CRgn rgn;
			if(rgn.CreatePolygonRgn(points,nCount,ALTERNATE))
			{
				FrameRgn(&rgn,style,0,0);
				rgn.DeleteObject();
			}
		}
	break;
	}

}

//@mfunc Fills a polygon with colour, hatch patterns or images as defined in the style
//@parm Pointer to an array of points holding the polygon to be filled
//@parm The number of points in the polygon
//@parm pointer to the style object used for the process
//@parm A value with style flags which may override the style object
void SRGraphComponent::FillPoly(LPPOINT points,int nCount,SRGraphStyle *style,DWORD StyleOverride)
{

	CBrush b,*OldBrush;
	CPen *OldPen;
	CBitmap mBitmap;
	int oldFillMode;
	
	DWORD StyleToUse;

	if(!StyleOverride)
	{
		switch(style->GetType())
		{
		case CX_TYPE_COMPONENT:
			StyleToUse=style->GetComponentFillStyle();
			break;
		case CX_TYPE_OBJECT:
			StyleToUse=style->GetInteriorStyle();
			break;
		}
	}
	else
		StyleToUse=StyleOverride;

	//NOTE Although we can use the region functions for everyting it is desirable to 
	//speed up processing by selecting only those operations which really need region 
	//clipping.

	switch(StyleToUse)
	{
	case CX_SOLID_FILL:
	case CX_INTERIOR_COLOR:	   // can be accomplished with gdi polygon functions
		b.CreateSolidBrush(style->GetColor());
		OldPen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);
		OldBrush=(CBrush *)GetpDC()->SelectObject(&b);
		oldFillMode=GetpDC()->SetPolyFillMode(ALTERNATE);
		GetpDC()->Polygon(points,nCount);
		GetpDC()->SetPolyFillMode(oldFillMode);
		GetpDC()->SelectObject(OldBrush);
		GetpDC()->SelectObject(OldPen);
		b.DeleteObject();
		break;
	case CX_INTERIOR_AUTOMATIC:
	case CX_LEGEND_AUTOMATIC:
		CreateAutoBrush(m_nCurrentIndex);
		OldPen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);
		OldBrush=(CBrush *)GetpDC()->SelectObject(&m_AutoBrush);
		oldFillMode=GetpDC()->SetPolyFillMode(ALTERNATE);
		GetpDC()->Polygon(points,nCount);
		GetpDC()->SetPolyFillMode(oldFillMode);
		GetpDC()->SelectObject(OldBrush);
		GetpDC()->SelectObject(OldPen);
		m_AutoBrush.DeleteObject();
		break;
	default:   // this deals with the use of bitmaps tiled onto an irregular area or custom fills
		{
			CRgn rgn;
			if(rgn.CreatePolygonRgn(points,nCount,ALTERNATE))
			{
				FillRgn(&rgn,style,0,0);
				rgn.DeleteObject();
			}
		}
		break;
	}
}

//@mfunc Interrogates the type information for this component.
//@rdesc UINT representing the type. For example "background" or "legend".
UINT SRGraphComponent::GetType()
{
	return m_nType;
}

//@mfunc Writes the text of the basic component type to a file
//@parm a pointer to a CFile object used to store the file
void SRGraphComponent::WriteComponent(CStdioFile* file)
{
	WriteName(file);
	WriteMainParameters(file);
	WriteFontParameters(file);
	WriteSpecificParameters(file);
	file->WriteString("\n");
}

//@mfunc Not implemented in the base class. Override this member for any special 
//components which you create.
//@parm pointer to a CStdioFile used in writing templates.
//@devnote In all derived classes this function is used to write the name of the template component to file.
// For example "display". This is then read by the <mf SRGraph::ReadTemplate> routine to load template details. 
//In a future release the reading of data may also be handled within the class.
void SRGraphComponent::WriteName(CStdioFile* file)
{
	file; //UNUSED
	ASSERT(FALSE);
	// The base component has no template name
}

//@mfunc Not implemented in the base class.
//@parm pointer to a CStdioFile used in writing templates.
//@devnote In all derived classes this function is used to write the font data of the template component to file. 
//This is then read by the <mf SRGraph::ReadTemplate> routine to load template details. 
//In a future release the reading of data may also be handled within the class.
void SRGraphComponent::WriteFontParameters(CStdioFile* file)
{
	file; //UNUSED
// The basic component does not store font data
}

//@mfunc Not implemented in the base class.
//@parm pointer to a CStdioFile used in writing templates.
//@devnote In all derived classes this function is used to write special data from the template component to file. 
//This is then read by the <mf SRGraph::ReadTemplate> routine to load template details. 
//In a future release the reading of data may also be handled within the class.
void SRGraphComponent::WriteSpecificParameters(CStdioFile* file)
{
	file;//UNUSED
// The basic component does not store specific data
}

//@mfunc Writes all the basic parameters for template components to file.
//@parm pointer to a CStdioFile used in writing templates.
//@devnote After the component name this base class writes all the rectangle data, the fill, shadowing and border styles. 
//the structure is identical for all component types.<nl>
//This data is read in by the <mf SRGraph::ReadTemplate> function.<nl>
//In a future release the reading of data may also be handled within the class.
void SRGraphComponent::WriteMainParameters(CStdioFile* file)
{
	CString cdata;
	cdata.Format("%g %g %g %g %d %d %d %d ",
				m_Rect.left,m_Rect.top,m_Rect.right,m_Rect.bottom,
				m_Rect.GetMeasurement(),
				m_Style.GetComponentBorderStyle()>>CX_BORDER_SHIFT,
				m_Style.GetComponentFillStyle()>>CX_FILL_SHIFT,
				m_Style.GetComponentShadowStyle()>>CX_SHADOW_SHIFT);
	file->WriteString(cdata);

	switch(m_Style.GetComponentFillStyle())
	{
	case 	CX_NO_FILL:				//0x00000000
		break;
	case 	CX_SOLID_FILL:				//0x00000100
		cdata.Format("%d %d %d ",
				GetRValue(m_Style.GetColor()),
				GetGValue(m_Style.GetColor()),
				GetBValue(m_Style.GetColor()));
		file->WriteString(cdata);
		break;
	case 	CX_VERTICAL_FILL:			//0x00000200
		cdata.Format("%d %d %d %d %d %d ",
				GetRValue(m_Style.GetColor()),
				GetGValue(m_Style.GetColor()),
				GetBValue(m_Style.GetColor()),
				GetRValue(m_Style.GetColor(TRUE)),
				GetGValue(m_Style.GetColor(TRUE)),
				GetBValue(m_Style.GetColor(TRUE)));
		file->WriteString(cdata);
		break;
	case 	CX_HORIZONTAL_FILL:		//0x00000300
		cdata.Format("%d %d %d %d %d %d ",
				GetRValue(m_Style.GetColor()),
				GetGValue(m_Style.GetColor()),
				GetBValue(m_Style.GetColor()),
				GetRValue(m_Style.GetColor(TRUE)),
				GetGValue(m_Style.GetColor(TRUE)),
				GetBValue(m_Style.GetColor(TRUE)));
		file->WriteString(cdata);
		break;
	case	CX_ANGLE_FILL:				//0x00000400
		cdata.Format("%d %d %d %d %d %d %g ",
				GetRValue(m_Style.GetColor()),
				GetGValue(m_Style.GetColor()),
				GetBValue(m_Style.GetColor()),
				GetRValue(m_Style.GetColor(TRUE)),
				GetGValue(m_Style.GetColor(TRUE)),
				GetBValue(m_Style.GetColor(TRUE)),
				m_Style.GetAngle());
		file->WriteString(cdata);
		break;
	case 	CX_RESOURCE_FILL:			//0x00000500
		cdata.Format("\"%s\" %d ",m_Style.GetResourceName(),m_Style.GetStretchMode() ? 1 : 0);
		file->WriteString(cdata);
		break;
	case 	CX_IMAGE_FILL:				//0x00000600
		break;	// not yet..
	}
}

//@mfunc This is the callback routine which orders arrays of <t GESorter> elements in ascending order.
//@parm Pointer to the first element
//@parm pointer to the second element
//@devnote This callback has been placed in the component class because it will be used by the various display 
//classes and routines derived from SRGraphComponent. Note that it does not deal with any member data and therefore 
//does not require a reference to the class instance which calls it. This routine is not virtual and should not be 
//altered or overridden without first understanding the implications of class member function callbacks.
int __cdecl SRGraphComponent::qsSorterCompareA(const void *a,const void *b)
{
	if(((GESorter *)a)->value == ((GESorter *)b)->value)
		return 0;
	if(((GESorter *)a)->value > ((GESorter *)b)->value)
		return 1;
	return -1;
}

//@mfunc This is the callback routine which orders arrays of <t GESorter> elements in decending order.
//@parm Pointer to the first element
//@parm pointer to the second element
//@see <mf SRGraphComponent::qsSorterCompareA>
int __cdecl SRGraphComponent::qsSorterCompareD(const void *a,const void *b)
{
	if(((GESorter *)a)->value == ((GESorter *)b)->value)
		return 0;
	if(((GESorter *)a)->value < ((GESorter *)b)->value)
		return 1;
	return -1;
}


//@mfunc callback routine for qsort. Sorts arrays of <t PointSorter> objects into ascending order.
//@parm Pointer to the first PointSorter cast to const void *
//@parm Pointer to the second PointSorter cast to const void *
//@rdesc a==b 0, a<gt>b 1, a<lt>b -1	See the qsort documentation for details
//@devnote This callback has been placed in the component class because it will be used by the various display 
//classes and routines derived from SRGraphComponent. Note that it does not deal with any member data and therefore 
//does not require a reference to the class instance which calls it. This routine is not virtual and should not be 
//altered or overridden without first understanding the implications of class member function callbacks.
int _cdecl SRGraphComponent::qsPointYSorterCompareA(const void *a,const  void *b)
{
	PointSorter *p1=(PointSorter*)a,
				*p2=(PointSorter*)b;

	if(p1->p.y==p2->p.y)
		return 0;
	if(p1->p.y>p2->p.y)
		return 1;
	return -1;
}

//@mfunc Moves a complete polygon stored in a point list by a set displacement
//@parm The point list
//@parm The number of points to move
//@parm The X displacement (may be negative)
//@parm The Y displacement (may be negative)
void SRGraphComponent::OffsetPolygon(CPoint * points,int nPointCount, int xOffset,int yOffset)
{
	for(int c=0;c<nPointCount;c++)
		points[c]=CPoint(points[c].x+xOffset,points[c].y+yOffset);
}


//@mfunc callback routine for qsort. Sorts arrays of CPoint objects into ascending order.
//@parm Pointer to the first CPoint cast to const void *
//@parm Pointer to the second CPoint cast to const void *
//@rdesc a==b 0, a<gt>b 1, a<lt>b -1 See the qsort documentation for details	or <mf SRGraphComponent::qsPointYSorterCompareA>
int _cdecl SRGraphComponent::qsPointYSorterCompareD(const void *a,const  void *b)
{
	CPoint *p1=(CPoint*)a,
		   *p2=(CPoint*)b;

	if(p1->y==p2->y)
		return 0;
	if(p1->y<p2->y)
		return 1;
	return -1;
}

#ifdef _DEBUG

//@mfunc Dumps this object
//@parm Reference to a dump context
void SRGraphComponent::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	m_Style.Dump(dc);
	dc << "m_PolygonList = " << m_PolygonList;
	dc << "m_bSignificantChange = " << m_bSignificantChange;
	dc << "m_nCountryCode = " << m_nCountryCode;
	dc << "m_bHiLo = " << m_bHiLo;
	dc << "m_nMaxIndex = " << m_nMaxIndex;
	dc << "m_nMinIndex = " << m_nMinIndex;
	dc << "m_nMaxGroup = " << m_nMaxGroup;
	dc << "m_nMinGroup = " << m_nMinGroup;
	dc << "m_nDivCount = " << m_nDivCount;
	dc << "m_nMaxDiv = " << m_nMaxDiv;
	dc << "m_nMinDiv = " << m_nMinDiv;
	dc << "m_nSetCount = " << m_nSetCount;
	dc << "m_nMinSet = " << m_nMinSet;
	dc << "m_nMaxSet = " << m_nMaxSet;
	dc << "m_ScaleY = " << m_ScaleY;
	dc << "m_ScaleX = " << m_ScaleX;
	dc << "m_dTickSize = " << m_dTickSize;
	dc << "m_dMarginY = " << m_dMarginY;
	dc << "m_dMarginX = " << m_dMarginX;
	dc << "m_AutoPen = " << m_AutoPen;
	dc << "m_AutoBrush = " << m_AutoBrush;
	dc << "m_Rect = " << m_Rect;
	dc << "m_nCurrentIndex = " << m_nCurrentIndex;
	dc << "m_Style = " << m_Style;
	dc << "m_bDrawing = " << m_bDrawing;
	dc << "m_pCWnd = " << m_pCWnd;
	dc << "m_pDC = " << m_pDC;
	dc << "m_pParentGraph = " << m_pParentGraph;
	dc << "m_nType  = " << m_nType;

}

#endif // _DEBUG


//@mfunc Returns the size of ticks used in this graph component. Performs conversions to
//millimeters or inches accordingly.
//@rdesc The tick size as calculated from the logical X and Y resolutions
double SRGraphComponent::GetTickSize()
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return m_dTickSize;
	}
	
	int res;
	
	// note res is not calculated here for the sake of speed
	// if you only want pixels you dont need to bother with the
	// device capabilities

	switch(m_Rect.GetMeasurement())
	{
	default:
	case 0:
		return m_dTickSize;
	case 1:
		res=(GetpDC()->GetDeviceCaps(LOGPIXELSX)+GetpDC()->GetDeviceCaps(LOGPIXELSY))/2;
		return m_dTickSize*res/25.4f;
	case 2:
		res=(GetpDC()->GetDeviceCaps(LOGPIXELSX)+GetpDC()->GetDeviceCaps(LOGPIXELSY))/2;
		return m_dTickSize*res;
	}
}

//@mfunc Returns the X margin size. Usually used to space the m_DisplayRect away from the borders
//@rdesc X margin as adjusted for screen resolution and measurement style as stored in the m_Rect member
double SRGraphComponent::GetXMargin()
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return m_dMarginX;
	}
	
	switch(m_Rect.GetMeasurement())
	{
	default:
	case 0:
		return m_dMarginX;
	case 1:
		return m_dMarginX*GetpDC()->GetDeviceCaps(LOGPIXELSX)/25.4f;
	case 2:
		return m_dMarginX*GetpDC()->GetDeviceCaps(LOGPIXELSX);
	}
}

//@mfunc Returns the Y margin size. Usually used to space the m_DisplayRect away from the borders
//@rdesc Y margin as adjusted for screen resolution and measurement style as stored in the m_Rect member
double SRGraphComponent::GetYMargin()
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return m_dMarginY;
	}

	switch(m_Rect.GetMeasurement())
	{
	default:
	case 0:
		return m_dMarginY;
	case 1:
		return m_dMarginY*GetpDC()->GetDeviceCaps(LOGPIXELSY)/25.4f;
	case 2:
		return m_dMarginY*GetpDC()->GetDeviceCaps(LOGPIXELSY);
	}
}

//@mfunc Sets the maximum index upon which this component should act
//@parm The index
void SRGraphComponent::SetMaxIndex(int index)
{
	m_nMaxIndex=index;
	m_bSignificantChange=TRUE;
}

//@mfunc Sets the minimum index upon which this component should act
//@parm The index
void SRGraphComponent::SetMinIndex(int index)
{
	m_nMinIndex=index;
	m_bSignificantChange=TRUE;
}

//@mfunc Sets the maximum group upon which this component should act
//@parm The group
void SRGraphComponent::SetMaxGroup(int group)
{
	m_nMaxGroup=group;
	m_bSignificantChange=TRUE;
}

//@mfunc Sets the minimum group upon which this component should act
//@parm The group
void SRGraphComponent::SetMinGroup(int group)
{
	m_nMinGroup=group;
	m_bSignificantChange=TRUE;
}

//@mfunc Gets the maximum index upon which this component should act
//@rdesc The index as set by SetMaxIndex
int SRGraphComponent::GetMaxIndex()
{
	return m_nMaxIndex;
}

//@mfunc Gets the minimum index upon which this component should act
//@rdesc The index as set by SetMinIndex
int SRGraphComponent::GetMinIndex()
{
	return m_nMinIndex;
}

//@mfunc Gets the maximum group upon which this component should act
//@rdesc The group as set by SetMaxGroup
int SRGraphComponent::GetMaxGroup()
{
	return m_nMaxGroup;
}

//@mfunc Gets the minimum group upon which this component should act
//@rdesc The group as set by SetMinGroup
int SRGraphComponent::GetMinGroup()
{
	return m_nMinGroup;
}

BOOL SRGraphComponent::IsDrawing()
{
	return m_bDrawing;
}




//@mfunc Handles storage and retrieval of this object.
//@rdesc reference to a CArchive object which manages the file operations.
//@devnote Component serialization cannot allow storage of dynamic pointers, like the m_pParentGraph pointer, 
//because only sure thing is that the memory will be allocated in a different place every time the component 
//is constructed. Therefore you must remember that if you load by serialization you must step through the 
//component list and re-assign all the dynamic pointers before the display routines kick in and crash everything
void SRGraphComponent::Serialize(CArchive& ar)
{
	
	CObject::Serialize(ar);

	m_Style.Serialize(ar);
	m_Rect.Serialize(ar);
	m_ScaleY.Serialize(ar);

	if(ar.IsStoring())
	{
	ar << m_bHiLo;
	ar << m_nMaxIndex;
	ar << m_nMinIndex;
	ar << m_nMaxGroup;
	ar << m_nMinGroup;
	ar << m_dTickSize;
	ar << m_dMarginY;
	ar << m_dMarginX;
 	ar << m_nType;
	}
	else
	{
	ar >> m_bHiLo;
	ar >> m_nMaxIndex;
	ar >> m_nMinIndex;
	ar >> m_nMaxGroup;
	ar >> m_nMinGroup;
	ar >> m_dTickSize;
	ar >> m_dMarginY;
	ar >> m_dMarginX;
 	ar >> m_nType;
	}
}

void SRGraphComponent::SetCountryCode(int n)
{
	if(n==-1)
	{
		//ASSERT(GetParent()!=NULL);
		m_nCountryCode=GetCountryCode();
	}
	else
		m_nCountryCode=n;
}


//////////////////////////////////////////////////////////////////////
// SRGraphBackground member functions


//@mfunc Used to write the "background" name to the template file
//@parm Pointer to the CStdioFile used in writing this template
void SRGraphBackground::WriteName(CStdioFile *file)
{
	file->WriteString("background ");
}

//@mfunc Does nothing.
//@rdesc FALSE. signifies no characters processed
int SRGraphBackground::ReadTemplateLine(LPCSTR line)
{
	line;//UNUSED
	return 0;
}
///////////////////////////////////////////////////////////////////
// SRGraphTitle member functions


//@mfunc Constructor
SRGraphTitle::SRGraphTitle()
{
	MakeFont=TRUE;
	m_nSize=10;
	m_nStyle=0;
	m_strFaceName="Arial";
	m_nType=IDS_SRG_TITLETYPE;
	m_RGBTextColor=CXCLR_BLACK;
	SetJustification(2);
}

//@mfunc Destructor
SRGraphTitle::~SRGraphTitle()
{
	if(m_Font.GetSafeHandle()!=NULL)
		m_Font.DeleteObject();
}

//@mfunc Initialises or renews the font used to draw text onto the foreground.
//@devnote This is called from within the draw loop to ensure a correct CWnd and CDC are used
BOOL SRGraphTitle::UpdateFont()
{


	ASSERT(IsDrawing()); //->Calling this outside of the draw loop may cause errors due to incorrect CWnd or CDC pointers in m_pCWnd and m_pDC
	LOGFONT lf;
	double xRes,yRes;
	xRes=(double)GetpDC()->GetDeviceCaps(LOGPIXELSX);
	yRes=(double)GetpDC()->GetDeviceCaps(LOGPIXELSY);
	
	if(m_nSize<0)
		lf.lfHeight=abs(m_nSize); // in device pixels
	else
		lf.lfHeight=(int)(yRes/72*m_nSize); // or in points
	
	lf.lfWidth=0;
	lf.lfEscapement=0;
	lf.lfOrientation=0;
	lf.lfWeight=m_nStyle & 1 ? 700 : 400;
	lf.lfItalic=(BYTE)(m_nStyle & 2);
	lf.lfUnderline=(BYTE)(m_nStyle & 4);
	lf.lfStrikeOut=(BYTE)(m_nStyle & 8);
	lf.lfCharSet=ANSI_CHARSET;
	lf.lfOutPrecision=0;
	lf.lfClipPrecision=0;
	lf.lfQuality=0;
	lf.lfPitchAndFamily=0;
	strncpy(lf.lfFaceName,(const char *)m_strFaceName,30);
	if(m_Font.GetSafeHandle()!=NULL)
		m_Font.DeleteObject();
	return m_Font.CreateFontIndirect(&lf);
}

//@mfunc Overridden to draw the foreground text. Updates fonts if needed.
void SRGraphTitle::DrawForeground()
{
	if(GetpDC()->IsPrinting() || m_bSignificantChange)
		MakeFont=TRUE;
	
	if(MakeFont)
		UpdateFont();

	if(!GetpDC()->IsPrinting())
		MakeFont=FALSE;

	GetParentContent();

	CFont *OldFont=(CFont*)GetpDC()->SelectObject(&m_Font); // select the font into the DC
	int justification;

	switch(m_nJustification)
	{
	case 0:
		justification=DT_LEFT;
		break;
	case 1:
		justification=DT_RIGHT;
		break;
	case 2:
		justification=DT_CENTER;
		break;
	}
	CRect r;
	r=m_Rect.CalcCRect(m_pDC,m_pCWnd);
	COLORREF OldColor=GetpDC()->SetTextColor(m_RGBTextColor);
	int OldMode=GetpDC()->SetBkMode(TRANSPARENT);
	CSize ts=GetpDC()->GetTextExtent((LPCSTR)m_strTitle,m_strTitle.GetLength());
	GetpDC()->DrawText((LPCSTR)m_strTitle,
					 m_strTitle.GetLength(),
					 CRect(	r.left+(r.Width()/2)-(ts.cx/2),
							r.top+(r.Height()/2)-(ts.cy/2),
							r.left+(r.Width()/2)+(ts.cx/2),
							r.top+(r.Height()/2)+(ts.cy/2)),
					 justification | DT_NOPREFIX); // display the text
	GetpDC()->SetBkMode(OldMode);
	GetpDC()->SelectObject(OldFont);
	GetpDC()->SetTextColor(OldColor);
}

//@mfunc Initialises the text for this title
//@parm Pointer to the text string
//@parm The justification to use<nl>
//	0=left<nl>
//	1=right<nl>
//	2=centered<nl>
void SRGraphTitle::SetText(LPCSTR text,int justification)
{
	m_strTitle= strcmp(text,"") ==0 ? " " : text ; // You have to go through this rigmarole becase the CString::operator =(LPCSTR) has a bug in it..
	m_nJustification=justification;
}

//@mfunc Initialises the font used in foreground drawing.
//@parm Size of font. Negative numbers signify height in pixels, positive numbers are for heights in points.
//@parm binary flags for style. Combine by logical OR.. 1=BOLD 2=ITALIC 4=UNDERLINE 8=STRIKEOUT
//@parm String containing typeface name. Must be no more than 30 characters eg.."Times New Roman"
//@rdesc TRUE to signify success. (This function always succeeds but derived ones may not)
BOOL SRGraphTitle::SetFont(int nSize,int nStyle,CString FaceName)
{
	m_nSize=nSize; m_nStyle=nStyle; m_strFaceName=FaceName;
	MakeFont=TRUE;
	return TRUE;
}

//@mfunc Loads or stores this object via the supplied archive
//@parm A reference to the CArchive object used for file operations
void SRGraphTitle::Serialize(CArchive & ar)
{
	SRGraphComponent::Serialize(ar);

	if(ar.IsStoring())
	{
	ar << m_nJustification;
	ar << m_RGBTextColor;
	ar << m_nSize;
	ar << m_nStyle;
	ar << m_strTitle;
	ar << m_strFaceName;
	}
	else
	{
	ar >> m_nJustification;
	ar >> m_RGBTextColor;
	ar >> m_nSize;
	ar >> m_nStyle;
	ar >> m_strTitle;
	ar >> m_strFaceName;

	MakeFont=TRUE; //Always rebuild after a load

	}

}

//@mfunc Retrieves pertinent content from the parent.
//@see <c SRGraphComponent>
void SRGraphTitle::GetParentContent()
{
	SetText(GetParent()->GetGraphTitle(),m_nJustification);
}

//@mfunc Used to write the font data to the template file. Note that only SRGraphTitle and derivatives carry font data.
//@parm Pointer to the CStdioFile used in writing this template
void SRGraphTitle::WriteFontParameters(CStdioFile *file)
{
	CString cdata;
	cdata.Format("%d %d %d \"%s\" %d %d %d ",
				GetRValue(m_RGBTextColor),
				GetGValue(m_RGBTextColor),
				GetBValue(m_RGBTextColor),
				(LPCSTR)m_strFaceName,
				m_nSize,
				m_nStyle,
				m_nJustification);
	file->WriteString(cdata);
}


//@mfunc Calculates the logical size of text using the currently selected font.
//@parm Pointer to the text to size
//@rdesc a CSize object containing the width and height of the text
//@devnote Both GetTextExtent and DrawText will not give true sizes for text. GetTextExtent 
//treats all text as a single line and so gives a width based on all the characters in the text. 
//Draw Text merely extends the height of an existing rectangle and does not give accurate information on widths. 
//Therefore this routine finds the maximum width of all the seperate lines and then returns that along with the 
//height calculated from the total number of lines.
CSize SRGraphTitle::GetTextDimension(LPCSTR text)
{
	if(strlen(text)==0)
		return CSize(0,0);
	
	CString Text=text;	// copy it so that we can use the CString functions and or chop the otherwise const string about.
	CStringList LineList;

	int next;
	CSize TotalSize(0,0),SubSize;
	
	do
	{
		next=Text.Find('\n');
		if(next!=-1)
		{
			CString substring=Text.Left(next);
			SubSize=GetpDC()->GetTextExtent(substring);
		}
		else
		{
			SubSize=GetpDC()->GetTextExtent(Text);
		}

		TotalSize.cx=max(TotalSize.cx,SubSize.cx);
		TotalSize.cy+=SubSize.cy;
		for(int n=0;n<next;n++)
			Text.SetAt(n,' ');
		Text.TrimLeft();
	}
	while(next!=-1);

	return TotalSize;
}

//@mfunc Used to write the "title" name to the template file
//@parm Pointer to the CStdioFile used in writing this template
void SRGraphTitle::WriteName(CStdioFile *file)
{
	file->WriteString("title ");
}

#ifdef _DEBUG

//@mfunc Debug dumping of the object
//@parm reference to a CDumpContext used to output the object data.
void SRGraphTitle::Dump(CDumpContext& dc) const
{
	SRGraphComponent::Dump(dc);
}

#endif //_DEBUG

//@mfunc Interrogates the name of the typeface currently being used to display titles
//@rdesc The contents of the m_strFaceName member.
//@see <mf SRGraphRect::SetFaceName> <md SRGraphRect::m_strFaceName>
CString & SRGraphTitle::GetFaceName()
{
  return m_strFaceName;
}

//@mfunc Initialises the typeface name, for example "Times New Roman" which is used to define the font used
//@parm The name.
void SRGraphTitle::SetFaceName(CString &FaceName)
{
	m_strFaceName=FaceName;
}

//@mfunc Sets or resets the MakeFont flag. When this flag is TRUE then a font will be built from the style information at the beginning of the next draw cycle.
//@parm TRUE to make the font next time the object is drawn.
void SRGraphTitle::SetMakeFont(BOOL make)
{
	MakeFont=make;	
}







//@mfunc Handles storage and retrieval of this object.
//@parm reference to a CArchive object which manages the file operations.
//@rem Component serialization routines can only save the data for the setup and appearence of the component. 
//It cannot save the dynamic data like the m_pParentGraph pointer. See the <mf SRGraph::Serialize> routine for 
//a more detailed explanation.
void SRGraphLegend::Serialize(CArchive& ar)
{
	SRGraphTitle::Serialize(ar);

	if(ar.IsStoring())
	{
	ar << m_bAutoSizeX << m_bAutoSizeY << m_bKeepLeft << m_bKeepTop << m_dKeySize;
	}
	else
	{
	ar >> m_bAutoSizeX >> m_bAutoSizeY >> m_bKeepLeft >> m_bKeepTop >> m_dKeySize;
	}
}




//@mfunc Retrieves pertinent content from the parent. Legends fetch the header list.
void SRGraphLegend::GetParentContent()
{

	int CurrentSet,x,nLabel;

	switch(m_Style.GetLegendStyle())
	{
	default:
	case CX_LEGEND_BY_GROUP:
	case CX_LEGEND_HEAD_GROUP:
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			m_LegendLabels.AddLabelIndex(GetParent()->GetHeader(m_nMinSet+CurrentSet),CurrentSet);
		m_nKeyCount=m_nSetCount;
		break;
	case CX_LEGEND_BY_INDEX:
	case CX_LEGEND_HEAD_INDEX:
		for(x=0;x<m_nDivCount;x++)
			m_LegendLabels.AddLabelIndex(GetParent()->GetGroup(0)->GetIndex(m_nMinDiv+x)->GetAnnotation(GetCountryCode()),x);
		m_nKeyCount=m_nDivCount;
		break;
	case CX_LEGEND_BY_RECTANGLE:
		nLabel=0;
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			for(x=0;x<m_nDivCount;x++)
			{
				CString strFormatString,strLabel=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(m_nMinDiv+x)->GetAnnotation(GetCountryCode());
				if(strLabel=="")
				{
					strFormatString.LoadString(IDS_SRG_LEGEND_DEFAULT);
					strLabel.Format(strFormatString,CurrentSet+m_nMinSet,x+m_nMinDiv);
				}
				m_LegendLabels.AddLabelIndex(strLabel,nLabel++);
			}
		}
		m_nKeyCount=m_nSetCount*m_nDivCount;
		break;
	}	
	while(m_LegendLabels.GetLabelCount()>m_nKeyCount)
		m_LegendLabels.RemoveLastLabel();
}

//@mfunc Called by the base class (<mf SRGraphComponent::Draw>) function to 
//draw the foreground of this component. Note that the legend will copy the m_nMinIndex, m_nMaxIndex, m_nMinGroup and m_nMaxGroup variables of
//any SRGraphDisplay based object which immediately preceeds it in the parent graphs component list. 
//@devnote Because Legends may change in size this component will resize itself to accommodate 
//the text to be displayed.
void SRGraphLegend::DrawForeground()
{

	if(GetpDC()->IsPrinting() || m_bSignificantChange)
		MakeFont=TRUE;
	
	if(MakeFont)
		UpdateFont();

	if(!GetpDC()->IsPrinting())
		MakeFont=FALSE;

	m_LegendLabels.DrawingUsing(m_pDC,m_pCWnd);
	m_LegendLabels.SetParent(GetParent());
	if(m_bSizeForeground)
	{
	
		m_LegendLabels.SetFontSize(GetFontSize());
		m_LegendLabels.SetFaceName(GetFaceName());
		m_LegendLabels.SetTextColor(GetTextColor());
		m_LegendLabels.SetFontStyle(GetFontStyle());
		m_LegendLabels.SetPass(0);
		m_LegendLabels.UpdateFont(); // To ensure the correct font size. (normally done by Draw()) but we are not using the pass 0 routine to place the labels.

		CObList *pCList=GetParent()->GetComponentList();

		int nComp=0;
		
		m_pDisplay=NULL;

		for(POSITION p=pCList->GetHeadPosition();p;)
		{
			if((SRGraphComponent *)pCList->GetNext(p)==this && nComp!=0)
			{
				if(p==NULL)
					p=pCList->GetTailPosition();
				pCList->GetPrev(p);
				SRGraphComponent *pDisp=(SRGraphComponent *)pCList->GetPrev(p);
				if(pDisp->IsKindOf(RUNTIME_CLASS(SRGraphDisplay)))
				{
					m_pDisplay=(SRGraphDisplay *)pDisp;
					SetMinIndex(pDisp->GetMinIndex());
					SetMinGroup(pDisp->GetMinGroup());
					SetMaxIndex(pDisp->GetMaxIndex());
					SetMaxGroup(pDisp->GetMaxGroup());
				}
				p=NULL; // we are finished anyhow
			}
			nComp++;
		}

		m_nMinSet=m_nMinGroup==-1 ? 0 : m_nMinGroup;
		m_nMaxSet=m_nMaxGroup==-1 ? GetParent()->GetGroupCount()-1 : m_nMaxGroup;
		m_nMinDiv=m_nMinIndex==-1 ? 0 : m_nMinIndex;
		m_nMaxDiv=m_nMaxIndex==-1 ? GetParent()->GetMaxXDivisions()-1 : m_nMaxIndex;

		m_nSetCount=1+(m_nMaxSet-m_nMinSet);
		m_nDivCount=1+(m_nMaxDiv-m_nMinDiv);

		m_LegendRect=m_Rect.CalcCRect(m_pDC,m_pCWnd);

		int thickness=0;

		switch(m_Style.GetComponentBorderStyle())
		{
		case CX_NO_BORDER:
			thickness=0;
			break;
		case CX_THIN_RAISED_BORDER:
		case CX_THIN_SUNKEN_BORDER:
			thickness=1;
			break;
		case CX_THIN_3D_BORDER:
			thickness=2;
			break;
		case CX_THICK_RAISED_BORDER:
		case CX_THICK_SUNKEN_BORDER:
			thickness=3;
			break;
		case CX_THICK_3D_BORDER:
			thickness=6;
			break;
		case CX_LINE_BORDER:
			thickness=1;
			break;
		}


		GetParentContent(); // this keeps the list dynamic
	
		CPoint points[5];
		m_LegendLabels.SetBlockOrientations(0);
		m_LegendLabels.SetBlockLocationPoints(SRGraphLabel::TopLeft);
		m_LegendLabels.SetBlockPositions(CPoint(0,0),CPoint(0,0));
		
		CScale Width,Height;

		for(int c=0;c<m_LegendLabels.GetLabelCount();c++)
		{
			m_LegendLabels.GetLabelIndex(c)->GetBoundingPolygon(&points[0],5);
			Width+=points[0].x;
			Height+=points[0].y;
			Width+=points[2].x;
			Height+=points[2].y;
		}

		double dNumAcross=(double)m_LegendRect.Width()/(Width.Max()+(m_dKeySize*2));
		if(dNumAcross<1)
			dNumAcross=1;
		double dNumDown=ceil((double)m_LegendLabels.GetLabelCount()/dNumAcross);

		dNumAcross=floor(dNumAcross);

		if(dNumDown==1 && dNumAcross>m_LegendLabels.GetLabelCount())
			dNumAcross=m_LegendLabels.GetLabelCount();


		int left,top,right,bottom;

		if(m_bAutoSizeX)
		{
			left=(int)(m_bKeepLeft ? m_LegendRect.left : m_LegendRect.right-(dNumAcross * (Width.Max()+(m_dKeySize*1.2))));
			right=(int)(m_bKeepLeft ? m_LegendRect.left+(dNumAcross * (Width.Max()+(m_dKeySize*1.2))) : m_LegendRect.right);
		}
		else
		{
			left=m_LegendRect.left;
			right=m_LegendRect.right;
		}

		if(m_bAutoSizeY)
		{
			top=(int)(m_bKeepTop ? m_LegendRect.top : m_LegendRect.bottom-(dNumDown * (Height.Max() *1.2)));
			bottom=(int)(m_bKeepTop ? m_LegendRect.top+(dNumDown * (Height.Max() *1.2)) : m_LegendRect.bottom);
		}
		else
		{
			top=m_LegendRect.top;
			bottom=m_LegendRect.bottom;
		}

		m_LegendRect=CRect(left,top,right,bottom);

		//m_Rect.CRectToSRGRect(m_LegendRect,m_pDC);

		m_LegendRect.DeflateRect((int)(thickness+(2*GetXMargin())),(int)(thickness+(2*GetYMargin())));					// minus the borders.. which makes the working area.


		double nXStep=(double)m_LegendRect.Width()/dNumAcross,
				nYStep=max((double)m_LegendRect.Height()/dNumDown,Height.Max());

		for(c=0;c<m_LegendLabels.GetLabelCount();c++)
		{
			int down=(int)(c/dNumAcross);
			int across=(int)(c-(down*dNumAcross));

			SRGraphLabel *pL=m_LegendLabels.GetLabelIndex(c);
			pL->SetLocationPoint(SRGraphLabel::MidRight);
			pL->SetTextPosition(m_LegendRect.left+(int)((across*nXStep)+Width.Max()+(m_dKeySize*1.1)),
								m_LegendRect.top+(int)(down*nYStep));
		}

		m_nKeyHeight=(int)Height.Max();
		
	}
	else
	{
		m_LegendLabels.SetPass(1);
		m_LegendLabels.Draw(m_pDC,m_pCWnd);
		
		int CurrentSet,x,nLabel,c=0;;
		DWORD dwStyle=m_Style.GetLegendStyle();
		
		SRGraphLabel *pL;
		ASSERT(pL!=NULL);

		CPoint tP;

		switch(dwStyle)
		{
		default:
		case CX_LEGEND_BY_GROUP:
		case CX_LEGEND_HEAD_GROUP:
			for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			{
				pL=m_LegendLabels.GetLabelIndex(c++);
				tP=pL->GetTextPosition();
				DrawKey(CPoint((int)(tP.x+(m_dKeySize/10)),(int)(tP.y-(m_nKeyHeight/2))),
						CPoint((int)(tP.x+(m_dKeySize/10)+m_dKeySize),(int)(tP.y-(m_nKeyHeight/2)+m_nKeyHeight)),
						m_nMinDiv,CurrentSet+m_nMinSet,CurrentSet+m_nMinSet);
			}
			break;
		case CX_LEGEND_BY_INDEX:
		case CX_LEGEND_HEAD_INDEX:
			for(x=0;x<m_nDivCount;x++)
			{
				pL=m_LegendLabels.GetLabelIndex(c++);
				tP=pL->GetTextPosition();
				DrawKey(CPoint((int)(tP.x+(m_dKeySize/10)),(int)(tP.y-(m_nKeyHeight/2))),
						CPoint((int)(tP.x+(m_dKeySize/10)+m_dKeySize),(int)(tP.y-(m_nKeyHeight/2)+m_nKeyHeight)),
						x+m_nMinDiv,m_nMinSet,x+m_nMinDiv);
			}
			break;
		case CX_LEGEND_BY_RECTANGLE:
			nLabel=0;
			c=0;
			for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			{
				for(x=0;x<m_nDivCount;x++)
				{
					pL=m_LegendLabels.GetLabelIndex(c++);
					tP=pL->GetTextPosition();
					DrawKey(CPoint((int)(tP.x+(m_dKeySize/10)),(int)(tP.y-(m_nKeyHeight/2))),
							CPoint((int)(tP.x+(m_dKeySize/10)+m_dKeySize),(int)(tP.y-(m_nKeyHeight/2)+m_nKeyHeight)),
							m_nMinDiv,CurrentSet+m_nMinSet,nLabel++);
				}
			}
			m_nKeyCount=m_nSetCount*m_nDivCount;
			break;
		}	
	}
}

//@mfunc Constuctor. Initialises component behaviour to default values. 
SRGraphLegend::SRGraphLegend()
{
	m_bAutoSizeX=FALSE;
	m_bAutoSizeY=FALSE;
	m_bKeepLeft=FALSE;
	m_bKeepTop=FALSE;
	m_nType=IDS_SRG_LEGENDTYPE;
	m_nJustification=0;
	m_dKeySize=20;
	m_nCountryCode=1;
}

//@mfunc Overridden from <mf SRGraphComponent::Draw> to resize the component 
//automatically before affecting the screen.
//@parm Pointer to the CDC for drawing onto.
//@parm Pointer to the CWnd object managing the paint message.
void SRGraphLegend::Draw(CDC *pDC,CWnd *pCWnd)
{
	m_bDrawing=TRUE;
	m_pDC=pDC;
	m_pCWnd=pCWnd;
	
	m_RectStore=m_Rect;
	
	m_bSizeForeground=TRUE;
	DrawForeground();
	m_bSizeForeground=FALSE;

	if(	!GetParent()->m_bHintPending || 
		(GetParent()->m_bHintPending && GetParent()->m_lHint & CX_FILL_MASK))
		DrawFill();
	if(	!GetParent()->m_bHintPending || 
		(GetParent()->m_bHintPending && GetParent()->m_lHint & CX_SHADOW_MASK))
		DrawShadow();
	if(	!GetParent()->m_bHintPending || 
		(GetParent()->m_bHintPending && GetParent()->m_lHint & CX_BORDER_MASK))
		DrawBorder();
	if(	!GetParent()->m_bHintPending || 
		(GetParent()->m_bHintPending && GetParent()->m_lHint & CX_FOREGROUND_MASK))
		DrawForeground();
	m_bDrawing=FALSE;
	m_bSignificantChange=FALSE;
	m_Rect=m_RectStore;
}

//@mfunc Initialises the auto-sizing feature of the component.<nl>
//@parm Bitwise flags to be placed in the autosize semaphores<nl>
//0x0001=AUTO_SIZE_X<nl>
//0x0002=AUTO_SIZE_Y<nl>
//0x0004=KEEP_LEFT<nl>
//0x0008=KEEP_TOP
void SRGraphLegend::SetAutoSize(int sizebits)
{
	m_bAutoSizeX=sizebits & 1 ? TRUE : FALSE;
	m_bAutoSizeY=sizebits & 2 ? TRUE : FALSE;
	m_bKeepLeft=sizebits & 4 ? TRUE : FALSE;
	m_bKeepTop=sizebits & 8 ? TRUE : FALSE;
}

//@mfunc Initialises the width of the graphic key on legends.
//@parm This key size is interpreted by using the  m_nType member of the m_Rect data member and so is device independant
void SRGraphLegend::SetKeySize(double keysize)
{
	m_dKeySize=keysize;
}

//@mfunc Calculates the width in logical pixels of the key. 
//Uses the m_nType member of the m_Rect data member to perform this calculation.
//@rdesc Calculated width of the legend key graphic
int SRGraphLegend::CalcKeySize()
{
	ASSERT(IsDrawing()); // Only ever call this from within the draw loop otherwise m_pDC and m_pCWnd may be invalid
	
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return 30; // a sensible default for non debug versions.
	}

	int xRes=GetpDC()->GetDeviceCaps(LOGPIXELSX);

	switch(m_Rect.GetMeasurement())
	{
	case 0:
		return (int)m_dKeySize;
		break;
	case 1:
		return (int)(((double)xRes/25.4)*m_dKeySize);
	case 2:
		return (int)((double)xRes*m_dKeySize);
	}

	return 30;
}

	

//@mfunc Called by the DrawForeground function to fill in the keys.
// using a style which fits the graph type being used.
void SRGraphLegend::DrawKey(CPoint topleft,CPoint bottomright, int index, int group, int CurrentKey)
{
	SRGraphData *pD=GetParent()->GetGroup(group)->GetIndex(index);
	
	m_nCurrentIndex=CurrentKey;

	switch(m_Style.GetLegendKeyStyle())
	{
	case CX_KEY_NONE:
		return;
	case CX_KEY_AUTOMATIC:
		{
			if(m_pDisplay==NULL)
			{
				if(GetStyle()->GetLegendLine())
					DrawStyledLine(CPoint(topleft.x,topleft.y+((bottomright.y-topleft.y)/2)),CPoint(topleft.x,topleft.y+((bottomright.y-topleft.y)/2)),pD->GetStyle(),CurrentKey);
				else
					DrawBar(topleft,bottomright,group,index,pD->GetStyle(),0);
			}
			else
			{
				switch(m_pDisplay->GetStyle()->GetGraphStyle())
				{
				case CX_GRAPH_LINE:
				case CX_GRAPH_STOCK:
				case CX_GRAPH_OSCILLISCOPE:
				case CX_GRAPH_WEB:
				case CX_GRAPH_AREAWEB:
				case CX_GRAPH_XYSCATTERA:
				case CX_GRAPH_XYSCATTERI:
				case CX_GRAPH_XYSCATTERG:
					//Has the possibility of drawing a wiget.
					if(GetStyle()->GetLegendLine())
						DrawStyledLine(CPoint(topleft.x,topleft.y+((bottomright.y-topleft.y)/2)),CPoint(bottomright.x,topleft.y+((bottomright.y-topleft.y)/2)),pD->GetStyle(),CurrentKey);;
					switch(pD->GetStyle()->GetObjectStyle())
					{
					case CX_OBJECT_LINE:
						break; // done it already.
					default:
						DrawWiget(CPoint(topleft.x+((bottomright.x-topleft.x)/2),topleft.y+((bottomright.y-topleft.y)/2)),pD->GetStyle());
					}
					break;
				default:
				case CX_GRAPH_AREA:
				case CX_GRAPH_STACK_VBAR:
				case CX_GRAPH_STACK_HBAR:
				case CX_GRAPH_EX_VBAR:
				case CX_GRAPH_EX_HBAR:
				case CX_GRAPH_HBAR:
				case CX_GRAPH_VBAR:
				case CX_GRAPH_HISTOGRAM:
				case CX_GRAPH_ISO_BAR:
				case CX_GRAPH_ISO_AREA:
				case CX_GRAPH_RIBBON:
				case CX_GRAPH_STRATA:
				case CX_GRAPH_TIMESPAN:
				case CX_GRAPH_AREASTOCK:
				case CX_GRAPH_PIE:
				case CX_GRAPH_ISO_PIE:
					// Draws boxes..
					DrawBar(topleft,bottomright,group,index,pD->GetStyle(),0);
					break;
				}
			}
		}
		break;
	case CX_KEY_LINE:
		DrawStyledLine(CPoint(topleft.x,topleft.y+((bottomright.y-topleft.y)/2)),CPoint(bottomright.x,topleft.y+((bottomright.y-topleft.y)/2)),pD->GetStyle(),CurrentKey);
		break;
	case CX_KEY_BOX:
		DrawBar(topleft,bottomright,group,index,pD->GetStyle(),0);
		break;
	case CX_KEY_WIGET:
		DrawWiget(CPoint(topleft.x+((bottomright.x-topleft.x)/2),topleft.y+((bottomright.y-topleft.y)/2)),pD->GetStyle());
		break;
	default:
	case CX_KEY_CUSTOM:
		DrawCustomKey(topleft,bottomright,group,index,pD->GetStyle());
		break;
	}

}

void SRGraphLegend::DrawCustomKey(CPoint topleft,CPoint bottomright,int group,int index,SRGraphStyle *ptrStyle)
{
	topleft;	//Unused
	bottomright;//Unused
	group;		//Unused
	index;		//Unused
	ptrStyle;	//Unused

	ASSERT(FALSE); //You must override this function to draw your custom keys
}

//@mfunc Used to write the "legend" name to the template file
//@parm Pointer to the CStdioFile used in writing this template
void SRGraphLegend::WriteName(CStdioFile *file)
{
	file->WriteString("legend ");
}

//@mfunc Calculates the Y axis scale and prepares the display before showing the graph.
//@see <mf SRGraphDisplay::PrepareDisplay>
void SRGraphDisplay::PrepareYScale()
{

	if(m_Style.GetUseMaxScale()) // max scale overrides automatic scaling
	{
		m_ScaleY+=m_dMaxRangeY;
		m_ScaleY+=m_dMinRangeY;
		return;
	}

	for(int CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{
		for(int xDiv=0;xDiv<m_nDivCount;xDiv++)
		{
			SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(xDiv+m_nMinDiv);
			if(!pD->GetNull())
			{
				if(m_bHiLo)
				{
					m_ScaleY+=pD->GetHiLoData().Min();
					m_ScaleY+=pD->GetHiLoData().Max();
				}
				m_ScaleY+=pD->GetValue();
			}
		}
	}
	
}

//@mfunc Prepares both X and Y axis scales before scatter graphs are drawn.
//@see <mf SRGraphDisplay::PrepareDisplay>
void SRGraphDisplay::PrepareScatter()
{
	int CurrentSet,x;

	if(m_Style.GetUseMaxScale()) // max scale overrides automatic scaling
	{
		m_ScaleY+=m_dMaxRangeY;
		m_ScaleY+=m_dMinRangeY;
		m_ScaleX+=m_dMaxRangeX;
		m_ScaleX+=m_dMinRangeX;
		return;
	}

	switch(GetStyle()->GetGraphStyle())
	{
	case CX_GRAPH_XYSCATTERA:
		{
			int nTest=m_nSetCount*m_nDivCount;
			if(nTest&1)
				SRThrowGraphException(SRGraphException::odd);
			BOOL bXorY=FALSE;
			for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			{
				for(x=0;x<m_nDivCount;x++)
				{
					SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE);
					if(pD->GetNull())
						SRThrowGraphException(SRGraphException::null);
					double dValue=pD->GetValue();
					if(!bXorY)
						m_ScaleX+=dValue;
					else
						m_ScaleY+=dValue;
					bXorY=~bXorY;
				}
			}
		}
		break;
	case CX_GRAPH_XYSCATTERI:
		if(m_nDivCount&1)
			SRThrowGraphException(SRGraphException::odd);
		for(x=0;x<m_nDivCount;x+=2)
		{
			for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			{
				SRGraphData *pDX=GetParent()->GetGroup(m_nMinSet+CurrentSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE),
							*pDY=GetParent()->GetGroup(m_nMinSet+CurrentSet,FALSE)->GetIndex(x+m_nMinDiv+1,FALSE);
				if(!pDX->GetNull() && !pDY->GetNull())
				{
					m_ScaleX+=pDX->GetValue();
					m_ScaleY+=pDY->GetValue();
				}
				else if(pDX->GetNull() || pDY->GetNull())
					SRThrowGraphException(SRGraphException::null);
			}
		}
		break;
	case CX_GRAPH_XYSCATTERG:
		if(m_nSetCount&1)
			SRThrowGraphException(SRGraphException::odd);
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet+=2)
		{
			for(x=0;x<m_nDivCount;x++)
			{
				SRGraphData *pDX=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(m_nMinDiv+x,FALSE),
							*pDY=GetParent()->GetGroup(CurrentSet+m_nMinSet+1,FALSE)->GetIndex(m_nMinDiv+x,FALSE);
				if(!pDX->GetNull() && !pDY->GetNull())
				{
					m_ScaleX+=pDX->GetValue();
					m_ScaleY+=pDY->GetValue();
				}
				else if(pDX->GetNull() || pDY->GetNull())
					SRThrowGraphException(SRGraphException::null);
			}
		}
		break;
	}
}

//@mfunc This virtual overridable is provided to allow you to add your own preparation routine. If preparation fails due to an 
//un recognised graph type the main preparation loop will call this function to allow you to prepare scales et before exiting via the
//return path in PrepareDisplay
void SRGraphDisplay::PrepareCustom()
{
	ASSERT(FALSE); // You must override this function to perform some useful tasks.
}

//@mfunc prepares the display for stage and platform style graphs
//@see <mf SRGraphDisplay::PrepareDisplay>
void SRGraphDisplay::PrepareIsoBar()
{
	if(!GetStyle()->DRIsLocked())
	{
		int nRSize=(int)(GetStyle()->GetUseMaxSize() ? GetStyle()->GetMaxObjectDepth() : 10);
		m_DisplayRect.right-=nRSize+1;
		m_DisplayRect.top+=nRSize+1;
	}
	PrepareYScale();
}


//@mfunc prepares the display before data is drawn.
//@rem Different graph types require different preparation. For example a standard line graph needs only the Y scale to be calculated whereas the XY scatter graphs will require X and Y scales.
// This routine will call the proper preparation routine for the graph style being handled. Note that an extention feature is available.
// To invoke this routine you need only to specify a style which is not recognised by this function. Any number may be used for your own style but
// you are strongly recommended to use only those above the reserved threshold. Normally styles are a two character hexadecimal code. These codes therefore have
// 255 possible combinations (code 00 is always CX_{SOMESTYLE}_NONE) and values from 01 to BF are reserved. You may therefore safely use C0 to FF for your own purposes.
void SRGraphDisplay::PrepareDisplay()
{
	DWORD dwAxisStyle;

	switch(GetStyle()->GetGraphStyle())
	{
	case CX_GRAPH_NONE:
		return;
	case CX_GRAPH_STOCK:
		SetHiLo(TRUE);
	case CX_GRAPH_LINE:
	case CX_GRAPH_HBAR:
	case CX_GRAPH_VBAR:
	case CX_GRAPH_PIE:
	case CX_GRAPH_HISTOGRAM:
	case CX_GRAPH_OSCILLISCOPE:
	case CX_GRAPH_ISO_BAR:
	case CX_GRAPH_ISO_AREA:
	case CX_GRAPH_ISO_PIE:
	case CX_GRAPH_RIBBON:
	case CX_GRAPH_WEB:
	case CX_GRAPH_TIMESPAN:
	case CX_GRAPH_AREAWEB:
	case CX_GRAPH_AREASTOCK:
	case CX_GRAPH_FREESTYLE:
	case CX_GRAPH_HILO_OPENCLOSE:
	case CX_GRAPH_CANDLE:
	case CX_GRAPH_AREA:
	case CX_GRAPH_GANTT:
		PrepareYScale();
		break;
	case CX_GRAPH_STRATA:
		PrepareStrata();
		break;
	case CX_GRAPH_STACK_VBAR:
	case CX_GRAPH_STACK_HBAR:
		dwAxisStyle=GetStyle()->GetAxisStyle();
		if(dwAxisStyle==CX_AXIS_ISO_CLASSIC || dwAxisStyle==CX_AXIS_ISO_CLASSIC_YX)
			PrepareIsoBar();
		else
			PrepareYScale();
		break;
	case CX_GRAPH_EX_VBAR:
	case CX_GRAPH_EX_HBAR:
		PrepareIsoBar();
		break;
	case CX_GRAPH_XYSCATTERA:
	case CX_GRAPH_XYSCATTERI:
	case CX_GRAPH_XYSCATTERG:
		PrepareScatter();
		break;
	case CX_GRAPH_POLAR:
	case CX_GRAPH_AREAPOLAR:
		PreparePolar();
		break;
	default:
		PrepareCustom();
		break;
	}
}

//@mfunc Called by the Draw function to display the graph data.
//@devnote The foreground begins by covering the whole of the components internal rectangle. 
//This rectangle is shrunk a little to keep it away from the borders if they exist and then reduced in size automatically 
//by the axis and label drawing function. What is left over is available for data display and the various 
//line, pie and bar routines make sensible scales from the resultant size of the m_DisplayRect member.<nl>
void SRGraphDisplay::DrawForeground()
{	// draw sub components of this graph display
	
	if(GetpDC()->IsPrinting() || m_bSignificantChange )
		MakeFont=TRUE;
	
	if(MakeFont)
		UpdateFont();

	if(!GetpDC()->IsPrinting())
		MakeFont=FALSE;

	BOOL bUnZoomX=FALSE,bUnZoomY=FALSE;

	m_LabelRectX=m_LabelRectY=CRect(0,0,0,0);			// labels get null rectangles to start with.

	if(!GetStyle()->DRIsLocked())
		m_DisplayRect=m_Rect.CalcCRect(m_pDC,m_pCWnd);	// and the display gets the full size of the component...
	
	int thickness;

	switch(m_Style.GetComponentBorderStyle())
	{
	case CX_NO_BORDER:
		thickness=0;
		break;
	case CX_THIN_RAISED_BORDER:
	case CX_THIN_SUNKEN_BORDER:
		thickness=1;
		break;
	case CX_THIN_3D_BORDER:
		thickness=2;
		break;
	case CX_THICK_RAISED_BORDER:
	case CX_THICK_SUNKEN_BORDER:
		thickness=3;
		break;
	case CX_THICK_3D_BORDER:
		thickness=6;
		break;
	case CX_LINE_BORDER:
		thickness=1;
		break;
	}

	m_nMinSet=m_nMinGroup==-1 ? 0 : m_nMinGroup;
	m_nMaxSet=m_nMaxGroup==-1 ? GetParent()->GetGroupCount()-1 : m_nMaxGroup;
	m_nMinDiv=m_nMinIndex==-1 ? 0 : m_nMinIndex;
	m_nMaxDiv=m_nMaxIndex==-1 ? GetParent()->GetMaxXDivisions()-1 : m_nMaxIndex;

	m_nSetCount=1+(m_nMaxSet-m_nMinSet);
	m_nDivCount=1+(m_nMaxDiv-m_nMinDiv);

	m_ScaleY.Zero();
	m_ScaleX.Zero();


	PrepareDisplay();
	
	if(GetStyle()->GetAlwaysShowZero())
	{
		m_ScaleY+=0; // You may think this is silly but you'd be wrong. Perhaps the optimizer thinks its silly too. Hmmm.
		m_ScaleX+=0;
	}

	// The scale needs to be adjusted to the nearest sensible value

	if(!GetStyle()->GetUseMaxScale())
	{
		if(GetStyle()->GetLog())
		{
			CScale tScale;
			tScale+=m_ScaleY.Min()>1 ? floor(log10(m_ScaleY.Min())) : 0;
			tScale+=0.778151250+ceil(log10(m_ScaleY.Max()));
			m_dMaxScaleY=tScale.Range();
			m_ScaleY.Zero();
			m_ScaleY+=pow(10.0,tScale.Max());
			m_ScaleY+=pow(10.0,tScale.Min());

 			tScale.Zero();
			tScale+=m_ScaleX.Min()>1 ? floor(log10(m_ScaleX.Min())) : 0;
			tScale+=0.778151250+ceil(log10(m_ScaleX.Max()));
			m_dMaxScaleX=tScale.Range();
			m_ScaleX.Zero();
			m_ScaleX+=pow(10.0,tScale.Max());
			m_ScaleX+=pow(10.0,tScale.Min());

		}
		else
		{

			m_ScaleY+=1.1*m_ScaleY.Max();
			m_ScaleY+=1.1*m_ScaleY.Min();
			m_ScaleX+=1.1*m_ScaleX.Max();
			m_ScaleX+=1.1*m_ScaleX.Min();

			m_dMaxScaleY=m_ScaleY.Range();
			m_dMaxScaleX=m_ScaleX.Range();
		}
	}
	else
	{
		m_dMaxScaleY=m_ScaleY.Range();
		m_dMaxScaleX=m_ScaleX.Range();
	}
		
	if(m_ZoomScaleX.IsClean())
	{
		m_ZoomScaleX=m_ScaleX;
		bUnZoomX=TRUE;
	}

	if(m_ZoomScaleY.IsClean())
	{
		m_ZoomScaleY=m_ScaleY;
		bUnZoomY=TRUE;
	}

	if(GetStyle()->GetLog())
	{
		m_ZoomScaleX+=pow(10.0,floor(log10(m_ZoomScaleX.Min())));
 		m_ZoomScaleY+=pow(10.0,floor(log10(m_ZoomScaleY.Min())));
		m_ZoomScaleX+=pow(10.0,ceil(log10(m_ZoomScaleX.Max())));
		m_ZoomScaleX+=pow(10.0,ceil(log10(m_ZoomScaleX.Max())));
	}

	// All drawing is to be confined to the data display so we set up a clipping region..

	CRgn DisplayClipRgn;
	HRGN hRgn=::CreateRectRgn(0,0,0,0);
	int nRestoreRgn=::GetClipRgn(m_pDC->m_hDC,hRgn); // First save the old one (its a pity CDC dosent have a member for this yet)

	m_pDC->SelectClipRgn(&DisplayClipRgn);

	if(!GetStyle()->DRIsLocked())
		m_DisplayRect.DeflateRect((int)(thickness+(2*GetXMargin())),(int)(thickness+(2*GetYMargin())));					// minus the borders.. which makes the working area.

	if(m_DisplayRect.Width()<2 || m_DisplayRect.Height()<2)
		goto DrawExit;	// pointless to continue really!!	
		
	if(	!GetParent()->m_bHintPending || 
		(GetParent()->m_bHintPending && ((GetParent()->m_lHint & CX_FOREGROUND_MASK)==DISPLAY_LABELS)))
		DrawLabels(); // labels may or may not change the size of the display area usable

	DisplayClipRgn.CreateRectRgn(m_DisplayRect.left,m_DisplayRect.top,
		                         m_DisplayRect.right+1,m_DisplayRect.bottom+1);
	

	if(GetStyle()->GetGridOrder())
	{
		if(	!GetParent()->m_bHintPending || 
			(GetParent()->m_bHintPending && ((GetParent()->m_lHint & CX_FOREGROUND_MASK)==DISPLAY_AXES)))
			DrawAxis(); // draw the axis taking into account the label sizes
		if(	!GetParent()->m_bHintPending || 
			(GetParent()->m_bHintPending && ((GetParent()->m_lHint & CX_FOREGROUND_MASK)==DISPLAY_DATA)))
			DrawData(); // and superimpose the data.
	}
	else
	{
		if(	!GetParent()->m_bHintPending || 
			(GetParent()->m_bHintPending && ((GetParent()->m_lHint & CX_FOREGROUND_MASK)==DISPLAY_DATA)))
			DrawData(); // Draw the data.
		if(	!GetParent()->m_bHintPending || 
			(GetParent()->m_bHintPending && ((GetParent()->m_lHint & CX_FOREGROUND_MASK)==DISPLAY_AXES)))
			DrawAxis(); // and superimpose the axis
	}

DrawExit:

	if(nRestoreRgn==1)
		::SelectClipRgn(m_pDC->m_hDC,hRgn);
	else
		m_pDC->SelectClipRgn(NULL);

	if(bUnZoomX)
		m_ZoomScaleX.Zero();

	if(bUnZoomY)
		m_ZoomScaleY.Zero();
}

//@mfunc Draws the labels in a style sutable for the axis in use.
//@rem This routine like many of the top-level routines is extendable. If you use a label style which is not recognised the routine DrawCustomLabels is invoked.
// This routine allows you to perform your own label drawing routine before finishing via the return path in this routine.
// To invoke this routine you need only to specify a style which is not recognised by this function. Any number may be used for your own style but
// you are strongly recommended to use only those above the reserved threshold. Normally styles are a two character hexadecimal code. These codes therefore have
// 255 possible combinations (code 00 is always CX_{SOMESTYLE}_NONE) and values from 01 to BF are reserved. You may therefore safely use C0 to FF for your own purposes.
void SRGraphDisplay::DrawLabels()
{
	switch(m_Style.GetAxisStyle())
	{
	case CX_AXIS_AUTOMATIC:
		switch(m_Style.GetGraphStyle())
		{
		default:
		case CX_GRAPH_LINE:
		case CX_GRAPH_VBAR:
		case CX_GRAPH_STACK_VBAR:
		case CX_GRAPH_STRATA:
		case CX_GRAPH_AREA:
		case CX_GRAPH_FREESTYLE:
		case CX_GRAPH_CANDLE:
			DrawClassicLabels();
			break;
		case CX_GRAPH_XYSCATTERA:
		case CX_GRAPH_XYSCATTERI:
		case CX_GRAPH_XYSCATTERG:
			DrawScatterLabels();
			break;
		case CX_GRAPH_PIE:
		case CX_GRAPH_ISO_PIE:
			DrawPieLabels();
			break;
		case CX_GRAPH_ISO_BAR:
		case CX_GRAPH_ISO_AREA:
			DrawStageLabels();
			break;
		case CX_GRAPH_HBAR:
		case CX_GRAPH_STACK_HBAR:
		case CX_GRAPH_TIMESPAN:
		case CX_GRAPH_EX_HBAR:
			DrawYXLabels();
			break;
		case CX_GRAPH_WEB:
		case CX_GRAPH_AREAWEB:
			DrawWebLabels();
			break;
		case CX_GRAPH_POLAR:
		case CX_GRAPH_AREAPOLAR:
			DrawPolarLabels();
			break;
		case CX_GRAPH_GANTT:
			DrawGanttLabels();
			break;
		}
	case CX_AXIS_NONE:
		return; // dont draw any labels
	case CX_AXIS_CLASSIC:
	case CX_AXIS_ISO_CLASSIC:
	case CX_AXIS_ISO_CLASSIC_R:
	case CX_AXIS_CLASSIC_RIGHT:
	case CX_AXIS_DATE:
	case CX_AXIS_DATE_R:
	case CX_AXIS_ISO_DATE:
	case CX_AXIS_ISO_DATE_R:
			DrawClassicLabels();
		break;
	case CX_AXIS_ISO_CLASSIC_YX:
	case CX_AXIS_YX:
	case CX_AXIS_DATE_YX:
	case CX_AXIS_ISO_DATE_YX:
		DrawYXLabels();
		break;
	case CX_AXIS_OSCILLISCOPE:
		//DrawOscilliscopeLabels();
		break;
	case CX_AXIS_PLATFORM_SE:
	case CX_AXIS_PLATFORM_SW:
	case CX_AXIS_STAGE_SE:
	case CX_AXIS_STAGE_SW:
		DrawStageLabels();
		break;
	case CX_AXIS_H_FRAME:
		//DrawHFrameLabels();
		break;
	case CX_AXIS_PIE:
		DrawPieLabels();
		break;
	case CX_AXIS_WEB:
		DrawWebLabels();
		break;
	case CX_AXIS_XYSCATTER:
		DrawScatterLabels();
		break;
	case CX_AXIS_POLAR:
		DrawPolarLabels();
		break;
	case CX_AXIS_GANTT_VALUE:
	case CX_AXIS_GANTT_DATE:
		DrawGanttLabels();
		break;
	default:
		DrawCustomLabels();
		break;
	}
}

//@mfunc Used to write the "data" name to the template file
//@parm Pointer to the CStdioFile used in writing this template
void SRGraphDisplay::WriteName(CStdioFile *file)
{
	file->WriteString("data ");
}

//@mfunc Draws the axis in the requested style
//@rem This routine like many of the top-level routines is extendable. If you use a label style which is not recognised the routine DrawCustomAxis is invoked.
// This routine allows you to perform your own label drawing routine before finishing via the return path in this routine.
// To invoke this routine you need only to specify a style which is not recognised by this function. Any number may be used for your own style but
// you are strongly recommended to use only those above the reserved threshold. Normally styles are a two character hexadecimal code. These codes therefore have
// 255 possible combinations (code 00 is always CX_{SOMESTYLE}_NONE) and values from 01 to BF are reserved. You may therefore safely use C0 to FF for your own purposes.
void SRGraphDisplay::DrawAxis()
{
	switch(m_Style.GetAxisStyle())
	{
	case CX_AXIS_PIE:
	case CX_AXIS_NONE:
		return; // dont draw an axis
	case CX_AXIS_AUTOMATIC:
		{
			switch(m_Style.GetGraphStyle())
			{
			case CX_GRAPH_LINE:
			case CX_GRAPH_VBAR:
			case CX_GRAPH_STACK_VBAR:
			case CX_GRAPH_STRATA:
			case CX_GRAPH_AREA:
			case CX_GRAPH_FREESTYLE:
			case CX_GRAPH_CANDLE:
			case CX_GRAPH_HILO_OPENCLOSE:
				DrawClassicAxis();
				break;
			case CX_GRAPH_XYSCATTERA:
			case CX_GRAPH_XYSCATTERI:
			case CX_GRAPH_XYSCATTERG:
				DrawScatterAxis();
				break;
			case CX_GRAPH_PIE:
				break;
			case CX_GRAPH_ISO_BAR:
			case CX_GRAPH_ISO_AREA:
				DrawStageAxis();
				break;
			case CX_GRAPH_HBAR:
			case CX_GRAPH_STACK_HBAR:
			case CX_GRAPH_TIMESPAN:
			case CX_GRAPH_GANTT:
				DrawYXAxis();
				break;
			case CX_GRAPH_WEB:
			case CX_GRAPH_AREAWEB:
				DrawWebAxis();
			case CX_GRAPH_EX_VBAR:
			case CX_GRAPH_EX_HBAR:
				DrawIsoClassicAxis();
				break;
			case CX_GRAPH_POLAR:
			case CX_GRAPH_AREAPOLAR:
				DrawPolarAxis();
				break;
			}
		}
		break;
	case CX_AXIS_CLASSIC:
	case CX_AXIS_CLASSIC_RIGHT:
	case CX_AXIS_DATE:
	case CX_AXIS_DATE_R:
	case CX_AXIS_GANTT_VALUE:
	case CX_AXIS_GANTT_DATE:
		DrawClassicAxis();
		break;
	case CX_AXIS_YX:
	case CX_AXIS_DATE_YX:
		DrawYXAxis();
		break;
	case CX_AXIS_OSCILLISCOPE:
		//DrawOscilliscopeAxis();
		break;
	case CX_AXIS_PLATFORM_SE:
	case CX_AXIS_PLATFORM_SW:
		DrawStageAxis();
		break;
	case CX_AXIS_STAGE_SE:
	case CX_AXIS_STAGE_SW:
		DrawStageAxis();
		break;
	case CX_AXIS_H_FRAME:
		//DrawHFrameAxis();
		break;
	case CX_AXIS_WEB:
		DrawWebAxis();
		break;
	case CX_AXIS_XYSCATTER:
		DrawScatterAxis();
		break;
	case CX_AXIS_POLAR:
		DrawPolarAxis();
		break;
	case CX_AXIS_ISO_CLASSIC:
	case CX_AXIS_ISO_CLASSIC_YX:
	case CX_AXIS_ISO_CLASSIC_R:
	case CX_AXIS_ISO_DATE:
	case CX_AXIS_ISO_DATE_R:
	case CX_AXIS_ISO_DATE_YX:
		DrawIsoClassicAxis();
		break;
	default:
		DrawCustomAxis();
		break;
	}
}

//@mfunc Draws the labels associated with scatter graphs
void SRGraphDisplay::DrawScatterLabels()
{


	if(m_ZoomScaleY.Max()==0 && m_ZoomScaleY.Min()==0 && m_ZoomScaleX.Max()==0 && m_ZoomScaleX.Min()==0)
		return;

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	if(m_ptrXLabels==NULL)
	{
		if(GetStyle()->GetLog())			
			m_ptrXLabels=new SRGLogScale;
		else
 			m_ptrXLabels=new SRGDecimalScale;
	}
	ASSERT(m_ptrXLabels!=NULL);

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())			
			m_ptrYLabels=new SRGLogScale;
		else
 			m_ptrYLabels=new SRGDecimalScale;
	}
	ASSERT(m_ptrYLabels!=NULL);

	SRGDecimalScale *yLabels=(SRGDecimalScale *)m_ptrYLabels,
					*xLabels=(SRGDecimalScale *)m_ptrXLabels;
	
	xLabels->SetParent(GetParent());
	xLabels->SetFontSize(GetFontSize());
	xLabels->SetFaceName(GetFaceName());
	xLabels->SetTextColor(GetTextColor());
	xLabels->SetFontStyle(GetFontStyle());
	xLabels->SetMinValue(m_ZoomScaleX.Min());
	xLabels->SetMaxValue(m_ZoomScaleX.Max());
	xLabels->SetResolution(0.1);
	xLabels->SetLabelSide(TRUE);
	xLabels->SetLogTickPositions(TRUE);
	xLabels->SetCheckOverlaps(FALSE);
	xLabels->SetptrTickPositionList(&m_XTickList);
	xLabels->SetAscending();

	yLabels->SetParent(GetParent());
	yLabels->SetFontSize(GetFontSize());
	yLabels->SetFaceName(GetFaceName());
	yLabels->SetTextColor(GetTextColor());
	yLabels->SetFontStyle(GetFontStyle());
	yLabels->SetMinValue(m_ZoomScaleY.Min());
	yLabels->SetMaxValue(m_ZoomScaleY.Max());
	yLabels->SetResolution(0.1);
	yLabels->DrawingUsing(m_pDC,m_pCWnd);
	yLabels->SetLabelSide(FALSE);
	yLabels->SetLogTickPositions(TRUE);
	yLabels->SetCheckOverlaps(FALSE);
	yLabels->SetptrTickPositionList(&m_YTickList);
	yLabels->SetAscending();

	for(int pass=0;pass<2;pass++)
	{
		if(m_Style.GetShowIndices()) // We will interpret this style as ShowX
		{
			xLabels->SetStartPos(CPoint((int)m_DisplayRect.left,(int)m_DisplayRect.bottom+5));
			xLabels->SetEndPos(CPoint((int)m_DisplayRect.right,(int)m_DisplayRect.bottom+5));
			xLabels->DrawingUsing(m_pDC,m_pCWnd);
			xLabels->CreateLabels();
			xLabels->SetPass(pass);
			xLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
				m_DisplayRect.bottom-=xLabels->GetBlockRect().Height()+5;

		}

		if(m_Style.GetShowNumerals())
		{
			yLabels->SetStartPos(CPoint(m_DisplayRect.left,m_DisplayRect.bottom));
			yLabels->SetEndPos(CPoint(m_DisplayRect.left,m_DisplayRect.top));
			yLabels->DrawingUsing(m_pDC,m_pCWnd);
			yLabels->CreateLabels();
			yLabels->SetPass(pass);
			yLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
				m_DisplayRect.left+=yLabels->GetBlockRect().Width()+5;
		}
	}
}

//@mfunc Draws the axes associated with scatter graphs
void SRGraphDisplay::DrawScatterAxis()
{
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0 && m_ScaleX.Max()==0 && m_ScaleX.Min()==0)
		return;

	DrawClassicAxis();
}


//@mfunc Draws scatter data in one of three formats
//@rem The formats are..<nl>
//CX_GRAPH_XYSCATTERA. This format requires an even number of data items and the X,Y data is taken from them alternateley. The first one will be the X data the second the Y and so on. 
//If the number of indices in a group is uneven the graph will treat the first entry on each alternate line as Y data but will always stick to x,y,x,y format. For example a graph data structure may 
// resemble...<nl><nl>X Y X Y X Y X<nl>Y X Y X Y X Y<nl>X Y X Y X Y X<nl>Y X Y X Y X Y<nl>
//<nl>
//CX_GRAPH_XYSCATTERG will treat even groups as X data, and odd groups as Y data so the structure may look like....<nl>
//X X X X X X<nl>
//Y Y Y Y Y Y<nl>
//X X X X X X<nl>
//Y Y Y Y Y Y<nl>
//<nl>
//Finally the CX_GRAPH_XYSCATTERI treats the even indices as Y and odd indices as Y..<nl>
//X Y X Y<nl>
//X Y X Y<nl>
//X Y X Y<nl>
//X Y X Y<nl>
//X Y X Y<nl>
//X Y X Y<nl>
//<nl>
void SRGraphDisplay::DrawScatterData()
{
	if(m_ZoomScaleY.Max()==0 && m_ZoomScaleY.Min()==0 && m_ZoomScaleX.Max()==0 && m_ZoomScaleX.Min()==0)
		return;

	//Scatter plots are the only ones which can draw outside of the axis area for zooming purposes
	//A clipping region is made to contain the data

	HRGN hRgn=::CreateRectRgn(0,0,0,0);
	int nRestoreRgn=::GetClipRgn(m_pDC->m_hDC,hRgn); // First save the old one (its a pity CDC dosent have a member for this yet)

	CRgn DisplayClipRgn;
	DisplayClipRgn.CreateRectRgn(m_DisplayRect.left,m_DisplayRect.top,
		                         m_DisplayRect.right+1,m_DisplayRect.bottom+1);
	m_pDC->SelectClipRgn(&DisplayClipRgn);

	double XScalingFactor,YScalingFactor;

	if(GetStyle()->GetLog())
	{
		XScalingFactor=((double)m_DisplayRect.Width())/log10(m_ZoomScaleX.Range());
		YScalingFactor=((double)m_DisplayRect.Height())/log10(m_ZoomScaleY.Range());
	}
	else
	{
		XScalingFactor=((double)m_DisplayRect.Width())/m_ZoomScaleX.Range();
		YScalingFactor=((double)m_DisplayRect.Height())/m_ZoomScaleY.Range();
	}

	int CurrentSet,x,xGroup,xIndex;
	BOOL bXorY,bFirstPoint=TRUE,bLog=GetStyle()->GetLog();
	double dX,dY;

	CPoint lastpoint;
	SRGraphData *xpD;

	BOOL bLogData=GetStyle()->GetLog();

	switch(GetStyle()->GetGraphStyle())
	{
	case CX_GRAPH_XYSCATTERA:
		bXorY=FALSE;
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			m_nCurrentIndex=(CurrentSet+m_nMinSet);
			for(x=0;x<m_nDivCount;x++)
			{
				SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE);
				if(pD->GetNull())
					SRThrowGraphException(SRGraphException::null);
				double dValue=pD->GetValue(bLogData);
				if(bXorY)
					dY=dValue;
				else
				{
					xGroup=CurrentSet+m_nMinSet;
					xIndex=x+m_nMinDiv;
					xpD=pD;
					dX=dValue;
				}
				bXorY=~bXorY;
				if(!bXorY)
				{
					if(bLogData)
					{
						dX-=log10(m_ZoomScaleX.Min());
						dY-=log10(m_ZoomScaleY.Min());
					}
					else
					{
						dX-=m_ZoomScaleX.Min();
						dY-=m_ZoomScaleY.Min();
					}

					CPoint datapoint=CPoint((int)((double)m_DisplayRect.left+(XScalingFactor*dX)),
											(int)((double)m_DisplayRect.bottom-(YScalingFactor*dY)));
					GetParent()->LogPoint(datapoint,xGroup,xIndex);
					if(bFirstPoint)
					{
						DrawWiget(datapoint,xpD->GetStyle());
						bFirstPoint=FALSE;
					}
					else
						DrawPointObject(lastpoint,datapoint,xpD->GetStyle(),1);
					lastpoint=datapoint;
				}																					 
			}
		}
		break;
	case CX_GRAPH_XYSCATTERI:
		for(x=0;x<m_nDivCount;x+=2)
		{

			bFirstPoint=TRUE;
			m_nCurrentIndex=(x+m_nMinDiv);
			for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			{
				SRGraphData *pDX=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE);
				SRGraphData *pDY=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv+1,FALSE);
				if(pDX->GetNull() || pDY->GetNull())
					SRThrowGraphException(SRGraphException::null);
				double dValueX=pDX->GetValue(bLogData);
				double dValueY=pDY->GetValue(bLogData);
				if(bLogData)
				{
					dValueX-=log10(m_ZoomScaleX.Min());
					dValueY-=log10(m_ZoomScaleY.Min());
				}
				else
				{
					dValueX-=m_ZoomScaleX.Min();
					dValueY-=m_ZoomScaleY.Min();
				}

				CPoint datapoint=CPoint((int)(m_DisplayRect.left+(XScalingFactor*dValueX)),(int)(m_DisplayRect.bottom-(YScalingFactor*dValueY)));

				GetParent()->LogPoint(datapoint,CurrentSet+m_nMinSet,x+m_nMinDiv);
				if(bFirstPoint)
				{
					DrawWiget(datapoint,pDX->GetStyle());
					bFirstPoint=FALSE;
				}
				else
					DrawPointObject(lastpoint,datapoint,pDX->GetStyle(),1);
				lastpoint=datapoint;
			}
		}
		break;
	case CX_GRAPH_XYSCATTERG:
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet+=2)
		{
			bFirstPoint=TRUE;
			m_nCurrentIndex=CurrentSet+m_nMinSet;
			for(x=0;x<m_nDivCount;x++)
			{
				SRGraphData *pDX=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE);
				SRGraphData *pDY=GetParent()->GetGroup(CurrentSet+m_nMinSet+1,FALSE)->GetIndex(x+m_nMinDiv,FALSE);
				if(pDX->GetNull() || pDY->GetNull())
					SRThrowGraphException(SRGraphException::null);
				double dValueX=pDX->GetValue(bLogData);
				double dValueY=pDY->GetValue(bLogData);
				if(bLogData)
				{
					dValueX-=log10(m_ZoomScaleX.Min());
					dValueY-=log10(m_ZoomScaleY.Min());
				}
				else
				{
					dValueX-=m_ZoomScaleX.Min();
					dValueY-=m_ZoomScaleY.Min();
				}
				CPoint datapoint=CPoint((int)(m_DisplayRect.left+(XScalingFactor*dValueX)),(int)(m_DisplayRect.bottom-(YScalingFactor*dValueY)));
				GetParent()->LogPoint(datapoint,CurrentSet+m_nMinSet,x+m_nMinDiv);
				if(bFirstPoint)
				{
					DrawWiget(datapoint,pDX->GetStyle());
					bFirstPoint=FALSE;
				}
				else
					DrawPointObject(lastpoint,datapoint,pDX->GetStyle(),1);
				lastpoint=datapoint;
			}
		}
		break;
	}

	if(nRestoreRgn==1)
		::SelectClipRgn(m_pDC->m_hDC,hRgn);
	else
		m_pDC->SelectClipRgn(NULL);


}

//@mfunc Draws a line plot of the data within the area left over after placement of axes and labels
void SRGraphDisplay::DrawLineData()
{
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bLogData=m_Style.GetLog();
	
	CPen datapen;

	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY; 
	
	int CurrentSet,x;
	for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{
		
		CPoint datapoint,lastpoint;

		BOOL FirstPoint=TRUE;

		m_nCurrentIndex=(CurrentSet+m_nMinSet);

		SRGraphDataList *pDL=GetParent()->GetGroup(CurrentSet+m_nMinSet);

		double dValue,xStep=((double)m_DisplayRect.Width()/(m_nDivCount>1 ? (m_nDivCount-1) : 1));

		for(x=0;x<m_nDivCount;x++)
		{
			SRGraphData *pgd=pDL->GetIndex(x+m_nMinDiv);

			if(!pgd->GetNull())
			{
				dValue=pgd->GetValue(bLogData)-(bLogData ? (m_ScaleY.Min()>1 ? log10(m_ScaleY.Min()) : 0) : m_ScaleY.Min());
				datapoint=CPoint((int)(xStep*x)+m_DisplayRect.left,
								 (int)((double)m_DisplayRect.bottom-(ScalingFactor*dValue)));
				GetParent()->LogPoint(datapoint,CurrentSet+m_nMinSet,x+m_nMinDiv);
				//determine the point type
				switch(pgd->GetStyle()->GetObjectStyle())
				{
				case CX_OBJECT_NONE:
					break;
				case CX_OBJECT_LINE:
drawLine:		
					if(!FirstPoint)
						DrawStyledLine(lastpoint,datapoint,pgd->GetStyle(),m_nCurrentIndex);
					break;
				default:
				case CX_OBJECT_AUTOMATIC:
				case CX_OBJECT_EXPOINT:
					DrawWiget(datapoint,pgd->GetStyle());
					goto drawLine;
					break;
				case CX_OBJECT_POINT:
 					DrawWiget(datapoint,pgd->GetStyle());
					break;
				}
				lastpoint=datapoint;
				FirstPoint=FALSE;
			}
		}

	}

}

//@mfunc Draws data in the style defined in the parent graph. 
//@rem This routine like many of the top-level routines is extendable. If you use a label style which is not recognised the routine DrawCustomData is invoked.
// That function allows you to perform your own data drawing routine before finishing via the return path in this routine.
// To invoke the custom routine you need only to specify a style which is not recognised by this function. Any number may be used for your own style but
// you are strongly recommended to use only those above the reserved threshold. Normally styles are a two character hexadecimal code. These codes therefore have
// 255 possible combinations (code 00 is always CX_{SOMESTYLE}_NONE) and values from 01 to BF are reserved. You may therefore safely use C0 to FF for your own purposes.
void SRGraphDisplay::DrawData()
{



	switch(m_Style.GetGraphStyle())
	{
	case CX_GRAPH_NONE:
		break;
	case CX_GRAPH_LINE:
		DrawLineData();
		break;
	case CX_GRAPH_EX_HBAR:
	case CX_GRAPH_HBAR:
		DrawHBarData();
		break;
	case CX_GRAPH_EX_VBAR:
	case CX_GRAPH_VBAR:
		DrawVBarData();
		break;
	case CX_GRAPH_ISO_BAR:
		DrawIsoBarData();
		break;
	case CX_GRAPH_RIBBON:
	case CX_GRAPH_ISO_AREA:
		DrawIsoAreaData();
		break;
	case CX_GRAPH_STACK_HBAR:
		DrawStackHBarData();
		break;
	case CX_GRAPH_STACK_VBAR:
		DrawStackVBarData();
		break;
	case CX_GRAPH_ISO_PIE:
	case CX_GRAPH_PIE:
		DrawIsoPieData();
		break;
	case CX_GRAPH_STRATA:
		DrawStrataData();
		break;
	case CX_GRAPH_STOCK:
	case CX_GRAPH_AREASTOCK:
		DrawStockData();
		break;
	case CX_GRAPH_WEB:
	case CX_GRAPH_AREAWEB:
		DrawWebData();
		break;
	case CX_GRAPH_XYSCATTERA:
	case CX_GRAPH_XYSCATTERI:
	case CX_GRAPH_XYSCATTERG:
		DrawScatterData();
		break;
	case CX_GRAPH_FREESTYLE:
		DrawFreeStyleData();
		break;
	case CX_GRAPH_CANDLE:
	case CX_GRAPH_HILO_OPENCLOSE:
		DrawCandleData();
		break;
	case CX_GRAPH_AREA:
		DrawAreaData();
		break;
	case CX_GRAPH_AREAPOLAR:
	case CX_GRAPH_POLAR:
		DrawPolarData();
		break;
	case CX_GRAPH_GANTT:
		DrawGanttData();
		break;
	default:
		DrawCustomData();
		break;
	}
}

//@mfunc Displays the classic style X Y axis for the iso bar type graphs.
void SRGraphDisplay::DrawIsoClassicAxis()
{
	CPen GridPen,FramePen,*OldPen;
	CBrush FillBrush,*OldBrush;

	int nDepth=(int)(GetStyle()->GetUseMaxSize() ? GetStyle()->GetMaxObjectDepth() : 10);

	FillBrush.CreateSolidBrush(GetStyle()->GetColor());

	if(GetStyle()->GetAxisFillStyle())
		OldBrush=(CBrush *)m_pDC->SelectObject(&FillBrush);
	else
		OldBrush=(CBrush *)m_pDC->SelectStockObject(NULL_BRUSH);

	FramePen.CreatePen(PS_SOLID,GetStyle()->GetFrameWidth(),GetStyle()->GetFrameColor());
	OldPen=(CPen *)m_pDC->SelectObject(&FramePen);

	CPoint points[5];

	m_pDC->Rectangle(m_DisplayRect.left+nDepth,m_DisplayRect.top-nDepth,m_DisplayRect.right+1+nDepth,m_DisplayRect.bottom+1-nDepth); // CDC::Rectangle really is the most annoying function ever!

	points[0]=CPoint(m_DisplayRect.left,m_DisplayRect.top);
	points[1]=points[0]+CPoint(nDepth,-nDepth);
	points[2]=points[1]+CPoint(0,m_DisplayRect.Height());
	points[3]=points[2]+CPoint(-nDepth,nDepth);
	points[4]=points[0];
	m_pDC->Polygon(&points[0],5);

	points[0]=CPoint(m_DisplayRect.left,m_DisplayRect.bottom);
	points[1]=points[0]+CPoint(nDepth,-nDepth);
	points[2]=points[1]+CPoint(m_DisplayRect.Width(),0);
	points[3]=points[2]+CPoint(-nDepth,nDepth);
	points[4]=points[0];
	m_pDC->Polygon(&points[0],5);

	DWORD dwAxisStyle=GetStyle()->GetAxisStyle();

	if(GetStyle()->GetShowYTicks() && !m_YTickList.IsEmpty())
	{
		for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
			switch(dwAxisStyle)
			{
			case CX_AXIS_ISO_CLASSIC:
			case CX_AXIS_ISO_DATE:
				m_pDC->MoveTo(m_DisplayRect.left,pP->y);
				m_pDC->LineTo(m_DisplayRect.left-(int)GetTickSize(),pP->y);
				break;
			case CX_AXIS_ISO_DATE_R:
			case CX_AXIS_ISO_CLASSIC_R:
				m_pDC->MoveTo(m_DisplayRect.right+nDepth,pP->y);
				m_pDC->LineTo(m_DisplayRect.right+nDepth+(int)GetTickSize(),pP->y);
				break;
			}
		}
	}

	if(GetStyle()->GetShowXTicks() && !m_XTickList.IsEmpty())
	{
		for(POSITION p=m_XTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
			m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
			m_pDC->LineTo(pP->x,m_DisplayRect.bottom+(int)GetTickSize());
		}
	}

	GridPen.CreatePen(PS_SOLID,1,GetStyle()->GetGridColor());
	m_pDC->SelectObject(&GridPen);

	if(GetStyle()->GetShowYGrid() && !m_YTickList.IsEmpty())
	{
		for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
			switch(dwAxisStyle)
			{
			case CX_AXIS_ISO_CLASSIC:
			case CX_AXIS_ISO_DATE:
				m_pDC->MoveTo(m_DisplayRect.left,pP->y);
				m_pDC->LineTo(m_DisplayRect.left+nDepth,pP->y-nDepth);
				m_pDC->LineTo(m_DisplayRect.right+nDepth,pP->y-nDepth);
				break;
			case CX_AXIS_ISO_CLASSIC_R:
			case CX_AXIS_ISO_DATE_R:
				m_pDC->MoveTo(m_DisplayRect.right+nDepth,pP->y);
				m_pDC->LineTo(m_DisplayRect.left+nDepth,pP->y);
				m_pDC->LineTo(m_DisplayRect.left,pP->y+nDepth);
				break;
			}
		}

	}

	if(GetStyle()->GetShowXGrid() && !m_XTickList.IsEmpty())
	{
		for(POSITION p=m_XTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
			switch(dwAxisStyle)
			{
			case CX_AXIS_ISO_DATE_YX:
			case CX_AXIS_ISO_CLASSIC_YX:
			case CX_AXIS_ISO_CLASSIC:
				m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
				m_pDC->LineTo(pP->x+nDepth,m_DisplayRect.bottom-nDepth);
				m_pDC->LineTo(pP->x+nDepth,m_DisplayRect.top-nDepth);
				break;
			case CX_AXIS_ISO_CLASSIC_R:
				m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
				m_pDC->LineTo(pP->x-nDepth,m_DisplayRect.bottom-nDepth);
				m_pDC->LineTo(pP->x-nDepth,m_DisplayRect.top-nDepth);
				break;
			}
		}
	}

	m_pDC->SelectObject(OldPen);
	m_pDC->SelectObject(OldBrush);

	FramePen.DeleteObject();
	GridPen.DeleteObject();

	FillBrush.DeleteObject();

}

//@mfunc Draws Freestyle graphs.
//@rem A Freestle 	graph is one in which the data items themselves are programmed with individual object styles. By setting up or even dynamically changing the SRGraphData style
//for each item you can change the apperance of a graph. One series may have lines and wigets. Another series may have bar graphs and a third series could have polygonal pictograph bars.
void SRGraphDisplay::DrawFreeStyleData()
{

	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY;

	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Width())/m_nDivCount,(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Width()/m_nDivCount)/1.1;
	
	double dBlockWidth=(double)m_DisplayRect.Width()/m_nDivCount;
	double dSetOffset=(dBlockWidth/2)-((m_nSetCount*BarWidth)/2);

	BOOL bLog=GetStyle()->GetLog();

	for(int CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{
		BOOL FirstPoint=TRUE;
		CPoint lastpoint;

		m_nCurrentIndex=CurrentSet+m_nMinSet;

		for(int x=0;x<m_nDivCount;x++)
		{
			SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE);

			int top,left,bottom,right,center;

			// Calculate all the possible permutations based on max widths etc
			double dData=pD->GetValue(bLog);
			if(bLog)
				dData-=log10(m_ScaleY.Min());
			else
				dData-=m_ScaleY.Min();

			top=(int)(m_DisplayRect.bottom-(ScalingFactor*(dData)));
			bottom=(int)(m_DisplayRect.bottom-(ScalingFactor*(bLog ? 0 : -m_ScaleY.Min())));
			center=(int)(m_DisplayRect.left+(x*dBlockWidth)+(dBlockWidth/2));
			left=(int)(center-(BarWidth/2));
			right=(int)(center+(BarWidth/2));
			
			
			if(!pD->GetNull())
			{
				SRGraphStyle *ptrStyle=pD->GetStyle();
				DWORD dwObjectStyle=ptrStyle->GetObjectStyle();
				switch(dwObjectStyle)
				{
				default:
				case CX_OBJECT_NONE:
					break;
				case CX_OBJECT_BAR:
				case CX_OBJECT_EXBAR:
				case CX_OBJECT_POLYGON:
					DrawBar(CPoint(left,top),CPoint(right,bottom),CurrentSet+m_nMinSet,x+m_nMinDiv,ptrStyle,0);
					break;
				case CX_OBJECT_LINE:
				case CX_OBJECT_EXLINE:
				case CX_OBJECT_POINT:
				case CX_OBJECT_EXPOINT:
					if(!FirstPoint)
						DrawStyledLine(lastpoint,CPoint(center,top),ptrStyle,CurrentSet+m_nMinSet);
					GetParent()->LogPoint(CPoint(center,top),CurrentSet+m_nMinSet,x+m_nMinDiv);
					DrawWiget(CPoint(center,top),ptrStyle);
					break;
				}
				FirstPoint=FALSE;
				lastpoint=CPoint(center,top);
			}
		}
	}
}


//@mfunc Draws the classic X,Y axes. Automatically scales to ensure that data fits into the display area.
void SRGraphDisplay::DrawClassicAxis()
{
	CPen GridPen,FramePen,*OldPen;
	CBrush FillBrush,*OldBrush;

	FillBrush.CreateSolidBrush(GetStyle()->GetColor());

	if(GetStyle()->GetAxisFillStyle())
		OldBrush=(CBrush *)m_pDC->SelectObject(&FillBrush);
	else
		OldBrush=(CBrush *)m_pDC->SelectStockObject(NULL_BRUSH);

	FramePen.CreatePen(PS_SOLID,GetStyle()->GetFrameWidth(),GetStyle()->GetFrameColor());
	OldPen=(CPen *)m_pDC->SelectObject(&FramePen);

	m_pDC->Rectangle(m_DisplayRect.left,m_DisplayRect.top,m_DisplayRect.right+1,m_DisplayRect.bottom+1); // CDC::Rectangle really is the most annoying function ever!

	if(GetStyle()->GetShowYTicks() && !m_YTickList.IsEmpty())
	{
		for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
			DWORD dwAxisStyle=GetStyle()->GetAxisStyle();

			switch(dwAxisStyle)
			{
			default:
				m_pDC->MoveTo(m_DisplayRect.left,pP->y);
				m_pDC->LineTo(m_DisplayRect.left-(int)GetTickSize(),pP->y);
				break;
			case CX_AXIS_CLASSIC_RIGHT:
				m_pDC->MoveTo(m_DisplayRect.right,pP->y);
				m_pDC->LineTo(m_DisplayRect.right+(int)GetTickSize(),pP->y);
				break;
			}
		}
	}

	if(GetStyle()->GetShowXTicks() && !m_XTickList.IsEmpty())
	{
		for(POSITION p=m_XTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
			m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
			m_pDC->LineTo(pP->x,m_DisplayRect.bottom+(int)GetTickSize());
		}
	}

	GridPen.CreatePen(PS_SOLID,1,GetStyle()->GetGridColor());
	m_pDC->SelectObject(&GridPen);

	if(GetStyle()->GetShowYGrid() && !m_YTickList.IsEmpty())
	{
		for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
			m_pDC->MoveTo(m_DisplayRect.left,pP->y);
			m_pDC->LineTo(m_DisplayRect.right,pP->y);
		}

	}

	if(GetStyle()->GetShowXGrid() && !m_XTickList.IsEmpty())
	{
		for(POSITION p=m_XTickList.GetHeadPosition();p!=NULL;)
		{
			CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
			m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
			m_pDC->LineTo(pP->x,m_DisplayRect.top);
		}
	}

	m_pDC->SelectObject(OldPen);
	m_pDC->SelectObject(OldBrush);

	FramePen.DeleteObject();
	GridPen.DeleteObject();

	FillBrush.DeleteObject();

}

//@mfunc Draws the axes suitable for horizontal bars. Automatically scales to ensure that data fits into the display area.
void SRGraphDisplay::DrawYXAxis()
{
	DrawClassicAxis(); // Use of label blocks has made the drawing of these two axes identical
}


//@mfunc Draws vertical bar chart data. Automatically sizes bars according to the number of groups that exist and 
//the number of data members within the groups.
void SRGraphDisplay::DrawVBarData()
{
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bLogData=m_Style.GetLog();

	DWORD dwGraphType=GetStyle()->GetGraphStyle();
	DWORD dwAxisType=GetStyle()->GetAxisStyle();
	
	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY;
	
	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Width())/(m_nSetCount*m_nDivCount),(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Width()/(m_nSetCount*m_nDivCount))/1.1;
	
	double dBlockWidth=(double)m_DisplayRect.Width()/m_nDivCount;
	double dSetOffset=(dBlockWidth/2)-((m_nSetCount*BarWidth)/2);

	int CurrentSet,x;

	// Note These now draw in reverse group / index order for the benefit of the Extended bar styles
	// If they draw in ascending order the bars overlap strangely

	double dData;

	for(x=0;x<m_nDivCount;x++)
	{
		int top,left,bottom,right;

		CRect BarRect;

		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			SRGraphDataList *gdl=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE);
			ASSERT(gdl!=NULL);
			SRGraphData *pD=gdl->GetIndex(x+m_nMinDiv);
			if(!pD->GetNull())
			{
				SRGraphStyle *ptrStyle=pD->GetStyle();
				if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE) // if there is no individual style defined
					ptrStyle=gdl->GetStyle();;					// pick up a global style for the group
				dData=pD->GetValue(bLogData);
				if(bLogData)
					dData-=log10(m_ScaleY.Min());
				else
					dData-=m_ScaleY.Min();

				m_nCurrentIndex=(CurrentSet+m_nMinSet);
				left=(int)((x*dBlockWidth)+dSetOffset+(CurrentSet*BarWidth)+m_DisplayRect.left);
				right=(int)(left+BarWidth);
				top=(int)((double)m_DisplayRect.bottom-(ScalingFactor*dData)+0.5);
				if(bLogData)
					bottom=m_DisplayRect.bottom;
				else
					bottom=(int)((double)top+(ScalingFactor*pD->GetValue(bLogData))+0.5);
								 
				DrawBar(CPoint(left,top),CPoint(right,bottom),CurrentSet+m_nMinSet,x+m_nMinDiv,ptrStyle,(dwGraphType==CX_GRAPH_EX_VBAR || dwAxisType==CX_AXIS_ISO_CLASSIC || dwAxisType==CX_AXIS_ISO_CLASSIC_R) ? CX_OBJECT_EXBAR : 0);				
			}
		}
	}
}

//@mfunc Displays labels on the classic axes. Labels are taken from the annotations 
//stored in the SRGraphDataList and SRGraphData objects.
void SRGraphDisplay::DrawClassicLabels()
{
	int NodeAdjust=0;
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0)
		return;

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	if(m_ptrXLabels==NULL)
		m_ptrXLabels=new SRGraphLabelBlock;

	ASSERT(m_ptrXLabels!=NULL);

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())			
			m_ptrYLabels=new SRGLogScale;
		else
		{
			switch(GetStyle()->GetAxisStyle())
			{
			default:
			case CX_AXIS_CLASSIC:
			case CX_AXIS_CLASSIC_RIGHT:
			case CX_AXIS_ISO_CLASSIC:
			case CX_AXIS_ISO_CLASSIC_R:
 				m_ptrYLabels=new SRGDecimalScale;
				break;
			case CX_AXIS_DATE:
			case CX_AXIS_DATE_R:
			case CX_AXIS_ISO_DATE:
			case CX_AXIS_ISO_DATE_R:
 				m_ptrYLabels=new SRGDateScale;
				break;

			}
		}
	}

	ASSERT(m_ptrYLabels!=NULL);

	switch(m_Style.GetGraphStyle())
	{
	case CX_GRAPH_LINE:
	case CX_GRAPH_STRATA:
	case CX_GRAPH_AREASTOCK:
	case CX_GRAPH_AREA:
		NodeAdjust=1;
		break;
	default:
		NodeAdjust=0;
		break;
	}

	SRGDecimalScale *yLabels=(SRGDecimalScale *)m_ptrYLabels;
	SRGraphLabelBlock *xLabels=m_ptrXLabels;

	xLabels->SetParent(GetParent());
	yLabels->SetParent(GetParent());

	
	CPoint ptStart,ptEnd;

	for(int pass=0;pass<2;pass++)
	{
		if(m_Style.GetShowIndices())
		{

 			xLabels->DrawingUsing(m_pDC,m_pCWnd);

			if(m_bSignificantChange)
			{
				xLabels->KillLabels();
				for(int x=0;x<m_nDivCount;x++)
					xLabels->AddLabel(GetParent()->GetGroup(0)->GetIndex(m_nMinDiv+x)->GetAnnotation(GetCountryCode()));
			}

			if(NodeAdjust)
			{
				SRGraphLabel * pL;
				if(!pass)
				{
					switch(m_Style.GetAxisStyle())
					{
					case CX_AXIS_CLASSIC_RIGHT:
					case CX_AXIS_ISO_CLASSIC_R:
					case CX_AXIS_DATE_R:
					case CX_AXIS_ISO_DATE_R:
						pL=xLabels->GetLabelIndex(SRGraphLabelBlock::LabelFirst);
						m_DisplayRect.left+=pL->GetLabelSize().cx/2;
						break;
					default:
						pL=xLabels->GetLabelIndex(SRGraphLabelBlock::LabelLast);
						m_DisplayRect.right-=pL->GetLabelSize().cx/2;
						break;
					}
				}
				ptStart=CPoint(m_DisplayRect.left,m_DisplayRect.bottom+5);
				ptEnd=CPoint(m_DisplayRect.right,m_DisplayRect.bottom+5);				
			}
			else
			{
				ptStart=CPoint(m_DisplayRect.left+((m_DisplayRect.Width()/(m_nDivCount))/2),m_DisplayRect.bottom+5);
				ptEnd=CPoint(m_DisplayRect.right-((m_DisplayRect.Width()/(m_nDivCount))/2),m_DisplayRect.bottom+5);
			}


			if(m_Style.GetTierXLabels())
				xLabels->GetStyle()->SetLabelStyle(CX_LABELBLOCK_MOVELABELS);
			else
				xLabels->GetStyle()->SetLabelStyle(CX_LABELBLOCK_HIDELABELS);
			xLabels->SetLogTickPositions(TRUE);
			xLabels->SetptrTickPositionList(&m_XTickList);
			xLabels->SetFontSize(GetFontSize());
			xLabels->SetFaceName(GetFaceName());
			xLabels->SetTextColor(GetTextColor());
			xLabels->SetFontStyle(GetFontStyle());
			xLabels->SetBlockLocationPoints(SRGraphLabel::TopCenter);
			xLabels->SetBlockPositions(ptStart,ptEnd);
			xLabels->SetBlockVectors(0,1);
			xLabels->SetPass(pass);
			xLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
			{
				CRect br=xLabels->GetBlockRect();
				m_DisplayRect.bottom-=br.Height();
			}
		}

		if(m_Style.GetShowNumerals())
		{


			if(m_Style.GetTierYLabels())
				yLabels->GetStyle()->SetLabelStyle(CX_LABELBLOCK_MOVELABELS);
			else
				yLabels->GetStyle()->SetLabelStyle(CX_LABELBLOCK_HIDELABELS);
			yLabels->SetFontSize(GetFontSize());
			yLabels->SetFaceName(GetFaceName());
			yLabels->SetTextColor(GetTextColor());
			yLabels->SetFontStyle(GetFontStyle());
			yLabels->SetMinValue(m_ScaleY.Min());
			yLabels->SetMaxValue(m_ScaleY.Max());
			yLabels->DrawingUsing(m_pDC,m_pCWnd);

			int nDepth;

			switch(m_Style.GetAxisStyle())
			{
			default:
			case CX_AXIS_CLASSIC:
			case CX_AXIS_DATE:
				ptStart=CPoint(m_DisplayRect.left-5,m_DisplayRect.bottom);
				ptEnd=CPoint(m_DisplayRect.left-5,m_DisplayRect.top);
				yLabels->SetLabelSide(FALSE);
				break;
			case CX_AXIS_ISO_CLASSIC_R:
			case CX_AXIS_ISO_DATE_R:
				nDepth=(int)(GetStyle()->GetUseMaxSize() ? GetStyle()->GetMaxObjectDepth() : 10);
				ptStart=CPoint(m_DisplayRect.right+5+nDepth,m_DisplayRect.bottom-nDepth);
				ptEnd=CPoint(m_DisplayRect.right+5+nDepth,m_DisplayRect.top-nDepth);
				yLabels->SetLabelSide(TRUE);
				break;
			case CX_AXIS_CLASSIC_RIGHT:
			case CX_AXIS_DATE_R:
				ptStart=CPoint(m_DisplayRect.right+5,m_DisplayRect.bottom);
				ptEnd=CPoint(m_DisplayRect.right+5,m_DisplayRect.top);
				yLabels->SetLabelSide(TRUE);
				break;
			}

			yLabels->SetLogTickPositions(TRUE);
			yLabels->SetptrTickPositionList(&m_YTickList);
			yLabels->SetStartPos(ptStart);
			yLabels->SetEndPos(ptEnd);
			yLabels->SetAscending();
			yLabels->CreateLabels();
			yLabels->SetPass(pass);
			yLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
			{
				switch(GetStyle()->GetAxisStyle())
				{
				default:
				case CX_AXIS_CLASSIC:
					m_DisplayRect.left+=yLabels->GetBlockRect().Width()+5;
					break;
				case CX_AXIS_CLASSIC_RIGHT:
				case CX_AXIS_ISO_CLASSIC_R:
					m_DisplayRect.right-=yLabels->GetBlockRect().Width()+5;
					break;
				}
			}

		}
	}

}

//@mfunc Draws standard pie chart data.
//@rem This routine is superceeded by the <mf SRGraphDisplay::DrawIsoPieData> routine
void SRGraphDisplay::DrawPieData()
{
	DrawIsoPieData();
}


//@mfunc Constructor. m_nMaxSet the m_nType member to IDS_SRG_BACKGROUNDTYPE
SRGraphBackground::SRGraphBackground()
{
	m_nType=IDS_SRG_BACKGROUNDTYPE;
}





//@mfunc Constructor
SRGraphDisplay::SRGraphDisplay()
{
	m_bDisplayDrawn=FALSE;
	m_nType=IDS_SRG_DISPLAYTYPE;
	m_ptrXLabels=m_ptrYLabels=m_ptrZLabels=NULL;
}


//@mfunc Draws labels on pie charts. To avoid confusion this draw routine will not allow
//two labels to overlap. If labels printed in the default positions would overlap then the rectangle 
//containing the text is moved away from the center of the pie chart intil they are in a clear area.
void SRGraphDisplay::DrawPieLabels()
{

	SRGraphLabelBlock LabelBlock;	// Added at beta 2 for the new labelling system

	LabelBlock.SetParent(GetParent());
	LabelBlock.SetCheckOverlaps(TRUE);

	LabelBlock.DrawingUsing(m_pDC,m_pCWnd);

	// labels are now drawn by a special component class (SRGraphLabel)
	// the LabelBlock manages label seperation and vectoring.

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	double total=GetParent()->GetDataTotal(m_nMinSet,m_nMinDiv,m_nMaxSet,m_nMaxDiv); // get the sum of all the data
	
	DWORD GraphType=m_Style.GetGraphStyle();

	if(!total)
		return;	//dont bother to process a null graph

	//m_DisplayRect contains the area available to pie chart data

	//First we use this data to set the text clipping rectangle
	
	CRgn rgn;
	rgn.CreateRectRgnIndirect(&m_DisplayRect);

	HRGN hRgn=::CreateRectRgn(0,0,0,0);
	int nRestoreRgn=::GetClipRgn(m_pDC->m_hDC,hRgn);
	
	GetpDC()->SelectClipRgn(&rgn);

	rgn.DeleteObject(); // We can delete it straight away because CDC copies it.

	//In the case of ordinary pie charts, we ensure that the display rectangle is in fact a square.
	/*if(m_DisplayRect.Width()!=m_DisplayRect.Height() && GraphType!=CX_GRAPH_ISO_PIE)
	{
		m_DisplayRect.DeflateRect(	(m_DisplayRect.Width() < m_DisplayRect.Height() ? 0 : ((m_DisplayRect.Width()-m_DisplayRect.Height())/2)),
									(m_DisplayRect.Width() > m_DisplayRect.Height() ? 0 : ((m_DisplayRect.Height()-m_DisplayRect.Width())/2)));
	}
	*/

	double dBasicRadius=min(0.6*m_DisplayRect.Width()/2 , 0.6*m_DisplayRect.Height()/2);
	
	double radius1=(double)dBasicRadius;

	double radius2= GraphType==CX_GRAPH_ISO_PIE ? sin(RADIANS(m_Style.GetXAngle()))*dBasicRadius : radius1;

	double dSkirtHeight = GraphType==CX_GRAPH_ISO_PIE ? ( cos(RADIANS(m_Style.GetXAngle())) * (m_Style.GetUseMaxSize() ? m_Style.GetMaxObjectHeight() : radius2/2)) : 0;
	
	int cx=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		cy=(int)(m_DisplayRect.top+(m_DisplayRect.Height()/2-(dSkirtHeight/2))); // find the central point of the pie


	double angle=0;	// begin at the zero degree mark

	for(int CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{
		for(int x=0;x<m_nDivCount;x++)
		{	// for every item which exists
			
			//calculate the arc swept by the pie section
			SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
			if(!pD->GetNull())
			{
				double Value=pD->GetValue();
				double arc=(TWO_PI/total)*Value;
				double midarc=arc/2; // The middle of the arc is the ident point of the wedge
				SRGraphStyle *myStyle,*ptrStyle;
				
				myStyle=ptrStyle=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv)->GetStyle();

				CString text,buffer;

				switch(ptrStyle->GetLabelStyle())
				{
				case CX_LABEL_NONE:
					buffer.Empty();
					break;
				default:
				case CX_LABEL_VALUE_PERCENT:
				case CX_LABEL_AUTOMATIC:
					text.Format("%g (%.2g%%)",Value,100.0/total*Value);
					break;
				case CX_LABEL_VALUE:
					text.Format("%g",Value);
					break;
				case CX_LABEL_PERCENT:
					text.Format("%.2g%%",100.0/total*Value);
					break;
				case CX_LABEL_TEXT:
					text.Format("%s",GetParent()->GetText(CurrentSet+m_nMinSet,x+m_nMinDiv));
					break;
				case CX_LABEL_TEXT_PERCENT:
					text.Format("%s (%.2g%%)",GetParent()->GetText(CurrentSet+m_nMinSet,x+m_nMinDiv),100.0/total*Value);
					break;
				case CX_LABEL_TEXT_VALUE:
					text.Format("%s (%g)",GetParent()->GetText(CurrentSet+m_nMinSet,x+m_nMinDiv),Value);
					break;
				case CX_LABEL_TEXT_VALUE_PERCENT:
					text.Format("%s=%g (%.2g%%)",GetParent()->GetText(CurrentSet+m_nMinSet,x+m_nMinDiv),Value,100.0/total*Value);
					break;
				}

				//myStyle currently holds the style for the individual data entry.
				//We do a quick check for highlighting

				SRGraphLabel *pL=new SRGraphLabel;
				LabelBlock.AddLabel(pL);

				pL->SetAnnotation((LPCSTR)text);

				double dHighlightExtra=1.1;

				if(myStyle->GetHighlight()!=CX_HIGHLIGHT_NONE)
				{
					// and act accordingly if needed
					switch(myStyle->GetHighlight())
					{
					case CX_HIGHLIGHT_SMALL:
						dHighlightExtra=1.15;
						break;
					case CX_HIGHLIGHT_MEDIUM:
					case CX_HIGHLIGHT_AUTOMATIC:
						dHighlightExtra=1.2;
						break;
					case CX_HIGHLIGHT_LARGE:
						dHighlightExtra=1.3;
						break;
					default:
						ASSERT(FALSE); // You have used a highlight value which was'nt on the list
						break;
					}

				}

				CPoint TextPoint((int)(cx+((3+(radius1*dHighlightExtra))*cos(angle+midarc))),
								 (int)(cy+((3+(radius2*dHighlightExtra))*sin(angle+midarc))));
				
				TextPoint.y+=(GraphType==CX_GRAPH_ISO_PIE && TextPoint.y>cy) ? (int)dSkirtHeight : 0;

				pL->SetTextPosition(TextPoint);
				
				double dLocationPoint=DEGREES((angle+midarc));

				if(dLocationPoint<0)
					dLocationPoint+=360;

				pL->SetLocationPoint(((int)(dLocationPoint/45)) & 0x07);

				pL->SetVector(DEGREES((angle+midarc)));

				pL->SetFontStyle(GetFontStyle());
				pL->SetFaceName(GetFaceName());
				pL->SetFontSize(GetFontSize());
				pL->SetJustification(GetJustification());
				pL->SetTextColor(GetTextColor());


				angle+=arc; // update the starting position for the next run
			}
		}
	}

	LabelBlock.Draw(m_pDC,m_pCWnd);

	if(nRestoreRgn==1)
		::SelectClipRgn(GetpDC()->m_hDC,hRgn);
	else
		GetpDC()->SelectClipRgn(NULL);

}

//@mfunc Draws the 3/4 stage type axes and the platform axes. Automatically scales to accomodate for 
//data values and object physical sizes as specified in the m_Style parameters.
void SRGraphDisplay::DrawStageAxis()
{
	CPen p,*OldPen;
	CBrush b,*OldBrush;
	
	// draws a 3/4 view stage axis from south east or south west viewpoints

	// work out a regtangle which gives a nice view for the aspect ratio of 
	// the display rectangle

	CRect BackPlane;
	int ExtraOffset=0;

	ExtraOffset=StageCalcSize(&BackPlane);


	CPoint poly[5];

	p.CreatePen(PS_SOLID,1,m_Style.GetFrameColor());
	b.CreateSolidBrush(m_Style.GetColor());

	OldPen=(CPen *)GetpDC()->SelectObject(&p);
	if(m_Style.GetAxisFillStyle())
		OldBrush=(CBrush *)GetpDC()->SelectObject(&b);
	else
		OldBrush=(CBrush *)GetpDC()->SelectStockObject(NULL_BRUSH);

	DWORD TheStyle=m_Style.GetAxisStyle();

	switch(TheStyle)
	{
	case CX_AXIS_PLATFORM_SE:
	case CX_AXIS_STAGE_SE:

		BackPlane.OffsetRect(ExtraOffset,0);	// offset vertically by the width of the margin

		if(TheStyle==CX_AXIS_STAGE_SE)
		{
			GetpDC()->Rectangle(BackPlane);
		
			poly[0]=CPoint(BackPlane.right,BackPlane.top);
			poly[1]=CPoint(BackPlane.right,BackPlane.bottom);
			poly[2]=CPoint(m_DisplayRect.right-ExtraOffset,m_DisplayRect.bottom);
			poly[3]=CPoint(m_DisplayRect.right-ExtraOffset,m_DisplayRect.bottom-BackPlane.Height());
			poly[4]=poly[0];

			//GetpDC()->Polygon((LPPOINT)poly.GetData(),5);
			GetpDC()->Polygon((LPPOINT)&poly,5);

			if(GetStyle()->GetShowYTicks() && !m_YTickList.IsEmpty())
			{
				for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
					m_pDC->MoveTo(pP->x+(int)GetTickSize(),pP->y);
					m_pDC->LineTo(pP->x-(int)GetTickSize(),pP->y);
				}
			}


			if(GetStyle()->GetShowYGrid() && !m_YTickList.IsEmpty())
			{
				CPen GridPen(PS_SOLID,1,GetStyle()->GetGridColor());
				CPen *ogp=(CPen *)m_pDC->SelectObject(&GridPen);

				for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
					m_pDC->MoveTo(m_DisplayRect.right-ExtraOffset,pP->y);
					m_pDC->LineTo(BackPlane.right,BackPlane.bottom-(m_DisplayRect.bottom-pP->y));
					m_pDC->LineTo(BackPlane.left,BackPlane.bottom-(m_DisplayRect.bottom-pP->y));
				}

				m_pDC->SelectObject(ogp);
			}

			if(GetStyle()->GetShowXGrid() && !m_XTickList.IsEmpty())
			{
				CPen GridPen(PS_SOLID,1,GetStyle()->GetGridColor());
				CPen *ogp=(CPen *)m_pDC->SelectObject(&GridPen);

				for(POSITION p=m_XTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
					m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
					m_pDC->LineTo(pP->x-(m_DisplayRect.Width()-BackPlane.Width()),BackPlane.bottom);
					m_pDC->LineTo(pP->x-(m_DisplayRect.Width()-BackPlane.Width()),BackPlane.top);
				}

				m_pDC->SelectObject(ogp);
			}

			if(GetStyle()->GetShowZGrid() && !m_ZTickList.IsEmpty())
			{
				CPen GridPen(PS_SOLID,1,GetStyle()->GetGridColor());
				CPen *ogp=(CPen *)m_pDC->SelectObject(&GridPen);

				for(POSITION p=m_ZTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
					m_pDC->MoveTo(pP->x+10,pP->y);
					m_pDC->LineTo(pP->x+10+BackPlane.Width(),pP->y);
					m_pDC->LineTo(pP->x+10+BackPlane.Width(),pP->y-BackPlane.Height());
				}

				m_pDC->SelectObject(ogp);
			}
		}

		// And the bottom
		poly[0]=CPoint(BackPlane.right,BackPlane.bottom);
		poly[1]=CPoint(BackPlane.left,BackPlane.bottom);
		poly[2]=CPoint(m_DisplayRect.right-(ExtraOffset+BackPlane.Width()),m_DisplayRect.bottom);
		poly[3]=CPoint(m_DisplayRect.right-ExtraOffset,m_DisplayRect.bottom);
		poly[4]=poly[0];
		GetpDC()->Polygon((LPPOINT)&poly,5);
		break;

	case CX_AXIS_PLATFORM_SW:
	case CX_AXIS_STAGE_SW:
		
		BackPlane.OffsetRect((int)(ExtraOffset ? 
		/* if true */		 ExtraOffset+(m_nDivCount*m_Style.GetMaxObjectWidth()) : 
		/* else */			 m_DisplayRect.Width()-BackPlane.Width()),
							 0);

		if(TheStyle==CX_AXIS_STAGE_SW)
		{
			GetpDC()->Rectangle(BackPlane);
		

			poly[0]=CPoint(BackPlane.left,BackPlane.top);
			poly[1]=CPoint(BackPlane.left,BackPlane.bottom);
			poly[2]=CPoint(ExtraOffset+m_DisplayRect.left,m_DisplayRect.bottom);
			poly[3]=CPoint(ExtraOffset+m_DisplayRect.left,m_DisplayRect.bottom-BackPlane.Height());
			poly[4]=poly[0];
			GetpDC()->Polygon((LPPOINT)&poly,5);

			if(GetStyle()->GetShowYTicks() && !m_YTickList.IsEmpty())
			{
				for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
					m_pDC->MoveTo(pP->x+(int)GetTickSize(),pP->y);
					m_pDC->LineTo(pP->x-(int)GetTickSize(),pP->y);
				}
			}


			if(GetStyle()->GetShowYGrid() && !m_YTickList.IsEmpty())
			{
				CPen GridPen(PS_SOLID,1,GetStyle()->GetGridColor());
				CPen *ogp=(CPen *)m_pDC->SelectObject(&GridPen);

				for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
					m_pDC->MoveTo(m_DisplayRect.left+ExtraOffset,pP->y);
					m_pDC->LineTo(BackPlane.left,BackPlane.bottom-(m_DisplayRect.bottom-pP->y));
					m_pDC->LineTo(BackPlane.right,BackPlane.bottom-(m_DisplayRect.bottom-pP->y));
				}

				m_pDC->SelectObject(ogp);
			}

			if(GetStyle()->GetShowXGrid() && !m_XTickList.IsEmpty())
			{
				CPen GridPen(PS_SOLID,1,GetStyle()->GetGridColor());
				CPen *ogp=(CPen *)m_pDC->SelectObject(&GridPen);

				for(POSITION p=m_XTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
					m_pDC->MoveTo(pP->x,m_DisplayRect.bottom);
					m_pDC->LineTo(pP->x+(m_DisplayRect.Width()-BackPlane.Width()),BackPlane.bottom);
					m_pDC->LineTo(pP->x+(m_DisplayRect.Width()-BackPlane.Width()),BackPlane.top);
				}

				m_pDC->SelectObject(ogp);
			}

			if(GetStyle()->GetShowZGrid() && !m_ZTickList.IsEmpty())
			{
				CPen GridPen(PS_SOLID,1,GetStyle()->GetGridColor());
				CPen *ogp=(CPen *)m_pDC->SelectObject(&GridPen);

				for(POSITION p=m_ZTickList.GetHeadPosition();p!=NULL;)
				{
					CPoint * pP=(CPoint *)m_XTickList.GetNext(p);
					m_pDC->MoveTo(pP->x-10,pP->y);
					m_pDC->LineTo(pP->x-(10+BackPlane.Width()),pP->y);
					m_pDC->LineTo(pP->x-(10+BackPlane.Width()),pP->y-BackPlane.Height());
				}

				m_pDC->SelectObject(ogp);
			}
		}
		// And the bottom
		poly[0]=CPoint(BackPlane.right,BackPlane.bottom);
		poly[1]=CPoint(BackPlane.left,BackPlane.bottom);
		poly[2]=CPoint(ExtraOffset+m_DisplayRect.left,m_DisplayRect.bottom);
		poly[3]=CPoint(ExtraOffset+m_DisplayRect.left+BackPlane.Width(),m_DisplayRect.bottom);
		poly[4]=poly[0];
		GetpDC()->Polygon((LPPOINT)&poly,5);
		break;
	}

	GetpDC()->SelectObject(OldBrush);
	GetpDC()->SelectObject(OldPen);
	p.DeleteObject();
	b.DeleteObject();
}

//@mfunc Draws "Manhattan" type bar graphs.
//@rem Because Manhattan graphs can only display positive data any values which are negative 
//will cause this chart type to re-program itself to be a bar chart.
//@see <mf SRGraphDisplay::DrawVBarData>
void SRGraphDisplay::DrawIsoBarData()
{

	if(m_ScaleY.Min()==0 && m_ScaleY.Max()==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	if(m_ScaleY.Min()<0)
	{
		//Manhattan graphs cannot show negative data;
		m_Style.SetAxisStyle(CX_AXIS_CLASSIC);
		m_Style.SetGraphStyle(CX_GRAPH_VBAR);
		m_Style.SetLegendStyle(CX_LEGEND_AUTOMATIC);
		m_pCWnd->Invalidate();
		return;
	}

	CRect BackPlane;
	int ExtraOffset=0;

	ExtraOffset=StageCalcSize(&BackPlane);

	switch(m_Style.GetAxisStyle())
	{
	case CX_AXIS_PLATFORM_SE:
	case CX_AXIS_STAGE_SE:
		BackPlane.OffsetRect(ExtraOffset,0);
		break;
	case CX_AXIS_AUTOMATIC:
	case CX_AXIS_NONE:
	case CX_AXIS_PLATFORM_SW:
	case CX_AXIS_STAGE_SW:
		BackPlane.OffsetRect((int)(ExtraOffset ? 
		/* if true */		 ExtraOffset+(m_nDivCount*m_Style.GetMaxObjectWidth()) : 
		/* else */			 m_DisplayRect.Width()-BackPlane.Width()),
							 0);
		break;
	default:
		break;
	}

	
	CPoint poly[5];
	
	// The number of groups and the depth of the lists allow us to calculate
	// the size of the bars based upon the area of the remaining display space

	double BarWidth=ExtraOffset ? m_Style.GetMaxObjectWidth() : (0.6*m_DisplayRect.Width())/m_nDivCount,
		   BarDepth=ExtraOffset ? m_Style.GetMaxObjectDepth() : (0.4*m_DisplayRect.Width())/m_nSetCount;

	double ScalingFactor=BackPlane.Height()/m_dMaxScaleY;
	BOOL bLogData=m_Style.GetLog();

	int nOffsetX,nOffsetY;

	CBitmap shadeBitmap,glareBitmap;
	CBrush shadeBrush,glareBrush,*oldBrush;
	BOOL shading=m_Style.GetObjectShadeStyle();
	if(shading)
	{
		shadeBitmap.LoadBitmap(SRG_IDB_LIGHTSHADOW);
		shadeBrush.CreatePatternBrush(&shadeBitmap);
		glareBitmap.LoadBitmap(SRG_IDB_LIGHTGLARE);
		glareBrush.CreatePatternBrush(&glareBitmap);
	}


	GESorter *elements=new GESorter[m_nSetCount];

	for(int z=0;z<m_nSetCount;z++)
	{
		elements[z].value=GetParent()->GetGroupAverage(m_nMinDiv,m_nMaxDiv,z+m_nMinSet);
		elements[z].index=z;
	}

	if(GetStyle()->GetSort())
		qsort(elements,m_nSetCount,sizeof(GESorter),qsSorterCompareD);

	for(z=0;z<m_nSetCount;z++)
	{// step through the groups


		m_nCurrentIndex=elements[z].index;

		for(int x=0;x<m_nDivCount;x++)
		{//item by item

			nOffsetX=(int)((x+1)*BarWidth);
			nOffsetY=(int)((x+1)*((double)(m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount));

			SRGraphStyle *ptrStyle=GetParent()->GetGroup(elements[z].index+m_nMinSet)->GetIndex(x+m_nMinDiv)->GetStyle();
			if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE)
				ptrStyle=GetParent()->GetGroup(elements[z].index+m_nMinSet)->GetStyle();
			SRGraphData *pD=GetParent()->GetGroup(elements[z].index+m_nMinSet)->GetIndex(x+m_nMinDiv);
			if(!pD->GetNull())
			{
				double Value=pD->GetValue(bLogData);

				// To get the draw order right we need to know whether the graph is a "south west" or "south east" type
				switch(m_Style.GetAxisStyle())
				{
				case CX_AXIS_AUTOMATIC:
				case CX_AXIS_NONE:
				case CX_AXIS_PLATFORM_SW:
				case CX_AXIS_STAGE_SW:
					{
						//draw the front panel
						poly[0]=CPoint((int)((z*BarDepth)+BackPlane.left-nOffsetX),BackPlane.bottom+nOffsetY);
						poly[1]=CPoint((int)(poly[0].x+BarDepth),poly[0].y);
						poly[2]=CPoint(poly[1].x,(int)(poly[1].y-(Value*ScalingFactor)));
						poly[3]=CPoint(poly[0].x,poly[2].y);
						poly[4]=poly[0];
						GetParent()->LogPoly((LPPOINT)&poly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
						FillPoly((LPPOINT)&poly,5,ptrStyle,0);

						if(shading)
						{
							int OldMode=GetpDC()->SetROP2(R2_MASKPEN);
							CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
							oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush);
							GetpDC()->Polygon((LPPOINT)&poly,5);
							GetpDC()->SelectObject(oldBrush);
							GetpDC()->SetROP2(OldMode);
							GetpDC()->SelectObject(OldPen);
						}
						

						FramePoly((LPPOINT)&poly,5,ptrStyle,0);
						
						//draw the side panel

						poly[0]=poly[1];
						poly[1]=poly[2];
						poly[2]=CPoint((int)(poly[1].x+BarWidth),(int)(poly[1].y-((double)m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount));
						poly[3]=CPoint((int)(poly[0].x+BarWidth),(int)(poly[0].y-((double)m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount));
						poly[4]=poly[0];
						GetParent()->LogPoly((LPPOINT)&poly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
						FillPoly((LPPOINT)&poly,5,ptrStyle,0);
						FramePoly((LPPOINT)&poly,5,ptrStyle,0);
						//and the top
						poly[0]=poly[1];
						poly[1]=poly[2];
						poly[2]=CPoint((int)(poly[1].x-BarDepth),poly[1].y);
						poly[3]=CPoint((int)(poly[0].x-BarDepth),poly[0].y);
						poly[4]=poly[0];
						GetParent()->LogPoly((LPPOINT)&poly,5,elements[z].index+m_nMinSet,x+m_nMinDiv); // we log 3 polygons so that the mouse cursor can
																													 // be placed over any part of the bar
						FillPoly((LPPOINT)&poly,5,ptrStyle,0);
						
						if(shading)
						{
							int OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
							oldBrush=(CBrush *)GetpDC()->SelectObject(&glareBrush);
							CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
							GetpDC()->Polygon((LPPOINT)&poly,5);
							GetpDC()->SelectObject(oldBrush);
							GetpDC()->SetROP2(OldMode);
							GetpDC()->SelectObject(OldPen);
						}
						
						FramePoly((LPPOINT)&poly,5,ptrStyle,0);
					}
					break;
				case CX_AXIS_PLATFORM_SE:
				case CX_AXIS_STAGE_SE:
					{
						// draw the front panel..
						poly[0]=CPoint((int)(BackPlane.right+nOffsetX-(z*BarDepth)),BackPlane.bottom+nOffsetY);
						poly[1]=CPoint((int)(poly[0].x-BarDepth),poly[0].y);
						poly[2]=CPoint(poly[1].x,(int)(poly[1].y-(Value*ScalingFactor)));
						poly[3]=CPoint(poly[0].x,poly[2].y);
						poly[4]=poly[0];
						GetParent()->LogPoly((LPPOINT)&poly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
						FillPoly((LPPOINT)&poly,5,ptrStyle,0);

						if(shading)
						{
							int OldMode=GetpDC()->SetROP2(R2_MASKPEN);
							oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush);
							CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
							GetpDC()->Polygon((LPPOINT)&poly,5);
							GetpDC()->SelectObject(oldBrush);
							GetpDC()->SetROP2(OldMode);
							GetpDC()->SelectObject(OldPen);
						}
						

						FramePoly((LPPOINT)&poly,5,ptrStyle,0);

						//draw the side panel

						poly[0]=poly[1];
						poly[1]=poly[2];
						poly[2]=CPoint((int)(poly[1].x-BarWidth),(int)(poly[1].y-((double)m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount));
						poly[3]=CPoint((int)(poly[0].x-BarWidth),(int)(poly[0].y-((double)m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount));
						poly[4]=poly[0];
						GetParent()->LogPoly((LPPOINT)&poly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
						FillPoly((LPPOINT)&poly,5,ptrStyle,0);
						FramePoly((LPPOINT)&poly,5,ptrStyle,0);

						//and the top

						poly[0]=poly[1];
						poly[1]=poly[2];
						poly[2]=CPoint((int)(poly[1].x+BarDepth),poly[1].y);
						poly[3]=CPoint((int)(poly[0].x+BarDepth),poly[0].y);
						poly[4]=poly[0];
						GetParent()->LogPoly((LPPOINT)&poly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
						FillPoly((LPPOINT)&poly,5,ptrStyle,0);
						
						if(shading)
						{
							int OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
							oldBrush=(CBrush *)GetpDC()->SelectObject(&glareBrush);
							CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
							GetpDC()->Polygon((LPPOINT)&poly,5);
							GetpDC()->SelectObject(oldBrush);
							GetpDC()->SetROP2(OldMode);
							GetpDC()->SelectObject(OldPen);
						}
						
						FramePoly((LPPOINT)&poly,5,ptrStyle,0);

					}
					break;
				default:
					break;		// dont draw this data on a incompatible axis
				}			
			}
		}
	}

	delete (elements);

	if(shading)
	{
		shadeBrush.DeleteObject();
		glareBrush.DeleteObject();
		glareBitmap.DeleteObject();
		shadeBitmap.DeleteObject();
	}

}

//@mfunc Draws "Rooftop" type graphs.
void SRGraphDisplay::DrawIsoAreaData()
{
	if(m_ScaleY.Min()==0 && m_ScaleY.Max()==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bRibbon=m_Style.GetGraphStyle()==CX_GRAPH_RIBBON; //Is this an area graph or a ribbon graph?

	if(m_ScaleY.Min()<0 || m_nDivCount<2 || m_ScaleY.Min()<0)
	{
		//rooftop cannot show negative data or have less than two indices
		m_Style.SetAxisStyle(CX_AXIS_CLASSIC);
		m_Style.SetGraphStyle(CX_GRAPH_VBAR);
		m_Style.SetLegendStyle(CX_LEGEND_AUTOMATIC);
		m_pCWnd->Invalidate();
		return;
	}

	CRect BackPlane;
	int ExtraOffset=0;

	ExtraOffset=StageCalcSize(&BackPlane);
	
	BOOL bLogData=m_Style.GetLog();

	switch(m_Style.GetAxisStyle())
	{
	case CX_AXIS_PLATFORM_SE:
	case CX_AXIS_STAGE_SE:
		BackPlane.OffsetRect(ExtraOffset,0);
		break;
	case CX_AXIS_AUTOMATIC:
	case CX_AXIS_NONE:
	case CX_AXIS_PLATFORM_SW:
	case CX_AXIS_STAGE_SW:
		BackPlane.OffsetRect((int)(ExtraOffset ? 
		/* if true */		 ExtraOffset+m_nDivCount*m_Style.GetMaxObjectWidth() :
		/* else */			 m_DisplayRect.Width()-BackPlane.Width()),
							 0);
		break;
	default:
		break;
	}

	// The number of groups and the depth of the lists allow us to calculate
	// the size of the bars based upon the area of the remaining display space

    CPoint *Sidepoly=new CPoint[m_nDivCount+3];
	// storage for our graphic object
	// worked out on the basis of one vertex for each data item plus
	// two for the line along the bottom plus a last one to close the polygon

	
	CPoint Toppoly[5];

	double BarWidth=ExtraOffset ? m_Style.GetMaxObjectWidth() : (0.6*m_DisplayRect.Width())/(m_nDivCount-1),
		   BarDepth=ExtraOffset ? m_Style.GetMaxObjectDepth() : (0.4*m_DisplayRect.Width())/m_nSetCount;

	double ScalingFactor=BackPlane.Height()/m_dMaxScaleY;

	int nOffsetX,nOffsetY,nPrevOffsetX,nPrevOffsetY; // current and previous offset values

	CBitmap shadeBitmap1,glareBitmap1,shadeBitmap2,glareBitmap2;
	CBrush shadeBrush1,glareBrush1,shadeBrush2,glareBrush2,*oldBrush;
	BOOL shading=m_Style.GetObjectShadeStyle();
	if(shading)
	{
		shadeBitmap1.LoadBitmap(SRG_IDB_LIGHTSHADOW);
		shadeBrush1.CreatePatternBrush(&shadeBitmap1);
		glareBitmap1.LoadBitmap(SRG_IDB_GLARE);
		glareBrush1.CreatePatternBrush(&glareBitmap1);
		shadeBitmap2.LoadBitmap(SRG_IDB_SHADOW);
		shadeBrush2.CreatePatternBrush(&shadeBitmap2);
		glareBitmap2.LoadBitmap(SRG_IDB_LIGHTGLARE);
		glareBrush2.CreatePatternBrush(&glareBitmap2);
	}

	double Value,LastValue;


	GESorter *elements=new GESorter[m_nSetCount];

	for(int z=0;z<m_nSetCount;z++)
	{
		elements[z].value=GetParent()->GetGroupAverage(m_nMinDiv,m_nMaxDiv,z+m_nMinSet);
		elements[z].index=z+m_nMinDiv;
	}

	if(GetStyle()->GetSort())
		qsort(elements,m_nSetCount,sizeof(GESorter),qsSorterCompareD);


	for(z=0;z<m_nSetCount;z++)
	{// step through the groups

		m_nCurrentIndex=elements[z].index;
		
		SRGraphStyle *ptrStyle=GetParent()->GetGroup(elements[z].index)->GetStyle();
		int nPolyCount=0;

		for(int x=0;x<m_nDivCount;x++)
		{//item by item

			nOffsetX=(int)(x*BarWidth);
			nOffsetY=(int)(x*((double)(m_DisplayRect.bottom-BackPlane.bottom)/(m_nDivCount-1)));

			SRGraphData *pD=GetParent()->GetGroup(elements[z].index)->GetIndex(x+m_nMinDiv);
			
			if(!pD->GetNull())
			{

				Value=pD->GetValue(bLogData);

				// To get the draw order right we need to know whether the graph is a "south west" or "south east" type
				switch(m_Style.GetAxisStyle())
				{
				case CX_AXIS_AUTOMATIC:
				case CX_AXIS_NONE:
				case CX_AXIS_PLATFORM_SW:
				case CX_AXIS_STAGE_SW:
					{
						//Add a point to the side panel

						Sidepoly[nPolyCount]=CPoint((int)(((1+z)*BarDepth)+BackPlane.left-nOffsetX),(int)((BackPlane.bottom+nOffsetY)-(Value*ScalingFactor)));

						//draw a top panel

						if(x>0)
						{
							Toppoly[0]=Sidepoly[nPolyCount];
							Toppoly[1]=CPoint((int)(Toppoly[0].x-BarDepth),Toppoly[0].y);
							Toppoly[2]=CPoint((int)((z*BarDepth)+BackPlane.left-nPrevOffsetX),(int)((BackPlane.bottom+nPrevOffsetY)-(LastValue*ScalingFactor)));
							Toppoly[3]=CPoint((int)(((1+z)*BarDepth)+BackPlane.left-nPrevOffsetX),(int)((BackPlane.bottom+nPrevOffsetY)-(LastValue*ScalingFactor)));
							Toppoly[4]=Toppoly[0];
							GetParent()->LogPoly((LPPOINT)&Toppoly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
							FillPoly((LPPOINT)&Toppoly,5,ptrStyle,0);
							if(shading)
							{
								int OldMode;
								CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
								double Slant=(BarWidth/(fabs(Value-LastValue)*ScalingFactor));
								if(Slant<0.3)
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush2);
									OldMode=GetpDC()->SetROP2(R2_MASKPEN);
								}
								else if(Slant<0.6)
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush1);
									OldMode=GetpDC()->SetROP2(R2_MASKPEN);
								}
								else if(Slant<1.5)
								{
									OldMode=GetpDC()->SetROP2(R2_NOP);
								}
								else if(Slant<2)
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&glareBrush1);
									OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
								}
								else
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&glareBrush2);
									OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
								}
								GetpDC()->Polygon((LPPOINT)&Toppoly,5);
								GetpDC()->SelectObject(oldBrush);
								GetpDC()->SetROP2(OldMode);
								GetpDC()->SelectObject(OldPen);
							}
							FramePoly((LPPOINT)&Toppoly,5,ptrStyle,0);
						}

						if(x==(m_nDivCount-1) && !bRibbon)
						{
							Sidepoly[nPolyCount+1]=CPoint((int)(((1+z)*BarDepth)+BackPlane.left-nOffsetX),(BackPlane.bottom+nOffsetY));
							Sidepoly[nPolyCount+2]=CPoint((int)(((1+z)*BarDepth)+BackPlane.left),BackPlane.bottom);
							Sidepoly[nPolyCount+3]=Sidepoly[0];

							// and draw the end panel

							Toppoly[0]=Sidepoly[nPolyCount];
							Toppoly[1]=CPoint((int)(Toppoly[0].x-BarDepth),Toppoly[0].y);
							Toppoly[2]=CPoint(Toppoly[1].x,BackPlane.bottom+nOffsetY);
							Toppoly[3]=CPoint(Toppoly[0].x,Toppoly[2].y);
							Toppoly[4]=Toppoly[0];
							GetParent()->LogPoly((LPPOINT)&Toppoly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
							FillPoly((LPPOINT)&Toppoly,5,ptrStyle,0);
							if(shading)
							{
								int OldMode=GetpDC()->SetROP2(R2_MASKPEN);
								CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
								oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush2);
								GetpDC()->Polygon((LPPOINT)&Toppoly,5);
								GetpDC()->SelectObject(oldBrush);
								GetpDC()->SetROP2(OldMode);
								GetpDC()->SelectObject(OldPen);
							}
							FramePoly((LPPOINT)&Toppoly,5,ptrStyle,0);

						}
						nPolyCount++;


					}
					break;
				case CX_AXIS_PLATFORM_SE:
				case CX_AXIS_STAGE_SE:
					{
						//Add a point to the side panel

						Sidepoly[nPolyCount]=CPoint((int)(BackPlane.right+nOffsetX-((1+z)*BarDepth)),(int)((BackPlane.bottom+nOffsetY)-(Value*ScalingFactor)));

						//draw a top panel

						if(x>0)
						{
							Toppoly[0]=Sidepoly[nPolyCount];
							Toppoly[1]=CPoint((int)(Toppoly[0].x+BarDepth),Toppoly[0].y);
							Toppoly[2]=CPoint((int)(BackPlane.right+nPrevOffsetX-(z*BarDepth)),(int)((BackPlane.bottom+nPrevOffsetY)-(LastValue*ScalingFactor)));
							Toppoly[3]=CPoint((int)(BackPlane.right+nPrevOffsetX-((1+z)*BarDepth)),(int)((BackPlane.bottom+nPrevOffsetY)-(LastValue*ScalingFactor)));
							Toppoly[4]=Toppoly[0];
							GetParent()->LogPoly((LPPOINT)&Toppoly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
							FillPoly((LPPOINT)&Toppoly,5,ptrStyle,0);
							if(shading)
							{
								int OldMode;
								CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
								double Slant=(BarWidth/(fabs(Value-LastValue)*ScalingFactor));
								if(Slant<0.3)
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush2);
									OldMode=GetpDC()->SetROP2(R2_MASKPEN);
								}
								else if(Slant<0.6)
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush1);
									OldMode=GetpDC()->SetROP2(R2_MASKPEN);
								}
								else if(Slant<1.5)
								{
									OldMode=GetpDC()->SetROP2(R2_NOP);
								}
								else if(Slant<2)
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&glareBrush1);
									OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
								}
								else
								{
									oldBrush=(CBrush *)GetpDC()->SelectObject(&glareBrush2);
									OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
								}
								GetpDC()->Polygon((LPPOINT)&Toppoly,5);
								GetpDC()->SelectObject(oldBrush);
								GetpDC()->SetROP2(OldMode);
								GetpDC()->SelectObject(OldPen);
							}
							FramePoly((LPPOINT)&Toppoly,5,ptrStyle,0);
						}

						if(x==(m_nDivCount-1) && !bRibbon)
						{
							Sidepoly[nPolyCount+1]=CPoint((int)(BackPlane.right+nOffsetX-((1+z)*BarDepth)),(BackPlane.bottom+nOffsetY));
							Sidepoly[nPolyCount+2]=CPoint((int)(BackPlane.right-((1+z)*BarDepth)),BackPlane.bottom);
							Sidepoly[nPolyCount+3]=Sidepoly[0];

							Toppoly[0]=Sidepoly[nPolyCount];
							Toppoly[1]=CPoint((int)(Toppoly[0].x+BarDepth),Toppoly[0].y);
							Toppoly[2]=CPoint(Toppoly[1].x,(int)(BackPlane.bottom+nOffsetY));
							Toppoly[3]=CPoint(Toppoly[0].x,Toppoly[2].y);
							Toppoly[4]=Toppoly[0];
							GetParent()->LogPoly((LPPOINT)&Toppoly,5,elements[z].index+m_nMinSet,x+m_nMinDiv);
							FillPoly((LPPOINT)&Toppoly,5,ptrStyle,0);
							if(shading)
							{
								int OldMode=GetpDC()->SetROP2(R2_MASKPEN);
								CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN);
								oldBrush=(CBrush *)GetpDC()->SelectObject(&shadeBrush2);
								GetpDC()->Polygon((LPPOINT)&Toppoly,5);
								GetpDC()->SelectObject(oldBrush);
								GetpDC()->SetROP2(OldMode);
								GetpDC()->SelectObject(OldPen);
							}
							FramePoly((LPPOINT)&Toppoly,5,ptrStyle,0);
						}
						
						nPolyCount++;


					}
					break;
				default:
					break;		// dont draw this data on a incompatible axis
				}
			
			nPrevOffsetX=nOffsetX;
			nPrevOffsetY=nOffsetY;
			LastValue=Value;
			}

		}

		if(!bRibbon)
		{
			GetParent()->LogPoly((LPPOINT)Sidepoly,(m_nDivCount+3),elements[z].index+m_nMinSet,-1);
			FillPoly((LPPOINT)Sidepoly,nPolyCount+3,ptrStyle,0);
			FramePoly((LPPOINT)Sidepoly,nPolyCount+3,ptrStyle,0);
		}

	}

	delete (elements);
	delete (Sidepoly);

	if(shading)
	{
		shadeBrush1.DeleteObject();
		glareBrush1.DeleteObject();
		glareBitmap1.DeleteObject();
		shadeBitmap1.DeleteObject();
		shadeBrush2.DeleteObject();
		glareBrush2.DeleteObject();
		glareBitmap2.DeleteObject();
		shadeBitmap2.DeleteObject();
	}

}

//@mfunc Used to write the graph style parameters to the template file
//@parm Pointer to the CStdioFile used in writing this template
void SRGraphDisplay::WriteSpecificParameters(CStdioFile* file)
{
	CString cdata;
	cdata.Format("%d %d %d %d",
				m_Style.GetAxisStyle()>>CX_AXIS_SHIFT,
				m_Style.GetGraphStyle()>>CX_GRAPH_SHIFT,
				m_Style.GetLabelStyle()>>CX_LABEL_SHIFT,
				m_Style.GetLog());
	file->WriteString(cdata);
}


//@mfunc When the graph type is not recognised this virtual function is called. 
//Override it to draw custom data displays.
//@devnote Customizing of graph types and displays is performed by overriding one or all of these trap routines. 
//You may extend a graph by changing the way its axis is drawn allowing all other operations to be handled by the 
//built in routines or you may override the whole lot with new graph, label and axis styles and trap every custom 
//routine to handle the extra styles.<nl><nl>
//To invoke any of the custom trap routines you only need to put a numeric style not recognised by the standard 
//software into an object or component. Traps will occur for the draw routines and for the preparation routine if you 
//have specified a new graph style.
void SRGraphDisplay::DrawCustomData()
{
	ASSERT(FALSE);	// Override this function to handle your own display drawing
}

//@mfunc When the label type is not recognised this virtual function is called. 
//Override it to draw custom labels.
//@see <mf SRGraphDisplay::DrawCustomData> for a full explanation of the customizing procedure.
void SRGraphDisplay::DrawCustomLabels()
{
	ASSERT(FALSE);	// Override this function to handle your own label drawing
}

//@mfunc When the axis type is not recognised this virtual function is called. 
//Override it to draw custom axes.
//@see <mf SRGraphDisplay::DrawCustomData> for a full explanation of the customizing procedure.
void SRGraphDisplay::DrawCustomAxis()
{
	ASSERT(FALSE);	// Override this function to handle your own axis drawing
}

//@mfunc Draws the labels for stage and platform charts (Manhattan, Rooftop and Ribbon)
void SRGraphDisplay::DrawStageLabels()
{

	CRect BackPlane;
	int ExtraOffset=0;
	
	if(m_nSetCount==0 || m_nDivCount<2)
		return;	// nothing to do!

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return; // honour the request not to draw the labels

	//select the font so that GetTextExtent() works.
	CFont *OldFont=(CFont *)GetpDC()->SelectObject(&m_Font);

	BOOL bLeftHandGraph=((m_Style.GetAxisStyle()==CX_AXIS_STAGE_SE) ||
						 (m_Style.GetAxisStyle()==CX_AXIS_PLATFORM_SE));

	GESorter *elements=NULL;

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())
			m_ptrYLabels=new SRGLogScale;
		else
			m_ptrYLabels=new SRGDecimalScale;
	}

	ASSERT(m_ptrYLabels!=NULL);

	if(m_ptrXLabels==NULL)
		m_ptrXLabels=new SRGraphLabelBlock;

	ASSERT(m_ptrXLabels!=NULL);

	if(m_ptrZLabels==NULL)
		m_ptrZLabels=new SRGraphLabelBlock;

	ASSERT(m_ptrZLabels!=NULL);

	SRGDecimalScale	*ptYLabels=(SRGDecimalScale *)m_ptrYLabels;


	TRY 
	{
		CScale DisplayAdjustmentRight,DisplayAdjustmentLeft,DisplayAdjustmentTop;

		for(int pass=0;pass<2;pass++)
		{

			//  In pass 1 we find all the label sizes and positions
			//  in pass 2 we output the labels with the re-sized display rectangle

 			ExtraOffset=StageCalcSize(&BackPlane);
			
			if(bLeftHandGraph)
				BackPlane.OffsetRect(ExtraOffset,0); // offset by the width of the margin
			else
				BackPlane.OffsetRect((int)(ExtraOffset ? 
					 ExtraOffset+(m_nDivCount*m_Style.GetMaxObjectWidth()) : 
					 m_DisplayRect.Width()-BackPlane.Width()),
					 0);


			ptYLabels->SetPass(pass);

			if(m_ScaleY.Min()<0) // check for negative numbers
				SRThrowGraphException(SRGraphException::label);

			LOGFONT lf;
			m_Font.GetLogFont(&lf);

			if(m_Style.GetShowNumerals())
			{
				ptYLabels->SetParent(GetParent());
				ptYLabels->DrawingUsing(m_pDC,m_pCWnd);
				ptYLabels->SetMinValue(m_ScaleY.Min());
				ptYLabels->SetMaxValue(m_ScaleY.Max());
				ptYLabels->SetFontSize(GetFontSize());
				ptYLabels->SetFaceName(GetFaceName());
				ptYLabels->SetTextColor(GetTextColor());
				ptYLabels->SetFontStyle(GetFontStyle());
				ptYLabels->SetMinValue(m_ScaleY.Min());
				ptYLabels->SetMaxValue(m_ScaleY.Max());
				ptYLabels->SetResolution(0.1);
				ptYLabels->SetLogTickPositions(TRUE);
				ptYLabels->SetptrTickPositionList(&m_YTickList);
				ptYLabels->SetAscending();
				if(bLeftHandGraph)
				{
					ptYLabels->SetStartPos(CPoint(m_DisplayRect.right,m_DisplayRect.bottom));
					ptYLabels->SetEndPos(CPoint(m_DisplayRect.right,m_DisplayRect.bottom-BackPlane.Height()));
					ptYLabels->SetLabelSide(TRUE);
				}
				else
				{
					ptYLabels->SetStartPos(CPoint(m_DisplayRect.left,m_DisplayRect.bottom));
					ptYLabels->SetEndPos(CPoint(m_DisplayRect.left,m_DisplayRect.bottom-BackPlane.Height()));
					ptYLabels->SetLabelSide(FALSE);
				}
				ptYLabels->CreateLabels();
				ptYLabels->Draw(m_pDC,m_pCWnd);
				CRect r=ptYLabels->GetBlockRect();

				if(!pass && r.Width()>ExtraOffset && !m_Style.DRIsLocked())
				{
					if(bLeftHandGraph)
						m_DisplayRect.right-=r.Width()-ExtraOffset;
					else
						m_DisplayRect.left+=r.Width()-ExtraOffset;
				}
			}

			if(m_Style.GetShowGroups())
			{
				elements=new GESorter[m_nSetCount];

				for(int z=0;z<m_nSetCount;z++)
				{
					elements[z].value=GetParent()->GetGroupAverage(m_nMinDiv,m_nMaxDiv,z+m_nMinSet);
					elements[z].index=z;
				}

				if(GetStyle()->GetSort())
					qsort(elements,m_nSetCount,sizeof(GESorter),qsSorterCompareD);

				m_ptrXLabels->SetParent(GetParent());
				m_ptrXLabels->DrawingUsing(m_pDC,m_pCWnd);
				m_ptrXLabels->SetPass(pass);

				if(m_bSignificantChange || GetStyle()->GetSort())
				{
					for(int g=0;g<m_nSetCount;g++)
						m_ptrXLabels->AddLabelIndex(GetParent()->GetHeader(elements[g].index),g);	//retrieve the header
					while(m_ptrXLabels->GetLabelCount()>m_nSetCount)
						m_ptrXLabels->RemoveLastLabel();
				}
				
	
				m_ptrXLabels->SetFontSize(GetFontSize());
				m_ptrXLabels->SetFaceName(GetFaceName());
				m_ptrXLabels->SetTextColor(GetTextColor());
				m_ptrXLabels->SetFontStyle(GetFontStyle());
				m_ptrXLabels->SetLogTickPositions(TRUE);
				m_ptrXLabels->SetptrTickPositionList(&m_XTickList);
				m_ptrXLabels->SetCheckOverlaps(FALSE);
				m_ptrXLabels->SetBlockVectors(0,1);
				if(bLeftHandGraph)
				{
					m_ptrXLabels->SetBlockLocationPoints(SRGraphLabel::MidRight);
					m_ptrXLabels->SetBlockOrientations(90);
					m_ptrXLabels->SetBlockPositions(CPoint(m_DisplayRect.right-((BackPlane.Width()/m_nSetCount)/2),
														   m_DisplayRect.bottom+5),
						                            CPoint(m_DisplayRect.right-(BackPlane.Width()-((BackPlane.Width()/m_nSetCount)/2)),
													       m_DisplayRect.bottom+5));
				}
				else
				{
					m_ptrXLabels->SetBlockLocationPoints(SRGraphLabel::MidLeft);
					m_ptrXLabels->SetBlockOrientations(270);
					m_ptrXLabels->SetBlockPositions(CPoint(m_DisplayRect.left+((BackPlane.Width()/m_nSetCount)/2),
						                                   m_DisplayRect.bottom+5),
						                            CPoint(m_DisplayRect.left+(BackPlane.Width()-((BackPlane.Width()/m_nSetCount)/2)),
													       m_DisplayRect.bottom+5));
				}

				m_ptrXLabels->Draw(m_pDC,m_pCWnd);
				
				if(!pass && !m_Style.DRIsLocked())
				{
					CRect r;
					r=m_ptrXLabels->GetBlockRect();
					m_DisplayRect.bottom-=r.Height();
				}

			

				delete (elements);
				elements=NULL;
			}

			if(m_Style.GetShowIndices())
			{
				BOOL bAreaGraph=((m_Style.GetGraphStyle()==CX_GRAPH_ISO_AREA) ||
								 (m_Style.GetGraphStyle()==CX_GRAPH_RIBBON));

				m_ptrZLabels->SetParent(GetParent());
				m_ptrZLabels->DrawingUsing(m_pDC,m_pCWnd);
				m_ptrZLabels->SetPass(pass);

				if(m_bSignificantChange)
				{
					for(int g=0;g<m_nDivCount;g++)
						m_ptrZLabels->AddLabelIndex(GetParent()->GetGroup(m_nMinSet)->GetIndex(m_nMinDiv+g)->GetAnnotation(GetParent()->GetCountryCode()),g);	//retrieve the header
					while(m_ptrZLabels->GetLabelCount()>m_nDivCount)
						m_ptrZLabels->RemoveLastLabel();
				}


	
				m_ptrZLabels->SetFontSize(GetFontSize());
				m_ptrZLabels->SetFaceName(GetFaceName());
				m_ptrZLabels->SetTextColor(GetTextColor());
				m_ptrZLabels->SetFontStyle(GetFontStyle());
				m_ptrZLabels->SetLogTickPositions(TRUE);
				m_ptrZLabels->SetptrTickPositionList(&m_ZTickList);
				m_ptrZLabels->SetCheckOverlaps(FALSE);
				m_ptrZLabels->SetBlockVectors(1,0);

				if(bLeftHandGraph)
				{
					int nOddX=((m_DisplayRect.right-BackPlane.right)/m_nDivCount)/2,
						nOddY=((m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount)/2;
					m_ptrZLabels->SetBlockLocationPoints(SRGraphLabel::TopRight);
					m_ptrZLabels->SetBlockPositions(CPoint(m_DisplayRect.left+ExtraOffset+(bAreaGraph ? 0 : nOddX)-10,
														   BackPlane.top+BackPlane.Height()+(bAreaGraph ? 0 : nOddY)),
													CPoint(m_DisplayRect.right-(ExtraOffset+(BackPlane.Width()+(bAreaGraph ? 0 : nOddX)))-10,
														   m_DisplayRect.bottom-(bAreaGraph ? 0 : nOddY)));
				}
				else
				{
					int nOddX=((BackPlane.left-m_DisplayRect.left)/m_nDivCount)/2,
						nOddY=((m_DisplayRect.bottom-BackPlane.bottom)/m_nDivCount)/2;
					m_ptrZLabels->SetBlockLocationPoints(SRGraphLabel::TopLeft);
					m_ptrZLabels->SetBlockPositions(CPoint(m_DisplayRect.right-(ExtraOffset+(bAreaGraph ? 0 : nOddX))+10,
														   BackPlane.top+BackPlane.Height()+(bAreaGraph ? 0 : nOddY)),
													CPoint(m_DisplayRect.left+ExtraOffset+(BackPlane.Width()+(bAreaGraph ? 0 : nOddX))+10,
														   m_DisplayRect.bottom-(bAreaGraph ? 0 : nOddY)));
				}

				m_ptrZLabels->Draw(m_pDC,m_pCWnd);
				
				if(!pass && !m_Style.DRIsLocked())
				{
					CRect r;
					r=m_ptrZLabels->GetBlockRect();
					if(bLeftHandGraph)
					{	
						if(r.left<m_DisplayRect.left)
							m_DisplayRect.left+=(m_DisplayRect.left-r.left);
					}
					else
					{
						if(r.right>m_DisplayRect.right)
							m_DisplayRect.right-=(r.right-m_DisplayRect.right);
					}
				}
			}
		}
	}
	CATCH(SRGraphException,e)
	{
		if(e->m_nCause!=SRGraphException::label)
			SRThrowGraphException(e->m_nCause);
	}
	END_CATCH;

}


//@mfunc Calculates the size of a stage axis
//@parm Pointer to a CRect which receives the calculated size of the stage backplane
//@rdesc The amount by which to offset the sides of the stage in order to centralise it and keep within the bounds of the object maximum sizes.
int SRGraphDisplay::StageCalcSize(CRect *BackPlane)
{
	int ExtraOffset=0;

	if(m_Style.GetUseMaxSize())
	{
		if(m_nDivCount * m_Style.GetMaxObjectWidth() > 0.6*m_DisplayRect.Width())
			BackPlane->SetRect(m_DisplayRect.left,m_DisplayRect.top,(int)(m_DisplayRect.left+m_DisplayRect.Width()*0.4),m_DisplayRect.top+(m_DisplayRect.Height()/2));
		else
		{
			int Width=(int)(min((double)m_DisplayRect.Width() - ((double)m_nDivCount * m_Style.GetMaxObjectWidth()),
								(double)(m_nSetCount) * m_Style.GetMaxObjectDepth()
								));
			int	Height=(int)((double)m_DisplayRect.Height()-((double)m_nDivCount * m_Style.GetMaxObjectWidth())/((double)m_DisplayRect.Width()/m_DisplayRect.Height()));		

			ExtraOffset=(int)((m_DisplayRect.Width()/2)-
						((m_nDivCount)*m_Style.GetMaxObjectWidth()+
						(m_nSetCount*m_Style.GetMaxObjectDepth())/2));

			BackPlane->SetRect(m_DisplayRect.left,m_DisplayRect.top,m_DisplayRect.left+Width,m_DisplayRect.top+Height);
														
		// Effectively.. the original display width minus the width of all the objects and
		// the original display height reduced in proportion to the aspect ratio of the 
		// display rectangle.

		}

	}
	else
	{
		BackPlane->SetRect(m_DisplayRect.left,m_DisplayRect.top,(int)(m_DisplayRect.left+m_DisplayRect.Width()*0.4),m_DisplayRect.top+(m_DisplayRect.Height()/2));
	}

	return ExtraOffset;
}

//@mfunc Draws the stacked horizontal bar chart data
void SRGraphDisplay::DrawStackHBarData()
{
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	if(m_ScaleY.Min()<0)
	{
		//Stacked bar graphs cannot show negative data;
		m_Style.SetAxisStyle(CX_AXIS_CLASSIC);
		m_Style.SetGraphStyle(CX_GRAPH_VBAR);
		m_Style.SetLegendStyle(CX_LEGEND_AUTOMATIC);
		m_pCWnd->Invalidate();
		return;
	}

	DWORD dwAxisStyle=GetStyle()->GetAxisStyle();

	BOOL bLogData=GetStyle()->GetLog();
	
	double ScalingFactor=(double)m_DisplayRect.Width()/m_dMaxScaleY;
	
	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Height())/(m_nDivCount),(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Height())/(m_nDivCount)/1.1;

	double dBlockWidth=(double)m_DisplayRect.Height()/m_nDivCount;

	int CurrentSet,x;

	GESorter *elements=new GESorter[m_nSetCount];

	for(x=0;x<m_nDivCount;x++)
	{
		int top,left,bottom,right;

		CRect BarRect;
		
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			SRGraphData *pd=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv);
			if(pd->GetNull())
				elements[CurrentSet].value=0;
			else
				elements[CurrentSet].value=pd->GetValue(bLogData);
			elements[CurrentSet].index=CurrentSet+m_nMinSet;
		}

		qsort(elements,m_nSetCount,sizeof(GESorter),qsSorterCompareA);

		double lastValue=0,thisValue;
		
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			SRGraphDataList *gdl=GetParent()->GetGroup(elements[CurrentSet].index,FALSE);
			ASSERT(gdl!=NULL);
			SRGraphStyle *ptrStyle=gdl->GetIndex(x+m_nMinDiv)->GetStyle();
			if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE) // if there is no individual style defined
				ptrStyle=gdl->GetStyle();					// pick up a global style for the group
			
			m_nCurrentIndex=elements[CurrentSet].index;
			
			if(m_nCurrentIndex==m_nSetCount)
				m_nCurrentIndex=0;

			thisValue=elements[CurrentSet].value;

			left=(int)(m_DisplayRect.left+(ScalingFactor*lastValue));
			right=(int)(m_DisplayRect.left+(ScalingFactor*thisValue));
			bottom=(int)(m_DisplayRect.bottom-((x*dBlockWidth)+(dBlockWidth/2)-(BarWidth/2)));
			top=(int)(bottom-BarWidth);

			DrawBar(CPoint(left,top),CPoint(right,bottom),elements[CurrentSet].index,x+m_nMinDiv,ptrStyle,dwAxisStyle==CX_AXIS_ISO_CLASSIC_YX ? CX_OBJECT_EXBAR : 0);
				
			lastValue=thisValue;

		}
	}

	delete (elements);
}

//@mfunc Draws the stacked vertical bar chart data
void SRGraphDisplay::DrawStackVBarData()
{
	if(m_ScaleY.Min()==0 && m_ScaleY.Max()==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	if(m_ScaleY.Min()<0)
	{
		//Stacked bar graphs cannot show negative data;
		m_Style.SetAxisStyle(CX_AXIS_CLASSIC);
		m_Style.SetGraphStyle(CX_GRAPH_VBAR);
		m_Style.SetLegendStyle(CX_LEGEND_AUTOMATIC);
		m_pCWnd->Invalidate();
		return;
	}

	DWORD dwAxisStyle=GetStyle()->GetAxisStyle();
	
	BOOL bLogData=m_Style.GetLog();
	
	double ScalingFactor=((m_ScaleY.Min()>=0 ? (double)m_DisplayRect.Height() : ((double)m_DisplayRect.Height())/2.0)/m_dMaxScaleY);
	
	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Width())/(m_nDivCount),(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Width())/(m_nDivCount)/1.1;
	double dBlockWidth=(double)m_DisplayRect.Width()/m_nDivCount;
	int CurrentSet,x;

	GESorter *elements=new GESorter[m_nSetCount];

	for(x=0;x<m_nDivCount;x++)
	{

		int top,left,bottom,right;

		CRect BarRect;
		
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			SRGraphData *pd=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
			if(pd->GetNull())
				elements[CurrentSet].value=0;
			else
				elements[CurrentSet].value=pd->GetValue(bLogData);
			elements[CurrentSet].index=CurrentSet+m_nMinSet;
		}

		double lastValue=0,thisValue;

		qsort(elements,m_nSetCount,sizeof(GESorter),qsSorterCompareA);
		
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			SRGraphDataList *gdl=GetParent()->GetGroup(elements[CurrentSet].index,FALSE);
			ASSERT(gdl!=NULL);
			SRGraphStyle *ptrStyle=gdl->GetIndex(x+m_nMinDiv)->GetStyle();
			if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE) // if there is no individual style defined
				ptrStyle=gdl->GetStyle();					// pick up a global style for the group
			
			m_nCurrentIndex=elements[CurrentSet].index;

			thisValue=elements[CurrentSet].value;

			left=(int)(m_DisplayRect.left+(x*dBlockWidth)+(dBlockWidth/2)-(BarWidth/2));
			top=(int)(m_DisplayRect.bottom-(ScalingFactor*thisValue));
			right=(int)(left+BarWidth);
			bottom=(int)(m_DisplayRect.bottom-(ScalingFactor*lastValue));

			lastValue=thisValue;
	
			DrawBar(CPoint(left,top),CPoint(right,bottom),elements[CurrentSet].index,x+m_nMinDiv,ptrStyle,dwAxisStyle==CX_AXIS_ISO_CLASSIC ? CX_OBJECT_EXBAR : 0);
		}
	}

	delete (elements);
}

//@mfunc Draws horizontal bars
void SRGraphDisplay::DrawHBarData()
{
	
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bLogData=m_Style.GetLog();
	DWORD dwGraphType=GetStyle()->GetGraphStyle();
	DWORD dwAxisType=GetStyle()->GetAxisStyle();
	double ScalingFactor=(double)m_DisplayRect.Width()/m_dMaxScaleY;
	
	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Height())/(m_nSetCount*m_nDivCount),(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Height())/(m_nSetCount*m_nDivCount)/1.1;

	double dBlockWidth=(double)m_DisplayRect.Height()/m_nDivCount;
	double dSetOffset=(dBlockWidth/2)-((m_nSetCount*BarWidth)/2);

	int CurrentSet,x;
	for(x=0;x<m_nDivCount;x++)
	{

		int top,left,bottom,right;

		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			SRGraphDataList *gdl=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE);
			ASSERT(gdl!=NULL);
			SRGraphStyle *ptrStyle=gdl->GetIndex(x+m_nMinDiv)->GetStyle();
			if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE) // if there is no individual style defined
				ptrStyle=gdl->GetStyle();					// pick up a global style for the group
			SRGraphData *pD=gdl->GetIndex(x+m_nMinDiv);
			if(!pD->GetNull())
			{
				m_nCurrentIndex=(CurrentSet+m_nMinSet);
				double dData=pD->GetValue(bLogData);
				if(bLogData)
					dData-=log10(m_ScaleY.Min());
				else
					dData-=m_ScaleY.Min();

				bottom=(int)(m_DisplayRect.bottom-(x*dBlockWidth)-dSetOffset-(CurrentSet*BarWidth));
				top=(int)(bottom-BarWidth);	
				right=(int)((double)m_DisplayRect.left+(ScalingFactor*dData)+0.5);
				if(bLogData)
					left=m_DisplayRect.left;
				else
					left=(int)((double)right-(ScalingFactor*pD->GetValue(bLogData))+0.5);
				DrawBar(CPoint(left,top),CPoint(right,bottom),CurrentSet+m_nMinSet,x+m_nMinDiv,ptrStyle,(dwGraphType==CX_GRAPH_EX_HBAR || dwAxisType==CX_AXIS_ISO_CLASSIC_YX) ? CX_OBJECT_EXBAR : 0);				
			}
		}
	}
}


//@mfunc Draws 3D pie chart data.
//@rem Because pie charts display proporional data any values which are negative or any attempt to
//display logarithmic data will cause this chart type to re-program itself to be a bar chart.
//@see <mf SRGraphDisplay::DrawVBarData> <mf SRGraphStyle::GetLog> 	<mf SRGraphStyle::SetLog>
void SRGraphDisplay::DrawIsoPieData()
{
	if(m_ScaleY.Min()<0)
	{	// Pie charts cannot show negative data... So do something sensible.
		m_Style.SetAxisStyle(CX_AXIS_CLASSIC);
		m_Style.SetGraphStyle(CX_GRAPH_VBAR);
		m_Style.SetLegendStyle(CX_LEGEND_AUTOMATIC);
		m_pCWnd->Invalidate();
		return;
	}

	double total=GetParent()->GetDataTotal(m_nMinSet,m_nMinDiv,m_nMaxSet,m_nMaxDiv); // get the sum of all the data

	if(total==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	double step=TWO_PI/72;	// this is the value of five degrees of arc in radians and will be used to 
							//step around the outside of the circle. Less than 5 degrees is overkill IMHO
	double angle=0;	// begin at the zero degree mark

	//The trouble with this sort of chart is that the wedges cannot be drawn one at a time because of z-ordering
	//problems. It is neccesary to work out a method whereby all the right parts of the chart are visible and all 
	//the internal bits are hidden. if we do this wrong way (ie. a wedge at a time) we'll end up with a chart that 
	//looks as though its "inside out".	Rather than a true 3D z-buffering system we can use the simple rule that 
	//the higher Y component the further the object is from the point of view

	//The method is..

	//(1) make a set of points which describe all the wall ends. Then sort these points into Y order
	//(2) draw the walls by describing a parallellogram with the center and sorted outside point as the top of the wall and the height determined by the radius.
	//(3) Draw the "skirt". remembering that we see only that area between 0 and 180 degrees.
	//(3) Draw the tops of the wedges

	
	int WedgeCount=m_nSetCount*m_nDivCount;
	PointSorter *pPS=new PointSorter[WedgeCount];		// an array of structures used to sort points into order
	double *angles =new double[WedgeCount];			// an array of angles. used to move the segment for highlighting
	int *indices=new int[WedgeCount];				// an array of ints used to store the index of the wedge
	int *groups=new int[WedgeCount];					// and an array of ints for the groups
	double *arcs=new double[WedgeCount];
											

	CPoint wall[5],*points;

	double arc,tarc;
	double radius1,radius2; // Because its an ellipse we use 2 radii

	double dBasicRadius=min(0.6*m_DisplayRect.Width()/2 , 0.6*m_DisplayRect.Height()/2);
	
	double dViewAngle= GetStyle()->GetGraphStyle()!=CX_GRAPH_PIE ? GetStyle()->GetXAngle() : 90;
																		   
	radius1=dBasicRadius;

	radius2=sin(RADIANS(dViewAngle))*dBasicRadius;

	
	double dSkirtHeight = cos(RADIANS(dViewAngle)) * (m_Style.GetUseMaxSize() ? m_Style.GetMaxObjectHeight() : radius2/2);
	
	int cx=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		cy=(int)(m_DisplayRect.top+(m_DisplayRect.Height()/2-(dSkirtHeight/2))); // find the central point of the pie


	SRGraphStyle *ptrStyle;

	angle=0;
	int ArrayIndex=0;

	SRGraphData *pD;


	int CurrentSet,x;

	if(radius1 != radius2)
	{
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			for(x=0;x<m_nDivCount;x++)
			{	//Find the outside points of all the walls (each point here deals with two walls back-to-back
				pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
				if(!pD->GetNull())
				{
	 				arc=arcs[ArrayIndex]=(TWO_PI/total)*pD->GetValue();
					pPS[ArrayIndex].p=CPoint((int)(cx+(radius1*cos(angle))),(int)(cy+(radius2*sin(angle))));
					pPS[ArrayIndex].i=ArrayIndex;
					indices[ArrayIndex]=x+m_nMinDiv;
					groups[ArrayIndex]=CurrentSet+m_nMinSet;
					angles[ArrayIndex]=angle+(arc/2);
					angle+=arc;
					ArrayIndex++;
				}
				else
					WedgeCount--;
			}
		}

		//sort the points into ascending Y order

		qsort(pPS,WedgeCount,sizeof(PointSorter),qsPointYSorterCompareA);

		for(int x=0;x<WedgeCount;x++)
		{
	 		arc=arcs[pPS[x].i];
			ptrStyle=GetParent()->GetGroup(groups[pPS[x].i])->GetIndex(indices[pPS[x].i])->GetStyle();
			m_nCurrentIndex=pPS[x].i;

			if(pPS[x].p.x>cx)
			{
				m_nCurrentIndex=pPS[x].i-1;
				if(m_nCurrentIndex==-1)
					m_nCurrentIndex=WedgeCount-1;
			}
				
			else
				m_nCurrentIndex = pPS[x].i; 

			wall[0]=CPoint(cx,cy);
			wall[1]=pPS[x].p;
			wall[2]=CPoint(wall[1].x,(int)(wall[1].y+dSkirtHeight));
			wall[3]=CPoint(wall[0].x,(int)(wall[0].y+dSkirtHeight));
			wall[4]=wall[0];

			ptrStyle=GetParent()->GetGroup(groups[m_nCurrentIndex])->GetIndex(indices[m_nCurrentIndex])->GetStyle();
			

			if(ptrStyle->GetHighlight()!=CX_HIGHLIGHT_NONE)
			{	//for comment on this routine see the DrawPieData routine. This is a variation on that theme.
				double highlightAngle=angles[m_nCurrentIndex],
					   highlightRadius1,highlightRadius2;
				int hx,hy; // the offsets
				switch(ptrStyle->GetHighlight())
				{
				case CX_HIGHLIGHT_SMALL:
					highlightRadius1=radius1/100*5; // small is five percent
					highlightRadius2=radius2/100*5; 
					break;
				case CX_HIGHLIGHT_MEDIUM:
				case CX_HIGHLIGHT_AUTOMATIC:
					highlightRadius1=radius1/100*10; // medium is ten pecent
					highlightRadius2=radius2/100*10; 
					break;
				case CX_HIGHLIGHT_LARGE:
					highlightRadius1=radius1/100*20; // large is 20 percent.
					highlightRadius2=radius2/100*20; 
					break;
				default:
					ASSERT(FALSE); // You have used a highlight value which was'nt on the list
					break;
				}
				hx=(int)(highlightRadius1*cos(highlightAngle));
				hy=(int)(highlightRadius2*sin(highlightAngle));
				for(int c=0;c<5;c++)
					wall[c].Offset(hx,hy);
				}


			FillPoly((LPPOINT)&wall,5,ptrStyle,0);
			FramePoly((LPPOINT)&wall,5,ptrStyle,0);
			
		}

		angle=0;
		
		// same trick but this time for the edge sections

		//By now Wedgecount is properly adjusted and if a data item has become null
		//or become valid inbetween the start and end of this routine TUFF.
		
		ArrayIndex=0;
		
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			for(x=0;x<m_nDivCount;x++)
			{	//Find the center points of all the edge sections
				pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
				if(!pD->GetNull())
				{
	 				arc=arcs[ArrayIndex]=(TWO_PI/total)*pD->GetValue();
					pPS[ArrayIndex].p=CPoint((int)(cx+(radius1*cos(angle+(arc/2)))),(int)(cy+(radius2*sin(angle+(arc/2)))));
					pPS[ArrayIndex].i=ArrayIndex;
					angles[ArrayIndex]=angle+(arc/2);
					groups[ArrayIndex]=CurrentSet+m_nMinSet;
					indices[ArrayIndex]=x+m_nMinDiv;
					angle+=arc;
					ArrayIndex++;
				}
			}
		}
		//sort the points into ascending Y order

		qsort(pPS,WedgeCount,sizeof(PointSorter),qsPointYSorterCompareA);


		for(x=0;x<WedgeCount;x++)
		{	// Then draw the skirt
	 		arc=tarc=arcs[pPS[x].i];
			angle=angles[pPS[x].i]-(arc/2);
			
				
			points=new CPoint[(4+(int)(arc/step))*2];
			
			pD=GetParent()->GetGroup(groups[pPS[x].i])->GetIndex(indices[pPS[x].i]);
			ptrStyle=pD->GetStyle();
		


			for(int n=0;tarc;)
			{
				if(sin(angle+tarc)>=0)
					points[n++]=CPoint((int)(cx+(radius1*cos(angle+tarc))),(int)(cy+(radius2*sin(angle+tarc)))); // dont add to the list if the edge is around the back
				if(tarc>step)
					tarc-=step;
				else
					tarc=0;
			}
			
			if(n && !pD->GetNull()) // only if there are points to fill
			{
				points[n++]=CPoint((int)(cx+(radius1*cos(angle))),(int)(cy+(radius2*sin(angle))));
			
				int p=n;

				//Now that the top edge is drawn we can simply duplicate all the points in reverse and move them
				//down to provide the bottom edge.

				for(int o=n;o;o--)
				{
					points[p++]=CPoint(points[o-1].x,(int)(points[o-1].y+dSkirtHeight));
				}
				
				points[p++]=points[0]; // close the polygon

				m_nCurrentIndex=pPS[x].i; // for the benefit of automatic pens and brushes.

				//The pie edge is now drawn into the point list and n holds the number
				//of points actually entered


				//ptrStyle currently holds the style for the individual data entry.
				//We do a quick check for highlighting

				if(ptrStyle->GetHighlight()!=CX_HIGHLIGHT_NONE)
				{
					// and act accordingly if needed
					// Take the arc, divide it by 2 and shift all the points in the array
					// by a percentage of the existing radius remembering not to push the
					// wedge over the edge of the display area.

					double highlightAngle=angles[pPS[x].i],
						   highlightRadius1,highlightRadius2;

					int hx,hy; // the offsets
					
					switch(ptrStyle->GetHighlight())
					{
					case CX_HIGHLIGHT_SMALL:
						highlightRadius1=radius1/100*5; // small is five percent
						highlightRadius2=radius2/100*5; 
						break;
					case CX_HIGHLIGHT_MEDIUM:
					case CX_HIGHLIGHT_AUTOMATIC:
						highlightRadius1=radius1/100*10; // medium is ten pecent
						highlightRadius2=radius2/100*10; 
						break;
					case CX_HIGHLIGHT_LARGE:
						highlightRadius1=radius1/100*20; // large is 20 percent.
						highlightRadius2=radius2/100*20; 
						break;
					default:
						ASSERT(FALSE); // You have used a highlight value which was'nt on the list
						break;
					}

					hx=(int)(highlightRadius1*cos(highlightAngle));
					hy=(int)(highlightRadius2*sin(highlightAngle));

					for(int c=0;c<p;c++)
						points[c].Offset(hx,hy);

					// the wall drawn earlier may have been overwritten by the end panel of an adjacent wedge

					//wall.SetSize(5);

					if(hx>0)
					{//re-draw the left side of the wedge (facing in)
						wall[0]=CPoint((int)(cx+(radius1*cos(angle+arc))),(int)(cy+(radius2*sin(angle+arc))));
						wall[1]=CPoint((int)(cx+((radius1-highlightRadius1)*cos(angle+arc))),(int)(cy+((radius2-highlightRadius2)*sin(angle+arc))));
						wall[2]=CPoint(wall[1].x,(int)(wall[1].y+dSkirtHeight));
						wall[3]=CPoint(wall[0].x,(int)(wall[0].y+dSkirtHeight));
						wall[4]=wall[0];
						SRGraphStyle *tptrStyle=ptrStyle;
						
						for(int c=0;c<5;c++)
							wall[c].Offset(hx,hy);
						GetParent()->LogPoly((LPPOINT)&wall,5,groups[pPS[x].i],indices[pPS[x].i]);
						FillPoly((LPPOINT)&wall,5,tptrStyle,0);
					}
					else
					{//re-draw the right side
						wall[0]=CPoint((int)(cx+(radius1*cos(angle))),(int)(cy+(radius2*sin(angle))));
						wall[1]=CPoint((int)(cx+((radius1-highlightRadius1)*cos(angle))),(int)(cy+((radius2-highlightRadius2)*sin(angle))));
						wall[2]=CPoint(wall[1].x,(int)(wall[1].y+dSkirtHeight));
						wall[3]=CPoint(wall[0].x,(int)(wall[0].y+dSkirtHeight));
						wall[4]=wall[0];
						SRGraphStyle *tptrStyle=ptrStyle;
						for(int c=0;c<5;c++)
							wall[c].Offset(hx,hy);
						GetParent()->LogPoly((LPPOINT)&wall,5,groups[pPS[x].i],indices[pPS[x].i]);
						FillPoly((LPPOINT)&wall,5,tptrStyle,0);
					}
				}

				FillPoly((LPPOINT)points,p,ptrStyle,0);
				FramePoly((LPPOINT)points,p,ptrStyle,0);
				GetParent()->LogPoly((LPPOINT)points,p,groups[pPS[x].i],indices[pPS[x].i]);
			}
			delete (points);
		}
	}
	angle=0;


	ArrayIndex=0;

	for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{
		for(x=0;x<m_nDivCount;x++)
		{	// Finally the pie shaped tops
			arc=tarc=(TWO_PI/total)*GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv)->GetValue();
			
			//points.SetSize(5+(int)(arc/step));
			points=new CPoint[5+(int)(arc/step)];
			
			points[0]=CPoint(cx,cy);

			pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
			ptrStyle=pD->GetStyle();

			if(!pD->GetNull())
			{

				for(int n=1; tarc; n++)
				{
					points[n]=CPoint((int)(cx+(radius1*cos(angle+tarc))),(int)(cy+(radius2*sin(angle+tarc))));
					if(tarc>step)
						tarc-=step;
					else
						tarc=0;
				}
				
				points[n++]=CPoint((int)(cx+(radius1*cos(angle))),(int)(cy+(radius2*sin(angle))));
				if(m_nDivCount*m_nSetCount==1)
					points[0]=points[n++]=points[1];
				else
					points[n++]=points[0];
				
				m_nCurrentIndex=ArrayIndex++; // for the benefit of automatic pens and brushes.

				//The pie wedge is now drawn into the point list and n holds the number
				//of points actually entered


				//ptrStyle currently holds the style for the individual data entry.
				//We do a quick check for highlighting

				if(ptrStyle->GetHighlight()!=CX_HIGHLIGHT_NONE)
				{
					// and act accordingly if needed
					// Take the arc, divide it by 2 and shift all the points in the array
					// by a percentage of the existing radius remembering not to push the
					// wedge over the edge of the display area.

					double highlightAngle=angle+(arc/2),
						   highlightRadius1,highlightRadius2;

					int hx,hy; // the offsets
					
					switch(ptrStyle->GetHighlight())
					{
					case CX_HIGHLIGHT_SMALL:
						highlightRadius1=radius1/100*5; // small is five percent
						highlightRadius2=radius2/100*5; 
						break;
					case CX_HIGHLIGHT_MEDIUM:
					case CX_HIGHLIGHT_AUTOMATIC:
						highlightRadius1=radius1/100*10; // medium is ten pecent
						highlightRadius2=radius2/100*10; 
						break;
					case CX_HIGHLIGHT_LARGE:
						highlightRadius1=radius1/100*20; // large is 20 percent.
						highlightRadius2=radius2/100*20; 
						break;
					default:
						ASSERT(FALSE); // You have used a highlight value which was'nt on the list
						break;
					}

					hx=(int)(highlightRadius1*cos(highlightAngle));
					hy=(int)(highlightRadius2*sin(highlightAngle));

					for(int c=0;c<n;c++)
						points[c].Offset(hx,hy);
				}
				GetParent()->LogPoly((LPPOINT)points,n,CurrentSet+m_nMinSet,x+m_nMinDiv);
				FillPoly((LPPOINT)points,n,ptrStyle,0);
				FramePoly((LPPOINT)points,n,ptrStyle,0);
				angle+=arc; // update the starting position for the next run
			}
			delete (points);
		}
	}

	delete (pPS);
	delete (angles);
	delete (groups);
	delete (indices);
	delete (arcs);
}


//@mfunc Draws strata plots.
//@rem A strata plot shows the summed total of a particular index for all groups. The topmost point of the plot is the 
//total and the stratified layers show how much each group contributes to the total. This plot is useful for example when 
//showing total sales of cars in a specific year and will also show what market share the individual manufacturers had.
//@see <mf SRGraphDisplay::DrawLineData> <mf SRGraphStyle::GetLog> <mf SRGraphStyle::SetLog>
void SRGraphDisplay::DrawStrataData()
{
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	int xt=m_nDivCount;

	CPen datapen(PS_SOLID,1,m_Style.GetFrameColor());

	BOOL bLog=GetStyle()->GetLog();
	
	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY;
	
	double	*LastValues=new double[m_nDivCount],dData;

	CPoint *points=new CPoint[(2*m_nDivCount)+2];

	for(int x=0;x<m_nDivCount;x++)
		LastValues[x]=0;

	for(int CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{
		int x2=(m_nDivCount*2)-1;
		SRGraphDataList *gdl=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE);

		m_nCurrentIndex=(CurrentSet+m_nMinSet);

		for(x=0;x<m_nDivCount;x++)
		{
			ASSERT(gdl!=NULL);
			SRGraphData *pD=gdl->GetIndex(x+m_nMinDiv,FALSE);
			if(pD->GetNull())
				SRThrowGraphException(SRGraphException::null);
			
			dData=pD->GetValue(bLog);
			if(bLog)
				dData-=log10(m_ScaleY.Min());
			else
				dData-=m_ScaleY.Min();

			points[x]=CPoint((int)((m_DisplayRect.left+(m_DisplayRect.Width()/(m_nDivCount>1 ? (m_nDivCount-1) : 1)*x))),(int)(m_DisplayRect.bottom-(ScalingFactor*LastValues[x])));
			points[x2--]=CPoint(points[x].x,(int)(points[x].y-(ScalingFactor*dData)));
			LastValues[x]+=dData;
		}
		points[m_nDivCount*2]=points[0];

		FillPoly((LPPOINT)points,1+(2*m_nDivCount),gdl->GetStyle(),0);

	}
	delete (points);
	delete (LastValues);
}



//@mfunc Stores or retrieves the object via the archive
//@parm A reference to the CArchive object used to manage the data file.
void SRGraphDisplay::Serialize(CArchive& ar)
{
	SRGraphTitle::Serialize(ar);


	if(ar.IsStoring())
	{
		ar << m_dMaxRangeY << m_dMinRangeY;
		ar << m_dMaxRangeX << m_dMinRangeX;
		ar << m_DisplayRect.left <<	m_DisplayRect.top << m_DisplayRect.right << m_DisplayRect.bottom;
	}
	else
	{
		ar >> m_dMaxRangeY >> m_dMinRangeY;
		ar >> m_dMaxRangeX >> m_dMinRangeX;
		ar >> m_DisplayRect.left >>	m_DisplayRect.top >> m_DisplayRect.right >> m_DisplayRect.bottom;
	}
}




//@mfunc Draws either nothing or an empty graph symbol in the case of null data
//being displayed in a graph
void SRGraphDisplay::DrawEmptyGraph()
{
		if(!m_Style.GetUseEmptyGraph())
			return;
		int cx=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		cy=m_DisplayRect.top+(m_DisplayRect.Height()/2); // find the central point of the pie
		BITMAP bmData;
		CBitmap nullbitmap,*OldBitmap;//,nullbitmapmask;
		CDC memDC;
		memDC.CreateCompatibleDC(m_pDC);
		nullbitmap.LoadBitmap(SRG_IDB_NULLPIE); // (there was a VERIFY here but the MFC librarys are full of ASSERTS anyway.) Did you include the Graph Resources?
		//nullbitmapmask.LoadBitmap(SRG_IDB_NULLPIEMASK);
		nullbitmap.GetBitmap(&bmData);
		OldBitmap=(CBitmap *)memDC.SelectObject(&nullbitmap);
		GetpDC()->BitBlt(cx-(bmData.bmWidth/2),cy-(bmData.bmHeight/2),bmData.bmWidth,bmData.bmHeight,&memDC,0,0,SRCCOPY);
		//GetpDC()->MaskBlt(cx-(bmData.bmWidth/2),cy-(bmData.bmHeight/2),bmData.bmWidth,bmData.bmHeight,&memDC,0,0,nullbitmapmask,0,0,MAKEROP4(SRCCOPY,SRCAND));
		memDC.SelectObject(OldBitmap);	
		nullbitmap.DeleteObject();
}

//@mfunc Draws the Y,X style labels for horizontal bars etc.
void SRGraphDisplay::DrawYXLabels()
{
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0)
		return;

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	SRGDecimalScale *yLabels;

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())
			m_ptrYLabels=new SRGLogScale;
		else
		{
			switch(GetStyle()->GetAxisStyle())
			{
			default:
			case CX_AXIS_YX:
 				m_ptrYLabels=new SRGDecimalScale;
				break;
			case CX_AXIS_ISO_DATE_YX:
			case CX_AXIS_DATE_YX:
 				m_ptrYLabels=new SRGDateScale;
				break;
			}
		}
	}

	ASSERT(m_ptrYLabels!=NULL);

	yLabels=(SRGDecimalScale *)m_ptrYLabels;

	if(m_ptrXLabels==NULL)
		m_ptrXLabels=new SRGraphLabelBlock;

	ASSERT(m_ptrXLabels!=NULL);

	m_ptrXLabels->SetParent(GetParent());
	yLabels->SetParent(GetParent());

	
	CString strTestString;

	CPoint ptStart,ptEnd;

	for(int pass=0;pass<2;pass++)
	{

		if(m_Style.GetShowIndices())
		{

			if(m_bSignificantChange)
			{
				for(int x=0;x<m_nDivCount;x++)
					m_ptrXLabels->AddLabelIndex(GetParent()->GetGroup(0)->GetIndex(m_nMinDiv+x)->GetAnnotation(GetCountryCode()),x);
				while(m_ptrXLabels->GetLabelCount()>m_nDivCount)
					m_ptrXLabels->RemoveLastLabel();
			}


			ptStart=CPoint(m_DisplayRect.left-5,m_DisplayRect.bottom-((m_DisplayRect.Height()/m_nDivCount)/2));
			ptEnd=CPoint(m_DisplayRect.left-5,m_DisplayRect.top+((m_DisplayRect.Height()/m_nDivCount)/2));

			m_ptrXLabels->DrawingUsing(m_pDC,m_pCWnd);
			m_ptrXLabels->SetLogTickPositions(TRUE);
			m_ptrXLabels->SetptrTickPositionList(&m_YTickList);
			m_ptrXLabels->SetFontSize(GetFontSize());
			m_ptrXLabels->SetFaceName(GetFaceName());
			m_ptrXLabels->SetTextColor(GetTextColor());
			m_ptrXLabels->SetFontStyle(GetFontStyle());
			m_ptrXLabels->SetBlockLocationPoints(SRGraphLabel::MidRight);
			m_ptrXLabels->SetBlockPositions(ptStart,ptEnd);
			m_ptrXLabels->SetBlockVectors(-1,0);
			m_ptrXLabels->SetPass(pass);
			m_ptrXLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
				m_DisplayRect.left+=m_ptrXLabels->GetBlockRect().Width();
		}

		if(m_Style.GetShowNumerals())
		{
			yLabels->DrawingUsing(m_pDC,m_pCWnd);
			yLabels->SetFontSize(GetFontSize());
			yLabels->SetFaceName(GetFaceName());
			yLabels->SetTextColor(GetTextColor());
			yLabels->SetFontStyle(GetFontStyle());
			yLabels->SetMinValue(m_ScaleY.Min());
			yLabels->SetMaxValue(m_ScaleY.Max());
			ptStart=(CPoint(m_DisplayRect.left,m_DisplayRect.bottom+5));
			ptEnd=CPoint(CPoint(m_DisplayRect.right,m_DisplayRect.bottom+5));
			yLabels->SetLogTickPositions(TRUE);
			yLabels->SetptrTickPositionList(&m_XTickList);
			yLabels->SetStartPos(ptStart);
			yLabels->SetEndPos(ptEnd);
			yLabels->SetAscending();
			yLabels->SetLabelSide(TRUE);
			yLabels->CreateLabels();
			yLabels->SetPass(pass);
			yLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
				m_DisplayRect.bottom-=yLabels->GetBlockRect().Height();
		}
	}


}

//@mfunc Called by the DrawForeground routine to draw Hi-Lo stock quote data
void SRGraphDisplay::DrawStockData()
{
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bLogData=m_Style.GetLog();

	SRGraphStyle InternalStyle,*ptrStyle;	// So that we can program polygons etc
	
	CPen datapen;

	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY;

	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Width())/(m_nSetCount*m_nDivCount),(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Width()/(m_nSetCount*m_nDivCount))/1.1;
	
	double dBlockWidth=(double)m_DisplayRect.Width()/m_nDivCount;
	double dSetOffset=(dBlockWidth/2)-((m_nSetCount*BarWidth)/2);

	int CurrentSet,x;

	m_nCurrentIndex=0;	

	switch(GetStyle()->GetGraphStyle())
	{
	default:
	case CX_GRAPH_STOCK:
	{
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			// NOTE it is possible to retrieve a pointer to the SRGraphData object and 
			// use it repeatedly. However if we dont know (or care) if the data is from 
			// SRGraphData or SRGraphDynamicData then we should use the correct (if lengthy)
			// indirections.

			datapen.CreatePen(PS_SOLID,1,GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetStyle()->GetFrameColor());
			CPen* OldPen=GetpDC()->SelectObject(&datapen);

			ptrStyle=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetStyle();

			for(x=0;x<m_nDivCount;x++)
			{
				SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
				if(!pD->GetNull())
				{
					double dValue=pD->GetValue();
					CScale hilo=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv)->GetHiLoData();

					int left,right,bottom,top;

					// Draw the HI bar..

					left=(int)((x*dBlockWidth)+dSetOffset+(CurrentSet*BarWidth)+m_DisplayRect.left);
					right=(int)(left+BarWidth);

					top=(int)((double)m_DisplayRect.bottom-(ScalingFactor*(hilo.Max()-m_ScaleY.Min()))+0.5);
					bottom=(int)((double)m_DisplayRect.bottom-(ScalingFactor*(dValue-m_ScaleY.Min()))+0.5);
									 
					if(bottom<top)
					{
						int t=top;
						top=bottom;
						bottom=t;
					}
					
					InternalStyle.SetType(CX_TYPE_OBJECT);
					
					if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_COLOR)
					{
						InternalStyle.SetInteriorStyle(CX_INTERIOR_COLOR);
						InternalStyle.SetColor(ptrStyle->GetColor(FALSE));
					}
					else
					{
						InternalStyle.SetInteriorStyle(CX_INTERIOR_AUTOMATIC);
						m_nCurrentIndex=2*CurrentSet;
					}

					DrawBar(CPoint(left,top),CPoint(right,bottom),CurrentSet+m_nMinSet,x+m_nMinDiv,&InternalStyle);

					//Then the LO bar..

					top=(int)((double)m_DisplayRect.bottom-(ScalingFactor*(hilo.Min()-m_ScaleY.Min()))+0.5);
					bottom=(int)((double)m_DisplayRect.bottom-(ScalingFactor*(dValue-m_ScaleY.Min()))+0.5);
									 
					if(bottom<top)
					{
						int t=top;
						top=bottom;
						bottom=t;
					}
					
					if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_COLOR)
					{
						InternalStyle.SetInteriorStyle(CX_INTERIOR_COLOR);
						InternalStyle.SetColor(ptrStyle->GetColor(TRUE));
					}
					else
					{
						InternalStyle.SetInteriorStyle(CX_INTERIOR_AUTOMATIC);
						m_nCurrentIndex=(2*CurrentSet)+1;
					}

					DrawBar(CPoint(left,top),CPoint(right,bottom),CurrentSet+m_nMinSet,x+m_nMinDiv,&InternalStyle);

					//Then the wiget which is selected automatically

					m_nCurrentIndex=(CurrentSet+m_nMinSet);

					InternalStyle.SetInteriorStyle(CX_INTERIOR_AUTOMATIC);
					GetParent()->LogPoint(CPoint((int)(left+(BarWidth/2)),top),m_nMinSet+CurrentSet,x+m_nMinDiv);
					DrawWiget(CPoint((int)(left+(BarWidth/2)),top),&InternalStyle);
				}
			}
			GetpDC()->SelectObject(OldPen);
			datapen.DeleteObject();
		}
		break;
	}
	case CX_GRAPH_AREASTOCK:
	{
		CPoint *pointsA,*pointsB;

		pointsA=new CPoint[1+(m_nDivCount*2)];
		pointsB=new CPoint[1+(m_nDivCount*2)];
		dBlockWidth=(double)m_DisplayRect.Width()/(m_nDivCount-1);
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
		{
			ptrStyle=GetParent()->GetGroup(m_nMinSet+CurrentSet)->GetStyle();

			for(x=0;x<m_nDivCount;x++)
			{
				SRGraphData *pD=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex(x+m_nMinDiv);
				if(pD->GetNull())
					SRThrowGraphException(SRGraphException::null);
				double dValue=pD->GetValue();
				pointsA[x]=pointsB[x]=CPoint((int)((double)m_DisplayRect.left+(x*dBlockWidth)),(int)((double)m_DisplayRect.bottom-(ScalingFactor*(dValue-m_ScaleY.Min())+0.5)));
			}

			int nX=x;

			for(x=m_nDivCount;x;x--)
			{
				CScale hilo=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex((x-1)+m_nMinDiv)->GetHiLoData();
				pointsA[nX]=CPoint((int)((double)m_DisplayRect.left+((x-1)*dBlockWidth)),(int)((double)m_DisplayRect.bottom-(ScalingFactor*(hilo.Max()-m_ScaleY.Min())+0.5)));
				pointsB[nX]=CPoint((int)((double)m_DisplayRect.left+((x-1)*dBlockWidth)),(int)((double)m_DisplayRect.bottom-(ScalingFactor*(hilo.Min()-m_ScaleY.Min())+0.5)));
				nX++;
			}

			pointsA[m_nDivCount*2]=pointsA[0];
			pointsB[m_nDivCount*2]=pointsB[0];

			switch(ptrStyle->GetInteriorStyle())
			{
			default:
			case CX_INTERIOR_AUTOMATIC:
				InternalStyle.SetInteriorStyle(CX_INTERIOR_AUTOMATIC);
				m_nCurrentIndex=(CurrentSet*2);
				break;
			case CX_INTERIOR_COLOR:
				InternalStyle.SetInteriorStyle(CX_INTERIOR_COLOR);
				InternalStyle.SetColor(ptrStyle->GetColor(FALSE));
				break;
			case CX_INTERIOR_RESOURCE:
				InternalStyle.SetInteriorStyle(CX_INTERIOR_RESOURCE);
				InternalStyle.SetResourceName(ptrStyle->GetResourceName());
				break;
			}

			InternalStyle.SetType(CX_TYPE_OBJECT);

			FillPoly(&pointsA[0],(m_nDivCount*2)+1,&InternalStyle,0);
			FramePoly(&pointsA[0],(m_nDivCount*2)+1,&InternalStyle,0);

			switch(ptrStyle->GetInteriorStyle())
			{
			default:
			case CX_INTERIOR_AUTOMATIC:
				m_nCurrentIndex=(CurrentSet*2)+1;
				break;
			case CX_INTERIOR_COLOR:
				InternalStyle.SetColor(ptrStyle->GetColor(TRUE));
				break;
			case CX_INTERIOR_RESOURCE:
				InternalStyle.SetResourceName(ptrStyle->GetFramePattern());
				break;
			}


			FillPoly(&pointsB[0],(m_nDivCount*2)+1,&InternalStyle,0);
			FramePoly(&pointsB[0],(m_nDivCount*2)+1,&InternalStyle,0);

		}

		delete (pointsA);
		delete (pointsB);
	}
	break;
	}

}



//@mfunc Returns the best scale for stepping between zero and m_dMaxScaleY
//@rdesc a floating point number which represents the best step
//@devnote This routine is redundant but has been left in place during the beta cycle. It will probably be removed soon so dont come to rely on it.
// The SRGDecimalScale class handles this descision much better.
double SRGraphDisplay::GetIdealScale()
{
	double multiplier;
	double temp=m_dMaxScaleY;

	if(m_dMaxScaleY>1)
	{	// scale is large..
		multiplier=1;
		while(temp>=10)
		{
			temp/=10;	// keep dividing by 10 until the number is less than 10
			multiplier*=10;
		} 
	}
	else
	{	// scale is small
		multiplier=0.1;
		while(temp<1)
		{
			temp*=10;  // or multiply by ten until its greater than 1
			multiplier/=10;
		}
	}
	
	return multiplier/10;
	
	// This returns the number which steps out to the next greatest or factor of 10 
	// So a maximum scale of 155305 will return a multiplier which steps to 160000 and 
	// a scale of 0.003435 will step to 0.004
}

//@mfunc Creates a list of numbers according to the step given in the "multiplier" parameter
//@parm a floating point number which is used to step from 0 to m_dMaxScaleY
//@parm A pointer to a CStringList which will be used to contain the number list text.
void SRGraphDisplay::CreateNumberList(double multiplier,CStringList *list)
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		SRThrowGraphException(SRGraphException::drawing);
	}
	CString buffer;	//temporary string
	double y=0;
	BOOL done=FALSE;
	do
	{
		if(y<=m_dMaxScaleY)
		{
			buffer.Format("%g",y);
			list->AddTail(buffer);
			y+=multiplier;
		}
		else
			done=1;
	}
	while(!done);
}


//@mfunc Draws "wigets". Polygonal tokens which represent data points on screen
//@parm The position of the wiget
//@parm a pointer to the SRGraphStyle object which controls the appearance of this token.
//@devnote After beta 1 a wiget and polygon cacheing system was added to reduce draw-time overheads
void SRGraphComponent::DrawWiget(CPoint point,SRGraphStyle *ptrStyle)
{
	CString wiget;

	switch(ptrStyle->GetObjectStyle())
	{
	case CX_OBJECT_NONE:
		return;
	case CX_OBJECT_AUTOMATIC:
		wiget.LoadString(IDS_SRG_WIGETNAMES+( m_nCurrentIndex & 7 )); // select an automatic wiget
		break;
	case CX_OBJECT_EXPOINT:
		if(ptrStyle->GetObjectName()=="")
			wiget.LoadString(IDS_SRG_WIGETNAMES+( m_nCurrentIndex & 7 )); // select an automatic wiget
		else
			wiget=ptrStyle->GetObjectName();
		break;
	default:
		wiget=ptrStyle->GetObjectName();
		break;
	}

	if(wiget.IsEmpty() || wiget=="") // if its empty
		return; // it could be a freestyle expoint with no wigetname!
	
	SRGCompiledPolygon *pCP=NULL;
	BOOL bFoundCompiledWiget=FALSE;
	
	for(POSITION p=m_PolygonList.GetHeadPosition();p!=NULL;)
	{
		pCP=(SRGCompiledPolygon *)m_PolygonList.GetNext(p);
		if(pCP->GetName()==wiget)
		{
			bFoundCompiledWiget=TRUE;
			p=NULL;
		}
	}

	if(!bFoundCompiledWiget)
	{
		pCP=new SRGCompiledPolygon;
		HINSTANCE hInst=AfxFindResourceHandle(wiget,"WIGETS");//AfxGetInstanceHandle();
		HRSRC polyResource=FindResource(hInst,wiget,"WIGETS");
		ASSERT(polyResource!=NULL);
		HGLOBAL polyHandle=LoadResource(hInst,polyResource);
		ASSERT(polyHandle!=NULL);
		char *polytext=(char *)LockResource(polyHandle);
		ASSERT(polytext!=NULL);
		int count,index,n;
		if(sscanf(polytext,"%d %n",&count,&index)!=1)
		{
			SRThrowGraphException(SRGraphException::wiget);
			delete (pCP);
			return;
		}
		
		for(int c=0;c<count;c++)
		{
			float x,y;
			if(sscanf(&polytext[index],"%g %g %n",&x,&y,&n)!=2)
			{
				SRThrowGraphException(SRGraphException::wiget);
				delete (pCP);
				return;
			}
			index+=n;
			pCP->AddCoord(x,y);
		}
		FreeResource(polyHandle);
		pCP->SetName(wiget);
		m_PolygonList.AddTail(pCP);
	}

	int nTempIndex=m_nCurrentIndex;
	m_nCurrentIndex=m_nCurrentIndex >> 3;

	if(ptrStyle->GetUseMaxSize())
	{
		CRect r(0,0,(int)(ptrStyle->GetMaxObjectWidth()*10),-(int)(ptrStyle->GetMaxObjectHeight()*10));	// the Y is made negative otherwise they come out upside-down.
		r.OffsetRect((-r.Width()/2)+point.x,(-r.Height()/2)+point.y);
		pCP->GetFittedPoints(r);
	}
	else
		pCP->OffsetPolygon((float)point.x,(float)point.y);
	FillPoly(pCP->GetPoints(),pCP->GetCount(),ptrStyle,0);
	FramePoly(pCP->GetPoints(),pCP->GetCount(),ptrStyle,CX_FRAME_LINE);

	m_nCurrentIndex=nTempIndex;
}

//@mfunc Draws a rectangular bar or polygonal object within the area set out by the coordinates
//@parm The top left of the bounding rectangle
//@parm The bottom right of the bounding rectangle
//@parm The group associated with this data.
//@parm The index associated with this data
//@parm A pointer to the SRGraphStyle object which controls the appearance of this object.
//@rem If the object style is OBJECT_POLYGON the system will first search in the "POLYGON" resource group
//for a named shape and then attempt to load a shape from disk using the m_strObjectName member in SRGraphStyle
void SRGraphComponent::DrawBar(CPoint topleft,CPoint bottomright, int group, int index,SRGraphStyle* ptrStyle,DWORD dwStyleOverride)
{

	CPoint *points=NULL;

	int temp;
	if(topleft.x>bottomright.x)
	{	
		temp=topleft.x; 
		topleft.x=bottomright.x; 
		bottomright.x=temp;
	}

	if(topleft.y>bottomright.y)
	{
		temp=topleft.y; 
		topleft.y=bottomright.y; 
		bottomright.y=temp;
	}

	int count;

	HGLOBAL polyHandle=NULL;

	DWORD TheStyle=dwStyleOverride ? dwStyleOverride : ptrStyle->GetObjectStyle();

	switch(TheStyle)
	{
	case CX_OBJECT_NONE:
		return;
	default:
	case CX_OBJECT_AUTOMATIC:
	case CX_OBJECT_BAR:
		points=new CPoint[5];
		count=5;

		//make a polygon from the positions

		points[0]=topleft;
		points[1]=CPoint(bottomright.x,topleft.y);
		points[2]=bottomright;
		points[3]=CPoint(topleft.x,bottomright.y);
		points[4]=points[0];
		GetParent()->LogPoly((LPPOINT)points,count,group,index);
		FillPoly((LPPOINT)points,count,ptrStyle,0);
		FramePoly((LPPOINT)points,count,ptrStyle,0);		
		break;
	case CX_OBJECT_POLYGON:
		{
			char *polytext;

			SRGCompiledPolygon *pCP=NULL;
			BOOL bFoundCompiledPoly=FALSE;

			CString PolyName=ptrStyle->GetObjectName();
			
			for(POSITION p=m_PolygonList.GetHeadPosition();p!=NULL;)
			{
				pCP=(SRGCompiledPolygon *)m_PolygonList.GetNext(p);
				if(pCP->GetName()==PolyName)
				{
					bFoundCompiledPoly=TRUE;
					p=NULL;
				}
			}

			BOOL bLoadedFromDisc=FALSE;

			if(!bFoundCompiledPoly)
			{
				pCP=new SRGCompiledPolygon;

				HINSTANCE hInst=AfxFindResourceHandle(PolyName,"POLYGON");
				ASSERT(hInst!=NULL);// resource cant be found in the app or any of the dll's
				HRSRC polyResource=FindResource(hInst,PolyName,"POLYGON");
				if(polyResource==NULL)
				{// load it from disk if possible
					TRY 
					{
						CFile tf(PolyName,CFile::modeRead);
						DWORD memSize=tf.GetLength();
						polyHandle=GlobalAlloc(GMEM_MOVEABLE,memSize);
						ASSERT(polyHandle!=NULL);
						polytext=(char *)GlobalLock(polyHandle);
						if(tf.ReadHuge(polytext,memSize)!=memSize)
						{
							GlobalUnlock(polyHandle);
							GlobalFree(polyHandle);
							AfxThrowFileException(CFileException::generic);
						}
						tf.Close();
						bLoadedFromDisc=TRUE;
					}
					CATCH(CFileException,e)
					{
						GlobalUnlock(polyHandle);
						GlobalFree(polyHandle);
						SRThrowGraphException(SRGraphException::polygon);
					}
					END_CATCH
				}
				else
				{
					polyHandle=LoadResource(hInst,polyResource);
					ASSERT(polyHandle!=NULL);
					polytext=(char *)LockResource(polyHandle);
					ASSERT(polytext!=NULL);
				}
			
				int count,index,n;
				if(sscanf(polytext,"%d %n",&count,&index)!=1)
				{
					SRThrowGraphException(SRGraphException::polygon);
					delete (pCP);
					return;
				}
				
				for(int c=0;c<count;c++)
				{
					float x,y;
					if(sscanf(&polytext[index],"%g %g %n",&x,&y,&n)!=2)
					{
						SRThrowGraphException(SRGraphException::polygon);
						delete (pCP);
						return;
					}
					index+=n;
					pCP->AddCoord(x,y);
				}
				if(bLoadedFromDisc)
				{
					GlobalUnlock(polyHandle);
					GlobalFree(polyHandle);
				}
				else
					FreeResource(polyHandle);
				pCP->SetName(PolyName);
				m_PolygonList.AddTail(pCP);
			}

			CRect barrect=CRect(topleft.x,topleft.y,bottomright.x,bottomright.y);
		
			GetParent()->LogPoly(pCP->GetFittedPoints(barrect),pCP->GetCount(),group,index);
			FillPoly(pCP->GetPoints(),pCP->GetCount(),ptrStyle,0);
			FramePoly(pCP->GetPoints(),pCP->GetCount(),ptrStyle,0);
		}
		break;
	case CX_OBJECT_EXBAR:
		{
			points=new CPoint[5];
			count=5;

			CBitmap shadeBitmap,glareBitmap;
			CBrush shadeBrush,glareBrush;
			BOOL shading=m_Style.GetObjectShadeStyle();
	
			if(shading)
			{
				shadeBitmap.LoadBitmap(SRG_IDB_LIGHTSHADOW);
				shadeBrush.CreatePatternBrush(&shadeBitmap);
				glareBitmap.LoadBitmap(SRG_IDB_LIGHTGLARE);
				glareBrush.CreatePatternBrush(&glareBitmap);
			}

			double MaxDepth=GetStyle()->GetUseMaxSize() ? GetStyle()->GetMaxObjectDepth() : 10;

			//make a polygon for the top face from the positions

			points[0]=topleft;
			points[1]=CPoint(bottomright.x,topleft.y);
			points[2]=CPoint((int)(points[1].x+MaxDepth),(int)(points[1].y-MaxDepth));
			points[3]=CPoint((int)(topleft.x+MaxDepth),(int)(topleft.y-MaxDepth));
			points[4]=points[0];
			
			GetParent()->LogPoly((LPPOINT)points,count,group,index);
			FillPoly((LPPOINT)points,count,ptrStyle,0);
			if(shading)
			{
				int OldMode=GetpDC()->SetROP2(R2_MERGEPEN);
				CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN),
				*oldBrush=GetpDC()->SelectObject(&glareBrush);
				GetpDC()->Polygon((LPPOINT)&points,5);
				GetpDC()->SelectObject(oldBrush);
				GetpDC()->SetROP2(OldMode);
				GetpDC()->SelectObject(OldPen);
			}
			FramePoly((LPPOINT)points,count,ptrStyle,0);

			//make a polygon for the side face from the positions

			points[0]=CPoint(bottomright.x,topleft.y);
			points[1]=CPoint((int)(bottomright.x+MaxDepth),(int)(topleft.y-MaxDepth));
			points[2]=CPoint((int)(bottomright.x+MaxDepth),(int)(bottomright.y-MaxDepth));
			points[3]=bottomright;
			points[4]=points[0];
			GetParent()->LogPoly((LPPOINT)points,count,group,index);
			FillPoly((LPPOINT)points,count,ptrStyle,0);
			if(shading)
			{
				int OldMode=GetpDC()->SetROP2(R2_MASKPEN);
				CGdiObject* OldPen=GetpDC()->SelectStockObject(NULL_PEN),
				*oldBrush=GetpDC()->SelectObject(&shadeBrush);
				GetpDC()->Polygon((LPPOINT)&points,5);
				GetpDC()->SelectObject(oldBrush);
				GetpDC()->SetROP2(OldMode);
				GetpDC()->SelectObject(OldPen);
			}
			FramePoly((LPPOINT)points,count,ptrStyle,0);
			FramePoly((LPPOINT)points,count,ptrStyle,0);

			//make a polygon for the front face from the positions

			points[0]=topleft;
			points[1]=CPoint(bottomright.x,topleft.y);
			points[2]=bottomright;
			points[3]=CPoint(topleft.x,bottomright.y);
			points[4]=points[0];
			GetParent()->LogPoly((LPPOINT)points,count,group,index);
			FillPoly((LPPOINT)points,count,ptrStyle,0);
			FramePoly((LPPOINT)points,count,ptrStyle,0);
			if(shading)
			{
				shadeBrush.DeleteObject();
				glareBrush.DeleteObject();
				glareBitmap.DeleteObject();
				shadeBitmap.DeleteObject();
			}
		}
		break;
	}

	if(points!=NULL)
		delete (points);
}

//@mfunc Sets the maximum Y value which can be used to display data in this graph
//@parm The range value
void SRGraphDisplay::SetMaxRangeY(double dValue)
{
	m_dMaxRangeY=dValue;
}

//@mfunc Sets the minimum Y value which can be used to display data in this graph
//@parm The range value
void SRGraphDisplay::SetMinRangeY(double dValue)
{
	m_dMinRangeY=dValue;
}

//@mfunc Interrogates the Y range setting
//@rdesc The maximum range value.
double SRGraphDisplay::GetMaxRangeY()
{
	return m_dMaxRangeY;
}

//@mfunc Interrogates the Y range setting
//@rdesc the minimum range value.
double SRGraphDisplay::GetMinRangeY()
{
	return m_dMinRangeY;
}

//@mfunc Sets the maximum X value which can be used to display data in this graph
//@parm The range value
void SRGraphDisplay::SetMaxRangeX(double dValue)
{
	m_dMaxRangeX=dValue;
}

//@mfunc Sets the minimum X value which can be used to display data in this graph
//@parm The range value
void SRGraphDisplay::SetMinRangeX(double dValue)
{
	m_dMinRangeX=dValue;
}

//@mfunc Interrogates the X range setting
//@rdesc The maximum X range value.
double SRGraphDisplay::GetMaxRangeX()
{
	return m_dMaxRangeX;
}

//@mfunc Interrogates the X range setting
//@rdesc the minimum X range value.
double SRGraphDisplay::GetMinRangeX()
{
	return m_dMinRangeX;
}


//@mfunc Removes all the elements from the specified tick list
//@parm A pointer to the tick list which should be emptied.
void SRGraphDisplay::KillTickList(CPtrList * pTickList)
{
	while(!pTickList->IsEmpty())
		delete ((CPoint *)pTickList->RemoveHead());
}



SRGraphLegend::~SRGraphLegend()
{
}

//@mfunc Overridden version of UpdateFont which decides whether to update and use the font contained in this label or to use the font in the parent SRGraphLabelBlock.
//@rem Generally labels have thier own individual fonts but when the label is part of a block the management of font style etc. is left to the label block itself. 
//This improves display speed.
BOOL SRGraphLabel::UpdateFont()
{
	ASSERT(IsDrawing()); //->Calling this outside of the draw loop may cause errors due to incorrect CWnd or CDC pointers in m_pCWnd and m_pDC
	if(!m_bUseLocalFont)
		return TRUE;
	LOGFONT lf;
	double xRes,yRes;
	xRes=(double)GetpDC()->GetDeviceCaps(LOGPIXELSX);
	yRes=(double)GetpDC()->GetDeviceCaps(LOGPIXELSY);
	
	if(m_nSize<0)
		lf.lfHeight=abs(m_nSize); // in device pixels
	else
		lf.lfHeight=(int)(yRes/72*m_nSize); // or in points
	
	lf.lfWidth=0;
	lf.lfOrientation=lf.lfEscapement=(int)( 10 * GetOrientation());
	lf.lfWeight=m_nStyle & 1 ? 700 : 400;
	lf.lfItalic=(BYTE)(m_nStyle & 2);
	lf.lfUnderline=(BYTE)(m_nStyle & 4);
	lf.lfStrikeOut=(BYTE)(m_nStyle & 8);
	lf.lfCharSet=ANSI_CHARSET;
	lf.lfOutPrecision=0;
	lf.lfClipPrecision=0;
	lf.lfQuality=0;
	lf.lfPitchAndFamily=0;
	strncpy(lf.lfFaceName,(const char *)m_strFaceName,30);
	if(m_Font.GetSafeHandle()!=NULL)
		m_Font.DeleteObject();
	return m_Font.CreateFontIndirect(&lf);
}

//@mfunc Calculates and returns a region from the polygon which describes the outside of this label.
//@rem In thier un-rotated states labels have polygonal outlines which are simple rectangles. This "text extent" is used to calculate overlaps and to draw filled backgrounds and shadows. 
//When the label is rotated off of the horizontal it is neccesary to draw backgrounds and shadows rotated and to do overlap calculations on a region. 
CRgn * SRGraphLabel::GetBoundingRegion()
{
	CPoint points[5];
	GetBoundingPolygon(&points[0]);
	CRgn *rgn=new CRgn;
	rgn->CreatePolygonRgn(&points[0],5,ALTERNATE);
	return rgn;
}

//@mfunc Fills a CPoint array with the 5 points needed to desgribe a closed rectangular polygon corresponding to the area covered by the text in this label. 
//@parm A pointer to the point array (must be at least 5 CPoint objects).
//@parm A count of the points available. Must be 5 or more.
//@rem This routine takes into account the text location point, the font and the angle of rotation to provide an accurate boundary for the label.
//@devnote This routine may only be used when the component has a valid CDC installed in its m_pDC member. 
void SRGraphLabel::GetBoundingPolygon(CPoint * points,int nPointCount)
{
	nPointCount;//UNUSED
	ASSERT(IsDrawing()); // This routine relies on a valid m_pDC
	ASSERT(nPointCount>=5);

	CFont * oldfont=(CFont *)GetpDC()->SelectObject(GetFont());
	CSize s=GetpDC()->GetTextExtent((LPCSTR)m_Labels.GetAnnotation(GetCountryCode()));
	GetpDC()->SelectObject(oldfont);

	switch(m_nLocationPoint)
	{
	case TopLeft:
		points[0]=CPoint(0,0);
		points[1]=CPoint(s.cx,0);
		points[2]=CPoint(s.cx,s.cy);
		points[3]=CPoint(0,s.cy);
		points[4]=points[0];
		break;
	case TopCenter:
		points[0]=CPoint(-(s.cx/2),0);
		points[1]=CPoint(s.cx/2,0);
		points[2]=CPoint(s.cx/2,s.cy);
		points[3]=CPoint(-(s.cx/2),s.cy);
		points[4]=points[0];
		break;
	case TopRight:
		points[0]=CPoint(-s.cx,0);
		points[1]=CPoint(0,0);
		points[2]=CPoint(0,s.cy);
		points[3]=CPoint(-s.cx,s.cy);
		points[4]=points[0];
		break;
	case MidLeft:
		points[0]=CPoint(0,-(s.cy/2));
		points[1]=CPoint(s.cx,-(s.cy/2));
		points[2]=CPoint(s.cx,s.cy/2);
		points[3]=CPoint(0,s.cy/2);
		points[4]=points[0];
		break;
	case MidCenter:
		points[0]=CPoint(-(s.cx/2),-(s.cy/2));
		points[1]=CPoint(s.cx/2,-(s.cy/2));
		points[2]=CPoint(s.cx/2,s.cy/2);
		points[3]=CPoint(-(s.cx/2),s.cy/2);
		points[4]=points[0];
		break;
	case MidRight:
		points[0]=CPoint(-s.cx,-(s.cy/2));
		points[1]=CPoint(0,-(s.cy/2));
		points[2]=CPoint(0,s.cy/2);
		points[3]=CPoint(-s.cx,s.cy/2);
		points[4]=points[0];
		break;
	case BottomLeft:
		points[0]=CPoint(0,-s.cy);
		points[1]=CPoint(s.cx,-s.cy);
		points[2]=CPoint(s.cx,0);
		points[3]=CPoint(0,0);
		points[4]=points[0];
		break;
	case BottomCenter:
		points[0]=CPoint(-(s.cx/2),-s.cy);
		points[1]=CPoint(s.cx/2,-s.cy);
		points[2]=CPoint(s.cx/2,0);
		points[3]=CPoint(-(s.cx/2),0);
		points[4]=points[0];
		break;
	case BottomRight:
		points[0]=CPoint(-s.cx,-s.cy);
		points[1]=CPoint(0,-s.cy);
		points[2]=CPoint(0,0);
		points[3]=CPoint(-s.cx,0);
		points[4]=points[0];
		break;
	}
	// The coordinate points of the rectangle are plotted
	// now to rotate them into thier final positions

	double angle=RADIANS(GetOrientation());
	
	// to do this we need to convert the device independant coords to pixels.
	CalcPos();
	for(int n=0;n<5;n++)
	{
		double x1=(points[n].x * cos(angle)) + (points[n].y * sin(angle)),
			   y1=(points[n].y * cos(angle)) - (points[n].x * sin(angle));

		points[n]=CPoint((int)x1,(int)y1)+m_TextPosition;
	}
}

//@mfunc Returns a point at which the CDC::TextOut function can place the text to position it exactly over the desired bounding rectangle.
//@rdesc A CPoint with the calculated screen position.
//@devnote Normally text is positioned according to the top-left corner of the text string. The location point system in SRGraphLabel requires that this 
//point be offset to account for any one of the nine location points and the desired rotation of text.
CPoint SRGraphLabel::GetTextOrigin()
{
	ASSERT(IsDrawing()); // This routine relies on a valid m_pDC

	CSize s=GetpDC()->GetTextExtent((LPCSTR)m_Labels.GetAnnotation(GetCountryCode()));

	CPoint point;

	switch(m_nLocationPoint)
	{
	case TopLeft:
		point=CPoint(0,0);
		break;
	case TopCenter:
		point=CPoint(-(s.cx/2),0);
		break;
	case TopRight:
		point=CPoint(-s.cx,0);
		break;
	case MidLeft:
		point=CPoint(0,-(s.cy/2));
		break;
	case MidCenter:
		point=CPoint(-(s.cx/2),-(s.cy/2));
		break;
	case MidRight:
		point=CPoint(-s.cx,-(s.cy/2));
		break;
	case BottomLeft:
		point=CPoint(0,-s.cy);
		break;
	case BottomCenter:
		point=CPoint(-(s.cx/2),-s.cy);
		break;
	case BottomRight:
		point=CPoint(-s.cx,-s.cy);
		break;
	}

	// The coordinate point of the text origin is plotted
	// now to rotate it into it's final position
	
	double angle=RADIANS(GetOrientation());
	double x1=(point.x * cos(angle)) + (point.y * sin(angle)),
		   y1=(point.y * cos(angle)) - (point.x * sin(angle));

	point=CPoint((int)x1,(int)y1)+m_TextPosition;
	return point;
}

//@mfunc Overrides the standard component fill to provide rotated rectangular backgrounds.
void SRGraphLabel::DrawFill()
{
	if(m_bBlockMember)
		return;
	CPoint points[5];
	GetBoundingPolygon(&points[0],5);
	FillPoly(&points[0],5,GetStyle(),NULL);
}

//@mfunc Overrides the standard component border routines to provide rotated borders.
//@rem Note that the 3D "bevelled" borders are not available for labels. This style is replaced by a simple line border.
void SRGraphLabel::DrawBorder()
{
	if(m_bBlockMember)
		return;

	if(GetStyle()->GetComponentBorderStyle()==CX_NO_BORDER)
		return;
	
	// This ia a toughie. We need to draw a border which is consistent(ish) with the
	// 10-o-clock sun (2-o-clock if youre in the southern hemisphere) style of the borders and shadows
	// 

	CPen pens[3],xpen;

	DWORD dwBorder=GetStyle()->GetComponentBorderStyle();

	int nPenWidth=((dwBorder==CX_THIN_RAISED_BORDER ||
					dwBorder==CX_THIN_SUNKEN_BORDER ||
					dwBorder==CX_THIN_3D_BORDER ||
					dwBorder==CX_FRAME_LINE) ? 1 : 3);
	
	pens[0].CreatePen(PS_SOLID,nPenWidth,CXCLR_WHITE);
	pens[1].CreatePen(PS_SOLID,nPenWidth,CXCLR_GRAY30);
	pens[2].CreatePen(PS_SOLID,nPenWidth,CXCLR_BLACK);

	int PenIndexArray[32]={0,2,2,0,
					       0,1,2,1,
						   0,0,2,2,
						   1,0,1,2,
						   2,0,0,2,
						   2,1,0,1,
						   2,2,0,0,
						   1,2,1,0};

	CPoint points[5];
	GetBoundingPolygon(&points[0],5);

	double dAngle=GetOrientation()+27.5;
	if(dAngle<0)
		dAngle+=360;
	while(dAngle>360)
		dAngle-=360;
	dAngle/=45;

	for(int nSide=0;nSide<4;nSide++)
	{
		CPen *oldpen;
		switch(dwBorder)
		{
		case CX_THIN_SUNKEN_BORDER:
		case CX_THICK_SUNKEN_BORDER:
			oldpen=(CPen *)GetpDC()->SelectObject(&pens[PenIndexArray[((4*(int)dAngle)+nSide+16)&0x1f]]);
			GetpDC()->MoveTo(points[nSide]);
			GetpDC()->LineTo(points[nSide+1]);
			GetpDC()->SelectObject(oldpen);
			break;
		case CX_THIN_RAISED_BORDER:
		case CX_THICK_RAISED_BORDER:
			oldpen=(CPen *)GetpDC()->SelectObject(&pens[PenIndexArray[((4*(int)dAngle)+nSide)&0x1f]]);
			GetpDC()->MoveTo(points[nSide]);
			GetpDC()->LineTo(points[nSide+1]);
			GetpDC()->SelectObject(oldpen);
			break;
		case CX_THIN_3D_BORDER:
		case CX_THICK_3D_BORDER:
		case CX_LINE_BORDER:
			xpen.CreatePen(PS_SOLID,1,GetStyle()->GetFrameColor());
			oldpen=(CPen *)GetpDC()->SelectObject(&xpen);
			GetpDC()->MoveTo(points[nSide]);
			GetpDC()->LineTo(points[nSide+1]);
			GetpDC()->SelectObject(oldpen);
			break;
		default:
			DrawCustomLabelBorder();
			break;
		}
	}
}

//@mfunc Draws a correctly oriented (ie rotated) shadow under the object.
void SRGraphLabel::DrawShadow()
{
	if(m_bBlockMember)
		return;

	if(GetStyle()->GetComponentShadowStyle()==CX_NO_SHADOW)
		return;

	int nThickness;

	switch(GetStyle()->GetComponentShadowStyle())
	{
	case CX_THIN_SHADOW:
		nThickness=3;
		break;
	case CX_MEDIUM_SHADOW:
		nThickness=8;
		break;
	case CX_THICK_SHADOW:
		nThickness=16;
		break;
	default:
		DrawCustomLabelShadow();
		return;
	}

	CRgn RgnA,RgnB,RgnC;

	CPoint points[5];
	GetBoundingPolygon(&points[0],5);

	RgnA.CreatePolygonRgn(&points[0],5,ALTERNATE);
	RgnB.CreatePolygonRgn(&points[0],5,ALTERNATE);
	RgnC.CreateRectRgn(0,0,1,1);
	RgnB.OffsetRgn(nThickness,nThickness);
	RgnC.CombineRgn(&RgnA,&RgnB,RGN_OR);
	RgnB.CombineRgn(&RgnA,&RgnC,RGN_XOR);

	CBrush b;
	CBitmap bm;

	bm.LoadBitmap(SRG_IDB_SHADOW);
	b.CreatePatternBrush(&bm);

	int OldMode=GetpDC()->SetROP2(R2_MASKPEN);

	GetpDC()->FillRgn(&RgnB,&b);

	GetpDC()->SetROP2(OldMode);
	
	b.DeleteObject();
	bm.DeleteObject();
	RgnA.DeleteObject();
	RgnB.DeleteObject();
	RgnC.DeleteObject();
}

//@mfunc User overridable shadow routine
//@devnote This routine and all the other "DrawCustom...." routines are provided to allow you the simplest method
//of expanding the capabilities of the component classes. If the DWORD style which is associated with fills, borders, shadows etc is unrecognised by the standard
// routines a call is made to one of these virtual functions. The debug versions of these functions ASSERT's if not overridden. The release versions do nothing.
void SRGraphLabel::DrawCustomLabelShadow()
{
	ASSERT(FALSE); // You must override this function to perform some useful task
}

//@mfunc User Overridable border routine
//@see <mf SRGraphLabel::DrawCustomLabelShadow> for details on custom component drawing.
void SRGraphLabel::DrawCustomLabelBorder()
{
	ASSERT(FALSE); // You must override this function to perform some useful task
}

//@mfunc User Overridable fill routine
//@see <mf SRGraphLabel::DrawCustomLabelShadow> for details on custom component drawing.
void SRGraphLabel::DrawCustomLabelFill()
{
	ASSERT(FALSE); // You must override this function to perform some useful task
}


//@mfunc Sets the text of this label.
//@parm Pointer to the zero terminated text or a CString object containing the text.
void SRGraphLabel::SetAnnotation(LPCSTR strText)
{
	SetAnnotation(GetParent()->GetCountryCode(),strText);
}

//@mfunc Sets the locale specific text of this label.
//@parm The integer value of the country code
//@parm Pointer to the zero terminated text or a CString object containing the text.
void SRGraphLabel::SetAnnotation(int code,LPCSTR strText)
{
	m_Labels.SetAnnotation(code,strText);
}

SRGraphLabel::SRGraphLabel()
{
	m_bBlockMember=FALSE;
	m_bMoveable=FALSE;
	m_bLabelStandard=TRUE;
	m_bVectorStandard=TRUE;
	m_bUseLocalFont=TRUE;
	SetOrientation(0);
	m_dVectorX=1;
	m_dVectorY=0;
	m_nLocationPoint=TopLeft;
	m_TextPosition=CPoint(0,0);
	m_nType=IDS_SRG_LABELTYPE;
	m_strFaceName="Arial";
	m_nSize=10;
	m_nJustification=0;
	m_nStyle=0;
}

//@mfunc Returns a pointer to the relavent SRGraphStyle object
//@rdesc The pointer to the SRGraphStyle
//@rem Labels which are a part of a label block do not use or maintain thier own styles. Rather they take thier style from the SRGraphStyle object of the parent block.
SRGraphStyle * SRGraphLabel::GetStyle()
{
	return m_bBlockMember ? m_pLabelBlock->GetStyle() : &m_Style;
}

//@mfunc Draws the text of this label in the correct position taking into account rotation etc.
void SRGraphLabel::DrawForeground()
{

	CFont *OldFont=(CFont*)GetpDC()->SelectObject(GetFont()); // select the font into the DC

	COLORREF OldColor=GetpDC()->SetTextColor(GetTextColor());
	int OldMode=GetpDC()->SetBkMode(TRANSPARENT);

	CalcPos();

	CPoint TextPoint=GetTextOrigin();

	GetpDC()->TextOut(TextPoint.x,TextPoint.y,m_Labels.GetAnnotation(GetCountryCode()));
	
	GetpDC()->SetBkMode(OldMode);
	GetpDC()->SelectObject(OldFont);
	GetpDC()->SetTextColor(OldColor);
}

//@mfunc Returns a pointer the correct font 
//@rdesc The pointer to the relavent CFont object
//@rem If this label is a "stand alone" item then it will use its own internal font. It will also use this font if it has been instructed to use the "local font". If however
// this label is a member of a label block and no local font usagewas specified then the routine will return a pointer to the font controlled by the SRGraphLabelBlock parent.
CFont * SRGraphLabel::GetFont()
{
	if(m_bBlockMember && !m_bUseLocalFont)
		return m_pLabelBlock->GetFont();
	else
		return &m_Font;
}

//@mfunc Writes the name of this object to the template file.
//@parm A pointer to the STDioFile object used in template operations
void SRGraphLabel::WriteName(CStdioFile *file)
{
   file->WriteString("label ");
}

//@mfunc Writes the SRGraphLabel parameters to the template file.
//@parm A pointer to the STDioFile object used in template operations
void SRGraphLabel::WriteSpecificParameters(CStdioFile * file)
{
	CString s;
	s.Format(" %g %g %g %g %g %d %d\n",
				m_dPosX,m_dPosY,
				m_dVectorX,m_dVectorY,
				GetOrientation(),
				m_nLocationPoint,
				m_bBlockMember);
	file->WriteString(s);
	m_Labels.WriteList(file);
}

//@mfunc Overridden Draw function.
//@parm The CDC to use in drawing operations
//@parm Points to the CWnd which is being used to display the graph.
void SRGraphLabel::Draw(CDC *pDC,CWnd *pWnd)
{
	m_pDC=pDC;
	m_pCWnd=pWnd;
	m_bDrawing=TRUE;
	UpdateFont();
	SRGraphTitle::Draw(pDC,pWnd);
	m_bDrawing=FALSE;
}

//@mfunc Sets a label vector by a simple integer code.
//@parm The integer code which may be one of..<nl>
//	VectorRight	<nl>
//	VectorLeft <nl>
//	VectorUp <nl>
//	VectorDown <nl>
//These vectoring options also set the m_bVectorStandard flag.
void SRGraphLabel::SetVector(int nVectorCode)
{
	switch(nVectorCode)
	{
	case VectorRight:
		m_dVectorX=1;
		m_dVectorY=0;
		break;
	case VectorLeft:
		m_dVectorX=-1;
		m_dVectorY=0;
		break;
	case VectorUp:
		m_dVectorX=0;
		m_dVectorY=-1;
		break;
	case VectorDown:
		m_dVectorX=0;
		m_dVectorY=1;
		break;
	default:
		m_bVectorStandard=FALSE;
		return;
	}

	m_bVectorStandard=TRUE;
}

//@mfunc Sets label orientation to a standard value.
//@parm The label orientation. The name of the standard denotes the reading direction. May be one of..<nl>
//	Normal<nl>
//	Downward<nl>
//	Upward<nl>
//	Inverted<nl>
void SRGraphLabel::SetLabelStandard(int nLabelStandard)
{
	switch(nLabelStandard)
	{
	case Normal:
		SetOrientation(0);
		break;
	case Downward:
		SetOrientation(270);
		break;
	case Upward:
		SetOrientation(90);
		break;
	case Inverted:
		SetOrientation(180);
		break;
	default:
		m_bLabelStandard=FALSE;
		return;
	}

	m_bLabelStandard=TRUE;
}

//@mfunc Sets the label anti-collision vector to a specified angle (in degrees)
//@parm The angle toward which this label should move in case of label overprinting.
//@rem If the chosen vector is a "standatr" one ie. 0, 90, 180 or 270 degrees then the m_bVectorStandard flag will be set.
void SRGraphLabel::SetVector(double dAngle)
{
	m_dVectorX=cos(RADIANS(dAngle));
	m_dVectorY=sin(RADIANS(dAngle));
	if(	dAngle==0 ||
		dAngle==90 ||
		dAngle==180 ||
		dAngle==270 )
		m_bVectorStandard=TRUE;
	else
		m_bVectorStandard=FALSE;
}

//@mfunc Adds a new label to the list and annotates the label with the provided text.
//@parm The text.
void SRGraphLabelBlock::AddLabel(LPCSTR strLabelText)
{
	SRGraphLabel *pL=new SRGraphLabel;
	
	pL->SetBlockMember(TRUE);
	pL->SetUseLocalFont(FALSE);
	pL->SetLabelBlock(this);
	pL->SetParent(m_pParentGraph);
	pL->SetAnnotation(strLabelText);

	m_LabelList.AddTail(pL);
}



//@mfunc Destroys the label block and ensures cleanup of the label list and any owned tick list.
SRGraphLabelBlock::~SRGraphLabelBlock()
{
	while(!m_LabelList.IsEmpty())
		delete ((SRGraphLabel *)m_LabelList.RemoveHead());

	if(m_bUseMyOwnTickList && m_pTickPositionList!=NULL)
	{
		while(!m_pTickPositionList->IsEmpty())
			delete ((CPoint *)m_pTickPositionList->RemoveHead());
	}
}



//@mfunc Sets the positions of the labels owned by this block.
//@parm The starting position
//@parm The end position
//@rem This routine strings out all of the label positions so that they are equally spaced along a line drawn between the start and end points.
// At this stage no anti collision measures are taken so the actual positions given to the labels may indeed mean that they could overlap. 
//All anti collision calculations are done at draw time.
void SRGraphLabelBlock::SetBlockPositions(CPoint StartPoint, CPoint EndPoint)
{
	if(m_LabelList.IsEmpty())
		return;
	
	int nLabelCount=m_LabelList.GetCount();

	double dXStep=(double)(EndPoint.x - StartPoint.x)/ (nLabelCount>1 ? nLabelCount-1 : 1),
	       dYStep=(double)(EndPoint.y - StartPoint.y)/ (nLabelCount>1 ? nLabelCount-1 : 1);
	
	int nLabelNumber=0;
	
	if(nLabelCount==1)
		nLabelNumber=1;
	
	for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;nLabelNumber++)
		((SRGraphLabel *)m_LabelList.GetNext(p))->SetTextPosition(StartPoint.x+(int)(dXStep*nLabelNumber),StartPoint.y+(int)(dYStep*nLabelNumber));

	UpdateTickList();
}

 

//@mfunc Overridden from the SRGraphLabel equivalent to draw all of the text in all of the labels held by this block.
void SRGraphLabelBlock::DrawForeground()
{
	if(m_LabelList.IsEmpty())
		return;

	if(m_pDC->IsPrinting() || m_bSignificantChange)
		MakeFont=TRUE;

	if(MakeFont)
		UpdateFont();

	m_BlockRect=CRect(0,0,0,0);

	switch(GetStyle()->GetLabelStyle())
	{
	default:
	case CX_LABELBLOCK_HIDELABELS:
		// This option leaves the ticks but dosent show certain labels.
		{	
			CScale Width,Height;
			POSITION p=m_LabelList.GetHeadPosition();
			SRGraphLabel *gL=(SRGraphLabel *)m_LabelList.GetNext(p); // we dont need the first one.
			CPoint gpoints[5];
			gL->GetBoundingPolygon(&gpoints[0],5);
			if(m_bPass1)
				gL->Draw(m_pDC,m_pCWnd);
			for(int c=0;c<5;c++)
			{
				Width+=gpoints[c].x;
				Height+=gpoints[c].y;
			}
			m_BlockRect=CRect((int)Width.Min(),(int)Height.Min(),(int)Width.Max(),(int)Height.Max());
			m_BlockRect.NormalizeRect();

			for(p;p!=NULL;)
			{
				
				SRGraphLabel *pL=(SRGraphLabel *)m_LabelList.GetNext(p);

				CPoint lpoints[5];
				pL->GetBoundingPolygon(&lpoints[0],5);
				CRgn lRgn,tRgn,wRgn;

				if(m_bCheckOverlaps)
				{
					if(pL->GetLabelStandard())
					{	// we can check for overlap using CRects
						CRect lR,gR,wR;
						lR=CRect(lpoints[0].x,lpoints[0].y,lpoints[2].x,lpoints[2].y);
						lR.NormalizeRect();
						gR=CRect(gpoints[0].x,gpoints[0].y,gpoints[2].x,gpoints[2].y);
						gR.NormalizeRect();
						if(wR.IntersectRect(&lR,&gR))
							continue;
						else
							goto outputlabel;
					}
					else
					{	// We must use CRgns to check overlaps (slow but sure)
						lRgn.CreatePolygonRgn(&lpoints[0],5,ALTERNATE);
						tRgn.CreatePolygonRgn(&gpoints[0],5,ALTERNATE);
						wRgn.CreateRectRgn(0,0,1,1);
						if(wRgn.CombineRgn(&lRgn,&tRgn,RGN_AND)!=NULLREGION)
							continue;
						else
						{
outputlabel:
							Width.Zero();
							Height.Zero();
							for(int c=0;c<5;c++)
							{
								Width+=lpoints[c].x;
								Height+=lpoints[c].y;
								gpoints[c]=lpoints[c];
							}
							Width+=m_BlockRect.left;
							Width+=m_BlockRect.right;
							Height+=m_BlockRect.top;
							Height+=m_BlockRect.bottom;
							m_BlockRect=CRect((int)Width.Min(),(int)Height.Min(),(int)Width.Max(),(int)Height.Max());
							m_BlockRect.NormalizeRect();
							
							if(m_bPass1)
								pL->Draw(m_pDC,m_pCWnd);
							gL=pL;
						}	
					}
				}
				else
				{
					Width.Zero();
					Height.Zero();
					for(int c=0;c<5;c++)
					{
						Width+=lpoints[c].x;
						Height+=lpoints[c].y;
						gpoints[c]=lpoints[c];
					}
					Width+=m_BlockRect.left;
					Width+=m_BlockRect.right;
					Height+=m_BlockRect.top;
					Height+=m_BlockRect.bottom;
					m_BlockRect=CRect((int)Width.Min(),(int)Height.Min(),(int)Width.Max(),(int)Height.Max());
					m_BlockRect.NormalizeRect();
					
					if(m_bPass1)
						pL->Draw(m_pDC,m_pCWnd);
				}
			}
		}
		break;

	case CX_LABELBLOCK_MOVELABELS:
		for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;)
		{
			SRGraphLabel *pL=(SRGraphLabel *)m_LabelList.GetNext(p);
			
			CPoint TextPosition=pL->GetTextPosition(); // The position is "stacked"
			
			CPoint points[5];
			pL->GetBoundingPolygon(&points[0],5);

			if(m_bCheckOverlaps)
			{
				for(POSITION t=m_LabelList.GetHeadPosition();t!=p;)
				{
					SRGraphLabel * pT=(SRGraphLabel *)m_LabelList.GetNext(t);
					
					CString strLabelString=pT->GetAnnotation(GetParent()->GetCountryCode());
					if(strLabelString.GetLength()!=0 && pL!=pT)
					{
						CPoint tpoints[5];
						pT->GetBoundingPolygon(&tpoints[0],5);

						if(pL->GetLabelStandard())
						{	// we can check for overlap using CRects
							CRect lR,tR,wR;
							lR=CRect(points[0].x,points[0].y,points[2].x,points[2].y);
							lR.NormalizeRect();
							tR=CRect(tpoints[0].x,tpoints[0].y,tpoints[2].x,tpoints[2].y);
							tR.NormalizeRect();
							while(wR.IntersectRect(&lR,&tR))
							{
								double dMultiplier=1;
								int xMove=0,yMove=0;
								do
								{
									xMove=(int)(dMultiplier*pL->GetVectorX()*wR.Width()),
									yMove=(int)(dMultiplier*pL->GetVectorY()*wR.Height());
								}
								while(xMove==0 && yMove==0 && ++dMultiplier<=10);
								if(dMultiplier>10)
									SRThrowGraphException(SRGraphException::label);
								lR.OffsetRect(xMove,yMove);
								OffsetPolygon(&points[0],5,xMove,yMove);
								pL->SetTextPosition(pL->GetTextPosition()+CPoint(xMove,yMove));
							}
							// At this point the label has been moved;
						}
						else
						{	// We must use CRgns to check overlaps (slow but sure)
							CRgn lRgn,tRgn,wRgn;
							lRgn.CreatePolygonRgn(&points[0],5,ALTERNATE);
							tRgn.CreatePolygonRgn(&tpoints[0],5,ALTERNATE);
							wRgn.CreateRectRgn(0,0,1,1);
							while(wRgn.CombineRgn(&lRgn,&tRgn,RGN_AND)!=NULLREGION)
							{
								CRect rcRgn;
								wRgn.GetRgnBox(&rcRgn);
								double dMultiplier=1;
								int xMove=0,yMove=0;
								do
								{
									xMove=(int)(dMultiplier*pL->GetVectorX()*rcRgn.Width()),
									yMove=(int)(dMultiplier*pL->GetVectorY()*rcRgn.Height());
								}
								while(xMove==0 && yMove==0 && ++dMultiplier<=10);
								if(dMultiplier>10)
									SRThrowGraphException(SRGraphException::label);
								lRgn.OffsetRgn(xMove,yMove);
								OffsetPolygon(&points[0],5,xMove,yMove);
								pL->SetTextPosition(pL->GetTextPosition()+CPoint(xMove,yMove));
							}
						}
					}
				}
			}
		
			CScale Width,Height;

			for(int c=0;c<5;c++)
			{
				Width+=points[c].x;
				Height+=points[c].y;
			}

			if(!m_BlockRect.IsRectEmpty())
			{
				Width+=m_BlockRect.left;
				Width+=m_BlockRect.right;
				Height+=m_BlockRect.top;
				Height+=m_BlockRect.bottom;
			}

			m_BlockRect=CRect((int)Width.Min(),(int)Height.Min(),(int)Width.Max(),(int)Height.Max());
			m_BlockRect.NormalizeRect();
			
			if(m_bPass1)
				pL->Draw(m_pDC,m_pCWnd);
		}
		break;
	}
	
	m_bSignificantChange=FALSE;
}

SRGraphLabelBlock::SRGraphLabelBlock()
{
	m_bUseMyOwnTickList=FALSE;
	m_bLogTickPositions=FALSE;
	m_pTickPositionList=NULL;
	m_bBlockMember=FALSE;
	m_bUseLocalFont=TRUE;
	m_nType=IDS_SRG_LABELBLOCKTYPE;
	m_bCheckOverlaps=TRUE;
	m_bPass1=TRUE;
}

//@mfunc Sets the label vectors of all labels owned by this block to the same value.
//@parm The cosine of the desired vector
//@parm The sine of the desired vector
void SRGraphLabelBlock::SetBlockVectors(double dVectorX,double dVectorY)
{
	for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;)
		((SRGraphLabel *)m_LabelList.GetNext(p))->SetVector(dVectorX,dVectorY);
}


//@mfunc Sets the label orientations of all labels owned by this block to the same value.
//@parm The angle (in degrees) of the orientation desired
void SRGraphLabelBlock::SetBlockOrientations(double dAngle)
{
	for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;)
	{
		SRGraphLabel * pL=(SRGraphLabel *)m_LabelList.GetNext(p);
		
		while(dAngle>=360)
			dAngle-=360;
		
		while(dAngle<0)
			dAngle+=360;

		if(dAngle==0 || dAngle==90 || dAngle==180 || dAngle==270)
			pL->SetLabelStandard((int)(dAngle/90));
		else
		{
			pL->SetOrientation(dAngle);
			pL->SetLabelStandard(SRGraphLabel::Angled);
		}

	}

	SetOrientation(dAngle);
			
}

//@mfunc Sets the location points of all labels in this block to the same value
//@parm The location point. May be one of....<nl>
//	TopLeft<nl>
//	TopCenter<nl>
//	TopRight<nl>
//	MidLeft<nl>
//	MidCenter<nl>
//	MidRight<nl>
//	BottomLeft<nl>
//	BottomCenter<nl>
//	BottomRight<nl>
void SRGraphLabelBlock::SetBlockLocationPoints(int nLocationPoint)
{
	for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;)
		((SRGraphLabel *)m_LabelList.GetNext(p))->SetLocationPoint(nLocationPoint);
}


//@mfunc Calculates the true (device independant) position of the label.
//@rem Labels may be placed at pixel, inch or millimeter resolutions.
void SRGraphLabel::CalcPos()
{	

	int nXres,nYres;

	if(!IsDrawing())
		return;
	nXres=GetpDC()->GetDeviceCaps(LOGPIXELSX),
	nYres=GetpDC()->GetDeviceCaps(LOGPIXELSY);

	switch(GetMeasurement())
	{
	case 0:	// pixels
		m_TextPosition=CPoint((int)m_dPosX,(int)m_dPosY);
		break;
	case 1: // millimeters
		m_TextPosition=CPoint((int)((m_dPosX*GetpDC()->GetDeviceCaps(LOGPIXELSX))/25.4),
			                  (int)((m_dPosY*GetpDC()->GetDeviceCaps(LOGPIXELSY))/25.4));
		break;
	case 2: // inches
		m_TextPosition=CPoint((int)(m_dPosX*GetpDC()->GetDeviceCaps(LOGPIXELSX)),
			                  (int)(m_dPosY*GetpDC()->GetDeviceCaps(LOGPIXELSY)));
		break;
	case 3: // Percent
		{
			CRect r;
			m_pCWnd->GetClientRect(&r);
			m_TextPosition=CPoint((int)((double)r.Width()/100.0*m_dPosX),(int)(r.Height()/100.0*m_dPosY));
		}
		break;
	}
}


//@mfunc Returns either the m_pDC of the label or if the label is a member of a label block the m_pDC of the parent.
//@rdesc the contents of the relavent m_pDC
CDC * SRGraphLabel::GetpDC()
{
	if(m_bBlockMember)
		return m_pLabelBlock->GetpDC();
	else
		return m_pDC;
}

//@mfunc Returns either the m_pCWnd of the label or if the label is a member of a label block the m_pCWnd of the parent.
//@rdesc the contents of the relavent m_pCWnd
CWnd * SRGraphLabel::GetpCWnd()
{
	if(m_bBlockMember)
		return m_pLabelBlock->GetpCWnd();
	else
		return m_pCWnd;	
}

//@mfunc Ascertains if this object is drawing.
//@rdesc TRUE if drawing (ie if there is a valid m_pDC and m_pCWnd to work with)
//@rem If this label is a block member then the m_bDrawing member of the parent block will be returned.
BOOL SRGraphLabel::IsDrawing()
{
	if(m_bBlockMember)
		return m_pLabelBlock->IsDrawing();
	else
		return m_bDrawing;
}


//@mfunc Performs serialization of this object
//@parm A reference to a CArchive object used to load or store the data
void SRGraphLabel::Serialize(CArchive &ar)
{
	SRGraphTitle::Serialize(ar);
	m_Labels.Serialize(ar);

	if(ar.IsLoading())
	{
		ar >> m_bBlockMember >> m_bLabelStandard >> m_bMoveable >> m_bUseLocalFont >>
		   m_bVectorStandard >> m_dPosX >> m_dPosY >> m_dVectorX >>
		   m_dVectorY >>  m_nLocationPoint >> m_TextPosition;
	}
	else
	{
		ar << m_bBlockMember << m_bLabelStandard << m_bMoveable << m_bUseLocalFont <<
		   m_bVectorStandard << m_dPosX << m_dPosY << m_dVectorX <<
		   m_dVectorY <<  m_nLocationPoint << m_TextPosition;
	}
}

//@mfunc Serializes the label block and all of the dependant labels.
//@parm A reference to a CArchive object used to load or store the data
void SRGraphLabelBlock::Serialize(CArchive & ar)
{
	SRGraphLabel::Serialize(ar);
	m_LabelList.Serialize(ar);

	if(ar.IsLoading())
	{
		ar >> m_nBlockTitle >> m_bCheckOverlaps >> m_bLogTickPositions;
	}
	else
	{
		ar << m_nBlockTitle << m_bCheckOverlaps << m_bLogTickPositions;

		for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;)
			((SRGraphLabel *)m_LabelList.GetNext(p))->SetLabelBlock(this);
	}
}

//@mfunc Adds a label to the label block list.
//@parm A pointer to the ready-constructed label.
//@parm if TRUE then the label will use its own font information rather than that of the new parent block.
void SRGraphLabelBlock::AddLabel(SRGraphLabel *label, BOOL bUseLocalFont)
{
	label->SetBlockMember(TRUE);
	label->SetUseLocalFont(bUseLocalFont);
	label->SetLabelBlock(this);
	label->SetParent(m_pParentGraph);
	m_LabelList.AddTail(label);
}

//@mfunc Sets the m_pDC, m_pCWnd and m_bDrawing members
//@parm A pointer to a valid CDC for the drawing processes
//@parm A pointer to a CWnd object in which drawing will take place
//@devnote When graph component objects are in the m_ComponentList of a graph this routine is not needed because all relavent information is given at draw time. However, if the component is created ad-hoc by 
//either the standard or your own software there is a need to tell the component which CDC and CWnd objects are being used. 
// This is because certain operations, like the use of fonts or the measurement of logical pixel sizes, depend upon a valid CDC object being available.
// This routine is particularly used by some of the updated display routines whic create and destroy new component objects themselves.<nl>
//For example a label drawing routine is part of an SRGraphDisplay object which is stored in the m_ComponentList. This routine may create a label block 
//on the stack, fill the block with labels and then test the labels for sizes before adjusting the m_)DisplayRect to its final size.
//In order to test the labels without actually drawing them the label block requires the m_pDC and m_pCWnd from the display object so that 
//things like GetTextExtent work properly.
void SRGraphComponent::DrawingUsing(CDC *pDC,CWnd *pCWnd)
{
	ASSERT(pDC!=NULL);
	ASSERT(pDC->IsKindOf(RUNTIME_CLASS(CDC)));
	ASSERT(pCWnd!=NULL);
	ASSERT(pCWnd->IsKindOf(RUNTIME_CLASS(CWnd)));

	m_bDrawing=TRUE;
	m_pDC=pDC;
	m_pCWnd=pCWnd;
}

//@mfunc Invalidates the m_bDrawing semaphore.
void SRGraphComponent::NotDrawing()
{
	m_bDrawing=FALSE;
}

//@mfunc Calculates the length of a line between two points using pythag
//@parm The start point of the line
//@parm The end point of the line
//@rdesc The line length (in logical units) between two points
double SRGraphComponent::GetLineLength(CPoint ptStart,CPoint ptEnd)
{
	return fabs(hypot((double)ptEnd.x-ptStart.x,(double)ptEnd.y-ptStart.y));
}

//@mfunc Returns the angle of a vector between two points
//@parm the starting point
//@parm the end point
//@rdesc The angle (in radians) of the end point with respect to the start point
double SRGraphComponent::GetLineAngle(CPoint ptStart,CPoint ptEnd)
{
	double dH=(double)ptEnd.y-ptStart.y,dW=(double)ptEnd.x-ptStart.x;
	double dHY=hypot(dH,dW);
	double dAngle=acos(dH/dHY);

	// If the angle is greater than 180 degrees some calculations will become dodgy

	if(ptEnd.x<=ptStart.x)
		dAngle=TWO_PI-dAngle; // so a simple fudge factor will suffice

	return dAngle;
}


//@mfunc Returns the width and height of a given polygon
//@parm Pointer to the array of polygon points
//@parm The number of points to take into account
//@rdesc a CSize object with the integer Width and Height of the polygon provided.
CSize SRGraphComponent::GetPolgonSize(CPoint * points,int nPointCount)
{
	CScale Width,Height;
	
	for(int c=0;c<nPointCount;c++)
	{
		Width+=points[c].x;
		Height+=points[c].y;
	}
	
	return CSize((int)(Width.Max()-Width.Min()),(int)(Height.Max()-Height.Min()));
}



//@mfunc Gets the color of the label text
//@rdesc a RGB value
//@rem Normally returns the color stored in the local SRGraphStyle. However if this label is a member of a label block then the color from the parent block's m_Style member is returned.
COLORREF SRGraphLabel::GetTextColor()
{
	if(m_bBlockMember)
	{
		ASSERT(m_pLabelBlock!=NULL);
		ASSERT(m_pLabelBlock->IsKindOf(RUNTIME_CLASS(SRGraphLabelBlock)));
		return m_pLabelBlock->GetTextColor();
	}
	else
		return m_RGBTextColor;
}



//@mfunc Finds a label in a block by its zero based index
//@parm The zero based index of the desired label
//@rdesc A pointer to a valid label or NULL if no label was found at that index
SRGraphLabel * SRGraphLabelBlock::GetLabelIndex(int nIndex)
{
	ASSERT(!m_LabelList.IsEmpty());

	if(m_LabelList.IsEmpty())
		return NULL;

	if(nIndex>-1)
	{
		ASSERT(nIndex<m_LabelList.GetCount());
		return (SRGraphLabel *)m_LabelList.GetAt(m_LabelList.FindIndex(nIndex));
	}
	else
	{
		switch(nIndex)
		{
		case -1:
			// return the first label
			return (SRGraphLabel *)m_LabelList.GetHead();
		case -2:
			// return the last label
			return (SRGraphLabel *)m_LabelList.GetTail();
		default:
			return NULL;
		}
	}

}

//@mfunc Finds the width  and height of a label
//@rdesc A CSize object containing the computed width and height of a labels bounding box
//@rem This routine takes into account the fact that a label may indeed be in a rotated state.
CSize SRGraphLabel::GetLabelSize()
{
	ASSERT(IsDrawing());

	CPoint points[5];
	CScale w,h;

	GetBoundingPolygon(&points[0],5);

	for(int c=0;c<5;c++)
	{
		w+=points[c].x;
		h+=points[c].y;
	}

	return CSize((int)w.Range(),(int)h.Range());

}

//@mfunc Destroys the display and empties any of the tick lists or label lists
//which may have been used.
SRGraphDisplay::~SRGraphDisplay()
{
	while(!m_XTickList.IsEmpty())
		delete ((CPoint *)m_XTickList.RemoveHead());
	while(!m_YTickList.IsEmpty())
		delete ((CPoint *)m_YTickList.RemoveHead());
	while(!m_ZTickList.IsEmpty())
		delete ((CPoint *)m_ZTickList.RemoveHead());
	if(m_ptrXLabels!=NULL)
		delete (m_ptrXLabels);
	if(m_ptrYLabels!=NULL)
		delete (m_ptrYLabels);
	if(m_ptrZLabels!=NULL)
		delete (m_ptrZLabels);
}


//@mfunc Generates or updates a tick list from the positions of all the labels in the block
void SRGraphLabelBlock::UpdateTickList()
{
	if(m_bLogTickPositions)
	{
		if(m_pTickPositionList==NULL)
		{
			m_bUseMyOwnTickList=TRUE;
			m_pTickPositionList = new CPtrList;
		}

		while(!m_pTickPositionList->IsEmpty())
			delete ((CPoint *)m_pTickPositionList->RemoveHead());

		for(POSITION p=m_LabelList.GetHeadPosition();p!=NULL;)
		{
			CPoint *cpp=new CPoint;
			*cpp=((SRGraphLabel *)m_LabelList.GetNext(p))->GetTextPosition();
			m_pTickPositionList->AddTail(cpp);
		}
	}
}

//@mfunc Adds or updates a label at a specified zero based index.
//@parm The text to place in the label
//@parm The zero based index of the label to add or update
//@parm the country code of the label
//@rem To imrove speed the display compnents use a semi-permanent system of labels for x, y, and z axis labels.
// By re-claiming and re-using label blocks rather than creating and destroying them at every draw cycle a significant increase in 
// re-draw speed is attained.
void SRGraphLabelBlock::AddLabelIndex(LPCSTR LabelText,int nIndex,int nCode)
{
	if(m_LabelList.GetCount()>nIndex)
		((SRGraphLabel *)m_LabelList.GetAt(m_LabelList.FindIndex(nIndex)))->SetAnnotation(nCode,LabelText);
	else
	{
		while(m_LabelList.GetCount()<nIndex)
			AddLabel(new SRGraphLabel);
		AddLabel(LabelText);
	}
}

// @mfunc Draws polar or web-chart axes.
void SRGraphDisplay::DrawWebAxis()
{
	
	int nCenterX=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		nCenterY=m_DisplayRect.top+(m_DisplayRect.Height()/2);

	double dRadius=0.8*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
	if(dRadius<1)
		return;

	if(m_nDivCount<3)
	{
	//	SRThrowGraphException(SRGraphException::graphtype); // Must have at least three divisions to make a proper web chart
	//	GetStyle()->SetGraphStyle(CX_GRAPH_LINE);
	//	GetStyle()->SetAxisStyle(CX_AXIS_CLASSIC);
	//	GetStyle()->SetLabelStyle(CX_LABEL_AUTOMATIC);
	//	m_pCWnd->Invalidate();
		return;
	}

	double dAngle=0,dAngleStep=TWO_PI/(m_nDivCount);

	CPen pen,*oldpen;
	CBrush brush,*oldbrush;


	oldpen=(CPen *)GetpDC()->SelectObject(&pen);
	CPoint *points=new CPoint[m_nDivCount+1];

	// First we make a list of points in an array. This way minor computational errors wont make the edges of the web all jaggy
	
	for(int x=0;x<m_nDivCount;x++)
	{
		points[x]=CPoint((int)(nCenterX+(dRadius*cos(dAngle-RADIANS(90)))),(int)(nCenterY+(dRadius*sin(dAngle-RADIANS(90)))));
		dAngle+=dAngleStep;
	}

	points[m_nDivCount]=points[0];

	if(GetStyle()->GetAxisFillStyle())
	{	//The axis sections are filled according to style.
		brush.CreateSolidBrush(GetStyle()->GetColor());
		oldbrush=(CBrush *)GetpDC()->SelectObject(&brush);
		oldpen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);	// frame lines can wait till later
		for(x=0;x<m_nDivCount;x++)
		{
			CPoint polypoints[4];
			polypoints[0]=polypoints[3]=CPoint(nCenterX,nCenterY); // center point and closing polygon point
			polypoints[1]=points[x];							   // first outside point
			polypoints[2]=points[x+1];							   // second outside point
			GetpDC()->Polygon(&polypoints[0],4);					   // Drawn with the selected pen and brush (PolyFillMode is ALTERNATE by default)
		}

		GetpDC()->SelectObject(oldpen);
		GetpDC()->SelectObject(oldbrush);
		brush.DeleteObject();
	}

	// Now we draw the framing polygons which go to make up the web

	if(GetStyle()->GetShowYGrid() && !m_YTickList.IsEmpty())
	{
		pen.CreatePen(PS_SOLID,GetStyle()->GetFrameWidth(),GetStyle()->GetGridColor());
		oldpen=(CPen *)GetpDC()->SelectObject(&pen);
		GetpDC()->SelectStockObject(NULL_BRUSH);
		for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
		{
			dAngle=0;
			CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
			double dTRadius=(double)pP->y-nCenterY;
			/*for(x=0;x<m_nDivCount;x++)
			{
				GetpDC()->MoveTo((int)(nCenterX+(dTRadius*cos(dAngle-RADIANS(90)))),
					             (int)(nCenterY+(dTRadius*sin(dAngle-RADIANS(90)))));
				GetpDC()->LineTo((int)(nCenterX+(dTRadius*cos((dAngle+dAngleStep)-RADIANS(90)))),
					             (int)(nCenterY+(dTRadius*sin((dAngle+dAngleStep)-RADIANS(90)))));
				dAngle+=dAngleStep;
			} */
			GetpDC()->Ellipse((int)(nCenterX-dTRadius),(int)(nCenterY-dTRadius),(int)(nCenterX+dTRadius),(int)(nCenterY+dTRadius));
		}
		GetpDC()->SelectObject(oldpen);
		pen.DeleteObject();
	}

	if(GetStyle()->GetShowXGrid())
	{
		pen.CreatePen(PS_SOLID,GetStyle()->GetFrameWidth(),GetStyle()->GetGridColor());
		oldpen=(CPen *)GetpDC()->SelectObject(&pen);

		for(x=0;x<m_nDivCount;x++)
		{
			GetpDC()->MoveTo(nCenterX,nCenterY);
			GetpDC()->LineTo(points[x].x,points[x].y);
		}
		GetpDC()->SelectObject(oldpen);
		pen.DeleteObject();
	}



	delete (points);
}


//@mfunc Draws polar or web-char labels
void SRGraphDisplay::DrawWebLabels()
{
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0)
		return;

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	int nCenterX=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		nCenterY=m_DisplayRect.top+(m_DisplayRect.Height()/2);

	double dRadius=0.85*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
	if(dRadius<1)
		return;

	if(m_ptrXLabels==NULL)
		m_ptrXLabels=new SRGraphLabelBlock;

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())
			m_ptrYLabels=new SRGLogScale;
		else
			m_ptrYLabels=new SRGDecimalScale;
	}

	double dAngle=0,dAngleStep=TWO_PI/(m_nDivCount);

	CPoint *points=new CPoint[m_nDivCount];

	for(int x=0;x<m_nDivCount;x++)
	{
		points[x]=CPoint((int)(nCenterX+(dRadius*cos(dAngle-RADIANS(90)))),(int)(nCenterY+(dRadius*sin(dAngle-RADIANS(90)))));
		dAngle+=dAngleStep;
	}

	SRGDecimalScale *yLabels=(SRGDecimalScale *)m_ptrYLabels;  // This looks odd but its done so that you dont need to cast the pointer every time you use it.
	SRGraphLabelBlock *xLabels=m_ptrXLabels;

	xLabels->SetParent(GetParent());
	yLabels->SetParent(GetParent());

	xLabels->DrawingUsing(m_pDC,m_pCWnd);
	yLabels->DrawingUsing(m_pDC,m_pCWnd);
	
	CString strFormatString,strTestString;

	strFormatString=" %g ";

	CPoint ptStart,ptEnd;

	xLabels->SetLogTickPositions(FALSE);

	xLabels->SetFontSize(GetFontSize());
	xLabels->SetFaceName(GetFaceName());
	xLabels->SetTextColor(GetTextColor());
	xLabels->SetFontStyle(GetFontStyle());

	if(m_Style.GetShowIndices())
	{
		dAngle=0;

		for(int x=0;x<m_nDivCount;x++)
		{
			if(m_bSignificantChange)
				xLabels->AddLabelIndex(GetParent()->GetGroup(0)->GetIndex(m_nMinDiv+x)->GetAnnotation(GetCountryCode()),x);
			SRGraphLabel *pL=xLabels->GetLabelIndex(x);
			double dLabelAngle=dAngle-(PI/2);
			if(dLabelAngle<0)
				dLabelAngle+=TWO_PI;
			pL->SetLocationPoint((int)(DEGREES(dLabelAngle)/45));
			pL->SetTextPosition(points[x]);
			pL->SetVector(DEGREES(dLabelAngle));
			dAngle+=dAngleStep;
		}
		while(xLabels->GetLabelCount()>m_nDivCount)
			xLabels->RemoveLastLabel();
		xLabels->Draw(m_pDC,m_pCWnd);
	}

	if(GetStyle()->GetShowNumerals())
	{
		dRadius=0.8*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
		yLabels->SetFontSize(GetFontSize());
		yLabels->SetFaceName(GetFaceName());
		yLabels->SetTextColor(GetTextColor());
		yLabels->SetFontStyle(GetFontStyle());
		yLabels->SetptrTickPositionList(&m_YTickList);
		yLabels->SetLogTickPositions(TRUE);
		yLabels->SetStartPos(CPoint(nCenterX,nCenterY));
		yLabels->SetEndPos(CPoint(nCenterX,nCenterY-(int)(dRadius+0.5)));
		yLabels->SetMaxValue(m_ScaleY.Max());
		yLabels->SetMinValue(m_ScaleY.Min());
		yLabels->SetAscending();
		yLabels->SetFormatString(strFormatString);
		yLabels->DrawingUsing(m_pDC,m_pCWnd);
		yLabels->CreateLabels();
		yLabels->Draw(m_pDC,m_pCWnd);
	}

	delete (points);

}

//@mfunc Draws polar or web-chart data
void SRGraphDisplay::DrawWebData()
{
	if(m_ScaleY.Max()==m_ScaleY.Min() && m_ScaleY.Min()==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}
	int nCenterX=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		nCenterY=m_DisplayRect.top+(m_DisplayRect.Height()/2);

	double dRadius=0.8*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
	if(dRadius<1)
		return;

	if(m_nDivCount<3)
	{
		SRThrowGraphException(SRGraphException::graphtype); // Must have at least three divisions to make a proper web chart
		GetStyle()->SetGraphStyle(CX_GRAPH_LINE);
		GetStyle()->SetAxisStyle(CX_AXIS_CLASSIC);
		GetStyle()->SetLabelStyle(CX_LABEL_AUTOMATIC);
		m_pCWnd->Invalidate();
		return;
	}

	double dAngle,dAngleStep=TWO_PI/(m_nDivCount);

	CPoint *points=new CPoint[m_nDivCount+1];

	double ScalingFactor=dRadius/m_dMaxScaleY;

	BOOL bLogData=GetStyle()->GetLog();

	for(int CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
	{

		dAngle=0;

		m_nCurrentIndex=(CurrentSet+m_nMinSet);

		for(int x=0;x<=m_nDivCount;x++)
		{
			double dValue=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex((x<m_nDivCount ? x : 0)+m_nMinDiv)->GetValue(bLogData);
			if(bLogData)
				dValue-=log10(m_ScaleY.Min());
			else
				dValue-=m_ScaleY.Min();
			points[x]=CPoint((int)(nCenterX+((ScalingFactor*dValue)*cos(dAngle-RADIANS(90)))),(int)(nCenterY+((ScalingFactor*dValue)*sin(dAngle-RADIANS(90)))));
			dAngle+=dAngleStep;
		}

		points[m_nDivCount]=points[0];

		switch(GetStyle()->GetGraphStyle())
		{
		default:
		case CX_GRAPH_WEB:
			{
				for(x=0;x<=m_nDivCount;x++)
				{
					SRGraphData *pgd=GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetIndex((x!=m_nDivCount ? x : 0)+m_nMinDiv);
					GetParent()->LogPoint(points[x],CurrentSet+m_nMinSet,(x!=m_nDivCount ? x : 0)+m_nMinDiv);
					//determine the point type
					switch(pgd->GetStyle()->GetObjectStyle())
					{
					case CX_OBJECT_NONE:
						break;
					case CX_OBJECT_LINE:
drawLine:		
						if(x!=0)
							DrawStyledLine(points[x-1],points[x],pgd->GetStyle(),m_nCurrentIndex);
						break;
					default:
					case CX_OBJECT_AUTOMATIC:
					case CX_OBJECT_EXPOINT:
						if(x!=m_nDivCount)
							DrawWiget(points[x],pgd->GetStyle());
						goto drawLine;
						break;
					case CX_OBJECT_POINT:
						if(x!=m_nDivCount)
 							DrawWiget(points[x],pgd->GetStyle());
						break;
					}
				}
			}
			break;
		case CX_GRAPH_AREAWEB:
			FillPoly(&points[0],m_nDivCount+1,GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetStyle(),0);
			FramePoly(&points[0],m_nDivCount+1,GetParent()->GetGroup(CurrentSet+m_nMinSet)->GetStyle(),0);
			break;
		}
	}

	delete (points);

}

//@mfunc Draws a line between two points using a frame style from a given SRGraphStyle object
//@parm The start point of the line
//@parm the end point of the line
//@parm The pointer to the SRGraphStyle object from which frame information will be drawn
//@parm The key or index which will be passed to CreateAutoPen in case the style is CX_FRAME_AUTOMATIC and an auto-styled line is required
//@rem This is a function intended to make addition of custom graph types easier.
void SRGraphComponent::DrawStyledLine(CPoint ptStart,CPoint ptEnd,SRGraphStyle *ptrStyle,int nKey)
{
	if(!IsDrawing())
	{
		ASSERT(FALSE);
		return;
	}

	SRGraphStyle DefaultStyle;

	if(ptrStyle==NULL)
		ptrStyle=&DefaultStyle;
	
	
	CPen datapen, *OldPen;
	int linecount=1;
	BOOL bExLine=FALSE;

	if(ptrStyle->GetObjectStyle()==CX_OBJECT_EXLINE)
	{
		linecount=(int)(GetStyle()->GetUseMaxSize() ? GetStyle()->GetMaxObjectDepth() : 10);
		bExLine=TRUE;
	}

	CreateAutoPen(nKey,ptrStyle->GetFrameWidth());
	datapen.CreatePen(PS_SOLID,ptrStyle->GetFrameWidth(),ptrStyle->GetFrameColor());


	for(int c=0;c<linecount;c++)
	{
		m_pDC->MoveTo(ptStart);
		switch(ptrStyle->GetFrameStyle())
		{
		case CX_FRAME_NONE:
			break;
		default:
		case CX_FRAME_AUTOMATIC:
			OldPen=(CPen *)GetpDC()->SelectObject(&m_AutoPen);
			GetpDC()->LineTo(ptEnd);
			GetpDC()->SelectObject(OldPen);
			break;
		case CX_FRAME_COLOR:
			OldPen=(CPen *)GetpDC()->SelectObject(&datapen);
			GetpDC()->LineTo(ptEnd);
			GetpDC()->SelectObject(OldPen);
			break;
		}
		
		if(bExLine)
		{
			CPoint offset=CPoint(1,-1);
			ptStart+=offset;
			ptEnd+=offset;
		}
	}

	datapen.DeleteObject();
}


//@mfunc Retrieves the pointer to the parent graph
//@rdesc A pointer to the graph which owns this component.
//@rem Note that if this label is the member of a label block then the pointer returned is that of the parent <c SRGraphLabelBlock>.
SRGraph * SRGraphLabel::GetParent()
{
	if(m_bBlockMember || m_pParentGraph==NULL)
		return GetLabelBlock()->GetParent();
	else
		return m_pParentGraph;
}

//@mfunc Cleans up the label block by removing and deleting all labels.
void SRGraphLabelBlock::KillLabels()
{
	while(!m_LabelList.IsEmpty())
		delete ((SRGraphLabel *)m_LabelList.RemoveHead());
}

//@mfunc Initialises the m_DisplayRect member. Optionally locks the display.
//@parm The desired rectangle.
//@parm if TRUE then the display is locked.
//@rem Under normal operating conditions the SRGraphDisplay system will automatically resize the display area to accommodate 
//changes in the font sizes, string lengths and number of labels in the chart. It is however possible to set and lock a display rectangle 
//so that the data display area will never change size automatically. This feature is particularly useful when making multi axis 
//charts and allows you to select the size and position of all data displays without them becoming misaligned due to the automatic 
//adjustment process.
void SRGraphDisplay::SetDisplayRect(CRect r,BOOL lock)
{
	m_DisplayRect=r;
	GetStyle()->DRLock(lock);
}

//@mfunc Draws a point or point-line combination
//@parm The position of the start point. 
//@parm the position of the end point
//@parm A pointer to an SRGraphStyle object
//@parm The mode
//@rem Two points on a graph may be connected by a line and / or covered by wigets. Normally the style is dictated by the object style but 
//in the case of the style being CX_OBJECT_AUTOMATIC or CX_OBJECT_EXPOINT you can choose to place a wiget at either the first, last or both points. 
//To do this the mode parameter must be set to...<nl>
//0=Wiget at first point<nl>
//1=Wiget at last point<nl>
//2=Wiget at both points<nl><nl>
//If you neet a line without wigets set your object style to CX_OBJECT_LINE. If you need one wiget only then set the object style to CX_OBJECT_POINT. The wiget will be displayed over the first point.
void SRGraphComponent::DrawPointObject(CPoint firstpoint,CPoint lastpoint,SRGraphStyle *ptrStyle,int nObjectMode)
{
			switch(ptrStyle->GetObjectStyle())
			{
			case CX_OBJECT_NONE:
				break;
			case CX_OBJECT_LINE:
				DrawStyledLine(firstpoint,lastpoint,ptrStyle,m_nCurrentIndex);
				break;
			default:
			case CX_OBJECT_AUTOMATIC:
			case CX_OBJECT_EXPOINT:
				DrawStyledLine(firstpoint,lastpoint,ptrStyle,m_nCurrentIndex);
				switch(nObjectMode)
				{
				case 2: // Wiget at both points
					DrawWiget(firstpoint,ptrStyle);
				case 1: // wiget at lastpoint
					DrawWiget(lastpoint,ptrStyle);
					break;
				default:
				case 0:	// wiget at firstpoint
					DrawWiget(firstpoint,ptrStyle);
					break;
				}
				break;
			case CX_OBJECT_POINT:
 				DrawWiget(firstpoint,ptrStyle);
				break;
			}
}


//@mfunc Deletes the last label from the list.
//@rem Used primarily to tidy label blocks. A Label block is kept between draw cycles to minimise the class allocation time overheads. 
//These blocks are updated and accessed using the <mf SRGraphLabelBlock::AddLabelIndex> function. If the label block size has increased then 
//labels will be automatically added. If the block size has decreased then the unwanted labels can  be flushed from the end of the list using this function.
//@ex Tail end removal can be accomplished like so.. |
//
//CString strLabelText;
//SRGraphLabelBlock *Labels=GetMyBlockPointer();
//
//for (int x=0;x<5;x++)
//{	// Add a few labels
//	strLabelText.Format("%d",x);
//	AddLabelIndex(strLabelText,x);
//}
//  //
//while(Labels.GetLabelCount()>5);
//	Labels->RemoveLastLabel();
void SRGraphLabelBlock::RemoveLastLabel()
{
	SRGraphLabel *pL=(SRGraphLabel *)m_LabelList.RemoveTail();
	delete (pL);
}

//@mfunc Returns the number of labels currently in the list.
//@rdesc The label count
int SRGraphLabelBlock::GetLabelCount()
{
  return m_LabelList.GetCount();
}

//@mfunc Used to read the standard parameters for all components from the template line.
//@parm A pointer to a zero terminated string containing the template line.
//@rdesc a count of all characters processed by this routine.
int SRGraphComponent::ReadBasicParameters(LPCSTR line)
{
	int n,i=0;
	int returns;
	float left,top,right,bottom;
	int measurement,borderstyle,fillstyle,shadowstyle;
	int r1,g1,b1,r2,g2,b2,angle,extra;
	char name[32];

	returns=sscanf(line,"%g %g %g %g %d %d %d %d %n",
				&left,
				&top,
				&right,
				&bottom,
				&measurement,
				&borderstyle,
				&fillstyle,
				&shadowstyle,
				&n); // fetch command and standard parameters
	if(returns!=8)
		SRThrowGraphException(SRGraphException::component);

	i+=n;

	switch(fillstyle)
	{
	case 0:
		r1=g1=b1=r2=g2=b2=0;
		i-=n;
		break;
	case 1:
		returns=sscanf(&line[i],"%d %d %d %n",&r1,&g1,&b1,&n);
		if(returns!=3)
			SRThrowGraphException(SRGraphException::fillstyle);
		break;
	case 2:
	case 3:
		returns=sscanf(&line[i],"%d %d %d %d %d %d %n",&r1,&g1,&b1,&r2,&g2,&b2,&n);
		if(returns!=6)
			SRThrowGraphException(SRGraphException::fillstyle);
		break;
	case 4:
		returns=sscanf(&line[i],"%d %d %d %d %d %d %d %n",&r1,&g1,&b1,&r2,&g2,&b2,&angle,&n);
		if(returns!=7)
			SRThrowGraphException(SRGraphException::fillstyle);
		break;
	case 5:
		returns=sscanf(&line[i]," \"%[0-9a-zA-Z-_. ]\" %d %n",&name,&extra,&n);
		if(returns!=2)
			SRThrowGraphException(SRGraphException::fillstyle);
		break;
	case 6:
	default:
			SRThrowGraphException(SRGraphException::fillstyle);
		break;
	}

	i+=n;	// and add new position to to accumulator

	SetRect(left,top,right,bottom);
	SetMeasurement(measurement);
	SetStyle(borderstyle | (shadowstyle<<4) | (fillstyle<<8));
	SetColor(RGB(r1,g1,b1),RGB(r2,g2,b2));
	SetAngle(angle);
	SetResourceName(name);
	GetStyle()->SetStretchMode(extra ? FALSE : TRUE);

	return i;

}

//@mfunc Reads the parameters from template line
//@parm A pointer to the zero terminated template line text.
//@rdesc The number of characters processed by this function.
int SRGraphTitle::ReadTemplateLine(LPCSTR line)
{
	char typeface[64];
	int rt,gt,bt,typestyle,typesize,justification,returns,n;

	returns=sscanf(line," %d %d %d \"%[0-9a-zA-Z-_. ]\" %d %d %d%n",&rt,&gt,&bt,typeface,&typesize,&typestyle,&justification,&n);
	if(returns!=7)
		SRThrowGraphException(SRGraphException::font);
	else
	{
		SetText("",justification);
		SetTextColor(RGB(rt,gt,bt));
		SetFont(typesize,typestyle,typeface);
	}
	return n;
}

//@mfunc Reads the parameters from template line
//@parm A pointer to the zero terminated template line text.
//@rdesc The number of characters processed by this function.
int SRGraphComponent::ReadTemplateLine(LPCSTR line)
{
	line; //UNUSED
	ASSERT(FALSE);	// Bare components dont normally have any more than basic parameters
					//You should however override this function in all of your component derivatives
	return 0;
}

//@mfunc Reads the parameters from template line
//@parm A pointer to the zero terminated template line text.
//@rdesc The number of characters processed by this function.
int SRGraphLegend::ReadTemplateLine(LPCSTR line)
{
	int n,i=SRGraphTitle::ReadTemplateLine(line); // pick up all the font details...
	int autosizecode,style,isline,keystyle;
	float keysize;

	int returns=sscanf(&line[i]," %d %g %d %d%n", // scan in
						&autosizecode,	// how to re-size if at all
						&keysize,	// width of the key data
						&style,
						&isline,
						&keystyle,
						&n);
	if(returns<4)
		SRThrowGraphException(SRGraphException::legend);

	SetAutoSize(autosizecode);
	SetKeySize(keysize);
	GetStyle()->SetLegendStyle(style<<CX_LEGEND_SHIFT);
	GetStyle()->SetLegendLine(isline);
	if(returns==4)
		GetStyle()->SetLegendKeyStyle(CX_KEY_AUTOMATIC);
	else
		GetStyle()->SetLegendKeyStyle(keystyle<<CX_KEY_SHIFT);

	i+=n;
	return i;
}

//@mfunc Reads the parameters from template line
//@parm A pointer to the zero terminated template line text.
//@rdesc The number of characters processed by this function.
int SRGraphDisplay::ReadTemplateLine(LPCSTR line)
{
	int n,i=SRGraphTitle::ReadTemplateLine(line); // pick up all the font details...
	int axis,graph,log,label;
	int returns=sscanf(&line[i]," %d %d %d %d %n",&axis,&graph,&label,&log,&n);
	if(returns!=4)
		SRThrowGraphException(SRGraphException::data);
	GetStyle()->SetAxisStyle(((DWORD)axis)<<CX_AXIS_SHIFT);
	GetStyle()->SetGraphStyle(((DWORD)graph)<<CX_GRAPH_SHIFT);
	GetStyle()->SetLog(log);
	i+=n;
	return i;
}

//@mfunc Writes the parameters of this object to the template.
//@parm A pointer to the StdioFile which is managing the disk operations.
void SRGraphLegend::WriteSpecificParameters(CStdioFile *f)
{
	CString s;
	int code=0;
	code |= m_bAutoSizeX ? 1 : 0;
	code |= m_bAutoSizeY ? 2 : 0;
	code |= m_bKeepLeft ? 4 : 0;
	code |= m_bKeepTop ? 8 : 0;
	s.Format(" %d %f %d %d %d ",code,(float)m_dKeySize,GetStyle()->GetLegendStyle()>>CX_LEGEND_SHIFT,GetStyle()->GetLegendLine(),GetStyle()->GetLegendKeyStyle()>>CX_KEY_SHIFT);
	f->WriteString((LPCSTR)s);
}

//@mfunc Called to tidy the display after changing from one type or style to another.
//@rem Certain graph types will autonomously change type in case of incorrect or invalid data. 
//For example Pie charts which cannot display negative numbers re-configure to be bar charts if 
//negative data is in the group and index range of the display. This reoutine will clean up the
//label blocks and set the m_SignificantChange flag to ensure that display's start from fresh 
//after such circumstances. You must also call this routine if you change from one graph type to 
//another otherwise the display will assume that data structures, labels and flags from the old 
//setup apply to the new one.
void SRGraphDisplay::Refresh()
{
	if(m_ptrXLabels!=NULL)
		delete (m_ptrXLabels);
	if(m_ptrYLabels!=NULL)
		delete (m_ptrYLabels);
	if(m_ptrZLabels!=NULL)
		delete (m_ptrZLabels);
	m_ptrXLabels=m_ptrYLabels=m_ptrZLabels=NULL;
	if(!m_XTickList.IsEmpty())
		KillTickList(&m_XTickList);
	if(!m_YTickList.IsEmpty())
		KillTickList(&m_YTickList);
	if(!m_ZTickList.IsEmpty())
		KillTickList(&m_ZTickList);
	SetSignificantChange();
	m_bDisplayDrawn=FALSE;
}

//@mfunc Special pre draw preparation routine for strata graphs.
void SRGraphDisplay::PrepareStrata()
{
	m_ScaleY.Zero();
	
	int x,CurrentSet;

	for(x=0;x<m_nDivCount;x++)
	{
		double dData=0;
		for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet++)
			dData+=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE)->GetValue();
		m_ScaleY+=dData;
	}
}

//@mfunc NEW Draws area charts
void SRGraphDisplay::DrawAreaData()
{
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bLogData=m_Style.GetLog();
	
	CPen datapen;

	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY; 

	CPoint * AreaPolygon;

	AreaPolygon=new CPoint[m_nDivCount+3];

	GESorter *elements=new GESorter[m_nSetCount];

	for(int z=0;z<m_nSetCount;z++)
	{
		elements[z].value=GetParent()->GetGroupAverage(m_nMinDiv,m_nMaxDiv,z+m_nMinSet);
		elements[z].index=z;
	}

	if(GetStyle()->GetSort())
		qsort(elements,m_nSetCount,sizeof(GESorter),qsSorterCompareD);

	int x;
	for(z=0;z<m_nSetCount;z++)
	{
		
		m_nCurrentIndex=elements[z].index;

		SRGraphDataList *pDL=GetParent()->GetGroup(elements[z].index+m_nMinSet);

		double dValue,xStep=((double)m_DisplayRect.Width()/(m_nDivCount>1 ? (m_nDivCount-1) : 1));

		double dZeroValue=(bLogData ? (m_ScaleY.Min()>1 ? log10(m_ScaleY.Min()) : 0) : m_ScaleY.Min());

		for(x=0;x<m_nDivCount;x++)
		{
			SRGraphData *pgd=pDL->GetIndex(x+m_nMinDiv);

			if(!pgd->GetNull())
			{
				dValue=pgd->GetValue(bLogData)-dZeroValue;
				AreaPolygon[x]=CPoint((int)(xStep*x)+m_DisplayRect.left,
									  (int)((double)m_DisplayRect.bottom-(ScalingFactor*dValue)));
				GetParent()->LogPoint(AreaPolygon[x],elements[z].index+m_nMinSet,x+m_nMinDiv);
			}
		}
		
		// Tidy and close the polygon..
		AreaPolygon[x++]=CPoint(m_DisplayRect.right,(int)((double)m_DisplayRect.bottom-(ScalingFactor*dZeroValue)));
		AreaPolygon[x++]=CPoint(m_DisplayRect.left,(int)((double)m_DisplayRect.bottom-(ScalingFactor*dZeroValue)));
		AreaPolygon[x]=AreaPolygon[0];

		// Frame and Fill the area..

		SRGraphStyle *ptrStyle=GetParent()->GetGroup(elements[z].index+m_nMinSet)->GetStyle();

		if(ptrStyle->GetFrameStyle()!=CX_FRAME_NONE)
			FramePoly(&AreaPolygon[0],x,ptrStyle,0l);
		FillPoly(&AreaPolygon[0],x,ptrStyle,0l);
	}

	delete (AreaPolygon);
	delete (elements);
}

//@mfunc NEW Prepares data for polar charts
void SRGraphDisplay::PreparePolar()
{
	m_ScaleY.Zero();

	if(m_Style.GetUseMaxScale()) // max scale overrides automatic scaling
	{
		m_ScaleY+=m_dMaxRangeY;
		m_ScaleY+=m_dMinRangeY;
		return;
	}

	int x,CurrentSet;

	// polar displays use alternating phase and magnitude with the first
	// group being phase, the second being magnitude and so on.

	for(CurrentSet=1;CurrentSet<m_nSetCount;CurrentSet+=2)
	{
		for(x=0;x<m_nDivCount;x++)
			m_ScaleY+=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE)->GetIndex(x+m_nMinDiv,FALSE)->GetValue();
	}
}

//@mfunc NEW Draws candle charts and hi-lo-open-close charts.
//@rem A single series is held in four consecutive groups. Group zero is the high value,
//group one the low, group two the open and group three the close. The sequence repeats thereafter 
//with group four being the highs of the second series.
void SRGraphDisplay::DrawCandleData()
{
	if((m_nSetCount & 0x03) || (m_nSetCount < 4))
	{
		SRThrowGraphException(SRGraphException::graphtype);
		return;
	}

	BOOL bLogData=m_Style.GetLog();

	DWORD dwGraphType=GetStyle()->GetGraphStyle();
	DWORD dwAxisType=GetStyle()->GetAxisStyle();
	
	double ScalingFactor=(double)m_DisplayRect.Height()/m_dMaxScaleY;
	
	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(((double)m_DisplayRect.Width())/((m_nSetCount/4)*m_nDivCount),(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=((double)m_DisplayRect.Width()/((m_nSetCount/4)*m_nDivCount))/1.1;
	
	double dBlockWidth=(double)m_DisplayRect.Width()/m_nDivCount;
	double dSetOffset=(dBlockWidth/2)-(((m_nSetCount/4)*BarWidth)/2);

	int CurrentSet,x;

	double dHiData,dLoData,dOpenData,dCloseData;

	for(x=0;x<m_nDivCount;x++)
	{
		int top0,top1,left,bottom0,bottom1,right;

		CRect BarRect;

		for(CurrentSet=0;CurrentSet<(m_nSetCount/4);CurrentSet++)
		{
			SRGraphDataList *gdl[4];
			SRGraphData *pD[4];
			for(int i=0;i<4;i++)
			{
				gdl[i]=GetParent()->GetGroup((4*CurrentSet)+i+m_nMinSet,FALSE);
				ASSERT(gdl[i]!=NULL);
				pD[i]=gdl[i]->GetIndex(x+m_nMinDiv);
				ASSERT(pD[i]!=NULL);
			}
				
			if(!pD[0]->GetNull()) 
			{
				SRGraphStyle *ptrStyle=pD[0]->GetStyle();
				if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE) // if there is no individual style defined
					ptrStyle=gdl[0]->GetStyle();;					// pick up a global style for the group
				
				dHiData=pD[0]->GetValue(bLogData);
				dLoData=pD[1]->GetValue(bLogData);
				dOpenData=pD[2]->GetValue(bLogData);
				dCloseData=pD[3]->GetValue(bLogData);

				if(bLogData)
				{
					double dAdjust=log10(m_ScaleY.Min());
					dHiData-=dAdjust;
					dLoData-=dAdjust;
					dOpenData-=dAdjust;
					dCloseData-=dAdjust;
				}
				else
				{
					dHiData-=m_ScaleY.Min();
					dLoData-=m_ScaleY.Min();
					dOpenData-=m_ScaleY.Min();
					dCloseData-=m_ScaleY.Min();
				}

				m_nCurrentIndex=(CurrentSet+m_nMinSet);

				left=(int)((x*dBlockWidth)+dSetOffset+(CurrentSet*BarWidth)+m_DisplayRect.left);
				right=(int)(left+BarWidth);
				top0=(int)((double)m_DisplayRect.bottom-(ScalingFactor*dHiData)+0.5);
				top1=(int)((double)m_DisplayRect.bottom-(ScalingFactor*dOpenData)+0.5);
				bottom0=(int)((double)m_DisplayRect.bottom-(ScalingFactor*dLoData)+0.5);
				bottom1=(int)((double)m_DisplayRect.bottom-(ScalingFactor*dCloseData)+0.5);

				GetParent()->LogPoint(CPoint((int)(left+(BarWidth/2)),top0),(CurrentSet*4)+m_nMinSet,x+m_nMinDiv);
				GetParent()->LogPoint(CPoint((int)(left+(BarWidth/2)),top1),2+(CurrentSet*4)+m_nMinSet,x+m_nMinDiv);
				GetParent()->LogPoint(CPoint((int)(left+(BarWidth/2)),bottom0),1+(CurrentSet*4)+m_nMinSet,x+m_nMinDiv);
				GetParent()->LogPoint(CPoint((int)(left+(BarWidth/2)),bottom1),3+(CurrentSet*4)+m_nMinSet,x+m_nMinDiv);
				
				BOOL bLogging=GetParent()->GetFeedbackLogging();

				GetParent()->SetFeedbackLogging(FALSE);

				switch(dwGraphType)
				{
				default:
				case CX_GRAPH_CANDLE:
					DrawStyledLine(CPoint((int)(left+(BarWidth/2)),top0),CPoint((int)(left+(BarWidth/2)),bottom0),ptrStyle,CurrentSet+m_nMinSet);
					DrawBar(CPoint(left,top1),CPoint(right,bottom1),CurrentSet+m_nMinSet,x+m_nMinDiv,ptrStyle,0);				
					break;
				case CX_GRAPH_HILO_OPENCLOSE:
					DrawStyledLine(CPoint((int)(left+(BarWidth/2)),top0),CPoint((int)(left+(BarWidth/2)),bottom0),ptrStyle,CurrentSet+m_nMinSet);
					DrawStyledLine(CPoint(left,top1),CPoint((int)(left+(BarWidth/2)),top1),ptrStyle,CurrentSet+m_nMinSet);
					DrawStyledLine(CPoint((int)(left+(BarWidth/2)),bottom1),CPoint(right,bottom1),ptrStyle,CurrentSet+m_nMinSet);
					break;

				}

				GetParent()->SetFeedbackLogging(bLogging);
			}
		}
	}
}

void SRGraphDisplay::DrawPolarData()
{

	if(m_nSetCount & 1)
	{	// there must be an even number of sets to provide data for this chart.
		SRThrowGraphException(SRGraphException::graphtype);
	}

	int nCenterX=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		nCenterY=m_DisplayRect.top+(m_DisplayRect.Height()/2);

	double dRadius=0.8*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
	if(dRadius<1)
		return;

	double ScalingFactor=dRadius/m_dMaxScaleY;

	int x,CurrentSet;

	CPoint *points=new CPoint[m_nDivCount+1];

	BOOL bLog=GetStyle()->GetLog();

	for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet+=2)
	{
		m_nCurrentIndex=CurrentSet/2;

		SRGraphDataList *pDLPhase=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE);
		SRGraphDataList *pDLMagnitude=GetParent()->GetGroup(1+CurrentSet+m_nMinSet,FALSE);
		
		ASSERT(pDLPhase!=NULL);
		ASSERT(pDLMagnitude!=NULL);

		for(x=0;x<=m_nDivCount;x++)
		{
			double dPhase=RADIANS(pDLPhase->GetIndex(x!=m_nDivCount ? x+m_nMinDiv : m_nMinDiv)->GetValue());
			double dMagnitude=ScalingFactor*pDLMagnitude->GetIndex(x!=m_nDivCount ? x+m_nMinDiv : m_nMinDiv)->GetValue(bLog);

			points[x!=m_nDivCount ? x : 0]=CPoint((int)(nCenterX+(dMagnitude*cos(dPhase-RADIANS(90)))),(int)(nCenterY+(dMagnitude*sin(dPhase-RADIANS(90)))));
		}

		points[m_nDivCount]=points[0]; // close the polygon.

		// Now draw the polygon in the correct mode..

		if(GetStyle()->GetGraphStyle()==CX_GRAPH_AREAPOLAR)
		{
			// Fill the polygon..

			FillPoly(&points[0],m_nDivCount+1,pDLPhase->GetStyle(),0);
		}

		for(x=0;x<=m_nDivCount;x++)
		{  // Frame the polygon and place the wigets

			SRGraphStyle *ptrStyle=pDLPhase->GetIndex(x!=m_nDivCount ? x+m_nMinDiv : m_nMinDiv)->GetStyle();

			//determine the point type
			switch(ptrStyle->GetObjectStyle())
			{
			case CX_OBJECT_NONE:
				break;
			case CX_OBJECT_LINE:
drawLine:		
				if(x!=0)
					DrawStyledLine(points[x-1],points[x],ptrStyle,m_nCurrentIndex);
				break;
			default:
			case CX_OBJECT_AUTOMATIC:
			case CX_OBJECT_EXPOINT:
				if(x<m_nDivCount)
					DrawWiget(points[x],ptrStyle);
				goto drawLine;
				break;
			case CX_OBJECT_POINT:
				if(x!=m_nDivCount)
 					DrawWiget(points[x],ptrStyle);
				break;
			}
		}
	}

	delete (points);

}

void SRGraphDisplay::DrawPolarAxis()
{
	int nCenterX=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		nCenterY=m_DisplayRect.top+(m_DisplayRect.Height()/2);

	double dRadius=0.8*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
	if(dRadius<1)
		return;

	double dAngle=0,dAngleStep=TWO_PI/(16); // A grid of 16..

	CPen pen,*oldpen;
	CBrush brush,*oldbrush;



	oldpen=(CPen *)GetpDC()->SelectObject(&pen);
	CPoint *points=new CPoint[16];

	// First we make a list of points in an array. This way minor computational errors wont make the edges of the web all jaggy
	
	for(int x=0;x<16;x++)
	{
		points[x]=CPoint((int)(nCenterX+(dRadius*cos(dAngle-RADIANS(90)))),(int)(nCenterY+(dRadius*sin(dAngle-RADIANS(90)))));
		dAngle+=dAngleStep;
	}

	points[15]=points[0];

	if(GetStyle()->GetAxisFillStyle())
	{	//The axis sections are filled according to style.
		brush.CreateSolidBrush(GetStyle()->GetColor());
		oldbrush=(CBrush *)GetpDC()->SelectObject(&brush);
		oldpen=(CPen *)GetpDC()->SelectStockObject(NULL_PEN);	// frame lines can wait till later
		for(x=0;x<m_nDivCount;x++)
		{
			CPoint polypoints[4];
			polypoints[0]=polypoints[3]=CPoint(nCenterX,nCenterY); // center point and closing polygon point
			polypoints[1]=points[x];							   // first outside point
			polypoints[2]=points[x+1];							   // second outside point
			GetpDC()->Polygon(&polypoints[0],4);					   // Drawn with the selected pen and brush (PolyFillMode is ALTERNATE by default)
		}

		GetpDC()->SelectObject(oldpen);
		GetpDC()->SelectObject(oldbrush);
		brush.DeleteObject();
	}

	// Now we draw the ellipses which go to make up the concentric rings of the polar axis

	if(GetStyle()->GetShowYGrid() && !m_YTickList.IsEmpty())
	{
		pen.CreatePen(PS_SOLID,GetStyle()->GetFrameWidth(),GetStyle()->GetGridColor());
		oldpen=(CPen *)GetpDC()->SelectObject(&pen);
		GetpDC()->SelectStockObject(NULL_BRUSH);
		for(POSITION p=m_YTickList.GetHeadPosition();p!=NULL;)
		{
			dAngle=0;
			CPoint * pP=(CPoint *)m_YTickList.GetNext(p);
			double dTRadius=(double)pP->y-nCenterY;
			GetpDC()->Ellipse((int)(nCenterX-dTRadius),(int)(nCenterY-dTRadius),(int)(nCenterX+dTRadius),(int)(nCenterY+dTRadius));
		}
		GetpDC()->SelectObject(oldpen);
		pen.DeleteObject();
	}

	if(GetStyle()->GetShowXGrid())
	{
		pen.CreatePen(PS_SOLID,GetStyle()->GetFrameWidth(),GetStyle()->GetGridColor());
		oldpen=(CPen *)GetpDC()->SelectObject(&pen);

		for(x=0;x<16;x++)
		{
			GetpDC()->MoveTo(nCenterX,nCenterY);
			GetpDC()->LineTo(points[x].x,points[x].y);
		}
		GetpDC()->SelectObject(oldpen);
		pen.DeleteObject();
	}



	delete (points);
}

//@mfunc Sets the text position of the label
//@parm The device independant X position
//@parm The device independant Y position
void SRGraphLabel::SetTextPosition(double dXPos,double dYPos)
{
	m_dPosX=dXPos;
	m_dPosY=dYPos;
	CalcPos();
}

//@mfunc Sets the text position of the label to a pixel value
//@parm The X position
//@parm The Y position
void SRGraphLabel::SetTextPosition(int x,int y)
{
	SetTextPosition(CPoint(x,y));
}

//@mfunc Sets the text position of the label
//@parm The position
void SRGraphLabel::SetTextPosition(CPoint pos)
{
	m_dPosX=(double)pos.x;
	m_dPosY=(double)pos.y;
	m_TextPosition=pos;
}

//@mfunc NEW Draws labels for polar coordinate charts
void SRGraphDisplay::DrawPolarLabels()
{
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0)
		return;

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	int nCenterX=m_DisplayRect.left+(m_DisplayRect.Width()/2),
		nCenterY=m_DisplayRect.top+(m_DisplayRect.Height()/2);

	double dRadius=0.85*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
	if(dRadius<1)
		return;

	if(m_ptrXLabels==NULL)
		m_ptrXLabels=new SRGraphLabelBlock;

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())
			m_ptrYLabels=new SRGLogScale;
		else
			m_ptrYLabels=new SRGDecimalScale;
	}

	double dAngle=0,dAngleStep=TWO_PI/(16);

	CPoint *points=new CPoint[16];

	for(int x=0;x<16;x++)
	{
		points[x]=CPoint((int)(nCenterX+(dRadius*cos(dAngle-RADIANS(90)))),(int)(nCenterY+(dRadius*sin(dAngle-RADIANS(90)))));
		dAngle+=dAngleStep;
	}

	SRGDecimalScale *yLabels=(SRGDecimalScale *)m_ptrYLabels;  // This looks odd but its done so that you dont need to cast the pointer every time you use it.
	SRGraphLabelBlock *xLabels=m_ptrXLabels;

	xLabels->SetParent(GetParent());
	yLabels->SetParent(GetParent());

	xLabels->DrawingUsing(m_pDC,m_pCWnd);
	yLabels->DrawingUsing(m_pDC,m_pCWnd);
	
	CString strFormatString,strTestString;

	strFormatString=" %g ";

	CPoint ptStart,ptEnd;

	xLabels->SetLogTickPositions(FALSE);

	xLabels->SetFontSize(GetFontSize());
	xLabels->SetFaceName(GetFaceName());
	xLabels->SetTextColor(GetTextColor());
	xLabels->SetFontStyle(GetFontStyle());

	if(m_Style.GetShowIndices())
	{
		dAngle=0;

		for(int x=0;x<16;x++)
		{
			if(m_bSignificantChange)
			{
				CString strPhase;
				strPhase.Format("%3.2f",(double)x*(360.0/16));
				xLabels->AddLabelIndex(strPhase,x);
			}
			SRGraphLabel *pL=xLabels->GetLabelIndex(x);
			double dLabelAngle=dAngle-(PI/2);
			if(dLabelAngle<0)
				dLabelAngle+=TWO_PI;
			pL->SetLocationPoint((int)(DEGREES(dLabelAngle)/45));
			pL->SetTextPosition(points[x]);
			pL->SetVector(DEGREES(dLabelAngle));
			dAngle+=dAngleStep;
		}
		xLabels->Draw(m_pDC,m_pCWnd);
	}

	if(GetStyle()->GetShowNumerals())
	{
		dRadius=0.8*(double)min(m_DisplayRect.Height(),m_DisplayRect.Width())/2;
		yLabels->SetFontSize(GetFontSize());
		yLabels->SetFaceName(GetFaceName());
		yLabels->SetTextColor(GetTextColor());
		yLabels->SetFontStyle(GetFontStyle());
		yLabels->SetptrTickPositionList(&m_YTickList);
		yLabels->SetLogTickPositions(TRUE);
		yLabels->SetStartPos(CPoint(nCenterX,nCenterY));
		yLabels->SetEndPos(CPoint(nCenterX,nCenterY-(int)(dRadius+0.5)));
		yLabels->SetMaxValue(m_ScaleY.Max());
		yLabels->SetMinValue(m_ScaleY.Min());
		yLabels->SetAscending();
		yLabels->SetFormatString(strFormatString);
		yLabels->DrawingUsing(m_pDC,m_pCWnd);
		yLabels->CreateLabels();
		yLabels->Draw(m_pDC,m_pCWnd);
	}

	delete (points);

}

//@mfunc Retrieves the device independant X position
//@rdesc The floating point X coordinate.
double SRGraphLabel::GetTextPosX()
{
	return m_dPosX;
}

//@mfunc Retrieves the device independant Y position
//@rdesc The floating point Y coordinate.
double SRGraphLabel::GetTextPosY()
{
	return m_dPosY;
}

//@mfunc Initialises the device independant X position
//@parm The floating point X coordinate.
void SRGraphLabel::SetTextPosX(double dPosX)
{
   m_dPosX=dPosX;
   if(m_Rect.GetMeasurement()==0)
	   m_TextPosition.x=(int)dPosX;
}

//@mfunc Initialises the device independant Y position
//@parm The floating point Y coordinate.
void SRGraphLabel::SetTextPosY(double dPosY)
{
	m_dPosY=dPosY;
	if(m_Rect.GetMeasurement()==0)
	   m_TextPosition.y=(int)dPosY;
}


int SRGraphLabel::ReadTemplateLine(LPCSTR line)
{

	float PosX,PosY,VectorX,VectorY,Orientation;
	int LocationPoint,BlockMember,n;
	int nIndex=SRGraphTitle::ReadTemplateLine(line);
	if(sscanf(&line[nIndex]," %f %f %f %f %f %d %d%n",
				&PosX,&PosY,
				&VectorX,&VectorY,
				&Orientation,
				&LocationPoint,
				&BlockMember,
				&n)==7)
				{
					SetTextPosition((double)PosX,(double)PosY);
					m_dVectorX=(double)VectorX;
					m_dVectorY=(double)VectorY;
					SetOrientation((double)Orientation);
					SetLocationPoint(LocationPoint);
					SetBlockMember(BlockMember);
				}
	nIndex+=n;
	return nIndex;
}

void SRGraphDisplay::DrawGanttData()
{
	if(m_nSetCount==0 && m_nDivCount==0)
	{	// a null graph gets represented by a warning sign
		DrawEmptyGraph();
		return;
	}

	BOOL bLogData=m_Style.GetLog();
	DWORD dwGraphType=GetStyle()->GetGraphStyle();
	DWORD dwAxisType=GetStyle()->GetAxisStyle();
	double ScalingFactor=(double)m_DisplayRect.Width()/m_dMaxScaleY;

	int nSets=m_nSetCount/2;
	
	double dBlockWidth=(double)m_DisplayRect.Height()/nSets;

	double BarWidth;
	if(m_Style.GetUseMaxSize())
		BarWidth=min(dBlockWidth/1.1,(double)m_Style.GetMaxObjectWidth());
	else
		BarWidth=dBlockWidth/1.1;


	int CurrentSet,x;
	for(CurrentSet=0;CurrentSet<m_nSetCount;CurrentSet+=2)
	{

		int top,left,bottom,right;

		SRGraphDataList *gdlStart=GetParent()->GetGroup(CurrentSet+m_nMinSet,FALSE);
		SRGraphDataList *gdlEnd=GetParent()->GetGroup(CurrentSet+1+m_nMinSet,FALSE);

		ASSERT(gdlStart!=NULL);
		ASSERT(gdlEnd!=NULL);

		for(x=0;x<m_nDivCount;x++)
		{
			SRGraphStyle *ptrStyle=gdlStart->GetIndex(x+m_nMinDiv)->GetStyle();
			if(ptrStyle->GetInteriorStyle()==CX_INTERIOR_NONE && ptrStyle->GetFrameStyle()==CX_FRAME_NONE) // if there is no individual style defined
				ptrStyle=gdlStart->GetStyle();					// pick up a global style for the group

			SRGraphData *pDStart=gdlStart->GetIndex(x+m_nMinDiv);
			SRGraphData *pDEnd=gdlEnd->GetIndex(x+m_nMinDiv);

			if(!pDStart->GetNull())
			{
				m_nCurrentIndex=((CurrentSet/2)+m_nMinSet);
				
				double dDataStart=pDStart->GetValue(bLogData);
				double dDataEnd=pDEnd->GetValue(bLogData);

				if(bLogData)
				{
					dDataStart-=log10(m_ScaleY.Min());
					dDataEnd-=log10(m_ScaleY.Min());
				}
				else
				{
					dDataStart-=m_ScaleY.Min();
					dDataEnd-=m_ScaleY.Min();
				}

				bottom=(int)(m_DisplayRect.bottom-(((CurrentSet/2)*dBlockWidth)+(dBlockWidth/2))-(BarWidth/2));
				top=(int)(bottom+BarWidth);	
				right=(int)((double)m_DisplayRect.left+(ScalingFactor*dDataEnd)+0.5);
				left=(int)((double)m_DisplayRect.left+(ScalingFactor*dDataStart)+0.5);
				if(left>right)
				{
					int temp=right;
					right=left;
					left=temp;
				}

				if(left<m_DisplayRect.left)
					left=m_DisplayRect.left;

				if(right<m_DisplayRect.left)
					right=m_DisplayRect.left;

				if(left>m_DisplayRect.right)
					left=m_DisplayRect.right;

				if(right>m_DisplayRect.right)
					right=m_DisplayRect.right;
				
				BOOL bFBack=GetParent()->IsFeedbackLogging();
				
				GetParent()->LogRectangle(CRect(left,top,left+((right-left)/2),bottom),dDataStart<=dDataEnd ? CurrentSet+m_nMinSet : CurrentSet+m_nMinSet+1 ,x+m_nMinDiv);
				GetParent()->LogRectangle(CRect(left+((right-left)/2),top,right,bottom),dDataStart<=dDataEnd ? CurrentSet+m_nMinSet+1 : CurrentSet+m_nMinSet ,x+m_nMinDiv);
				
				GetParent()->SetFeedbackLogging(FALSE);

				DrawBar(CPoint(left,top),CPoint(right,bottom),0,0,ptrStyle,(dwAxisType==CX_AXIS_ISO_CLASSIC_YX) ? CX_OBJECT_EXBAR : 0);				

				GetParent()->SetFeedbackLogging(bFBack);
			}
		}
	}
}


void SRGraphLabelBlock::CreateLabels()
{
	ASSERT(FALSE); // Your derived class should create its own labels
}

void SRGraphDisplay::DrawGanttLabels()
{
	if(m_ScaleY.Max()==0 && m_ScaleY.Min()==0)
		return;

	if(m_Style.GetLabelStyle()==CX_LABEL_NONE)
		return;

	SRGDecimalScale *yLabels;

	if(m_ptrYLabels==NULL)
	{
		if(GetStyle()->GetLog())
			m_ptrYLabels=new SRGLogScale;
		else
		{
			switch(GetStyle()->GetAxisStyle())
			{
			default:
			case CX_AXIS_GANTT_VALUE:
 				m_ptrYLabels=new SRGDecimalScale;
				break;
			case CX_AXIS_GANTT_DATE:
 				m_ptrYLabels=new SRGDateScale;
				break;
			}
		}
	}

	ASSERT(m_ptrYLabels!=NULL);

	yLabels=(SRGDecimalScale *)m_ptrYLabels;

	if(m_ptrXLabels==NULL)
		m_ptrXLabels=new SRGraphLabelBlock;

	ASSERT(m_ptrXLabels!=NULL);

	m_ptrXLabels->SetParent(GetParent());
	yLabels->SetParent(GetParent());

	
	CString strTestString;

	CPoint ptStart,ptEnd;

	for(int pass=0;pass<2;pass++)
	{

		if(m_Style.GetShowIndices())
		{

			int nIdx=0;
			if(m_bSignificantChange)
			{
				for(int nSet=0;nSet<m_nSetCount;nSet+=2)
					m_ptrXLabels->AddLabelIndex(GetParent()->GetGroup(nSet+m_nMinSet)->GetAnnotation(GetCountryCode()),nIdx++);
				while(m_ptrXLabels->GetLabelCount()>nIdx)
					m_ptrXLabels->RemoveLastLabel();
			}

			int nDist=(m_DisplayRect.Height()/(m_nSetCount/2))/2;
			ptStart=CPoint(m_DisplayRect.left-5,m_DisplayRect.bottom-nDist);
			ptEnd=CPoint(m_DisplayRect.left-5,m_DisplayRect.top+nDist);

			m_ptrXLabels->DrawingUsing(m_pDC,m_pCWnd);
			m_ptrXLabels->SetLogTickPositions(TRUE);
			m_ptrXLabels->SetptrTickPositionList(&m_YTickList);
			m_ptrXLabels->SetFontSize(GetFontSize());
			m_ptrXLabels->SetFaceName(GetFaceName());
			m_ptrXLabels->SetTextColor(GetTextColor());
			m_ptrXLabels->SetFontStyle(GetFontStyle());
			m_ptrXLabels->SetBlockLocationPoints(SRGraphLabel::MidRight);
			m_ptrXLabels->SetBlockPositions(ptStart,ptEnd);
			m_ptrXLabels->SetBlockVectors(-1,0);
			m_ptrXLabels->SetPass(pass);
			m_ptrXLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
				m_DisplayRect.left+=m_ptrXLabels->GetBlockRect().Width();
		}

		if(m_Style.GetShowNumerals())
		{
			yLabels->DrawingUsing(m_pDC,m_pCWnd);
			yLabels->SetFontSize(GetFontSize());
			yLabels->SetFaceName(GetFaceName());
			yLabels->SetTextColor(GetTextColor());
			yLabels->SetFontStyle(GetFontStyle());
			yLabels->SetMinValue(m_ScaleY.Min());
			yLabels->SetMaxValue(m_ScaleY.Max());
			ptStart=(CPoint(m_DisplayRect.left,m_DisplayRect.bottom+5));
			ptEnd=CPoint(CPoint(m_DisplayRect.right,m_DisplayRect.bottom+5));
			yLabels->SetLogTickPositions(TRUE);
			yLabels->SetptrTickPositionList(&m_XTickList);
			yLabels->SetStartPos(ptStart);
			yLabels->SetEndPos(ptEnd);
			yLabels->SetAscending();
			yLabels->SetLabelSide(TRUE);
			yLabels->CreateLabels();
			yLabels->SetPass(pass);
			yLabels->Draw(m_pDC,m_pCWnd);
			if(!pass && !m_Style.DRIsLocked())
				m_DisplayRect.bottom-=yLabels->GetBlockRect().Height();
		}
	}


}
